package com.rootcloud.utils;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.Objects;

/**
 * jdk8 时间API
 *
 * @author leilei.zang
 * @date 2021-09-13 11:27
 */
public class LocalDateUtils {

    public static final String YYYY_MM_DD = "yyyy-MM-dd";
    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    public static final String YYYY_MM_DD_HH_MM_SS_SSS = "yyyyMMddhhmmssSSS";

    //格式化date
    public static String formatDate(Date date) {
        if (Objects.isNull(date)) {
            return null;
        }
        LocalDateTime localDateTime = date.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        return localDateTime.format(DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS));
    }

    //格式化date
    public static String formatDate(Date date, String formatter) {
        if (Objects.isNull(date)) {
            return null;
        }
        return formatDate(date.toInstant(), formatter);
    }

    //格式化instant
    public static String formatDate(Instant instant) {
        if (Objects.isNull(instant)) {
            return null;
        }
        LocalDateTime localDateTime = instant.atZone(ZoneId.systemDefault()).toLocalDateTime();
        return localDateTime.format(DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS));
    }

    //格式化instant
    public static String formatDate(Instant instant, String formatter) {
        if (Objects.isNull(instant)) {
            return null;
        }
        LocalDateTime localDateTime = instant.atZone(ZoneId.systemDefault()).toLocalDateTime();
        if (StringUtil.isNotBlank(formatter)) {
            return localDateTime.format(DateTimeFormatter.ofPattern(formatter));
        }
        return localDateTime.format(DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS));
    }

    //获取当前时间
    public static String now(String format) {
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(format);
        LocalDateTime now = LocalDateTime.now();
        return now.format(dateTimeFormatter);
    }

    //获取当前时间
    public static String now() {
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS);
        LocalDateTime now = LocalDateTime.now();
        return now.format(dateTimeFormatter);
    }

}
