package com.rootcloud.utils;

import org.apache.commons.lang3.StringUtils;

import java.util.regex.Pattern;

public class StringUtil extends StringUtils {

    private static final String[] specialWordArr = {"\\", "$", "(", ")", "*", "+", ".", "[", "]", "?", "^", "{", "}", "|"};
    private static String regexPrefix = "^.*";
    private static String regexSuffix = ".*$";
    public static final String comma = ":";

    //fixed by: yangnenghao / date: 2021/4/19 / 导入null
    //override String.valueOf()
    public static String valueOf(Object obj) {
        return (obj == null) ? "" : obj.toString();
    }


    /**
     * 转义特殊字符
     *
     * @param keyword
     * @return java.lang.String
     * @date 2021-02-18 16:59
     */
    public static String escapeExprSpecialWordStr(String keyword) {
        if (isNotBlank(keyword)) {
            for (String key : specialWordArr) {
                if (keyword.contains(key)) keyword = keyword.replace(key, "\\" + key);
            }
        }
        return keyword;
    }

    /**
     * 转义特殊字符,并编译
     *
     * @param keyword
     * @return java.util.regex.Patternl
     * @date 2021-02-18 16:59
     */
    public static Pattern escapeExprSpecialWord(String keyword) {
        return Pattern.compile(regexPrefix + escapeExprSpecialWordStr(keyword) + regexSuffix, Pattern.CASE_INSENSITIVE);
    }

    /**
     * 转义特殊字符,自定义编译
     *
     * @param keyword
     * @param pattern
     * @return java.util.regex.Pattern
     * @date 2021-02-19 15:53
     */
    public static Pattern escapeExprSpecialWord(String keyword, Integer pattern) {
        if (null != pattern)
            return Pattern.compile(regexPrefix + escapeExprSpecialWordStr(keyword) + regexSuffix, pattern);
        return escapeExprSpecialWord(keyword);
    }

    /**
     * 对老代码使用String.valueOf()的null过滤处理
     * 新代码：建议使用重写的StringUtil.valueOf()
     *
     * @param value
     * @return boolean
     * @author leilei.zang
     * @date 2021-08-17 10:37
     */
    public static boolean isNotBlank(String value) {
        return StringUtils.isNotBlank(value) && !"null".equals(value);
    }

    /**
     * 配合Optional使用,按需使用
     *
     * @param value
     * @return java.lang.String
     * @author leilei.zang
     * @date 2021-10-12 10:49
     */
    public static String values(String value) {
        return StringUtil.isNotBlank(value) ? value : null;
    }
}
