package com.rootcloud.common;

import jline.internal.Log;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @ClassName DateUtil
 * @Description TODO
 * @Author w-liuyi01
 * @Date 2022/4/22 10:49
 * @Version 1.0
 */
public class DateUtil {

    // Grace style
    public static final String PATTERN_GRACE = "yyyy/MM/dd HH:mm:ss";
    public static final String PATTERN_GRACE_NORMAL = "yyyy/MM/dd HH:mm";
    public static final String PATTERN_GRACE_SIMPLE = "yyyy/MM/dd";// Classical style
    public static final String PATTERN_CLASSICAL = "yyyy-MM-dd HH:mm:ss";
    public static final String PATTERN_CLASSICAL_ISO = "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'";
    public static final String PATTERN_CLASSICAL_ISO_SIMPLE = "yyyy-MM-dd'T'HH:mm:ss'Z'";
    public static final String PATTERN_CLASSICAL_NORMAL = "yyyy-MM-dd HH:mm";
    public static final String PATTERN_CLASSICAL_SIMPLE = "yyyy-MM-dd";
    //CH style
    public static final String PATTERN_CH = "yyyy年MM月dd日 HH时mm分ss秒";
    public static final String PATTERN_CH_NORMAL = "yyyy年MM月dd日 HH时mm分";
    public static final String PATTERN_CH_SIMPLE = "yyyy年MM月dd日";
    public static final String PATTERN_MMDD_SIMPLE = "MM-dd";


    //把标准世界时转换为北京时间
    public static String UTCtoBeijing(String Timestamp_Local)
    {
        //先去掉格式
        Timestamp_Local = DateUtil.reverseHandlerTime(Timestamp_Local);
        //转换为北京时间
        LocalDateTime date = LocalDateTime.parse(Timestamp_Local, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusHours(8);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String formatedDateStr = date.format(formatter);
        return  formatedDateStr;
    }

    public static String UTCtoBeijing_1(String Timestamp_Local)
    {
        //先去掉格式
        Timestamp_Local = DateUtil.reverseHandlerTime_1(Timestamp_Local);
        //转换为北京时间
        LocalDateTime date = LocalDateTime.parse(Timestamp_Local, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusHours(8);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String formatedDateStr = date.format(formatter);
        return  formatedDateStr;
    }


    // 获取当前时间
    public static String getCurrentTime(){
        Calendar calendar= Calendar.getInstance();
        SimpleDateFormat dateFormat= new SimpleDateFormat(PATTERN_CLASSICAL);
        return dateFormat.format(calendar.getTime());
    }

/**
     * @Author w-liuyi01
     * @Description //带时区的时间格式处理
     * @Date 2022/4/22 10:55
     * @Param [time]
     * @return java.lang.String
     **/
    public static String handlerTime(String time) {
        Date date = null;
        try {
            date = new SimpleDateFormat(PATTERN_CLASSICAL).parse(time);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        //国际时间格式转换东八区-8小时
//        calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR)-8);
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL_ISO);
        //format将Date型转换为String型，parse将String型转换为Date型
        String UTCDate = sdf.format(calendar.getTime());
        return UTCDate;
    }

    public static String handlerTimeNoUTC(String time,int timeZone) {
        Date date = null;
        try {
            date = new SimpleDateFormat(PATTERN_CLASSICAL_ISO).parse(time);
        } catch (ParseException e) {
            try {
                date = new SimpleDateFormat(PATTERN_CLASSICAL_ISO_SIMPLE).parse(time);
            }catch  (ParseException ex) {
                e.printStackTrace();
            }
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        //国际时间格式转换东八区-8小时
        calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR) + timeZone);
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL);
        //format将Date型转换为String型，parse将String型转换为Date型
        String UTCDate = sdf.format(calendar.getTime());
        return UTCDate;
    }


    /**
     * @Author w-lxf
     * @Description 根据string日期获取时间戳
     * @return
     */
    public static Long getTimesByString(String time){
        Long result = 0l;
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL);
//        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL_ISO);//yyyy-MM-dd'T'HH:mm:ss.SSS'Z'
        // 设置时区为CST，中国标准时区
//        sdf.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        try {
            Date date = sdf.parse(time);
            result = date.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 根据string日期格式yyyy-MM-dd'T'HH:mm:ss.SSS'Z'转化为long型时间戳
     * @param time 日期格式yyyy-MM-dd'T'HH:mm:ss.SSS'Z'
     * @return
     */
    public static Long timeStringToLong(String time){
        Long result = 0l;
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL_ISO);
        sdf.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        try {
            Date date = sdf.parse(time);
            result = date.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return result;
    }

    public static Long getTimesByStringType(String time){
        Long result = 0l;
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL);
        if (time.contains("Z")){
            sdf = new SimpleDateFormat(PATTERN_CLASSICAL_ISO);
        }
        try {
            Date date = sdf.parse(time);
            result = date.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return result;
    }


    /**
     * 计算两个字符串时间相差天数
     * @param date1
     * @param date2
     * @return
     */
    public static float dayDiff(String date1,String date2) throws ParseException{
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar calendar = Calendar.getInstance();

        Date old = sdf.parse(date1);
        calendar.setTime(old);
        Long oTime = calendar.getTimeInMillis();

        Date now = sdf.parse(date2);
        calendar.setTime(now);
        Long nTime = calendar.getTimeInMillis();
        return (nTime - oTime)/(3600F * 1000 * 24);
    }


    /**
     * UCT时间转换成北京时间
     * @param time
     * @return
     */
    public static String getDateByUct(String time){
        String result = null;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        Date uctDate = null;
        try {
            //处理平台的utc时间格式统一为yyyy-MM-dd'T'HH:mm:ss.SSS'Z'
            String subTime = time.substring(time.indexOf(".")+1,time.length());
            String fillTime = "";
            if(subTime.length()<4){
                time = time.substring(0,time.indexOf(".")+1);
                for(int i=0;i<4-subTime.length();i++){
                    fillTime+="0";
                }
                time = time+fillTime+subTime;
            }
            uctDate = sdf.parse(time);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(uctDate);
            //20220830时间不对去掉+8小时的逻辑
//            calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR)+8);
//            calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR));
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            result = simpleDateFormat.format(calendar.getTime());
        } catch (ParseException e) {
            Log.error("时间转换异常："+e.getMessage());
            //e.printStackTrace();
        }
        return result;
    }


    public static String reverseHandlerTimeByType(String time) {
        Date date = null;
        try {
            if (time.contains("Z")){
                time = time.replace("Z", " UTC");
                date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS Z").parse(time);
            }else{
                date = new SimpleDateFormat(PATTERN_CLASSICAL).parse(time);
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL);
        //format将Date型转换为String型，parse将String型转换为Date型
        String UTCDate = sdf.format(date);
        return UTCDate;
    }


    public static String reverseHandlerTime(String time) {
        Date date = null;
        try {
            date = new SimpleDateFormat(PATTERN_CLASSICAL_ISO).parse(time);
        } catch (ParseException e) {
            try {
                date = new SimpleDateFormat(PATTERN_CLASSICAL_ISO_SIMPLE).parse(time);
            } catch (ParseException ex) {
                ex.printStackTrace();
            }
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        //国际时间格式转换东八区+8小时
        calendar.set(Calendar.HOUR,calendar.get(Calendar.HOUR)+8);
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL);
        //format将Date型转换为String型，parse将String型转换为Date型
        String UTCDate = sdf.format(calendar.getTime());
        return UTCDate;
    }

    public static String reverseHandlerTime_1(String time) {
        Date date = null;
        try {
            if(time.length()<= 20)
            {
                time = time.replace('Z','.')+"000Z"; //2022-05-17T13:15:48Z  2022-05-14T15:00:15Z
                date = new SimpleDateFormat(PATTERN_CLASSICAL_ISO).parse(time);
            }
            else
                date = new SimpleDateFormat(PATTERN_CLASSICAL_ISO).parse(time);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_CLASSICAL);
        //format将Date型转换为String型，parse将String型转换为Date型
        String UTCDate = sdf.format(date);
        return UTCDate;
    }

    /**
     * 根据默认格式将指定字符串解析成日期
     *
     * @param str
     * 指定字符串
     * @return 返回解析后的日期
     */
    public static Date parse(String str) {
        return parse(str, PATTERN_CLASSICAL);
    }/**
     * 根据指定格式将指定字符串解析成日期
     *
     * @param str
     * 指定日期
     * @param pattern
     * 指定格式
     * @return 返回解析后的日期
     */
    public static Date parse(String str, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        try {
            return sdf.parse(str);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }/**
     * 根据默认格式将日期转格式化成字符串
     *
     * @param date
     * 指定日期
     * @return 返回格式化后的字符串
     */
    public static String format(Date date) {
        return format(date, PATTERN_CLASSICAL);
    }
    /**
     * 根据默认格式将日期转格式化成字符串
     *
     * @param date
     * 指定日期
     * @return 返回格式化后的字符串
     */
    public static String formatForDate(Date date) {
        return format(date, PATTERN_CLASSICAL_SIMPLE);
    }

    /**
     * 根据指定格式将指定日期格式化成字符串
     *
     * @param date
     * 指定日期
     * @param pattern
     * 指定格式
     * @return 返回格式化后的字符串
     */
    public static String format(Date date, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.format(date);
    }/**
     * 获取时间date1与date2相差的秒数
     *
     * @param date1
     * 起始时间
     * @param date2
     * 结束时间
     * @return 返回相差的秒数
     */
    public static int getOffsetSeconds(Date date1, Date date2) {
        int seconds = (int) ((date2.getTime() - date1.getTime()) / 1000);
        return seconds;
    }/**
     * 获取时间date1与date2相差的分钟数
     *
     * @param date1
     * 起始时间
     * @param date2
     * 结束时间
     * @return 返回相差的分钟数
     */
    public static int getOffsetMinutes(Date date1, Date date2) {
        return getOffsetSeconds(date1, date2) / 60;
    }/**
     * 获取时间date1与date2相差的小时数
     *
     * @param date1
     * 起始时间
     * @param date2
     * 结束时间
     * @return 返回相差的小时数
     */
    public static int getOffsetHours(Date date1, Date date2) {
        return getOffsetMinutes(date1, date2) / 60;
    }/**
     * 获取时间date1与date2相差的天数数
     *
     * @param date1
     * 起始时间
     * @param date2
     * 结束时间
     * @return 返回相差的天数
     */
    public static int getOffsetDays(Date date1, Date date2) {
        return getOffsetHours(date1, date2) / 24;
    }/**
     * 获取时间date1与date2相差的周数
     *
     * @param date1
     * 起始时间
     * @param date2
     * 结束时间
     * @return 返回相差的周数
     */
    public static int getOffsetWeeks(Date date1, Date date2) {
        return getOffsetDays(date1, date2) / 7;
    }/**
     * 获取重置指定日期的时分秒后的时间
     *
     * @param date
     * 指定日期
     * @param hour
     * 指定小时
     * @param minute
     * 指定分钟
     * @param second
     * 指定秒
     * @return 返回重置时分秒后的时间
     */
    public static Date getResetTime(Date date, int hour, int minute, int second) {
        Calendar cal = Calendar.getInstance();
        if (date != null) {
            cal.setTime(date);
        }
        cal.set(Calendar.HOUR_OF_DAY, hour);
        cal.set(Calendar.SECOND, minute);
        cal.set(Calendar.MINUTE, second);
        return cal.getTime();
    }/**
     * 返回指定日期的起始时间
     *
     * @param date
     * 指定日期(例如2014-08-01)
     * @return 返回起始时间(例如2014-08-01 00:00:00)
     */
    public static Date getIntegralStartTime(Date date) {
        return getResetTime(date, 0, 0, 0);
    }/**
     * 返回指定日期的结束时间
     *
     * @param date
     * 指定日期(例如2014-08-01)
     * @return 返回结束时间(例如2014-08-01 23:59:59)
     */
    public static Date getIntegralEndTime(Date date) {
        return getResetTime(date, 23, 59, 59);
    }/**
     * 获取指定日期累加年月日后的时间
     *
     * @param date
     * 指定日期
     * @param year
     * 指定年数
     * @param month
     * 指定月数
     * @param day
     * 指定天数
     * @return 返回累加年月日后的时间
     */
    public static Date rollDate(Date date, int year, int month, int day) {
        Calendar cal = Calendar.getInstance();
        if (date != null) {
            cal.setTime(date);
        }
        cal.add(Calendar.YEAR, year);
        cal.add(Calendar.MONTH, month);
        cal.add(Calendar.DAY_OF_MONTH, day);
        return cal.getTime();
    }/**
     * 获取指定日期累加指定月数后的时间
     *
     * @param date
     * 指定日期
     * @param month
     * 指定月数
     * @return 返回累加月数后的时间
     */
    public static Date rollMonth(Date date, int month) {
        return rollDate(date, 0, month, 0);
    }/**
     * 获取指定日期累加指定天数后的时间
     *
     * @param date
     * 指定日期
     * @param day
     * 指定天数
     * @return 返回累加天数后的时间
     */
    public static Date rollDay(Date date, int day) {
        return rollDate(date, 0, 0, day);
    }/**
     * 计算指定日期所在月份的天数
     *
     * @param date
     * 指定日期
     * @return 返回所在月份的天数
     */
    public static int getDayOfMonth(Date date) {
        Calendar cal = Calendar.getInstance();
        if (date != null) {
            cal.setTime(date);
        }
        int dayOfMonth = cal.getActualMaximum(Calendar.DATE);
        return dayOfMonth;
    }/**
     * 获取当月第一天的起始时间,例如2014-08-01 00:00:00
     *
     * @return 返回当月第一天的起始时间
     */
    public static Date getMonthStartTime() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_MONTH, 1);
        return getIntegralStartTime(cal.getTime());
    }/**
     * 获取当月最后一天的结束时间,例如2014-08-31 23:59:59
     *
     * @return 返回当月最后一天的结束时间
     */
    public static Date getMonthEndTime() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_MONTH, getDayOfMonth(cal.getTime()));
        return getIntegralEndTime(cal.getTime());
    }/**
     * 获取上个月第一天的起始时间,例如2014-07-01 00:00:00
     *
     * @return 返回上个月第一天的起始时间
     */
    public static Date getLastMonthStartTime() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, -1);
        cal.set(Calendar.DAY_OF_MONTH, 1);
        return getIntegralStartTime(cal.getTime());
    }/**
     * 获取上个月最后一天的结束时间,例如2014-07-31 23:59:59
     *
     * @return 返回上个月最后一天的结束时间
     */
    public static Date getLastMonthEndTime() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, -1);
        cal.set(Calendar.DAY_OF_MONTH, getDayOfMonth(cal.getTime()));
        return getIntegralEndTime(cal.getTime());
    }/**
     * 获取下个月第一天的起始时间,例如2014-09-01 00:00:00
     *
     * @return 返回下个月第一天的起始时间
     */
    public static Date getNextMonthStartTime() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, 1);
        cal.set(Calendar.DAY_OF_MONTH, 1);
        return getIntegralStartTime(cal.getTime());
    }/**
     * 获取下个月最后一天的结束时间,例如2014-09-30 23:59:59
     *
     * @return 返回下个月最后一天的结束时间
     */
    public static Date getNextMonthEndTime() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, 1);
        cal.set(Calendar.DAY_OF_MONTH, getDayOfMonth(cal.getTime()));
        return getIntegralEndTime(cal.getTime());
    }/**
     * 获取当前季度第一天的起始时间
     *
     * @return 返回当前季度第一天的起始时间
     */
    public static Date getQuarterStartTime() {
        Calendar cal = Calendar.getInstance();
        int month = cal.get(Calendar.MONTH);
        if (month < 3) {
            cal.set(Calendar.MONTH, 0);
        } else if (month < 6) {
            cal.set(Calendar.MONTH, 3);
        } else if (month < 9) {
            cal.set(Calendar.MONTH, 6);
        } else {
            cal.set(Calendar.MONTH, 9);
        }
        cal.set(Calendar.DAY_OF_MONTH, 1);
        return getIntegralStartTime(cal.getTime());
    }/**
     * 获取当前季度最后一天的结束时间
     *
     * @return 返回当前季度最后一天的结束时间
     */
    public static Date getQuarterEndTime() {
        Calendar cal = Calendar.getInstance();
        int month = cal.get(Calendar.MONTH);
        if (month < 3) {
            cal.set(Calendar.MONTH, 2);
        } else if (month < 6) {
            cal.set(Calendar.MONTH, 5);
        } else if (month < 9) {
            cal.set(Calendar.MONTH, 8);
        } else {
            cal.set(Calendar.MONTH, 11);
        }
        cal.set(Calendar.DAY_OF_MONTH, getDayOfMonth(cal.getTime()));
        return getIntegralEndTime(cal.getTime());
    }/**
     * 获取前一个工作日
     *
     * @return 返回前一个工作日
     */
    public static Date getPrevWorkday() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DAY_OF_MONTH, -1);
        if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
            cal.add(Calendar.DAY_OF_MONTH, -2);
        }
        if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) {
            cal.add(Calendar.DAY_OF_MONTH, -1);
        }
        return getIntegralStartTime(cal.getTime());
    }/**
     * 获取下一个工作日
     *
     * @return 返回下个工作日
     */
    public static Date getNextWorkday() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DAY_OF_MONTH, 1);
        if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) {
            cal.add(Calendar.DAY_OF_MONTH, 2);
        }
        if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
            cal.add(Calendar.DAY_OF_MONTH, 1);
        }
        return getIntegralStartTime(cal.getTime());
    }/**
     * 获取当周的第一个工作日
     *
     * @return 返回第一个工作日
     */
    public static Date getFirstWorkday() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        return getIntegralStartTime(cal.getTime());
    }/**
     * 获取当周的最后一个工作日
     *
     * @return 返回最后一个工作日
     */
    public static Date getLastWorkday() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_WEEK, Calendar.FRIDAY);
        return getIntegralStartTime(cal.getTime());
    }/**
     * 判断指定日期是否是工作日
     *
     * @param date
     * 指定日期
     * @return 如果是工作日返回true,否则返回false
     */
    public static boolean isWorkday(Date date) {
        Calendar cal = Calendar.getInstance();
        if (date != null) {
            cal.setTime(date);
        }
        int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);
        return !(dayOfWeek == Calendar.SATURDAY || dayOfWeek == Calendar.SUNDAY);
    }/**
     * 获取指定日期是星期几
     *
     * @param date
     * 指定日期
     * @return 返回星期几的描述
     */
    public static String getWeekdayDesc(Date date) {
        final String[] weeks = new String[]{"星期日", "星期一", "星期二", "星期三", "星期四",
                "星期五", "星期六"};
        Calendar cal = Calendar.getInstance();
        if (date != null) {
            cal.setTime(date);
        }
        return weeks[cal.get(Calendar.DAY_OF_WEEK) - 1];
    }/**
     * 获取指定日期距离当前时间的时间差描述(如3小时前、1天前)
     *
     * @param date
     * 指定日期
     * @return 返回时间差的描述
     */
    public static String getTimeOffsetDesc(Date date) {
        int seconds = getOffsetSeconds(date, new Date());
        if (Math.abs(seconds) < 60) {
            return Math.abs(seconds) + "秒" + (seconds > 0 ? "前" : "后");
        }
        int minutes = seconds / 60;
        if (Math.abs(minutes) < 60) {
            return Math.abs(minutes) + "分钟" + (minutes > 0 ? "前" : "后");
        }
        int hours = minutes / 60;
        if (Math.abs(hours) < 60) {
            return Math.abs(hours) + "小时" + (hours > 0 ? "前" : "后");
        }
        int days = hours / 24;
        if (Math.abs(days) < 7) {
            return Math.abs(days) + "天" + (days > 0 ? "前" : "后");
        }
        int weeks = days / 7;
        if (Math.abs(weeks) < 5) {
            return Math.abs(weeks) + "周" + (weeks > 0 ? "前" : "后");
        }
        int monthes = days / 30;
        if (Math.abs(monthes) < 12) {
            return Math.abs(monthes) + "个月" + (monthes > 0 ? "前" : "后");
        }
        int years = monthes / 12;
        return Math.abs(years) + "年" + (years > 0 ? "前" : "后");
    }

    /**
     * 根据时间区间获取时间段的所有日期
     * @param startTime
     * @param endTime
     * @return
     */
    public static List<Date> getDates(Date startTime, Date endTime) {
        startTime = getDateStart(startTime);
        endTime = getDateStart(endTime);
        List lDate = new ArrayList();
        lDate.add(startTime);
        Calendar calBegin = Calendar.getInstance();
        // 使用给定的 Date 设置此 Calendar 的时间
        calBegin.setTime(startTime);
        Calendar calEnd = Calendar.getInstance();
        // 使用给定的 Date 设置此 Calendar 的时间
        calEnd.setTime(endTime);
        // 测试此日期是否在指定日期之后
        while (endTime.after(calBegin.getTime())){
            // 根据日历的规则，为给定的日历字段添加或减去指定的时间量
            calBegin.add(Calendar.DAY_OF_MONTH, 1);
            lDate.add(calBegin.getTime());
        }
        return lDate;
    }

    /**
     * 获取指定日期的开始时间
     * @param date
     * @return
     */
    public static Date getDateStart(Date date){
        if(date == null){
            return null;
        }
        Calendar calendar =  Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取指定日期的结束时间
     * @return str
     */
    public static Date getDateEnd(Date date){
        if(date == null){
            return null;
        }
        Calendar calendar =  Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 根据格式把date时间转为字符串
     * @param date
     * @param format
     * @return
     */
    public static String dateToString(Date date, String format){
        SimpleDateFormat dateFormat = new SimpleDateFormat(format);
        return dateFormat.format(date);
    }

    public static Date stringToDate(String dateStr, String format) {
        Date d = null;
        SimpleDateFormat dateFormat = new SimpleDateFormat(format);
        try {
            dateFormat.setLenient(false);
            d = dateFormat.parse(dateStr);
        } catch (Exception e) {
            d = null;
        }
        return d;
    }

    /**
     * 时间戳转为date
     * @param times
     * @param format
     * @return
     */
    public static Date timestampToDate(Long times, String format){
        Date date = null;
        try {
            SimpleDateFormat dateFormat =  new SimpleDateFormat(format);
            String d = dateFormat.format(times);
            date = dateFormat.parse(d);
        }catch (Exception e){
            date = null;
        }
        return date;
    }

    /**
     * 时间戳转为date
     * @param times
     * @return
     */
    public static Date timestampToDate(Long times){
        Date date = null;
        try {
            SimpleDateFormat dateFormat =  new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String d = dateFormat.format(times);
            date = dateFormat.parse(d);
        }catch (Exception e){
            date = null;
        }
        return date;
    }

    /**
     * 获取指定小时
     * @param date 时间
     * @param hours 小时数
     * @param minutes 分钟数
     * @return Date
     */
    public static Date getBeforeHour(Date date, int hours, int minutes){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.HOUR_OF_DAY, hours);
        calendar.add(Calendar.MINUTE, minutes);
        return calendar.getTime();
    }


    /**
     * 时间戳转时间
     */
    public static String timeStamp2Time(Long timeStamp) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date(timeStamp);
        return sdf.format(date);
    }


    /**
     * 得到几天前的时间
     * @param date
     * @param day
     * @return
     */
    public static Date getDateBefore(Date date,int day){
        Calendar now =Calendar.getInstance();
        now.setTime(date);
        now.set(Calendar.DATE,now.get(Calendar.DATE)-day);
        return now.getTime();
    }

    /**
     * 得到几天后的时间
     * @param date
     * @param day
     * @return
     */
    public static Date getDateAfter(Date date,int day){
        Calendar now = Calendar.getInstance();
        now.setTime(date);
        now.set(Calendar.DATE,now.get(Calendar.DATE)+day);
        return now.getTime();
    }


    /**
     * 得到几天后的时间
     * @param date
     * @param day
     * @return
     */
    public static String getDateStringAfter(Date date,int day){
        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar now = Calendar.getInstance();
        now.setTime(date);
        now.set(Calendar.DATE,now.get(Calendar.DATE)+day);
        return sf.format(now.getTime());
    }

    public static Boolean isSameYear(String date){
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(stringToDate(date, PATTERN_CLASSICAL));
        int a =  cal1.get(Calendar.YEAR);
        Calendar cal2 = Calendar.getInstance();
        int b =  cal2.get(Calendar.YEAR);
        return a == b;
    }


    public static String getSysdateStr() {
        SimpleDateFormat formatter= new SimpleDateFormat(PATTERN_CLASSICAL);
        Date date = new Date(System.currentTimeMillis());
        return formatter.format(date);
    }


    public static String specialDate(String spcialString){
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");
        Date  date = null;
        try {
            date = df.parse(spcialString);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        SimpleDateFormat df1 = new SimpleDateFormat ("EEE MMM dd HH:mm:ss Z yyyy", Locale.UK);
        Date date1 = null;
        try {
            date1 = df1.parse(date.toString());
        } catch (ParseException e) {
            e.printStackTrace();
        }
        SimpleDateFormat df2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return df2.format(date1);
    }

    /**
     * 获取两个日期之间的所有日期
     * @param startTime 开始日期
     * @param endTime 结束日期
     * @return
     */
    public static List<String> getDays(String startTime, String endTime) {
        // 返回的日期集合
        List<String> days = new ArrayList<String>();
        DateFormat dateFormat = new SimpleDateFormat(DateUtil.PATTERN_CLASSICAL_SIMPLE);
        try {
            Date start = dateFormat.parse(startTime);
            Date end = dateFormat.parse(endTime);
            Calendar tempStart = Calendar.getInstance();
            tempStart.setTime(start);
            Calendar tempEnd = Calendar.getInstance();
            tempEnd.setTime(end);
            tempEnd.add(Calendar.DATE, +1);// 日期加1(包含结束)
            while (tempStart.before(tempEnd)) {
                days.add(dateFormat.format(tempStart.getTime()));
                tempStart.add(Calendar.DAY_OF_YEAR, 1);
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return days;
    }




    /**
     * @param args
     */
    public static void main(String[] args) {
        String dateString = "2022-09-14T02:12:13Z";
        String startDate = "2023-08-08";
        String endDate = "2023-08-07";
        System.out.println("固定时间段:" + DateUtil.getDays(startDate, endDate));
        //System.out.println(DateUtil.handlerTimeNoUTC(dateString,8));
//        System.out.println(DateUtil.handlerTime(dateString));

//        Date date = new Date();
//        Date date1 = rollDay(date, -2);
//        Date date2 = rollMonth(date, -3);
//        System.out.println(format(date));
//        System.out.println(format(date1));
//        System.out.println(format(date2));
//        System.out.println(getTimeOffsetDesc(date1));
//        System.out.println(getTimeOffsetDesc(date2));
//        System.out.println("获取下个月第一天的起始时间:"+format(getNextMonthStartTime(),PATTERN_CH));
//        System.out.println("获取下个月最后一天的结束时间:"+format(getNextMonthEndTime()));
//        System.out.println("获取前一个工作日:"+format(getPrevWorkday(),PATTERN_CLASSICAL_SIMPLE));
        System.out.println("获取后一个工作日:"+format(getNextWorkday(),PATTERN_CLASSICAL_SIMPLE));
//        System.out.println("获取当前季度第一天的起始时间:"+format(getQuarterStartTime()));
//        System.out.println("获取当前季度最后一天的结束时间:"+format(getQuarterEndTime()));
//        System.out.println("获取当周的第一个工作日:"+format(getFirstWorkday(),PATTERN_CLASSICAL_SIMPLE));
//        System.out.println("获取当周的最后一个工作日:"+format(getLastWorkday(),PATTERN_CLASSICAL_SIMPLE));
//        System.out.println(getWeekdayDesc(null));
//        System.out.println(getSysdateStr());
    }

}
