package com.rootcloud.common;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 通用返回对象
 */
@Getter
@Setter
@ToString
@ApiModel("接口调用结果")
public class JsonResult<T> {
    protected static final String CODE_SUCCESS = "200";
    private static final String CODE_CLIENT_ERROR = "450";
    /**
     * 状态码
     */
    @ApiModelProperty(notes = "返回码")
    private String code;
    /**
     * 数据封装
     */
    @ApiModelProperty(notes = "数据")
    private T data;
    /**
     * 提示信息
     */
    @ApiModelProperty(notes = "提示信息")
    private String message;

    public JsonResult(String code) {
        this(code, null);
    }

    public JsonResult(String code, T data) {
        this(code, data, null);
    }

    public JsonResult(String code, T data, String message) {
        this.code = code;
        this.message = message;
        this.data = data;
    }


    public static <T> JsonResult<T> success() {
        return new JsonResult<T>(CODE_SUCCESS, null, null);
    }

    public static <T> JsonResult<T> success(T data) {
        return new JsonResult<T>(CODE_SUCCESS, data, null);
    }

    public static <T> JsonResult<T> successOK(T data,String message) {
        return new JsonResult<T>("OK", data, message);
    }

    public static <T> JsonResult<T> clientFailed(String message) {
        return new JsonResult<T>(CODE_CLIENT_ERROR, null, message);
    }

    public static <T> JsonResult<T> failed(String code, String message) {
        return new JsonResult<T>(code, null, message);
    }

    /**
     * 失败返回结果
     *
     * @param message 提示信息
     */
    public static <T> JsonResult<T> failed(String code, String message, T data) {
        return new JsonResult(code, data, message);
    }
}
