package com.rootcloud.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.rootcloud.common.constant.LappLiveConstant;
import com.rootcloud.common.net.JsonHelper;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * 用于调取萤石api，实现视频直播操作。
 * @author fandi
 *
 */

@Component
public class LappLiveClient {

    private static Logger log = LoggerFactory.getLogger(LappLiveClient.class);

    private static CloseableHttpClient httpClient   = HttpClients.createDefault();

    /**
     * 获取
     * @param //url 萤石接口
     * @param //appKey
     * @param //appSecret
     * @return String
     */
    public String getAccessToken() throws Exception{
        if(StringUtils.isBlank(LappLiveConstant.GET_TOKEN_URL) || StringUtils.isBlank(LappLiveConstant.APP_KEY) || StringUtils.isBlank(LappLiveConstant.APP_SECRET)) {
            log.info("getAccessToken param can not be null| getAccessTokenUrl:{}|appKey:{}|appSecret:{}", LappLiveConstant.GET_TOKEN_URL, LappLiveConstant.APP_KEY, LappLiveConstant.APP_SECRET);
            return null;
        }

        log.info("getAccessTokenparam is | getAccessTokenUrl:{} | appKey:{} | appSecret:{} ", LappLiveConstant.GET_TOKEN_URL, LappLiveConstant.APP_KEY, LappLiveConstant.APP_SECRET);
        String getTokenUri = LappLiveConstant.GET_TOKEN_URL + "?appKey=" + LappLiveConstant.APP_KEY + "&appSecret=" + LappLiveConstant.APP_SECRET;
        HttpPost post = new HttpPost(getTokenUri);

        String message = "";
        post.setHeader("Content-type", "application/x-www-form-urlencoded");
        CloseableHttpResponse response = httpClient.execute(post);
        message = EntityUtils.toString(response.getEntity(), "utf-8");

        // 首先获取data数据，然后获取data中的accessToken中的数据
        String data = "";
        if(JsonHelper.getJsonObject(message).get("code").toString().equals("200")) {
            data = JsonHelper.getJsonObject(message).get("data").toString();
        }else {
            return null;
        }

        if(StringUtils.isBlank(data)) {
            return null;
        }

        // 获取accessToken的有效地址
        String accessToken = JsonHelper.getJsonObject(data).get("accessToken").toString();

        return accessToken;
    }

    /**
     * 获取
     * @param //url 乐橙接口
     * @param //appKey
     * @param //appSecret
     * @return String
     */
    public String getAccessTokenLeCheng() throws Exception{
        if(StringUtils.isBlank(LappLiveConstant.GET_TOKEN_LECHENG_URL) || StringUtils.isBlank(LappLiveConstant.APP_KEY_LECHENG) || StringUtils.isBlank(LappLiveConstant.APP_SECRET_LECHENG)) {
            log.error("getAccessToken param can not be null| getAccessTokenUrl:{}|appKey:{}|appSecret:{}", LappLiveConstant.GET_TOKEN_LECHENG_URL, LappLiveConstant.APP_KEY_LECHENG, LappLiveConstant.APP_SECRET_LECHENG);
            return null;
        }
        Map<String, Object> map = paramsInit();
        log.error("getAccessTokenparam is | getAccessTokenUrl:{} | appKey:{} | appSecret:{} ", LappLiveConstant.GET_TOKEN_LECHENG_URL, LappLiveConstant.APP_KEY_LECHENG, LappLiveConstant.APP_SECRET_LECHENG);
        //String getTokenUri = LappLiveConstant.GET_TOKEN_LECHENG_URL + "?appKey=" + LappLiveConstant.APP_KEY_LECHENG + "&appSecret=" + LappLiveConstant.APP_SECRET_LECHENG;
        String getTokenUri = LappLiveConstant.GET_TOKEN_LECHENG_URL;
        HttpClient client = HttpClients.createDefault();
        HttpPost post = new HttpPost(getTokenUri);
        post.setHeader("Content-Type", "application/json");
        post.setHeader("Accept", "application/json");

        long time = System.currentTimeMillis() / 1000;
        String nonce = UUID.randomUUID().toString();
        String id = UUID.randomUUID().toString();

        StringBuilder paramString = new StringBuilder();
        paramString.append("time:").append(time).append(",");
        paramString.append("nonce:").append(nonce).append(",");
        paramString.append("appSecret:").append(LappLiveConstant.APP_SECRET_LECHENG); //SECRET
        String sign = "";
        // 计算MD5得值
        try {
            //System.out.println("传入参数：" + paramString.toString().trim());
            sign = DigestUtils.md5Hex(paramString.toString().trim().getBytes("UTF-8"));
        } catch (Exception e) {
            log.error("md5 exception:"+e.toString());
            e.printStackTrace();
        }
        log.error("sign:"+sign);
        Map<String, Object> systemMap = new HashMap<String, Object>();
        systemMap.put("ver", "1.0");
        systemMap.put("sign", sign);
        systemMap.put("appId", LappLiveConstant.APP_KEY_LECHENG); //AppID
        systemMap.put("nonce", nonce);
        systemMap.put("time", time);

        // 构建JSON对象
        JsonObject systemObject = new JsonObject();
        systemObject.addProperty("ver", "1.0");
        systemObject.addProperty("appId", LappLiveConstant.APP_KEY_LECHENG);
        systemObject.addProperty("sign", sign);
        systemObject.addProperty("time", time);
        systemObject.addProperty("nonce", nonce);

        JsonObject jsonObject = new JsonObject();
        jsonObject.add("system", systemObject);
        jsonObject.addProperty("id", id);
        jsonObject.add("params", new JsonObject());

        String json = jsonObject.toString();
        StringEntity entity = new StringEntity(json);
        post.setEntity(entity);
        // 发送请求并获取响应
        HttpResponse response = client.execute(post);
        // 获取响应状态码
        int statusCode = response.getStatusLine().getStatusCode();
        String result = "";
        // 获取accessToken的有效地址
        String accessToken = "";
        try{
            if (statusCode == 200) {
                // 获取响应内容
                result = EntityUtils.toString(response.getEntity());
                //System.out.println("响应内容: " + result);
            } else {
                //System.err.println("HTTP请求失败，状态码: " + statusCode);
            }
            if(JsonHelper.getJsonObject(JsonHelper.getJsonObject(result).get("result")).get("code").toString().equals("0")){
                String data = JsonHelper.getJsonObject(JsonHelper.getJsonObject(result).get("result")).get("data").toString();
                //accessToken = JsonHelper.getJsonObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(result).get("result")).get("data")).get("accessToken");
                accessToken = JsonHelper.getJsonObject(data).get("accessToken").toString();
            }else {
                return null;
            }
            log.error("accessToken="+accessToken);
        }catch (Exception ex){
            log.error("accessToken exception:"+ex.toString());
        }
        return accessToken;
    }

    protected static Map<String, Object> paramsInit() {
        long time = System.currentTimeMillis() / 1000;
        String nonce = UUID.randomUUID().toString();
        String id = UUID.randomUUID().toString();

        StringBuilder paramString = new StringBuilder();
        paramString.append("time:").append(time).append(",");
        paramString.append("nonce:").append(nonce).append(",");
        paramString.append("appSecret:").append(LappLiveConstant.APP_SECRET_LECHENG); //SECRET

        String sign = "";
        // 计算MD5得值
        try {
            System.out.println("传入参数：" + paramString.toString().trim());
            sign = DigestUtils.md5Hex(paramString.toString().trim().getBytes("UTF-8"));
        } catch (Exception e) {
            e.printStackTrace();
        }

        Map<String, Object> systemMap = new HashMap<String, Object>();
        systemMap.put("ver", "1.0");
        systemMap.put("sign", sign);
        systemMap.put("appId", LappLiveConstant.APP_KEY_LECHENG); //AppID
        systemMap.put("nonce", nonce);
        systemMap.put("time", time);

        Map<String, Object> paramsMap = new HashMap<>();
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("system", systemMap);
        map.put("params", paramsMap);
        map.put("id", id);
        return map;
    }


    /**
     * 开通直播功能
     * @param accessToken 令牌号
     * @param //String url 萤石接口url
     * @param //String deviceSerial 设备序列号
     * @param //String channelNo 通道号
     * @return boolean (true表示成功，false表示失败)
     * @throws IOException
     * @throws ParseException
     */
    public String openLive(String url, String accessToken, String deviceSerial, String channelNo) throws ParseException, IOException {
        if(StringUtils.isBlank(url) || StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(accessToken) || StringUtils.isBlank(channelNo)) {
            log.info("openLive parame can not be null| accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);
            return null;
        }

        log.info("openLive parame is |accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);

        // 获取直播源，格式为：[设备序列号]:[通道号]
        String source = deviceSerial + ":" + channelNo;
        String openLiveUri = url + "?accessToken=" + accessToken + "&source=" + source;
        HttpPost post = new HttpPost(openLiveUri);

        String message = "";
        post.setHeader("Content-type", "application/x-www-form-urlencoded");
        CloseableHttpResponse response = httpClient.execute(post);
        message = EntityUtils.toString(response.getEntity(), "utf-8");
        return message;
    }

    /**
     * 获取直播地址
     * @return
     * @throws IOException
     * @throws ClientProtocolException
     */
    public String getLiveAddress(String url, String accessToken, String deviceSerial, String channelNo) throws ClientProtocolException, IOException{
        if(StringUtils.isBlank(url) || StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(accessToken) || StringUtils.isBlank(channelNo)) {
            log.info("getLiveAddress parame can not be null| accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);
            return null;
        }
        log.info("getLiveAddress parame is |accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);

        // 获取直播源，格式为：[设备序列号]:[通道号]
        String source = deviceSerial + ":" + channelNo;
        String getLiveAddressUri = url + "?accessToken="+ accessToken + "&source=" + source;
        HttpPost post = new HttpPost(getLiveAddressUri);

        String message = "";
        post.setHeader("Content-type", "application/x-www-form-urlencoded");
        CloseableHttpResponse response = httpClient.execute(post);
        message = EntityUtils.toString(response.getEntity(), "utf-8");

        return message;
    }

    public String getLiveAddressLeCheng0(String url, String accessToken, String deviceSerial, String channelNo) throws ClientProtocolException, IOException{
        if(StringUtils.isBlank(url) || StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(accessToken) || StringUtils.isBlank(channelNo)) {
            log.info("getLiveAddress parame can not be null| accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);
            return null;
        }
        log.info("getLiveAddress parame is |accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);
        //查询设备通道下创建的直播信息
        String getLiveAddressUri = "https://openapi.lechange.cn/openapi/getLiveStreamInfo";
        HttpClient client = HttpClients.createDefault();
        HttpPost post = new HttpPost(getLiveAddressUri);
        post.setHeader("Content-Type", "application/json");
        post.setHeader("Accept", "application/json");
        long time = System.currentTimeMillis() / 1000;
        String nonce = UUID.randomUUID().toString();
        String id = UUID.randomUUID().toString();

        StringBuilder paramString = new StringBuilder();
        paramString.append("time:").append(time).append(",");
        paramString.append("nonce:").append(nonce).append(",");
        paramString.append("appSecret:").append(LappLiveConstant.APP_SECRET_LECHENG); //SECRET
        String sign = "";
        // 计算MD5得值
        try {
            //System.out.println("传入参数：" + paramString.toString().trim());
            sign = DigestUtils.md5Hex(paramString.toString().trim().getBytes("UTF-8"));
        } catch (Exception e) {
            e.printStackTrace();
        }

        Map<String, Object> systemMap = new HashMap<String, Object>();
        systemMap.put("ver", "1.0");
        systemMap.put("sign", sign);
        systemMap.put("appId", LappLiveConstant.APP_KEY_LECHENG); //AppID
        systemMap.put("nonce", nonce);
        systemMap.put("time", time);

        // 构建JSON对象
        JsonObject systemObject = new JsonObject();
        systemObject.addProperty("ver", "1.0");
        systemObject.addProperty("appId", LappLiveConstant.APP_KEY_LECHENG);
        systemObject.addProperty("sign", sign);
        systemObject.addProperty("time", time);
        systemObject.addProperty("nonce", nonce);

        JsonObject jsonObject = new JsonObject();
        jsonObject.add("system", systemObject);
        jsonObject.addProperty("id", id);
        JsonObject paramsObject = new JsonObject();
        paramsObject.addProperty("deviceId", deviceSerial);
        paramsObject.addProperty("channelId", channelNo);
        paramsObject.addProperty("token", accessToken);
        jsonObject.add("params", paramsObject);

        String json = jsonObject.toString();
        StringEntity entity = new StringEntity(json);
        post.setEntity(entity);
        HttpResponse response = client.execute(post);
        int statusCode = response.getStatusLine().getStatusCode();
        String result = "";
        String message = "";
        if (statusCode == 200) {
            result = EntityUtils.toString(response.getEntity());
            System.out.println("响应内容: " + result);
            log.info("响应内容: " + result);
        } else {
            System.err.println("HTTP请求失败，状态码: " + statusCode);
            log.info("HTTP请求失败，状态码: " + statusCode);
        }
        return result;
    }


    public String getLiveAddressLeCheng1(String url, String accessToken, String deviceSerial, String channelNo) throws ClientProtocolException, IOException{
        if(StringUtils.isBlank(url) || StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(accessToken) || StringUtils.isBlank(channelNo)) {
            log.info("getLiveAddress parame can not be null| accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);
            return null;
        }
        log.info("getLiveAddress parame is |accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);

        // 获取直播源，格式为：[设备序列号]:[通道号]
        String getLiveAddressUri = url;
        HttpPost post = new HttpPost(getLiveAddressUri);

        Map<String, Object> map1 = paramsLiveInit1(deviceSerial,channelNo,accessToken);
        StringEntity entity = new StringEntity(map1.toString(), ContentType.APPLICATION_JSON);
        post.setEntity(entity);

        String message = "";
        post.setHeader("Content-type", "application/x-www-form-urlencoded");
        CloseableHttpResponse response = httpClient.execute(post);
        message = EntityUtils.toString(response.getEntity(), "utf-8");
        return message;
    }

    protected static Map<String, Object> paramsLiveInit1(String deviceSerial,String channelNo,String accessToken) {
        long time = System.currentTimeMillis() / 1000;
        String nonce = UUID.randomUUID().toString();
        String id = UUID.randomUUID().toString();

        StringBuilder paramString = new StringBuilder();
        paramString.append("time:").append(time).append(",");
        paramString.append("nonce:").append(nonce).append(",");
        paramString.append("appSecret:").append(LappLiveConstant.APP_SECRET_LECHENG); //SECRET

        String sign = "";
        // 计算MD5得值
        try {
            System.out.println("传入参数：" + paramString.toString().trim());
            sign = DigestUtils.md5Hex(paramString.toString().trim().getBytes("UTF-8"));
        } catch (Exception e) {
            e.printStackTrace();
        }

        Map<String, Object> systemMap = new HashMap<String, Object>();
        systemMap.put("ver", "1.0");
        systemMap.put("sign", sign);
        systemMap.put("appId", LappLiveConstant.APP_KEY_LECHENG); //AppID
        systemMap.put("nonce", nonce);
        systemMap.put("time", time);

        Map<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("streamId", "0"); //0:高清主码流 1:标清辅码流
        paramsMap.put("deviceId", deviceSerial); //设备序列号
        paramsMap.put("channelId", channelNo); //通道号
        paramsMap.put("token", accessToken);//管理员accessToken

        Map<String, Object> map = new HashMap<String, Object>();
        map.put("system", systemMap);
        map.put("params", paramsMap);
        map.put("id", id);
        return map;
    }

    protected static Map<String, Object> paramsLiveInit2(String deviceSerial,String channelNo,String accessToken) {
        long time = System.currentTimeMillis() / 1000;
        String nonce = UUID.randomUUID().toString();
        String id = UUID.randomUUID().toString();

        StringBuilder paramString = new StringBuilder();
        paramString.append("time:").append(time).append(",");
        paramString.append("nonce:").append(nonce).append(",");
        paramString.append("appSecret:").append(LappLiveConstant.APP_SECRET_LECHENG); //SECRET

        String sign = "";
        // 计算MD5得值
        try {
            System.out.println("传入参数：" + paramString.toString().trim());
            sign = DigestUtils.md5Hex(paramString.toString().trim().getBytes("UTF-8"));
        } catch (Exception e) {
            e.printStackTrace();
        }

        Map<String, Object> systemMap = new HashMap<String, Object>();
        systemMap.put("ver", "1.0");
        systemMap.put("sign", sign);
        systemMap.put("appId", LappLiveConstant.APP_KEY_LECHENG); //AppID
        systemMap.put("nonce", nonce);
        systemMap.put("time", time);

        Map<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("streamId", "1"); //0:高清主码流 1:标清辅码流
        paramsMap.put("deviceId", deviceSerial); //设备序列号
        paramsMap.put("channelId", channelNo); //通道号
        paramsMap.put("token", accessToken);//管理员accessToken

        Map<String, Object> map = new HashMap<String, Object>();
        map.put("system", systemMap);
        map.put("params", paramsMap);
        map.put("id", id);
        return map;
    }


    public String getLiveAddressLeCheng2(String url, String accessToken, String deviceSerial, String channelNo) throws ClientProtocolException, IOException{
        if(StringUtils.isBlank(url) || StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(accessToken) || StringUtils.isBlank(channelNo)) {
            log.info("getLiveAddress parame can not be null| accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);
            return null;
        }
        log.info("getLiveAddress parame is |accessToken:{}|deviceSerial:{}|channelNo:{}|url:{}", accessToken, deviceSerial, channelNo, url);

        // 获取直播源，格式为：[设备序列号]:[通道号]
        String getLiveAddressUri = url;
        HttpPost post = new HttpPost(getLiveAddressUri);

        Map<String, Object> map2 = paramsLiveInit2(deviceSerial,channelNo,accessToken);
        StringEntity entity = new StringEntity(map2.toString(), ContentType.APPLICATION_JSON);
        post.setEntity(entity);

        String message = "";
        post.setHeader("Content-type", "application/x-www-form-urlencoded");
        CloseableHttpResponse response = httpClient.execute(post);
        message = EntityUtils.toString(response.getEntity(), "utf-8");
        return message;
    }

}
