package com.rootcloud.common;

import com.alibaba.fastjson.JSONObject;

import java.io.Serializable;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.HashMap;

public class OAuthResult implements Serializable {

    private String access_token;

    private Integer expires_in;

    private String refresh_token;

    private String code;

    private String value;

    private String message;

    private HashMap additionalInformation;

    //契合云杉登录的相关属性（赋默认值）
    private Boolean isSystemAdmin;
    private String scope;
    private String tenantId;
    private String token_type;

    public OAuthResult() {

    }


    public OAuthResult(JSONObject res) {
        this.access_token = res.getString("access_token");
        this.refresh_token = res.getString("refresh_token");
        this.expires_in = res.getIntValue("expires_in");
        this.isSystemAdmin = false;
        this.scope = "read";
        this.tenantId = "";
        this.token_type = "bearer";
    }

    public OAuthResult(String auth) {
        this.access_token = auth;
        this.refresh_token = "";
        this.expires_in = 3000;
        this.isSystemAdmin = false;
        this.scope = "read";
        this.tenantId = "";
        this.token_type = "bearer";
    }

    public OAuthResult(String accessToken, Integer expiresIn, String refreshToken) {
        this.access_token = accessToken;
        this.expires_in = expiresIn;
        this.refresh_token = refreshToken;
    }

    public String getExpiresAt() {
        Instant instant = Instant.now();
        Instant plus = instant.plus(expires_in, ChronoUnit.SECONDS);
        return String.valueOf(plus.getEpochSecond());
    }

    public String getAccess_token() {
        return access_token;
    }

    public void setAccess_token(String access_token) {
        this.access_token = access_token;
    }

    public Integer getExpires_in() {
        return expires_in;
    }

    public void setExpires_in(Integer expires_in) {
        this.expires_in = expires_in;
    }

    public String getRefresh_token() {
        return refresh_token;
    }

    public void setRefresh_token(String refresh_token) {
        this.refresh_token = refresh_token;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public HashMap getAdditionalInformation() {
        return additionalInformation;
    }

    public void setAdditionalInformation(HashMap additionalInformation) {
        this.additionalInformation = additionalInformation;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public Boolean getSystemAdmin() {
        return isSystemAdmin;
    }

    public void setSystemAdmin(Boolean systemAdmin) {
        isSystemAdmin = systemAdmin;
    }

    public String getScope() {
        return scope;
    }

    public void setScope(String scope) {
        this.scope = scope;
    }

    public String getTenantId() {
        return tenantId;
    }

    public void setTenantId(String tenantId) {
        this.tenantId = tenantId;
    }

    public String getToken_type() {
        return token_type;
    }

    public void setToken_type(String token_type) {
        this.token_type = token_type;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }
}
