package com.rootcloud.common;

import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotNull;

public class PageAO {

    @ApiModelProperty("当前页数")
    @NotNull(message = "当前页数不能为空")
    private Integer pageNum;

    @ApiModelProperty("分页大小")
    @NotNull(message = "分页大小不能为空")
    private Integer pageSize;

    /**
     * 默认页
     */
    private static final int DEFAULT_PAGE = 1;

    /**
     * 每页大小
     */
    private static final int DEFAULT_SIZE = 12;

    public Integer getPageNum() {
        if (this.pageNum == null){
            return DEFAULT_PAGE;
        }
        return Math.max(DEFAULT_PAGE, pageNum);
    }

    public Integer getPageSize() {

        if (this.pageSize == null){
            return DEFAULT_SIZE;
        }
        // 检验pageSize小于等于0时，给默认大小
        if (pageSize <= 0){
            return Math.max(pageSize, DEFAULT_SIZE);
        }
        // 校验页面大小不能大于DEFAULT_SIZE
        return Math.min(pageSize, DEFAULT_SIZE);
    }

}
