package com.rootcloud.common;

import com.alibaba.fastjson.JSONObject;
import com.ejlchina.http.HTTP;
import com.rootcloud.common.constant.DeviceConstant;
import com.rootcloud.common.constant.RootCloudApi;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * TODO
 *
 * @author wei.liao
 * @version 1.0
 * @date 2022/4/23 16:26
 */
@Component
@Slf4j
public class RootCloudOpenAPIUtil {

    @Value("${os.iam.domain}")
    private String reqUrl;

    @Value("${os.iam.superToken}")
    private String superToken;

    @Autowired
    private TokenUtil tokenUtil;

    @Value("${os.iam.uri:http://iothub-historian.rootcloud-platform-pre.rootcloudapp.com}")
    private String openAPIUri;

    protected Map<String, String> defaultHeader() {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer " + tokenUtil.getToken());
        return headers;
    }

    protected HTTP httpClient(String baseUrl) {
        return HTTP.builder().config((OkHttpClient.Builder builder) -> {
            // 连接超时时间（默认10秒）
            builder.connectTimeout(5, TimeUnit.SECONDS);
            // 写入超时时间（默认10秒）
            builder.writeTimeout(5, TimeUnit.SECONDS);
            // 读取超时时间（默认10秒）
            builder.readTimeout(5, TimeUnit.SECONDS);
        }).baseUrl(baseUrl).build();
    }

    public Integer getDeviceCurrentStatus(String thingId) {
        JSONObject device = this.httpClient(reqUrl).sync(RootCloudApi.DEVICE_CURRENT_STATUS_URI).addHeader(this.defaultHeader()).addUrlParam("thingId", thingId).get().getBody().toJsonObject();
        log.info("平台设备信息:[{}]", device);
        Object online = device.get("online");
        if (online != null) {
            if (online instanceof Boolean) {
                return (Boolean) online ? DeviceConstant.ONLINE_STATUS_CODE : DeviceConstant.OFFLINE_STATUS_CODE;
            }
        }
        return DeviceConstant.OFFLINE_STATUS_CODE;
    }

    public Integer getCountUnclosedDevice() {
        JSONObject device = this.httpClient(openAPIUri).sync(RootCloudApi.DEVICE_UNCLOSE_COUNT_URI).addHeader(this.defaultHeader()).get().getBody().toJsonObject();
        log.info("平台设备信息:[{}]", device);
        Object count = device.get("count");
        if (count != null) {
            return Integer.parseInt((String) count);
        }
        return 0;
    }
}
