package com.rootcloud.common;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSONArray;
//import jline.internal.Log;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Slf4j
@Component
public class SendMessageUtils {

    @Value(value = "${sms.server-url}")
    private String  SERVER_URL;

    @Value(value = "${sms.app-key}")
    private String  APP_KEY;

    @Value(value = "${sms.app-secret}")
    private String  APP_SECRET;

    @Value(value = "${sms.nonce}")
    private String  NONCE;

    @Value(value = "${sms.templateid}")
    private String  TEMPLATEID;

    public  void send(List<String> mobiles,List<String> params) {
        this.send(mobiles,params,null);
    }

    public  void send(List<String> mobiles,List<String> params,String templateId) {
        DefaultHttpClient httpClient = new DefaultHttpClient();
        HttpPost httpPost = new HttpPost(SERVER_URL);
        String curTime = String.valueOf((new Date()).getTime() / 1000L);
        /*
         * 参考计算CheckSum的java代码，在上述文档的参数列表中，有CheckSum的计算文档示例
         */
        String checkSum = CheckSumBuilder.getCheckSum(APP_SECRET, NONCE, curTime);

        // 设置请求的header
        httpPost.addHeader("AppKey", APP_KEY);
        httpPost.addHeader("Nonce", NONCE);
        httpPost.addHeader("CurTime", curTime);
        httpPost.addHeader("CheckSum", checkSum);
        httpPost.addHeader("Content-Type", "application/x-www-form-urlencoded;charset=utf-8");

        // 设置请求的的参数，requestBody参数
        List<NameValuePair> nvps = new ArrayList<NameValuePair>();
        /*
         * 1.如果是模板短信，请注意参数mobile是有s的，详细参数配置请参考“发送模板短信文档”
         * 2.参数格式是jsonArray的格式，例如 "['13888888888','13666666666']"
         * 3.params是根据你模板里面有几个参数，那里面的参数也是jsonArray格式
         */
        if (StringUtils.isNotEmpty(templateId)){
            nvps.add(new BasicNameValuePair("templateid", templateId));
        }else {
            nvps.add(new BasicNameValuePair("templateid", TEMPLATEID));
        }
        nvps.add(new BasicNameValuePair("mobiles",  JSONArray.toJSONString(mobiles)));
        nvps.add(new BasicNameValuePair("params",  JSONArray.toJSONString(params)));
        try{
            httpPost.setEntity(new UrlEncodedFormEntity(nvps, "utf-8"));
            // 执行请求
            HttpResponse response = httpClient.execute(httpPost);
            /*
             * 1.打印执行结果，打印结果一般会200、315、403、404、413、414、500
             * 2.具体的code有问题的可以参考官网的Code状态表
             */
            //System.out.println(EntityUtils.toString(response.getEntity(), "utf-8"));
            log.info("调用网易云信接口参数：{}，返回值:{}",httpPost.getEntity(),EntityUtils.toString(response.getEntity(), "utf-8"));
           // Log.info(EntityUtils.toString(response.getEntity(), "utf-8"));
        }catch (IOException e){
            e.printStackTrace();
            log.error("发送短信失败！");
        }

    }


}
