package com.rootcloud.common;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.domain.mapper.CommonMapper;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Component
public class TokenUtil {
    @Resource
    private RedisTemplate redisTemplate;
    @Value(value = "${os.iam.clientid}")
    private String clientId;
    @Value(value = "${os.iam.clientsecret}")
    private String clientSecret;
    @Value(value = "${os.iam.domain}")
    private String domain;
    @Value(value = "${os.iam.loginuri}")
    private String loginUri;


//    public String superToken(){
//        return "aHViLXN1cGVyLXVzZXI";
//    }

    @Autowired
    private CommonMapper commonMapper;

    public String getToken(String clientId, String clientSecret) {
        String result = "";
        JSONObject json = new JSONObject();
        json.set("grant_type", "client_credentials");
        json.set("client_id", clientId);
        json.set("client_secret", clientSecret);
        HttpRequest request = HttpUtil.createPost(domain + loginUri);
        request.body(json.toString(), "application/json");
        HttpResponse response = request.execute();
        result = response.body();
        JSONObject json2 = new JSONObject(result);
        result = json2.get("access_token").toString();
        return result;
    }

    public String getToken(String tenantId) {
        String rediskey = "common_accessToken";
        if (StringUtils.isNotBlank(tenantId)) {
            rediskey = tenantId + "_accessToken";
        }
        Object token = null;
        try {
            token = redisTemplate.opsForValue().get(rediskey);
        } catch (Exception e) {
            redisTemplate.delete(rediskey);
        }
        if (token != null) {
            return token.toString();
        } else {
            String result = "";
            Map<String, String> rootClient = commonMapper.getRootClient(tenantId);
            if (rootClient != null) {
                clientId = rootClient.get("CLIENT_ID");
                clientSecret = rootClient.get("CLIENT_SECRET");
                JSONObject json = new JSONObject();
                json.set("grant_type", "client_credentials");
                json.set("client_id", clientId);
                json.set("client_secret", clientSecret);
                HttpRequest request = HttpUtil.createPost(domain + loginUri);
                request.body(json.toString(), "application/json");
                HttpResponse response = request.execute();
                result = response.body();
                JSONObject json2 = new JSONObject(result);
                result = json2.get("access_token").toString();
                redisTemplate.opsForValue().set(rediskey, result, 3500, TimeUnit.SECONDS);
            } else {
                Log.error("查询clientId为null,租户id：" + tenantId);
            }
            return result;
        }
    }


    /*public String getToken() {
        String tenantId = UserContext.getTenantId();  //获取当前租户
        String rediskey = "common_accessToken";
        if (StringUtils.isNotBlank(tenantId)) {
            rediskey = tenantId + "_accessToken";
        }
        Object token = null;
        try {
            token = redisTemplate.opsForValue().get(rediskey);
        } catch (Exception e) {
            redisTemplate.delete(rediskey);
        }
        if (token != null) {
            return token.toString();
        } else {
            String result = "";
            Map<String, String> rootClient = commonMapper.getRootClient(tenantId);
//            rootClient = new HashMap<>();  测试使用
//            rootClient.put("CLIENT_ID","2022041578413b61ae181695");
//            rootClient.put("CLIENT_SECRET","a7ed2906927805a0f6d4fa0019ea017f");

            if (rootClient != null) {
                clientId = rootClient.get("CLIENT_ID");
                clientSecret = rootClient.get("CLIENT_SECRET");
                JSONObject json = new JSONObject();
                json.set("grant_type", "client_credentials");
                json.set("client_id", clientId);
                json.set("client_secret", clientSecret);
                Log.info("获取参数：" + json.toString());
                HttpRequest request = HttpUtil.createPost(domain + loginUri);
                Log.info("获取路径：" + domain + loginUri);
                request.body(json.toString(), "application/json");
                try {
                    HttpResponse response = request.execute();
                    result = response.body();
                    JSONObject json2 = new JSONObject(result);
                    result = json2.get("access_token").toString();
                    redisTemplate.opsForValue().set(rediskey, result, 3500, TimeUnit.SECONDS);
                } catch (Exception e) {
                    Log.error("获取access_token失败，失败原因：" + e.getMessage());
                }
            } else {
                Log.error("查询clientId为null,租户id：" + tenantId);
            }
            return result;
        }
    }*/

    public String getToken() {
        String tenantId = UserContext.getTenantId();  //获取当前租户
        String rediskey = "common_accessToken";
        if(StringUtils.isNotBlank(tenantId)){
            rediskey= tenantId+"_accessToken";
        }
        Object token = null;
        try{
            token = redisTemplate.opsForValue().get(rediskey);
        }catch (Exception e){
            redisTemplate.delete(rediskey);
        }
        if (token != null) {
            return token.toString();
        } else {
            String result = "";
            Map<String,String> rootClient = commonMapper.getRootClient(tenantId);
            /*Map<String,String> rootClient  = new HashMap<>();  //测试使用
            rootClient.put("CLIENT_ID","20221206c509e669fe968c13");
            rootClient.put("CLIENT_SECRET","b9ad3dddce304e311439ee8ad7202a36");*/
            if(rootClient!=null) {
                clientId = rootClient.get("CLIENT_ID");
                clientSecret = rootClient.get("CLIENT_SECRET");
                JSONObject json = new JSONObject();
                json.set("grant_type", "client_credentials");
                json.set("client_id", clientId);
                json.set("client_secret", clientSecret);
                HttpRequest request = HttpUtil.createPost(domain + loginUri);
                Log.info("访问路径："+domain + loginUri+"，访问参数："+json);
                request.body(json.toString(), "application/json");
                HttpResponse response = request.execute();
                result = response.body();
                JSONObject json2 = new JSONObject(result);
                result = json2.get("access_token").toString();
                redisTemplate.opsForValue().set(rediskey, result, 3500, TimeUnit.SECONDS);
            }else{
                Log.error("查询clientId为null,租户id："+tenantId);
            }
            return result;
        }
    }


    public static void main(String[] args) {
        Map<String, String> rootClient = new HashMap<>();  //测试使用
        rootClient.put("CLIENT_ID", "20221029150bbb6e7936972d");
        rootClient.put("CLIENT_SECRET", "a76b4a99495e8b94154b499e88f13d00");
        if (rootClient != null) {
          String   clientId = rootClient.get("CLIENT_ID");
            String clientSecret = rootClient.get("CLIENT_SECRET");
            JSONObject json = new JSONObject();
            json.set("grant_type", "client_credentials");
            json.set("client_id", clientId);
            json.set("client_secret", clientSecret);
            Log.info("获取参数：" + json.toString());
            HttpRequest request = HttpUtil.createPost("https://openapi.zengcaicloud.com/account-manage/v1/auth/login");
            request.body(json.toString(), "application/json");
            try {
                HttpResponse response = request.execute();
                String result = response.body();
                JSONObject json2 = new JSONObject(result);
                result = json2.get("access_token").toString();
                System.out.println(result);
            } catch (Exception e) {
                Log.error("获取access_token失败，失败原因：" + e.getMessage());
            }
        }
    }
}
