package com.rootcloud.common;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.rootcloud.domain.entity.NxPartFileEntity;
import com.rootcloud.domain.entity.NxPartRespEntity;
import com.rootcloud.domain.entity.Part;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPMessage;
import java.io.*;
import java.math.BigDecimal;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class WebServiceClient {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private String wsdlLocation;// webservice地址
    private String action;// 命名空间
    private String methodName;// 调用方法
    private Map<String, String> params;// 参数键值对

    /**
     * 初始化调用webservice参数
     *
     * @param wsdlLocation webservice地址
     * @param namespace    命名空间
     * @param methodName   调用方法
     * @param params       参数键值对
     */
    public WebServiceClient(String wsdlLocation, String action, String methodName, Map<String, String> params) {
        this.wsdlLocation = wsdlLocation;
        this.action = action;
        this.methodName = methodName;
        this.params = params;
    }

    /**
     * 获取SOAP的请求头，并替换其中的标志符号为用户输入的参数及数据
     *
     * @param params 用户输入的参数键值对
     * @return 客户将要发送给服务器的SOAP请求
     */
    private String getSoapRequestSap() {
        StringBuilder soapRequestData = new StringBuilder();
        soapRequestData = new StringBuilder();
        soapRequestData.append("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:urn=\"urn:sap-com:document:sap:rfc:functions\">");
        soapRequestData.append("<soapenv:Header/>");
        soapRequestData.append("<soapenv:Body>");
        soapRequestData.append("<urn:" + methodName + ">");
        Set<String> nameSet = params.keySet();
        for (String name : nameSet) {
            soapRequestData.append("<" + name + ">" + params.get(name) + "</" + name + ">");
        }
        soapRequestData.append("</urn:" + methodName + ">");
        soapRequestData.append("</soapenv:Body>");
        soapRequestData.append("</soapenv:Envelope>");
        return soapRequestData.toString();
    }

    /**
     * 用户把SOAP请求发送给服务器端，并返回服务器点返回的输入流
     *
     * @return 服务器端返回的输入流，供客户端读取
     * @throws Exception
     */
    public InputStream getSoapInputStreamBySap() throws Exception {
        try {
            String soap = getSoapRequestSap();
            if (logger.isDebugEnabled()) {
                logger.debug("调用SAP请求报文内容: {}", soap);
            }
            URL url = new URL(wsdlLocation);
            URLConnection conn = url.openConnection();
            conn.setUseCaches(false);
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setRequestProperty("Content-Length", Integer.toString(soap.length()));
            conn.setRequestProperty("Content-Type", "text/xml;charset=UTF-8");
            conn.setConnectTimeout(100000000);
            conn.setReadTimeout(100000000);
            conn.setRequestProperty("SOAPAction", action);
            //认证   用户名:密码 base64
            conn.setRequestProperty("Authorization", "Basic cHJkX3Bpc3VwZXI6aGFuZGhhbmQw");
            OutputStream os = conn.getOutputStream();
            OutputStreamWriter osw = new OutputStreamWriter(os, StandardCharsets.UTF_8);
            osw.write(soap);
            osw.flush();
            osw.close();
            InputStream is = conn.getInputStream();
            return is;
        } catch (Exception e) {
            logger.info("服务端返回异常", e);
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 对服务器端返回的XML进行解析
     */
    public List<NxPartFileEntity> getPartData() {
        List<NxPartFileEntity> partList = new ArrayList<NxPartFileEntity>();
        try {
//            InputStream is = getSoapInputStreamBySap();
            //出入流直接传承xml字符串
//            String result = convertStreamToString(is);
//            logger.info("获取xml数据{}",result);
            //String ji = "<SOAP:Envelope xmlns:SOAP='http://schemas.xmlsoap.org/soap/envelope/'><SOAP:Header/><SOAP:Body><ns0:ZDM_INVENTORYINFResponse xmlns:ns0='urn:sap-com:document:sap:rfc:functions'><ZTDM_INVENTORYINF><item><MATNR>000000000001165762</MATNR><MAKTX>三元乙丙橡胶发泡（硬度20）\\415*87*25</MAKTX><MTART>Z01</MTART><GROES>415*87*25</GROES><MATKL>D01004</MATKL><MEINS>EA</MEINS><NTGEW>0</NTGEW><GEWEI>KG</GEWEI><ZEINR/><BISMT/><CHARG/><WERKS>1430</WERKS><LGORT>1004</LGORT><LABST>50.000</LABST><LVORM/><ADD1>                                           973.45</ADD1><ADD2/><ADD3/><ADD4>000000</ADD4><ADD5/><ADD6/><ADD7/><ADD8>                                            19.47</ADD8><ADD9/><ADD10/></item><item><MATNR>000000000001096794</MATNR><MAKTX>气1缸\\MGPM25-125Z-XC22</MAKTX><MTART>Z01</MTART><GROES>MGPM25-125Z-XC22</GROES><MATKL>D04000</MATKL><MEINS>EA</MEINS><NTGEW>0</NTGEW><GEWEI>KG</GEWEI><ZEINR/><BISMT/><CHARG/><WERKS>1430</WERKS><LGORT>1004</LGORT><LABST>3.000</LABST><LVORM/><ADD1>                                          4348.67</ADD1><ADD2/><ADD3/><ADD4>000000</ADD4><ADD5/><ADD6/><ADD7/><ADD8>                                           724.78</ADD8><ADD9/><ADD10/></item><item><MATNR>000000000001102941</MATNR><MAKTX>压力传感器\\ZSE30AF-01-D-MGA2K-X580</MAKTX><MTART>Z01</MTART><GROES>ZSE30AF-01-D-MGA2K-X580</GROES><MATKL>D01004</MATKL><MEINS>EA</MEINS><NTGEW>0</NTGEW><GEWEI>KG</GEWEI><ZEINR/><BISMT/><CHARG/><WERKS>1430</WERKS><LGORT>1004</LGORT><LABST>4.000</LABST><LVORM/><ADD1>                                         68134.48</ADD1><ADD2/><ADD3/><ADD4>000000</ADD4><ADD5/><ADD6/><ADD7/><ADD8>                                          2129.20</ADD8><ADD9/><ADD10/></item></ZTDM_INVENTORYINF></ns0:ZDM_INVENTORYINFResponse></SOAP:Body></SOAP:Envelope>";
            //logger.info("获取xml数据{}",result);
            //SOAPMessage msg = formatSoapString(new ByteArrayInputStream(ji.getBytes(StandardCharsets.UTF_8)));
            SOAPMessage msg = formatSoapString(getSoapInputStreamBySap());
            if (msg == null) {
                return new ArrayList<>();
            }
            SOAPBody body = msg.getSOAPBody();
            if (logger.isTraceEnabled()) {
                logger.trace("调用SAP获取配件结果: {}", body);
            }
            NodeList nodes = body.getFirstChild().getFirstChild().getChildNodes();
            if (logger.isDebugEnabled()) {
                logger.debug("调用SAP获取配件结果: {}", nodes);
            }
            for (int count = 0; count < nodes.getLength(); count++) {
                Node node = nodes.item(count);
                NodeList childNodes = node.getChildNodes();
                NxPartFileEntity part = new NxPartFileEntity();
                for (int i = 0; i < childNodes.getLength(); i++) {
                    Node childNode = childNodes.item(i);
                    String NodeName = childNode.getNodeName();
                    if (null == childNode.getFirstChild()) {
                        continue;
                    }
//                    logger.info("配件数据=======，{}", childNode.getFirstChild().getTextContent());
                    part.setNxPartFileId(JEUuidUtil.uuid());
                    part.setSyCreatetime(DateUtil.getCurrentTime());
                    part.setSyCreateuser("system");
                    part.setFileSfsgjpj("0");
                    if ("MATNR".equals(NodeName)) {  //物料编号
                        String code = childNode.getFirstChild().getTextContent();
                        code = code.replaceFirst("^0*", "");
                        part.setFilePjbm(code);
                    } else if ("MEINS".equals(NodeName)) {//基本计量单位 ==> 单位
                        part.setFileDw(childNode.getFirstChild().getTextContent());
                    } else if ("LABST".equals(NodeName)) {//库存数量
                        String strSl = childNode.getFirstChild().getTextContent();
                        if (strSl == null) break;
                        String replace = strSl.replace(" ", "");
                        BigDecimal kcsl = new BigDecimal(replace);
                        part.setFileKcsl(kcsl);
                    } else if ("ADD8".equals(NodeName)) {//实际价格==> 单位
                        String strPrice = childNode.getFirstChild().getTextContent();
                        if (strPrice == null) break;
                        String replace = strPrice.replace(" ", "");
                        BigDecimal price = new BigDecimal(replace);
                        part.setFileDj(price);
                    } else if ("MAKTX".equals(NodeName)) {//规格型号和配件名称
                        String content = childNode.getFirstChild().getTextContent();
                        String[] split = content.split("\\\\");
                        part.setFilePjmc(split[0]);
                        part.setFileGgxh(content);
                    } else if ("WERKS".equals(NodeName)) {//仓库名称和编码
                        String content = childNode.getFirstChild().getTextContent();
                        part.setFileCkbm(content);
                        part.setFileCkmc(content);
                    } else if ("LGORT".equals(NodeName)) {//库位编码
                        String content = childNode.getFirstChild().getTextContent();
                        part.setFileKwbm(content);
                    }
                }

//                if (!"1004".equals(part.getFileCkbm())) {
//                    continue;
//                }
                logger.info("规格型号配件名称=======，{}", JSONObject.toJSON(part));
                if (StringUtils.isNotBlank(part.getFilePjbm())) {
                    partList.add(part);
                }
            }
        } catch (Exception e) {
            logger.info("解析异常", e);
            e.printStackTrace();
        }
        if (logger.isDebugEnabled()) {
            logger.debug("处理结果: {}", JSONObject.toJSONString(partList));
        }
        logger.info("获取数据大小为{}", partList.size());
        return partList;
    }


    public List<Part> getPartData2() {
        List<Part> partList = new ArrayList<Part>();
        try {
            InputStream is = getSoapInputStreamBySap();
            //出入流直接传承xml字符串
            //String result = convertStreamToString(is);
            //logger.info("获取xml数据{}",result);
            SOAPMessage msg = formatSoapString(is);
            SOAPBody body = msg.getSOAPBody();

            logger.info("调用SAP获取配件xml:" + body);
            if (logger.isTraceEnabled()) {
                logger.trace("调用SAP获取配件结果: {}", body);
            }

            NodeList nodes = body.getFirstChild().getFirstChild().getChildNodes();
            if (logger.isDebugEnabled()) {
                logger.debug("调用SAP获取配件结果: {}", nodes);
            }

            for (int count = 0; count < nodes.getLength(); count++) {
                Node node = nodes.item(count);
                NodeList childNodes = node.getChildNodes();
                Part part = new Part();

                String MATNR_code = "";
                String MEINS = "";
                String ZENDME = "";
                double a = 0.0;
                BigDecimal ZSJJG_price = new BigDecimal(a);
                String LWEDT = "";
                String WERKS = "";
                String LGORT = "";


                for (int i = 0; i < childNodes.getLength(); i++) {
                    Node childNode = childNodes.item(i);
                    String NodeName = childNode.getNodeName();
                    if (null == childNode.getFirstChild()) {
                        continue;
                    }

                    if ("MATNR".equals(NodeName)) {  //物料编号
                        String code = childNode.getFirstChild().getTextContent();
                        MATNR_code = code.replaceFirst("^0*", "");
                    } else if ("MEINS".equals(NodeName)) {//基本计量单位 ==> 单位
                        MEINS = childNode.getFirstChild().getTextContent();
                    } else if ("ZENDME".equals(NodeName)) {//库存数量 => 单位说明
                        ZENDME = childNode.getFirstChild().getTextContent();
                    } else if ("ZSJJG".equals(NodeName)) {//实际价格==> 单位
                        String strPrice = childNode.getFirstChild().getTextContent();
                        ZSJJG_price = new BigDecimal(strPrice);
                    } else if ("LWEDT".equals(NodeName)) {//入库日期 ==> 重量单位说明
                        LWEDT = childNode.getFirstChild().getTextContent();
                    } else if ("WERKS".equals(NodeName)) {
                        WERKS = childNode.getFirstChild().getTextContent();
                        //logger.info("工厂编号:"+WERKS);
                    } else if ("LGORT".equals(NodeName)) {
                        LGORT = childNode.getFirstChild().getTextContent();
                        //logger.info("库房编号:"+LGORT);
                    }
                }
                //判断当前item集合中是否出现1430库下面的1004
                if (WERKS.equals("1430")) {
                    if (LGORT.equals("1004")) {
                        part.setPartCode(MATNR_code);
                        part.setCode(MATNR_code + ".0");
                        part.setUnit(MEINS); //单位
                        part.setUnitDesc("");//单位说明
                        try {
                            float countValue = Float.parseFloat(ZENDME);//库存数量
                            part.setPartInventoryCount((int) countValue);
                        } catch (Exception ex) {
                            logger.info("库存转换异常..." + ex.toString());
                        }
                        part.setPrice(ZSJJG_price);
                        part.setWeightUnitDesc(LWEDT);
                        logger.info("1430工厂编号1004 库房编号找到了...");
                    }
                } else {
                    part.setPartCode(MATNR_code);
                    part.setCode(MATNR_code + ".0");
                    part.setUnit(MEINS); //单位
                    part.setUnitDesc("");//单位说明
                    try {
                        float countValue = Float.parseFloat(ZENDME);//库存数量
                        part.setPartInventoryCount((int) countValue);
                    } catch (Exception ex) {
                        logger.info("库存转换异常..." + ex.toString());
                    }
                    part.setPrice(ZSJJG_price);
                    part.setWeightUnitDesc(LWEDT);
//					logger.info("工厂编号:"+WERKS);
//					logger.info("库房编号:"+LGORT);
                }
                if (StringUtils.isNotBlank(part.getPartCode())) {
                    partList.add(part);
                }
            }
        } catch (Exception e) {
            logger.info("解析异常", e);
            e.printStackTrace();
        }

        if (logger.isDebugEnabled()) {
            logger.debug("处理结果: {}", JSONObject.toJSONString(partList));
        }

        logger.info("获取数据大小为{}", partList.size());
        return partList;
    }

    //专门刷新1430/1004库存数据
    public List<Part> getPartData3() {
        List<Part> partList = new ArrayList<Part>();
        try {
            InputStream is = getSoapInputStreamBySap();
            //出入流直接传承xml字符串
            //String result = convertStreamToString(is);
            //logger.info("获取xml数据{}",result);
            SOAPMessage msg = formatSoapString(is);
            SOAPBody body = msg.getSOAPBody();

            logger.info("调用SAP获取配件xml:" + body);
            if (logger.isTraceEnabled()) {
                logger.trace("调用SAP获取配件结果: {}", body);
            }

            NodeList nodes = body.getFirstChild().getFirstChild().getChildNodes();
            if (logger.isDebugEnabled()) {
                logger.debug("调用SAP获取配件结果: {}", nodes);
            }

            for (int count = 0; count < nodes.getLength(); count++) {
                Node node = nodes.item(count);
                NodeList childNodes = node.getChildNodes();
                Part part = new Part();

                String MATNR_code = "";
                String MEINS = "";
                String ZENDME = "";
                double a = 0.0;
                BigDecimal ZSJJG_price = new BigDecimal(a);
                String LWEDT = "";
                String WERKS = "";
                String LGORT = "";


                for (int i = 0; i < childNodes.getLength(); i++) {
                    Node childNode = childNodes.item(i);
                    String NodeName = childNode.getNodeName();
                    if (null == childNode.getFirstChild()) {
                        continue;
                    }

                    if ("MATNR".equals(NodeName)) {  //物料编号
                        String code = childNode.getFirstChild().getTextContent();
                        MATNR_code = code.replaceFirst("^0*", "");
                    } else if ("MEINS".equals(NodeName)) {//基本计量单位 ==> 单位
                        MEINS = childNode.getFirstChild().getTextContent();
                    } else if ("ZENDME".equals(NodeName)) {//库存数量 => 单位说明
                        ZENDME = childNode.getFirstChild().getTextContent();
                    } else if ("ZSJJG".equals(NodeName)) {//实际价格==> 单位
                        String strPrice = childNode.getFirstChild().getTextContent();
                        ZSJJG_price = new BigDecimal(strPrice);
                    } else if ("LWEDT".equals(NodeName)) {//入库日期 ==> 重量单位说明
                        LWEDT = childNode.getFirstChild().getTextContent();
                    } else if ("WERKS".equals(NodeName)) {
                        WERKS = childNode.getFirstChild().getTextContent();
                        //logger.info("工厂编号:"+WERKS);
                    } else if ("LGORT".equals(NodeName)) {
                        LGORT = childNode.getFirstChild().getTextContent();
                        //logger.info("库房编号:"+LGORT);
                    }
                }
                //判断当前item集合中是否出现1430库下面的1004
                if (WERKS.equals("1430")) {
                    if (LGORT.equals("1004")) {
                        part.setPartCode(MATNR_code);
                        part.setCode(MATNR_code + ".0");
                        part.setUnit(MEINS); //单位
                        part.setUnitDesc("");//单位说明
                        try {
                            float countValue = Float.parseFloat(ZENDME);//库存数量
                            part.setPartInventoryCount((int) countValue);
                        } catch (Exception ex) {
                            logger.info("库存转换异常..." + ex.toString());
                        }
                        part.setPrice(ZSJJG_price);
                        part.setWeightUnitDesc(LWEDT);
                        logger.info("1430工厂编号1004 库房编号找到了...");
                    }
                }
//				else
//				{
//					part.setPartCode(MATNR_code);
//					part.setCode(MATNR_code+".0");
//					part.setUnit(MEINS); //单位
//					part.setUnitDesc("");//单位说明
//					try
//					{
//						float countValue = Float.parseFloat(ZENDME);//库存数量
//						part.setPartInventoryCount((int) countValue);
//					}catch (Exception ex)
//					{
//						logger.info("库存转换异常..."+ex.toString());
//					}
//					part.setPrice(ZSJJG_price);
//					part.setWeightUnitDesc(LWEDT);
////					logger.info("工厂编号:"+WERKS);
////					logger.info("库房编号:"+LGORT);
//				}
                if (StringUtils.isNotBlank(part.getPartCode())) {
                    partList.add(part);
                }
            }
        } catch (Exception e) {
            logger.info("解析异常", e);
            e.printStackTrace();
        }

        if (logger.isDebugEnabled()) {
            logger.debug("处理结果: {}", JSONObject.toJSONString(partList));
        }

        logger.info("获取数据大小为{}", partList.size());
        return partList;
    }

    @SuppressWarnings("unused")
    private String convertStreamToString(InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is));
        StringBuilder sb = new StringBuilder();
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line);
            }
        } catch (IOException e) {
            e.printStackTrace();

        } finally {
            try {
                reader.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }

    private static SOAPMessage formatSoapString(InputStream is) {
        MessageFactory msgFactory;
        try {
            msgFactory = MessageFactory.newInstance();
            SOAPMessage reqMsg = msgFactory.createMessage(new MimeHeaders(), is);
            reqMsg.saveChanges();
            return reqMsg;
        } catch (Exception e) {
            Log.error("获取数据失败，失败原因：" + e.getMessage());
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 判断是否为数字
     *
     * @param str
     * @return
     */
    public boolean isNumeric(String str) {
        Pattern pattern = Pattern.compile("[0-9]*");
        Matcher isNum = pattern.matcher(str);
        if (!isNum.matches()) {
            return false;
        }
        return true;
    }

    /**
     * 测试
     *
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {

//		String action = "http://sap.com/xi/WebService/soap1.1";
//		String methodName = "ZMM_RP002";
//		String wsdlLocation = "http://61.133.215.5:50000/XISOAPAdapter/MessageServlet?senderParty=&senderService=BS_LIMS_PRD&receiverParty=&receiverService=&interface=SI_ZMM_RP002&interfaceNamespace=http://www.kocel.com/lims/zmm_rp002";
//		Map<String, String> params = new ConcurrentHashMap<String, String>();
//		WebServiceClient client = new WebServiceClient(wsdlLocation, action, methodName, params);
//		List<Part> partList = client.getPartData();
//		System.out.println(partList.size());


    }
}