package com.rootcloud.common.enums;

import com.rootcloud.common.ResourceUtil;
import lombok.Getter;

/**
 * <p>
 *     设备打印状态
 * </p>
 * @author HeJia
 * @since 2022/5/12
 */
public enum DeviceOnlineStatus implements BaseEnums<Integer, String> {
    /**
     * 离线，待机，打印中
     */
    OFFLINE(1,"离线", ""),
    ONLINE(2,"待机", "__online__.connected"),
    READY(3,"待机", "SB_ZT_Ready"),
    RUNNING(4,"打印中", "SB_ZT_Running");


    private final Integer code;
    private final String desc;

    @Getter
    private final String propertyId;

    DeviceOnlineStatus(Integer code, String desc, String propertyId) {
        this.code = code;
        this.desc = desc;
        this.propertyId = propertyId;
    }

    @Override
    public Integer getCode() {
        return code;
    }

    @Override
    public String getDesc() {
        switch (code){
            case 1:
                return ResourceUtil.getI18nMessage("60018");
            case 2:
                return ResourceUtil.getI18nMessage("60019");
            case 3:
                return ResourceUtil.getI18nMessage("60019");
            case 4:
                return ResourceUtil.getI18nMessage("60020");
            default:
                return "";
        }
    }

    public static DeviceOnlineStatus getByPropertyId(String propertyId) {
        for (DeviceOnlineStatus instance : values()) {
            if (instance.propertyId.equals(propertyId)) {
                return instance;
            }
        }
        return null;
    }

    public static DeviceOnlineStatus getByCode(Integer code) {
        for (DeviceOnlineStatus instance : values()) {
            if (instance.code.equals(code)) {
                return instance;
            }
        }
        return null;
    }
}
