package com.rootcloud.common.enums;


/**
 * @Author lujin
 * @Description 社区枚举
 * @Date on 2019/1/17.
 */
public interface ServicePlanEnums {

    /**
     * 计划周期【1：日2：周3：月4：年】
     */
    enum PlanCycle implements BaseEnums<Integer, String> {

        DAY(1, "日"),
        WEEK(2, "周"),
        MONTH(3, "月"),
        YEAR(4, "年");

        private Integer code;
        private String desc;

        PlanCycle(Integer code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public Integer getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }


//    计划类型 1维修、2保养、3交机、4技改
enum PlanType implements BaseEnums<Integer, String> {

    REPAIR(1, "维修"),
    MAINTAIN(2, "保养"),
    HANDOVER(3, "交机"),
    TECHNICAL_TRANSFORMATION(4, "技改");

    private Integer code;
    private String desc;

    PlanType(Integer code, String desc) {
        this.code = code;
        this.desc = desc;
    }

    @Override
    public Integer getCode() {
        return this.code;
    }

    @Override
    public String getDesc() {
        return this.desc;
    }
}

    //计划状态【0：待启动1：进行中2：已完成3：已暂停】
    enum PlanStatus implements BaseEnums<Integer, String> {

        TO_BE_STARTED(0, "待启动"),
        IN_PROGRESS(1, "进行中"),
        FINISHED(2, "已完成"),
        PAUSED(3, "已暂停");

        private Integer code;
        private String desc;

        PlanStatus(Integer code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public Integer getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }

//        是否超时【0：未超时 1：已超时】

    enum IsTimeout implements BaseEnums<Integer, String> {

        NO_TIMEOUT(0, "未超时"),
        TIMED_OUT(1, "已超时");

        private Integer code;
        private String desc;

        IsTimeout(Integer code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public Integer getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }

    //        提醒方式【0：邮件1：短信 2：站内信】
    enum RemindWay implements BaseEnums<Integer, String> {

        EMAIL(0, "邮件"),
        MSG(1, "短信"),
        STATION_MSG(2, "站内信");

        private Integer code;
        private String desc;

        RemindWay(Integer code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public Integer getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }

    /**
     *  触发条件 1.系统时间 2.工况，3.报警
     */
    enum TriggerType implements BaseEnums<String, String> {

        YSY_TIME("1", "系统时间"),
        WORK_CONDITION("2", "工况"),
        ALARM("3", "报警");

        private final String code;
        private final String desc;

    TriggerType(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public String getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }

    /**
     *  触发条件
     */
    enum Condition implements BaseEnums<String, String> {

        EQ("1", "等于"),
        GT("2", "大于"),
        GEQ("3", "大于等于"),
        LT("4", "小于"),
        LEQ("5", "小于等于");
        private final String code;
        private final String desc;

        Condition(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public String getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }


    enum CheckType implements BaseEnums<String, String>{

        CHECK_FALSE("0", "否"),
        CHECK_TURE("1", "是");

        private String code;
        private String desc;

        CheckType(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public String getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }

    enum DetailStatus implements BaseEnums<Integer, String> {

        DEFAULT(0, "默认的"),
        DETAIL(1, "需要处理的");

        private final Integer code;
        private final String desc;

        DetailStatus(Integer code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public Integer getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }


    enum PrintStatus implements BaseEnums<String, String>{

        PRINTING("1", "打印中"),
        STANDBY("2", "待机"),
        OFFLINE("3", "离线");

        private String code;
        private String desc;

        PrintStatus(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public String getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }


    enum MessageType implements BaseEnums<String, String>{

        FAULT_ALARM("1", "故障报警"),
        PARAMETER_ALARM("2", "参数报警"),
        MAINTENANCE_REMINDERS("3", "保养提醒"),
        DEVICE_STATUS_REMINDER("4", "设备状态提醒"),
        SPARE_PARTS_LIFE_WARNING_REMINDER("5", "备件寿命预警提醒"),
        CONTRACT_SETTLEMENT("6", "合同及结算消息通知"),
        REPAIR("001", "维修"),
        MAINTENANCE("002", "保养"),
        HANDOVER("003", "交机"),
        TRANSFORMATION("004", "技改"),
        INSPECTION("005", "巡检");

        private String code;
        private String desc;

        MessageType(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public String getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }


    enum MessageCodeType implements BaseEnums<String, String>{

        ORDER_MESSAGE("1", "订单消息通知"),
        MAINTENANCE_MESSAGE("2", "保养计划通知"),
        CONDITION_ALARM("3", "工况报警通知"),
        CONTRACTS_SETTLEMENTS("4", "合同以及结算消息通知"),
        SPARE_PARTS_LIFE("5", "备件寿命预警消息通知"),
        REPAIR_ORDER_MESSAGE("6", "维修单消息通知");

        private String code;
        private String desc;

        MessageCodeType(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public String getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }



    enum MaintenanceType implements BaseEnums<String, String>{
        STATUS_NO_LAUNCH("1", "未发起状态灰色"),
        STATUS_LAUNCH("2", "已发起状态蓝色"),
        STATUS_FINISH("3", "正常完成绿色"),
        STATUS_EXTENDED_NO_FINISH("4", "延期未完成-浅绿色"),
        STATUS_EXTENDED_FINISH("5", "拖期完成橙色"),
        STATUS_NO_FINISH("6", "未完成红色");

        private String code;
        private String desc;

        MaintenanceType(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @Override
        public String getCode() {
            return this.code;
        }

        @Override
        public String getDesc() {
            return this.desc;
        }
    }


}
