package com.rootcloud.common.intercept;

import com.alibaba.druid.sql.SQLUtils;
import com.alibaba.druid.sql.ast.SQLStatement;
import com.alibaba.druid.sql.dialect.mysql.visitor.MySqlSchemaStatVisitor;
import com.alibaba.druid.util.JdbcConstants;
import com.baomidou.mybatisplus.core.toolkit.PluginUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.handlers.AbstractSqlParserHandler;
import com.rootcloud.common.constant.TemplateConstant;
import com.rootcloud.config.auth.UserContext;
import jline.internal.Log;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.JSQLParserException;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.parser.CCJSqlParserManager;
import net.sf.jsqlparser.parser.CCJSqlParserUtil;
import net.sf.jsqlparser.schema.Column;
import net.sf.jsqlparser.statement.select.*;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import javax.sql.DataSource;
import java.io.StringReader;
import java.sql.Connection;
import java.util.*;

@Slf4j
@AllArgsConstructor
@Intercepts({@Signature(type = StatementHandler.class, method = "prepare", args = {Connection.class, Integer.class})})
@Component
public class DataScopeInterceptor  extends AbstractSqlParserHandler implements Interceptor {


    private DataSource dataSource;

    @Autowired
    private RedisTemplate redisTemplate;

    private static final List NO_STATEMENT_SQL = new ArrayList();
    static {
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.orderStatisticByDay");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.orderStatisticByDayMonth");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.ConfigDao.listTable_zhid");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.ConfigDao.listTable_syzhid");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.JeCoreEnduserDao.getTenantInfoNOSQL");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.gkjk.NxJeDeviceDao.devicePage");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.checkPersonStatistic");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxTodoLibraryMapper.getPage");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.ServicePlanMapper.getCustomerAndRoles");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.ServicePlanMapper.selectOutletsIdByCustomerId");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.ServicePlanMapper.selectListXMBY");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.ServicePlanMapper.selectByParams");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.ServicePlanMapper.selectSysTimeList");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.ServicePlanMapper.selectOne");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.gkjk.NxJeDeviceDao.selectOne");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.getSpartPartList");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.countSpartPartList");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxUserInfoMapper.selectBySeviceCode");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.SpareFilerMapper.getSparePartByAzwh");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.SpareFilerMapper.getxtmkBypjbm2");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.SpareFilerMapper.getSparePartBySbbmAndPjbm");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.SpareFilerMapper.getSparePartByAzwh");
        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.SpareFilerMapper.getSparePartBySbbm");






//        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.JeCoreEnduserDao.getUser");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.JeCoreEnduserDao.getSiteByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.JeCoreEnduserDao.getCustomsByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.JeCoreEnduserDao.getCustomsBySiteAreaName");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.JeCoreEnduserDao.selectList");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxJeDeviceDao.selectByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxJeDeviceDao.countByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.ServicePlanMapper.selectByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.ServicePlanMapper.countByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.selectByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.countByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.getSpartPartList");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.countSpartPartList");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.getNxgxJcdaFwryByUserId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.getCoreEnduserByUsrId");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.getfwwd");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.mapper.NxOrderMapper.getKhBykhdzs");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.ds.selectBykhmc");
//        NO_STATEMENT_SQL.add("com.rootcloud.domain.dao.ds.selectByUserId");
    }
    //不拦截sql


    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        StatementHandler statementHandler = PluginUtils.realTarget(invocation.getTarget());
        MetaObject metaObject = SystemMetaObject.forObject(statementHandler);
        this.sqlParser(metaObject);
        // 先判断是不是SELECT操作 不是直接过滤
        MappedStatement mappedStatement = (MappedStatement) metaObject.getValue("delegate.mappedStatement");
        if (!SqlCommandType.SELECT.equals(mappedStatement.getSqlCommandType())) {
            return invocation.proceed();
        }

        BoundSql boundSql = (BoundSql) metaObject.getValue("delegate.boundSql");
        String sql = boundSql.getSql();
       // Log.info("原sql:" + sql);
        if(NO_STATEMENT_SQL.contains(mappedStatement.getId())){   //不拦截sql
            return invocation.proceed();
        }

        //判断是否在表名列表
        String zhid = UserContext.getUserTenantId();  //需要进行租户ID过滤
        //zhid = "aaa";  //TODO
        if(StringUtils.isBlank(zhid)){
            return invocation.proceed();
        }
       // Log.info("需要进行租户ID过滤"+zhid);
        CCJSqlParserManager parserManager = new CCJSqlParserManager();
        Select select = (Select) parserManager.parse(new StringReader(sql));
        PlainSelect plainSelect = (PlainSelect) select.getSelectBody();
        addWhere(plainSelect,null, zhid);

        if (plainSelect.getJoins() != null) {    // 是否存关联连接, 主表已经加了过滤条件了,除非特殊情况才进行处理.
            List<Join> joins = plainSelect.getJoins();
            for (Join join : joins){
                addWhere(plainSelect,join,zhid);
            }
        }

        //获取select列项,判断是否有子查询
        if (plainSelect.getSelectItems() != null) {
            List<SelectItem> subSelectItem = plainSelect.getSelectItems();
            for (SelectItem subSelect :
                    subSelectItem) {
                if (subSelect instanceof SelectExpressionItem) {
                    SelectExpressionItem selectExpressionItem = (SelectExpressionItem) subSelect;
                    if (!(selectExpressionItem.getExpression() instanceof Column)) {
                        //如果子列，是一个子查询
                        Expression expression_col = selectExpressionItem.getExpression();
                        if (expression_col instanceof SubSelect) {
                            SubSelect subselect = (SubSelect) expression_col;
                            PlainSelect subPlainSelect  =  (PlainSelect)subselect.getSelectBody();
                            addWhere(subPlainSelect,null,zhid);
                        }
                    }
                }
            }
        }
        //Log.info("改造sql:" + select.toString());
        metaObject.setValue("delegate.boundSql.sql", select.toString());

        return invocation.proceed();
    }



    /**
     * 增加 where条件
     * @param plainSelect
     */
    private void addWhere(PlainSelect plainSelect,Join join,String zhid) throws JSQLParserException {

        String tableName = "";
        String alias = "";
        if (join == null) {
            tableName = plainSelect.getFromItem().toString().toLowerCase();
            alias = plainSelect.getFromItem().getAlias()!=null?plainSelect.getFromItem().getAlias().getName():"";
        } else {
            tableName = join.getRightItem().toString().toLowerCase();
            alias = join.getRightItem().getAlias()!=null?join.getRightItem().getAlias().getName():"";
        }

        if (tableName.indexOf(" ") != -1) {
            tableName = tableName.substring(0, tableName.indexOf(" "));
        }
        // 判断表明是否存在ZHID或者SY_ZHID字段   从redis获取,如果为null,直接return
        Object obj = redisTemplate.opsForValue().get(TemplateConstant.NXGX_ZH_SPLIT_TABLES + tableName);
       // obj = "aa";//TODO
        //System.out.println(tableName + "表获取resids换缓存:" + obj);
        if (obj != null && StringUtils.isNotBlank(obj.toString())) {
            StringBuffer whereSql = new StringBuffer();
            if (StringUtils.isNotBlank(alias)) {
                //存在别名
                whereSql.append(alias + "." + obj.toString() + " = '" + zhid + "'");
            } else {
                whereSql.append(obj.toString() + " = '" + zhid + "'");    //需要进行租户ID过滤
            }

            Expression where = plainSelect.getWhere();
            if (where == null) {
                if (whereSql.length() > 0) {
                    Expression expression = CCJSqlParserUtil
                            .parseCondExpression(whereSql.toString());
                    Expression whereExpression = (Expression) expression;
                    plainSelect.setWhere(whereExpression);
                }
            } else {
                if (whereSql.length() > 0) {
                    //where条件之前存在，需要重新进行拼接
                    whereSql.append(" and " + where.toString());
                } else {
                    //新增片段不存在，使用之前的sql
                    whereSql.append(where.toString());
                }
                Expression expression = CCJSqlParserUtil
                        .parseCondExpression(whereSql.toString());
                plainSelect.setWhere(expression);
            }
        }
    }



    /**
     * 获取sql中所有的表名
     * @param boundSql
     * @return
     */
    private List<String> getTableName(BoundSql boundSql) {
        List<SQLStatement> stmtList = SQLUtils.parseStatements(boundSql.getSql(), JdbcConstants.MYSQL);

        List<String> tableNameList = new ArrayList<>();
        for (SQLStatement stmt : stmtList) {
            MySqlSchemaStatVisitor visitor = new MySqlSchemaStatVisitor();
            stmt.accept(visitor);
            visitor.getTables().keySet().forEach(s -> tableNameList.add(s.getName()));
        }

        return tableNameList;
    }

    /**
     * 生成拦截对象的代理
     *
     * @param target 目标对象
     * @return 代理对象
     */
    @Override
    public Object plugin(Object target) {
        if (target instanceof StatementHandler) {
            return Plugin.wrap(target, this);
        }
        return target;
    }

    /**
     * mybatis配置的属性
     *
     * @param properties mybatis配置的属性
     */
    @Override
    public void setProperties(Properties properties) {

    }
}