package com.rootcloud.common.live;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 使用ConcurrentHashMap做一个简易的缓存，目前只是添加，如需其他的方法可自行添加
 * @author fandi
 *
 */

public class CacheMap {

    // 设置过期时间为5天
    public static int EXPIRE_DAY = 5;

    /**
     * 数据缓存map
     */
    public static Map<String, String> dataMap = new ConcurrentHashMap<String, String>();

    /**
     * 数据缓存过期map
     */
    public static Map<String, Date> dataExpireMap = new ConcurrentHashMap<String, Date>();

    /**
     * 将一个key、value值放入内存缓存,并设置过期天数
     *
     * @param key
     * @param val
     * @param EXPIRE_DAY
     */
    public static void put(String key, String val) {
        dataMap.put(key, val);
        dataExpireMap.put(key, DateUtil.getAfterDate(new Date(), EXPIRE_DAY));
    }

    /**
     * 从缓存中获取一个key的数据(若过期返回null)
     *
     * @param key
     * @return
     */
    public static String get(String cacheKey) {
    	String obj = "";
        Date expireDate = CacheMap.dataExpireMap.get(cacheKey);

        //如果过期时间不为空，或者当前时间与过期时间的差小于0，则说明当前的缓存有效。
        if (expireDate != null && DateUtil.getDateSpace(expireDate, new Date()) < 0) {
            obj = CacheMap.dataMap.get(cacheKey);
        }
        return obj;
    }
    
	/**
	 * 判断是否存在一个key
	 * @param key
	 * @return
	 */
	public static boolean hasCache(String key) { 
	      return CacheMap.dataMap.containsKey(key); 
	   } 
}
