package com.rootcloud.common.live;

import com.rootcloud.common.constant.SyncSimpleDateFormat;
import com.rootcloud.common.constant.WeekTypeEnum;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 日期组件
 * @author Adams.huang
 *
 */
public class DateUtil {

	public static final String YYYY_MM_DD = "yyyy-MM-dd";
	public static final String HH_MM_SS = "HH:mm:ss";
	public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
	public static final String YYYY_MM_DD_HH_MM = "yyyy-MM-dd HH:mm";
	public static final String YYYY_MMDD = "yyyyMMdd";
	public static final String yyyy_MM = "yyyy/MM";
	public static final String yyyy_MM_dd = "yyyy/MM/dd";
	public static final String yyyyMM = "yyyyMM";
	public static final String MM_DD_HH_SS_TEMPLATE = "{0}-{1}({2}){3}:{4}";
	public static final String HH_SS_TEMPLATE = "{0}:{1}";
	public static final String yyyyMMddhhmmss = "yyyyMMddhhmmss";
	private static final Calendar calendar = Calendar.getInstance();
	private final static SimpleDateFormat dateformater1 = new SimpleDateFormat("yyyy-MM-dd");
	public static boolean defaultTimeFlag = true; // 查询条件默认时间显示标志

	/* 开始时间点 */
	public final static String startTimePoint = " 00:00:00";
	/* 结束时间点 */
	public final static String endTimePoint = " 23:59:59";

	/**
	 * Number of milliseconds in a standard second.
	 * 
	 * @since 2.1
	 */
	public static final long MILLISECOND_PER_SECOND = 1000;
	/**
	 * Number of milliseconds in a standard minute.
	 * 
	 * @since 2.1
	 */
	public static final long MILLISECOND_PER_MINUTE = 60 * MILLISECOND_PER_SECOND;
	/**
	 * Number of milliseconds in a standard hour.
	 * 
	 * @since 2.1
	 */
	public static final long MILLISECOND_PER_HOUR = 60 * MILLISECOND_PER_MINUTE;
	/**
	 * Number of milliseconds in a standard day.
	 * 
	 * @since 2.1
	 */
	public static final long MILLISECOND_PER_DAY = 24 * MILLISECOND_PER_HOUR;

	private static SimpleDateFormat dateformater, timeformater, dateTimeformater, dateIDformater;

	private DateUtil() {
	}

	/**
	 * 得到本月的第一天
	 * 
	 * @return
	 */
	public static String getMonthFirstDay() {
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));

		return dateformater1.format(calendar.getTime());
	}

	public static String getDisplayTime(String time) {
		Date dateTime = parseDateTime(time);
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(dateTime);
		//判断是否是今天
		Calendar todayCalendar = Calendar.getInstance();
		Date now = new Date();
		todayCalendar.setTime(now);
		todayCalendar.set(Calendar.HOUR_OF_DAY, 0);
		todayCalendar.set(Calendar.MINUTE, 0);
		int todayYear = todayCalendar.get(Calendar.YEAR);
		
		int month = calendar.get(Calendar.MONTH) + 1;
		int day = calendar.get(Calendar.DAY_OF_MONTH);
		int hour = calendar.get(Calendar.HOUR_OF_DAY);
		int minute = calendar.get(Calendar.MINUTE);
		int week = calendar.get(Calendar.DAY_OF_WEEK);
		int year = calendar.get(Calendar.YEAR);
		String weekStr = WeekTypeEnum.getDisplay(week);
		String minuteStr = minute > 9 ? Integer.toString(minute) : "0" + Integer.toString(minute);
		if (calendar.after(todayCalendar)){
			return MessageFormat.format(HH_SS_TEMPLATE,hour,minuteStr);
		}
		if(todayYear > year){//显示年
			return formatDate(dateTime);
		}
		return MessageFormat.format(MM_DD_HH_SS_TEMPLATE, month, day, weekStr,
				hour, minuteStr);
	}

	/**
	 * 得到本月的最后一天
	 * 
	 * @return
	 */
	public static String getMonthLastDay() {
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
		return dateformater1.format(calendar.getTime());
	}

	/**
	 * 获取日期格式的DateFormater
	 * 
	 * @return DateFormat
	 */
	private static DateFormat getDateFormater() {
		if (dateformater == null)
			dateformater = new SyncSimpleDateFormat(YYYY_MM_DD);
		return dateformater;
	}

	/**
	 * 获取日期格式的DateFormater  2017/12/12
	 * 
	 * @return DateFormat
	 */
	private static DateFormat getDateEnFormater() {
		if (dateformater == null)
			dateformater = new SyncSimpleDateFormat(yyyy_MM_dd);
		return dateformater;
	}
	
	/**
	 * 获取时间格式的DateFormater
	 * 
	 * @return DateFormat
	 */
	private static DateFormat getTimeFormater() {
		if (timeformater == null)
			timeformater = new SyncSimpleDateFormat(HH_MM_SS);
		return timeformater;
	}

	/**
	 * 获取日期时间格式的DateFormater
	 * 
	 * @return DateFormat
	 */
	private static DateFormat getDateTimeFormater() {
		if (dateTimeformater == null)
			dateTimeformater = new SyncSimpleDateFormat(YYYY_MM_DD_HH_MM_SS);
		return dateTimeformater;
	}

	/**
	 * 获得没有秒的时间字符串
	 * 
	 * @return yyyy-MM-dd HH:mm
	 */
	public static String formatDateTimeFormaterExceptSeconds(Date date) {
		dateTimeformater = new SyncSimpleDateFormat(YYYY_MM_DD_HH_MM);
		return dateTimeformater.format(date);
	}

	/**
	 * 获取日期时间格式的DateFormater
	 *
	 * @return DateFormat
	 */
	private static DateFormat getDateIDFormater() {
		if (dateIDformater == null)
			dateIDformater = new SyncSimpleDateFormat(YYYY_MMDD);
		return dateIDformater;
	}

	/**
	 * 获取现在的时间(yyyy-MM-dd HH:mm:ss)
	 *
	 * @return String
	 */
	public static String getNow() {
		return getDateTimeFormater().format(new Date());
	}

	/**
	 * 获取现在的日期(yyyy-MM-dd)
	 *
	 * @return String
	 */
	public static String getDate() {
		return getDateFormater().format(new Date());
	}

	/**
	 * 获取现在的时间(HH:mm:ss)
	 *
	 * @return String
	 */
	public static String getTime() {
		return getTimeFormater().format(new Date());
	}

	/**
	 * 获取日期(yyyy-MM-dd)
	 *
	 * @param checkTime
	 *            Timestamp
	 *
	 * @return String
	 */
	public static Date getDate(Date checkTime) {
		return parseDate(getDate());
	}

	/**
	 * 获取年
	 *
	 * @param checkTime
	 * @return
	 * @author Adams Fum
	 * @date 2015年8月7日 上午11:12:19
	 */
	public static int getYear(Date checkTime) {
		GregorianCalendar ca = new GregorianCalendar();
		ca.setTime(checkTime);

		return ca.get(GregorianCalendar.YEAR);
	}

	/**
	 * 获取月
	 *
	 * @param checkTime
	 * @return
	 * @author Adams Fum
	 * @date 2015年8月7日 上午11:12:23
	 */
	public static int getMonth(Date checkTime) {
		GregorianCalendar ca = new GregorianCalendar();
		ca.setTime(checkTime);

		return ca.get(GregorianCalendar.MONTH) + 1;
	}

	/**
	 * 获取天
	 *
	 * @param checkTime
	 * @return
	 * @author Adams Fum
	 * @date 2015年8月7日 上午11:12:26
	 */
	public static int getDay(Date checkTime) {
		GregorianCalendar ca = new GregorianCalendar();
		ca.setTime(checkTime);

		return ca.get(GregorianCalendar.DAY_OF_MONTH);
	}

	/**
	 * 获取小时
	 *
	 * @param checkTime
	 * @return
	 * @author ShiChunyun
	 * @date 2015年11月10日 下午8:23:27
	 */
	public static int getHour(Date checkTime) {
		GregorianCalendar ca = new GregorianCalendar();
		ca.setTime(checkTime);

		return ca.get(GregorianCalendar.HOUR_OF_DAY);
	}

	/**
	 * 获取日期(yyyy-MM-dd)
	 *
	 * @param year
	 * @param month
	 * @param day
	 * @return
	 * @author Adams Fum
	 * @date 2015年8月7日 上午10:57:51
	 */
	public static Date getDate(int year, int month, int day) {
		if (year == 0)
			year = 1900;
		if (month == 0)
			month = 1;
		if (day == 0)
			day = 1;
		GregorianCalendar gCal = new GregorianCalendar(year, month - 1, day);
		return gCal.getTime();
	}

	/**
	 * 获取日期(yyyy-MM-dd)
	 *
	 * @param checkTime
	 *            Timestamp
	 *
	 * @return String
	 */
	public static String getDate(Timestamp checkTime) {
		return formatDate(checkTime);
	}

	/**
	 * 获取时间(HH:mm:ss)
	 *
	 * @param checkTime
	 *            Timestamp
	 *
	 * @return String
	 */
	public static String getTime(Timestamp checkTime) {
		return getTimeFormater().format(checkTime);
	}

	/**
	 * 获取多少天之后的日期
	 *
	 * @param startDate
	 *            Date : 起始时间
	 * @param days
	 *            int : 指定天数
	 *
	 * @return Date java.util.Date
	 */
	public static Date getDate(Date startDate, int days) {
		Date returnDay = dateDayAdd(startDate, days);
		return returnDay;
	}

	/**
	 * 获取多少天之前的日期
	 *
	 * @param startDate
	 * @param days
	 * @return
	 * @author WuXiaoJun
	 * @date 2014年12月29日
	 */

	public static Date getBeforeDate(Date startDate, int days) {
		Date returnDay = dateDayDel(startDate, days);
		return returnDay;
	}

	/**
	 * 获取多少天之后的日期
	 *
	 * @param getAfterDate
	 *            String : 起始日期
	 * @param days
	 *            int : 指定天数
	 *
	 * @return Date java.util.Date
	 */
	public static Date getAfterDate(Date startDate, int days) {
		Date date = dateDayAdd(startDate, days);
		return date;
	}

	/**
	 * 获取多少月之后的日期
	 *
	 * @param startDate
	 *            Date : 起始日期
	 * @param months
	 *            int : 指定月数
	 *
	 * @return Date java.util.Date
	 */
	public static Date getMonthDate(Date startDate, int months) {
		Date returnDay = dateMonthAdd(startDate, months);
		return returnDay;
	}

	/**
	 * 获取多少月之后的日期
	 *
	 * @param startDate
	 *            String : 起始日期
	 * @param months
	 *            int : 指定月数
	 *
	 * @return Date java.util.Date
	 */
	public static Date getMonthDate(String startDate, int months) {
		Date start = parseDate(startDate);
		Date date = dateMonthAdd(start, months);
		return date;
	}

	/**
	 * 获取多少分钟之后的日期时间
	 *
	 * @param strDateTime
	 *            : 起始日期时间
	 * @param minutes
	 *            : 指定分钟数
	 */
	public static Date getMinuteTime(String strDateTime, int minutes) {
		Date start = parseDateTime(strDateTime);
		return dateDateTimeAdd(start, minutes);
	}

	/**
	 * 获取多少分钟之后的日期时间
	 *
	 * @param startTimestamp
	 *            : 起始日期时间
	 * @param minutes
	 *            : 指定分钟数
	 */
	public static Timestamp getMinuteTimestamp(String startTimestamp, int minutes) {
		Timestamp start = parseTimestamp(startTimestamp);
		return dateDateTimeAdd(start, minutes);
	}

	/**
	 *
	 * 获取当前月份 字符串
	 *
	 * @return yyyy-mm
	 * @author Adams Fum
	 * @date 2014年9月21日
	 */
	public static String getCurMonth() {
		return new SimpleDateFormat("yyyy-MM").format(new Date());
	}

	/**
	 *
	 * 获取当前月份 字符串
	 *
	 * @return yyyy/MM
	 * @author Adams Fum
	 * @date 2014年9月21日
	 */
	public static String getCurMonthPath() {
		SimpleDateFormat format = new SyncSimpleDateFormat(yyyy_MM);
		Date date = new Date();
		return format.format(date);
	}

	/**
	 *
	 * 获取当前月份 字符串
	 *
	 * @return yyyyMM
	 * @author Adams Fum
	 * @date 2014年9月21日
	 */
	public static String getCurMonthStr() {
		SimpleDateFormat format = new SyncSimpleDateFormat(yyyyMM);
		Date date = new Date();
		return format.format(date);
	}

	/**
	 * 根据日期获得当前月的开始日期
	 *
	 * @param month
	 *            日期
	 *
	 * @return 所属月份的开始日期(java.util.Date)
	 */
	public static Date getMonthBegin(Date month) {
		GregorianCalendar ca = new GregorianCalendar();
		ca.setTime(month);
		int year = ca.get(GregorianCalendar.YEAR);
		int mon = ca.get(GregorianCalendar.MONTH);
		GregorianCalendar gCal = new GregorianCalendar(year, mon, 1);
		return gCal.getTime();
	}

	/**
	 * 根据年份和月份获得当前月的开始日期
	 *
	 * @param year
	 *            年份
	 * @param month
	 *            月份
	 *
	 * @return 所属月份的开始日期(java.util.Date)
	 */
	public static Date getMonthBegin(int year, int month) {
		if (month <= 0 || month > 12)
			throw new IllegalArgumentException("month must from 1 to 12");
		GregorianCalendar gCal = new GregorianCalendar(year, month - 1, 1);
		return gCal.getTime();
	}

	/**
	 * 根据日期获得当前月的结束日期
	 *
	 * @param month
	 *            日期
	 *
	 * @return 所属月份的结束日期(java.util.Date)
	 */
	public static Date getMonthEnd(Date month) {
		GregorianCalendar ca = new GregorianCalendar();
		ca.setTime(month);
		int year = ca.get(GregorianCalendar.YEAR);
		int mon = ca.get(GregorianCalendar.MONTH);
		int lastDay = ca.getActualMaximum(GregorianCalendar.DAY_OF_MONTH);
		GregorianCalendar gCal = new GregorianCalendar(year, mon, lastDay);
		return gCal.getTime();
	}

	/**
	 * 根据年份和月份获得当前月的结束日期
	 *
	 * @param year
	 *            年份
	 * @param month
	 *            月份
	 *
	 * @return 所属月份的结束日期(java.util.Date)
	 */
	public static Date getMonthEnd(int year, int month) {
		Date start = getMonthBegin(year, month);
		return getMonthEnd(start);
	}

	/**
	 * 根据日期参数获取当前周的起始日期(周日)
	 *
	 * @param date
	 *            日期
	 *
	 * @return 所属星期的起始日期(java.util.Date)
	 */
	public static Date getWeekBegin(Date date) {
		GregorianCalendar gCal = new GregorianCalendar();
		gCal.setTime(date);
		int startDay = gCal.getActualMinimum(GregorianCalendar.DAY_OF_WEEK);
		gCal.set(GregorianCalendar.DAY_OF_WEEK, startDay);
		return gCal.getTime();
	}

	/**
	 * 根据日期参数获取当前周的结束日期(周六)
	 *
	 * @param date
	 *            日期
	 *
	 * @return 所属星期的结束日期(java.util.Date)
	 */
	public static Date getWeekEnd(Date date) {
		GregorianCalendar gCal = new GregorianCalendar();
		gCal.setTime(date);
		int lastDay = gCal.getActualMaximum(GregorianCalendar.DAY_OF_WEEK);
		gCal.set(GregorianCalendar.DAY_OF_WEEK, lastDay);
		return gCal.getTime();
	}

	/**
	 * 获取指定时间段内的天数
	 *
	 * @param beginTime
	 *            Timestamp
	 * @param endTime
	 *            Timestamp
	 *
	 * @return int
	 */
	public static int getDays(Timestamp beginTime, Timestamp endTime) {
		int days = (int) ((endTime.getTime() - beginTime.getTime()) / MILLISECOND_PER_DAY);
		return days;
	}

	/**
	 * 获取指定时间段内的天数
	 *
	 * @param beginTime
	 *            Date
	 * @param endTime
	 *            Date
	 *
	 * @return int
	 */
	public static int getDays(Date beginTime, Date endTime) {
		int days = (int) ((endTime.getTime() - beginTime.getTime()) / MILLISECOND_PER_DAY);
		return days;
	}

	/**
	 * 生成日期ID(yyyyMMdd) ex: 传入参数:2004-5-1 返回:20040501
	 *
	 * @param date
	 *            Date
	 *
	 * @return String
	 */
	public static String getDateID(Date date) {
		return getDateIDFormater().format(date);
	}

	/**
	 * 生成日期ID(yyyyMMdd) ex: 传入参数:2004-5-1 12:32:33 返回:20040501
	 *
	 * @param timestamp
	 *            Timestamp
	 *
	 * @return String
	 */
	public static String getDateID(Timestamp timestamp) {
		return getDateIDFormater().format(timestamp);
	}

	/**
	 * 根据传入的日期以及年数计算并返回N年后的日期
	 *
	 * @param date
	 *            Date
	 * @param years
	 *            int
	 * @return Date
	 */
	public static Date getAfterYear(Date date, int years) {

		calendar.setTime(date);
		calendar.add(Calendar.YEAR, years);

		return calendar.getTime();
	}

	/**
	 * 将java.util.Date格式转换成日期字符串(yyyy-MM-dd)
	 *
	 * @return String
	 */
	public static String formatDate(Date date) {
		return getDateFormater().format(date);
	}

	/**
	 * 将java.util.Date格式转换成时间字符串(HH:mm:ss)
	 *
	 * @return String
	 */
	public static String formatTime(Date date) {
		return getTimeFormater().format(date);
	}

	/**
	 * 将java.util.Date格式转换成日期时间字符串(yyyy-MM-dd HH:mm:ss)
	 *
	 * @return String
	 */
	public static String formatDateTime(Date date) {
		return getDateTimeFormater().format(date);
	}

	/**
	 * 将Timestamp格式转换成日期时间字符串(yyyy-MM-dd HH:mm:ss)
	 *
	 * @return String
	 */
	public static String formatDateTime(Timestamp timestamp) {
		return getDateTimeFormater().format(timestamp);
	}

	/**
	 * 根据字符串转换成日期格式()
	 *
	 * @param strDate
	 *            日期格式字符串(yyyy-MM-dd)
	 *
	 * @return java.util.Date
	 */
	public static Date parseDate(String strDate) {
		Date date = null;
		try {
			date = getDateFormater().parse(strDate);
		} catch (Exception ex) {
			// System.err.println(ex.getMessage());
		}
		return date;
	}

	/**
	 * 根据字符串转换成日期格式()
	 *
	 * @param strDate
	 *            日期格式字符串(yyyy/MM/dd)
	 *
	 * @return java.util.Date
	 */
	public static Date parseDateEn(String strDate) {
		Date date = null;
		try {
			date = getDateEnFormater().parse(strDate);
		} catch (Exception ex) {
			// System.err.println(ex.getMessage());
		}
		return date;
	}

	/**
	 * 根据字符串转换成日期格式
	 *
	 * @param strDate
	 *            日期格式字符串
	 *
	 * @return java.sql.Date
	 */
	public static java.sql.Date parseSqlDate(String strDate) {
		return java.sql.Date.valueOf(strDate);
	}

	/**
	 * 将Timestamp转换成java.sql.Date格式
	 *
	 * @param checkTime
	 *            Timestamp
	 *
	 * @return java.sql.Date
	 */
	public static java.sql.Date parseSqlDate(Timestamp checkTime) {
		String sDate = formatDate(checkTime);
		return java.sql.Date.valueOf(sDate);
	}

	/**
	 * 根据字符串转换成日期格式
	 *
	 * @param strDateTime
	 *            日期时间格式字符串(yyyy-MM-dd HH:mm:ss)
	 *
	 * @return java.util.Date
	 */
	public static Date parseDateTime(String strDateTime) {
		Date date = null;
		if (strDateTime.length() < 19) {
			strDateTime += ":00";
		}
		try {
			date = getDateTimeFormater().parse(strDateTime);
		} catch (Exception ex) {
			System.err.println(ex.getMessage());
		}
		return date;
	}

	/**
	 * 获取当前时间的前一个月时间
	 *
	 * @param strDateTime
	 *            日期时间格式字符串
	 *
	 * @return java.util.Date
	 */
	public static Date parseLastMonthDate() {
		GregorianCalendar gCal = new GregorianCalendar();
		gCal.setTime(new Date());
		gCal.add(GregorianCalendar.MONTH, -1);
		return gCal.getTime();
	}

	/**
	 * 获取当前时间的前一个月时间
	 *
	 * @param strDateTime
	 *            日期时间格式字符串
	 *
	 * @return java.sql.Timestamp
	 */
	public static Timestamp parseLastMonthTimestamp() {
		GregorianCalendar gCal = new GregorianCalendar();
		gCal.setTime(new Date());
		gCal.add(GregorianCalendar.MONTH, -1);
		return new Timestamp(gCal.getTime().getTime());
	}

	/**
	 * 获取当前时间的后几天时间
	 *
	 * @param strDateTime
	 *            日期时间格式字符串
	 *
	 * @return java.sql.Timestamp
	 */
	public static Timestamp getNextTimestamp(Timestamp e, int i) {
		if (e == null)
			return new Timestamp(new Date().getTime());
		GregorianCalendar gCal = new GregorianCalendar();
		gCal.setTime(new Date(e.getTime() + i * 1000L * 60 * 60 * 24));
		return new Timestamp(gCal.getTime().getTime());
	}

	/**
	 * 获取当前时间的后几天时间的字符串((yyyy-MM-dd HH:mm:ss))
	 *
	 * @param e
	 * @param i
	 * @return
	 */
	public static String getNextTimeStr(Timestamp e, int i) {
		if (e == null) {
			e = new Timestamp(new Date().getTime());
		}
		GregorianCalendar gCal = new GregorianCalendar();
		gCal.setTime(new Date(e.getTime() + i * 1000L * 60 * 60 * 24));
		return formatDateTime(gCal.getTime());
	}

	/**
	 * 将字符串转换成Timestamp格式
	 *
	 * @return Timestamp
	 */
	public static Timestamp parseTimestamp(String strTimestamp) {
		if (strTimestamp == null)
			return new Timestamp(new Date().getTime());
		String datetime = getDateTimeStr(strTimestamp);
		return new Timestamp(parseDateTime(datetime).getTime());
	}

	/**
	 * 获取日期
	 *
	 * @param date
	 *            指定日期
	 * @param days
	 *            经过的天数
	 *
	 * @return Date
	 */
	private static Date dateDayAdd(Date date, int days) {
		long now = date.getTime() + days * MILLISECOND_PER_DAY;
		return (new Date(now));
	}

	/**
	 * 获得日期
	 *
	 * @param date
	 *            指定日期
	 * @param days
	 *            经过的天数
	 * @return
	 * @author WuXiaoJun
	 * @date 2014年12月29日
	 */

	private static Date dateDayDel(Date date, int days) {
		long now = date.getTime() - days * MILLISECOND_PER_DAY;
		return (new Date(now));
	}

	/**
	 * 获取时间
	 *
	 * @param timestamp
	 *            指定日期时间
	 * @param minutes
	 *            经过的分钟数
	 *
	 * @return Timestamp
	 */
	private static Date dateDateTimeAdd(Date time, int minutes) {
		long now = time.getTime() + minutes * 1000 * 60;
		return (new Date(now));
	}

	/**
	 * 获取时间
	 *
	 * @param timestamp
	 *            指定日期时间
	 * @param minutes
	 *            经过的分钟数
	 *
	 * @return Timestamp
	 */
	private static Timestamp dateDateTimeAdd(Timestamp timestamp, int minutes) {
		long now = timestamp.getTime() + minutes * 1000 * 60;
		return (new Timestamp(now));
	}

	/**
	 * 根据当前时间以及传入的时间计算相差的分钟数。
	 *
	 * @param startDate
	 *            开始时间
	 * @param endDate
	 *            结束时间
	 *
	 * @return long
	 */
	public static long getDiffMinu(Date startDate, Date endDate) {
		long startTime = startDate.getTime();
		long endTime = endDate.getTime();

		long minu = (endTime - startTime) / (1000 * 60);
		return minu;
	}

	/**
	 * 根据当前时间以及传入的时间计算相差的分钟数。
	 *
	 * @param startDate
	 *            开始时间
	 * @param endDate
	 *            结束时间
	 *
	 * @return long
	 */
	public static long getDiffMinu(Timestamp startDate, Timestamp endDate) {
		long startTime = startDate.getTime();
		long endTime = endDate.getTime();

		long minu = (endTime - startTime) / (1000 * 60);
		return minu;
	}

	/**
	 * 获取日期
	 *
	 * @param date
	 *            指定日期
	 * @param months
	 *            经过的月数
	 *
	 * @return Date
	 */
	private static Date dateMonthAdd(Date date, int months) {
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(date);
		int m = cal.get(GregorianCalendar.MONTH) + months;
		if (m < 0)
			m += -12;
		cal.roll(GregorianCalendar.YEAR, m / 12);
		cal.roll(GregorianCalendar.MONTH, months);
		return cal.getTime();
	}

	/**
	 * 根据字符串生成日期时间格式的字符串
	 * 
	 * @param str
	 *            日期型字符串,不含时间
	 * 
	 * @return String
	 */
	private static String getDateTimeStr(String str) {
		if (str.indexOf(" ") == -1)
			str += "  00:00:00";
		else if (str.lastIndexOf(":") == 13)
			str += ":00";
		return str;
	}

	/**
	 * 本月有多少天
	 * 
	 * @return 天
	 * 
	 * @author
	 */
	public static int GetDaysOfMonth(int iYear, int Month) {
		int days = 0;
		switch (Month) {
		case 1:
			days = 31;
			break;
		case 2:
			if (IsRuYear(iYear)) {
				// 闰年多 1 天 即：2 月为 29 天
				days = 29;
			} else {
				// --非闰年少1天 即：2 月为 28 天
				days = 28;
			}

			break;
		case 3:
			days = 31;
			break;
		case 4:
			days = 30;
			break;
		case 5:
			days = 31;
			break;
		case 6:
			days = 30;
			break;
		case 7:
			days = 31;
			break;
		case 8:
			days = 31;
			break;
		case 9:
			days = 30;
			break;
		case 10:
			days = 31;
			break;
		case 11:
			days = 30;
			break;
		case 12:
			days = 31;
			break;
		}

		return days;

	}

	/**
	 * 判断当前日期所属的年份是否是闰年，私有函数 是闰年：True ，不是闰年：False
	 * 
	 */
	private static boolean IsRuYear(int year) {

		if ((year % 400 == 0) || (year % 4 == 0 && year % 100 != 0)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * 得到星期
	 * 
	 */
	public static String getWeekOfDate(Date dt) {
		String[] weekDays = { "日", "一", "二", "三", "四", "五", "六" };
		Calendar cal = Calendar.getInstance();

		cal.setTime(dt);
		int w = cal.get(Calendar.DAY_OF_WEEK) - 1;
		if (w < 0)
			w = 0;
		return weekDays[w];

	}

	/**
	 * 按格式转换时间
	 * 
	 * @param format
	 *            ，常用格式："yyMMdd","yyyyMMddhhmmss", "yyyy-MM-dd HH:mm:ss",
	 *            "yyyy-MM-dd HH:mm:ss.SS"
	 * 
	 */
	public static String convertDateFormat(Date date, String format) throws Exception {
		String time = null;
		try {
			time = (new SimpleDateFormat(format)).format(date);
		} catch (RuntimeException e) {
			throw e;
		}

		return time;
	}

	/**
	 * 
	 * 根据输入的日期字符串返回相应的秒数
	 * 
	 * @param dateStr
	 *            (yyyy-MM-dd)
	 * @return
	 * @author Adams Fum
	 * @date 2014年9月21日
	 */
	public static long getSecondsOfDate(String dateStr) {
		try {
			Date date = getDateFormater().parse(dateStr);
			Calendar ca = Calendar.getInstance();
			ca.setTime(date);
			return ca.getTimeInMillis() / 1000;
		} catch (ParseException e) {
			throw new IllegalArgumentException("输入的日期字符串：'" + dateStr + "'不合法！");
		}
	}

	/**
	 * 
	 * 根据传入的日期时间字符串，返回相应的秒数
	 * 
	 * @param dateStr
	 *            (yyyy-MM-dd HH:mm:ss)
	 * @return
	 * @author Adams Fum
	 * @date 2014年9月21日
	 */
	public static long getSecondsOfDateTime(String dateStr) {
		try {
			Date date = getDateTimeFormater().parse(dateStr);
			Calendar ca = Calendar.getInstance();
			ca.setTime(date);
			return ca.getTimeInMillis() / 1000;
		} catch (ParseException e) {
			throw new IllegalArgumentException("输入的日期时间字符串：'" + dateStr + "'不合法！");
		}
	}

	/**
	 * 
	 * 根据传入的日期时，返回相应的秒数
	 * 
	 * @param dateTime
	 * @return
	 * @author Adams Fum
	 * @date 2014年9月21日
	 */
	public static long getSecondsOfDateTime(Date dateTime) {
		Calendar ca = Calendar.getInstance();
		ca.setTime(dateTime);
		return ca.getTimeInMillis() / 1000;
	}

	/**
	 * 根据传入的秒数返回日期时间
	 */
	public static Date getDateFromSeconds(long seconds) {
		Calendar ca = Calendar.getInstance();
		ca.setTimeInMillis(seconds * 1000);
		return ca.getTime();
	}

	/**
	 * 根据传入的持续时间返回持续时间秒数
	 * 
	 * @param duration
	 *            00:20:00
	 * 
	 * @return long 秒数
	 */
	public static long getSecondsFromDurTime(String duration) {
		long seconds = 0;
		String[] dur = duration.split(":");
		seconds = new Long(dur[0]).longValue() * 3600 + new Long(dur[1]).longValue() * 60 + new Long(dur[2]).longValue();
		return seconds;
	}

	/**
	 * 格式化持续时间秒数为持续时间(hh:mm:ss或xx小时xx分xx秒)
	 * 
	 * @param durLong
	 *            持续时间秒数
	 * @param formatStr
	 *            时间格式(hh:mm:ss)
	 * @return xx小时xx分xx秒
	 * @author Adams Fum
	 * @date 2014年9月21日
	 */
	public static String formatDurTime(long durLong, String formatStr) {
		String durTime = "";
		if (formatStr == null || "".equals(formatStr))
			formatStr = "hh:mm:ss";
		String[] fmtArray = formatStr.split(":");
		for (int i = 0; i < fmtArray.length; i++) {
			if (fmtArray[i].equals("hh") || fmtArray[i].endsWith("小时")) {
				long hl = durLong / 3600;
				if (hl == 0)
					continue;
				durLong = durLong % 3600;
				if (fmtArray[i].endsWith("小时"))
					durTime += hl + fmtArray[i];
				else
					durTime += hl + ":";
			} else if (fmtArray[i].equals("mm") || fmtArray[i].startsWith("分")) {
				long ml = durLong / 60;
				durLong = durLong % 60;
				if (fmtArray[i].startsWith("分"))
					durTime += ml + fmtArray[i];
				else
					durTime += ml + ":";
			} else if (fmtArray[i].equals("ss") || fmtArray[i].equals("秒")) {
				long sl = durLong;
				if (fmtArray[i].equals("秒"))
					durTime += sl + "秒";
				else
					durTime += sl;
			}
		}

		return durTime;
	}

	/**
	 * 得到指定日期的一天的的最后时刻23:59:59
	 * 
	 * @param date
	 *            Date对象
	 * @return java.util.Date 返回指定日期的最后时间 比如：2010-8-11 23:59:59
	 */
	public static Date getFinallyDate(Date date) {
		SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");

		String fm = format.format(date);
		fm += " 23:59:59";
		try {
			return DateUtils.parseDate(fm, new String[] { "yyyyMMdd HH:mm:ss" });
		} catch (ParseException e1) {
			return null;
		}
	}

	/**
	 * 得到指定日期的一天的开始时刻00:00:00
	 * 
	 * @param date
	 *            Date对象
	 * @return java.util.Date 返回指定日期的最开始时间 比如：2010-8-11 00:00:00
	 */
	public static Date getStartDate(Date date) {
		SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");

		String fm = format.format(date);
		fm += " 00:00:00";
		try {
			return DateUtils.parseDate(fm, new String[] { "yyyyMMdd HH:mm:ss" });
		} catch (ParseException e1) {
			return null;
		}
	}

	/**
	 * 将String型格式化,比如想要将2011-11-11格式化成2011年11月11日,就StringPattern("2011-11-11",
	 * "yyyy-MM-dd","yyyy年MM月dd日").
	 * 
	 * @param date
	 *            String 想要格式化的日期
	 * @param oldPattern
	 *            String 想要格式化的日期的现有格式
	 * @param newPattern
	 *            String 想要格式化成什么格式
	 * @return String
	 */
	public static String StringPattern(String date, String oldPattern, String newPattern, Locale locale) {
		if (date == null || oldPattern == null || newPattern == null) {
			return null;
		}
		SimpleDateFormat oldSdf = new SimpleDateFormat(oldPattern); // 实例化模板对象
		SimpleDateFormat newSdf = new SimpleDateFormat(newPattern, locale); // 实例化模板对象
		Date d = null;
		try {
			d = oldSdf.parse(date); // 将给定的字符串中的日期提取出来
		} catch (Exception e) { // 如果提供的字符串格式有错误，则进行异常处理
			e.printStackTrace(); // 打印异常信息
		}
		return newSdf.format(d);
	}

	/**
	 * 
	 * 计算时间段
	 * 
	 * @param start
	 * @return
	 * @author Adams Fum
	 * @date 2015年1月28日
	 */
	public static Long getStartTimeForSearch(int start) {
		if (start == -1) {
			return null;
		}
		long startTime = 0L;
		switch (start) {
		case 1:
			Calendar todayCalendar = Calendar.getInstance();
			Date now = new Date();
			todayCalendar.setTime(now);
			todayCalendar.set(Calendar.HOUR_OF_DAY, 0);
			todayCalendar.set(Calendar.MINUTE, 0);
			startTime = todayCalendar.getTime().getTime()/1000;
			break;
		case 2:
			Calendar calendar = Calendar.getInstance();
			calendar.add(Calendar.DATE, -3);
			startTime = calendar.getTime().getTime()/1000;
			break;
		case 3:
			Calendar weekCalendar = Calendar.getInstance();
			weekCalendar.add(Calendar.DATE, -7);
			startTime = weekCalendar.getTime().getTime()/1000;
			break;
		case 4:
			Calendar week2Calendar = Calendar.getInstance();
			week2Calendar.add(Calendar.DATE, -14);
			startTime = week2Calendar.getTime().getTime()/1000;
			break;
		case 5:
			Calendar monthCalendar = Calendar.getInstance();
			monthCalendar.add(Calendar.MONTH, -1);
			startTime = monthCalendar.getTime().getTime()/1000;
			break;
		case 6:
			Calendar month3Calendar = Calendar.getInstance();
			month3Calendar.add(Calendar.MONTH, -3);
			startTime = month3Calendar.getTime().getTime()/1000;
			break;
		case 7:
			Calendar month6Calendar = Calendar.getInstance();
			month6Calendar.add(Calendar.MONTH, -6);
			startTime = month6Calendar.getTime().getTime()/1000;
			break;
		case 8:
			Calendar yearCalendar = Calendar.getInstance();
			yearCalendar.add(Calendar.YEAR, -1);
			startTime = yearCalendar.getTime().getTime()/1000;
			break;
		default:
			startTime = 0L;
			break;
		}
		return startTime;
	}

	/**
	 * 得到本年的第一天
	 * 
	 * @return
	 */
	public static Date getYearFirstDayTime() {
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_YEAR, calendar.getActualMinimum(Calendar.DAY_OF_YEAR));
		
		return parseDateTime(dateformater1.format(calendar.getTime())+ " 00:00:00");
	}
	
	/**
	 * 得到本年的最后一天
	 * 
	 * @return
	 */
	public static Date getYearLastDayTime() {
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_YEAR, calendar.getMaximum(Calendar.DAY_OF_YEAR));
		
		return parseDateTime(dateformater1.format(calendar.getTime())+ " 23:59:59");
	}
	
	/**
	 * 获取两个时间相差天数
	 * 
	 * @param begin
	 * @param end
	 * @return
	 * @throws ParseException     
	 * @author ShiChunyun
	 * @date 2016年7月13日 下午1:22:36
	 */
	public static int getDateSpace(Date begin, Date end){

		Calendar calst = Calendar.getInstance();;
		Calendar caled = Calendar.getInstance();

		calst.setTime(begin);
		caled.setTime(end);

        //设置时间为0时   
        calst.set(Calendar.HOUR_OF_DAY, 0);   
        calst.set(Calendar.MINUTE, 0);   
        calst.set(Calendar.SECOND, 0);   
        caled.set(Calendar.HOUR_OF_DAY, 0);   
        caled.set(Calendar.MINUTE, 0);   
        caled.set(Calendar.SECOND, 0);   
       
        //得到两个日期相差的天数   
        int days = ((int)(caled.getTime().getTime()/1000)-(int)(calst.getTime().getTime()/1000))/3600/24;   
        
        return days;   
	}
	
	/**
	 * 获取两个时间相差年数
	 * 
	 * @param begin
	 * @param end
	 * @return
	 * @throws ParseException     
	 * @author ShiChunyun
	 * @date 2016年7月13日 下午1:22:36
	 */
	public static int getYearSpace(Date begin, Date end){

		Calendar calst = Calendar.getInstance();
		Calendar caled = Calendar.getInstance();

		calst.setTime(begin);
		caled.setTime(end);
       
		int calstYear = calst.get(Calendar.YEAR);
		int caledYear = caled.get(Calendar.YEAR);
		
		return caledYear - calstYear;   
	}

	/**
	 * 根据开始时间，结束时间获取中间的时间列表
	 * @param dBegin	开始时间
	 * @param dEnd		结束时间
	 * @param dateType		时间类型：1,年	2，月	3，日
	 * @param dateFormat	格式化时间类型：yyyy-MM-dd
	 * @return
	 */
	public static List<String> findDates(Date dBegin, Date dEnd, String dateType, String dateFormat) {
		List<String> lDate = new ArrayList<String>();
		SimpleDateFormat sd = new SimpleDateFormat(dateFormat);
		lDate.add(sd.format(dBegin));
		// 使用给定的 Date 设置此 Calendar 的时间
		Calendar calBegin = Calendar.getInstance();
		calBegin.setTime(dBegin);
		// 使用给定的 Date 设置此 Calendar 的时间
		Calendar calEnd = Calendar.getInstance();
		calEnd.setTime(dEnd);
		// 测试此日期是否在指定日期之后
		while (dEnd.after(calBegin.getTime())) {
			// 根据日历的规则，为给定的日历字段添加或减去指定的时间量
			if("1".equals(dateType)){
				calBegin.add(Calendar.MONTH, 1);
			}else{
				calBegin.add(Calendar.DAY_OF_MONTH, 1);
			}
			lDate.add(sd.format(calBegin.getTime()));
		}
		return lDate;
	}


	/**
	 * 根据开始时间，结束时间获取中间的时间列表 并且只要周期类的list
	 * @param dBegin 开始时间
	 * @param dEnd 结束时间
	 * @param dateType 时间类型：1,日	2，周	3，月，4.年
	 * @param planInterval 相隔多久
	 * @param firstDay 周期开始时间
	 * @param lastDay 周期结束时间
	 * @return
	 */
	@SneakyThrows
	public static List<String> findDates(String dBegin, String dEnd, String dateType, Integer planInterval, String firstDay, String lastDay) {
		List<String> lDate = new ArrayList<String>();
		SimpleDateFormat sd = new SimpleDateFormat(YYYY_MM_DD);
		if(StringUtils.isBlank(dEnd)){
			dEnd = lastDay;
		}
		lDate.add(dBegin.substring(0,10));
		// 使用给定的 Date 设置此 Calendar 的时间
		Calendar calBegin = Calendar.getInstance();
		calBegin.setTime(sd.parse(dBegin));
		// 使用给定的 Date 设置此 Calendar 的时间
		Calendar calEnd = Calendar.getInstance();
		calEnd.setTime(sd.parse(dEnd));
		// 测试此日期是否在指定日期之后
		if(StringUtils.isNotBlank(dateType)){
			while (sd.parse(dEnd).after(calBegin.getTime())) {
				// 根据日历的规则，为给定的日历字段添加或减去指定的时间量
				if("1".equals(dateType)){
					calBegin.add(Calendar.DAY_OF_MONTH, planInterval);
				}else if("2".equals(dateType)){
					calBegin.add(Calendar.DAY_OF_MONTH, planInterval*7);
				}else if("3".equals(dateType)){
					calBegin.add(Calendar.MONTH, planInterval);
				}else if("4".equals(dateType)){
					calBegin.add(Calendar.YEAR, planInterval);
				}
				if(sd.format(calBegin.getTime()).compareTo(dEnd)<=0 ){
					lDate.add(sd.format(calBegin.getTime()));
				}
			}
		}
		lDate = lDate.stream().filter(itm -> itm.compareTo(firstDay)>=0 && itm.compareTo(lastDay)<=0).collect(Collectors.toList());
		return lDate;
	}






	public static void main(String[] args) {

	}

}
