package com.rootcloud.common.net;

import net.sf.ezmorph.object.DateMorpher;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;
import net.sf.json.util.JSONUtils;
import net.sf.json.util.PropertyFilter;
import org.apache.commons.lang3.StringUtils;

import java.io.*;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * JSON数据转换工具
 * @version 1.0
 * @author Adams Fum 
 * @date 2015年7月31日 下午10:05:17
 */
public class JsonHelper {

	private static final String ENCODING = "UTF-8";
	
	// 输出配置
	static JsonConfig toJsonConfig = new JsonConfig();
	// 输入配置
	static JsonConfig getJsonConfig = new JsonConfig();
	
	static PropertyFilter filter = new PropertyFilter(){
	    @SuppressWarnings("rawtypes")
		public boolean apply(Object source, String name, Object value) {
	        if ( JSONUtils.isNull( value )) {
	            return true;   // 返回 true, 表示这个属性将被过滤掉
	        }else if(JSONUtils.isString(value) && StringUtils.isBlank((String)value)){
	        	return true;
	        }else if(value instanceof String[] && ((String[])value).length == 0){
	        	return true;
	        }else if(value instanceof Collection && ((Collection)value).size() == 0){
	        	return true;
	        }
	        return false;
	    }
	};
	static{
		getJsonConfig.setJsonPropertyFilter(filter);
		toJsonConfig.setJsonPropertyFilter(filter);
		toJsonConfig.registerJsonValueProcessor(Date.class, new DateJsonValueProcessor());  
		JSONUtils.getMorpherRegistry().registerMorpher(new TimestampToDateMorpher());
		String[] InputFormats = new String[] {"yyyy-MM-dd HH:mm:ss","yyyyMMddHHmmss","yyyy-MM-dd"};
		JSONUtils.getMorpherRegistry().registerMorpher(new DateMorpher(InputFormats));
	}

    /**
    *
    * 从普通的Bean转换为字符串
    * @param o
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static String getJson(Object o){
       JSONObject jo=JSONObject.fromObject(o,toJsonConfig);
       return jo.toString();
   }
/**
    *
    * 从普通的Bean转换为Json对象
    * @param o
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static JSONObject getJsonObject(Object o){
       JSONObject jo=JSONObject.fromObject(o,toJsonConfig);
       return jo;
   }
   /**
    * 从Java的列表转换为字符串
    * 
    * @param list
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static <T> String getJson(List<T> list){
       JSONArray ja=JSONArray.fromObject(list,toJsonConfig);
       return ja.toString();
   }
   /**
    * 从Java的列表转换为Json数组
    * 
    * @param list
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static <T> JSONArray getJsonArray(List<T> list){
       JSONArray ja=JSONArray.fromObject(list,toJsonConfig);
       return ja;
   }
   /**
    * 从Java对象数组转换为字符串
    * @param arry
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static String getJson(Object[] arry){
       JSONArray ja=JSONArray.fromObject(arry,toJsonConfig);
       return ja.toString();
   }
   /**
    * 从Java对象数组转换为Json数组
    * @param arry
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static JSONArray getJsonArray(Object[] arry){
       JSONArray ja=JSONArray.fromObject(arry,toJsonConfig);
       return ja;
   }
   /**
    * 从Json对象转换为Java对象
    * @param jsonObject
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static Object toBean(JSONObject jsonObject){
       return JSONObject.toBean(jsonObject);
   }
   /**
    * 从Json对象转换为Java对象
    * 
    * @param jsonObject
    * @param beanClass
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static <T> Object toBean(JSONObject jsonObject,Class<T> beanClass){
       return JSONObject.toBean(jsonObject, beanClass);
   }

   /**
   * 从json格式的字符串转换为Map对象
   * @param s
   * @return
   * @author Administrator
   * @date  2014-1-9
   */
  public static Object toBean(String s){
      return JSONObject.fromObject(s, getJsonConfig);
  }
  
   /**
    * 从json格式的字符串转换为某个Bean
    * 
    * @param s
    * @param cls
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static <T> Object toBean(String s,Class<T> cls){
       JSONObject jo=JSONObject.fromObject(s, getJsonConfig);
       return JSONObject.toBean(jo, cls);
   }
   
   /**
    * 从json字符串格式转换为Collection
    * 
    * @param jsonArray
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   @SuppressWarnings("unchecked")
   public static <T> Collection<T> toCollection(String s){
       return JSONArray.toCollection(JSONArray.fromObject(s,getJsonConfig));
   }

   /**
    * 从json字符串格式转换为Collection
    * 
    * @param jsonArray
    * @param cls
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   @SuppressWarnings("unchecked")
   public static <T> Collection<T> toCollection(String s,Class<T> cls){
       return JSONArray.toCollection(JSONArray.fromObject(s,getJsonConfig),cls);
   }

   /**
    * 从json数组格式转换为Collection
    * 
    * @param jsonArray
    * @param cls
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   @SuppressWarnings("unchecked")
   public static <T> Collection<T> toCollection(JSONArray jsonArray,Class<T> cls){
	   
       return JSONArray.toCollection(jsonArray, cls);
   }

   /**
    * 从json数组格式转换为Collection
    * 
    * @param jsonArray
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   @SuppressWarnings("unchecked")
   public static <T>Collection<T> toCollection(JSONArray jsonArray){
       return JSONArray.toCollection(jsonArray);
   }
   /**
    * 从json格式的字符串转换为JSONArray数组
    * 
    * @param s
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   public static JSONArray getJsonArray(String s){
       return JSONArray.fromObject(s,toJsonConfig);
   }
   
   /**
    * 从json格式的字符串转换为某类对象的数组
    * 
    * @param s
    * @param cls
    * @return
    * @author Administrator
    * @date  2014-1-9
    */
   @SuppressWarnings("rawtypes")
   public static Object getArray(String s,Class cls){
       JSONArray ja=JSONArray.fromObject(s,toJsonConfig);
       return JSONArray.toArray(ja, cls);
   }
   
   /**
    * 读取json文件 
    * 
    * @param filePath 文件路+文件名称，如：d:\\json\myfile.json
    * @return     
    * @author Adams Fum
    * @date  2014年12月16日
    */
	public static JSONObject readJsonObject(String filePath) {
		BufferedReader reader = null;
		// 返回值,使用StringBuffer
		StringBuffer data = new StringBuffer();
		try {
			// 指定读取文件的编码格式，要和写入的格式一致，以免出现中文乱码
			reader = new BufferedReader(new InputStreamReader(new FileInputStream(filePath),
					ENCODING));
			String str = null;
			while ((str = reader.readLine()) != null) {
				data.append(str);
			}
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			// 关闭文件流
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		return getJsonObject(data.toString());
	}
	
	/**
	 * 读取json文件  
	 * 
	 * @param filePath 文件路+文件名称，如：d:\\json\myfile.json
	 * @param beanClass
	 * @return     
	 * @author Adams Fum
	 * @date  2014年12月16日
	 */
	public static <T> Object readJsonToBean(String filePath,Class<T> beanClass) {
		return toBean(readJsonObject(filePath), beanClass);
	}

	/**
	 * 读取json文件  
	 * 
	 * @param filePath 文件路+文件名称，如：d:\\json\myfile.json
	 * @return     
	 * @author Adams Fum
	 * @date  2014年12月16日
	 */
	public static Object readJsonToBean(String filePath) {
		return toBean(readJsonObject(filePath));
	}
	
   /**
    * 读取json文件 
    * 
    * @param filePath 文件路+文件名称，如：d:\\json\myfile.json
    * @return     
    * @author Adams Fum
    * @date  2014年12月16日
    */
	public static JSONArray readJsonArray(String filePath) {
		BufferedReader reader = null;
		// 返回值,使用StringBuffer
		StringBuffer data = new StringBuffer();
		try {
			// 指定读取文件的编码格式，要和写入的格式一致，以免出现中文乱码
			reader = new BufferedReader(new InputStreamReader(new FileInputStream(filePath),
					ENCODING));
			String str = null;
			while ((str = reader.readLine()) != null) {
				data.append(str);
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			// 关闭文件流
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		return getJsonArray(data.toString());
	}
	
	/**
	 * 读取json文件  
	 * 
	 * @param filePath 文件路+文件名称，如：d:\\json\myfile.json
	 * @param beanClass
	 * @return     
	 * @author Adams Fum
	 * @date  2014年12月16日
	 */
	public static <T> Collection<T> readJsonToCollection(String filePath,Class<T> beanClass) {
		return toCollection(readJsonArray(filePath),beanClass);
	}
	
	/**
	 * 读取json文件  
	 * 
	 * @param filePath 文件路+文件名称，如：d:\\json\myfile.json
	 * @return     
	 * @author Adams Fum
	 * @date  2014年12月16日
	 */
	public static <T> Collection<T> readJsonToCollection(String filePath) {
		return toCollection(readJsonArray(filePath));
	}
	
	/**
	 * 给定路径与Json文件，存储到硬盘
	 * 
	 * @param path 文件路徑，不包括文件名
	 * @param json
	 * @param fileName 如：myfile.json  
	 * @author Adams Fum
	 * @date  2014年12月16日
	 */
	public static void writeJson(String path, Object json, String fileName) {
		writeJson(path+fileName,json);
	}

	/**
	 * 给定路径与Json文件，存储到硬盘
	 *  
	 * @param filePath 文件路+文件名称，如：d:\\json\myfile.json
	 * @param json     
	 * @author Adams Fum
	 * @date  2014年12月16日
	 */
	public static void writeJson(String filePath, Object json) {
		BufferedWriter writer = null;
		File file = new File(filePath);
		// 如果文件不存在，则新建一个
		if (!file.exists()) {
			try {
				file.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		// 写入
		try {
			writer = new BufferedWriter(new FileWriter(file));
			writer.write(json.toString());
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (writer != null) {
					writer.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	 /**
	    * 读取json文件 
	    * 
	    * @param filePath 文件路+文件名称，如：d:\\json\myfile.json
	    * @return     
	    * @author Adams Fum
	    * @date  2014年12月16日
	    */
		public static JSONObject readJsonToObject(InputStream is) {
			BufferedReader reader = null;
			// 返回值,使用StringBuffer
			StringBuffer data = new StringBuffer();
			try {
				// 指定读取文件的编码格式，要和写入的格式一致，以免出现中文乱码
				reader = new BufferedReader(new InputStreamReader(is,
						ENCODING));
				String str = null;
				while ((str = reader.readLine()) != null) {
					data.append(str);
				}
				
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				// 关闭文件流
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}

			return getJsonObject(data.toString());
		}
		
	public static void main(String [] args){
		System.out.println(JsonHelper.readJsonObject("D://json/domain/loginPage_zh_TW.json"));
		
	}
}
