package com.rootcloud.common.util;

import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.domain.entity.sp.ServicePlanEntity;
import jline.internal.Log;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Random;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @Auther poppy.lu
 * @Date 2023/01/10
 * @Version 1.0
 * @Describe:
 */
public class CommonUtils {

    /**
     * 生成固定长度随机码
     *
     * @param n 长度
     */

    public static String getRandom(long n) {
        long min = 1, max = 9;
        for (int i = 1; i < n; i++) {
            min *= 10;
            max *= 10;
        }
        long rangeLong = (((long) (new Random().nextDouble() * (max - min)))) + min;
        return String.valueOf(rangeLong);
    }

    //length用户要求产生字符串的长度
    public static String getRandomString(int length) {
        String str = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(62);
            sb.append(str.charAt(number));
        }
        return sb.toString();
    }


    public static int get_random(int start, int end) {
        return (int) (Math.random() * (end - start + 1) + start);
    }


    /**
     * Object转BigDecimal类型-王雷-2018年5月14日09:56:26
     *
     * @param value 要转的object类型
     * @return 转成的BigDecimal类型数据
     */
    public static BigDecimal getBigDecimal(Object value) {
        BigDecimal ret = null;
        if (value != null) {
            if (value instanceof BigDecimal) {
                ret = (BigDecimal) value;
            } else if (value instanceof String) {
                ret = new BigDecimal((String) value);
            } else if (value instanceof BigInteger) {
                ret = new BigDecimal((BigInteger) value);
            } else if (value instanceof Number) {
                ret = new BigDecimal(((Number) value).doubleValue());
            } else {
                throw new ClassCastException("Not possible to coerce [" + value + "] from class " + value.getClass() + " into a BigDecimal.");
            }
        }
        return ret;
    }


    public static String uuid() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }


    /**
     * 获取下一个周期时间
     *
     * @param servicePlan
     * @return
     */
    public String getNextDateByStartDateNew(ServicePlanEntity servicePlan) {
        Date endDate = null;
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
            //获取当前日期
            String current = df.format(new Date());
            Date currentDate = df.parse(current);
            //获取计划开始日期
            endDate = df.parse(servicePlan.getPlanStartTime());
            Integer planCycle = servicePlan.getPlanCycle();
            Integer planInterval = servicePlan.getPlanInterval();
            do {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(endDate);
                //日
                if (ServicePlanEnums.PlanCycle.DAY.getCode().equals(planCycle)) {
                    calendar.add(Calendar.DATE, planInterval);
                }
                //周
                if (ServicePlanEnums.PlanCycle.WEEK.getCode().equals(planCycle)) {
                    calendar.add(Calendar.DATE, 7 * planInterval);
                }
                //月
                if (ServicePlanEnums.PlanCycle.MONTH.getCode().equals(planCycle)) {
                    calendar.add(Calendar.MONTH, planInterval);
                }
                //年
                if (ServicePlanEnums.PlanCycle.YEAR.getCode().equals(planCycle)) {
                    calendar.add(Calendar.YEAR, planInterval);
                }
                endDate = calendar.getTime();
                System.out.println("下一周期时间：" + df.format(endDate));
            } while (endDate.getTime()<=currentDate.getTime());
            return df.format(endDate);
        } catch (Exception e) {
            Log.error("获取数据失败，失败原因：" + e.getMessage());
            return null;
        }

    }


    public static void main(String[] args) {
        CommonUtils commonUtils = new CommonUtils();
        ServicePlanEntity servicePlan = new ServicePlanEntity();
        servicePlan.setPlanStartTime("2023-02-04");
        servicePlan.setPlanCycle(1);
        servicePlan.setPlanInterval(3);
        String endData = commonUtils.getNextDateByStartDateNew(servicePlan);
        System.out.println(endData);
    }


}
