package com.rootcloud.common.util;

import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * 2.0系统中的时间转换工具类
 * */
public class DateUtils {

    public static final String C_DATE_DIVISION = "-";

    public static final String C_TIME_PATTON_DEFAULT = "yyyy-MM-dd HH:mm:ss";

    public static final String C_DATE_PATTON_DEFAULT = "yyyy-MM-dd";

    public static final String C_DATA_PATTON_YYYYMMDD = "yyyyMMdd";

    public static final String C_TIME_PATTON_HHMMSS = "HH:mm:ss";

    public static final int C_ONE_SECOND = 1000;

    public static final int C_ONE_MINUTE = 60 * C_ONE_SECOND;

    public static final int C_ONE_HOUR = 60 * C_ONE_MINUTE;

    public static final long C_ONE_DAY = 24 * C_ONE_HOUR;

    /**
     * 按指定类型将字符类型转为日期类型
     *
     * @param strFormat
     * @param dateValue
     * @return Date
     */
    public static Date parseDate(String strFormat, String dateValue) {
        strFormat = StringUtils.trimToNull(strFormat);
        dateValue = StringUtils.trimToNull(dateValue);

        if (dateValue == null)
            return null;

        if (strFormat == null) {
            strFormat = C_TIME_PATTON_DEFAULT;
        }

        SimpleDateFormat dateFormat = new SimpleDateFormat(strFormat);
        Date newDate = null;

        try {
            newDate = dateFormat.parse(dateValue);
        } catch (ParseException pe) {
            newDate = null;
        }

        return newDate;
    }

    /**
     * 获取两日期的天数差
     *
     * @param fDate
     *            开始时间
     * @param oDate
     *            结束时间
     * @return 天数差
     */
    public static int daysOfTwo(Date fDate, Date oDate) {

        Calendar aCalendar = Calendar.getInstance();

        aCalendar.setTime(fDate);

        int day1 = aCalendar.get(Calendar.DAY_OF_YEAR);

        aCalendar.setTime(oDate);

        int day2 = aCalendar.get(Calendar.DAY_OF_YEAR);

        return day2 - day1;

    }

    /**
     * 获取时间的日
     *
     * @param nowDate
     * @return
     */
    public static int getDay(Date nowDate) {
        Calendar nowCalendar = Calendar.getInstance();
        nowCalendar.setTime(nowDate);
        return nowCalendar.get(Calendar.DATE);
    }

    /**
     * 获取本周的开始时间
     *
     * @return
     */
    public static Date getBeginDayOfWeek(Date nowdate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(nowdate);
        int dayofweek = cal.get(Calendar.DAY_OF_WEEK);
        if (dayofweek == 1) {
            dayofweek += 7;
        }
        cal.add(Calendar.DATE, 2 - dayofweek);
        return cal.getTime();
    }

    /**
     * 获取本周的结束时间
     *
     * @return
     */
    public static Date getEndDayOfWeek(Date nowdate) {
        return addDays(getBeginDayOfWeek(nowdate), 6);
    }

    /**
     * 将Date类型的日期转换为指定格式的字符串
     *
     * @param aTs_Datetime
     * @param as_Pattern
     * @return
     */
    public static String format(Date aTs_Datetime, String as_Pattern) {
        if (aTs_Datetime == null || as_Pattern == null)
            return null;

        SimpleDateFormat dateFromat = new SimpleDateFormat();
        dateFromat.applyPattern(as_Pattern);

        return dateFromat.format(aTs_Datetime);
    }

    /**
     * 取得指定日期N天后的日期
     *
     * @param date
     * @param days
     * @return
     */
    public static Date addDays(Date date, int days) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);

        cal.add(Calendar.DATE, days);

        return cal.getTime();
    }

    /**
     * 获取时间的月份
     *
     * @param nowDate
     * @return
     */
    public static int getMonth(Date nowDate) {
        int re = 0;
        Calendar nowCalendar = Calendar.getInstance();
        nowCalendar.setTime(nowDate);

        re = nowCalendar.get(2) + 1;
        return re;
    }

    /**
     * 获取时间的年份
     *
     * @param nowDate
     * @return
     */
    public static int getYear(Date nowDate) {
        int re = 0;
        Calendar nowCalendar = Calendar.getInstance();
        nowCalendar.setTime(nowDate);
        re = nowCalendar.get(1);
        return re;
    }

    /***
     *
     * @Title: getCurrentMonthDays
     * @Description: 获取当月天数
     * @Author: gw_renwm2
     * @Datetime: Mar 9, 2017 2:51:19 PM
     * @param year		年；4位数
     * @param month	月；2位数
     * @return
     */
    public static int getCurrentMonthDays(String year, String month) {
        Date date = DateUtils.parseDate(DateConstant.FMT_CH_YEARMON, year+"-"+month);
        return getCurrentMonthDays(date);
    }

    /***
     *
     * @Title: getCurrentMonthDays
     * @Description: 获取当月天数
     * @Author: gw_renwm2
     * @Datetime: Mar 9, 2017 2:51:19 PM
     * @param date		日期
     * @return
     */
    public static int getCurrentMonthDays(Date date) {
        if (date == null) {
            return 0;
        }

        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.set(Calendar.DATE, 1);
        c.roll(Calendar.DATE, -1);
        return c.get(Calendar.DATE);
    }

    /***
     * 获取当前月的最后一天
     * params:年月（2017-02）
     * return:本月的最后一天日期（2017-02-28）
     */
    public static String getCurMonLastDay(String yearMonth) {
        SimpleDateFormat sdf = new SimpleDateFormat(DateConstant.FMT_CH_DATE);
        Calendar ca = Calendar.getInstance();
        String d = yearMonth;
        ca.setTime(DateUtils.parseDate(DateConstant.FMT_CH_YEARMON, d));
        ca.set(Calendar.DAY_OF_MONTH, ca.getActualMaximum(Calendar.DAY_OF_MONTH));
        String last = sdf.format(ca.getTime());
        return last;
    }

    /**
     *
     * @Title: getTheMonLastDay
     * @Description: 获取某月的最后一天，如果是当年当月，则是当天
     * @param year
     * @param mon
     * @return
     */
    public static int getTheMonLastDay(String year, String month) {
        SimpleDateFormat sdf = new SimpleDateFormat(DateConstant.FMT_CH_DATE);
        String strNow = sdf.format(new Date());
        String curYear = strNow.substring(0,4);
        String curMonth = strNow.substring(5,7);

        if(curYear.equals(year) && curMonth.equals(month)) {
            String curDay = strNow.substring(8);
            return Integer.parseInt(curDay);
        } else {
            return getCurrentMonthDays(year, month);
        }
    }

    /**
     * 获取指定年月的最后一天；如果是当前月，那么取当前日期
     */
    public static int getLastDay(int year, int month) {
        return getTheMonLastDay(String.valueOf(year), String.valueOf(month));
    }

    /**
     *
     * @Title: getTheYearLastMon
     * @Description: 获取某年的最后一月，如果是当年，则是当月
     * @param year
     * @param mon
     * @return
     */
    public static int getTheYearLastMon(String year) {
        SimpleDateFormat sdf = new SimpleDateFormat(DateConstant.FMT_CH_DATE);
        String strNow = sdf.format(new Date());
        String curYear = strNow.substring(0,4);
        String curMonth = strNow.substring(5,7);

        if(curYear.equals(year)) {
            return Integer.parseInt(curMonth);
        } else {
            return 12;
        }
    }

    /**
     * 获得指定季度月份
     * @param quarter
     * @return
     */
    public static Integer[] getQuarterMonths(int quarter){

        if( 1==quarter ){
            return new Integer[]{1,2,3};
        }else if( 2==quarter ){
            return new Integer[]{4,5,6};
        }else if( 3==quarter ){
            return new Integer[]{7,8,9};
        }else{
            return new Integer[]{10,11,12};
        }
    }

    /**
     * 获得指定周开始日期和结束日期
     * 需要注意跨年和跨月
     * @param quarter
     * @return
     */
    public static List<Map<String,Object >> getWeekDays(int year, int week){
        List<Map<String,Object >> result = new ArrayList<Map<String,Object >>();

        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd");
        Calendar cal=Calendar.getInstance();

        // 设置每周的开始日期
        cal.setFirstDayOfWeek(Calendar.MONDAY);


        cal.set(Calendar.YEAR, year);
        cal.set(Calendar.WEEK_OF_YEAR, week);

        cal.set(Calendar.DAY_OF_WEEK, cal.getFirstDayOfWeek());
        String beginDate = sdf.format(cal.getTime());
        int beginYear = cal.get(Calendar.YEAR);
        int beginMonth = cal.get(Calendar.MONTH)+1;
        int beginDay = cal.get(Calendar.DAY_OF_MONTH);
        int beginLastDay = 0 ;



        cal.add(Calendar.DAY_OF_WEEK, 6);
        String endDate = sdf.format(cal.getTime());
        int endYear = cal.get(Calendar.YEAR);
        int endMonth = cal.get(Calendar.MONTH)+1;
        int endDay = cal.get(Calendar.DAY_OF_MONTH);

        Integer[] beginDays = null ;
        Integer[] endDays = null ;
        if( beginMonth!=endMonth ){//跨月
            beginLastDay = getLastDay(beginYear,beginMonth) ;
            beginDays = new Integer[beginLastDay-beginDay+1];
            int j=0;
            for(int i= beginDay;i<=beginLastDay; i++ ){
                beginDays[j]=i;
                j++;
            }

            endDays = new Integer[endDay];
            for(int i= 1;i<=endDay; i++ ){
                endDays[i-1] = i;
            }

        }

        if( beginMonth!=endMonth ){//跨月
            Map<String,Object> beginData = new HashMap<String, Object>();
            beginData.put("year", beginYear);
            beginData.put("month", beginMonth);
            beginData.put("day", beginDays);
            result.add(beginData);

            Map<String,Object> endData = new HashMap<String, Object>();
            endData.put("year", endYear);
            endData.put("month", endMonth);
            endData.put("day", endDays);
            result.add(endData);
        }else{
            beginDays = new Integer[endDay-beginDay+1];
            int j = 0;
            for(int i= beginDay;i<=endDay; i++ ){
                beginDays[j]=i;
                j++;
            }

            Map<String,Object> beginData = new HashMap<String, Object>();
            beginData.put("year", beginYear);
            beginData.put("month", beginMonth);
            beginData.put("day", beginDays);
            result.add(beginData);
        }
        System.out.println( result );
        System.out.println( beginDate );
        System.out.println( endDate );

        return result ;
    }

    /**
     * 根据年份和月份获得当前月的开始日期
     *
     * @param year
     *            年份
     * @param month
     *            月份
     *
     * @return 所属月份的开始日期(java.util.Date)
     */
    public static Date getMonthBegin(int year, int month) {
        if (month <= 0 || month > 12)
            throw new IllegalArgumentException("month must from 1 to 12");
        GregorianCalendar gCal = new GregorianCalendar(year, month - 1, 1);
        return gCal.getTime();
    }
    /**
     * 根据日期获得当前月的结束日期
     *
     * @param month
     *            日期
     *
     * @return 所属月份的结束日期(java.util.Date)
     */
    public static Date getMonthEnd(Date month) {
        GregorianCalendar ca = new GregorianCalendar();
        ca.setTime(month);
        int year = ca.get(GregorianCalendar.YEAR);
        int mon = ca.get(GregorianCalendar.MONTH);
        int lastDay = ca.getActualMaximum(GregorianCalendar.DAY_OF_MONTH);
        GregorianCalendar gCal = new GregorianCalendar(year, mon, lastDay);
        return gCal.getTime();
    }

    /**
     * 根据年份和月份获得当前月的结束日期
     *
     * @param year
     *            年份
     * @param month
     *            月份
     *
     * @return 所属月份的结束日期(java.util.Date)
     */
    public static Date getMonthEnd(int year, int month) {
        Date start = getMonthBegin(year, month);
        return getMonthEnd(start);
    }

    /**
     * 根据日期参数获取当前周的起始日期(周日)
     *
     * @param date
     *            日期
     *
     * @return 所属星期的起始日期(java.util.Date)
     */
    public static Date getWeekBegin(Date date) {
        GregorianCalendar gCal = new GregorianCalendar();
        gCal.setTime(date);
        int startDay = gCal.getActualMinimum(GregorianCalendar.DAY_OF_WEEK);
        gCal.set(GregorianCalendar.DAY_OF_WEEK, startDay);
        return gCal.getTime();
    }

    /**
     * 根据日期参数获取当前周的结束日期(周六)
     *
     * @param date
     *            日期
     *
     * @return 所属星期的结束日期(java.util.Date)
     */
    public static Date getWeekEnd(Date date) {
        GregorianCalendar gCal = new GregorianCalendar();
        gCal.setTime(date);
        int lastDay = gCal.getActualMaximum(GregorianCalendar.DAY_OF_WEEK);
        gCal.set(GregorianCalendar.DAY_OF_WEEK, lastDay);
        return gCal.getTime();
    }

    /**
     * 获取两个日期相差几个月
     * @param startDate
     * @param endDate
     * @return
     */
    public static int getMonthsOfTime(Date startDate,Date endDate){
        Calendar c = Calendar.getInstance();
        c.setTime(startDate);
        c.add(Calendar.YEAR, 1);  //年份加一年
        int year = c.get(Calendar.YEAR);
        int month = c.get(Calendar.MONTH);

        c.setTime(endDate);
        int year1 = c.get(Calendar.YEAR);
        int month1 = c.get(Calendar.MONTH);

        return 12*(year1-year)+month1-month;
    }

    /**
     * 计算两个日期的多少天数(d1-d2+1)
     * @param startDate
     * @param endDate
     * @return
     * @throws ParseException
     */
    public static int getDayDifference(String startDateStr,String endDateStr) throws ParseException{

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-M-d");
        Date d1=sdf.parse(startDateStr);
        Date d2=sdf.parse(endDateStr);
        long num = (d1.getTime()-d2.getTime())/1000;
        long days  = num/(3600*24);
        if(days<0)days=days*-1;
        return Integer.parseInt(days+"")+1;
    }

    public static final String YYYY_MM_DD = "yyyy-MM-dd";
    /**
     * 将java.util.Date格式转换成日期字符串(yyyy-MM-dd)
     *
     * @return String
     */
    public static String formatDate(Date date) {

        SimpleDateFormat dateFromat = new SimpleDateFormat();
        SimpleDateFormat df =new SimpleDateFormat("");
        return df.format(date);
    }



}
