package com.rootcloud.common.util;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @author w-chenhengqi
 */
@Component
public class GaoDeAPIUtils {

    private final static Logger log = LoggerFactory.getLogger(GaoDeAPIUtils.class);

    private  static String addressUrl;

    private static String key;
    /**api接口参数常量 */
    private static final String CODE = "code";
    private static final String STATUS = "status";
    private static final String INFO = "info";
    private static final String INFO_CODE = "infocode";
    private static final String KEY = "key";

    @Value("${gode.api.key:1ad65e39043ea039f82ac56d62ea1559}")
    public void setKey(String key) {
        GaoDeAPIUtils.key = key;
    }


    @Value("${gode.api.addressUrl:https://restapi.amap.com/v3/geocode/regeo}")
    public void setAddressUrl(String addressUrl){
        GaoDeAPIUtils.addressUrl = addressUrl;
    }

    @Resource
    private RedisTemplate redisTemplate;

    private final String redisJwdPrefix = "gps_";


    /**
     * 发送请求到高德
     *
     * @param param 发送参数
     * @return JSONArray
     */
    private JSONObject getGaoDeAddress(Map<String, String> param) {
        JSONObject bodyJson= null;
        ResponseEntity<JSONObject> responseEntity = RestTemplateUtils.get(addressUrl, JSONObject.class, param);
        //调接口成功
        if (responseEntity.getStatusCode().is2xxSuccessful()) {
            //获取返回内容
            bodyJson = responseEntity.getBody();
            //status  返回结果状态值，值为0或1，0表示请求失败；1表示请求成功
            if (bodyJson != null && Objects.equals(0, bodyJson.getInteger(STATUS))) {
                //info 成功时返回OK，否则返回错误原因
                log.warn("GaoDeAPIUtils.requestGaoDeApiGps[{}],infocode[{}]", bodyJson.getString(INFO), bodyJson.getString(INFO_CODE));
            }
        } else { //网络请求失败：1超时，2服务器错误
            log.warn("GaoDeAPIUtils.requestGaoDeApiGps[{}]", "网络请求失败");
        }
        return bodyJson;
    }

    public  String  getAddressByJwd(String jwd){
        Object addressRedis = redisTemplate.opsForValue().get(redisJwdPrefix+jwd);
        String address = null;
        if(addressRedis == null){
            Map<String, String> param = new HashMap<>();
            //key 请求服务权限标识
            param.put(KEY, key);
            param.put("location",jwd);
            try {
                log.info("高德地图查询数据==>");
                JSONObject object = getGaoDeAddress(param);
                log.info("高德地图返回结果==>{}",object);
                if (object != null) {
                    JSONObject dataJson = object.getJSONObject("regeocode");
                    if (dataJson != null) {
                        address = String.valueOf(dataJson.get("formatted_address"));
                    }
                }
            }catch (Exception e){
                log.info("调用高德地图查询地址失败==>{}",e.getMessage());
            }
            if(StringUtils.isNotBlank(address)){
                redisTemplate.opsForValue().set(redisJwdPrefix+jwd, address, 1, TimeUnit.HOURS);
            }
        }else{
            address =  String.valueOf(addressRedis);
        }
        return address;
    }


}
