package com.rootcloud.common.util;

import java.util.HashMap;
import java.util.Map;
import java.util.function.BooleanSupplier;

public class MapBuilder<V> {
    private Map<String, V> map;

    private MapBuilder() {}

    /**
     * 创建一个Map构建器
     * @param <V> Value类型
     * @return 构建器
     */
    public static <V> MapBuilder<V> of() {
        MapBuilder<V> mapBuilder = new MapBuilder<>();
        mapBuilder.map = new HashMap<>(16);
        return mapBuilder;
    }

    /**
     * 创建一个Map构建器
     * @param <V> Value类型
     * @return 构建器
     */
    public static <V> MapBuilder<V> of(int size) {
        MapBuilder<V> mapBuilder = new MapBuilder<>();
        mapBuilder.map = new HashMap<>(size);
        return mapBuilder;
    }

    /**
     * 根据指定的Map创建一个Map构建器
     * @param <V> Value类型
     * @return 构建器
     */
    public static <V> MapBuilder<V> of(Map<String, V> map) {
        MapBuilder<V> mapBuilder = new MapBuilder<>();
        mapBuilder.map = null == map ? new HashMap<>(16) : map;
        return mapBuilder;
    }

    public MapBuilder<V> put(String k, V value) {
        this.map.put(k, value);
        return this;
    }

    public MapBuilder<V> putNotNull(String k, V value) {
        if (null != value) {
            this.map.put(k, value);
        }

        return this;
    }

    public MapBuilder<V> putIf(String k, V value, BooleanSupplier supplier) {
        if (supplier.getAsBoolean()) {
            this.map.put(k, value);
        }

        return this;
    }

    public MapBuilder<V> clear() {
        this.map.clear();
        return this;
    }

    public MapBuilder<V> remove(String k) {
        this.map.remove(k);
        return this;
    }

    public Map<String, V> build() {
        return map;
    }
}