package com.rootcloud.config;


import com.rootcloud.common.JsonResult;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.config.exception.AbstractException;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.exception.ExceptionCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.context.request.NativeWebRequest;

import java.util.Objects;

/**
 * @author jun2.wang
 */
@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {
    @ExceptionHandler(Exception.class)
    public JsonResult errorHandler(Exception e) {
        log.error("error:", e);
        return JsonResult.failed(ExceptionCode.SERVER_ERROR, e.getMessage());
    }

    /**
     * 字段格式验证失败
     *
     * @param e
     * @return
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public JsonResult errorHandler(MethodArgumentNotValidException e) {
        log.error("error:", e);
        return JsonResult.failed(ExceptionCode.ARGUMENT_NOT_VALID, "字段验证失败：" + Objects.requireNonNull(e.getBindingResult().getFieldError()).getDefaultMessage());
    }

    @ExceptionHandler(AbstractException.class)
    public JsonResult handleBusinessException(AbstractException ex, NativeWebRequest request) {
        if (log.isDebugEnabled()) {
            log.debug("AbstractException 异常信息 {}", ex.getMessage());
        }
        return JsonResult.failed(ex.getCode(), ex.getMessage());
    }

    @ExceptionHandler(BusinessException.class)
    public JsonResult handleBusinessException(BusinessException ex, NativeWebRequest request) {
        if (log.isDebugEnabled()) {
            log.debug("BusinessException 异常信息 {}", ex.getMessage());
        }
        return JsonResult.failed(ex.getCode(), ResourceUtil.getI18nMessage(ex.getCode()));

    }
}
