package com.rootcloud.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.builders.RequestParameterBuilder;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.service.ParameterType;
import springfox.documentation.service.RequestParameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author: jun2.wang
 * @Date: 2021/5/26
 */
@Configuration
public class SwaggerConfig {
    @Bean
    public Docket createRestApi() {
        return new Docket(DocumentationType.OAS_30).apiInfo(apiInfo())
                // 是否开启
                .enable(true).select()
                // 扫描的路径包
                .apis(RequestHandlerSelectors.basePackage("com.rootcloud"))
                // 指定路径处理PathSelectors.any()代表所有的路径
                .paths(PathSelectors.any()).build().pathMapping("/").globalRequestParameters(getParameters());
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("SpringBoot-Swagger3集成")
                .description("springboot | swagger")
                // 作者信息
                .contact(new Contact("sprucer", "www.rootcloud.com", "developer@rootcloud.com"))
                .version("1.0.0")
                .build();
    }

    private List<RequestParameter>  getParameters(){
        List<RequestParameter> requestParameterList=new ArrayList<>();
        RequestParameter parameter = new RequestParameterBuilder()
                .name("authorization")
                .description("请求用户authorization")
                .in(ParameterType.HEADER)
                .required(false)
                .build();
        RequestParameter parameterZH = new RequestParameterBuilder()
                .name("lang")
                .description("请求用户语言类型")
                .in(ParameterType.HEADER)
                .required(false)
                .build();
        RequestParameter parameterCookies = new RequestParameterBuilder()
                .name("Cookie")
                .description("请求用户Cookie")
                .in(ParameterType.HEADER)
                .required(false)
                .build();
        requestParameterList.add(parameter);
        requestParameterList.add(parameterZH);
        requestParameterList.add(parameterCookies);
        return requestParameterList;
    }
}
