package com.rootcloud.config;

import cn.hutool.extra.spring.SpringUtil;
import com.rootcloud.common.intercept.LanguageIntercept;
import com.rootcloud.config.auth.AuthenticationInterceptor;
import com.rootcloud.config.auth.AuthorizationInterceptor;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;

import javax.annotation.Resource;

/**
 * @Author: jun2.wang
 * @Date: 2021/1/21
 */
@Configuration
@Import(SpringUtil.class)
public class WebMvcConfigurer implements org.springframework.web.servlet.config.annotation.WebMvcConfigurer {

    @Value("${application.jeboot.authUrl}")
    private String authUrl;

    @Resource
    private RedisTemplate redisTemplate;

    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        //认证拦截器
        registry.addInterceptor(new AuthenticationInterceptor(authUrl,redisTemplate,enduserService))
                .addPathPatterns("/**")
                .excludePathPatterns("/dev/help/**")
                .excludePathPatterns("/menu-info/mul-add")
                .excludePathPatterns("/job/gbsjSendMessage","/job/synxPartData")
                .excludePathPatterns("/swagger-resources/**", "/webjars/**", "/v3/**", "/swagger-ui/**");

        //鉴权拦截器
        registry.addInterceptor(new AuthorizationInterceptor());
        //语言拦截器
        registry.addInterceptor(getLanguageIntercept());
    }

    @Bean
    public LanguageIntercept getLanguageIntercept(){
        return new LanguageIntercept();
    }
}
