package com.rootcloud.config.auth;

import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author: jun2.wang
 * @Date: 2021/6/22
 */
public class AuthorizationInterceptor implements HandlerInterceptor {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        boolean haveAnnotataion = handler.getClass().isAssignableFrom(HandlerMethod.class);
        if (haveAnnotataion) {
            RequiresUserType requiresUserType = ((HandlerMethod) handler).getMethodAnnotation(RequiresUserType.class);
            if (requiresUserType != null) {
                List<UserTypeEnum> userTypes = getUserType(UserContext.current());
                for (UserTypeEnum item : requiresUserType.value()) {
                    if (userTypes.contains(item)) {
                        return true;
                    }
                }
                response.sendError(HttpServletResponse.SC_FORBIDDEN, "功能未授权");
                return false;
            }
        }
        return true;
    }

    private List<UserTypeEnum> getUserType(LoginUserInfo user) {
        List<UserTypeEnum> userTypes = new ArrayList<>();
        userTypes.add(UserTypeEnum.NORMAL);
        if (user.isTenantAdmin()) {
            userTypes.add(UserTypeEnum.TENANT_ADMIN);
        }
        if (user.isSupper()) {
            userTypes.add(UserTypeEnum.SUPPER);
        }
        return userTypes;
    }
}
