package com.rootcloud.config.rabbitmq;

import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.ContentTypeDelegatingMessageConverter;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;


@Configuration
public class RabbitMQConfig {
    @Value("${spring.rabbitmq.addresses}")
    private String addresses;
    @Value("${spring.rabbitmq.port}")
    private int port;
    @Value("${spring.rabbitmq.username}")
    private String username;
    @Value("${spring.rabbitmq.password}")
    private String password;


    /**
     * -----------------------------------------------------------
     * 创建项目中所有用到的队列
     * -----------------------------------------------------------
     */
    /**
     * 1.创建队列：QUEUE_CREATE_TENANT_OPERATION
     *
     * @return
     */
   /* @Bean(name = "QueueCreateTenant")
    public Queue QueueCreateTenant() {
        return new Queue(RabbitMQConstant.QUEUE_CREATE_TENANT_OPERATION);
    }*/


    /**
     * -----------------------------------------------------------
     * 创建项目中所有用到的交换机
     * -----------------------------------------------------------
     */
    /**
     * 1. FANOUT类型交换机：FANOUT_EXCHANGE_TENANT
     *
     * @return
     */
  /*  @Bean(name = "ExchangeByCreateTenant")
    public FanoutExchange createExchangeByCreateTenant() {
        return new FanoutExchange(RabbitMQConstant.FANOUT_EXCHANGE_TENANT);
    }*/


    /**
     * -----------------------------------------------------------
     * 把项目中所有用到的交换机和队列进行绑定
     * -----------------------------------------------------------
     */
    /**
     * 1. 把队列（Queue）绑定到交换器（Exchange）
     *
     * @param queue    QueueCreateTenant
     * @param exchange
     * @return
     */
 /*   @Bean
    public Binding bindingByCreateTenant(@Qualifier("QueueCreateTenant") Queue queue, @Qualifier("ExchangeByCreateTenant") FanoutExchange exchange) {
        return BindingBuilder.bind(queue).to(exchange);
    }*/


    /**
     * -----------------------------------------------------------
     * 公用配置
     * -----------------------------------------------------------
     */
    @Bean
    public ConnectionFactory connectionFactory() {
        CachingConnectionFactory connectionFactory = new CachingConnectionFactory(addresses, port);
        connectionFactory.setUsername(username);
        connectionFactory.setPassword(password);
        return connectionFactory;
    }

    @Bean
    public SimpleRabbitListenerContainerFactory rabbitListenerContainerFactory(
            ConnectionFactory connectionFactory, MessageConverter messageConverter) {
        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        factory.setConnectionFactory(connectionFactory);
        factory.setMessageConverter(messageConverter);
        return factory;
    }

    @Bean
    public MessageConverter messageConverter() {
        return new ContentTypeDelegatingMessageConverter(new Jackson2JsonMessageConverter());
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate rabbitTemplate(
            ConnectionFactory connectionFactory, MessageConverter messageConverter) {
        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
        rabbitTemplate.setMessageConverter(messageConverter);
        return rabbitTemplate;
    }

}
