package com.rootcloud.config.rabbitmq;

import com.alibaba.fastjson.JSON;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.UUID;

/**
 * 发送消息
 * @param <T>
 */
@Component

public class Sender<T> implements RabbitTemplate.ConfirmCallback{
    private final Logger log = LoggerFactory.getLogger(Sender.class);

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @PostConstruct
    public void init() {
        rabbitTemplate.setConfirmCallback(this);
    }

    @Override
    public void confirm(CorrelationData correlationData, boolean ack, String cause) {
        if (ack) {
            log.info("消息发送成功，msgId: {}",correlationData.getId());
        } else {
            log.error("消息发送失败：" + cause);
            throw new RuntimeException("消息发送失败：" + cause);
        }
    }


    /**
     * 发送消息
     * @param exchange 交换机名称
     * @param routingKey 路由关键字
     * @param object 发送的消息内容
     */
    public void send(String exchange,String routingKey, T object){
        CorrelationData correlationId = new CorrelationData(UUID.randomUUID().toString());
        log.info("开始发送消息: " + JSON.toJSONString(object));
        rabbitTemplate.convertAndSend(exchange,routingKey, object, correlationId);
    }

    /**
     * 发送消息
     *
     * @param routingKey 路由关键字
     * @param object     发送的消息内容
     */
    public void send(String routingKey, T object) {
        CorrelationData correlationId = new CorrelationData(UUID.randomUUID().toString());
        log.info("开始发送消息: " + JSON.toJSONString(object));
        rabbitTemplate.convertAndSend(routingKey, object, correlationId);
    }

}
