package com.rootcloud.config.unipush;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.getui.push.v2.sdk.ApiHelper;
import com.getui.push.v2.sdk.GtApiConfiguration;
import com.getui.push.v2.sdk.api.PushApi;
import com.getui.push.v2.sdk.common.ApiResult;
import com.getui.push.v2.sdk.dto.req.Audience;
import com.getui.push.v2.sdk.dto.req.Settings;
import com.getui.push.v2.sdk.dto.req.Strategy;
import com.getui.push.v2.sdk.dto.req.message.PushChannel;
import com.getui.push.v2.sdk.dto.req.message.PushDTO;
import com.getui.push.v2.sdk.dto.req.message.PushMessage;
import com.getui.push.v2.sdk.dto.req.message.android.AndroidDTO;
import com.getui.push.v2.sdk.dto.req.message.android.ThirdNotification;
import com.getui.push.v2.sdk.dto.req.message.android.Ups;
import com.rootcloud.common.HttpUtils;
import com.rootcloud.common.util.CommonUtils;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * @author lujiumin
 * @date 2023/1/6
 * 描述：个推 2.0 版本
 */
@Component
public class GtUtils {

    /**
     * app-id: U4wShph5E16OQcRBw9QYQ7
     * app-key: 7PjMXLUJ0a6rlNJU2TifF1
     * master-secret: tPvjXYksxhAVaihZZaRPZ
     * url: http://api.getui.com/apiex.htm
     */
    @Value(value = "${unipush.app-id}")
    private String appId;
    @Value(value = "${unipush.app-key}")
    private String appKey;
    @Value(value = "${unipush.master-secret}")
    private String masterSecret;

    private String getui_url = "https://restapi.getui.com/v2/";

    /**
     * 生成签名
     *
     * @param appKey
     * @param masterSecret
     */
    public Map<String, String> build(String appKey, String masterSecret) {
        Map<String, String> maps = new HashMap<>();
        Long timestamp = System.currentTimeMillis();
        maps.put("appkey", appKey);
        maps.put("timestamp", timestamp + "");
        maps.put("sign", sha256(appKey + timestamp + masterSecret));
        return maps;
    }

    private static String sha256(String s) {
        try {
            return hex(MessageDigest.getInstance("SHA-256").digest(s.getBytes(Charset.forName("UTF-8"))));
        } catch (NoSuchAlgorithmException var3) {
            throw new AssertionError(var3);
        }
    }

    private static final char[] HEX_DIGITS = new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    public static String hex(byte[] data) {
        char[] result = new char[data.length * 2];
        int c = 0;
        byte[] var3 = data;
        int var4 = var3.length;

        for (int var5 = 0; var5 < var4; ++var5) {
            byte b = var3[var5];
            result[c++] = HEX_DIGITS[b >> 4 & 15];
            result[c++] = HEX_DIGITS[b & 15];
        }

        return new String(result);
    }

    /**
     * 登录
     */
    private String auth() {
        String url = getui_url + appId + "/auth";
        System.out.println("个推登录URL:" + url);
        Map<String, String> maps = build(appKey, masterSecret);
        System.out.println("个推登录maps:" + JSON.toJSONString(maps));
        String result = HttpUtils.httpPostRaw(url, JSON.toJSONString(maps), null, null);
        System.out.println("结果:" + result);
        if (StringUtils.isNotBlank(result)) {
            JSONObject jsonObject = JSON.parseObject(result);
            if (jsonObject.containsKey("code")) {
                String code = jsonObject.getString("code");
                if ("0".equals(code)) {
                    String data = jsonObject.getString("data");
                    JSONObject dataJson = JSON.parseObject(data);
                    if (dataJson.containsKey("token")) {
                        String token = dataJson.getString("token");
                        return token;
                    }
                }
            }
        }
        return null;
    }

    /**
     * 推送
     */
    public void pushMessage(String body, String title, String cid) {
        Log.info("在线推送CID：" + cid + ",标题：" + title + ",内容：" + body);
        //路径
        String url = getui_url + appId + "/push/single/cid";
        //准备参数
        Map<String, Object> param = new HashMap<>();
        param.put("request_id", CommonUtils.uuid());
        Map<String, Object> settings = new HashMap<>();
        settings.put("ttl", 7200000);
        param.put("settings", settings);
        Map<String, Object> audience = new HashMap<>();
        List<String> list = new ArrayList<>();
        list.add(cid);
        audience.put("cid", list);
        param.put("audience", audience);
        Map<String, Object> push_message = new HashMap<>();
        Map<String, Object> notification = new HashMap<>();
        notification.put("title", title);
        notification.put("body", body);
        notification.put("big_text", body);
        notification.put("click_type", "url");
        notification.put("url", "/pages/preview/preview");
        push_message.put("notification", notification);
        param.put("push_message", push_message);
        //获取token
        String token = auth();
        if (StringUtils.isNotBlank(token)) {
            Map<String, String> headers = new HashMap<>();
            headers.put("token", token);
            String str = HttpUtils.httpPostRaw(url, JSON.toJSONString(param), headers, null);
            Log.info("在线个推结果：" + str);
        }

    }

    /**
     * 华为推送
     *
     * @param pushTitle
     * @param cid
     */
    public void pushHw(String pushTitle, String cid) {
        Log.info("离线推送CID：" + cid + ",内容：" + pushTitle);
        GtApiConfiguration apiConfiguration = new GtApiConfiguration();
        //填写应用配置
        apiConfiguration.setAppId(appId);
        apiConfiguration.setAppKey(appKey);
        apiConfiguration.setMasterSecret(masterSecret);
        apiConfiguration.setDomain(getui_url);
        // 实例化ApiHelper对象，用于创建接口对象
        ApiHelper apiHelper = ApiHelper.build(apiConfiguration);
        // 创建对象，建议复用。目前有PushApi、StatisticApi、UserApi
        PushApi pushApi = apiHelper.creatApi(PushApi.class);
        //根据cid进行单推
        PushDTO<Audience> pushDTO = new PushDTO<>();
        //设置推送参数
        pushDTO.setRequestId(System.currentTimeMillis() + "");//requestid需要每次变化唯一
        //配置推送条件
        // 1: 表示该消息在用户在线时推送个推通道，用户离线时推送厂商通道;
        // 2: 表示该消息只通过厂商通道策略下发，不考虑用户是否在线;
        // 3: 表示该消息只通过个推通道下发，不考虑用户是否在线；
        // 4: 表示该消息优先从厂商通道下发，若消息内容在厂商通道代发失败后会从个推通道下发。
        Strategy strategy = new Strategy();
        strategy.setDef(1);
        Settings settings = new Settings();
        settings.setStrategy(strategy);
        pushDTO.setSettings(settings);
        settings.setTtl(3600000);//消息有效期，走厂商消息需要设置该值
        PushChannel pushChannel = new PushChannel();
        // 安卓离线厂商通道推送消息体
        AndroidDTO androidDTO = new AndroidDTO();
        Ups ups = new Ups();
        Map<String, Map<String, Object>> options = new HashMap<>();
        Map<String, Object> optionsMap = new HashMap<>();
        optionsMap.put("/message/android/category", "WORK");
        options.put("HW", optionsMap);
        ups.setOptions(options);
        ThirdNotification notification1 = new ThirdNotification();
        ups.setNotification(notification1);
        notification1.setTitle("您有一条新通知,点击查看");
        notification1.setBody(pushTitle);
        notification1.setClickType("startapp");
        androidDTO.setUps(ups);
        pushChannel.setAndroid(androidDTO);
        pushDTO.setPushChannel(pushChannel);
        // PushMessage
        PushMessage pushMessage = new PushMessage();
        pushDTO.setPushMessage(pushMessage);
        Map<String, Object> map = new HashMap<>();
        pushMessage.setTransmission(JSON.toJSONString(map));
        // 设置接收人信息
        Audience audience = new Audience();
        pushDTO.setAudience(audience);
        audience.addCid(cid);   // cid
        // 进行cid单推
        ApiResult<Map<String, Map<String, String>>> apiResult = pushApi.pushToSingleByCid(pushDTO);
        Log.info("离线个推结果：" + apiResult);
    }

    public static void main(String[] args) {
        GtUtils g = new GtUtils();
        String body = "哈哈哈是的撒旦哈哈哈是的撒旦哈哈哈是的撒旦哈哈哈是的撒旦哈哈哈是的撒旦哈哈哈是的撒旦的士大夫但是";
        String title = "哈哈哈是的撒旦";
        String cid = "3b62fc0d992e29293a8bff93e075233d";
        g.pushHw(body, cid);
    }
}
