package com.rootcloud.config.unipush;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.constant.Properties;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.auth.UserInfo;
import com.rootcloud.domain.entity.JeCoreEnduserEntity;
import com.rootcloud.domain.entity.NxJeDeviceEntity;
import com.rootcloud.domain.entity.NxOrderEntity;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jline.internal.Log;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.Map;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@Log4j2
@IgnoreSecurity
@RestController
@RequestMapping("/message")
@Api(value = "message", tags = "消息")
public class MessageController {

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    UniPushAppsConfig uniPushAppsConfig;

    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;

    @Autowired
    private GtUtils gtUtils;


    @ApiOperation(value = "发送通知接口")
    @PostMapping("/send-notice")
    public JsonResult<String> sendNotice(@RequestBody UniPushVO upv) {
//        SendUniPushUtil.sendUniPush(upv.getTitle(),upv.getContent(),upv.getUserId());
        // 发送个推消息 优先从redis取推送cid，如果redis中不存在则从数据库中取
        String clientInfo = redisTemplate.opsForValue().get("UNI_PUSH_KEY_PREFIX~" + upv.userId);
        if (StringUtils.isEmpty(clientInfo)) {
            Log.info(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL) + " 发送的通知内容：", upv.title, upv.content, upv.userId);
            JeCoreEnduserEntity enduserEntity = enduserService.getOne((Wrappers.lambdaQuery(JeCoreEnduserEntity.class).select(JeCoreEnduserEntity::getCid)
                    .eq(JeCoreEnduserEntity::getUserid, upv.userId)));
            if (enduserEntity != null && !StringUtils.isEmpty(enduserEntity.getCid())) {
                clientInfo = enduserEntity.getCid();
                //重新加载到redis中
                redisTemplate.opsForValue().set("UNI_PUSH_KEY_PREFIX~" + upv.userId, clientInfo);
            } else {
                return JsonResult.failed("500", ResourceUtil.getI18nMessage("40000"));
            }
        }
        Log.info(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL) + " 发送的通知内容：", upv.title, upv.content, upv.userId, clientInfo);
        String[] infos = StringUtils.delimitedListToStringArray(clientInfo, "-");
        if (ArrayUtil.isNotEmpty(infos) && infos.length == 3) {
            // app clientId
            String clientId = infos[0];
            // 系统类型
            String systemType = infos[1];
            // 程序来源
            String appSource = infos[2];
            // 取得uni-push配置信息
            if (ObjectUtil.isNotEmpty(uniPushAppsConfig)) {
                switch (systemType) {
                    // 安卓
                    case "ANDROID":
                        //UnipushUtils.singlePush(upv.title, upv.content, clientId, uniPushAppsConfig);
                        gtUtils.pushMessage(upv.getContent(), upv.title, clientId);
                        gtUtils.pushHw(upv.getContent(), clientId);
                        break;
                    // ios
                    case "IOS":
                        UnipushUtils.singlePushForIOS(upv.title, upv.content, clientId, uniPushAppsConfig);
                        break;
                }
            } else {
                // 无配置信息
                log.error("Not config unipush for client [{}]", appSource);
            }
        }
        return JsonResult.success();
    }


    /**
     * * 存储client信息
     * * APP用户登录系统时调用
     *
     * @param userId     用户id
     * @param clientInfo 自定义client信息
     * @return Boolean
     */
    @ApiOperation(value = "存储clientInfo")
    @PostMapping("/save-client-info")
    public Boolean saveClientInfoToRedis(String userId, String clientInfo) {
        Log.info("clientId参数接收：", clientInfo, userId);
        String key = "UNI_PUSH_KEY_PREFIX~" + userId;
        redisTemplate.opsForValue().set(key, clientInfo);
        //更新用户cid数据库
        JeCoreEnduserEntity jeCoreEnduserEntity = new JeCoreEnduserEntity();
        jeCoreEnduserEntity.setUserid(userId);
        jeCoreEnduserEntity.setCid(clientInfo);
        enduserService.updateById(jeCoreEnduserEntity);
        return Boolean.TRUE;
    }


    /**
     * 清除client信息
     *
     * @param userId 用户id
     * @return Boolean
     */
    @ApiOperation(value = "清除clientInfo")
    @DeleteMapping("/remove-client-info")
    public Boolean removeClientInfoForRedis(String userId) {
        String key = "UNI_PUSH_KEY_PREFIX~" + userId;
        redisTemplate.delete(key);
        //更新用户cid数据库
        JeCoreEnduserEntity jeCoreEnduserEntity = new JeCoreEnduserEntity();
        jeCoreEnduserEntity.setUserid(userId);
        jeCoreEnduserEntity.setCid("");
        enduserService.updateById(jeCoreEnduserEntity);
        return Boolean.TRUE;
    }
}

