package com.rootcloud.config.unipush;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import com.rootcloud.common.DateUtil;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.StringUtils;

import java.util.Date;

/**
 * @author LiPingAn 2022/7/11 20:33
 * @version 1.0
 **/
@Slf4j
public class SendUniPushUtil {

    private static StringRedisTemplate redisTemplate;
    @Autowired
    private static UniPushAppsConfig uniPushAppsConfig;

    static {
        redisTemplate = (StringRedisTemplate) new RedisTemplate<String,String>();
    }

    public static void sendUniPush(String title,String content,String userId){
        /**
         * 业务处理根据自己的需求来写，
         * 先根据用户取得Redis里缓存信息，前端用户登陆app时调接口储存的自定义信息clientInfo （clientId、系统类型、程序来源，具体想存什么根据自己业务），
         * 然后获取提前配置在nacos上的配置信息，
         * 再根据不同的系统类型（android、ios）调用不同的工具类方法
         */
        // 发送个推消息
        String clientInfo = redisTemplate.opsForValue().get("UNI_PUSH_KEY_PREFIX~" + userId);
        Log.info(DateUtil.format(new Date(),DateUtil.PATTERN_CLASSICAL)+" 发送的通知内容：",title,content,userId);
        String[] infos = StringUtils.delimitedListToStringArray(clientInfo, "-");
        if (ArrayUtil.isNotEmpty(infos) && infos.length == 3) {
            // app clientId
            String clientId = infos[0];
            // 系统类型
            String systemType = infos[1];
            // 程序来源
            String appSource = infos[2];
            // 取得uni-push配置信息
            if (ObjectUtil.isNotEmpty(uniPushAppsConfig)) {
                switch (systemType) {
                    // 安卓
                    case "ANDROID":
                        UnipushUtils.singlePush(title, content, clientId, uniPushAppsConfig);
                        break;
                    // ios
                    case "IOS":
                        UnipushUtils.singlePushForIOS(title, content, clientId, uniPushAppsConfig);
                        break;
                }
            } else {
                // 无配置信息
                log.error("Not config unipush for client [{}]", appSource);
            }
        }
    }
}
