package com.rootcloud.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.rootcloud.config.auth.LoginUserInfo;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.domain.dto.EnterpriseInfoDTO;
import com.rootcloud.domain.dto.UserRoleDeptDTO;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * @author LiPingAn 2022/6/17 2:14
 * @version 1.0
 **/
@RestController
@Api(tags = "APP端租户用户相关接口")
@RequestMapping("/api")
public class AppBaseController {

    @Resource(name = "endUserService")
    JeCoreEnduserService enduserService;


    @ApiOperation(value = "查询租户信息")
    @GetMapping("/tenants/tenant-info")
    public ResponseEntity<EnterpriseInfoDTO> tenantInfo(HttpServletResponse response) {
        //Log.info("查询租户信息", "===============》",response);
        LoginUserInfo current = UserContext.current();
        if (current!=null && StringUtils.isNotBlank(current.getUserId())) {
            return ResponseEntity.ok().body(enduserService.getTenantInfo(current.getUserId()));
        }else {
            return null;
        }
    }

    @ApiOperation(value = "查询用户信息")
    @GetMapping("/user-info")
    public ResponseEntity<UserRoleDeptDTO> userInfo() {
//        Log.info("查询用户信息", "===============》");
        LoginUserInfo current = UserContext.current();
        if (current!=null && StringUtils.isNotBlank(current.getUserId())) {
            return ResponseEntity.ok().body(enduserService.getUserLoginInfo(current.getUserId()));
        }else {
            return null;
        }
    }
    @ApiOperation(value = "查询用户信息")
    @GetMapping("/v3/system")
    public ResponseEntity<Map<Object, Object>> systemInfo() {
        LoginUserInfo current = UserContext.current();
        HashMap<Object, Object> result = new HashMap<>(8);
        if (current!=null && StringUtils.isNotBlank(current.getUserId())) {
            result.put("tenantInfo",enduserService.getTenantInfo(current.getUserId()));
            result.put("userInfo",enduserService.getUserLoginInfo(current.getUserId()));
            return ResponseEntity.ok().body(result);
        }else {
            result.put(String.valueOf(HttpServletResponse.SC_FORBIDDEN),"用户未授权");
            return ResponseEntity.ok().body(result);
        }
    }
}
