package com.rootcloud.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.domain.constant.AppConstant;
import com.rootcloud.domain.service.device.DeviceAlarmService;
import com.rootcloud.vo.rest.AppDeviceAlarmParam;
import com.rootcloud.domain.service.gkjk.NxJeDeviceService;
import com.rootcloud.vo.device.DeviceStatusVO;
import com.rootcloud.vo.device.DeviceTypeVO;
import com.rootcloud.domain.dto.DeviceVO;
import com.rootcloud.vo.rest.DevicePropertyValueParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jline.internal.Log;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.ApiParam;

import java.util.Map;

import javax.annotation.Resource;

/**
 * @author:xxxxx
 * @create: 2022-05-12 10:15
 * @Description: APP端设备相关接口
 */
@RestController
@Api(tags = "APP端设备信息查询相关接口")
@RequestMapping("/app_device")
public class AppDeviceController {
    @Resource(name="deviceAlarmServiceImpl")
    private DeviceAlarmService deviceAlarmService;

    @Resource(name = "nxJeDeviceServiceImpl")
    private NxJeDeviceService nxJeDeviceService;

    @ApiOperation(value = "报警信息查询")
    @PostMapping("query-alarm-info")
    public JsonResult<Object> queryDeviceAlarmInfo(@RequestBody AppDeviceAlarmParam deviceAlarmParam) {
        //规则报警
        if("1".equals(deviceAlarmParam.getAlarmType())){
            return JsonResult.success(deviceAlarmService.getDeviceRuleAlarmCount(deviceAlarmParam));
        }
        //故障码报警
        if("2".equals(deviceAlarmParam.getAlarmType())){
            return JsonResult.success(deviceAlarmService.getDeviceCodeAlarmCount(deviceAlarmParam));
        }
        if("all".equals(deviceAlarmParam.getAlarmType())){
            return JsonResult.success(deviceAlarmService.getDeviceAllAlarmCount(deviceAlarmParam));
        }

        //return JsonResult.failed(StatusCodeEnum.ALARM_TYPE_REQUIRED.getRet(),StatusCodeEnum.ALARM_TYPE_REQUIRED.getMessage());
        return JsonResult.failed(StatusCodeEnum.ALARM_TYPE_REQUIRED.getRet(), ResourceUtil.getI18nMessage("13507"));
    }



    @ApiOperation("获取设备首页详情")
    @GetMapping("/getByDeviceId")
    public JsonResult<DeviceTypeVO> getByDeviceId(@RequestParam @ApiParam("设备ID")String deviceId) {
        try{
            return JsonResult.success(nxJeDeviceService.getByDeviceId(deviceId));
        }catch (Exception e){
            e.printStackTrace();
            return JsonResult.failed("500",e.getMessage());
        }
    }



    @ApiOperation("获取设备状态")
    @GetMapping("/getStatusByDeviceId")
    public JsonResult<DeviceStatusVO> getStatusByDeviceId(@RequestParam @ApiParam("设备ID")String deviceId) {
        try{
            return JsonResult.success(nxJeDeviceService.getStatusByDeviceId(deviceId));
        }catch (Exception e){
            return JsonResult.failed("500",e.getMessage());
        }
    }

    @ApiOperation("设备列表")
    @GetMapping("/device/page")
    public JsonResult<IPage<DeviceVO>> devicePage(
            @ApiParam("页码数，默认为1") @RequestParam(value = "page", defaultValue = AppConstant.PAGE) int page,
            @ApiParam("每页数，默认为10") @RequestParam(value = "size", defaultValue = AppConstant.SIZE) int size,
            @ApiParam("查询条件(支持地点、状态、型号、名称)") @RequestParam(value = "queryData", required = false) String queryData,
            @ApiParam("当前登录人") @RequestParam(value = "userId") String userId,
            @ApiParam("是否关心") @RequestParam(value = "isCare", required = false) String isCare) {
        //Log.info("设备列表租户id："+UserContext.getTenantId());
        return JsonResult.success(nxJeDeviceService.devicePage(queryData, isCare, page, size, userId, UserContext.getTenantId()));
    }

    @ApiOperation("查询设备指定的状态")
    @PostMapping("/queryPropertyValueByParam")
    public JsonResult<Map<String,Object>> queryPropertyValueByParam(@RequestBody DevicePropertyValueParam devicePropertyValueParam){
        return JsonResult.success(nxJeDeviceService.queryPropertyValueByParam(devicePropertyValueParam));
    }



    @ApiOperation("修改入云时间")
    @GetMapping("/saleCustomerStatistic")
    public JsonResult updateOnlineTime(@RequestParam @ApiParam("更新设备IDs")String deviceIds){
        try{
            nxJeDeviceService.updateOnlineTime(deviceIds);
            return JsonResult.success("更新成功");
        }catch (Exception e){
            e.printStackTrace();
            return JsonResult.failed("500",e.getMessage());
        }
    }
}
