package com.rootcloud.controller;

import com.github.pagehelper.PageInfo;
import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.PageRequest;
import com.rootcloud.domain.entity.MyAdviceEntity;
import com.rootcloud.domain.entity.MyMessageEntity;
import com.rootcloud.domain.entity.ds.DeviceListEntity;
import com.rootcloud.domain.service.app.MyAdviceService;
import com.rootcloud.domain.service.app.MyMessageService;
import com.rootcloud.domain.service.todo.NxTodoLibraryService;
import com.rootcloud.vo.app.AdviceResultVo;
import com.rootcloud.vo.app.MessageResultVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import jline.internal.Log;
import javax.annotation.Resource;
import java.util.Iterator;
import java.util.List;


@RestController
@Slf4j
@Api(tags = "app消息")
@RequestMapping("/message")
public class AppMessageController {
    @Resource(name = "myAdviceServiceImpl")
    private MyAdviceService myAdviceService;
    @Resource(name = "myMessageServiceImpl")
    private MyMessageService myMessageService;
    @Resource(name = "nxTodoLibraryServiceImpl")
    private NxTodoLibraryService nxTodoLibraryService;


    @ApiOperation(value = "查询app我的订单")
    @GetMapping("/myOrder")
    public JsonResult<PageInfo<MyMessageEntity>> myOrder(PageRequest pageRequest, @RequestParam @ApiParam("用户ID") String userId, @ApiParam("类型") @RequestParam(value = "type", required = false) String type){
        try{
            return  myMessageService.getOrderMessage(pageRequest,userId,type);
        }catch (Exception e){
            log.info("错误信息==>{}",e.getMessage());
            return JsonResult.failed("500",e.getMessage());
        }
    }

    @ApiOperation(value = "查询app我的通知")
    @GetMapping("/advice")
    public JsonResult<PageInfo<MyAdviceEntity>> getAdvice(PageRequest pageRequest,@RequestParam @ApiParam("用户ID") String userId, @ApiParam("类型") @RequestParam(value = "type", required = false) String type){
        try{
            return  myAdviceService.getMyAdvices(pageRequest,userId,type);
        }catch (Exception e){
            Log.info("错误信息==>{}",e.getMessage());
            return JsonResult.failed("500",e.getMessage());
        }
    }

    @ApiOperation(value = "查询app我的消息")
    @GetMapping("/message")
    public JsonResult<MessageResultVo> getMessage(@RequestParam("userId") String userId, @ApiParam("类型") @RequestParam(value = "type", required = false) String type){
       // try{
            return JsonResult.success( myMessageService.getMyMessage(userId,type));
        /*}catch (Exception e){
            Log.info("查询app我的消息错误信息==>{}",e.getMessage());
            return JsonResult.failed("500",e.getMessage());
        }*/
    }

    @ApiOperation(value = "查询故障报警消息")
    @GetMapping("/alrmMessage")
    public JsonResult<MessageResultVo> getAlrmMessage(PageRequest pageRequest,@RequestParam("userId") String userId, @ApiParam("类型") @RequestParam(value = "type", required = false) String type){
        // try{
        int pageNum = pageRequest.getPage() == null ? 1 : pageRequest.getPage();
        int pageSize = pageRequest.getSize() == null ? 10 : pageRequest.getSize();
        int startIndex  = (pageNum -1 ) * pageSize;
        MessageResultVo messageResultVo= myMessageService.getMyAlrmMessage(userId,type);
        List<MyMessageEntity> messageEntityList= messageResultVo.getMessageEntityList();
        //截取数组
        if(messageEntityList!=null&&messageEntityList.size()>startIndex+pageSize-1){
            messageResultVo.setMessageEntityList(messageEntityList.subList(startIndex,startIndex+pageSize-1));
        }

        return JsonResult.success( messageResultVo);
        /*}catch (Exception e){
            Log.info("查询app我的消息错误信息==>{}",e.getMessage());
            return JsonResult.failed("500",e.getMessage());
        }*/
    }


    @ApiOperation(value = "消息移入代办库")
    @PostMapping("/message-insert-libaray")
    public JsonResult messageInsertLibaray(@RequestBody MyMessageEntity myMessageEntity){
        try{
            nxTodoLibraryService.insertMessageLibarary(myMessageEntity);
        }catch (Exception e){
            return JsonResult.failed("500",e.getMessage());
        }
        return JsonResult.success();
    }



    @ApiOperation(value = "通知移入代办库")
    @PostMapping("/advice-insert-libaray")
    public JsonResult adviceInsertLibaray(@RequestBody MyAdviceEntity myAdviceEntity){
        try{
            nxTodoLibraryService.insertAdviceToLibarary(myAdviceEntity);
        }catch (Exception e){
            return JsonResult.failed("500",e.getMessage());
        }
        return JsonResult.success();
    }

    @ApiOperation(value = "订单通知消息已解决")
    @GetMapping("/mesSfjj")
    public JsonResult mesSfjj(@RequestParam String orderId){
        try{
            myMessageService.mesSfjj(orderId);
        }catch (Exception e){
            return JsonResult.failed("500",e.getMessage());
        }
        return JsonResult.success();
    }
    @ApiOperation(value = "根据主键删除待办库")
    @GetMapping("/delTodoLibrary")
    public JsonResult delTodoLibrary(@RequestParam String nxTodoLibraryId){
        try{
            nxTodoLibraryService.delTodoLibrary(nxTodoLibraryId);
        }catch (Exception e){
            return JsonResult.failed("500",e.getMessage());
        }
        return JsonResult.success();
    }

    @ApiOperation(value = "设备通知消息已解决")
    @GetMapping("/eqMesSfjj")
    public JsonResult eqMesSfjj(@RequestParam String id,String type){
        try{
            myAdviceService.eqMesSfjj(id,type);
        }catch (Exception e){
            return JsonResult.failed("500",e.getMessage());
        }
        return JsonResult.success();
    }

    @ApiOperation(value = "服务通知消息已解决")
    @GetMapping("/serMesSfjj")
    public JsonResult serMesSfjj(@RequestParam String id,String type){
        try{
            myMessageService.serMesSfjj(id,type);
        }catch (Exception e){
            return JsonResult.failed("500",e.getMessage());
        }
        return JsonResult.success();
    }
}
