package com.rootcloud.controller;

import javax.annotation.Resource;

import jline.internal.Log;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.rootcloud.common.JsonResult;
import com.rootcloud.domain.ao.OrderDispatchForm;
import com.rootcloud.domain.constant.AppConstant;
import com.rootcloud.domain.dto.OrderVO;
import com.rootcloud.domain.service.order.OrderService;
import com.rootcloud.vo.order.OrderSignInParam;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * APP端订单相关接口
 * </P>
 *
 * @author HeJia
 * @since 2022/5/17
 */
@Api(tags = "APP端订单相关接口")
@RestController
@Slf4j
@RequestMapping("/app/order")
public class AppOrderController {

    @Resource(name = "orderServiceImpl")
    private OrderService orderService;

    @ApiOperation("订单列表")
    @GetMapping("/page")
    public JsonResult<IPage<OrderVO>> orderPage(
            @ApiParam("页码数，默认为1") @RequestParam(value = "page", defaultValue = AppConstant.PAGE) int page,
            @ApiParam("每页数，默认为10") @RequestParam(value = "size", defaultValue = AppConstant.SIZE) int size,
            @ApiParam("服务类型") @RequestParam(value = "serviceType", required = false) String serviceType,
            @ApiParam("订单状态") @RequestParam(value = "orderStatus", required = false) String orderStatus) {
        return JsonResult.success(orderService.orderPage(page, size, serviceType, orderStatus));
    }

    @ApiOperation("订单详情")
    @GetMapping("/{orderId}")
    public JsonResult<OrderVO> orderInfo(@ApiParam("订单主键") @PathVariable String orderId) {
        return JsonResult.success(orderService.orderInfo(orderId));
    }

    @ApiOperation("派工")
    @PutMapping("/dispatch")
    public JsonResult<Boolean> dispatch(@RequestBody @Validated OrderDispatchForm form) {
        return JsonResult.success(orderService.dispatch(form));
    }

    @ApiOperation("作废")
    @PatchMapping("/invalid/{orderId}")
    public JsonResult<Boolean> invalid(@ApiParam("订单主键") @PathVariable String orderId) {
        return JsonResult.success(orderService.invalid(orderId));
    }


    @ApiOperation("订单签到")
    @PostMapping("/orderSignIn")
    public JsonResult orderSignIn(@RequestBody @Validated OrderSignInParam orderSignInParam) {
        try {
            orderService.orderSignIn(orderSignInParam);
        } catch (Exception e) {
            Log.info("错误信息：{}", e.getMessage());
            e.printStackTrace();
            return JsonResult.failed("500", e.getMessage());
        }
        return JsonResult.success("订单签到");
    }

    @ApiOperation("订单处理步骤")
    @GetMapping("/orderProcess")
    public JsonResult<List> orderProcess(@RequestParam String orderId) {
        return JsonResult.success(orderService.orderProcess(orderId));
    }

    @ApiOperation("订单移入代办库")
    @GetMapping("/order-insert-library")
    public JsonResult<List> orderInsertLibrary(@RequestParam String orderId) {
        try {
            orderService.orderInsertLibrary(orderId);
        } catch (Exception e) {
            return JsonResult.failed("500", e.getMessage());
        }
        return JsonResult.success();
    }

    @ApiOperation("获取订单数量汇总")
    @GetMapping("/getOrderTypeNum")
    public JsonResult<HashMap> getOrderTypeNum() {
        return JsonResult.success(orderService.getOrderTypeNum());
    }

    @ApiOperation("获取订单数量汇总2")
    @GetMapping("/getOrderTypeNum2")
    public JsonResult<HashMap> getOrderTypeNum2() {
        return JsonResult.success(orderService.getOrderTypeNum2());
    }

    @Scheduled(cron = "0 0/2 8-20 * * ?")
    public JsonResult cron() {
        return JsonResult.success(orderService.getOrderTypeNum());
    }
}
