package com.rootcloud.controller;

import com.alibaba.fastjson.JSONObject;
import com.rootcloud.common.JsonResult2;
import com.rootcloud.common.JsonResultStatus;
import com.rootcloud.domain.dto.CarouselDTO;
import com.rootcloud.domain.vo.CarouselVO;
import com.rootcloud.service.impl.CarouselServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.Asserts;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author w-yanxiang
 * @Date 2022/6/16 10:20
 * @Version 1.0
 * @Description
 */
@RestController
@Slf4j
@Api(tags = "轮播图")
@RequestMapping("/carousel")
public class CarouselController {

    @Resource
    private CarouselServiceImpl carouselService;

    @PostMapping("/save/submit")
    @ApiOperation("保存轮播数据")
    public JsonResult2 saveOrSubmitCarousel(@RequestBody CarouselDTO carouselDTO){
        log.info("saveOrSubmitCarousel param [{}]", JSONObject.toJSONString(carouselDTO));
        try {
            Asserts.notBlank(carouselDTO.getOperationType(),"operationType");
            carouselService.saveOrsubmitCarousel(carouselDTO);
            return new JsonResult2();
        } catch (Exception e) {
           log.error("saveOrSubmitCarousel error",e);
           return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(),e.getMessage());
        }
    }


    @GetMapping("/find/carousel")
    @ApiOperation("查询轮播数据")
    public JsonResult2 findCarouselInfoData(@RequestParam String source){
        log.info("findCarouselInfoData param [{}]",source);
        try {
            CarouselVO carouselInfoData = carouselService.findCarouselInfoData(source);
            return new JsonResult2(carouselInfoData);
        } catch (Exception e) {
            log.error("findCarouselInfoData error",e);
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(),e.getMessage());
        }
    }

    @PostMapping("/saveUpdate/carousel")
    @ApiOperation("保存或修改一个轮播图数据")
    public JsonResult2 saveOrUpdateOneCarousel(@RequestParam(required = false) String id,@RequestParam String imageUrl){
        log.info("saveOrUpdateOneCarousel param [{}]----[{}]",id,imageUrl);
        try {
            carouselService.saveOrUpdateOneCarousel(id,imageUrl);
            return new JsonResult2();
        } catch (Exception e) {
            log.error("saveOrUpdateOneCarousel error",e);
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(),e.getMessage());
        }
    }

    @DeleteMapping("/delete/carousel")
    @ApiOperation("删除轮播数据")
    public JsonResult2 deleteCarousel(@RequestParam List<String> ids){
        log.info("deleteCarousel param [{}]",ids);
        try {
            carouselService.deleteCarousel(ids);
            return new JsonResult2();
        } catch (Exception e) {
            log.error("deleteCarousel error",e);
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(),e.getMessage());
        }
    }
}
