package com.rootcloud.controller;

import com.github.pagehelper.PageInfo;
import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.PageRequest;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.domain.dto.DeviceDTO;
import com.rootcloud.domain.dto.DeviceSpecialDTO;
import com.rootcloud.domain.dto.NxDeviceDTO;
import com.rootcloud.domain.dto.NxDeviceDTO;
import com.rootcloud.domain.entity.device.NxDeviceCameraEntity;
import com.rootcloud.domain.model.WorkStatusMonitorModel;
import com.rootcloud.domain.service.device.DeviceAlarmService;
import com.rootcloud.domain.service.device.DeviceService;
import com.rootcloud.domain.service.device.NxDeviceService;
import com.rootcloud.vo.device.*;
import com.rootcloud.vo.rest.DeviceAlarmParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 设备管理
 *
 * @author wei.liao
 * @version 1.0
 * @date 2022/4/18 14:02
 */
@RestController
@Api(tags = "设备管理接口")
@RequestMapping("/device")
public class DeviceController {

    @Resource(name = "deviceServiceImpl")
    private DeviceService deviceService;
    @Resource(name="deviceAlarmServiceImpl")
    private DeviceAlarmService deviceAlarmService;
    @Resource(name = "nxDeviceServiceImpl")
    private NxDeviceService nxDeviceService;

    @ApiOperation("设备总览")
    @GetMapping("/display")
    public JsonResult<DeviceView> display() {
        return deviceService.display();
    }

    @ApiOperation("设备信息-工作折现图")
    @GetMapping("/details")
    public JsonResult<DeviceVO> display(@RequestParam("deviceId") String deviceId) {
        return deviceService.getDeviceInfoWithMonthWorkTimeLineChart(deviceId);

    }

    @ApiOperation("设备月份工时")
    @GetMapping("/work-time/month")
    public JsonResult<DeviceMonthWorkTimeStatistics> monthWorkTimeStatistics(@RequestParam("deviceId") String deviceId, @RequestParam("searchDate") String searchDate) throws ParseException {
        Date date = DateUtils.parseDate(searchDate + " 00:00:00", "YYYY-MM-dd HH:mm:ss");
        return deviceService.getMonthWorkTimeStatistics(deviceId, date);
    }

    @ApiOperation(value = "报警信息查询")
    @PostMapping("query-alarm-info")
    public JsonResult<Object> queryDeviceAlarmInfo(@RequestBody DeviceAlarmParam deviceAlarmParam) {
        //规则报警
        if("1".equals(deviceAlarmParam.getAlarmRule())){
            return JsonResult.success(deviceAlarmService.getDeviceRuleAlarmInfo(deviceAlarmParam));
        }
        //故障码报警
        if("2".equals(deviceAlarmParam.getAlarmRule())){
            return JsonResult.success(deviceAlarmService.getDeviceCodeAlarmInfo(deviceAlarmParam));
        }
        //return JsonResult.failed(StatusCodeEnum.ALARM_TYPE_REQUIRED.getRet(),StatusCodeEnum.ALARM_TYPE_REQUIRED.getMessage());
        return JsonResult.failed(StatusCodeEnum.ALARM_TYPE_REQUIRED.getRet(), ResourceUtil.getI18nMessage("13507"));
    }



    @ApiOperation("服务计划查询所有设备编码")
    @GetMapping("/selectAllDevice")
    public JsonResult<List<DeviceSpecialDTO>>  selectAllDevice(){
        return JsonResult.success(deviceService.selectAllDevice());
    }

    @GetMapping("/camera/list")
    public JsonResult<PageInfo<NxDeviceDTO>> cameraList(PageRequest pageRequest, NxDeviceParam nxDeviceParam) {
        return nxDeviceService.findList(pageRequest, nxDeviceParam);

    }

    @PostMapping("/camera/bind")
    public JsonResult<Void> cameraBind(@RequestBody @Valid CameraParam cameraParam) {
        return nxDeviceService.bindCamera(cameraParam);
    }


    @PostMapping("/camera/unbound")
    public JsonResult<Void> cameraUnbound(@RequestParam("cameraId") String cameraId) {
        return nxDeviceService.UnboundCamera(cameraId);
    }

    @ApiOperation(value = "查询设备对应摄像头数据")
    @GetMapping("cameras-by-deviceId")
    public JsonResult<Object> camerasByDeviceId(@RequestParam String deviceId) {
        List<NxDeviceCameraEntity> entities = nxDeviceService.camerasByDeviceId(deviceId);
        return JsonResult.success(entities);
    }



    @ApiOperation(value = "通过设备编码返回中文地址")
    @GetMapping("/returnAddress")
    public JsonResult<String> returnAddress(@RequestParam("sblbSbbm") String sblbSbbm){
        String address = null;
        address = deviceService.returnAddress(sblbSbbm);
        return JsonResult.success(address);
    }

}
