package com.rootcloud.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.alibaba.fastjson.JSONObject;
import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.ao.ExportHistorySituationAO;
import com.rootcloud.domain.ao.HistorySituationAO;
import com.rootcloud.domain.dto.DeviceDTO;
import com.rootcloud.domain.model.WorkStatusMonitorModel;
import com.rootcloud.domain.service.device.DeviceService;
import com.rootcloud.domain.service.gkjk.NxJeDeviceService;
import com.rootcloud.vo.device.DeviceHistorySituationVO;
import com.rootcloud.vo.resp.OtherDeviceResp;
import com.rootcloud.vo.rest.DeviceTrendParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author w-liuyi01
 * @Description //历史工况控制器
 * @Date 2022/4/20 14:43
 * @Param 
 * @return 
 **/
@RestController
@Api(tags = "历史工况控制器")
@RequestMapping("/history_situation")
public class DeviceHistorySituationController {


    @Resource(name = "deviceServiceImpl")
    private DeviceService deviceService;

    @Resource(name = "nxJeDeviceServiceImpl")
    private NxJeDeviceService nxJeDeviceService;

    @ApiOperation(value = "查询设备详情历史工况数据")
    @PostMapping("/queryHistorySituation")
    public JsonResult<DeviceHistorySituationVO> queryHistorySituation(@RequestBody @Valid HistorySituationAO historySituationAO) {
        return nxJeDeviceService.queryHistorySituation(historySituationAO);
    }


    @ApiOperation(value = "导出数据")
    @PostMapping("/export")
    public void export(HttpServletResponse response,@RequestBody @Valid ExportHistorySituationAO historySituationAO) {



        JsonResult<DeviceHistorySituationVO> deviceHistorySituationVOJsonResult = nxJeDeviceService.exportQueryHistorySituation(historySituationAO);
        List<DeviceHistorySituationVO> data = (List<DeviceHistorySituationVO>) deviceHistorySituationVOJsonResult.getData();
        if (ObjectUtils.isEmpty(data)){
            //throw new BusinessException(StatusCodeEnum.HISTORY_DEVICE_NOT_EXIST.getRet(),StatusCodeEnum.HISTORY_DEVICE_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.HISTORY_DEVICE_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("14001"));
        }

//        List<DeviceHistorySituationVO> data1 = new ArrayList<>();
//        DeviceHistorySituationVO vo = new DeviceHistorySituationVO();
//        vo.setDeviceSbbm("ISO419821");
//        vo.setDeviceSbmc("测试设备");
//        vo.setDeviceSblx("高级测试设备");
//        vo.setDeviceSbxh("213213212");
//        vo.setUpdateTime("2020-04-21 12:22:06");
//        data1.add(vo);

//        ExcelWriter writer = ExcelUtil.getWriter();
        ExcelWriter writer = ExcelUtil.getWriter(true);
        List<Map<String, Object>> rows = data.stream().map(item -> {
            Map<String, Object> maps = new HashMap<>();
            maps.put("deviceSbbm", item.getDeviceSbbm());
            maps.put("deviceSbmc", item.getDeviceSbmc());
            maps.put("deviceSblx", item.getDeviceSblxName());
            maps.put("deviceSbxh", item.getDeviceSbxhName());
            maps.put("updateTime", item.getUpdateTime());
            if (historySituationAO.getPropertyList()!=null && historySituationAO.getPropertyList().size()>0){
                for (WorkStatusMonitorModel wd:historySituationAO.getPropertyList()) {
                    maps.put(wd.getPropertyId(), item.getMap().get(wd.getPropertyId()));
                }
            }
            return maps;
        }).collect(Collectors.toList());


        // Title
        int columns = 5;
        try {
//            columns = Class.forName("com.rootcloud.vo.device.DeviceHistorySituationVO").getDeclaredFields().length;
            if (historySituationAO.getPropertyList()!=null && historySituationAO.getPropertyList().size()>0){
                columns = columns + historySituationAO.getPropertyList().size();
            }
            writer.merge(columns-1, "工况信息");
        } catch (Exception e) {
            e.printStackTrace();
        }
        // Header
        //自定义标题别名
        writer.addHeaderAlias("deviceSbbm", "设备编号");
        writer.addHeaderAlias("deviceSbmc", "设备名称");
        writer.addHeaderAlias("deviceSblx", "设备类型");
        writer.addHeaderAlias("deviceSbxh", "设备型号");
        writer.addHeaderAlias("updateTime", "更新时间");
        if (historySituationAO.getPropertyList()!=null && historySituationAO.getPropertyList().size()>0){
            for (WorkStatusMonitorModel wd:historySituationAO.getPropertyList()) {
                writer.addHeaderAlias(wd.getPropertyId(), wd.getPropertyName());
            }
        }

        // Body
        for (int i = 0; i < columns; i++) {
            writer.setColumnWidth(i, 30);
        }
        writer.write(rows, true);
        ServletOutputStream out = null;
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        try {
            response.setHeader("Content-disposition", "attachment; filename=" + URLEncoder.encode("工况信息表-" + DateUtil.today() + ".xls", "utf-8"));
            out = response.getOutputStream();
        } catch (Exception e) {
            e.printStackTrace();
        }
        writer.flush(out, true);
        writer.close();
        IoUtil.close(out);
    }


    @ApiOperation(value = "查询工况趋势除设备1之外的其他设备编号")
    @GetMapping("query-other-devices")
    public JsonResult<Object> queryOtherDevices(@RequestParam String deviceId){
        List<DeviceDTO> deviceDTOS = deviceService.deviceDTOListByRole();
        List<OtherDeviceResp> list = new ArrayList<>();
        deviceDTOS.stream().forEach(temp -> {
            OtherDeviceResp otherDeviceResp = new OtherDeviceResp();
            otherDeviceResp.setDeviceId(temp.getId());
            otherDeviceResp.setDeviceSbbm(temp.getCode());
            otherDeviceResp.setDeviceName(temp.getName());
            list.add(otherDeviceResp);
        });
        //List<OtherDeviceResp> list = nxJeDeviceService.queryOtherDevices(deviceId);
        return JsonResult.success(list);
    }

    @ApiOperation(value = "查询工况趋势数据")
    @PostMapping("query-work-trend")
    public JsonResult<Object> queryWorkTrendData(@RequestBody @Valid DeviceTrendParam deviceTrendParam){
        JSONObject  json = nxJeDeviceService.queryWorkTrendData(deviceTrendParam);
        return JsonResult.success(json);
    }
}
