package com.rootcloud.controller;

import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.domain.mapper.CommonMapper;
import com.rootcloud.domain.service.ds.ModelPropertySyncService;
import com.rootcloud.domain.service.fittings.SparePartService;
import com.rootcloud.domain.service.gbsj.GbsjbService;
import com.rootcloud.domain.service.part.NxPartFileService;
import groovy.util.logging.Slf4j;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;


/**
 * @author LiPingAn 2022/4/12 13:55
 * @version 1.0
 **/
@Api(tags = "JP定时任务调用接口")
@RestController
@Slf4j
@RequestMapping("/job")
@EnableScheduling
@IgnoreSecurity
public class JobController {

    private final static Logger log = LoggerFactory.getLogger(JobController.class);



    @Resource(name = "gbsjService")
    private GbsjbService gbsjbService;
    @Resource(name = "nxpartFileService")
    private NxPartFileService nxPartFileService;
    @Resource(name = "modelPropertySyncService2")
    private ModelPropertySyncService modelPropertySyncService;
    @Resource(name = "sparePartServiceImpl")
    private SparePartService sparePartService;

    @Autowired
    private CommonMapper commonMapper;


    @GetMapping("/gbsjSendMessage")
    @ApiOperation(value = "过保商机定时推送相关站内消息")
    @Scheduled(cron = "0 0 0 * * ?")
    public void gbsjSendMessage() {
        gbsjbService.sendGbsjTipMsg();
    }

    @GetMapping("/syncData")
    @ApiOperation(value = "同步数据")
    //@Scheduled(cron ="0/20 * * * * ? ")
    @Scheduled(cron ="0 */30 * * * ?")
    public void syncThingModelData() {
        // modelSyncService.syncModelData();
        //同步所有租户数据
        List<Map<String, String>> rootClientList = commonMapper.getRootClientList();
        for (Map<String,String> map :
                rootClientList) {
            modelPropertySyncService.syncModelPropertyData(map.get("CLIENT_ID"),map.get("CLIENT_SECRET"));
        }
    }

    @GetMapping("/syncPartData")
    @ApiOperation(value = "同步配件信息接口")
    @Scheduled(cron = "0 0 2 * * ?")
    public void syncPartData() {
        nxPartFileService.syncPartData();  //(运维中心、泉州中心租户)已启用ERP同步逻辑逻辑，此逻辑作废2024-02-22
    }

    @GetMapping("/syncPartWareData")
    @ApiOperation(value = "同步配件仓库信息接口")
    @Scheduled(cron = "0 0 3 * * ?")
  //  @Scheduled(cron = "30 * * * * ?")  //测试30秒执行一次
    public void syncPartWareData() {
        nxPartFileService.syncPartWareData(); //(运维中心、泉州中心租户)已启用ERP同步逻辑逻辑，此逻辑作废2024-02-22
//        List<String> lgortCodes = nxPartFileService.queryLgortCode();
//        if (lgortCodes!=null){
//            lgortCodes.forEach(item -> {
//                String[] split = item.split("~");
//                NxPartRespEntity entity = new NxPartRespEntity();
//                entity.setNxPartRespId(JEUuidUtil.uuid());
//                entity.setRespCkbm(split[0]);
//                entity.setRespCkmc(split[0]);
//                entity.setRespKwbm(split[1]);
//                entity.setSyCreatetime(DateUtil.getCurrentTime());
//                entity.setSyCreateuser("system");
//                nxPartRespService.save(entity);
//            });
//        }
    }

    @GetMapping("/partLifeWarning")
    @ApiOperation(value = "备件寿命预警")
    public void partLifeWarning(){
        sparePartService.uptPartSmyj();
    }

    @GetMapping("/userUnlock")
    @ApiOperation(value = "解除账号锁定")
//    @Scheduled(cron = "0 0,30 * * * ? ")
    public void userUnlock(){
        gbsjbService.userUnlock();
    }
}
