package com.rootcloud.controller;

import com.rootcloud.common.JsonResult;
import com.rootcloud.common.constant.JsonResultStatusBpm;
import com.rootcloud.domain.model.LappLiveCondition;
import com.rootcloud.domain.service.device.LappLiveService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

@RestController
@Api(value = "LappLiveController", description = "直播功能接口", protocols = "HTTP/1.1")
@RequestMapping("/lapp-live")
public class LappLiveController {

    private final Logger log = LoggerFactory.getLogger(this.getClass());

    @Resource(name = "appLiveSetvice")
    private LappLiveService lappLiveService;

    @RequestMapping(value = "get-live", method = RequestMethod.POST)
    @ApiOperation(value = "根据设备序列号和通道号获取", httpMethod = "POST", response = JsonResult.class, notes = "根据设备序列号和通道号获取")
    public JsonResult getLiveAdress(HttpServletRequest request, @RequestBody LappLiveCondition condition) {

        String deviceSerial = condition.getDeviceSerial();
        String channelNo = condition.getChannelNo();

        if(StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(channelNo)) {
            log.error("设备序列号和通道号不能为空");
            return new JsonResult(JsonResultStatusBpm.RET_EXCEPTION,"设备序列号和通道号不能为空");
        }
        // 获取直播地址(只获取普通的直播地址)
        try {
            //JsonResult jsonResult = lappLiveService.getLiveAddress(deviceSerial, channelNo);
            //return jsonResult;
            JsonResult jsonResult =null;
            if(deviceSerial.length() == 9){  //萤石云
                jsonResult = lappLiveService.getLiveAddress(deviceSerial, channelNo);
            }
            if(deviceSerial.length() == 15){ //乐橙云
                jsonResult = lappLiveService.getLeChengLiveAddress(deviceSerial, channelNo);
            }
            return jsonResult;
        } catch (Exception e) {
            log.error("获取视频直播地址出现异常:{}",e);
            return new JsonResult(JsonResultStatusBpm.RET_EXCEPTION,"解析直播地址有异常,请检查直播状态和设备状态或重新进行该操作!");
        }
    }

}
