package com.rootcloud.controller;

import cn.hutool.core.codec.Base64;
import com.rootcloud.common.*;
import com.rootcloud.domain.service.auth.RootCloudAuthService;
import com.rootcloud.vo.UserLogin;
import com.rootcloud.vo.UserLogout;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jline.internal.Log;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.Map;

@RestController
@Api(tags = "登录登出")
@RequestMapping("/auth")
@IgnoreSecurity
public class LoginController {
    @Value("${application.source}")
    private String source;

    private static RootCloudAuthService rootCloudAuthService;

    private void init() {
        if ("CLOUD".equals(source)) {
            rootCloudAuthService = SprintBeanUtils.getBean("cloudAuthService");
        }
    }

    @PostMapping("/login")
    @ApiOperation(value = "登录", notes = "登录")
    public OAuthResult login(HttpServletResponse response, @Valid @RequestBody UserLogin userLogin) {
        init();
        OAuthResult oAuthResult = rootCloudAuthService.login(DESUtil.decrypt(userLogin.getUsername()), DESUtil.decrypt(userLogin.getPassword()),userLogin.getCode());
        if (oAuthResult == null){
            response.setHeader("status_code","550");
            response.setHeader("status_msg",ResourceUtil.getI18nMessage("60015"));
        }else if ("WARNLOCKED".equals(oAuthResult.getCode())){
            response.setHeader("status_code","550");
            response.setHeader("status_msg", Base64.encode(oAuthResult.getMessage().getBytes()));
        }
        return oAuthResult;
    }

    @PostMapping("/logout")
    @ApiOperation(value = "登出", notes = "登出")
    public JsonResult<String> logout(@Valid @RequestBody UserLogout userLogout) {
        init();
        String res = rootCloudAuthService.logout(userLogout.getAccess_token());
        return JsonResult.success(res);
    }


}
