package com.rootcloud.controller;

import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.PageRequest;
import com.rootcloud.domain.dto.PendingOrderModel;
import com.rootcloud.domain.dto.ServiceViewResultModel;
import com.rootcloud.domain.entity.FwryDTO;
import com.rootcloud.domain.entity.JeCoreEnduserEntity;
import com.rootcloud.domain.entity.NxOrderEntity;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.entity.device.NxDeviceLocationEntity;
import com.rootcloud.domain.service.network.NetworkUserService;
import com.rootcloud.vo.network.NetWorkUserWeekVO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 网点服务人员
 *
 * @author wei.liao
 * @version 1.0
 * @date 2022/4/27 10:27
 */
@RestController
@RequestMapping("/network/user")
public class NetworkUserController {

    @Autowired
    NetworkUserService networkUserService;

    @GetMapping("/list")
    public JsonResult<List<JeCoreEnduserEntity>> userList(@RequestParam("roleId") String roleId) {
        return JsonResult.success(networkUserService.getList(roleId));
    }

    @GetMapping("/week/list")
    public JsonResult<Page<NetWorkUserWeekVO>> weekUserList(PageRequest pageRequest, @RequestParam("roleId") String roleId) {
        return JsonResult.success(networkUserService.getWeekList(pageRequest, roleId));
    }

    @GetMapping("/find-bind")
    public JsonResult<List<JeCoreEnduserEntity>> findBind() {
        return JsonResult.success(networkUserService.findBindUser());
    }

    @GetMapping("/engineerList")
    public JsonResult<List<ServiceViewResultModel>> engineerList() {
        return JsonResult.success(networkUserService.engineerList());
    }

    @GetMapping("/pendingOrder")
    public List<PendingOrderModel> pendingOrder() {
        return networkUserService.pendingOrder();
    }

    /**
     * 设备待处理订单数量
     *
     * @param orderDeviceCode 设备编码
     * @return
     */
    @GetMapping("/equipmentOrderNum")
    public int equipmentOrderNum(@RequestParam("orderDeviceCode") String orderDeviceCode) {
        return networkUserService.equipmentOrderNum(orderDeviceCode);
    }

    /**
     * 查询网点服务人员
     *
     * @param reqDto
     * @return
     */
    @PostMapping("/getNxgxJcdaFwryList")
    public JsonResult<List<FwryDTO>> getNxgxJcdaFwryList(@RequestBody FwryDTO reqDto) {
        return JsonResult.success(networkUserService.getNxgxJcdaFwryList(reqDto));
    }

    /**
     * 删除网点服务人员
     *
     * @param nxgxJcdaFwryId 服务人员主键id
     * @return
     */

    @DeleteMapping("/delFwry")
    public JsonResult<Integer> delFwry(@RequestParam("nxgxJcdaFwryId") String nxgxJcdaFwryId) {
        return JsonResult.success(networkUserService.delFwry(nxgxJcdaFwryId));
    }

    /**
     * 根据该员工名下订单状态跟新员工状态
     *
     * @param reqDto
     * @return
     */
    @PostMapping("/updateNxgxJcdaFwry")
    public JsonResult<FwryDTO> updateNxgxJcdaFwry(@RequestBody FwryDTO reqDto) {
        return JsonResult.success(networkUserService.updateNxgxJcdaFwry(reqDto));
    }

    /**
     * 查询订单总数量
     *
     * @param startDate 开始时间
     * @param endDate   结束时间
     * @return
     */
    @GetMapping("/getOrderNum")
    public JsonResult<Integer> getOrderNum(@RequestParam String startDate, String endDate) {
        //return JsonResult.success(networkUserService.getOrderNum(startDate, endDate));
        return JsonResult.success(networkUserService.getOrderNum(startDate, endDate)*100+ RandomUtil.randomInt(1,99));
        //<!-- 手动创建订单 订单号重复问题处理 2024-11-04 commit-->
    }

    @ApiOperation("查询用户下设备列表信息")
    @GetMapping("/getSblbByUserId")
    public JsonResult<Object> getSblbByUserId(@RequestParam String userId,
                                                            @RequestParam(value = "sblbSbbm", required = false) String sblbSbbm,
                                                            @RequestParam(value = "nxJcxxSblbId", required = false) String nxJcxxSblbId,
                                                            @RequestParam(value = "pageSize") Integer pageSize,
                                                            @RequestParam(value = "pageNum") Integer pageNum) {
        return networkUserService.getSblbByUserId(userId, sblbSbbm, nxJcxxSblbId,pageSize,pageNum);
    }

    /**
     * 服务视图信息
     *
     * @param isService 0 待服务订单， 1 服务中订单，2，待处理服务订单
     * @return
     */
    @ApiOperation("服务视图信息")
    @GetMapping("/getServiceView")
    public JsonResult<List<PendingOrderModel>> getServiceView(@RequestParam String isService) {
        return networkUserService.getServiceView(isService);
    }

    /**
     * 服务视图信息各类型数量
     */
    @ApiOperation("服务视图信息各类型数量")
    @GetMapping("/getServiceViewCount")
    public JsonResult<HashMap<String, Integer>> getServiceViewCount() {
        return networkUserService.getServiceViewCount();
    }

    /**
     * 通过设备编码查询订单
     *
     * @param isService 0 待服务订单， 1 服务中订单，2，待处理服务订单
     * @return
     */
    @ApiOperation("服务视图信息")
    @GetMapping("/getServiceViewBySbbm")
    public JsonResult<List<NxOrderEntity>> getServiceViewBySbbm(@RequestParam String sbbm, String isService) {
        return networkUserService.getServiceViewBySbbm(sbbm, isService);
    }

    /**
     * 通过设备编码查询设备经纬度
     *
     * @param deviceId 设备id
     * @return
     */
    @ApiOperation("通过设备编码查询设备经纬度")
    @GetMapping("/getSbjwd")
    public JsonResult<List<NxDeviceLocationEntity>> getSbjwd(@RequestParam String deviceId) {
        return networkUserService.getSbjwd(deviceId);
    }
}
