package com.rootcloud.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.JsonResult;
import com.rootcloud.domain.constant.AppConstant;
import com.rootcloud.domain.service.todo.NxTodoLibraryService;
import com.rootcloud.vo.todo.TodoVo;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * <p>
 * 宁夏_待办库 前端控制器
 * </p>
 *
 * @author yangyijun
 * @since 2022-05-27
 */
@RestController
@Slf4j
@RequestMapping("/nx-todo-library")
public class NxTodoLibraryController {

    @Resource(name = "nxTodoLibraryServiceImpl")
    private NxTodoLibraryService nxTodoLibraryService;

    @ApiOperation("分页查询待办库")
    @GetMapping("/page")
    public JsonResult<IPage<TodoVo>> selectPageList(
            @ApiParam("页码数，默认为1") @RequestParam(value = "page", defaultValue = AppConstant.PAGE) int page,
            @ApiParam("每页数，默认为10") @RequestParam(value = "size", defaultValue = AppConstant.SIZE) int size,
            @ApiParam("查询条件(设备名称等)") @RequestParam(value = "queryData", required = false) String queryData,
            @ApiParam("用户ID") @RequestParam(value = "userId", required = false) String userId){
        return JsonResult.success(nxTodoLibraryService.getPage(page,size,queryData,userId));
    }


    @ApiOperation("待办库左滑操作")
    @GetMapping("/slideLeft")
    public JsonResult slideLeft(@ApiParam("待办库主键") @RequestParam(value = "nxTodoLibraryId") String nxTodoLibraryId){
        try{
            nxTodoLibraryService.updateStatus(nxTodoLibraryId);
            return JsonResult.success();
        }catch (Exception e){
            log.info("代办库状态修改错误信息==>{}",e.getMessage());
            return JsonResult.failed("500",e.getMessage());
        }
    }



    @ApiOperation("修改待办库操作")
    @GetMapping("/updateLibrary")
    public JsonResult updateLibrary(@ApiParam("待办库主键") @RequestParam(value = "nxTodoLibraryId") String nxTodoLibraryId,
                                    @ApiParam("订单主键") @RequestParam(value = "nxOrderId") String nxOrderId){
        try{
            nxTodoLibraryService.updateLibrary(nxTodoLibraryId,nxOrderId);
            return JsonResult.success();
        }catch (Exception e){
            log.info("修改待办库操作错误信息==>{}",e.getMessage());
            return JsonResult.failed("500",e.getMessage());
        }
    }

}

