package com.rootcloud.controller;


import com.rootcloud.common.JsonResult;
import com.rootcloud.common.enums.ReportStatisticsEnums;
import com.rootcloud.domain.ao.ReportStatisticAO;
import com.rootcloud.domain.service.order.OrderService;
import com.rootcloud.domain.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

@RestController
@Api(tags = "报表统计")
@RequestMapping("/report/statistics")
public class ReportStatisticController {


    @Resource(name = "orderServiceImpl")
    private OrderService orderService;

    private final static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");


    @ApiOperation("订单统计报表")
    @PostMapping("/orderStatusStatistic")
    public JsonResult<OrderStatusStatistic> selectOrderStatusStatistic(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        OrderStatusStatistic orderStatusStatistic = orderService.orderStatusStatistic(reportStatisticAO);
        return JsonResult.success(orderStatusStatistic);
    }


    @ApiOperation("设备状态统计报表")
    @PostMapping("/deviceStatusStatistic")
    public JsonResult<DeviceStatusStatistic>  selectDeviceStatusStatisti(@RequestBody ReportStatisticAO reportStatisticAO){
        return JsonResult.success(orderService.deviceStatusStatistic());
    }



    @ApiOperation("订单/设备状态统计报表")
    @PostMapping("/selectOrderAndDevice")
    public JsonResult<StatusVO>  selectOrderAndDevice(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.orderAndDeviceStatus(reportStatisticAO));
    }


    @ApiOperation("区域信息统计报表")
    @PostMapping("/regionStatistic")
    public JsonResult<List<Outlets>> regionStatistic(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.regionStatistic(reportStatisticAO));
    }


    @ApiOperation("订单按照日期统计报表")
    @PostMapping("/orderStatisticByDay")
    public JsonResult<List<OrderStatistic>> orderStatisticByDay(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.orderStatisticByDay(reportStatisticAO));
    }


    @ApiOperation("人员统计相关所有查询")
    @PostMapping("/checkAllPersonStatistic")
    public JsonResult<PersonAllStatistic>  checkAllPersonStatistic(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.checkPersonAllStatistic(reportStatisticAO));
    }


    @ApiOperation("人员订单统计报表")
    @PostMapping("/personOrderNum")
    public JsonResult<List<PersonnelStatistic>> personOrderNum(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.checkPersonStatistic(reportStatisticAO,"orderNum"));
    }


    @ApiOperation("人员服务评价统计报表")
    @PostMapping("/personServiceAppraise")
    public JsonResult<List<PersonnelStatistic>> personServiceAppraise(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.checkPersonStatistic(reportStatisticAO,"serviceNum"));
    }


    @ApiOperation("人员工作时长统计报表")
    @PostMapping("/personWorkTime")
    public JsonResult<List<PersonnelStatistic>> personWorkTime(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.checkPersonStatistic(reportStatisticAO,"workTimeNum"));
    }


    @ApiOperation("设备类型故障统计报表")
    @PostMapping("/faultStatistic")
    public JsonResult<List<FaultNumber>> faultStatistic(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.faultStatistic(reportStatisticAO));
    }


    @ApiOperation("配件故障统计报表")
    @PostMapping("/partsFaultStatistic")
    public JsonResult<List<PartsFaultNumber>>  partsFaultStatistic(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.partsFaultStatistic(reportStatisticAO));
    }


    @ApiOperation("配件销售额统计报表")
    @PostMapping("/saleFittingsStatistic")
    public JsonResult<List<FittingsSales>>  saleFittingsStatistic(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.saleFittingsStatistic(reportStatisticAO));
    }


    @ApiOperation("客户配件销售额统计报表")
    @PostMapping("/saleCustomerStatistic")
    public JsonResult<List<CustomerSales>>  saleCustomerStatistic(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.saleCustomerStatistic(reportStatisticAO));
    }


    @ApiOperation("测试工作实长sql报错问题")
    @PostMapping("/getGzsc")
    public JsonResult<PersonAllStatistic>  getGzsc(@RequestBody ReportStatisticAO reportStatisticAO){
        if(StringUtils.isNotBlank(reportStatisticAO.getType())){
            reportStatisticAO.setStartTime(returnStartTime(reportStatisticAO.getType()));
            reportStatisticAO.setEndTime(sdf.format(new Date()));
        }
        return JsonResult.success(orderService.checkPersonAllStatistic(reportStatisticAO));
    }



    public String returnStartTime(String type){
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        if(ReportStatisticsEnums.DataStatus.NEARLY_SEVEN_DAYS.getCode().equals(type)){
            //近七天
            calendar.add(Calendar.DATE, -6);
        }else if(ReportStatisticsEnums.DataStatus.NEARLY_A_MONTH.getCode().equals(type)){
            //近30天
            calendar.add(Calendar.DATE, -29);
        }else if(ReportStatisticsEnums.DataStatus.MONTH.getCode().equals(type)){
            //当月
            calendar.add(Calendar.MONTH, 0);
            calendar.set(Calendar.DAY_OF_MONTH,1);
        }else if(ReportStatisticsEnums.DataStatus.THAT_YEAR.getCode().equals(type)){
            //当年
            calendar.add(Calendar.YEAR, 0);
            calendar.set(Calendar.MONTH, 0);
            calendar.set(Calendar.DAY_OF_MONTH,1);
        }
        String endDate = sdf.format(calendar.getTime());
        return endDate;
    }






}
