package com.rootcloud.controller;

import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.JsonResult;
import com.rootcloud.domain.ao.ServicePlanAO;
import com.rootcloud.domain.ao.ServicePlanCalendarAO;
import com.rootcloud.domain.ao.ServicePlanCalendarNewAO;
import com.rootcloud.domain.entity.ds.ModelPropertySyncEntity;
import com.rootcloud.domain.service.sp.NxUserInfoService;
import com.rootcloud.domain.service.sp.ServicePlanService;
import com.rootcloud.vo.device.ServicePlanCalendarVO;
import com.rootcloud.vo.device.ServicePlanDetailVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @ClassName ServicePlanController
 * @Description 服务计划控制器
 * @Author w-liuyi01
 * @Date 2022/4/28 16:30
 * @Version 1.0
 */
@RestController
@Api(tags = "服务计划")
@RequestMapping("/servicePlan")
public class ServicePlanController {

    @Resource(name = "servicePlanServiceImpl")
    ServicePlanService planService;

    @Resource(name = "NxUserInfoServiceImpl")
    NxUserInfoService nxUserInfoService;


    /**
     * @return
     * @Author w-liuyi01
     * @Description //新增服务计划接口
     * @Date 2022/4/29 10:32
     * @Param
     **/
    @PostMapping(path = "/addPlan")
    @ApiOperation("新增服务计划")
    public JsonResult addPlan(@RequestBody @Validated ServicePlanAO planAO) {
        planService.addPlan(planAO);
        return JsonResult.success("新建计划成功");
    }


    @GetMapping("/startServicePlan")
    public JsonResult startServicePlan(String servicePlanId) {
        planService.startServicePlan(servicePlanId);
        return JsonResult.success("服务计划启动成功");
    }

    @ApiOperation("根据设备编码查询工况条件")
    @GetMapping("/workCondition/{deviceNo}")
    public JsonResult<List<ModelPropertySyncEntity>> getWorkCondition(@PathVariable String deviceNo) {
        return JsonResult.success(planService.getWorkCondition(deviceNo));
    }

    @ApiOperation("手动触发服务计划订单")
    @GetMapping("/manualTrigger/{planId}")
    public JsonResult manualTrigger(@PathVariable String planId) {
        //查询是否有订单，存在订单与删除
        return planService.manualTrigger(planId);
    }


    @ApiOperation("修改服务计划状态")
    @GetMapping("/updateStatus")
    public JsonResult<Void> updateStatus(@RequestParam @ApiParam("服务计划主键") String servicePlanId, @RequestParam @ApiParam("服务计划状态") Integer servicePlanStatus) {
        try {
            planService.updateServicePlanStatus(servicePlanId, 0, servicePlanStatus, false, null);
            return JsonResult.success();
        } catch (Exception e) {
            return JsonResult.failed("500", e.getMessage());
        }
    }


    @ApiOperation("查询计划任务详情")
    @GetMapping("/selectDataDetail")
    public JsonResult<ServicePlanDetailVO> selectDataDetail(@RequestParam @ApiParam("服务计划主键") String servicePlanId) {
        return JsonResult.success(planService.selectDetailById(servicePlanId));
    }


    @ApiOperation("删除计划任务")
    @GetMapping("/deleteServicePlan")
    public JsonResult deleteServicePlan(@RequestParam @ApiParam("服务计划主键") String servicePlanId) {
        return JsonResult.success(planService.deleteServicePlan(servicePlanId));
    }


    @PostMapping(path = "/checkServicePlanCalendar")
    @ApiOperation("查询任务日历接口")
    public JsonResult<ServicePlanCalendarVO> checkServicePlanCalendar(@RequestBody @Validated ServicePlanCalendarAO planCalendarAO) {
        ServicePlanCalendarVO servicePlanCalendarVO = planService.checkServicePlanCalendar(planCalendarAO);
        return JsonResult.success(servicePlanCalendarVO);
    }


    @ApiOperation("通过设备编码查询对应的用户数据")
    @GetMapping("/getCustomerAndRoles")
    public JsonResult<List<String>> getCustomerAndRoles(@RequestParam @ApiParam("设备编码") String deviceCode) {
        return JsonResult.success(nxUserInfoService.getCustomerAndRoles(deviceCode));
    }


    @PostMapping(path = "/checkServicePlanCalendarAll")
    @ApiOperation("查询任务日历总接口")
    public JsonResult<ServicePlanCalendarVO> checkServicePlanCalendarAll(@RequestBody @Validated ServicePlanCalendarNewAO servicePlanCalendarNewAO) {
        ServicePlanCalendarVO servicePlanCalendarVO = planService.checkDataByParams(servicePlanCalendarNewAO);
        return JsonResult.success(servicePlanCalendarVO);
    }

    @PostMapping(path = "/updateDate")
    @ApiOperation("更新服务计划下一个周期日期")
    @IgnoreSecurity
    public JsonResult<Object> updateDate() {
        planService.updateDate();
        return JsonResult.success();
    }

    @PostMapping(path = "/scheduledProcessing")
    @ApiOperation("执行服务计划")
    @IgnoreSecurity
    public JsonResult<Object> scheduledProcessing(@RequestParam("thatDay") String thatDay,
                                                  @RequestParam("planId") String planId) {
        planService.scheduledProcessingNew(thatDay, planId);
        return JsonResult.success();
    }

    @PostMapping(path = "/pause")
    @ApiOperation("暂停服务计划")
    @IgnoreSecurity
    public JsonResult<Object> pause(@RequestParam("planId") String planId,
                                    @RequestParam("type") String type) {
        return planService.pause(planId,type);
    }

    @PostMapping(path = "/pretrigger")
    @ApiOperation("提前触发")
    @IgnoreSecurity
    public JsonResult<Object> pretrigger(@RequestParam("planId") String planId) {
        return planService.pretrigger(planId);
    }
}
