package com.rootcloud.domain.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.rootcloud.domain.dto.*;
import com.rootcloud.domain.entity.*;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 宁夏_订单
 * 
 * @author lipingan
 * @email lpa2273597772@163.com
 * @date 2022-05-07 13:41:38
 */
@Mapper
public interface NxOrderMapper extends BaseMapper<NxOrderEntity> {

    List<NxOrderEntity> getOrderByEngineer(String fwryYhxm);

    List<EngineerModel> getEngineer();

    List<PendingOrderModel> pendingOrder();

    /**
     * 订单分页
     * 
     * @param page
     *            分页对象
     * @param serviceType
     *            服务类型
     * @param orderStatus
     *            订单状态
     * @return 分页的数据
     */
    IPage<OrderVO> orderPage(IPage<OrderVO> page, @Param("serviceType") String serviceType,
                             @Param("orderStatus") String orderStatus);

    /**
     * 订单详情
     * 
     * @param orderId
     *            订单主键
     * @return 详情数据
     */
    OrderInfoVO orderInfo(@Param("orderId") String orderId);

    int equipmentOrderNum(String orderDeviceCode);


    /**
     * 通过工程师和预约时间 来查询是否有任务
     * @param engineerId
     * @param bookingTime
     * @return
     */
    List<NxOrderEntity> selectByEngineer(@Param("engineerId")String engineerId,@Param("bookingTime")String bookingTime);

    /**
     * 判断工程师未来7填是否空闲/繁忙
     * @param engineerId
     * @return
     */
    int selectByEngineerBusyness(@Param("engineerId")String engineerId);

    List<FwryDTO> getNxgxJcdaFwryList(FwryDTO reqDto);

    Integer delFwry(String nxgxJcdaFwryId);


    /**
     * 根据任务计划主键查询订单
     * @param servicePlanId
     * @return
     */
    NxOrderEntity selectByServicePlanId(@Param("servicePlanId")String servicePlanId,@Param("time")String time);

    void updateNxgxJcdaFwryByUserId(HashMap req);

    List<NxOrderEntity> getOrderByEngineerId(String userId);

    List<FwryDTO> getNxgxJcdaFwryByUserId(String userId);

    Integer getOrderNum(HashMap<String, String> req);

    JeCoreEnduserEntity getCoreEnduserByUsrId(String userId);

    List<KhglDTO> getKhBykhdzs(String[] split);

    List<NxDeviceEntity> getNxDeviceEntity(Map<String, Object> req);
    int getNxDeviceEntityCount(Map<String, Object> req);

    NetworkEntity getfwwd(FwryDTO fwryDTO);

    List<NxDeviceEntity> getNxDeviceEntityList(Map<String, Object> req);
    int getNxDeviceEntityListCount(Map<String, Object> req);

    ServiceViewResultModel getPhotoByUserId(String userId);

    List<PendingOrderModel> getServiceView(HashMap req);

    List<NxOrderEntity> getServiceViewBySbbm(HashMap req);

    String getUserInfo(String userId);


    List<NxOrderEntity> orderStatusStatistic(@Param("startTime")String startTime,@Param("endTime")String endTime,@Param("orderStatus")String orderStatus,@Param("userIds") String userIds,@Param("wdUserId") String wdUserId);


    OrderStatusStatistic orderCount(@Param("startTime")String startTime, @Param("endTime")String endTime,@Param("userIds") String userIds, @Param("wdUserId") String wdUserId);


    DeviceStatusStatistic deviceStatusStatistic(@Param("userIds") String userIds);


    List<Outlets> outletsStatistic(@Param("startTime")String startTime, @Param("endTime")String endTime, @Param("wdUserId") String wdUserId);


    List<OrderStatistic> orderStatisticByDay(@Param("syzhid")String syzhid,@Param("startTime")String startTime, @Param("endTime")String endTime,@Param("userIds") String userIds, @Param("wdUserId") String wdUserId);


    List<OrderStatistic> orderStatisticByDayMonth(@Param("syzhid")String syzhid,@Param("startTime")String startTime, @Param("endTime")String endTime,@Param("userIds") String userIds, @Param("wdUserId") String wdUserId);


    List<PersonnelStatistic>  checkPersonStatistic(@Param("startTime")String startTime, @Param("endTime")String endTime, @Param("wdUserId") String wdUserId,@Param("sortType") String sortType,@Param("serviceType") String serviceType,@Param("sortReverseOrder") String sortReverseOrder,@Param("orderStatus") String orderStatus,@Param("syZhid") String syZhid);

    /**
     * 工作实长重写 ，需加上协助人员
     * @param startTime
     * @param endTime
     * @param wdUserId
     * @param serviceType
     * @param sortReverseOrder
     * @param orderStatus
     * @param syZhid
     * @return
     */
     List<PersonnelStatistic>  getGzsc(@Param("startTime")String startTime, @Param("endTime")String endTime, @Param("wdUserId") String wdUserId,@Param("serviceType") String serviceType,@Param("sortReverseOrder") String sortReverseOrder,@Param("orderStatus") String orderStatus,@Param("syZhid") String syZhid);

     List<PersonnelStatistic> getGcsIds(@Param("startTime")String startTime, @Param("endTime")String endTime, @Param("wdUserId") String wdUserId,@Param("serviceType") String serviceType,@Param("syZhid") String syZhid);
     BigDecimal getGcsGzsc(@Param("startTime")String startTime, @Param("endTime")String endTime,@Param("serviceType") String serviceType,@Param("syZhid") String syZhid,@Param("gcsId") String gcsId);

     List<FaultNumber>  faultStatistic(@Param("startTime")String startTime, @Param("endTime")String endTime,@Param("userIds") String userIds, @Param("wdUserId") String wdUserId);


    List<PartsFaultNumber>  partsFaultStatistic(@Param("startTime")String startTime, @Param("endTime")String endTime, @Param("wdUserId") String wdUserId,@Param("serviceType") String serviceType);



    List<FittingsSales> saleFittingsStatistic(@Param("startTime")String startTime,@Param("endTime")String endTime,@Param("wdUserId") String wdUserId,@Param("businessType") String businessType);



    List<CustomerSales> saleCustomerStatistic(@Param("startTime")String startTime,@Param("endTime")String endTime,@Param("userIds") String userIds,@Param("businessType") String businessType);



    List<NxOrderEntity>  selectByUserId(@Param("zhId")String zhId,@Param("userId")String userId,@Param("type")String type,@Param("startNo") Integer startNo,@Param("endNo") Integer endNo);

    Integer countByUserId(@Param("zhId")String zhId,@Param("userId")String userId, @Param("type")String type);

    String  getContentByMbId(@Param("mbId")String mbId);



    List<ContractEntity>  jsGetByUserId(@Param("userIds")String userIds);


    List<ContractEntity>  fwGetByUserId(@Param("userIds")String userIds);


    void mesSfjj(String orderId);

    void eqMesSfjj(String id);

    void htSerMesSfjj(String id);

    void jsSerMesSfjj(String id);

    List<String> getKmcList(HashMap<String, String> req);

    List<NxSparePartEntity> getSpartPartList(HashMap<String, Object> req);

    Integer countSpartPartList(HashMap<String, Object> req);

    List<NxOrderEntity> getOrderTypeNum(String userId);
    List<NxOrderEntity> getOrderTypeNum2(String userId);

    List<NxOrderEntity> getOrderTypeNumByCustomer(Map<String,Object> query);

    //List<KhVO> getKuList(Map<String,Object> query);
    //List<KhVO> getKuList(@Param("userId") String userId);

    List<DeviceVO> getKuDeviceList(Map<String,Object> query);
    /**
     * 修改订单审核时间，完工时间，指派工程师时间
     */
    void updateOrderShsj();
    void updateOrderWgsj();
    void updateOrderZpgcssj();

    /*******************生成订单模块*********************/
    Integer getOrderCount(@Param("date") String date);
}
