package com.rootcloud.domain.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.domain.ao.ServicePlanCalendarNewAO;
import com.rootcloud.domain.dto.ConditionModel;
import com.rootcloud.domain.entity.NxZskFaultCodeEntity;
import com.rootcloud.domain.ao.ServicePlanParamsAO;
import com.rootcloud.domain.entity.sp.ServicePlanEntity;
import com.rootcloud.vo.service.MaintenanceListVo;
import com.rootcloud.vo.service.OutletsVO;
import com.rootcloud.vo.service.ServicePlanInfoVO;
import com.rootcloud.vo.service.ServicePlanVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

@Mapper
public interface ServicePlanMapper extends BaseMapper<ServicePlanEntity> {
    /**
     * 根据故障码查询等级和设备型号
     * 
     * @param codes
     *            故障码
     * @return 等级和设备型号
     */
    List<NxZskFaultCodeEntity> selectAlarmLevel(@Param("codes") List<String> codes);

    /**
     * 查询所有满足条件的触发条件
     * 
     * @param date
     *            时间yyyy-MM-dd
     * @param planStatus
     *            计划状态
     * @param triggerType
     *            触发类型
     * @param detailStatus
     *            是否执行
     * @param alarmTypeId
     *            报警类型Id
     * @return 触发条件
     */
    List<ConditionModel> findConditionList(@Param("date") String date, @Param("planStatus") Integer planStatus,
        @Param("triggerType") String triggerType, @Param("detailStatus") Integer detailStatus,
        @Param("alarmTypeId") String alarmTypeId);

    /**
     * 根据计划ID修改执行状态
     * 
     * @param planIds
     *            计划ID
     * @param status
     *            状态码 1.可执行； 0.不可执行
     */
    void updateDetailStatusByPlanIds(@Param("planIds") List<String> planIds, @Param("status") Integer status);

    /**
     * 修改状态
     * 
     * @param planIds
     *            计划主键
     * @param planStatus
     *            计划状态
     * @param detailStatus
     *            执行状态
     */
    void updateStatus(@Param("planIds") List<String> planIds, @Param("planStatus") ServicePlanEnums.PlanStatus planStatus,
        @Param("detailStatus") Integer detailStatus);

    /**
     * 查询触发任务是系统时间的
     * 
     * @return
     */
    List<ServicePlanVo> selectSysTimeList();

    /**
     * 通过条件查询计划任务
     *
     * @param planParamsAO
     * @return
     */
    List<ServicePlanEntity> selectListByParams(@Param("planParamsAO") ServicePlanParamsAO planParamsAO);

    /**
     * 查询十点需要发送消息的服务计划数据
     * 
     * @return
     */
    List<ServicePlanEntity> selectScheduledTasksList();


    /**
     * 根据保养内容名称查询保养信息
     * @param maintainContentName
     * @return
     */
    List<MaintenanceListVo> selectListByName(@Param("maintainContentName") String maintainContentName);

    List<MaintenanceListVo> selectListByName2(@Param("maintainContentName") String maintainContentName,@Param("maintainContent") String maintainContent);


    List<Map> selectListXMBY(@Param("code") String code,@Param("syZhid") String syZhid);

    int insertXmbynr(@Param("list") List<Map> xmbynr,@Param("nxOrderId") String nxOrderId);


    List<String> getCustomerAndRoles(@Param("customerId") String customerId,@Param("syZhid") String syZhid);



    List<ServicePlanEntity>  selectByUserId(@Param("userId") String userId,@Param("startNo")Integer startNo,@Param("endNo")Integer endNo);

    Integer  countByUserId(@Param("userId") String userId);

    void eqMesSfjj(String id);



    List<ServicePlanInfoVO> selectByParams(@Param("servicePlanCalendarNewAO") ServicePlanCalendarNewAO servicePlanCalendarNewAO);


    Integer queryLgortsByCode(@Param("configCode") String configCode);


    String queryStringByCode(@Param("configCode") String configCode);


    String selectOutletsIdByCustomerId(@Param("customerId") String customerId, @Param("syZhid") String syZhid);


    OutletsVO returnUserNames(@Param("wdId") String wdId);


    List<Map<String,String>> getTodoLibrary(Map param);

    List<Map<String,String>> getPlanExsitOrder(Map param);

    //获取是否派工标识
    Map<String,String> getIslcpzb(String syZhid);

    /**
     * 查询触发任务是系统时间的
     *
     * @return
     */
    List<ServicePlanVo> selectExecutedServicePlan(@Param("thatDay") String thatDay);

    List<ServicePlanVo> selectExecutedServicePlanByPlanId(@Param("planId") String planId);

    List<ServicePlanEntity> selectData();

    void updateData(@Param("id") String id,@Param("date") String date);

    Integer checkOrder(@Param("servicePlanId") String servicePlanId,@Param("date") String date);

    Integer checkOrderCount(@Param("servicePlanId") String servicePlanId,@Param("startDate") String startDate,@Param("endDate") String endDate);

    String lastServicePlanOrderStatus(@Param("servicePlanId") String servicePlanId);

    void updateServicePlanStatus(@Param("id") String id,@Param("date") String date,@Param("status") String status,@Param("ztrq") String ztrq);

    void updateTqcfrq(@Param("id") String id,@Param("date") String date);
}
