package com.rootcloud.domain.service.app.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageInfo;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.PageRequest;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.config.auth.UserInfo;
import com.rootcloud.domain.entity.MyAdviceEntity;
import com.rootcloud.domain.entity.MyMessageEntity;
import com.rootcloud.domain.entity.NxSparePartEntity;
import com.rootcloud.domain.entity.ds.DeviceListEntity;
import com.rootcloud.domain.mapper.NxOrderMapper;
import com.rootcloud.domain.mapper.ServicePlanMapper;
import com.rootcloud.domain.service.app.MyAdviceService;
import com.rootcloud.domain.service.device.DeviceService;
import com.rootcloud.domain.service.ds.DeviceListSyncService;
import com.rootcloud.domain.service.fittings.SparePartService;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import com.rootcloud.domain.service.sp.ServicePlanService;
import com.rootcloud.vo.app.AdviceResultVo;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service("myAdviceServiceImpl")
public class MyAdviceServiceImpl implements MyAdviceService {
    @Resource(name = "servicePlanServiceImpl")
    private ServicePlanService servicePlanService;
    @Resource(name = "sparePartServiceImpl")
    private SparePartService sparePartService;
    @Resource(name = "deviceListSyncService2")
    private DeviceListSyncService deviceListSyncService;
    @Resource(name = "deviceServiceImpl")
    private DeviceService deviceService;
    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;
    @Autowired
    ServicePlanMapper servicePlanMapper;
    @Resource
    private NxOrderMapper nxOrderMapper;

    @Override
    public JsonResult<PageInfo<MyAdviceEntity>> getMyAdvices(PageRequest pageRequest, String userId, String type) {
        int pageNum = pageRequest.getPage() == null ? 1 : pageRequest.getPage();
        int pageSize = pageRequest.getSize() == null ? 10 : pageRequest.getSize();
        int startIndex = (pageNum - 1) * pageSize;
        Integer count = 0;
        UserInfo userInfo = enduserService.getUser(userId);
        AdviceResultVo adviceResultVo = new AdviceResultVo();
        List<MyAdviceEntity> myAdviceEntityList = new ArrayList<>();

        //todo 设备状态提醒通知
/*        if(deviceListEntities.isEmpty()){
            adviceResultVo.setAdviceEntityList(myAdviceEntityList);
            adviceResultVo.setAdviceCount(String.valueOf(myAdviceEntityList.size()));
            return adviceResultVo;
        }*/
        //保养提醒
        List<MyAdviceEntity> planList = new ArrayList<>();
        if (ServicePlanEnums.MessageType.MAINTENANCE_REMINDERS.getCode().equals(type) || StringUtils.isBlank(type)) {
            planList = servicePlanService.getmMaintainPlanService(userId, startIndex, pageSize);
            //count+=servicePlanMapper.countByUserId(userId);
            List<MyAdviceEntity> planListCount = servicePlanService.getmMaintainPlanService(userId, 0, 10000000);
            if (planListCount != null && planListCount.size() > 0) {
                count += planListCount.size();
            }
        }
        //配件寿命预警提醒
        List<MyAdviceEntity> partList = new ArrayList<>();
        if (ServicePlanEnums.MessageType.SPARE_PARTS_LIFE_WARNING_REMINDER.getCode().equals(type) || StringUtils.isBlank(type)) {
            //根据用户id获取所归属的设备信息
            List<DeviceListEntity> deviceListEntities = deviceListSyncService.queryDeviceList(userId);
            //Log.info("查询通知用户ID：" + userId + "设备数量：" + deviceListEntities.size());
            if (!deviceListEntities.isEmpty()) {
                partList = sparePartService.getWarnedSparePart(deviceListEntities, startIndex, pageSize, userInfo.getTenantId());
                List<String> deviceCodes = deviceListEntities.stream().map(DeviceListEntity::getNxJcxxSblbId).collect(Collectors.toList());

                HashMap<String, Object> req = new HashMap<>();
                req.put("startTime", DateUtil.format(DateUtil.rollDay(new Date(), -3)));
                req.put("endTime", DateUtil.format(new Date()));
                req.put("deviceCodes", deviceCodes);
                count += nxOrderMapper.countSpartPartList(req);
            }
        }
        if (StringUtils.isBlank(type)) {
            myAdviceEntityList.addAll(planList);
            myAdviceEntityList.addAll(partList);
        } else {
            if (ServicePlanEnums.MessageType.MAINTENANCE_REMINDERS.getCode().equals(type)) {
                myAdviceEntityList.addAll(planList);
            }
            if (ServicePlanEnums.MessageType.SPARE_PARTS_LIFE_WARNING_REMINDER.getCode().equals(type)) {
                myAdviceEntityList.addAll(partList);
//                Log.info("最终备件寿命预警消息",JSON.toJSON(partList));
            }
        }

        PageInfo<MyAdviceEntity> pageInfo = new PageInfo<>(myAdviceEntityList);
        pageInfo.setPageNum(pageNum);
        pageInfo.setPageSize(pageSize);
        pageInfo.setTotal(count);
        return JsonResult.success(pageInfo);
    }

    @Override
    public void eqMesSfjj(String id, String type) {
        if (ServicePlanEnums.MessageType.MAINTENANCE_REMINDERS.getCode().equals(type)) {
            servicePlanService.eqMesSfjj(id);
        }
        if (ServicePlanEnums.MessageType.SPARE_PARTS_LIFE_WARNING_REMINDER.getCode().equals(type)) {
            sparePartService.eqMesSfjj(id);
        }
    }


}
