package com.rootcloud.domain.service.app.impl;

import com.github.pagehelper.PageInfo;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.PageRequest;
import com.rootcloud.common.constant.RoleIdConstant;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.config.auth.UserInfo;
import com.rootcloud.domain.entity.MyMessageEntity;
import com.rootcloud.domain.entity.SiteDTO;
import com.rootcloud.domain.entity.ds.DeviceListEntity;
import com.rootcloud.domain.mapper.NxOrderMapper;
import com.rootcloud.domain.service.app.MyMessageService;
import com.rootcloud.domain.service.device.DeviceAlarmService;
import com.rootcloud.domain.service.ds.DeviceListSyncService;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import com.rootcloud.domain.service.order.OrderService;
import com.rootcloud.vo.app.MessageResultVo;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service("myMessageServiceImpl")
public class MyMessageServiceImpl implements MyMessageService {
    @Resource(name = "deviceListSyncService2")
    private DeviceListSyncService deviceListSyncService;
    @Resource(name="deviceAlarmServiceImpl")
    private DeviceAlarmService deviceAlarmService;
    @Resource(name = "orderServiceImpl")
    private OrderService orderService;
    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;
    @Autowired
    private NxOrderMapper nxOrderMapper;

    /**
     * 获取合同及结算消息通知
     * @param userId
     * @param type
     * @return
     */
    @Override
    public MessageResultVo getMyMessage(String userId,String type) {
        MessageResultVo messageResultVo = new MessageResultVo();
        List<MyMessageEntity> messageEntityList = new ArrayList<>();
        //根据角色获取合同结算通知
        UserInfo userInfo = enduserService.getUser(userId);
        String roleIds = userInfo.getRoleIds();
        String userIds = null;
        List<String> userIdList = new ArrayList<>();
//        Log.info("登录用户权限 ==>{}",roleIds);
        if (roleIds.contains(RoleIdConstant.KHLD) || roleIds.contains(RoleIdConstant.KH)){
            //userIdList.add(userId);
            String custom = enduserService.getCustomsByUserId(userId);
            if(StringUtils.isNotBlank(custom)){
                userIdList.add(custom);
            }
        }
        if (roleIds.contains(RoleIdConstant.FWJL)){
            SiteDTO siteByUserId = enduserService.getSiteByUserId(userId);
            if(!ObjectUtils.isEmpty(siteByUserId)){
                String[] strings = siteByUserId.getServiceArea().split(",");
                for (int i = 0; i < strings.length; i++) {
                    String areaName = strings[i];
                    String customs = enduserService.getCustomsBySiteAreaName(areaName);
                    if(StringUtils.isNotBlank(customs)){
                        userIdList.add(customs);
                    }
                }
            }
        }
        if(roleIds.contains(RoleIdConstant.SHLD) || roleIds.contains(RoleIdConstant.GLY)){
            userIdList = new ArrayList<>();
        }
        if(userIdList.size()>0){
            userIds = String.join(",", userIdList);
        }


        //合同及结算消息通知
        List<MyMessageEntity> jsMessages = new ArrayList<>();
        List<MyMessageEntity> fwMessages = new ArrayList<>();
        jsMessages = orderService.jsGetByUserId(userIds);
        fwMessages = orderService.fwGetByUserId(userIds);

        messageEntityList.addAll(jsMessages);
        messageEntityList.addAll(fwMessages);

//        Log.info("查询结果messageEntityList ==>{}",JSON.toJSONString(messageEntityList));
        messageResultVo.setMessageCount(String.valueOf(messageEntityList.size()));
        messageResultVo.setMessageEntityList(messageEntityList);
        return messageResultVo;
    }

    /**
     * 获取报警故障信息
     * @param userId
     * @return
     */
    @Override
    public MessageResultVo getMyAlrmMessage(String userId, String type) {
        MessageResultVo messageResultVo = new MessageResultVo();
        List<MyMessageEntity> messageEntityList = new ArrayList<>();

        //根据用户id获取所归属的设备信息
        List<DeviceListEntity> deviceListEntities = deviceListSyncService.queryDeviceList(userId);
        //Log.info("获取到设备ID："+JSON.toJSONString(lista));
        //报警信息
        List<MyMessageEntity> alarmMessages = new ArrayList<>();
        List<MyMessageEntity> alarmMes = new ArrayList<>();
        if(!deviceListEntities.isEmpty()) {
            alarmMes = deviceAlarmService.queryAlarmMessages(deviceListEntities);
            alarmMessages.addAll(alarmMes);
           /* List<List<DeviceListEntity>> lists = splitListByCount(deviceListEntities, 50);
            Log.info("获取lists ==>{}",lists.size());
            for (List<DeviceListEntity> list:lists) {
                alarmMes = deviceAlarmService.queryAlarmMessages(list);
                alarmMessages.addAll(alarmMes);
            }*/
        }

        messageEntityList.addAll(alarmMessages);
        Log.info("查询结果messageEntityList ==>{}",messageEntityList.size());
        messageResultVo.setMessageCount(String.valueOf(messageEntityList.size()));
        if(ServicePlanEnums.MessageType.FAULT_ALARM.getCode().equals(type)){
            messageResultVo.setMessageEntityList(messageEntityList);
        }

        return messageResultVo;
    }

    @Override
    public JsonResult<PageInfo<MyMessageEntity>> getOrderMessage(PageRequest pageRequest, String userId, String type) {
        int pageNum = pageRequest.getPage() == null ? 1 : pageRequest.getPage();
        int pageSize = pageRequest.getSize() == null ? 10 : pageRequest.getSize();
        int startIndex  = (pageNum -1 ) * pageSize;
        //根据角色获取合同结算通知
        UserInfo userInfo = enduserService.getUser(userId);
        MessageResultVo messageResultVo = new MessageResultVo();
        List<MyMessageEntity> orderMessages = orderService.selectByUserId(userInfo.getTenantId(),userId,type,startIndex,pageSize);
        messageResultVo.setMessageCount(String.valueOf(orderMessages.size()));
        messageResultVo.setMessageEntityList(orderMessages);

        Integer count = nxOrderMapper.countByUserId(userInfo.getTenantId(),userId,type);

        PageInfo<MyMessageEntity> pageInfo = new PageInfo<>(orderMessages);
        pageInfo.setPageNum(pageNum);
        pageInfo.setPageSize(pageSize);
        pageInfo.setTotal(count);

        return JsonResult.success(pageInfo);
    }

    @Override
    public void mesSfjj(String orderId) {
        orderService.mesSfjj(orderId);
    }

    @Override
    public void serMesSfjj(String id, String type) {
        //合同或清算 0：合同，1：结算
        if("0".equals(type)){
            orderService.htSerMesSfjj(id);
        }else{
            orderService.jsSerMesSfjj(id);
        }
    }

    public List<List<DeviceListEntity>> splitListByCount(List<DeviceListEntity> list,int count) {
        List<List<DeviceListEntity>> listAll = new ArrayList<>();
        int size = list.size();
        if(size > count){
            int absInt = Math.abs(size/count);
            if(size - absInt * count > 0){
                listAll.add(list.subList(absInt*count,size));
            }
            for (int i =1;i<absInt+1;i++){
                listAll.add(list.subList((i-1)*count,i*count));
            }
        }else{
            listAll.add(list);
        }
        return  listAll;
    }


}
