package com.rootcloud.domain.service.auth;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.rootcloud.common.*;
import com.rootcloud.common.constant.Properties;
import com.rootcloud.common.enums.HttpRequestMethedEnum;
import com.rootcloud.domain.dao.JeCoreEnduserDao;
import com.rootcloud.domain.entity.JeCoreEnduserEntity;
import com.rootcloud.domain.mapper.CommonMapper;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import javax.annotation.Resource;
import java.util.Base64.Encoder;
import java.util.Base64.Decoder;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 认证授权
 */
@Component("cloudAuthService")
public class RootCloudAuthServiceImpl implements RootCloudAuthService {

    private final Logger logger = LoggerFactory.getLogger(RootCloudAuthServiceImpl.class);

    @Autowired
    private OAuth2Properties auth2Properties;

    private static final String ROOT_CLOUD_CODE = "code";

    @Autowired
    Request request;

    @Autowired
    Properties wfProperties;

    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;

    @Autowired
    JeCoreEnduserDao enduserDao;

    @Autowired
    private CommonMapper commonMapper;


    /**
     * ouath2.0 passsword 验证方式
     *
     * @param username
     * @param desEncrypt
     * @return
     */
    @Override
    public OAuthResult login(String username, String desEncrypt, String code) {
        //判断用户是否为平行租户账户
        LambdaQueryWrapper<JeCoreEnduserEntity> mpeWrapper = new LambdaQueryWrapper<JeCoreEnduserEntity>();
        mpeWrapper.eq(JeCoreEnduserEntity::getBackusercode,username);
        mpeWrapper.like(JeCoreEnduserEntity::getUserid, "~");
        mpeWrapper.last("limit 1");
        List<JeCoreEnduserEntity> one = enduserDao.selectList(mpeWrapper);
        if (one != null&&one.size()>0&&!username.contains("@")) {
            String url = auth2Properties.getWebClientConfiguration().getServerApiPrefix() + "/account-manage/v1/auth/login";
            Map<String, String> map = getClientInfo("password");
            //20221109 替换用户当前租户下CLIENT_ID和CLIENT_SECRET
            Map<String,String> rootClient = commonMapper.getRootClient(one.get(0).getZhid());
            if(rootClient!=null) {
                if(StringUtils.isNotEmpty(rootClient.get("CLIENT_ID"))&&StringUtils.isNotEmpty(rootClient.get("CLIENT_SECRET"))){
                    map.put("client_id",rootClient.get("CLIENT_ID"));
                    map.put("client_secret",rootClient.get("CLIENT_SECRET"));
                }
            }
            map.put("username", username);
            map.put("password", desEncrypt);
            Log.info("登录" + map);
            JSONObject res = request.requestPost(url, map);
            if (res != null) {
                return solveToken(res);
            }
        }
        //PingAn.Li 新增当根云用户无法登陆时，去登录应用大师
        String jpLoginUrl = wfProperties.getWfHost() + "/rbac/login/login";
        String encodeUserName = "";
        String encodePassword = "";
        try {
            Encoder encoder = Base64.getEncoder();
//            encodeUserName = new BASE64Encoder().encode(new BASE64Encoder().encode(username.getBytes("UTF-8")).getBytes("UTF-8"));
            encodeUserName = encoder.encodeToString(encoder.encodeToString(username.getBytes("UTF-8")).getBytes("UTF-8"));
            encodePassword = encoder.encodeToString(encoder.encodeToString(desEncrypt.getBytes("UTF-8")).getBytes("UTF-8"));
//            encodePassword = new BASE64Encoder().encode(new BASE64Encoder().encode(desEncrypt.getBytes("UTF-8")).getBytes("UTF-8"));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Type", "application/x-www-form-urlencoded");
        String body = "u=" + encodeUserName + "&p=" + encodePassword+ "&loginType=MAPP";
        if(StringUtils.isNotEmpty(code)){
            body+="&code=" + code;
        }
        HttpEntity<String> requestJp = new HttpEntity<>(body, headers);
//        RestTemplate restTemplate = new RestTemplate();
//        ResponseEntity<String> entity = restTemplate.postForEntity(jpLoginUrl, requestJp, String.class);
        ResponseEntity<String> entity = request.requestForJP(jpLoginUrl, requestJp);
        JSONObject jpRes = JSONObject.parseObject(entity.getBody());
        Log.info("登录返回值:" + jpRes);
        if (jpRes != null && jpRes.getBoolean("success")) {
            return solveTokenByJP(jpRes);
        } else if (jpRes != null && ("WARNLOCKED".equals(jpRes.getString("code"))||"WARNCODE".equals(jpRes.getString("code")))) {
            OAuthResult result = new OAuthResult("");
            result.setCode("WARNLOCKED");
            result.setValue(jpRes.getString("obj"));
            result.setMessage(jpRes.getString("message"));
            return result;
        }
        return null;
    }

    @Override
    public String getUserProfile(String accessToken) {
        if (null == accessToken || "".equals(accessToken)) {
            return "";
        }
        String[] splitToken = accessToken.split("\\.");
        byte[] bt = Base64.getUrlDecoder().decode(splitToken[1]);
        JSONObject jsonObject = JSONObject.parseObject(bt, JSONObject.class);
        return jsonObject.getJSONObject("user").getString("id");
    }


    @Override
    public String publicKey() {
        String url = auth2Properties.getWebClientConfiguration().getServerApiPrefix() + "/system/public-key";
        return request.requestCasGet(url, null).getString("publicKey");
    }

    @Override
    public String logout(String accessToken) {
        if (accessToken.length() < 60) {
            String jpLoginUrl = wfProperties.getWfHost() + "/je/j_spring_security_logout";
            HttpHeaders headers = new HttpHeaders();
            headers.add("authorization", accessToken);
            HttpEntity<String> requestJp = new HttpEntity<>(null, headers);
            try {
                ResponseEntity<String> response = request.requestForJP(jpLoginUrl, requestJp);
                String body=response.getBody();
                Log.info("退出登录je/j_spring_security_logout返回值："+body);
            } catch (Exception e) {
                logger.error("退出登录异常：", e);
            }
        } else {
            String url = auth2Properties.getWebClientConfiguration().getServerApiPrefix() + "/account-manage/v1/auth/logout";
            MultiValueMap<String, String> headers = iamAuthorization(accessToken);
            Map<String, String> map = new HashMap<>();
            try {
                JSONObject  jsonResult =request.request(url, HttpMethod.POST, headers, map);
                Log.info("退出登录account-manage/v1/auth/logout返回值："+jsonResult.toJSONString());

            } catch (Exception e) {
                logger.error("退出登录异常：", e);
            }
        }
        return null;
    }


    /**
     * 客户端模式认证
     *
     * @return
     */
    @Override
    public OAuthResult clientCredentials() {
        String url = auth2Properties.getWebClientConfiguration().getServerApiPrefix() + "/openapi/v1/auth/login";
        Map<String, String> map = getClientInfo("client_credentials");
        if (StringUtils.isBlank(auth2Properties.getWebClientConfiguration().getPhoenixClientId())) {
            return null;
        }
        map.put("client_id", auth2Properties.getWebClientConfiguration().getPhoenixClientId());
        map.put("client_secret", auth2Properties.getWebClientConfiguration().getPhoenixSecret());
        JSONObject res = request.requestPost(url, map);
        if (res == null) {
            return null;
        }

        return solveToken(res);
    }

    /**
     * 刷新token
     *
     * @param refreshToken
     * @return
     */
    @Override
    public OAuthResult refreshAccessToken(String refreshToken) {
        String url = auth2Properties.getWebClientConfiguration().getServerApiPrefix() + "/openapi/v1/auth/login";
        Map<String, String> map = getClientInfo("refresh_token");
        map.put("refresh_token", refreshToken);
        JSONObject res = request.requestPost(url, map);
        if (res == null) {
            return null;
        }

        return solveToken(res);
    }

    @Override
    public OAuthResult switchTenant(String tenantId, String accessToken) {
        String url = auth2Properties.getWebClientConfiguration().getServerApiPrefix() + "/openapi/v1/auth/change-active-organization";
        Map<String, String> map = new HashMap<>(1);
        map.put("organization", tenantId);
        JSONObject res = null;
        try {
            MultiValueMap<String, String> headers = iamAuthorization(accessToken);
            res = request.request(url, HttpMethod.POST, headers, map);
        } catch (Exception e) {
            logger.error("切换租户异常：", e);
        }
        if (res == null) {
            return null;
        }
        return solveToken(res);
    }

    private MultiValueMap<String, String> iamAuthorization(String accessToken) {
        MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
        headers.add(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_JSON_UTF8_VALUE);
        headers.add(HttpHeaders.ACCEPT, MediaType.APPLICATION_JSON_UTF8_VALUE);
        headers.add("authorization", String.format("Bearer %s", accessToken));
        return headers;
    }

    private OAuthResult solveToken(JSONObject res) {
        // String accessToken = res.getString("access_token");
        //String refreshToken = res.getString("refresh_token");
        OAuthResult oAuth2AccessToken = new OAuthResult(res);
        oAuth2AccessToken.setAdditionalInformation(new HashMap<>(1));

        if (res.getString(ROOT_CLOUD_CODE) != null) {
            oAuth2AccessToken.getAdditionalInformation().put(ROOT_CLOUD_CODE, res.getString(ROOT_CLOUD_CODE));
            oAuth2AccessToken.setValue(res.getString(Request.X_ROOTCLOUD_SETPW_TOKEN));
        }

        return oAuth2AccessToken;
    }

    private OAuthResult solveTokenByJP(JSONObject res) {
        OAuthResult oAuth2AccessToken = new OAuthResult(res.getString("obj"));
        oAuth2AccessToken.setAdditionalInformation(new HashMap<>(1));
        if (res.getString(ROOT_CLOUD_CODE) != null) {
            oAuth2AccessToken.getAdditionalInformation().put(ROOT_CLOUD_CODE, res.getString(ROOT_CLOUD_CODE));
            oAuth2AccessToken.setValue(res.getString(Request.X_ROOTCLOUD_SETPW_TOKEN));
        }
        return oAuth2AccessToken;
    }

    private Map<String, String> getClientInfo(String grantType) {
        Map<String, String> map = new HashMap<>(6);
        map.put("client_id", auth2Properties.getWebClientConfiguration().getClientId());
        map.put("client_secret", auth2Properties.getWebClientConfiguration().getSecret());
        map.put("grant_type", grantType);
        return map;
    }


    /**
     * 获取验证码 逻辑：当用户在小程序页面输入手机号点击获取验证码，判断该用户是否存在。如果存在，直接后端调用应用大师发送验证码接口
     * 如果不存在，后端调用应用大师注册用户接口，先注册用户再调发送验证码接口
     *
     * @return
     */
    @Override
    public JsonResult<Object> getCode(Map<String, Object> param) {

        String phone = param.get("phoneNumber").toString();
        String cz ="LOGIN";
        if(param.get("cz")!=null){
            cz = param.get("cz").toString();
        }
        String type ="PHONE";
        if(param.get("type")!=null){
            type = param.get("type").toString();
        }
        //验证码路径
        String jpLoginUrl = wfProperties.getWfHost() + "/rbac/login/sendRandom";
        //用户注册路径
        String registerUserUrl = wfProperties.getWfHost() + "/je/rbac/user/registerUser";
        //判断当前用户存不存在
        boolean flag = enduserService.getBackusercode(phone);
        if (flag) {
            //20221015 如果移动端发送验证码则直接返回
            if ("MRZ".equals(cz)){
                if ("PHONE".equals(type)) {
                    return JsonResult.failed("500", "该手机号已被其他用户使用!");
                } else if ("EMAIL".equals(type)) {
                    return JsonResult.failed("500", "该邮箱已被其他用户使用!");
                }
            }
            //存在，直接调用应用大师获取验证码
            Map<String, String> maps = new HashMap<>();
            maps.put("email", phone);
            maps.put("type", type);
            maps.put("cz", cz);
            String str = HttpUtils.post(jpLoginUrl, maps);
            return JsonResult.success(str);
        } else {
            //20221015 直接发送验证码不注册
            Map<String, String> maps = new HashMap<>();
            maps.put("email", phone);
            maps.put("type", type);
            maps.put("cz", cz);
            String strs = HttpUtils.post(jpLoginUrl, maps);
            return JsonResult.successOK(strs,"发送成功");
            //20221031 注释自动注册账户逻辑
//            String roleCode = enduserService.getDataByKey("ROLECODE");
//            String deptCode = enduserService.getDataByKey("DEPTCODE");
//            String pwd = enduserService.getDataByKey("PASSWORD");
//            String str = "ROLECODE=" + roleCode + "&DEPTCODE=" + deptCode + "&USERNAME=" + phone + "&PASSWORD=" + pwd + "&BACKUSERCODE=" + phone;
//            //注册
//            String res = HttpClientUtil.getPostRaw(registerUserUrl, str);
//            logger.info("注册结果：{}", res);
//            JSONObject object = JSONObject.parseObject(res);
//            if (object.containsKey("code")) {
//                String code = object.getString("code");
//                if (code.equals("1000")) {
//                    //发送验证码
//                    Map<String, String> maps = new HashMap<>();
//                    maps.put("email", phone);
//                    maps.put("type", type);
//                    maps.put("cz", cz);
//                    String strs = HttpUtils.post(jpLoginUrl, maps);
//                    return JsonResult.successOK(strs,"发送成功");
//                }
//            }

        }
        //return JsonResult.failed("500","发送失败");
    }

    @Override
    public JsonResult<Object> findPassWord(Map<String, Object> param) {
        Map<String, String> header = new HashMap<>();
        String modifyPwUrl = wfProperties.getWfHost() + "/rbac/login/modifyPw";
        String type ="PHONE";
        if(param.get("type")!=null){
            type = param.get("type").toString();
        }
        Map<String,String> sendParam = new HashMap<>();
        sendParam.put("code",param.get("validateCode").toString());
        sendParam.put("type",type);
        sendParam.put("d","");
        sendParam.put("u",param.get("mobile").toString());
        sendParam.put("p",param.get("password").toString());

        Log.info("找回密码参数"+param);
        String strs = HttpClientUtil.postForm(modifyPwUrl, sendParam,header);
        Log.info("找回密码："+strs);

        JSONObject object = JSONObject.parseObject(strs);
        if (object.containsKey("code")) {
            String code = object.getString("code");
            if (code.equals("1000")) {
                return JsonResult.successOK(strs,"重置成功");
            }
        }
        return JsonResult.failed("500","找回失败");

    }

    @Override
    public JsonResult<Object> modifyPassWord(Map<String, Object> param) {
        Map<String, String> header = new HashMap<>();
        header.put("authorization",param.get("Authorization").toString());
        String type ="PHONE";
        if(param.get("type")!=null){
            type = param.get("type").toString();
        }
        //验证-验证码url
        String validateUrl = wfProperties.getWfHost() + "/rbac/login/validateRandom";
        Map<String,String> validateMap = new HashMap();
        validateMap.put("email",param.get("mobileEmail").toString());
        validateMap.put("type",type);
        validateMap.put("cz",param.get("cz").toString());
        validateMap.put("code",param.get("vcCode").toString());
        Log.info("修改密码-验证码,参数"+validateMap+header);
        String validatestr = HttpClientUtil.postForm(validateUrl, validateMap,header);

        Log.info("修改密码-验证码返回值："+validatestr);
        JSONObject object = JSONObject.parseObject(validatestr);
        if (object.containsKey("code")) {
            String code = object.getString("code");
            if (!code.equals("1000")) {
                String message =object.getString("message");
                return JsonResult.failed("500",message);
            }
        }
        Map<String, String> modifyPwparam = new HashMap<>();
        modifyPwparam.put("o",param.get("oldPwd").toString());
        modifyPwparam.put("n",param.get("newPwd").toString());
        modifyPwparam.put("newPw2",param.get("newPwd").toString());
        String modifyPwUrl = wfProperties.getWfHost() + "/je/rbac/user/modifyPassWord";
        Log.info("修改密码-提交参数"+modifyPwparam);
        String strs = HttpClientUtil.postForm(modifyPwUrl, modifyPwparam, header);
        Log.info("修改密码返回值："+strs);

        JSONObject object1 = JSONObject.parseObject(strs);
        if (object1.containsKey("success")) {
            String code = object1.getString("success");
            if (code.equals("true")) {
                return JsonResult.successOK(strs,"修改成功");
            }
        }
        return JsonResult.failed("500","修改失败");
    }

    @Override
    public JsonResult<Object> updateEmail(Map<String, Object> param) {
        Map<String, String> header = new HashMap<>();
        String modifyEmailUrl = wfProperties.getWfHost() + "/rbac/login/modifyEmail";
        String type ="EMAIL";
        if(param.get("type")!=null){
            type = param.get("type").toString();
        }
        Map<String,String> sendParam = new HashMap<>();
        sendParam.put("code",param.get("code").toString());
        sendParam.put("type",type);
        sendParam.put("u",param.get("u").toString());
        sendParam.put("e",param.get("e").toString());

        Log.info("修改邮箱参数"+param);
        String strs = HttpClientUtil.postForm(modifyEmailUrl, sendParam,header);
        Log.info("修改邮箱："+strs);

        JSONObject object = JSONObject.parseObject(strs);
        if (object.containsKey("code")) {
            String code = object.getString("code");
            if (code.equals("1000")) {
                return JsonResult.successOK(strs,"修改成功");
            }
        }
        return JsonResult.failed("500","修改失败");
    }

}
