package com.rootcloud.domain.service.device.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpStatus;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.TokenUtil;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.entity.MyMessageEntity;
import com.rootcloud.domain.entity.NxTodoLibrary;
import com.rootcloud.domain.entity.NxZskFaultCodeEntity;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.entity.ds.DeviceListEntity;
import com.rootcloud.domain.mapper.NxDeviceMapper;
import com.rootcloud.domain.mapper.NxOrderMapper;
import com.rootcloud.domain.mapper.NxZskFaultCodeMapper;
import com.rootcloud.domain.service.device.DeviceAlarmService;
import com.rootcloud.domain.service.device.FaultCodeService;
import com.rootcloud.domain.service.todo.NxTodoLibraryService;
import com.rootcloud.vo.resp.AppDeviceAlarmResp;
import com.rootcloud.vo.resp.DeviceCodeAlarmModel;
import com.rootcloud.vo.resp.DeviceRuleAlarmModel;
import com.rootcloud.vo.rest.AppDeviceAlarmParam;
import com.rootcloud.vo.rest.DeviceAlarmParam;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service("deviceAlarmServiceImpl")
@Slf4j
public class DeviceAlarmServiceImpl implements DeviceAlarmService {
    @Value(value = "${os.iam.domain}")
    private String domain;
    @Value(value = "${os.iam.superToken}")
    private String superToken;
    @Value(value = "${cms.rulealarm.uri}")
    private String alaramUri;
    @Value(value = "${cms.codealarm.uri}")
    private String codeAlarmUri;

    @Value(value = "${cms.codealarmget.uri}")
    private String codeAlarmUriGet;

    @Value(value = "${cms.eventtype.uri}")
    private String eventTypeUri;


    @Value(value = "${cms.codealarmbatch.uri}")
    private String codeAlarmBatchUri;
    @Autowired
    private TokenUtil tokenUtil;
    @Resource(name = "faultCodeServiceImpl")
    private FaultCodeService faultCodeService;
    @Resource(name = "nxTodoLibraryServiceImpl")
    private NxTodoLibraryService nxTodoLibraryService;

    @Autowired
    private NxZskFaultCodeMapper nxZskFaultCodeMapper;

    @Autowired
    private NxDeviceMapper nxDeviceMapper;


    @Resource
    private RedisTemplate redisTemplate;

    /**
     * 规则报警查询接口
     *
     * @param deviceAlarmParam
     * @return
     */
    @Override
    public Map<String, Object> getDeviceRuleAlarmInfo(DeviceAlarmParam deviceAlarmParam) {
        List<DeviceRuleAlarmModel> resultList = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();
        HttpRequest request = HttpRequest.get(domain + alaramUri);
        //处理入参
        Map<String, Object> map = handleRuleAlarmParam(deviceAlarmParam);
        request.form(map);
        request.header("Authorization", "Bearer " + tokenUtil.getToken());
        request.header("accept", "application/json");
        request.header("Content-Type", "application/json");
        HttpResponse response = request.execute();
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("调用平台接口出错" + response);
            //throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
        }
        String result = response.body();
        JSONObject jsonObject = new JSONObject(result);
        JSONObject metadata = (JSONObject) jsonObject.get("metadata");
        String pageTotal = "0";
        if (metadata != null) {
            pageTotal = metadata.get("totalCount").toString();
        }
        JSONArray array = jsonObject.getJSONArray("payload");
        /*Log.info("平台报警数据信息查询--->" + array);
        Log.info("平台报警数据信息查询条数--->" + array.size());*/
        array.stream().forEach(temp -> {
            JSONObject obj = (JSONObject) temp;
            DeviceRuleAlarmModel deviceAlarmModel = new DeviceRuleAlarmModel();
            deviceAlarmModel.setAlarmTime(obj.get("createdTime").toString());
            deviceAlarmModel.setAlarmCode(obj.get("alarmTopicId") == null ? "" : obj.get("alarmTopicId").toString());
            deviceAlarmModel.setAlarmName(obj.get("alarmTopicName") == null ? "" : obj.get("alarmTopicName").toString());
            deviceAlarmModel.setAlarmDesc(obj.get("subject").toString());
            deviceAlarmModel.setAlarmLevel(obj.get("severityName").toString());
            //状态处理 自动解除（inActiveTime！=null）手动解除(closed=true&&inActiveTime=null) 持续中（close=false&&inActiveTime=null）
            if (obj.get("inActiveTime") != null) {
                deviceAlarmModel.setAlarmStatus(ResourceUtil.getI18nMessage("60000"));
            } else if ((boolean) obj.get("closed")) {
                deviceAlarmModel.setAlarmStatus(ResourceUtil.getI18nMessage("60001"));
            } else {
                deviceAlarmModel.setAlarmStatus(ResourceUtil.getI18nMessage("60002"));
            }
            resultList.add(deviceAlarmModel);
        });
        resultMap.put("pageTotal", pageTotal);
        resultMap.put("deviceAlarmModels", resultList);
        return resultMap;
    }

    /**
     * 故障码报警接口查询
     *
     * @param deviceAlarmParam
     * @return
     */
    @Override
    public Map<String, Object> getDeviceCodeAlarmInfo(DeviceAlarmParam deviceAlarmParam) {
        //获取设备型号
        String sbxh = nxZskFaultCodeMapper.getSbxh(deviceAlarmParam.getSbxhId());
        //Log.info("参数sbxh-->" + sbxh);
        List<DeviceCodeAlarmModel> list = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();

        //通过redis去取该租户的事件id
        String tenantid = UserContext.getTenantId();
        String rediskey = tenantid + "_GZM:";
        Object event = null;
        String eventTypeId = "";
        try {
            event = redisTemplate.opsForValue().get(rediskey);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (event != null) {
            eventTypeId = event.toString();
        } else {
            //redis取不到，进行接口查询
            //通过租户id，查询eventtypeid
            HttpRequest requestEvent = HttpRequest.get(domain + eventTypeUri);
            Map<String, Object> mapEvent = new HashMap<>();
            mapEvent.put("classId", 1);
            mapEvent.put("eventType", "alarm");
            mapEvent.put("scope", "TENANT");
            mapEvent.put("type", "ALARM");
            requestEvent.form(mapEvent);
            requestEvent.header("Authorization", "Bearer " + tokenUtil.getToken());
            requestEvent.header("accept", "application/json");
            requestEvent.header("Content-Type", "application/json");
            requestEvent.header("X-RCD-Tenant-Id", tenantid);
            HttpResponse responseEvent = requestEvent.execute();
            if (responseEvent.getStatus() != HttpStatus.HTTP_OK) {
                log.error("调用平台接口出错" + responseEvent);
                throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
            }
            String resultEvent = responseEvent.body();
            JSONObject jsonObjectEvent = new JSONObject(resultEvent);
            JSONArray arrayEvent = jsonObjectEvent.getJSONArray("payload");

            if (arrayEvent == null || arrayEvent.size() == 0) {
                // 新增eventtypeid 事件
                HttpRequest addEvent = HttpRequest.post(domain + eventTypeUri);
                mapEvent.clear();
                mapEvent.put("alarmMode", "INTERVAL");
                mapEvent.put("classDefine", "新增故障码事件");
                mapEvent.put("classId", 1);
                mapEvent.put("description", "description");
                mapEvent.put("eventType", "alarm");
                mapEvent.put("name", "新增故障码事件");
                mapEvent.put("scope", "TENANT");
                mapEvent.put("type", "ALARM");
                List<Map<String, Object>> eventTypeList = new ArrayList<>();
                eventTypeList.add(mapEvent);
                addEvent.body(JSON.toJSONString(eventTypeList));
                addEvent.header("Authorization", "Bearer " + tokenUtil.getToken());
                addEvent.header("accept", "application/json");
                addEvent.header("Content-Type", "application/json");
                addEvent.header("X-RCD-Tenant-Id", tenantid);
                Log.info("参数-->" + JSON.toJSONString(mapEvent));
                HttpResponse responseAddEvent = addEvent.execute();
                if (responseAddEvent.getStatus() != HttpStatus.HTTP_OK) {
                    log.error("调用平台接口出错" + responseAddEvent);
                    throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
                }

                String resultAddEvent = responseAddEvent.body();
                JSONArray jsonAddEvent = new JSONArray(resultAddEvent);
                if (jsonAddEvent != null && jsonAddEvent.size() > 0) {
                    eventTypeId = jsonAddEvent.getJSONObject(0).getStr("eventTypeId");
                }
            } else {
                eventTypeId = arrayEvent.getJSONObject(0).getStr("eventTypeId");
            }

            if (org.apache.commons.lang3.StringUtils.isNotBlank(eventTypeId)) {
                redisTemplate.opsForValue().set(rediskey, eventTypeId, 10, TimeUnit.DAYS);
            }
        }
        HttpRequest request = HttpRequest.post(domain + codeAlarmUri);
        Map<String, Object> map = handelCodeAlarmParam(deviceAlarmParam, sbxh);
        map.put("eventTypeId", eventTypeId);
        //Log.info("平台故障码报警查询参数-->" + JSON.toJSON(map));
        //request.form(map);
        request.body(JSON.toJSONString(map));
        request.header("Authorization", "Bearer " + tokenUtil.getToken());
        request.header("accept", "application/json");
        request.header("Content-Type", "application/json");
        HttpResponse response = request.execute();
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("调用平台接口出错" + response);
            //throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
        }
        String result = response.body();
        //Log.info("平台故障码报警返回数据-->" + result);
        JSONObject jsonObject = new JSONObject(result);
        JSONObject metadata = (JSONObject) jsonObject.get("metadata");
        String pageTotal = "0";
        if (metadata != null) {
            pageTotal = metadata.get("totalCount").toString();
        }
        //获取故障码数据
        List<NxZskFaultCodeEntity> faultCodeEntities = faultCodeService.queryFaultCodes(deviceAlarmParam.getSeverityLevel(), sbxh);
        Map<String, NxZskFaultCodeEntity> collect = new HashMap<>();
        faultCodeEntities.forEach(o -> {
            String gzm = o.getFaultcodeGzm();
            if (!collect.containsKey(gzm)) {
                collect.put(gzm, o);
            }
        });
        //Log.info("故障码数据-->" + collect);
        //Map<String, NxZskFaultCodeEntity> collect = faultCodeEntities.stream().collect(Collectors.toMap(NxZskFaultCodeEntity::getFaultcodeGzm, Function.identity()));
        JSONArray array = jsonObject.getJSONArray("payload");
        //Log.info("平台故障码报警数据信息查询条数--->" + array.size());
//        LambdaQueryWrapper<NxZskFaultCodeEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(NxZskFaultCodeEntity.class);
//        if(com.baomidou.mybatisplus.core.toolkit.StringUtils.isNotBlank(deviceAlarmParam.getSeverityLevel())){
//            lambdaQueryWrapper.eq(NxZskFaultCodeEntity::getFaultcodeGzdjcode,deviceAlarmParam.getSeverityLevel());
//        }
        array.stream().forEach(temp -> {
            JSONObject obj = (JSONObject) temp;
           /* Log.info("返回数据-->" + obj);
            Log.info("返回数据-->" + obj.get("code").toString());*/
            //时间暂定上报时间  对应平台  originTs  解除时间createTs
            Long times = (Long) obj.get("originTs");
            //SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            //故障描述，等级和设备型号从故障码库中读取，不读平台数据
//            lambdaQueryWrapper.eq(NxZskFaultCodeEntity::getFaultcodeGzm,obj.get("code").toString()).last("limit 1");
//            NxZskFaultCodeEntity code = faultCodeService.getOne(lambdaQueryWrapper);
            NxZskFaultCodeEntity code = collect.get(obj.get("code").toString());
            //Log.info("故障码数据-->" + code);
            // 2022-12-16 增加设备型号匹配  传过来的sbxhId参数与故障码的型号id匹配
            if (code != null) {
                DeviceCodeAlarmModel deviceCodeAlarmModel = new DeviceCodeAlarmModel();
                //deviceCodeAlarmModel.setAlarmTime(sdf.format(DateUtil.timestampToDate(times,"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'")));
                deviceCodeAlarmModel.setAlarmTime(sdf.format(DateUtil.timestampToDate(times, "yyyy-MM-dd HH:mm:ss")));
                deviceCodeAlarmModel.setAlarmCode(obj.get("code") == null ? "" : obj.get("code").toString());
                deviceCodeAlarmModel.setAlarmName(obj.get("alarmName") == null ? "" : obj.get("alarmName").toString());
                deviceCodeAlarmModel.setAlarmDesc(code.getFaultcodeGzms());
                deviceCodeAlarmModel.setAlarmLevel(code.getFaultcodeGzdj());
                deviceCodeAlarmModel.setDeviceType(code.getFaultcodeSblx());
                deviceCodeAlarmModel.setNxZskFaultcodeId(code.getNxZskFaultcodeId());
                list.add(deviceCodeAlarmModel);
            }
//            else {
//                DeviceCodeAlarmModel deviceCodeAlarmModel = new DeviceCodeAlarmModel();
//                //deviceCodeAlarmModel.setAlarmTime(sdf.format(DateUtil.timestampToDate(times,"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'")));
//                deviceCodeAlarmModel.setAlarmTime(sdf.format(DateUtil.timestampToDate(times, "yyyy-MM-dd HH:mm:ss")));
//                deviceCodeAlarmModel.setAlarmCode(obj.get("code").toString());
//                list.add(deviceCodeAlarmModel);
//            }
//            faultCodeEntities.stream().forEach(code->{
//                if(code.getFaultcodeGzm().equals(obj.get("code").toString())){
//                    DeviceCodeAlarmModel deviceCodeAlarmModel = new DeviceCodeAlarmModel();
//                    //deviceCodeAlarmModel.setAlarmTime(sdf.format(DateUtil.timestampToDate(times,"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'")));
//                    deviceCodeAlarmModel.setAlarmTime(sdf.format(DateUtil.timestampToDate(times, "yyyy-MM-dd HH:mm:ss")));
//                    deviceCodeAlarmModel.setAlarmCode(obj.get("code").toString());
//                    deviceCodeAlarmModel.setAlarmDesc(code.getFaultcodeGzms());
//                    deviceCodeAlarmModel.setAlarmLevel(code.getFaultcodeGzdj());
//                    deviceCodeAlarmModel.setDeviceType(code.getFaultcodeSblx());
//                    deviceCodeAlarmModel.setNxZskFaultcodeId(code.getNxZskFaultcodeId());
//                    list.add(deviceCodeAlarmModel);
//                }
//            });

        });
        resultMap.put("pageTotal", pageTotal);
        resultMap.put("deviceAlarmModels", list);
        return resultMap;
    }

    /**
     * app端规则报警查询接口 只查待处理报警
     *
     * @param appDeviceAlarmParam
     * @return
     */
    @Override
    public Map<String, Object> getDeviceRuleAlarmCount(AppDeviceAlarmParam appDeviceAlarmParam) {
        List<AppDeviceAlarmResp> resultList = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String, Object> resultMap = new HashMap<>();
        HttpRequest request = HttpRequest.get(domain + alaramUri);
        //处理入参
        Map<String, Object> map = appAlarmHandleParam(appDeviceAlarmParam);
        request.form(map);
        request.header("Authorization", "Bearer " + tokenUtil.getToken());
        request.header("accept", "application/json");
        request.header("Content-Type", "application/json");
        HttpResponse response = request.execute();
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("调用平台接口出错" + response);
            //throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
        }
        String result = response.body();
        JSONObject jsonObject = new JSONObject(result);
        JSONObject metadata = (JSONObject) jsonObject.get("metadata");
        String pageTotal = "0";
        if (metadata != null) {
            pageTotal = metadata.get("totalCount").toString();
        }
        JSONArray array = jsonObject.getJSONArray("payload");
        List<String> codes = new ArrayList<>();
        List<Integer> pageTotals = new ArrayList<>();
        array.stream().forEach(temp -> {
            JSONObject obj = (JSONObject) temp;
            String code = obj.get("alarmTopicName").toString();
            //统计次数
            if (codes.contains(code)) {
                //更新已放入resultList中对象的次数和最大时间
                resultList.stream().forEach(temp1 -> {
                    //更新次数
                    if (code.equals(temp1.getAlarmCode())) {
                        temp1.setAlarmTimes(temp1.getAlarmTimes() + 1);
                    }
                    //更新最近时间
                    if (DateUtil.getTimesByStringType(obj.get("activeTime").toString()) > (DateUtil.getTimesByStringType(temp1.getAlarmTime()))) {
                        temp1.setAlarmTime(DateUtil.reverseHandlerTimeByType(obj.get("activeTime").toString()));

                    }
                });
                //记录总共合并多少次，在总页数中减去该list的size等于实际总页数
                pageTotals.add(1);
            } else {
                AppDeviceAlarmResp appDeviceAlarmResp = new AppDeviceAlarmResp();
                appDeviceAlarmResp.setAlarmCode(obj.get("alarmTopicName").toString());
                appDeviceAlarmResp.setAlarmDesc(obj.get("subject").toString());
                appDeviceAlarmResp.setAlarmTimes(1);
                appDeviceAlarmResp.setAlarmTime(DateUtil.reverseHandlerTimeByType(obj.get("activeTime").toString()));
                appDeviceAlarmResp.setAlaramLevel(obj.get("severityName").toString());
                resultList.add(appDeviceAlarmResp);
            }
            codes.add(code);
        });
        resultMap.put("pageTotal", resultList.size());
        resultMap.put("deviceAlarmModels", resultList);
        return resultMap;
    }

    /**
     * app端故障码报警信息统计查询
     *
     * @param appDeviceAlarmParam
     * @return
     */

    @Override
    public Map<String, Object> getDeviceCodeAlarmCount(AppDeviceAlarmParam appDeviceAlarmParam) {
        List<AppDeviceAlarmResp> list = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();
        HttpRequest request = HttpRequest.get(domain + codeAlarmUriGet);
        Map<String, Object> map = appAlarmHandleParam(appDeviceAlarmParam);
        request.form(map);
        request.header("Authorization", "Bearer " + tokenUtil.getToken());
        request.header("accept", "application/json");
        request.header("Content-Type", "application/json");
        HttpResponse response = request.execute();
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("调用平台接口出错" + response);
            //throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
        }
        String result = response.body();
        JSONObject jsonObject = new JSONObject(result);
        JSONObject metadata = (JSONObject) jsonObject.get("metadata");
      /*  String pageTotal="0";
        if(metadata!=null){
            pageTotal = metadata.get("totalCount").toString();
        }*/
        //app默认查紧急和重要
        List<String> levels = new ArrayList<>();
        List<Integer> pageTotals = new ArrayList<>();
        levels.add("1");
        levels.add("2");
        List<NxZskFaultCodeEntity> faultCodeEntities = faultCodeService.queryFaultCodesOfApp(levels, appDeviceAlarmParam.getAlarmDesc(), appDeviceAlarmParam.getCode());
        JSONArray array = jsonObject.getJSONArray("payload");
        //SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<String> codes = new ArrayList<>();
        array.stream().forEach(temp -> {
            JSONObject obj = (JSONObject) temp;
            //统计次数和最近时间
            String code = obj.get("code").toString();
            //时间暂定上报时间  对应平台createTs
            Long time = (Long) obj.get("createTs");
            if (codes.contains(code)) {
                //更新已放入resultList中对象的次数和最大时间
                list.stream().forEach(temp1 -> {
                    //更新次数
                    if (code.equals(temp1.getAlarmCode())) {
                        temp1.setAlarmTimes(temp1.getAlarmTimes() + 1);
                    }
                    //更新最近时间
                    if (time > (DateUtil.getTimesByString(temp1.getAlarmTime()))) {
                        temp1.setAlarmTime(sdf.format(DateUtil.timestampToDate(time, "yyyy-MM-dd HH:mm:ss")));
                    }
                });
                //记录总共合并多少次，在总页数中减去该list的size等于实际总页数
                pageTotals.add(1);
            } else {
                //故障描述，等级和设备型号从故障码库中读取，不读平台数据
                faultCodeEntities.stream().forEach(tempCode -> {
                    if (tempCode.getFaultcodeGzm().equals(obj.get("code").toString())) {
                        AppDeviceAlarmResp appDeviceAlarmResp = new AppDeviceAlarmResp();
                        appDeviceAlarmResp.setAlarmTime(sdf.format(DateUtil.timestampToDate(time, "yyyy-MM-dd HH:mm:ss")));
                        appDeviceAlarmResp.setAlarmCode(obj.get("code").toString());
                        appDeviceAlarmResp.setAlarmDesc(tempCode.getFaultcodeGzms());
                        appDeviceAlarmResp.setAlaramLevel(tempCode.getFaultcodeGzdj());
                        appDeviceAlarmResp.setAlarmTimes(1);
                        list.add(appDeviceAlarmResp);
                    }
                });
            }
            codes.add(code);
        });
        resultMap.put("pageTotal", list.size());
        resultMap.put("deviceAlarmModels", list);
        return resultMap;
    }


    @Override
    public Map<String, Object> getDeviceAllAlarmCount(AppDeviceAlarmParam appDeviceAlarmParam) {
        List<AppDeviceAlarmResp> list = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();
        HttpRequest request = HttpRequest.get(domain + codeAlarmUriGet);
        Map<String, Object> map = appAlarmHandleParam(appDeviceAlarmParam);
        request.form(map);
        request.header("Authorization", "Bearer " + tokenUtil.getToken());
//        request.header("AuthHeaderAlias", tokenUtil.superToken());
//        request.header("X-RCD-Tenant-Id",UserContext.getTenantId());
        request.header("accept", "application/json");
        request.header("Content-Type", "application/json");
        HttpResponse response = request.execute();
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("调用平台接口出错" + response);
            //throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
        }
        String result = response.body();
        JSONObject jsonObject = new JSONObject(result);
        JSONObject metadata = (JSONObject) jsonObject.get("metadata");
      /*  String pageTotal="0";
        if(metadata!=null){
            pageTotal = metadata.get("totalCount").toString();
        }*/
        //app默认查紧急和重要
        List<String> levels = new ArrayList<>();
        List<Integer> pageTotals = new ArrayList<>();
        levels.add("1");
        levels.add("2");
        List<NxZskFaultCodeEntity> faultCodeEntities = faultCodeService.queryFaultCodesOfApp(levels, appDeviceAlarmParam.getAlarmDesc(), appDeviceAlarmParam.getCode());
        JSONArray array = jsonObject.getJSONArray("payload");
        //SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<String> codes = new ArrayList<>();
        array.stream().forEach(temp -> {
            JSONObject obj = (JSONObject) temp;
            //统计次数和最近时间
            String code = obj.get("code").toString();
            //时间暂定上报时间  对应平台createTs
            Long time = (Long) obj.get("createTs");
            if (codes.contains(code)) {
                //更新已放入resultList中对象的次数和最大时间
                list.stream().forEach(temp1 -> {
                    //更新次数
                    if (code.equals(temp1.getAlarmCode())) {
                        temp1.setAlarmTimes(temp1.getAlarmTimes() + 1);
                    }
                    //更新最近时间
                    if (time > (DateUtil.getTimesByString(temp1.getAlarmTime()))) {
                        temp1.setAlarmTime(sdf.format(DateUtil.timestampToDate(time, "yyyy-MM-dd HH:mm:ss")));
                    }
                });
                //记录总共合并多少次，在总页数中减去该list的size等于实际总页数
                pageTotals.add(1);
            } else {
                //故障描述，等级和设备型号从故障码库中读取，不读平台数据
                faultCodeEntities.stream().forEach(tempCode -> {
                    if (tempCode.getFaultcodeGzm().equals(obj.get("code").toString())) {
                        AppDeviceAlarmResp appDeviceAlarmResp = new AppDeviceAlarmResp();
                        appDeviceAlarmResp.setAlarmTime(sdf.format(DateUtil.timestampToDate(time, "yyyy-MM-dd HH:mm:ss")));
                        appDeviceAlarmResp.setAlarmCode(obj.get("code").toString());
                        appDeviceAlarmResp.setAlarmDesc(tempCode.getFaultcodeGzms());
                        appDeviceAlarmResp.setAlaramLevel(tempCode.getFaultcodeGzdj());
                        appDeviceAlarmResp.setAlarmTimes(1);
                        list.add(appDeviceAlarmResp);
                    }
                });
            }
            codes.add(code);
        });
        resultMap.put("pageTotal", list.size());
        resultMap.put("deviceAlarmModels", list);
        return resultMap;
    }


    /**
     * 故障码处理入参
     *
     * @param deviceAlarmParam
     * @return
     */
    public Map<String, Object> handelCodeAlarmParam(DeviceAlarmParam deviceAlarmParam, String sbxh) {
        Map<String, Object> map = new HashMap<>();
        //待处理报警
        if ("1".equals(deviceAlarmParam.getTag())) {
            map.put("active", true);
        } else {//历史报警
            map.put("active", false);
        }
        List<String> thingIds = new ArrayList<>();
        thingIds.add(deviceAlarmParam.getDeviceId());
        map.put("thingIds", thingIds);
        if (!StringUtils.isEmpty(deviceAlarmParam.getCode())) {
            map.put("code", deviceAlarmParam.getCode());
        } else {
            if (!StringUtils.isEmpty(deviceAlarmParam.getSeverityLevel())) {
                //按级别,设备型号查询
                Map<String, Object> maps = new HashMap<>();
                maps.put("level", deviceAlarmParam.getSeverityLevel());
                maps.put("sbxh", sbxh);
                List<String> codes = nxZskFaultCodeMapper.getCodes(maps);
                map.put("codes", codes);
            }

        }
        map.put("minActiveTs", DateUtil.getTimesByString(deviceAlarmParam.getTime().split("~")[0]));
        map.put("maxActiveTs", DateUtil.getTimesByString(deviceAlarmParam.getTime().split("~")[1]));

        //分页信息处理
        map.put("_skip", deviceAlarmParam.getPageSize() * (deviceAlarmParam.getCurrentPage() - 1));
        map.put("_limit", deviceAlarmParam.getPageSize());
        //按时间降序
        //map.put("sort",":[{\"field\":\"createdTime\",\"sortType\":\"DESC\"}]");
        map.put("_includeMetadata", true);
        return map;
    }

    /**
     * 规则报警入参处理
     */
    public Map<String, Object> handleRuleAlarmParam(DeviceAlarmParam deviceAlarmParam) {
        Map<String, Object> map = new HashMap<>();
        map.put("thingIds", "[\"" + deviceAlarmParam.getDeviceId() + "\"]");
        //时间
        if (!StringUtils.isEmpty(deviceAlarmParam.getTime())) {
            String[] times = deviceAlarmParam.getTime().split("~");
            map.put("startTime", DateUtil.handlerTime(times[0]));
            map.put("endTime", DateUtil.handlerTime(times[1]));
        }
        if (!StringUtils.isEmpty(deviceAlarmParam.getSeverityLevel())) {
            int[] level = {Integer.parseInt(deviceAlarmParam.getSeverityLevel())};
            map.put("severityLevels", JSON.toJSON(level).toString());
        }
     /*   //报警级别
        if(!StringUtils.isEmpty(deviceAlarmParam.getSeverityLevel())){
            map.put("severityLevels", "["+deviceAlarmParam.getSeverityLevel()+"]");
        }*/
        //待处理报警只查持续中(close=false&&active=true)的，历史报警查所有状态的  俊全已确认
        if ("1".equals(deviceAlarmParam.getTag())) {
            map.put("closed", false);
            map.put("active", true);
        }
        //分页信息处理
        map.put("skip", deviceAlarmParam.getPageSize() * (deviceAlarmParam.getCurrentPage() - 1));
        map.put("limit", deviceAlarmParam.getPageSize());
        map.put("sort", "[{\"field\":\"createdTime\",\"sortType\":\"DESC\"}]");
        map.put("includeMetadata", true);
        log.info("请求入参==>{}", map.toString());
        return map;
    }

    /**
     * app端规则报警入参处理
     */
    public Map<String, Object> appAlarmHandleParam(AppDeviceAlarmParam appDeviceAlarmParam) {
        Map<String, Object> map = new HashMap<>();
        if (org.apache.commons.lang3.StringUtils.isNotBlank(appDeviceAlarmParam.getStartTime())
                && org.apache.commons.lang3.StringUtils.isNotBlank(appDeviceAlarmParam.getEndTime())) {
            //规则报警
            if ("1".equals(appDeviceAlarmParam.getAlarmType())) {
                String stTime = DateUtil.handlerTime(appDeviceAlarmParam.getStartTime());
                String enTime = DateUtil.handlerTime(appDeviceAlarmParam.getEndTime());
                map.put("startTime", stTime);
                map.put("endTime", enTime);
            } else {
                map.put("minActiveTs", DateUtil.getTimesByString(appDeviceAlarmParam.getStartTime()));
                map.put("maxActiveTs", DateUtil.getTimesByString(appDeviceAlarmParam.getEndTime()));
            }
        }
        //规则报警
        if ("1".equals(appDeviceAlarmParam.getAlarmType())) {
            //故障描述
            if (org.apache.commons.lang.StringUtils.isNotBlank(appDeviceAlarmParam.getAlarmDesc())) {
                map.put("subject", appDeviceAlarmParam.getAlarmDesc());
            }
            if (org.apache.commons.lang.StringUtils.isNotBlank(appDeviceAlarmParam.getCode())) {
                map.put("alarmTopicName", appDeviceAlarmParam.getCode());
            }
            //map.put("subject",appDeviceAlarmParam.getAlarmDesc());
            //map.put("alarmTopicName",appDeviceAlarmParam.getCode());
            map.put("thingIds", "[\"" + appDeviceAlarmParam.getDeviceId() + "\"]");
            map.put("severityLevels", "[1,2]");
            //app只查待处理
            map.put("closed", false);
            map.put("active", true);
            map.put("skip", appDeviceAlarmParam.getPageSize() * (appDeviceAlarmParam.getCurrentPage() - 1));
            map.put("limit", appDeviceAlarmParam.getPageSize());
            map.put("includeMetadata", true);
            map.put("sort", "[{\"field\":\"createdTime\",\"sortType\":\"DESC\"}]");
        }//故障码报警
        else {
            map.put("thingId", appDeviceAlarmParam.getDeviceId());
            //map.put("code",appDeviceAlarmParam.getCode());
            map.put("active", true);
            map.put("_skip", appDeviceAlarmParam.getPageSize() * (appDeviceAlarmParam.getCurrentPage() - 1));
            map.put("_limit", appDeviceAlarmParam.getPageSize());
            map.put("_includeMetadata", true);
        }
        log.info("请求入参==>{}", map.toString());
        return map;
    }

    /**
     * 根据设备ID查询设备报警总数据（故障码+规则）
     */
    @Override
    public Integer getDeviceAlarmCount(String deviceId, String startTime, String endTime) {
        AppDeviceAlarmParam appDeviceAlarmParam = new AppDeviceAlarmParam();
        appDeviceAlarmParam.setDeviceId(deviceId);
        appDeviceAlarmParam.setStartTime(startTime);
        appDeviceAlarmParam.setEndTime(endTime);
        appDeviceAlarmParam.setCurrentPage(1);
        appDeviceAlarmParam.setPageSize(10000);
        appDeviceAlarmParam.setAlarmType("1");
        Map<String, Object> deviceRuleAlarmCount = getDeviceRuleAlarmCount(appDeviceAlarmParam);
        appDeviceAlarmParam.setAlarmType("2");
        Map<String, Object> deviceCodeAlarmCount = getDeviceCodeAlarmCount(appDeviceAlarmParam);
        Object pageTotal = deviceRuleAlarmCount.get("pageTotal");
        Object pageTotal1 = deviceCodeAlarmCount.get("pageTotal");
        int i = Integer.parseInt(pageTotal.toString()) + Integer.valueOf(pageTotal1.toString());
        return i;
        /*//规则报警
        HttpRequest ruleAlarmRequest = HttpRequest.get(domain + alaramUri);
        Map<String,Object> ruleAlarmMap = new HashMap<>();
        ruleAlarmMap.put("thingIds", "[\""+deviceId+"\"]");
        ruleAlarmMap.put("includeMetadata",true);
        ruleAlarmMap.put("severityLevels","[1,2]");
        if(org.apache.commons.lang3.StringUtils.isNotBlank(startTime)
                && org.apache.commons.lang3.StringUtils.isNotBlank(endTime)){
            String stTime = DateUtil.handlerTime(startTime);
            String enTime = DateUtil.handlerTime(endTime);
            ruleAlarmMap.put("startTime", stTime);
            ruleAlarmMap.put("endTime", enTime);
        }
        //app只查待处理
        ruleAlarmMap.put("closed",false);
        ruleAlarmMap.put("active",true);
        log.info("规则报警请求入参==>{}",ruleAlarmMap.toString());
        ruleAlarmRequest.form(ruleAlarmMap);
        ruleAlarmRequest.header("Authorization", "Bearer " + tokenUtil.getToken());
        ruleAlarmRequest.header("accept", "application/json");
        ruleAlarmRequest.header("Content-Type", "application/json");
        HttpResponse ruleAlarmResponse = ruleAlarmRequest.execute();
        if(ruleAlarmResponse.getStatus()!= HttpStatus.HTTP_OK){
            log.error("调用平台接口出错"+ruleAlarmResponse);
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
        }
        String ruleAlarmResult = ruleAlarmResponse.body();
        JSONObject ruleAlarmJson = new JSONObject(ruleAlarmResult);
        JSONObject ruleAlarmMetadata = (JSONObject)ruleAlarmJson.get("metadata");
        Integer ruleAlarmCount=0;
        if(ruleAlarmMetadata!=null){
            ruleAlarmCount = Integer.parseInt(ruleAlarmMetadata.get("totalCount").toString());
        }
        //故障码报警
        HttpRequest codeAlarmRequest = HttpRequest.get(domain + codeAlarmUri);
        Map<String, Object> codeAlarmMap = new HashMap<>();
        codeAlarmMap.put("thingId", deviceId);
        codeAlarmMap.put("active",true);
        codeAlarmMap.put("_includeMetadata",true);
        if(org.apache.commons.lang3.StringUtils.isNotBlank(startTime)
                && org.apache.commons.lang3.StringUtils.isNotBlank(endTime)){
            codeAlarmMap.put("minActiveTs",DateUtil.getTimesByString(startTime));
            codeAlarmMap.put("maxActiveTs",DateUtil.getTimesByString(endTime));
        }
        log.info("故障码报警请求入参==>{}",codeAlarmMap.toString());
        codeAlarmRequest.form(codeAlarmMap);
        codeAlarmRequest.header("Authorization", "Bearer " + tokenUtil.getToken());
        codeAlarmRequest.header("accept", "application/json");
        codeAlarmRequest.header("Content-Type", "application/json");
        HttpResponse response = codeAlarmRequest.execute();
        if(response.getStatus()!= HttpStatus.HTTP_OK){
            log.error("调用平台接口出错"+response);
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
        }
        String codeAlarmResult = response.body();
        JSONObject codeAlarmJson = new JSONObject(codeAlarmResult);
        JSONObject codeAlarmMetadata = (JSONObject)codeAlarmJson.get("metadata");
        Integer codeAlarmCount=0;
        if(codeAlarmMetadata!=null){
            codeAlarmCount = Integer.parseInt(codeAlarmMetadata.get("totalCount").toString());
        }

        return ruleAlarmCount+codeAlarmCount;*/
    }

    @Override
    public List<MyMessageEntity> queryAlarmMessages(List<DeviceListEntity> deviceListEntities) {
        //4.0返回结果集
        List<MyMessageEntity> resultList = new ArrayList<>();
        //筛选代办库后结果集
        List<MyMessageEntity> mlist = new ArrayList<>();
        //筛选紧急数据
        List<MyMessageEntity> jjlist = new ArrayList<>();
        //需要获取掉哪些非4.0的数据   才能调用4.0接口，返回报设备不符
        List<DeviceListEntity> filter_deviceListEntities = deviceListEntities.stream().filter(item -> "1".equals(item.getSblbLwztCode())).collect(Collectors.toList());
        //规则报警
        if (filter_deviceListEntities.size() > 0) {
            List<String> sbbm = filter_deviceListEntities.stream().map(DeviceListEntity::getNxJcxxSblbId).collect(Collectors.toList());

            //通过redis去取该租户的事件id
            String tenantid = UserContext.getTenantId();
            String rediskey = tenantid + "_GZM:";
            Object event = null;
            String eventTypeId = "";
            try {
                event = redisTemplate.opsForValue().get(rediskey);
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (event != null) {
                eventTypeId = event.toString();
            } else {
                //redis取不到，进行接口查询
                //通过租户id，查询eventtypeid
                HttpRequest requestEvent = HttpRequest.get(domain + eventTypeUri);
                Map<String, Object> mapEvent = new HashMap<>();
                mapEvent.put("classId", 1);
                mapEvent.put("eventType", "alarm");
                mapEvent.put("scope", "TENANT");
                mapEvent.put("type", "ALARM");
                mapEvent.put("X-RCD-Tenant-Id", tenantid);
                requestEvent.form(mapEvent);
                requestEvent.header("Authorization", "Bearer " + tokenUtil.getToken());
                requestEvent.header("accept", "application/json");
                requestEvent.header("Content-Type", "application/json");
                HttpResponse responseEvent = requestEvent.execute();
                if (responseEvent.getStatus() != HttpStatus.HTTP_OK) {
                    log.error("调用平台接口出错" + responseEvent);
                    throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
                }
                String resultEvent = responseEvent.body();
                JSONObject jsonObjectEvent = new JSONObject(resultEvent);
                JSONArray arrayEvent = jsonObjectEvent.getJSONArray("payload");
                //查询不到eventtypeid
                if (arrayEvent == null || arrayEvent.size() == 0) {
                    // 新增eventtypeid 事件
                    HttpRequest addEvent = HttpRequest.post(domain + eventTypeUri);
                    mapEvent.clear();
                    mapEvent.put("alarmMode", "INTERVAL");
                    mapEvent.put("classDefine", "新增故障码事件");
                    mapEvent.put("classId", 1);
                    mapEvent.put("description", "description");
                    mapEvent.put("eventType", "alarm");
                    mapEvent.put("name", "新增故障码事件");
                    mapEvent.put("scope", "TENANT");
                    mapEvent.put("type", "ALARM");
                    mapEvent.put("X-RCD-Tenant-Id", tenantid);
                    List<Map<String, Object>> eventTypeList = new ArrayList<>();
                    eventTypeList.add(mapEvent);
                    addEvent.body(JSON.toJSONString(eventTypeList));
                    addEvent.header("Authorization", "Bearer " + tokenUtil.getToken());
                    addEvent.header("accept", "application/json");
                    addEvent.header("Content-Type", "application/json");
                    Log.info("参数-->" + JSON.toJSONString(mapEvent));
                    HttpResponse responseAddEvent = addEvent.execute();
                    if (responseAddEvent.getStatus() != HttpStatus.HTTP_OK) {
                        log.error("调用平台接口出错" + responseAddEvent);
                        throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
                    }
                    String resultAddEvent = responseAddEvent.body();
                    JSONArray jsonAddEvent = new JSONArray(resultAddEvent);
                    if (jsonAddEvent != null && jsonAddEvent.size() > 0) {
                        eventTypeId = jsonAddEvent.getJSONObject(0).getStr("eventTypeId");
                    }
                } else {
                    eventTypeId = arrayEvent.getJSONObject(0).getStr("eventTypeId");
                }

                if (org.apache.commons.lang3.StringUtils.isNotBlank(eventTypeId)) {
                    redisTemplate.opsForValue().set(rediskey, eventTypeId, 3500, TimeUnit.SECONDS);
                }
            }

            //故障码报警
            HttpRequest codeAlarmRequest = HttpRequest.post(domain + codeAlarmBatchUri);
            Map<String, Object> codeAlarmMap = new HashMap<>();
            codeAlarmMap.put("eventTypeId", eventTypeId);
            codeAlarmMap.put("thingIds", sbbm);
            codeAlarmMap.put("level", 1);
            codeAlarmMap.put("active", true);
            codeAlarmMap.put("minActiveTs", (DateUtil.rollDay(new Date(), -3)).getTime());
            codeAlarmMap.put("maxActiveTs", new Date().getTime());
            codeAlarmRequest.form(codeAlarmMap);
            codeAlarmRequest.header("Authorization", "Bearer " + tokenUtil.getToken());
//        codeAlarmRequest.header("AuthHeaderAlias", tokenUtil.superToken());
//        codeAlarmRequest.header("X-RCD-Tenant-Id",UserContext.getTenantId());
            codeAlarmRequest.header("accept", "application/json");
            codeAlarmRequest.header("Content-Type", "application/json");

            String jsonString = JSON.toJSONString(codeAlarmMap);
            //Log.info("【查询报警信息】请求参数 ==> {}", jsonString);
            codeAlarmRequest.body(jsonString);
            HttpResponse codeAlarmResponse = codeAlarmRequest.execute();
            if (codeAlarmResponse.getStatus() != HttpStatus.HTTP_OK) {
                log.error("调用平台接口出错" + codeAlarmResponse);
                //throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
                throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
            }
            String codeAlarmResult = codeAlarmResponse.body();
            //Log.info("【查询报警信息】返回值 ==> {}", codeAlarmResult);
            JSONObject codeAlarmJson = new JSONObject(codeAlarmResult);
            JSONArray codeAlarmArray = codeAlarmJson.getJSONArray("payload");
           /* List<String> levels = new ArrayList<>();
            levels.add("1");
            levels.add("2");
            List<NxZskFaultCodeEntity> faultCodeEntities = faultCodeService.queryFaultCodesOfMessage(levels);*/
            //SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            codeAlarmArray.stream().forEach(temp -> {
                JSONObject ruleAlarm = (JSONObject) temp;
                MyMessageEntity myMessageEntity = new MyMessageEntity();
                myMessageEntity.setMessageClass(ResourceUtil.getI18nMessage("60003"));
                /*Optional<NxZskFaultCodeEntity> nxZskFaultCodeEntityOptional = faultCodeEntities.stream().filter(o -> Objects.equals(o.getFaultcodeGzm(),ruleAlarm.get("code").toString())).findFirst();
                if(nxZskFaultCodeEntityOptional.isPresent()) { //判空
                    NxZskFaultCodeEntity nxZskFaultCodeEntity = nxZskFaultCodeEntityOptional.get();
                    myMessageEntity.setMessageName(nxZskFaultCodeEntity.getFaultcodeGzm());
                    myMessageEntity.setMessageContent(nxZskFaultCodeEntity.getFaultcodeGzms());
                    myMessageEntity.setGzdj(nxZskFaultCodeEntity.getFaultcodeGzdj());
                }*/
               /* faultCodeEntities.stream().forEach(temp1 -> {
                    if (temp1.getFaultcodeGzm().equals(ruleAlarm.get("code").toString())) {
                        myMessageEntity.setMessageName(temp1.getFaultcodeGzm());
                        myMessageEntity.setMessageContent(temp1.getFaultcodeGzms());
                    }
                });*/
                myMessageEntity.setMessageObjectCode(getDeviceCode(ruleAlarm.get("thingId").toString(), filter_deviceListEntities).getSblbSbbm());
                myMessageEntity.setMessageObjectName(getDeviceCode(ruleAlarm.get("thingId").toString(), filter_deviceListEntities).getSblbSbmc());
                myMessageEntity.setTypeCode(ServicePlanEnums.MessageType.FAULT_ALARM.getCode());
                myMessageEntity.setAdviceCode(ServicePlanEnums.MessageCodeType.CONDITION_ALARM.getCode());
                myMessageEntity.setSignData(getDeviceCode(ruleAlarm.get("thingId").toString(), filter_deviceListEntities).getSblbSbbm());
                myMessageEntity.setSbxh(getDeviceCode(ruleAlarm.get("thingId").toString(), filter_deviceListEntities).getSblbSbxhCode());
                myMessageEntity.setBjbm(ruleAlarm.get("code").toString());//报警编码
                //myMessageEntity.setGzdj(ruleAlarm.get("level").toString());//故障等级
                //20221031  根据故障码查询故障描述
                /*LambdaQueryWrapper<NxZskFaultCodeEntity> nxzsOne = new LambdaQueryWrapper<>();
                nxzsOne.eq(NxZskFaultCodeEntity::getFaultcodeGzm, myMessageEntity.getBjbm());
                nxzsOne.eq(NxZskFaultCodeEntity::getSyZhid, tenantid);
                List<NxZskFaultCodeEntity> nxZskFaultCodeEntityList = faultCodeService.list(nxzsOne);
                if (nxZskFaultCodeEntityList.size() > 0) {
                    NxZskFaultCodeEntity nxZskFaultCodeEntity = nxZskFaultCodeEntityList.get(0);
                    if (nxZskFaultCodeEntity != null) {
                        myMessageEntity.setMessageContent(nxZskFaultCodeEntity.getFaultcodeGzms());
                    }
                }*/
                //String ts = ruleAlarm.get("ts").toString();
                //long timeLong = Long.parseLong(ts);
                myMessageEntity.setBjsj(ruleAlarm.get("ts").toString());//报警时间
                resultList.add(myMessageEntity);
                /*List<NxTodoLibrary> nxTodoLibraries = nxTodoLibraryService.selectBySign(myMessageEntity.getAdviceCode(), myMessageEntity.getSignData(), myMessageEntity.getMessageTime());
                if (nxTodoLibraries.isEmpty()) {
                    resultList.add(myMessageEntity);
                }*/
            });

            //查询历史报警，满足24小时的数据
            codeAlarmMap.clear();
            codeAlarmMap.put("thingIds", sbbm);
            codeAlarmMap.put("level", 1);
            codeAlarmMap.put("active", false);
            codeAlarmMap.put("minActiveTs", (DateUtil.rollDay(new Date(), -1)).getTime());
            codeAlarmMap.put("maxActiveTs", new Date().getTime());
            codeAlarmRequest.form(codeAlarmMap);
            codeAlarmRequest.header("Authorization", "Bearer " + tokenUtil.getToken());
            codeAlarmRequest.header("accept", "application/json");
            codeAlarmRequest.header("Content-Type", "application/json");
            jsonString = JSON.toJSONString(codeAlarmMap);
            //Log.info("【查询历史报警信息】请求参数 ==> {}", jsonString);
            //Log.info("【查询历史报警信息】请求开始时间 ==> {}", DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL));
            codeAlarmRequest.body(jsonString);
            codeAlarmResponse = codeAlarmRequest.execute();
            if (codeAlarmResponse.getStatus() != HttpStatus.HTTP_OK) {
                Log.error("调用平台接口出错" + codeAlarmResponse);
                throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
            }

            String codeAlarmResult_h = codeAlarmResponse.body();
            JSONObject codeAlarmJson_h = new JSONObject(codeAlarmResult_h);
            JSONArray codeAlarmArray_h = codeAlarmJson_h.getJSONArray("payload");
            //Log.info("【查询历史报警信息】请求结束时间 ==> {}", DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL));
            codeAlarmArray_h.stream().forEach(temp -> {
                JSONObject ruleAlarm = (JSONObject) temp;

                MyMessageEntity myMessageEntity = new MyMessageEntity();
                myMessageEntity.setMessageClass(ResourceUtil.getI18nMessage("60003"));
                /*Optional<NxZskFaultCodeEntity> nxZskFaultCodeEntityOptional = faultCodeEntities.stream().filter(o -> Objects.equals(o.getFaultcodeGzm(),ruleAlarm.get("code").toString())).findFirst();
                if(nxZskFaultCodeEntityOptional.isPresent()) { //判空
                    NxZskFaultCodeEntity nxZskFaultCodeEntity = nxZskFaultCodeEntityOptional.get();
                    myMessageEntity.setMessageName(nxZskFaultCodeEntity.getFaultcodeGzm());
                    myMessageEntity.setMessageContent(nxZskFaultCodeEntity.getFaultcodeGzms());
                    myMessageEntity.setGzdj(nxZskFaultCodeEntity.getFaultcodeGzdj());
                }*/
                /*faultCodeEntities.stream().forEach(temp1 -> {
                    if (temp1.getFaultcodeGzm().equals(ruleAlarm.get("code").toString())) {
                        myMessageEntity.setMessageName(temp1.getFaultcodeGzm());
                        myMessageEntity.setMessageContent(temp1.getFaultcodeGzms());
                    }
                });*/
                myMessageEntity.setMessageObjectCode(getDeviceCode(ruleAlarm.get("thingId").toString(), filter_deviceListEntities).getSblbSbbm());
                myMessageEntity.setMessageObjectName(getDeviceCode(ruleAlarm.get("thingId").toString(), filter_deviceListEntities).getSblbSbmc());
                myMessageEntity.setTypeCode(ServicePlanEnums.MessageType.FAULT_ALARM.getCode());
                myMessageEntity.setAdviceCode(ServicePlanEnums.MessageCodeType.CONDITION_ALARM.getCode());
                myMessageEntity.setSignData(getDeviceCode(ruleAlarm.get("thingId").toString(), filter_deviceListEntities).getSblbSbbm());
                myMessageEntity.setBjbm(ruleAlarm.get("code").toString());//报警编码
                //myMessageEntity.setGzdj(ruleAlarm.get("level").toString());//故障等级
                myMessageEntity.setBjsj(ruleAlarm.get("ts").toString());//报警时间
                myMessageEntity.setSbxh(getDeviceCode(ruleAlarm.get("thingId").toString(), filter_deviceListEntities).getSblbSbxhCode());
                /*LambdaQueryWrapper<NxZskFaultCodeEntity> nxzsOne = new LambdaQueryWrapper<>();
                nxzsOne.eq(NxZskFaultCodeEntity::getFaultcodeGzm, myMessageEntity.getBjbm());
                nxzsOne.eq(NxZskFaultCodeEntity::getSyZhid, tenantid);
                List<NxZskFaultCodeEntity> nxZskFaultCodeEntityList = faultCodeService.list(nxzsOne);
                if (nxZskFaultCodeEntityList.size() > 0) {
                    NxZskFaultCodeEntity nxZskFaultCodeEntity = nxZskFaultCodeEntityList.get(0);
                    if (nxZskFaultCodeEntity != null) {
                        myMessageEntity.setMessageContent(nxZskFaultCodeEntity.getFaultcodeGzms());
                    }
                }*/
                resultList.add(myMessageEntity);
                /*List<NxTodoLibrary> nxTodoLibraries = nxTodoLibraryService.selectBySign(myMessageEntity.getAdviceCode(), myMessageEntity.getSignData(), myMessageEntity.getMessageTime());
                if (nxTodoLibraries.isEmpty()) {
                    resultList.add(myMessageEntity);
                }*/
            });
            //已存在代办库的数据不显示在页面
            List<String> lists = resultList.stream().map(MyMessageEntity::getSignData).collect(Collectors.toList());
            if (lists.size() > 0) {
                List<String> signDatas = nxTodoLibraryService.selectByData(ServicePlanEnums.MessageCodeType.CONDITION_ALARM.getCode(), lists);
                resultList.forEach(o -> {
                    if (!signDatas.contains(o.getSignData())) {
                        mlist.add(o);
                    }
                });
            }
            // 根据故障码查询故障描述
            List<String> bjbmList = mlist.stream().map(MyMessageEntity::getBjbm).collect(Collectors.toList());
            List<NxZskFaultCodeEntity> nxZskFaultCodeEntityList = faultCodeService.selectData(tenantid, bjbmList);
            mlist.forEach(o -> {
                Optional<NxZskFaultCodeEntity> infoOptional = nxZskFaultCodeEntityList.stream().
                        filter(a -> (Objects.equals(a.getFaultcodeGzm(), o.getBjbm()) && Objects.equals(a.getFaultcodeSbxhid(),o.getSbxh()))).findFirst();
                if (infoOptional.isPresent()) {
                    NxZskFaultCodeEntity nxZskFaultCodeEntity = infoOptional.get();
                    //只要紧急数据
                    /*if ("1".equals(nxZskFaultCodeEntity.getFaultcodeGzdjcode())) {
                        o.setMessageContent(nxZskFaultCodeEntity.getFaultcodeGzms());
                        o.setMessageName(nxZskFaultCodeEntity.getFaultcodeGzm());
                        o.setGzdj(nxZskFaultCodeEntity.getFaultcodeGzdj());
                        jjlist.add(o);
                    }*/
                    o.setMessageContent(nxZskFaultCodeEntity.getFaultcodeGzms());
                    o.setMessageName(nxZskFaultCodeEntity.getFaultcodeGzm());
                    if(nxZskFaultCodeEntity.getFaultcodeGzdj().equals("紧急")) {
                        o.setGzdj(nxZskFaultCodeEntity.getFaultcodeGzdj());
                        jjlist.add(o);
                    }
                }
            });
            //Log.info("数据处理结束时间 ==> {}", DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL));
            if (jjlist.size() > 0) {
                jjlist.stream().sorted(Comparator.comparing(MyMessageEntity::getBjsj).reversed()).collect(Collectors.toList());
            }
        } else {
            Log.error("调用平台接口出错，没有查询满足4.0设备的数据");
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
        }
        return jjlist;
    }

    public DeviceListEntity getDeviceCode(String deviceId, List<DeviceListEntity> deviceListEntities) {
        List<String> result = new ArrayList<>();
        /*for (DeviceListEntity temp : deviceListEntities) {
            if (deviceId.equals(temp.getNxJcxxSblbId())) {
                result.add(temp.getSblbSbbm());
                result.add(temp.getSblbSbmc());
                break;
            }
        }*/
        Optional<DeviceListEntity> info = deviceListEntities.stream().filter(o -> Objects.equals(o.getNxJcxxSblbId(), deviceId)).findFirst();
        if (info.isPresent()) {
            return info.get();
        }
        return null;
    }
}
