package com.rootcloud.domain.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.RootCloudOpenAPIUtil;
import com.rootcloud.common.constant.DeviceConstant;
import com.rootcloud.common.constant.RoleIdConstant;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.common.util.GaoDeAPIUtils;
import com.rootcloud.config.auth.LoginUserInfo;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.auth.UserInfo;
import com.rootcloud.common.util.GaoDeAPIUtils;
import com.rootcloud.domain.dto.DeviceDTO;
import com.rootcloud.domain.dto.DeviceSpecialDTO;
import com.rootcloud.domain.dto.DeviceWorkTimeDTO;
import com.rootcloud.domain.entity.SiteDTO;
import com.rootcloud.domain.entity.device.DeviceEntity;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.entity.device.NxDeviceLocationEntity;
import com.rootcloud.domain.mapper.DeviceMapper;
import com.rootcloud.domain.mapper.NxDeviceLocationMapper;
import com.rootcloud.domain.mapper.NxDeviceMapper;
import com.rootcloud.domain.service.device.DeviceService;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import com.rootcloud.domain.service.gkjk.NxJeDeviceService;
import com.rootcloud.vo.device.*;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * TODO
 *
 * @author wei.liao
 * @version 1.0
 * @date 2022/4/18 14:18
 */
@Slf4j
@Service("deviceServiceImpl")
public class DeviceServiceImpl extends ServiceImpl<DeviceMapper, DeviceEntity> implements DeviceService {


    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private NxDeviceMapper nxDeviceMapper;

    @Autowired
    private RootCloudOpenAPIUtil rootCloudOpenAPIUtil;

    @Autowired
    private GaoDeAPIUtils gaoDeAPIUtils;

    @Autowired
    private NxDeviceLocationMapper deviceLocationMapper;

    @Resource(name = "nxJeDeviceServiceImpl")
    private NxJeDeviceService deviceService;

    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;

    @Override
    public JsonResult<DeviceView> display() {
        DeviceView deviceView = new DeviceView();
        //查询所有的设备信息
        //设备详情：设备编号、地址详情、定位时间、在线时间、在线时长、设备别名、设备监控、设备在线状态
        List<DeviceInfo> deviceInfoList = new ArrayList<>();

        //查询设备位置
        List<NxDeviceLocationEntity> allDeviceLocation = deviceLocationMapper.findAllDeviceLocation();
        Map<String, NxDeviceLocationEntity> deviceLocationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(allDeviceLocation)) {
            deviceLocationMap = allDeviceLocation.stream().collect(Collectors.toMap(NxDeviceLocationEntity::getDeviceId, Function.identity()));
        }

        List<DeviceDTO> deviceDTOList = deviceDTOListByRole();
        if (CollectionUtils.isEmpty(deviceDTOList)) {
            return JsonResult.clientFailed(ResourceUtil.getI18nMessage("60004"));
        }

        List<String> pList = new ArrayList<>();
        pList.add("SB_ZT_Running");
        pList.add("SB_ZT_Ready");
        pList.add("SB_ZT_alarm");
        pList.add("__online__.connected");

        Map<String, Map<String, Object>> combineResultMap = new HashMap();
        Map<String, List<DeviceDTO>> collect = deviceDTOList.stream().filter(item->item.getModelId()!=null).collect(Collectors.groupingBy(DeviceDTO::getModelId));
        for (String key:collect.keySet()) {
            List<DeviceDTO> deviceDTOS = collect.get(key);
            List<String> deviceIds  = deviceDTOS.stream().map(DeviceDTO::getId).collect(Collectors.toList());
            Map<String, Map<String, Object>> workingMapByModelGroup = deviceService.getWorkingMapByModelGroup(key, deviceIds, pList);
            combineResultMap.putAll(workingMapByModelGroup);
        }

        for (DeviceDTO deviceDTO : deviceDTOList){
            DeviceInfo device = DeviceInfo.builder().build();
            BeanUtils.copyProperties(deviceDTO, device);

            //查询是否同步了地理位置
            String gps = null;
            NxDeviceLocationEntity nxDeviceLocationEntity = deviceLocationMap.get(deviceDTO.getId());
            if (nxDeviceLocationEntity != null) {
                gps = nxDeviceLocationEntity.getLongitude() + "," + nxDeviceLocationEntity.getLatitude();
                device.setLocation(new String[]{String.valueOf(nxDeviceLocationEntity.getLongitude()), String.valueOf(nxDeviceLocationEntity.getLatitude())});
            } else {
                //处理经纬度
                String location = deviceDTO.getLocation();
                if (StringUtils.isNotBlank(location)) {
                    gps = location;
                    device.setLocation(location.split(","));
                }
            }

            // 2024-9-26新需求 需要根据设备的坐标经纬度筛选，经纬度为空的设备不统计
            if (device.getLocation() == null) {
                continue;
            }

            //看在保修内还是保修外
            if(StringUtils.isNotBlank(device.getDeviceStartTime())){
                if(new SimpleDateFormat("yyyy-MM-dd").format(new Date()).compareTo(device.getDeviceStartTime())<=0){
                    device.setWarrantyStatus("质保期内");
                }else{
                    device.setWarrantyStatus("质保期外");
                }
            }else{
                device.setWarrantyStatus("质保期外");
            }

            // 2024-9-26新需求 先从数据库取设备的详细地址，取不到再通过经纬度解析
            if(StringUtils.isNotBlank(deviceDTO.getAddress())){
                device.setAddress(deviceDTO.getAddress());
            } else if(StringUtils.isNotBlank(deviceDTO.getAddress2())){
                device.setAddress(deviceDTO.getAddress2());
            } else if(StringUtils.isNotBlank(gps)){
                String address = null;
                //获取经纬度返回地址
                address = gaoDeAPIUtils.getAddressByJwd(gps);
                device.setAddress(address);
            }
            if(combineResultMap.containsKey(deviceDTO.getId())){
                Map<String, Object> stringObjectMap = combineResultMap.get(deviceDTO.getId());
                if (!stringObjectMap.isEmpty()) {
                    Map<String, Object> running = (Map) stringObjectMap.get("SB_ZT_Running");
                    Map<String, Object> reading = (Map) stringObjectMap.get("SB_ZT_Ready");
                    Map<String, Object> alarm = (Map) stringObjectMap.get("SB_ZT_alarm");
                    Map<String, Object> online = (Map) stringObjectMap.get("__online__.connected");
                    boolean flg = false;
                    if (online != null && (boolean) online.get("value")) {
                        this.setStatus(online, device);
                        device.setStatus(DeviceConstant.ONLINE_STATUS_CODE);
                        flg = true;
                    }
                    if (reading != null && (boolean) reading.get("value")) {
                        this.setStatus(reading, device);
                        device.setStatus(DeviceConstant.READY_STATUS_CODE);
                        flg = true;
                    }
                    if (alarm != null && (boolean) alarm.get("value")) {
                        this.setStatus(alarm, device);
                        device.setStatus(DeviceConstant.ALARM_STATUS_CODE);
                        flg = true;
                    }
                    if (running != null && (boolean) running.get("value")) {
                        this.setStatus(running, device);
                        device.setStatus(DeviceConstant.RUN_STATUS_CODE);
                        flg = true;
                    }
                    if (online != null && !(boolean) online.get("value")) {
                        this.setStatus(online, device);
                        device.setStatus(DeviceConstant.OFFLINE_STATUS_CODE);
                        flg = true;
                    }
                    if (!flg) {
                        device.setStatus(DeviceConstant.OFFLINE_STATUS_CODE);
                    }
                }
            }
            deviceInfoList.add(device);
        }

       /* for (DeviceDTO deviceDTO : deviceDTOList) {
            DeviceInfo device = DeviceInfo.builder().build();
            BeanUtils.copyProperties(deviceDTO, device);
            //查询是否同步了地理位置
            String gps = null;
            NxDeviceLocationEntity nxDeviceLocationEntity = deviceLocationMap.get(deviceDTO.getId());
            if (nxDeviceLocationEntity != null) {
                gps = nxDeviceLocationEntity.getLongitude() + "," + nxDeviceLocationEntity.getLatitude();
                device.setLocation(new String[]{String.valueOf(nxDeviceLocationEntity.getLongitude()), String.valueOf(nxDeviceLocationEntity.getLatitude())});
            } else {
                //处理经纬度
                String location = deviceDTO.getLocation();
                if (StringUtils.isNotBlank(location)) {
                    gps = location;
                    device.setLocation(location.split(","));
                }
            }

            //获取经纬度返回地址
            if(StringUtils.isNotBlank(gps)){
                String address = null;
                address = gaoDeAPIUtils.getAddressByJwd(gps);
                device.setAddress(address);
            }
            //判断是否为平台同步的设备: 物模型id不为空
            if (StringUtils.isNotBlank(deviceDTO.getModelId())) {
                List<String> paramsList = new ArrayList<>();
//            设备running： SB_ZT_Running
//            设备ready：SB_ZT_Ready
//            设备alarm：SB_ZT_alarm
//            在线 __online__.connected
                paramsList.add("SB_ZT_Running");
                paramsList.add("SB_ZT_Ready");
                paramsList.add("SB_ZT_alarm");
                paramsList.add("__online__.connected");
                Map<String, Object> workingMap = deviceService.getRootcloudWorkingMap(device.getId(), paramsList);
                if (!workingMap.isEmpty()) {
                    Map<String, Object> running = (Map) workingMap.get("SB_ZT_Running");
                    Map<String, Object> reading = (Map) workingMap.get("SB_ZT_Ready");
                    Map<String, Object> alarm = (Map) workingMap.get("SB_ZT_alarm");
                    Map<String, Object> online = (Map) workingMap.get("__online__.connected");
                    boolean flg = false;
                    if (online != null && (boolean) online.get("value")) {
                        this.setStatus(online, device);
                        device.setStatus(DeviceConstant.ONLINE_STATUS_CODE);
                        flg = true;
                    }
                    if (running != null && (boolean) running.get("value")) {
                        this.setStatus(running, device);
                        device.setStatus(DeviceConstant.RUN_STATUS_CODE);
                        flg = true;
                    }
                    if (alarm != null && (boolean) alarm.get("value")) {
                        this.setStatus(alarm, device);
                        device.setStatus(DeviceConstant.ALARM_STATUS_CODE);
                        flg = true;
                    }
                    if (reading != null && (boolean) reading.get("value")) {
                        this.setStatus(reading, device);
                        device.setStatus(DeviceConstant.READY_STATUS_CODE);
                        flg = true;
                    }
                    if(online != null && !(boolean) online.get("value")){
                        this.setStatus(online, device);
                        device.setStatus(DeviceConstant.OFFLINE_STATUS_CODE);
                        flg = true;
                    }
                    if (!flg) {
                        if (online != null) {
                            this.setStatus(online, device);
                        }
                        device.setStatus(DeviceConstant.OFFLINE_STATUS_CODE);
                    }
                }
            }
            deviceInfoList.add(device);
        }*/
        deviceView.setDevices(deviceInfoList);
        //统计总数、在线、离线、故障报警、在线率
        this.setDeviceStatusStatistics(deviceView, deviceInfoList, deviceInfoList.size());
        //工时统计：入云时间到昨天的24时的时间差
        this.setDeviceWorkTimeStatistics(deviceView, deviceDTOList);
        return JsonResult.success(deviceView);
    }

    private void setStatus(Map<String, Object> map, DeviceInfo device) {
        String timeLocal = (String) map.get("timeLocal");
        if (StringUtils.isNotBlank(timeLocal)) {
            ZonedDateTime utcZoned = ZonedDateTime.parse(timeLocal);
            ZoneId swissZone = ZoneId.of("Asia/Shanghai");
            ZonedDateTime swissZoned = utcZoned.withZoneSameInstant(swissZone);
            LocalDateTime localDate = swissZoned.toLocalDateTime();
            device.setOnlineTime(DateUtil.dateToString(Date.from(localDate.atZone(ZoneId.systemDefault()).toInstant()), "YYYY-MM-dd HH:mm:ss"));
        }
        /*String timeCloud = (String) map.get("timeCloud");
        if (StringUtils.isNotBlank(timeCloud)) {
            ZonedDateTime utcZoned = ZonedDateTime.parse(timeCloud);
            ZoneId swissZone = ZoneId.of("Asia/Shanghai");
            ZonedDateTime swissZoned = utcZoned.withZoneSameInstant(swissZone);
            LocalDateTime localDate = swissZoned.toLocalDateTime();
            device.setCloudTime(DateUtil.dateToString(Date.from(localDate.atZone(ZoneId.systemDefault()).toInstant()), "YYYY-MM-dd HH:mm:ss"));
        }*/
    }

    @Override
    public JsonResult<DeviceVO> getDeviceInfoWithMonthWorkTimeLineChart(String deviceId) {
        DeviceEntity deviceEntity = deviceMapper.selectById(deviceId);
        if (deviceEntity == null) {
            return JsonResult.clientFailed(StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
        }
        DeviceInfo deviceInfo = DeviceInfo.builder().id(deviceEntity.getNxJeDeviceId()).modelName(deviceEntity.getDeviceSbxhName()).typeName(deviceEntity.getDeviceSblxName()).address(deviceEntity.getDeviceSbxxdz()).code(deviceEntity.getDeviceSbbm()).name(deviceEntity.getDeviceSbmc()).status((deviceEntity.getDeviceStatusCode() == null || "".equals(deviceEntity.getDeviceStatusCode())) ? null : Integer.valueOf(deviceEntity.getDeviceStatusCode())).networkStatus((deviceEntity.getDeviceNetworkStatusCode() == null || "".equals(deviceEntity.getDeviceNetworkStatusCode())) ? null : Integer.valueOf(deviceEntity.getDeviceNetworkStatusCode())).build();
        NxDeviceEntity nxDeviceEntity = nxDeviceMapper.selectById(deviceInfo.getId());

        //查询设备位置
        List<NxDeviceLocationEntity> allDeviceLocation = deviceLocationMapper.findAllDeviceLocation();
        Map<String, NxDeviceLocationEntity> deviceLocationMap = new HashMap<>();
        String gps = null;
        if (CollectionUtils.isNotEmpty(allDeviceLocation)) {
            deviceLocationMap = allDeviceLocation.stream().collect(Collectors.toMap(NxDeviceLocationEntity::getDeviceId, Function.identity()));
        }
        NxDeviceLocationEntity nxDeviceLocationEntity = deviceLocationMap.get(deviceId);
        if (nxDeviceLocationEntity != null) {
            gps = nxDeviceLocationEntity.getLongitude() + "," + nxDeviceLocationEntity.getLatitude();
            deviceInfo.setLocation(new String[]{String.valueOf(nxDeviceLocationEntity.getLongitude()), String.valueOf(nxDeviceLocationEntity.getLatitude())});
        }else{
            gps = deviceEntity.getDeviceJwd();
        }
        // 2024-9-26新需求 先从数据库取设备的详细地址，取不到再通过经纬度解析
        if(StringUtils.isNotBlank(deviceInfo.getAddress())){
            deviceInfo.setAddress(deviceInfo.getAddress());
        } else if(nxDeviceEntity != null && StringUtils.isNotBlank(nxDeviceEntity.getSblbSbxxdz())){
            deviceInfo.setAddress(nxDeviceEntity.getSblbSbxxdz());
        } else if(StringUtils.isNotBlank(gps)){
            String address = null;
            //获取经纬度返回地址
            address = gaoDeAPIUtils.getAddressByJwd(gps);
            deviceInfo.setAddress(address);
        }
        //判断是否为平台同步的设备: 物模型id不为空
        if (StringUtils.isNotBlank(deviceEntity.getModelId())) {
            List<String> paramsList = new ArrayList<>();
//            设备running： SB_ZT_Running
//            设备ready：SB_ZT_Ready
//            设备alarm：SB_ZT_alarm
//            在线 __online__.connected
            paramsList.add("SB_ZT_Running");
            paramsList.add("SB_ZT_Ready");
            paramsList.add("SB_ZT_alarm");
            paramsList.add("__online__.connected");
            Map<String, Object> workingMap = deviceService.getRootcloudWorkingMap(deviceId, paramsList);

            Map<String, Object> running = (Map) workingMap.get("SB_ZT_Running");
            Map<String, Object> reading = (Map) workingMap.get("SB_ZT_Ready");
            Map<String, Object> alarm = (Map) workingMap.get("SB_ZT_alarm");
            Map<String, Object> online = (Map) workingMap.get("__online__.connected");
            boolean flg = false;
            if (online != null && (boolean) online.get("value")) {
                this.setStatus(online, deviceInfo);
                deviceInfo.setStatus(DeviceConstant.ONLINE_STATUS_CODE);
                flg = true;
            }
            if (reading != null && (boolean) reading.get("value")) {
                this.setStatus(reading, deviceInfo);
                deviceInfo.setStatus(DeviceConstant.READY_STATUS_CODE);
                flg = true;
            }
            if (alarm != null && (boolean) alarm.get("value")) {
                this.setStatus(alarm, deviceInfo);
                deviceInfo.setStatus(DeviceConstant.ALARM_STATUS_CODE);
                flg = true;
            }
            if (running != null && (boolean) running.get("value")) {
                this.setStatus(running, deviceInfo);
                deviceInfo.setStatus(DeviceConstant.RUN_STATUS_CODE);
                flg = true;
            }
            if(online != null && !(boolean) online.get("value")){
                this.setStatus(online, deviceInfo);
                deviceInfo.setStatus(DeviceConstant.OFFLINE_STATUS_CODE);
                flg = true;
            }
            if (!flg) {
                if (online != null) {
                    this.setStatus(online, deviceInfo);
                }
                deviceInfo.setStatus(DeviceConstant.OFFLINE_STATUS_CODE);
            }
        }
        //获取历史工况信息
        String endDateString = DateUtil.format(new Date(), "YYYY-MM-01 00:00:00");
        Date endDate = DateUtil.parse(endDateString);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(endDate);
        calendar.add(Calendar.MONTH, -5);
        Date startDate = calendar.getTime();
        //查询工时
        List<DeviceWorkTimeDTO> allWithWorkTime = deviceMapper.findAllWithMonthWorkBetweenTime(deviceId, startDate, endDate);
        allWithWorkTime = allWithWorkTime.stream().filter(c -> c.getTime()!=null).filter(c -> c.getOnlinedTimes()!=null).collect(Collectors.toList());
        //分组月工时
        Map<String, DeviceWorkTimeDTO> timeDTOMap = allWithWorkTime.stream().collect(Collectors.toMap(itm -> DateUtil.format(itm.getTime(), "YYYY-MM"), Function.identity()));

        List<Object> y = new ArrayList<>();
        List<Object> xData = new ArrayList<>();
        for (int i = 0; i < 6; i++) {
            calendar.setTime(startDate);
            calendar.add(Calendar.MONTH, i);
            String key = DateUtil.format(calendar.getTime(), "YYYY-MM");
            String xKey = DateUtil.format(calendar.getTime(), "MM");
            if (!timeDTOMap.isEmpty() && timeDTOMap.get(key) != null && timeDTOMap.get(key).getOnlinedTimes()!=null) {
                BigDecimal onlinedTimes = timeDTOMap.get(key).getOnlinedTimes();
                BigDecimal value = onlinedTimes.setScale(2,RoundingMode.HALF_UP);
                DecimalFormat decimalFormat = new DecimalFormat("0.00");
                String strVal = decimalFormat.format(value.doubleValue());
                value = new BigDecimal(strVal);
                y.add(value);
            } else {
                y.add(0);
            }
            xData.add(xKey);
        }
        DeviceMonthWorkTimeLineChart deviceMonthWorkTimeLineChart = DeviceMonthWorkTimeLineChart.builder().build();
        List<Object> data = new ArrayList<>();
        data.add(y);
        deviceMonthWorkTimeLineChart.setData(data);
        deviceMonthWorkTimeLineChart.setXData(xData);

        return JsonResult.success(DeviceVO.builder().deviceInfo(deviceInfo).deviceMonthWorkTimeLineChart(deviceMonthWorkTimeLineChart).build());
    }

    @Override
    public JsonResult<DeviceMonthWorkTimeStatistics> getMonthWorkTimeStatistics(String deviceId, Date searchDate) {
        DeviceMonthWorkTimeStatistics result = DeviceMonthWorkTimeStatistics.builder().build();

        //获取月工时
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(searchDate);
        int month = calendar.get(Calendar.MONTH) + 1;
        int year = calendar.get(Calendar.YEAR);
        Date date = DateUtil.parse(year + "-" + month + "-01" + " 00:00:00");
        List<DeviceWorkTimeDTO> allWithMonthWorkTime = deviceMapper.findAllWithMonthWorkTime(deviceId, date);
        if (CollectionUtils.isNotEmpty(allWithMonthWorkTime)) {
            DeviceWorkTimeDTO deviceWorkTimeDTO = allWithMonthWorkTime.get(0);
            //result.setTotal(deviceWorkTimeDTO.getOnlinedTimes().divide(new BigDecimal( 60 * 60), 1, RoundingMode.HALF_UP).intValue());
            if(deviceWorkTimeDTO.getOnlinedTimes()!=null){
                BigDecimal onlinedTimes = deviceWorkTimeDTO.getOnlinedTimes();
                DecimalFormat decimalFormat = new DecimalFormat("0.00");
                String strVal = decimalFormat.format(onlinedTimes.doubleValue());
                result.setTotal(new BigDecimal(strVal));
            }
        }
        //获取天工时
        //获取昨天日期
        calendar.setTime(searchDate);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        Date yesterday = calendar.getTime();

        List<DeviceWorkTimeDTO> allWithDayWorkTime = deviceMapper.findAllWithDayWorkTime(deviceId, yesterday, searchDate);
        if (CollectionUtils.isNotEmpty(allWithDayWorkTime)) {
            for (DeviceWorkTimeDTO deviceWorkTimeDTO : allWithDayWorkTime) {
                if(deviceWorkTimeDTO.getTime()!=null){
                    if (deviceWorkTimeDTO.getTime().getTime() == searchDate.getTime()) {
                        //result.setTodayWorkTime(deviceWorkTimeDTO.getOnlinedTimes().divide(new BigDecimal( 60 * 60), 1, RoundingMode.HALF_UP).intValue());
                        BigDecimal onlinedTimes = deviceWorkTimeDTO.getOnlinedTimes();
                        DecimalFormat decimalFormat = new DecimalFormat("0.00");
                        String strVal = decimalFormat.format(onlinedTimes.doubleValue());
                        result.setTodayWorkTime(new BigDecimal(strVal));
                    } else {
                        //result.setYesterdayWorkTime(deviceWorkTimeDTO.getOnlinedTimes().divide(new BigDecimal( 60 * 60), 1, RoundingMode.HALF_UP).intValue());
                        BigDecimal onlinedTimes = deviceWorkTimeDTO.getOnlinedTimes();
                        DecimalFormat decimalFormat = new DecimalFormat("0.00");
                        String strVal = decimalFormat.format(onlinedTimes.doubleValue());
                        result.setYesterdayWorkTime(new BigDecimal(strVal));
                    }
                }
            }
        }
        //当月：总工时、昨日、今日
        return JsonResult.success(result);
    }

    @Override
    public List<DeviceSpecialDTO> selectAllDevice() {
        return deviceMapper.findSpecialAll();
    }

    @Override
    public String returnAddress(String sblbSbbm) {
        QueryWrapper<DeviceEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("DEVICE_SBBM",sblbSbbm);
        List<DeviceEntity> deviceEntities = deviceMapper.selectList(queryWrapper);
        if(!deviceEntities.isEmpty()){
            DeviceEntity deviceEntity = deviceEntities.get(0);
            //查询设备位置
            List<NxDeviceLocationEntity> allDeviceLocation = deviceLocationMapper.findAllDeviceLocation();
            Map<String, NxDeviceLocationEntity> deviceLocationMap = new HashMap<>();
            if (CollectionUtils.isNotEmpty(allDeviceLocation)) {
                deviceLocationMap = allDeviceLocation.stream().collect(Collectors.toMap(NxDeviceLocationEntity::getDeviceId, Function.identity()));
            }
            NxDeviceLocationEntity nxDeviceLocationEntity = deviceLocationMap.get(deviceEntities.get(0).getNxJeDeviceId());
            System.out.print("deviceId:"+deviceEntities.get(0).getNxJeDeviceId());
            if(!ObjectUtils.isEmpty(nxDeviceLocationEntity)){
                if(nxDeviceLocationEntity.getLongitude() != null && nxDeviceLocationEntity.getLatitude() != null){
                    String gps = nxDeviceLocationEntity.getLongitude() + "," + nxDeviceLocationEntity.getLatitude();
                    log.info(deviceEntities.get(0).getNxJeDeviceId()+"gps:"+gps);
                    System.out.print(deviceEntities.get(0).getNxJeDeviceId()+"gps:"+gps);
                    String addressByJwd = gaoDeAPIUtils.getAddressByJwd(gps);
                    return addressByJwd;
                }
            }
        }
        return null;
    }

    @Override
    public List<DeviceDTO> deviceDTOListByRole() {
        LoginUserInfo current = UserContext.current();
        log.info("登录用户Id ==>{}",current.getUserId());
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        String userIds = null;
        log.info("登录用户权限 ==>{}",roleIds);
        if(roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)){
            //管理员
            return deviceMapper.findAll();
        }
        List<String> userIdList = new ArrayList<>();
        if (roleIds.contains(RoleIdConstant.FWJL) || roleIds.contains(RoleIdConstant.GCS)){
            SiteDTO siteByUserId = enduserService.getSiteByUserId(current.getUserId());
            if(!ObjectUtils.isEmpty(siteByUserId)){
                String[] strings = siteByUserId.getServiceArea().split(",");
                for (int i = 0; i < strings.length; i++) {
                    String areaName = strings[i];
                    String customs = enduserService.getCustomsBySiteAreaName(areaName);
                    if(StringUtils.isNotBlank(customs)){
                        userIdList.add(customs);
                    }
                }
            }
        }else if (roleIds.contains(RoleIdConstant.KHLD) || roleIds.contains(RoleIdConstant.KH)){
            String custom = enduserService.getCustomsByUserId(current.getUserId());
            if(StringUtils.isNotBlank(custom)){
                userIdList.add(custom);
            }
        }
        if(userIdList.size()>0){
            userIds = String.join(",", userIdList);
            return deviceMapper.findAllByUserId(userIds);
        }
        return new ArrayList<DeviceDTO>();
    }

    private void setDeviceStatusStatistics(DeviceView deviceView, List<DeviceInfo> deviceInfoList, int totalCount) {
        DeviceStatusStatistics deviceStatusStatistics = DeviceStatusStatistics.builder().build();
        deviceStatusStatistics.setDeviceTotal(totalCount);
        //计算在线设备数
        Map<Integer, List<DeviceInfo>> deviceMap = deviceInfoList.stream().collect(Collectors.groupingBy(itm -> itm.getStatus() == null ? 0 : itm.getStatus()));
        List<DeviceInfo> OffDeviceList = deviceMap.get(DeviceConstant.OFFLINE_STATUS_CODE);
        List<DeviceInfo> faultDeviceList = deviceMap.get(DeviceConstant.ALARM_STATUS_CODE);
        //计算百分比
        int offCount = 0;
        int faultCount = 0;
        if (CollectionUtils.isNotEmpty(OffDeviceList)) {
            offCount = OffDeviceList.size();
        }
        if (CollectionUtils.isNotEmpty(faultDeviceList)) {
            faultCount = faultDeviceList.size();
        }
        if (CollectionUtils.isNotEmpty(OffDeviceList)) {
            deviceStatusStatistics.setOnlineCount(totalCount - offCount);
            BigDecimal total = new BigDecimal(totalCount);
            BigDecimal onlineRate = new BigDecimal(totalCount - offCount).multiply(new BigDecimal(100)).divide(total, 2, RoundingMode.HALF_UP);
            deviceStatusStatistics.setOnlineRate(onlineRate.floatValue());
        } else {
            deviceStatusStatistics.setOnlineRate(0f);
        }
        //获取故障设备数
        //deviceStatusStatistics.setFault(rootCloudOpenAPIUtil.getCountUnclosedDevice());
        deviceStatusStatistics.setFault(faultCount);
        deviceStatusStatistics.setOfflineCount(offCount);
        deviceView.setDeviceStatusStatistics(deviceStatusStatistics);
    }

    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
    private static final SimpleDateFormat stf = new SimpleDateFormat("yyyy-MM-dd");

    private void setDeviceWorkTimeStatistics(DeviceView deviceView, List<DeviceDTO> deviceDTOList) {

        List<DeviceWorkTimeStatistics> deviceWorkTimeStatistics = new ArrayList<>();
        //获取去昨天24小时
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        Date startDate = calendar.getTime();
        String yesterday = sdf.format(startDate);
        Date yssterDate = null;
        try {
            yssterDate = DateUtils.parseDate(stf.format(new Date()) + " 00:00:00", "YYYY-MM-dd HH:mm:ss");
        } catch (ParseException e) {
            log.info("时间转换错误");
        }

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime dateTime = LocalDateTime.parse(yesterday, formatter);
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        // 本月起始
        Calendar thisMonthFirstDateCal = Calendar.getInstance();
        thisMonthFirstDateCal.set(Calendar.DAY_OF_MONTH, 1);
        String thisMonthFirstTime = format.format(thisMonthFirstDateCal.getTime()) + " 00:00:00";

        //查询工时
        List<DeviceWorkTimeDTO> allWithWorkTime = null;
        try {
            allWithWorkTime = deviceMapper.findAllWithMonthWorkTime(null, format.parse(thisMonthFirstTime));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        if (CollectionUtils.isNotEmpty(allWithWorkTime)) {
            Map<String, List<DeviceWorkTimeDTO>> map = allWithWorkTime.stream().collect(Collectors.groupingBy(DeviceWorkTimeDTO::getDeviceId));
            Set<String> deviceWithWorkTimeIds = map.keySet();
            List<DeviceDTO> deviceDTOS = deviceDTOList.stream().filter(itm -> deviceWithWorkTimeIds.contains(itm.getId())).collect(Collectors.toList());
            for (DeviceDTO itm : deviceDTOS) {
                List<DeviceWorkTimeDTO> deviceWorkTimeDTOS = map.get(itm.getId());
                BigDecimal totalTimeMS = new BigDecimal(0);
                for(DeviceWorkTimeDTO dto : deviceWorkTimeDTOS){
                    if(dto.getOnlinedTimes()!=null){
                        totalTimeMS = totalTimeMS.add(dto.getOnlinedTimes());
                    }
                }
                //减去昨天的
                BigDecimal yesterDecimal = new BigDecimal(0);
                if(yssterDate!=null){
                    List<DeviceWorkTimeDTO> allWithDayWorkTime = deviceMapper.findAllWithDayWorkTime(itm.getId(), yssterDate, yssterDate);
                    if (CollectionUtils.isNotEmpty(allWithDayWorkTime)) {
                        for (DeviceWorkTimeDTO deviceWorkTimeDTO : allWithDayWorkTime) {
                            if(deviceWorkTimeDTO.getTime()!=null &&deviceWorkTimeDTO.getOnlinedTimes()!=null){
                                yesterDecimal = yesterDecimal.add(deviceWorkTimeDTO.getOnlinedTimes());
                            }
                        }
                    }
                }
                totalTimeMS  = totalTimeMS.subtract(yesterDecimal).setScale(2, RoundingMode.HALF_UP);
                DecimalFormat decimalFormat = new DecimalFormat("0.00");
                String strVal = decimalFormat.format(totalTimeMS.doubleValue());
                totalTimeMS = new BigDecimal(strVal);
                //BigDecimal totalTimeMS = deviceWorkTimeDTOS.stream().map(DeviceWorkTimeDTO::getOnlinedTimes).reduce(BigDecimal.ZERO, BigDecimal::add);
                //计算时间差
                LocalDateTime endTime = LocalDateTime.parse(thisMonthFirstTime, formatter);
//                LocalDateTime endTime = LocalDateTime.parse(itm.getCloudTime() != null ? itm.getCloudTime() : "2022-01-01 00:00:00", formatter);
                long betweenMS = ChronoUnit.SECONDS.between(endTime, dateTime) ;
                //BigDecimal rs = totalTimeMS.divide(new BigDecimal(betweenMS), 1, RoundingMode.HALF_UP).divide(new BigDecimal(  60 * 60), 2, RoundingMode.HALF_UP);
                BigDecimal rs = totalTimeMS.multiply(new BigDecimal( 60 * 60 * 100)).divide(new BigDecimal(betweenMS), 2, RoundingMode.HALF_UP);

                //计算时间差
                DeviceWorkTimeStatistics device = DeviceWorkTimeStatistics.builder().name(itm.getCode()).workTotalTimeRate(rs).workTotalTime(totalTimeMS).build();
                deviceWorkTimeStatistics.add(device);
            }
        }
        //排序
       /* Collections.sort(deviceWorkTimeStatistics, new Comparator<DeviceWorkTimeStatistics>() {
            @Override
            public int compare(DeviceWorkTimeStatistics device1, DeviceWorkTimeStatistics device2) {
                return -device1.getWorkTotalTime().compareTo(device1.getWorkTotalTime());
            }
        });*/
        List<DeviceWorkTimeStatistics> list = deviceWorkTimeStatistics.stream().sorted(
                Comparator.comparing(DeviceWorkTimeStatistics::getWorkTotalTime).reversed()).collect(Collectors.toList());
        deviceView.setDeviceWorkTimeStatistics(list);
    }
}