package com.rootcloud.domain.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.domain.entity.NxZskFaultCodeEntity;
import com.rootcloud.domain.mapper.NxZskFaultCodeMapper;
import com.rootcloud.domain.service.device.FaultCodeService;
import com.rootcloud.vo.faultcode.FaultCodeDetailVo;
import com.rootcloud.vo.faultcode.FaultCodeVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.List;

/**
 * @author:xxxxx
 * @create: 2022-05-07 15:18
 * @Description: 故障码服务实现类
 */
@Service("faultCodeServiceImpl")
public class FaultCodeServiceImpl extends ServiceImpl<NxZskFaultCodeMapper, NxZskFaultCodeEntity> implements FaultCodeService {

    @Autowired
    private NxZskFaultCodeMapper nxZskFaultCodeMapper;


    @Override
    public List<NxZskFaultCodeEntity> queryFaultCodes(String level, String sbxh) {
        LambdaQueryWrapper<NxZskFaultCodeEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(NxZskFaultCodeEntity.class);
        //设备级别
        if (StringUtils.isNotBlank(level)) {
            lambdaQueryWrapper.eq(NxZskFaultCodeEntity::getFaultcodeGzdjcode, level);
        }
        //根据设备型号查询
        if (StringUtils.isNotBlank(sbxh)) {
            lambdaQueryWrapper.eq(NxZskFaultCodeEntity::getFaultcodeSbxhcode, sbxh);
        }
        return this.list(lambdaQueryWrapper);
    }

    @Override
    public List<NxZskFaultCodeEntity> queryFaultCodesOfApp(List<String> levels, String codeName, String code) {
        LambdaQueryWrapper<NxZskFaultCodeEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(NxZskFaultCodeEntity.class);
        if (StringUtils.isNotBlank(code)) {
            lambdaQueryWrapper.like(NxZskFaultCodeEntity::getFaultcodeGzm, code);
        }
        if (StringUtils.isNotBlank(codeName)) {
            lambdaQueryWrapper.eq(NxZskFaultCodeEntity::getFaultcodeGzms, codeName);
        }
        //app端固定查紧急和重要 1和2
        lambdaQueryWrapper.in(NxZskFaultCodeEntity::getFaultcodeGzdjcode, levels);

        return this.list(lambdaQueryWrapper);
    }

    @Override
    public List<NxZskFaultCodeEntity> queryFaultCodesOfMessage(List<String> levels) {
        LambdaQueryWrapper<NxZskFaultCodeEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(NxZskFaultCodeEntity.class);
        //app端固定查紧急和重要 1和2
        lambdaQueryWrapper.in(NxZskFaultCodeEntity::getFaultcodeGzdjcode, levels);

        return this.list(lambdaQueryWrapper);
    }

    @Override
    public IPage<FaultCodeVo> getPage(int page, int size, String queryData, String faultcodeSblxcode) {
        IPage<FaultCodeVo> iPage = new Page(page, size);
        return nxZskFaultCodeMapper.getPage(iPage, queryData, faultcodeSblxcode);
    }

    @Override
    public FaultCodeDetailVo selectById(String nxZskFaultcodeId) {
        QueryWrapper<NxZskFaultCodeEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("NX_ZSK_FAULTCODE_ID", nxZskFaultcodeId);
        NxZskFaultCodeEntity nxZskFaultCodeEntity = nxZskFaultCodeMapper.selectOne(queryWrapper);
        FaultCodeDetailVo vo = new FaultCodeDetailVo();
        if (!ObjectUtils.isEmpty(nxZskFaultCodeEntity)) {
            BeanUtils.copyProperties(nxZskFaultCodeEntity, vo);
        }
        return vo;
    }

    @Override
    public List<NxZskFaultCodeEntity> selectData(String syZhid, List<String> list) {
        LambdaQueryWrapper<NxZskFaultCodeEntity> nxzsOne = new LambdaQueryWrapper<>();
        if (list.size() > 0) {
            nxzsOne.in(NxZskFaultCodeEntity::getFaultcodeGzm, list);
        }
        nxzsOne.eq(NxZskFaultCodeEntity::getSyZhid, syZhid);
        List<NxZskFaultCodeEntity> nxZskFaultCodeEntityList = nxZskFaultCodeMapper.selectList(nxzsOne);
        return nxZskFaultCodeEntityList;
    }
}
