package com.rootcloud.domain.service.device.impl;

import com.rootcloud.common.JsonResult;
import com.rootcloud.common.LappLiveClient;
import com.rootcloud.common.constant.JsonResultStatusBpm;
import com.rootcloud.common.constant.LappLiveConstant;
import com.rootcloud.common.live.CacheMap;
import com.rootcloud.common.live.CacheMap2;
import com.rootcloud.common.net.JsonHelper;
import com.rootcloud.domain.service.device.LappLiveService;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("appLiveSetvice")
public class LappLiveSetviceImpl implements LappLiveService {

    private final Logger log = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private LappLiveClient lappLiveClient;

    @Override
    public boolean openLive(String deviceSerial, String channelNo) throws Exception {

        if (StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(channelNo)) {
            log.error("设备序列号和通道号不能为空");
            return false;
        }

        // 从缓存中获取accessToken
        String accessToken = this.getAccessToken();
        String message = "";

        try {
            message = lappLiveClient.openLive(LappLiveConstant.OPEN_LIVE_URL, accessToken, deviceSerial, channelNo);
            if (StringUtils.isBlank(message)) {
                return false;
            } else {
                String code = JsonHelper.getJsonObject(message).get("code").toString();
                String msg = JsonHelper.getJsonObject(message).get("msg").toString();
                if ("200".equals(code)) {
                    return true;
                } else {
                    log.error("打开直播地址失败,原因:{}", msg);
                    if ("10002".equals(code)) {
                        log.error("token失效,将会重新获取");
                        CacheMap.dataMap.remove("accessToken");
                        return false;
                    }
                }
            }
        } catch (Exception e) {
            log.error("直播打开异常", e);
            return false;
        }

        return false;
    }

    @Override
    public JsonResult getLiveAddress(String deviceSerial, String channelNo) throws Exception {

        if (StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(channelNo)) {
            log.error("设备序列号和通道号不能为空");
            return null;
        }
        // 打开直播
        boolean flag = this.openLive(deviceSerial, channelNo);
        if (flag) {
            String message = lappLiveClient.getLiveAddress(LappLiveConstant.GET_LIVE_URL, this.getAccessToken(),
                    deviceSerial, channelNo);
            String code = JsonHelper.getJsonObject(message).get("code").toString();
            String msg = JsonHelper.getJsonObject(message).get("msg").toString();

            List<Map<String, String>> resultList = new ArrayList<Map<String, String>>();
            Map<String, String> map = new HashMap<String, String>();

            // token失效时将会重新获取token
            if ("10002".equals(code)) {
                log.error("token失效,将会重新获取");
                CacheMap.dataMap.remove("accessToken");
                return null;
            }

            // 为200时表示请求成功
            if ("200".equals(code)) {
                // 解析参数
                String result = JsonHelper.getJsonObject(message).get("data").toString();
                log.info("地址信息:{}", result);
                JSONArray jsonArray = JsonHelper.getJsonArray(result);
                JSONObject jsonObject = jsonArray.getJSONObject(0);
                String ret = jsonObject.getString("ret");
                String desc = jsonObject.getString("desc");
                String status = jsonObject.getString("status");
                String exception = jsonObject.getString("exception");

                // 200表示解析成功
                if ("200".equals(ret) && "1".equals(status) && "0".equals(exception)) {
                    String hls = jsonObject.getString("hls");
                    String hlsHd = jsonObject.getString("hlsHd");
                    String rtmp = jsonObject.getString("rtmp");
                    String rtmpHd = jsonObject.getString("rtmpHd");
                    map.put("hls", hls);
                    map.put("hlsHd", hlsHd);
                    map.put("rtmp", rtmp);
                    map.put("rtmpHd", rtmpHd);
                    map.put("deviceSerial", deviceSerial);
                    map.put("channelNo", channelNo);
                    resultList.add(map);
                    return new JsonResult(JsonResultStatusBpm.RET_SUCCESS, resultList);
                } else {
                    log.error("解析直播地址有异常,解析信息:{},设备状态(1表示正常):{},异常信息(0表示没有异常):{}", desc, status, exception);
                    log.error("设备状态:0-未使用或直播已关闭，1-使用中，2-已过期，3-直播已暂停");
                    log.error("地址异常信息：0-正常，1-设备不在线，2-设备开启视频加密，3-设备删除，4-失效，5-未绑定，6-账户下流量已超出");
                    return new JsonResult(JsonResultStatusBpm.RET_EXCEPTION, "解析直播地址有异常,请检查直播状态和设备状态");
                }
            } else {
                log.error("请求获取直播地址失败,失败信息为:{}", msg);
                return new JsonResult(JsonResultStatusBpm.RET_EXCEPTION, msg);
            }
        } else {
            log.error("打开直播功能失败");
            return new JsonResult(JsonResultStatusBpm.RET_EXCEPTION, "打开直播功能失败,请检查参数否正确 或重新进行该操作!");
        }
    }

    public String getAccessToken() throws Exception {
        String accessToken = "";
        boolean flag = CacheMap.hasCache("accessToken");
        if (flag) {
            // 判断token时是否过期.
            accessToken = CacheMap.get("accessToken");
            if (StringUtils.isBlank(accessToken)) {
                accessToken = lappLiveClient.getAccessToken();
                CacheMap.put("accessToken", accessToken);
            }
        } else {
            // 缓存中不存在token
            accessToken = lappLiveClient.getAccessToken();
            CacheMap.put("accessToken", accessToken);
        }
        return accessToken;
    }


    @Override
    public JsonResult getLeChengLiveAddress(String deviceSerial, String channelNo) throws Exception {
        List<Map<String, String>> resultList = new ArrayList<Map<String, String>>();
        if (StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(channelNo)) {
            log.error("设备序列号和通道号不能为空");
            return null;
        }
        if (true) {
            String tokenString = this.getAccessTokenLeCheng();
            if (tokenString == null) {
                log.error("token失效,将会重新获取...");
                CacheMap2.dataMap.remove("accessTokenLeCheng");
                return null;
            }
            if(tokenString.equals("")){
                log.error("token为空,开始重新获取...");
                tokenString = this.getAccessTokenLeCheng2();
            }
            if(tokenString.equals("")){
                log.error("token重新获取为空返回...");
                return null;
            }
            log.error("LECHENG_URL:"+LappLiveConstant.LIVE_LECHENG_URL);
            log.error("token:"+tokenString);
            log.error("deviceSerial:"+deviceSerial+"|channelNo:"+channelNo+"|token:"+tokenString);
            String message0 = lappLiveClient.getLiveAddressLeCheng0(LappLiveConstant.LIVE_LECHENG_URL, tokenString,
                    deviceSerial, channelNo);
            log.error("message0:"+message0);

            String hls01 = JSONArray.fromObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(message0).get("result")).get("data")).get("streams")).getJSONObject(0).get("hls").toString();
            String streamId01 = JSONArray.fromObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(message0).get("result")).get("data")).get("streams")).getJSONObject(0).get("streamId").toString();
            String hls02 = JSONArray.fromObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(message0).get("result")).get("data")).get("streams")).getJSONObject(1).get("hls").toString();
            String streamId02 = JSONArray.fromObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(JsonHelper.getJsonObject(message0).get("result")).get("data")).get("streams")).getJSONObject(1).get("streamId").toString();
            log.error("hls01:"+hls01);
            log.error("streamId01:"+streamId01);
            log.error("hls02:"+hls02);
            log.error("streamId02:"+streamId02);

            Map<String, String> map = new HashMap<String, String>();
            if (streamId01.equals("1") && streamId02.equals("0")) { //0:高清主码流；1:标清辅码流
                map.put("hlsHd", hls02);
                map.put("hls", hls01);
            }
            if (streamId01.equals("0") && streamId02.equals("1")) {
                map.put("hlsHd", hls01);
                map.put("hls", hls02);
            }
            map.put("rtmp", "");
            map.put("rtmpHd", "");
            map.put("deviceSerial", deviceSerial);
            map.put("channelNo", channelNo);
            resultList.add(map);
            return new JsonResult(JsonResultStatusBpm.RET_SUCCESS, resultList);
        } else {
            log.error("打开直播功能失败");
            return new JsonResult(JsonResultStatusBpm.RET_EXCEPTION, "打开直播功能失败,请检查参数否正确 或重新进行该操作!");
        }
    }

    public String openLiveLeCheng(String deviceSerial, String channelNo) throws Exception {

        if (StringUtils.isBlank(deviceSerial) || StringUtils.isBlank(channelNo)) {
            log.error("设备序列号和通道号不能为空");
            return "";
        }
        // 从缓存中获取accessToken
        String accessToken = this.getAccessTokenLeCheng();
        String message = "";
        return accessToken;

        /*
        try {
            message = lappLiveClient.openLive(LappLiveConstant.OPEN_LIVE_LECHENG_URL, accessToken, deviceSerial, channelNo);
            if (StringUtils.isBlank(message)) {
                return false;
            } else {
                String code = JsonHelper.getJsonObject(message).get("code").toString();
                String msg = JsonHelper.getJsonObject(message).get("msg").toString();
                if ("200".equals(code)) {
                    return true;
                } else {
                    log.error("打开直播地址失败,原因:{}", msg);
                    if("10002".equals(code)) {
                        log.error("token失效,将会重新获取");
                        CacheMap.dataMap.remove("accessToken");
                        return false;
                    }
                }
            }
        } catch (Exception e) {
            log.error("直播打开异常", e);
            return false;
        }
        return false;
        */
    }

    public String getAccessTokenLeCheng() throws Exception {
        String accessToken = "";
        boolean flag = CacheMap2.hasCache("accessTokenLeCheng");
        if (flag) {
            try{
                // 判断token时是否过期.
                accessToken = CacheMap2.get("accessTokenLeCheng");
                log.error("判断token时是否过期,accessToken:{}",accessToken);
                if (StringUtils.isBlank(accessToken)) {
                    accessToken = lappLiveClient.getAccessTokenLeCheng();
                    log.error("token已过期重新获取,accessToken:{}",accessToken);
                    CacheMap2.put("accessTokenLeCheng", accessToken);
                }
                else{
                    log.error("token未过期,accessToken:{}",accessToken);
                }
            }catch(Exception ex){
                log.error("getAccessTokenLeCheng1:{}",ex);
            }
        } else {
            try{
                // 缓存中不存在token
                accessToken = lappLiveClient.getAccessTokenLeCheng();
                log.error("缓存中不存在token重新获取,accessToken:{}",accessToken);
                CacheMap2.put("accessTokenLeCheng", accessToken);
            }catch (Exception ex){
                log.error("getAccessTokenLeCheng2:{}",ex);
            }
        }
        return accessToken;
    }

    public String getAccessTokenLeCheng2() throws Exception {
        String accessToken = "";
        try{
            // 缓存中不存在token
            accessToken = lappLiveClient.getAccessTokenLeCheng();
            log.error("缓存中不存在token重新获取,accessToken:{}",accessToken);
            CacheMap2.put("accessTokenLeCheng", accessToken);
        }catch (Exception ex){
            log.error("getAccessTokenLeCheng2:{}",ex);
        }
        return accessToken;
    }
}