package com.rootcloud.domain.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import com.rootcloud.common.JEUuidUtil;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.PageRequest;
import com.rootcloud.domain.dto.CameraDTO;
import com.rootcloud.domain.dto.NxDeviceDTO;
import com.rootcloud.domain.entity.device.NxDeviceCameraEntity;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.mapper.DeviceMapper;
import com.rootcloud.domain.mapper.NxDeviceCameraMapper;
import com.rootcloud.domain.mapper.NxDeviceMapper;
import com.rootcloud.domain.service.device.NxDeviceService;
import com.rootcloud.vo.device.CameraParam;
import com.rootcloud.vo.device.NxDeviceParam;
import com.rootcloud.vo.order.DeviceDataVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;

/**
 * TODO
 *
 * @author wei.liao
 * @version 1.0
 * @date 2022/4/18 14:18
 */
@Slf4j
@Service("nxDeviceServiceImpl")
public class NxDeviceServiceImpl extends ServiceImpl<NxDeviceMapper, NxDeviceEntity> implements NxDeviceService {

    @Autowired
    private DeviceMapper deviceMapper;
    @Autowired
    private NxDeviceCameraMapper cameraMapper;

    @Autowired
    private NxDeviceMapper nxDeviceMapper;

    @Override
    public JsonResult<PageInfo<NxDeviceDTO>> findList(PageRequest pageRequest, NxDeviceParam nxDeviceParam) {
        int pageNum = pageRequest.getPage() == null ? 1 : pageRequest.getPage();
        int pageSize = pageRequest.getSize() == null ? 10 : pageRequest.getSize();
        int startIndex = (pageNum - 1) * pageSize;

        List<NxDeviceDTO> cameraList = deviceMapper.findCameraList(nxDeviceParam.getNo(), nxDeviceParam.getName(), nxDeviceParam.getType(), startIndex, pageSize);

        if (CollectionUtils.isNotEmpty(cameraList)) {
            cameraList.forEach(itm -> {
                itm.setCameras(deviceMapper.selectCameras(itm.getDeviceId()));
            });
            cameraList.sort(Comparator.comparing(NxDeviceDTO::getDeviceId));
            cameraList.forEach(itm -> {
                itm.getCameras().sort(Comparator.comparing(CameraDTO::getOrderIndex));
            });
        }
        Integer count = deviceMapper.findCameraCount(nxDeviceParam.getNo(), nxDeviceParam.getName(), nxDeviceParam.getType());

        PageInfo<NxDeviceDTO> pageInfo = new PageInfo<>(cameraList);
        pageInfo.setPageNum(pageNum);
        pageInfo.setPageSize(pageSize);
        pageInfo.setTotal(count);
        return JsonResult.success(pageInfo);
    }

    @Override
    public JsonResult<Void> bindCamera(CameraParam cameraParam) {
        NxDeviceCameraEntity cameraEntity = new NxDeviceCameraEntity();
        BeanUtils.copyProperties(cameraParam, cameraEntity);
        if (StringUtils.isNotBlank(cameraParam.getId())) {
            cameraEntity.setNxJcxxSblbCameraId(cameraParam.getId());
            cameraMapper.updateById(cameraEntity);
        } else {
            cameraEntity.setNxJcxxSblbCameraId(JEUuidUtil.uuid());
            cameraMapper.insert(cameraEntity);
        }
        return JsonResult.success();
    }

    @Override
    public JsonResult<Void> UnboundCamera(String cameraId) {
        cameraMapper.deleteById(cameraId);
        return JsonResult.success();
    }

    @Override
    public List<NxDeviceCameraEntity> camerasByDeviceId(String deviceId) {
        LambdaQueryWrapper<NxDeviceCameraEntity> cameraWrapper = new LambdaQueryWrapper<>();
        cameraWrapper.eq(NxDeviceCameraEntity::getDeviceId, deviceId);
        cameraWrapper.orderByAsc(NxDeviceCameraEntity::getOrderIndex);
        List<NxDeviceCameraEntity> nxDeviceCameraEntities = cameraMapper.selectList(cameraWrapper);
        return nxDeviceCameraEntities;
    }

    @Override
    public DeviceDataVo getJcxxByIds(String ids) {
        return nxDeviceMapper.getJcxxByIds(ids);
    }

    @Override
    public String selectFwwdId(String khid,String zhId) {
        return nxDeviceMapper.selectFwwdId(khid,zhId);
    }


    @Override
    public List<DeviceDataVo> getKuDeviceList(String userId) {
        return nxDeviceMapper.getKuDeviceList(userId);
    }

}

