package com.rootcloud.domain.service.ds.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.constant.RoleIdConstant;
import com.rootcloud.config.auth.UserInfo;
import com.rootcloud.domain.dao.ds.DeviceListMapper;
import com.rootcloud.domain.entity.*;
import com.rootcloud.domain.entity.ds.DeviceListEntity;
import com.rootcloud.domain.mapper.NxOrderMapper;
import com.rootcloud.domain.service.ds.DeviceListSyncService;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service("deviceListSyncService2")
public class DeviceListSyncServiceImpl extends ServiceImpl<DeviceListMapper, DeviceListEntity> implements DeviceListSyncService {

    @Autowired
    private DeviceListMapper deviceListMapper;

    @Autowired
    private NxOrderMapper nxOrderMapper;

    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;

    @Override
    public void updateDeviceListData(DeviceListEntity entity, String action) {
        //更新前先判断数据库数据是否存在设备
        LambdaQueryWrapper<DeviceListEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(DeviceListEntity.class);
        lambdaQueryWrapper.eq(DeviceListEntity::getNxJcxxSblbId, entity.getNxJcxxSblbId());
        List<DeviceListEntity> list = this.list(lambdaQueryWrapper);
        if ("add".equals(action) && CollectionUtils.isEmpty(list)) {
            this.save(entity);
        } else if ("mod".equals(action) && !CollectionUtils.isEmpty(list)) {
            this.updateById(entity);
        } else if ("del".equals(action) && !CollectionUtils.isEmpty(list)) {
            this.removeById(entity.getNxJcxxSblbId());
        }
    }

    @Override
    public List<DeviceListEntity> queryDeviceList(String userId) {
        //网点的设备列表
        JeCoreEnduserEntity user = nxOrderMapper.getCoreEnduserByUsrId(userId);
        //一个人只能绑定一个服务网点，所以list里只有一个或者没有
        List<FwryDTO> fwryList = nxOrderMapper.getNxgxJcdaFwryByUserId(userId);
        List<DeviceListEntity> sblbList = new ArrayList<>();
        UserInfo userInfo = enduserService.getUser(userId);
        if (!org.apache.commons.collections.CollectionUtils.isEmpty(fwryList)) {
            FwryDTO fwryDTO = fwryList.get(0);
            //通过网点id获取服务网点信息
            NetworkEntity networkEntity = nxOrderMapper.getfwwd(fwryDTO);
            //获取网点下的服务区域
            String[] split = networkEntity.getFwwdSsqy().split(",");
            //通过网点的服务区域，去查询客户
            List<KhglDTO> khglDTOList = nxOrderMapper.getKhBykhdzs(split);
            //通过客户名称查询设备列表信息
            //所有的设备列表信息
            if (!org.apache.commons.collections.CollectionUtils.isEmpty(khglDTOList)) {
                HashMap<String, String> req = new HashMap<>();
                String khmc = "";
                for (KhglDTO dto : khglDTOList) {
                    khmc += "'" + dto.getKhglKhmc() + "',";
                }
                khmc = khmc.substring(0, khmc.length() - 1);
                req.put("khmc", khmc);
//                        Log.info("寿命预警工程师或服务经理查询客户名称:{}", khmc);
                // List<NxDeviceEntity> sblbList = nxOrderMapper.getNxDeviceEntity(req);
                if (StringUtils.isNotEmpty(khmc)) {
                    sblbList = deviceListMapper.selectBykhmc(khmc, userInfo.getTenantId());
                }
            }
        }

        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.SHLD) || roleIds.contains(RoleIdConstant.GLY)) {
            userId = null;
        }
        //客户管理下的设备列表
        List<DeviceListEntity> deviceListEntities = deviceListMapper.selectByUserId(userId, userInfo.getTenantId());
        sblbList.addAll(deviceListEntities);
        return sblbList;
    }
}
