package com.rootcloud.domain.service.ds.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpStatus;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.TokenUtil;
import com.rootcloud.domain.dao.ds.DeviceSyncMapper;
import com.rootcloud.domain.entity.ds.DeviceSyncEntity;
import com.rootcloud.domain.service.ds.DeviceSyncService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service("deviceSyncService2")
@Slf4j
public class DeviceSyncServiceImpl extends ServiceImpl<DeviceSyncMapper, DeviceSyncEntity> implements DeviceSyncService {

    @Value(value = "${os.iam.domain}")
    private String domain;
    @Value(value = "${os.iam.superToken}")
    private String superToken;
    @Value(value = "${cms.thing.uri}")
    private String thingUri;
    @Value(value = "${cms.thingstatus.uri}")
    private String thingStatusUri;
    @Autowired
    private TokenUtil tokenUtil;

    /**
     * 同步物实例
     */
    @Override
    public void syncDeviceData() {
        //查询目前在库的物模型
        LambdaQueryWrapper<DeviceSyncEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(DeviceSyncEntity.class);
        lambdaQueryWrapper.eq(DeviceSyncEntity::getDeviceSource,"1");
        List<DeviceSyncEntity> list = this.list(lambdaQueryWrapper);
        //判断在库的设备是否为空，为空直接同步全量数据
        if (CollectionUtils.isEmpty(list)) {
            this.saveBatch(getDeviceList());
            return;
        }
        //不为空时先判断是否有增量数据，有先同步增量，再比较变更
        List<DeviceSyncEntity> deviceList = getDeviceList();
        sync(list, deviceList);
    }

    private void sync(List<DeviceSyncEntity> dbList, List<DeviceSyncEntity> interfaceList) {
        List<DeviceSyncEntity> deleteList = new ArrayList<>(16);
        List<DeviceSyncEntity> updateList = new ArrayList<>(16);
        List<DeviceSyncEntity> addList = new ArrayList<>(16);
        //更新的 集合
        for (int i = 0; i < dbList.size(); i++) {
            DeviceSyncEntity temp = dbList.get(i);
            if (containsId(interfaceList, temp.getNxJeDeviceId())) {
                updateList.add(temp);
            }
        }
        //删除的集合
        deleteList = dbList;
        deleteList.removeAll(updateList);
        //插入的集合
        addList = interfaceList;
        addList.removeAll(updateList);


        //批量插入
        this.saveBatch(addList);
        //删除的
        this.removeByIds(deleteList.stream().map(temp -> temp.getNxJeDeviceId()).collect(Collectors.toList()));

        //更新的
        updateList.stream().forEach(temp -> {
            //通过ID 从接口的 集合中获取这条数据
            for (int i = 0; i < interfaceList.size(); i++) {
                DeviceSyncEntity temp2 = interfaceList.get(i);
                if (temp.getNxJeDeviceId().equals(temp2.getNxJeDeviceId())) {
                    //接口返回更新时间时更新最新设备信息到数据库
                    if (temp2.getUpdateTime() != null) {
                        BeanUtils.copyProperties(temp, temp2);
                        this.updateById(temp);
                    } else {
                        continue;
                    }
                }
            }
        });
    }

    private boolean containsId(List<DeviceSyncEntity> modelEntityList, String deviceId) {
        for (int i = 0; i < modelEntityList.size(); i++) {
            DeviceSyncEntity temp = modelEntityList.get(i);
            if (temp.getNxJeDeviceId().equals(deviceId)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 调用物实例查询接口获取设备信息
     * @return
     */
    public List<DeviceSyncEntity> getDeviceList() {
        List<DeviceSyncEntity> resultList = new ArrayList<>(16);;
        String thingUrl = domain+thingUri;
        HttpRequest thingRequest = HttpRequest.get(thingUrl);
        String thingStatusUrl = domain+thingStatusUri;
        HttpRequest thingStatusRequest = HttpRequest.get(thingStatusUrl);
        String token = tokenUtil.getToken();
        thingRequest.header("Authorization", "Bearer " + token);
        thingStatusRequest.header("Authorization", "Bearer " + token);
        HttpResponse thingResponse = thingRequest.execute();
        if(thingResponse.getStatus()!= HttpStatus.HTTP_OK){
            log.error("调用平台接口"+thingUrl+"返回"+thingResponse);
        }
        HttpResponse thingStatusResponse = thingStatusRequest.execute();
        if(thingStatusResponse.getStatus()!= HttpStatus.HTTP_OK){
            log.error("调用平台接口"+thingStatusUrl+"返回"+thingStatusResponse);
        }
        JSONObject thingJsonObj = new JSONObject(thingResponse.body());
        JSONObject thingStatusJsonObj = new JSONObject(thingStatusResponse.body());
        JSONArray thingArray = thingJsonObj.getJSONArray("payload");
        JSONArray thingStatusArray = thingStatusJsonObj.getJSONArray("payload");
        thingStatusArray.stream().forEach(temp->{
            JSONObject obj = (JSONObject)temp;
            DeviceSyncEntity deviceEntity = new DeviceSyncEntity();
            deviceEntity.setNxJeDeviceId(obj.get("thingId").toString());
            deviceEntity.setDeviceSbbm(obj.get("assetId").toString());
            deviceEntity.setDeviceSbmc(obj.get("name").toString());
            deviceEntity.setModelId(obj.get("modelId").toString());
            //设备型号和设备类型给默认值
            deviceEntity.setDeviceSblx("0000");
            deviceEntity.setDeviceSblxName("4.0平台设备");
            deviceEntity.setDeviceSbxh("yHG8OtzRKz3oEDhKZop");
            deviceEntity.setDeviceSbxhName("4.0平台设备型号");
            //来源1 代表来之平台
            deviceEntity.setDeviceSource("1");
            if(obj.get("country") != null){
                deviceEntity.setDeviceSsgj(obj.get("country").toString());
            }
            if(obj.get("state") != null){
                deviceEntity.setDeviceSssfmc(obj.get("state").toString());
            }
            if(obj.get("city") != null){
                deviceEntity.setDeviceSscsmc(obj.get("city").toString());
            }
            if(obj.get("district") != null){
                deviceEntity.setDeviceSsqxmc(obj.get("district").toString());
            }
            if(obj.get("longitude") != null){
                deviceEntity.setDeviceJd(obj.get("longitude").toString());
            }
            if(obj.get("latitude") != null){
                deviceEntity.setDeviceWd(obj.get("latitude").toString());
            }
            if(obj.get("online") != null){
                boolean online = (boolean)obj.get("online");
                if(online){
                    deviceEntity.setDeviceNetworkStatusCode("1");
                }else{
                    deviceEntity.setDeviceNetworkStatusCode("0");
                }

            }
   /*         if(temp.get("workingStatus") != null){
                boolean workingStatus = (boolean)temp.get("workingStatus");
                if(workingStatus){
                    deviceEntity.setDeviceStatusCode("1");
                }else {
                    deviceEntity.setDeviceStatusCode("0");
                }
            }*/
            thingArray.stream().forEach(temp1->{
                JSONObject obj1 = (JSONObject) temp1;
                if(obj.get("thingId").equals(obj1.get("thingId"))){
                    JSONObject deviceInfo = new JSONObject(obj1.get("deviceInfo"));
                    if(deviceInfo.get("manufacturer") != null){
                        deviceEntity.setDeviceSskhmc(deviceInfo.get("manufacturer").toString());
                    }
                    if(deviceInfo.get("manufacturerTs") != null){
                        deviceEntity.setDeviceSskh(deviceInfo.get("manufacturerTs").toString());
                    }
                    if(deviceInfo.get("modelTs") != null){
                        deviceEntity.setDeviceSbxh(deviceInfo.get("modelTs").toString());
                    }
                    if(deviceInfo.get("model") != null){
                        deviceEntity.setDeviceSbxhName(deviceInfo.get("model").toString());
                    }
                }
            });
            resultList.add(deviceEntity);
        });
        return resultList;
    }

    /**
     * 新增物实例（mq读取数据）
     */

    @Override
    public void updateDeviceData(DeviceSyncEntity entity,String action) {
        //判断设备是否在设备台账已存在
        LambdaQueryWrapper<DeviceSyncEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(DeviceSyncEntity.class);
        lambdaQueryWrapper.eq(DeviceSyncEntity::getNxJeDeviceId,entity.getNxJeDeviceId());
        List<DeviceSyncEntity> list = this.list(lambdaQueryWrapper);
        if("add".equals(action) && CollectionUtils.isEmpty(list)){
            this.save(entity);
        }else if("mod".equals(action) && !CollectionUtils.isEmpty(list)){
            this.updateById(entity);
        }else if("del".equals(action) && !CollectionUtils.isEmpty(list)){
            this.removeById(entity.getNxJeDeviceId());
        }
    }

}
