package com.rootcloud.domain.service.ds.impl;

import cn.hutool.core.lang.UUID;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.TokenUtil;
import com.rootcloud.domain.dao.ds.ModelPropertySyncMapper;
import com.rootcloud.domain.entity.ds.ModelEntity;
import com.rootcloud.domain.entity.ds.ModelPropertySyncEntity;
import com.rootcloud.domain.service.ds.ModelPropertySyncService;
import com.rootcloud.domain.service.ds.ModelSyncService;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service("modelPropertySyncService2")
@Slf4j
public class ModelPropertySyncServiceImpl extends ServiceImpl<ModelPropertySyncMapper, ModelPropertySyncEntity> implements ModelPropertySyncService {
    @Value(value = "${os.iam.domain}")
    private String domain;
    @Value(value = "${os.iam.superToken}")
    private String superToken;
    @Value(value = "${cms.modelproperty.uri}")
    private String modelPropertyUri;
    @Resource(name = "modelSyncService2")
    private ModelSyncService modelSyncService;
    @Autowired
    private TokenUtil tokenUtil;
    @Resource(name = "modelPropertySyncMapper")
    private ModelPropertySyncMapper modelPropertySyncMapper;


    /**
     * 物模型数据通过监听平台mq消息同步，由于平台mq没有同步属性单位，临时方案改为调用平台接口同步属性单位
     */
    @Override
    public void syncModelPropertyData(String clientId, String clientSecret) {
        //查询目前在库的物模型
        LambdaQueryWrapper<ModelPropertySyncEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(ModelPropertySyncEntity.class);
        List<ModelPropertySyncEntity> dbList = this.list(lambdaQueryWrapper);

        //调用平台接口查询平台的物模型数据
        List<ModelPropertySyncEntity> modelEntityList = getModelPropertys(clientId, clientSecret);
        //只同步工况属性的单位
        modelEntityList.stream().forEach(temp -> {
                    for (ModelPropertySyncEntity dbTemp : dbList) {
                        if (temp.getPropertyDataUnit() == null) {
                            break;
                        }
                        if (temp.getModelId().equals(dbTemp.getModelId()) && temp.getPropertyId().equals(dbTemp.getPropertyId())) {
                            dbTemp.setPropertyDataUnit(temp.getPropertyDataUnit());
                        }
                    }
                }
        );
        this.updateBatchById(dbList);
    }

    @Override
    public List<String> queryPropertysByModelId(String modelId) {
        LambdaQueryWrapper<ModelPropertySyncEntity> queryWrapper = Wrappers.lambdaQuery(ModelPropertySyncEntity.class);
        queryWrapper.eq(ModelPropertySyncEntity::getModelId, modelId);
        List<ModelPropertySyncEntity> list = modelPropertySyncMapper.selectList(queryWrapper);
        List<String> resultList = new ArrayList<>();
        list.stream().forEach(temp -> {
            resultList.add(temp.getPropertyId());
        });
        return resultList;
    }


    @Override
    public List<ModelPropertySyncEntity> queryListByModelId(String modelId) {
        return modelPropertySyncMapper.selectList(
                Wrappers.lambdaQuery(ModelPropertySyncEntity.class).eq(ModelPropertySyncEntity::getModelId, modelId));
    }

    @Override
    public ModelPropertySyncEntity queryByModelIdAndPropertyId(String modelId, String propertyId) {
        return modelPropertySyncMapper.selectOne(Wrappers.lambdaQuery(ModelPropertySyncEntity.class)
                .eq(ModelPropertySyncEntity::getModelId, modelId).eq(ModelPropertySyncEntity::getPropertyId, propertyId));
    }

    /**
     * 数据同步，比较db查询到的dbList和接口返回的interfaceList，
     * 只在dbList的做删除，只在interface的做新增，交集部分做变更
     *
     * @param dbList
     * @param interfaceList
     */
    private void sync(List<ModelPropertySyncEntity> dbList, List<ModelPropertySyncEntity> interfaceList) {
        List<ModelPropertySyncEntity> deleteList = new ArrayList<>(16);
        List<ModelPropertySyncEntity> updateList = new ArrayList<>(16);
        List<ModelPropertySyncEntity> addList = new ArrayList<>(16);
        //更新的 集合
        for (int i = 0; i < dbList.size(); i++) {
            ModelPropertySyncEntity temp = dbList.get(i);
            if (containsId(interfaceList, temp.getModelId())) {
                updateList.add(temp);
            }
        }
        //删除的集合
        deleteList = dbList;
        deleteList.removeAll(updateList);
        //插入的集合
        addList = interfaceList;
        addList.removeAll(updateList);

        //批量插入
        this.saveBatch(addList);
        //删除的
        this.removeByIds(deleteList.stream().map(temp -> temp.getModelId()).collect(Collectors.toList()));

        //更新的
        updateList.stream().forEach(temp -> {
            //通过ID 从接口的 集合中获取这条数据
            for (int i = 0; i < interfaceList.size(); i++) {
                ModelPropertySyncEntity temp2 = interfaceList.get(i);
                if (temp.getModelId().equals(temp2.getModelId())) {
                    BeanUtils.copyProperties(temp, temp2);
                    this.updateById(temp);
                }
            }
        });
    }

    private boolean containsId(List<ModelPropertySyncEntity> modelEntityList, String modelId) {
        for (int i = 0; i < modelEntityList.size(); i++) {
            ModelPropertySyncEntity temp = modelEntityList.get(i);
            if (temp.getModelId().equals(modelId)) {
                return true;
            }
        }
        return false;
    }

    //调用平台接口查询所有的物模型属性
    public List<ModelPropertySyncEntity> getModelPropertys(String clientId, String clientSecret) {

        //先查询物模型List
        List<ModelEntity> modelList = modelSyncService.getModelList(clientId, clientSecret);

        // modelList = modelList.subList(0,50);
        List<ModelPropertySyncEntity> resultList = new ArrayList<>(16);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //根据物模型ID查询对应的属性list
        modelList.stream().forEach(temp -> {
            String uri = modelPropertyUri.replace("{modelId}", temp.getModelId());
            HttpRequest request = HttpRequest.get(domain + uri);
            request.header("Authorization", "Bearer " + tokenUtil.getToken(clientId, clientSecret));
            try {
                HttpResponse response = request.execute();
                JSONObject jsonObject = new JSONObject(response.body());
                JSONArray array = jsonObject.getJSONArray("payload");
                if (Objects.isNull(array)) {
                    log.error("JSONArray array = jsonObject.getJSONArray(\"payload\")为NULL");

                } else {
                    List<JSONObject> list = JSONUtil.toList(array.toString(), JSONObject.class);
                    list.stream().forEach(jsonObj -> {
                        ModelPropertySyncEntity modelPropertyEntity = new ModelPropertySyncEntity();
                        modelPropertyEntity.setPropertyUuid(UUID.randomUUID().toString().substring(0, 10));
                        modelPropertyEntity.setModelId(temp.getModelId());
                        modelPropertyEntity.setPropertyId(jsonObj.get("name").toString());
                        modelPropertyEntity.setPropertyName(jsonObj.get("displayName").toString());
                        modelPropertyEntity.setPropertyDataType(jsonObj.get("propertyType").toString());

                        if (jsonObj.get("unit") != null) {
                            modelPropertyEntity.setPropertyDataUnit(jsonObj.get("unit").toString());
                        }
                        if (jsonObj.get("createdAt") != null) {
                            String createTime = jsonObj.get("createdAt").toString();
                            try {
                                modelPropertyEntity.setCreateTime(sdf1.format(sdf.parse(createTime)));
                            } catch (ParseException e) {
                                e.printStackTrace();
                            }
                        }
                        resultList.add(modelPropertyEntity);

                    });
                }
            } catch (Exception e) {
                log.error("调用平台接口：" + domain + uri + "出错，异常信息：" + e);
                e.printStackTrace();
            }
        });
        return resultList;
    }

    @Override
    public void updateModelProperty(List<ModelPropertySyncEntity> entitys) {
        //根据物模型ID查询是否已同步过
        if (!CollectionUtils.isEmpty(entitys)) {
            ModelPropertySyncEntity modelPropertySync = entitys.get(0);
            LambdaQueryWrapper<ModelPropertySyncEntity> queryWrapper = Wrappers.lambdaQuery(ModelPropertySyncEntity.class);
            queryWrapper.eq(ModelPropertySyncEntity::getModelId, modelPropertySync.getModelId());
            List<ModelPropertySyncEntity> resultList = this.list(queryWrapper);
            //没有同步过该物模型，直接批量新增
            if (CollectionUtils.isEmpty(resultList)) {
                log.info("物模型触发批量新增" + entitys);
                this.saveBatch(entitys);
            } else {
                //物模型已有时，可以先删除数据，同步最新的物模型数据
                log.info("物模型触发修改" + entitys);
                this.remove(queryWrapper);
                this.saveBatch(entitys);
            }
        }
    }


    @Override
    public void delModelProperty(ModelPropertySyncEntity entity) {
        LambdaQueryWrapper<ModelPropertySyncEntity> queryWrapper = Wrappers.lambdaQuery(ModelPropertySyncEntity.class);
        queryWrapper.eq(ModelPropertySyncEntity::getModelId, entity.getModelId());
        this.remove(queryWrapper);
    }
}
