package com.rootcloud.domain.service.ds.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.TokenUtil;
import com.rootcloud.domain.dao.ds.ModelSyncMapper;
import com.rootcloud.domain.entity.ds.ModelEntity;
import com.rootcloud.domain.service.ds.ModelSyncService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * 同步模型
 *
 * @author lixiafen
 * @date 2022-04-15 14:22:17
 */
@Service("modelSyncService2")
@Slf4j
public class ModelSyncServiceImpl extends ServiceImpl<ModelSyncMapper, ModelEntity> implements ModelSyncService {
    @Value(value = "${os.iam.domain}")
    private String domain;
    @Value(value = "${os.iam.superToken}")
    private String superToken;
    @Value(value = "${cms.thingmodel.uri}")
    private String thingModelUri;
    @Autowired
    private TokenUtil tokenUtil;

    /**
     * 同步物模型
     */
    @Override
    public void syncModelData(String clientId,String clientSecret) {
        //查询目前在库的物模型
        LambdaQueryWrapper<ModelEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(ModelEntity.class);
        List<ModelEntity> list = this.list(lambdaQueryWrapper);
        //判断在库的物模型是否为空，为空直接同步全量数据
        if (CollectionUtils.isEmpty(list)) {
            this.saveBatch(getModelList(clientId,clientSecret));
            return;
        }
        //不为空时先判断是否有增量数据，有先同步增量，再比较变更
        List<ModelEntity> modelEntityList = getModelList(clientId,clientSecret);
        sync(list, modelEntityList);
    }

    private void sync(List<ModelEntity> dbList, List<ModelEntity> interfaceList) {
        List<ModelEntity> deleteList = new ArrayList<>(16);
        List<ModelEntity> updateList = new ArrayList<>(16);
        List<ModelEntity> addList = new ArrayList<>(16);
        //更新的 集合
        for (int i = 0; i < dbList.size(); i++) {
            ModelEntity temp = dbList.get(i);
            if (containsId(interfaceList, temp.getModelId())) {
                updateList.add(temp);
            }
        }
        //删除的集合
        deleteList = dbList;
        deleteList.removeAll(updateList);
        //插入的集合
        addList = interfaceList;
        addList.removeAll(updateList);


        addList = addList.stream().filter(temp -> {
            if ("16AKncuuPVs".equals(temp.getModelId())) {
                return false;
            }
            return true;
        }).collect(Collectors.toList());

        //批量插入
        this.saveBatch(addList);
        //删除的
        this.removeByIds(deleteList.stream().map(temp -> temp.getModelId()).collect(Collectors.toList()));

        //更新的
        updateList.stream().forEach(temp -> {
            //通过ID 从接口的 集合中获取这条数据
            for (int i = 0; i < interfaceList.size(); i++) {
                ModelEntity temp2 = interfaceList.get(i);
                if (temp.getModelId().equals(temp2.getModelId())) {
                    if (temp.getUpdateTime() != temp2.getUpdateTime()) {
                        BeanUtils.copyProperties(temp, temp2);
                        this.updateById(temp);
                    } else {
                        continue;
                    }
                }
            }
        });
    }

    private boolean containsId(List<ModelEntity> modelEntityList, String modelId) {
        for (int i = 0; i < modelEntityList.size(); i++) {
            ModelEntity temp = modelEntityList.get(i);
            if (temp.getModelId().equals(modelId)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public List<ModelEntity> getModelList(String clientId,String clientSecret) {
        List<ModelEntity> resultList = new ArrayList<>();
        String result = null;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");
            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String thingType = "device";
            HttpRequest request = HttpRequest.get(domain + thingModelUri);
            Map<String, Object> map = new HashMap<>(16);
            map.put("thingType", thingType);
            request.form(map);
            request.header("Authorization", "Bearer " + tokenUtil.getToken(clientId,clientSecret));
            request.header("accept", "application/json");
            request.header("Content-Type", "application/json");

            HttpResponse response = request.execute();
            result = response.body();
            JSONObject jsonObject = new JSONObject(result);
            JSONArray array = jsonObject.getJSONArray("payload");
            array.stream().forEach(temp->{
                JSONObject obj = (JSONObject)temp;
                ModelEntity modelEntity = new ModelEntity();
                modelEntity.setModelId(obj.get("modelId").toString());
                modelEntity.setModelName(obj.get("name").toString());
       /*    if(obj.get("created") != null){
               String created = obj.get("created").toString();
               try {
                   modelEntity.setCreateTime(sdf1.parse(sdf1.format(sdf.parse(created))));
               } catch (ParseException e) {
                   e.printStackTrace();
               }
           }
           if(obj.get("updated") != null){
               String updated = obj.get("updated").toString();
               try {
                   modelEntity.setCreateTime(sdf1.parse(sdf1.format(sdf.parse(updated))));
               } catch (ParseException e) {
                   e.printStackTrace();
                   log.error("调用平台物模型接口："+domain + thingModelUri+"出错，异常信息："+e);
               }
           }*/
                resultList.add(modelEntity);
            });
        }catch (Exception e){
            e.printStackTrace();
            log.error("调用平台物模型接口："+domain + thingModelUri+"出错,返回信息："+result+"异常信息："+e);
        }
        //List<ModelEntity> list = JSONUtil.toList(array.toString(), ModelEntity.class);
        return resultList;
    }
}