package com.rootcloud.domain.service.ds.impl;

import cn.hutool.json.JSONObject;
import com.rabbitmq.client.Channel;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.DeflaterUtils;
import com.rootcloud.domain.entity.ds.DeviceListEntity;
import com.rootcloud.domain.entity.ds.DeviceSyncEntity;
import com.rootcloud.domain.entity.ds.NxDsLogEntity;
import com.rootcloud.domain.service.ds.DeviceListSyncService;
import com.rootcloud.domain.service.ds.DeviceSyncService;
import com.rootcloud.domain.service.ds.NxDsLogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.UUID;

@Component
@Slf4j
public class SyncDeviceInfoService {
    @Resource(name = "nxDsLogServiceImpl")
    NxDsLogService nxDsLogService;
    @Resource(name = "deviceSyncService2")
    DeviceSyncService deviceSyncService;
    @Resource(name = "deviceListSyncService2")
    DeviceListSyncService deviceListSyncService;

    @RabbitListener(queues = "base.queue.nxgx.device")
    public void onMessage(Message message, Channel channel) throws Exception {
        try {
            String body = new String(message.getBody(), "UTF-8");
            JSONObject object = new JSONObject(body);

            log.info("设备同步消息" + object);
            if (object != null) {
                String payload = object.get("payload").toString();
                //取租户ID
                String tenantId = object.get("tenantId").toString();
                JSONObject jsonPayload = new JSONObject(payload);
                String compressedContent = jsonPayload.get("compressedContent").toString();
                // String modelType = jsonPayload.get("modelType").toString();
                System.out.println(payload);
                String unzipString = DeflaterUtils.unzipString(compressedContent);
                JSONObject unzipJsonPayload = new JSONObject(unzipString);
                jsonPayload.set("assetId", unzipJsonPayload.get("assetId"));
                jsonPayload.set("tenantId", tenantId);
                System.out.println(unzipString);
                String action = object.get("action").toString();
                //直接在平台订阅中过滤
               /* if(!("device".equals(modelType) || "thing".equals(modelType))){
                    return;
                }*/
                //新增设备
                if ("createInstance".equals(action)) {
                    updateDevice(jsonPayload, "add");
                }
                //更新设备 只关注物模型和物标识
                if ("updateInstance".equals(action)) {
                    updateDevice(jsonPayload, "mod");
                }
                //删除设备 更新设备的状态
                if ("deleteInstance".equals(action)) {
                    updateDevice(jsonPayload, "del");
                }

            }
        } catch (Exception e) {
            log.error("设备数据同步异常" + e);
            NxDsLogEntity entity = new NxDsLogEntity();
            entity.setNxDsLogId(UUID.randomUUID().toString().substring(0, 10));
            entity.setLogContent(new String(message.getBody()));
            entity.setLogType("things");
            //nxDsLogService.saveLog(entity);
            //throw e ;
        }
    }


    /**
     * 设备新增 变更  删除
     */
    public void updateDevice(JSONObject json, String action) {
        if ("add".equals(action)) {
            log.info("触发设备变更代码");
            //同步设备台账
            DeviceSyncEntity deviceSync = new DeviceSyncEntity();
            deviceSync.setNxJeDeviceId(json.get("thingId").toString());
            deviceSync.setDeviceSbbm(json.get("assetId").toString());
            deviceSync.setDeviceSbmc(json.get("name").toString());
            deviceSync.setDeviceSbxh("yHG8OtzRKz3oEDhKZop");
            deviceSync.setDeviceSbxhName("4.0平台设备型号");
            deviceSync.setDeviceSblx("0000");
            deviceSync.setDeviceSblxName("4.0平台设备");
            deviceSync.setDeviceSource("1");
            deviceSync.setModelId(json.get("modelId").toString());
            //平台同步过来的设备默认联网
            deviceSync.setDeviceNetworkStatusCode("1");
            deviceSync.setCreateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            //设置设备租户ID
            deviceSync.setSyZhid(json.get("tenantId").toString());
            deviceSyncService.updateDeviceData(deviceSync, action);
            //同步设备列表
            DeviceListEntity deviceList = new DeviceListEntity();
            deviceList.setNxJcxxSblbId(json.get("thingId").toString());
            deviceList.setSblbSbmc(json.get("name").toString());
            deviceList.setSblbSbbm(json.get("assetId").toString());
            deviceList.setSblbSbxhCode("yHG8OtzRKz3oEDhKZop");
            deviceList.setSblbSbxhName("4.0平台设备型号");
            deviceList.setSblbSblxCode("0000");
            deviceList.setSblbSblxName("4.0平台设备");
            //平台同步过来的设备默认联网 1
            deviceList.setSblbLwztCode("1");
            //平台设备1 自有设备0
            deviceList.setSblbSbly("1");
            //设置设备租户ID
            deviceList.setSyZhid(json.get("tenantId").toString());
            deviceListSyncService.updateDeviceListData(deviceList, action);
        } else if ("mod".equals(action)) {
            DeviceSyncEntity deviceSync = new DeviceSyncEntity();
            deviceSync.setNxJeDeviceId(json.get("thingId").toString());
            deviceSync.setModelId(json.get("modelId").toString());
            deviceSync.setDeviceSbmc(json.get("name").toString());
            deviceSync.setUpdateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
            //设置设备租户ID
            deviceSync.setSyZhid(json.get("tenantId").toString());
            deviceSyncService.updateDeviceData(deviceSync, action);

            DeviceListEntity deviceList = new DeviceListEntity();
            deviceList.setNxJcxxSblbId(json.get("thingId").toString());
            deviceList.setSblbSbmc(json.get("name").toString());
            //设置设备租户ID
            deviceList.setSyZhid(json.get("tenantId").toString());
            deviceListSyncService.updateDeviceListData(deviceList, action);
        } else {
            DeviceSyncEntity deviceSync = new DeviceSyncEntity();
            deviceSync.setNxJeDeviceId(json.get("thingId").toString());
            //设置设备租户ID
            deviceSync.setSyZhid(json.get("tenantId").toString());
            deviceSyncService.updateDeviceData(deviceSync, action);

            DeviceListEntity deviceList = new DeviceListEntity();
            deviceList.setNxJcxxSblbId(json.get("thingId").toString());
            //设置设备租户ID
            deviceList.setSyZhid(json.get("tenantId").toString());
            deviceListSyncService.updateDeviceListData(deviceList, action);
        }
    }

}
