package com.rootcloud.domain.service.ds.impl;

import cn.hutool.core.lang.UUID;
import cn.hutool.json.JSONObject;
import com.rabbitmq.client.Channel;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.DeflaterUtils;
import com.rootcloud.domain.entity.ds.ModelPropertySyncEntity;
import com.rootcloud.domain.entity.ds.NxDsLogEntity;
import com.rootcloud.domain.mapper.ModelPropertyMapper;
import com.rootcloud.domain.service.ds.ModelPropertySyncService;
import com.rootcloud.domain.service.ds.NxDsLogService;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Logger;


@Component
@Slf4j
public class SyncModelProertySyncService {
    @Resource(name = "modelPropertySyncService2")
    ModelPropertySyncService modelPropertySyncService;
    @Resource(name = "nxDsLogServiceImpl")
    NxDsLogService nxDsLogService;
    @Autowired
    private ModelPropertyMapper modelPropertyMapper;

    @RabbitListener(queues = "base.queue.nxgx.model")
    public void onMessage(Message message, Channel channel) throws Exception {
        String body = new String(message.getBody(), "UTF-8");
        //JSONObject object = null;
        try {
            Log.info("物模型变更消息队列" + body);
            JSONObject object = new JSONObject(body);
            String payload = object.get("payload").toString();
            JSONObject jsonPayload = new JSONObject(payload);
            String compressedContent = jsonPayload.get("compressedContent").toString();
            String unzipString = DeflaterUtils.unzipString(compressedContent);
            JSONObject unzipJson = new JSONObject(unzipString);
            String action = object.get("action").toString();
            //直接在平台订阅中过滤
           /* String modelType = jsonPayload.get("modelType").toString();
            //只同步设备和复合物
            if("gateway".equals(modelType)){
                return;
            }*/
            //物模型发布 新增和修改
            if ("activateModel".equals(action)) {
                log.info("触发了action");
                updateModelProperty(unzipJson, "add");
            }
            //删除物模型
            if ("deleteModel".equals(action)) {
                updateModelProperty(unzipJson, "del");
            }

        } catch (Exception e) {
            NxDsLogEntity entity = new NxDsLogEntity();
            entity.setNxDsLogId(java.util.UUID.randomUUID().toString().substring(0, 10));
            entity.setLogContent(new String(message.getBody()));
            entity.setLogType("model");
            //nxDsLogService.saveLog(entity);
            Log.error("数据同步失败" + e.getMessage());
            //channel.basicAck(1,true);
            //throw e;
        }
    }


    /**
     * 设备新增 变更  删除
     */
    public void updateModelProperty(JSONObject json, String action) {
        if ("add".equals(action)) {
            List<ModelPropertySyncEntity> resultList = new ArrayList<>(16);
            //同步工况属性
            json.getJSONArray("schema").stream().forEach(temp -> {
                JSONObject schema = new JSONObject(temp);
                ModelPropertySyncEntity modelPropertySync = new ModelPropertySyncEntity();
                modelPropertySync.setModelId(json.get("modelId").toString());
                modelPropertySync.setModelName(json.get("name").toString());
                String uuid = modelPropertyMapper.selectUuidByModelId(schema.get("propertyName").toString(), json.get("modelId").toString());
                if (StringUtils.isEmpty(uuid)) {
                    modelPropertySync.setPropertyUuid(UUID.randomUUID().toString().substring(0, 10));
                } else {
                    modelPropertySync.setPropertyUuid(uuid);
                }
                modelPropertySync.setPropertyId(schema.get("propertyName").toString());
                modelPropertySync.setPropertyName(schema.get("displayName").toString());
                modelPropertySync.setPropertyDataType(schema.get("type").toString());
                if (schema.get("unit") != null) {
                    modelPropertySync.setPropertyDataUnit(schema.get("unit").toString());
                }
                modelPropertySync.setCreateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
                resultList.add(modelPropertySync);
            });

            modelPropertySyncService.updateModelProperty(resultList);
        } else if ("del".equals(action)) {
            ModelPropertySyncEntity modelPropertySync = new ModelPropertySyncEntity();
            modelPropertySync.setModelId(json.get("modelId").toString());

            modelPropertySyncService.delModelProperty(modelPropertySync);
        }
    }

}
