package com.rootcloud.domain.service.fittings;

import cn.hutool.core.lang.UUID;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.JEUuidUtil;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.dao.PrinterHeadRecordDao;
import com.rootcloud.domain.dto.NxSparePartVo;
import com.rootcloud.domain.entity.NxPartFileEntity;
import com.rootcloud.domain.entity.NxSparePartEntity;
import com.rootcloud.domain.mapper.SpareFilerMapper;
import com.rootcloud.domain.vo.PrinterHeadRecordVO;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Service
@Slf4j
public class SpareFileService {
    @Autowired
    SpareFilerMapper spareFilerMapper;

    @Autowired
    private PrinterHeadRecordDao recodeDao;
    public Boolean importSparePart(List<NxSparePartVo> nxSparePartVos) {

        if (nxSparePartVos == null || nxSparePartVos.size() == 0) {
            throw new BusinessException("409", ResourceUtil.getI18nMessage("60025"));
        }
        if (nxSparePartVos.size() > 200) {
            throw new BusinessException("409", ResourceUtil.getI18nMessage("60026") + "200");
        }

        for (int i = 0; i < nxSparePartVos.size(); i++) {
            String invalidateData = invalidateData(nxSparePartVos.get(i));
            if (!"success".equals(invalidateData)) {
                throw new BusinessException("409", (i + 1) + ResourceUtil.getI18nMessage("60024") + invalidateData);
            }

//            NxSparePartVo sparePartBySbbmAndStatus = spareFilerMapper.getSparePartBySbbmAndStatus(nxSparePartVos.get(i).getPartSbbm(), nxSparePartVos.get(i).getPartBjbm());
//            if (sparePartBySbbmAndStatus!=null && "正常".equals(nxSparePartVos.get(i).getPartBjzt())){
//                sparePartBySbbmAndStatus.setPartGzsc(addGzsc(sparePartBySbbmAndStatus.getPartGzsc(),nxSparePartVos.get(i).getPartGzsc()));
//                sparePartBySbbmAndStatus.setPartDrgzsc(nxSparePartVos.get(i).getPartGzsc());
//                spareFilerMapper.updateNxSparePartGzsc(sparePartBySbbmAndStatus);
//            }else {
            LambdaQueryWrapper<NxPartFileEntity> npe = new LambdaQueryWrapper<NxPartFileEntity>();
            npe.eq(NxPartFileEntity::getFilePjbm, nxSparePartVos.get(i).getPartBjbm());
            npe.last("limit 1");
            NxPartFileEntity one = spareFilerMapper.selectOne(npe);
            if (one == null) {
                throw new BusinessException("409", (i + 1) + ResourceUtil.getI18nMessage("60024") + ResourceUtil.getI18nMessage("60027"));
            }
            nxSparePartVos.get(i).setFilePjmc(one.getFilePjmc());
            nxSparePartVos.get(i).setPartXtmk(one.getFileXtmk());
            nxSparePartVos.get(i).setPartXtmkBm(one.getFileXtmkbm());
            nxSparePartVos.get(i).setPartGgxh(one.getFileGgxh());
            nxSparePartVos.get(i).setPartSm(String.valueOf(one.getFileLlsm() != null ? one.getFileLlsm() : 0));
            nxSparePartVos.get(i).setPartSfgjbj(one.getFileSfsgjpj());
            nxSparePartVos.get(i).setNxSparePartId(JEUuidUtil.uuid());
            spareFilerMapper.importSparePart(nxSparePartVos.get(i));

//            }
        }
        return true;
    }

    public String invalidateData(NxSparePartVo nspv) {

        if (StringUtils.isBlank(nspv.getPartSbbm())) {
            return ResourceUtil.getI18nMessage("60028");
        }

        if (StringUtils.isBlank(nspv.getPartAzwh())) {
            return ResourceUtil.getI18nMessage("60029");
        }

        if (StringUtils.isBlank(nspv.getPartSn())) {
            return ResourceUtil.getI18nMessage("60030");
        }

        if (StringUtils.isBlank(nspv.getPartAzrq())) {
            return ResourceUtil.getI18nMessage("60031");
        }

        if (StringUtils.isBlank(nspv.getPartGhrq())) {
            return ResourceUtil.getI18nMessage("60032");
        }

        if (StringUtils.isBlank(nspv.getPartSmyjz())) {
            return ResourceUtil.getI18nMessage("60033");
        }

        if (StringUtils.isBlank(nspv.getPartBjzt())) {
            return ResourceUtil.getI18nMessage("60034");
        }

        if (StringUtils.isBlank(nspv.getPartBjbm())) {
            return ResourceUtil.getI18nMessage("60035");
        }

        return "success";
    }


    /**
     * 工作时长相加
     *
     * @return
     */
    public String addGzsc(String math1, String math2) {
        BigDecimal s1 = new BigDecimal(math1);
        BigDecimal s2 = new BigDecimal(math2);
        BigDecimal decimal = s1.add(s2);
        return String.format("%.1f", decimal);
    }

    public String updateSparePart(String orderId) {
        spareFilerMapper.getSparePart(orderId).forEach(e -> {
            //1 是关键配件，0否
            if ("1".equals(e.getPartSfgjbj())) {
                //通过配件编码在配件档案中查询系统模块编码 jsq 打印头
                String xtmkbm = spareFilerMapper.getxtmkBypjbm(e);
                if (StringUtils.isNotBlank(xtmkbm)) {
                    if ("jsq".equals(xtmkbm)) {//打印头模块
                        //加一层判断逻辑,若设备长期不联网(>60天)认为是长期不联网设备则进行同步(打印头自动更新暂时停掉，已有自动检测更新任务了)
                        PrinterHeadRecordVO v1 = new PrinterHeadRecordVO();
                        v1.setTenantId(e.getSyZhid()); //624e7ce1549c110050c5da47
                        v1.setDeviceCode(e.getPartSbbm());
                        v1.setThingId(e.getPartSbid());
                        List<PrinterHeadRecordVO> recordList = recodeDao.printerHeadList(v1);
                        boolean isValid = false;
                        for(PrinterHeadRecordVO item:recordList){
                            String updateTime = item.getUpdatetime();
                            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                            LocalDate dateToCompare = LocalDate.parse(updateTime, formatter);
                            LocalDate currentDate = LocalDate.now();
                            long daysBetween = ChronoUnit.DAYS.between(dateToCompare, currentDate);
                            // 判断是否是30天之前,长期离线设备
                            if (daysBetween >= 30) {
                                isValid = true;
                                //System.out.println(updateTime + " 是60天之前的日期。");
                            } else {
                                //System.out.println(updateTime + " 不是60天之前的日期。");
                            }
                        }
                        //SN序列号读数异常、错误的、未采集SN数据的 需要工单同步
                        if(e.getSyZhid().equals("624e7ce1549c110050c5da47") && (e.getPartSbbm().equals("2218-3") || e.getPartSbbm().equals("2218-4")
                                || e.getPartSbbm().equals("2218-5")|| e.getPartSbbm().equals("SDreamC")|| e.getPartSbbm().equals("SDreamD")
                                || e.getPartSbbm().equals("Max-A")|| e.getPartSbbm().equals("Max-B")|| e.getPartSbbm().equals("Max-C")
                                || e.getPartSbbm().equals("2215-4")|| e.getPartSbbm().equals("2215-7")|| e.getPartSbbm().equals("2215-8")
                                || e.getPartSbbm().equals("2215-10")|| e.getPartSbbm().equals("2215-12")|| e.getPartSbbm().equals("2215-13")
                                || e.getPartSbbm().equals("2215-14")|| e.getPartSbbm().equals("2215-16")|| e.getPartSbbm().equals("2215-17")
                                || e.getPartSbbm().equals("2215-19")|| e.getPartSbbm().equals("2215-21")|| e.getPartSbbm().equals("2215-24")
                                || e.getPartSbbm().equals("2215-27")

                                || e.getPartSbbm().equals("1800-1")|| e.getPartSbbm().equals("1800-2")|| e.getPartSbbm().equals("1800-3")
                                || e.getPartSbbm().equals("1800-4")|| e.getPartSbbm().equals("1800-5")|| e.getPartSbbm().equals("1800-6")
                                || e.getPartSbbm().equals("1800-7")|| e.getPartSbbm().equals("1800-8")|| e.getPartSbbm().equals("1800-9")
                                || e.getPartSbbm().equals("1800-12")|| e.getPartSbbm().equals("1800-13")|| e.getPartSbbm().equals("1800-15")
                                || e.getPartSbbm().equals("1800-16")||  e.getPartSbbm().equals("1800-18")|| e.getPartSbbm().equals("1800-19")

                                || e.getPartSbbm().equals("1800E-2")|| e.getPartSbbm().equals("1800E-3")|| e.getPartSbbm().equals("1800E-5")
                                || e.getPartSbbm().equals("1800E-7")|| e.getPartSbbm().equals("1800E-9")|| e.getPartSbbm().equals("1800E-12")
                                || e.getPartSbbm().equals("1800E-13")|| e.getPartSbbm().equals("1800E-14")|| e.getPartSbbm().equals("1800E-17")
                                || e.getPartSbbm().equals("1800E-22")

                                //1800B-1(山东浩信)、1800B-2（海斯特）、1800B-3(陕西鑫源) 联网
                                || e.getPartSbbm().equals("1800B-4")|| e.getPartSbbm().equals("1800B-6")

                                || e.getPartSbbm().equals("2500-2")|| e.getPartSbbm().equals("2500-3")
                                || e.getPartSbbm().equals("2500-5")|| e.getPartSbbm().equals("2500-6")|| e.getPartSbbm().equals("2500-17")
                                || e.getPartSbbm().equals("2500-18")|| e.getPartSbbm().equals("2500-19")|| e.getPartSbbm().equals("2500-20")
                                || e.getPartSbbm().equals("2500-21")|| e.getPartSbbm().equals("2500-22")|| e.getPartSbbm().equals("2500-24")
                                || e.getPartSbbm().equals("2500-25")|| e.getPartSbbm().equals("2500-26")|| e.getPartSbbm().equals("2500-27")
                                || e.getPartSbbm().equals("2500-37")|| e.getPartSbbm().equals("2500-38")|| e.getPartSbbm().equals("2500-39")
                                || e.getPartSbbm().equals("2500-42")|| e.getPartSbbm().equals("2500-43")|| e.getPartSbbm().equals("2500-44")
                                || e.getPartSbbm().equals("2500-45")|| e.getPartSbbm().equals("2500-46")|| e.getPartSbbm().equals("2500-47")

                                //南通三责精密陶瓷有限公司(211)
                                || e.getPartSbbm().equals("2500-52")|| e.getPartSbbm().equals("2500C-1")|| e.getPartSbbm().equals("2500C-2")
                                || e.getPartSbbm().equals("800G-1")|| e.getPartSbbm().equals("5000-1") || e.getPartSbbm().equals("5000-2")

                                //安徽应流
                                || e.getPartSbbm().equals("2500-54")|| e.getPartSbbm().equals("2500-55")|| e.getPartSbbm().equals("2500-56")
                                || e.getPartSbbm().equals("2500-57")|| e.getPartSbbm().equals("2500-59")|| e.getPartSbbm().equals("2500-60")
                                || e.getPartSbbm().equals("2500-61")|| e.getPartSbbm().equals("2500-63")
                                || e.getPartSbbm().equals("2500-81")|| e.getPartSbbm().equals("2500-82")|| e.getPartSbbm().equals("2500-84")

                                || e.getPartSbbm().equals("2500-64")|| e.getPartSbbm().equals("2500-65")|| e.getPartSbbm().equals("2500-66")
                                || e.getPartSbbm().equals("2500-67") || e.getPartSbbm().equals("2500-68")
                                || e.getPartSbbm().equals("2500-69") || e.getPartSbbm().equals("2500-70")|| e.getPartSbbm().equals("2500-71")
                                || e.getPartSbbm().equals("2500-72")//六盘山
                                || e.getPartSbbm().equals("2500-73")//北京星航
                                || e.getPartSbbm().equals("2500-74")//那山（朝鲜）
                                || e.getPartSbbm().equals("2500-83")//通裕重工
                                //2500-87、88已联网 潍坊机床二厂

                                || e.getPartSbbm().startsWith("4600")//1、2、3、4、5
                                || e.getPartSbbm().startsWith("2800")//1、2、3、4
                                || e.getPartSbbm().startsWith("2600B")//1、2、3、4、5、6、7、8、9、10、11、12、13
                                || e.getPartSbbm().startsWith("300")//1、2、3、4
                                || e.getPartSbbm().startsWith("500")//1、2、3
                                //1200-5 已采集
                                || e.getPartSbbm().equals("1200-1") || e.getPartSbbm().equals("1200-2") || e.getPartSbbm().equals("1200-3") || e.getPartSbbm().equals("1200-4")
                                || e.getPartSbbm().startsWith("1000")
                                || e.getPartSbbm().equals("AE1000A")
                                || e.getPartSbbm().equals("800E-1")|| e.getPartSbbm().equals("800-1")|| e.getPartSbbm().equals("800-2")|| e.getPartSbbm().equals("800-3")
                                //AJM金属机
                                || e.getPartSbbm().equals("AJM160-1")|| e.getPartSbbm().equals("AJM160-2")|| e.getPartSbbm().equals("AJM160-3")|| e.getPartSbbm().equals("AJM160-4")
                                || e.getPartSbbm().equals("AJM160-5")|| e.getPartSbbm().equals("AJM160-6")|| e.getPartSbbm().equals("AJM160-7")
                                || e.getPartSbbm().equals("AJM500-1") || e.getPartSbbm().equals("AJM500-5") || e.getPartSbbm().equals("AJM500-10")
                                || e.getPartSbbm().equals("AJM800-1")
                        ))
                            isValid = true;

                        if(recordList.size() == 0)
                            isValid = true; //未更新程序、未采集到打印头序列号的设备，需要做同步
                        if(isValid) {  //长期不联网(>30天)认为是长期不联网则进行备件同步
                            //配将状态 1 正常  2 更换
                            //应该是判断当前位号下是否存在状态正常的记录，若存在，则修改旧记录状态(正常)为更换状态及更换时间，新增当前同步过来的备件记录(状态为正常、安装日期、更换日期、理论寿命等）；
                            List<NxSparePartVo> NxSparePartVos = spareFilerMapper.getSparePartByAzwh(e);
                            if (CollectionUtils.isNotEmpty(NxSparePartVos)) {
                                NxSparePartVos.stream().forEach(e1 -> {
                                    spareFilerMapper.updateNxSparePart(e1);
                                });
                                e.setNxSparePartId(UUID.randomUUID().toString().substring(0, 10));
                                e.setPartGhrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                                e.setPartAzrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                                e.setPartSmyjz("10"); //200H 8.3天≈10天
                                if(e.getPartGgxh().contains("QSWJM025630"))
                                    e.setPartSm("396");
                                if(e.getPartGgxh().contains("PQ系列"))
                                    e.setPartSm("321");
                                if(e.getPartGgxh().contains("SGAPH102480"))
                                    e.setPartSm("306");
                                if(e.getPartGgxh().contains("SGAPH102430"))
                                    e.setPartSm("297");
                                if(e.getPartGgxh().contains("喷头WX"))
                                    e.setPartSm("90");
                                if(e.getPartBz().equals(""))
                                    e.setPartBz("人工更换");
                                else
                                    e.setPartBz("人工更换"+","+e.getPartBz());
                                spareFilerMapper.insertSparePart(e);
                            } else {
                                e.setNxSparePartId(UUID.randomUUID().toString().substring(0, 10));
                                e.setPartGhrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                                e.setPartAzrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                                e.setPartSmyjz("10"); //200H 8.3天≈10天
                                if(e.getPartGgxh().contains("QSWJM025630"))
                                    e.setPartSm("396");
                                if(e.getPartGgxh().contains("PQ系列"))
                                    e.setPartSm("321");
                                if(e.getPartGgxh().contains("SGAPH102480"))
                                    e.setPartSm("306");
                                if(e.getPartGgxh().contains("SGAPH102430"))
                                    e.setPartSm("297");
                                if(e.getPartGgxh().contains("喷头WX"))
                                    e.setPartSm("90");
                                if(e.getPartBz().equals(""))
                                    e.setPartBz("人工更换");
                                else
                                    e.setPartBz("人工更换"+","+e.getPartBz());
                                spareFilerMapper.insertSparePart(e);
                            }
                        }
                        /*
                        //配将状态 1 正常  2 更换
                        //应该是判断当前位号下是否存在状态正常的记录，若存在，则修改旧记录状态(正常)为更换状态及更换时间，新增当前同步过来的备件记录(状态为正常、安装日期、更换日期、理论寿命等）；
                        List<NxSparePartVo> NxSparePartVos = spareFilerMapper.getSparePartByAzwh(e);
                        if (CollectionUtils.isNotEmpty(NxSparePartVos)) {
                            NxSparePartVos.stream().forEach(e1 -> {
                                spareFilerMapper.updateNxSparePart(e1);
                            });
                            e.setNxSparePartId(UUID.randomUUID().toString().substring(0, 10));
                            e.setPartGhrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                            e.setPartAzrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                            spareFilerMapper.insertSparePart(e);
                        } else {
                            e.setNxSparePartId(UUID.randomUUID().toString().substring(0, 10));
                            e.setPartGhrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                            e.setPartAzrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                            spareFilerMapper.insertSparePart(e);
                        }*/
                    }
                    else {
                        //其他非打印头备件 通过设备编码和配件编码查询是否存在该配件
                        String nxSparePartId = spareFilerMapper.getSparePartBySbbm(e);
                        if (StringUtils.isNotEmpty(nxSparePartId)) {
                            NxSparePartVo nxSparePartVo = new NxSparePartVo();
                            nxSparePartVo.setNxSparePartId(nxSparePartId);
                            spareFilerMapper.updateNxSparePart(nxSparePartVo);
                            e.setNxSparePartId(UUID.randomUUID().toString().substring(0, 10));
                            e.setPartGhrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                            e.setPartAzrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                            spareFilerMapper.insertSparePart(e);
                        } else {
                            e.setNxSparePartId(UUID.randomUUID().toString().substring(0, 10));
                            e.setPartGhrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                            e.setPartAzrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                            spareFilerMapper.insertSparePart(e);
                        }
                    }
                }
            }
        });
        return null;
    }

    public void dealWith(String sbId,String partId) {
        //维修订单关联NX_BGB_WXHJ  拿到租户id，设备编码，设备id，审核通过时间 和 NX_BGB_WXHJ 的 是关键配件 按照审核时间顺序
        List<NxSparePartVo> list = spareFilerMapper.getOrderData(sbId,partId);
        if (list.size() > 0) {
            list.forEach(o -> {
                //设置寿命预警值
                if (StringUtils.isBlank(o.getPartSmyjz())) {
                    o.setPartSmyjz("200");
                }
                Log.info("获取到数据：" + o);
                //通过设备id + 配件编码查询NX_SPARE_PART 的设备id 和配件编码下是否有记录；如果无，则根据设备id + 配件编码+ 审核通过时间 新增一条记录，
                Map<String, Object> maps = new HashMap<>();
                maps.put("partSbid", o.getPartSbid());
                maps.put("partAzrq", o.getOrderShtgsj());
                maps.put("partAzwh", o.getPartAzwh());
                List<NxSparePartEntity> lists = spareFilerMapper.getDataByPartSbidAndPartBjbm(maps);
                Log.info("获取到数据lists：" + lists);
                if (lists.size() > 0) {
                    //存在数据
                    //通过设备id + 配件编码+ 审核通过时间  查询NX_SPARE_PART 的设备id 和配件编码 +创建时间 匹配是否出现过，
                    List<NxSparePartEntity> list2 = spareFilerMapper.getDataByPartSbidAndPartBjbmAndPartAzrq(maps);
                    Log.info("获取到数据list2：" + list2);
                    if (list2.size() == 0) {
                        //没出现则添加计算时间和寿命 --取已有记录中最近时间的记录 ，此记录更换日期替换为 新插入记录订单的审核通过时间，
                        //用安装时间和 更新时间去查询 设备运行时间表，统计出 安装时间到更新时间段内的运行时间，替换掉现有的历史 工作时长；
                        //新增一条记录 ，需插入的订单审核通过时间 作为安装日期，当前日期为更新时间，用安装时间和 更新时间去查询 设备运行时间表，
                        //统计出 安装时间到更新时间段内的运行时间，替换掉现有的历史 工作时长；
                        NxSparePartEntity nxSparePartEntity = lists.get(0);
                        Log.info("获取到数据nxSparePartEntity：" + nxSparePartEntity);
                        //查询该配件工作时长
                        String hours = spareFilerMapper.getWorkingHours(o.getPartSbid(), nxSparePartEntity.getPartAzrq(), o.getOrderShtgsj());
                        Log.info("获取到数据hours：" + hours);
                        spareFilerMapper.updateData(nxSparePartEntity.getNxSparePartId(), o.getOrderShtgsj(), hours);
                        //新增
                        //不存在数据，新增操作
                        o.setNxSparePartId(UUID.randomUUID().toString().substring(0, 10));
                        o.setPartAzrq(o.getOrderShtgsj());
                        o.setPartGhrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                        //查询配件工作时长
                        String hours2 = spareFilerMapper.getWorkingHours(o.getPartSbid(), o.getPartAzrq(), o.getPartGhrq());
                        Log.info("获取到数据hours2：" + hours2);
                        o.setPartGzsc(hours2);
                        Log.info("insertSparePart：" + o);
                        spareFilerMapper.insertSparePart(o);
                    }

                } else {
                    //不存在数据，新增操作
                    o.setNxSparePartId(UUID.randomUUID().toString().substring(0, 10));
                    o.setPartAzrq(o.getOrderShtgsj());
                    o.setPartGhrq(DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
                    //查询配件工作时长
                    String hours = spareFilerMapper.getWorkingHours(o.getPartSbid(), o.getPartAzrq(), o.getPartGhrq());
                    Log.info("获取到数据hours：" + hours);
                    o.setPartGzsc(hours);
                    Log.info("insertSparePart：" + o);
                    spareFilerMapper.insertSparePart(o);
                }
            });
        }
    }


    public void dealWithBySbId(String sbId) {

    }
}
