package com.rootcloud.domain.service.fittings.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.constant.TemplateConstant;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.domain.dto.NxSparePartVo;
import com.rootcloud.domain.entity.MyAdviceEntity;
import com.rootcloud.domain.entity.NxSparePartEntity;
import com.rootcloud.domain.entity.NxTodoLibrary;
import com.rootcloud.domain.entity.ds.DeviceListEntity;
import com.rootcloud.domain.mapper.NxOrderMapper;
import com.rootcloud.domain.mapper.SpareFilerMapper;
import com.rootcloud.domain.mapper.SparePartMapper;
import com.rootcloud.domain.service.fittings.SparePartService;
import com.rootcloud.domain.service.todo.NxTodoLibraryService;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

@Service("sparePartServiceImpl")
public class SparePartServiceImpl extends ServiceImpl<SparePartMapper, NxSparePartEntity> implements SparePartService {
    @Resource
    private NxOrderMapper nxOrderMapper;
    @Resource(name = "nxTodoLibraryServiceImpl")
    private NxTodoLibraryService nxTodoLibraryService;

    @Resource
    private SparePartMapper sparePartMapper;
    @Autowired
    SpareFilerMapper spareFilerMapper;

    @Override
    public void uptPartSmyj() {
        List<NxSparePartEntity> udtList = new ArrayList<>();
        BigDecimal sdf = new BigDecimal("0.00");
        List<NxSparePartEntity> list = this.list();
        if (list == null) {
            return;
        }
        list.stream().forEach(temp -> {
            BigDecimal workTimes = new BigDecimal(temp.getPartGzsc() == null ? 0 : temp.getPartGzsc());
            if (temp.getPartSm() != null && temp.getPartSmyjz() != null) {
                BigDecimal lifeTotalTimes = new BigDecimal(temp.getPartSm()).setScale(2, BigDecimal.ROUND_HALF_UP);
                BigDecimal warnTimes = new BigDecimal(temp.getPartSmyjz()).setScale(2, BigDecimal.ROUND_HALF_UP);
                String partYjsj = temp.getPartYjsj();
                if (lifeTotalTimes.subtract(workTimes).compareTo(warnTimes) <= 0) {
                    if (StringUtils.isBlank(partYjsj)) {
                        NxSparePartEntity nxSparePartEntity = new NxSparePartEntity();
                        nxSparePartEntity.setNxSparePartId(temp.getNxSparePartId());
                        //0000代表该配件寿命预警，仅用于后端程序判断是否配件预警
                        nxSparePartEntity.setPartYjzt("0000");
                        nxSparePartEntity.setPartYjsj(DateUtil.format(new Date()));
                        nxSparePartEntity.setPartSfjj("0");
                        nxSparePartEntity.setPartSbbm(temp.getPartSbbm());
                        nxSparePartEntity.setPartBjmc(temp.getPartBjmc());
                        udtList.add(nxSparePartEntity);
                    }
                }
            }
        });
        this.updateBatchById(udtList);
    }

    public static void main(String[] args) {
        BigDecimal workTimes = new BigDecimal(4800);
        BigDecimal lifeTotalTimes = new BigDecimal(5000);
        BigDecimal warnTimes = new BigDecimal(200);
        if (lifeTotalTimes.subtract(workTimes).compareTo(warnTimes) <= 0) {
            System.out.println("aaa");
        }
    }

    /**
     * 添加分页参数
     *
     * @param devices
     * @param startNo
     * @param endNo
     * @return
     */
    @Override
    public List<MyAdviceEntity> getWarnedSparePart(List<DeviceListEntity> devices, Integer startNo, Integer endNo, String zhid) {
        //获取模板
        String contentData = nxOrderMapper.getContentByMbId(TemplateConstant.TEMPLATE_SPARE);
        List<MyAdviceEntity> result = new ArrayList<>();
        List<String> deviceCodes = devices.stream().map(DeviceListEntity::getNxJcxxSblbId).collect(Collectors.toList());

        HashMap<String, Object> req = new HashMap<>();
        req.put("startTime", DateUtil.format(DateUtil.rollDay(new Date(), -3)));
        req.put("endTime", DateUtil.format(new Date()));
        req.put("deviceCodes", deviceCodes);
        req.put("startNo", startNo);
        req.put("endNo", endNo);
        req.put("zhid", zhid);
//        Log.info("寿命预警通知查询前设备id"+ JSON.toJSONString(deviceCodes));
        List<NxSparePartEntity> sparePartEntityList = nxOrderMapper.getSpartPartList(req);
        //Log.info("获取配件数据："+sparePartEntityList.size());
        /*LambdaQueryWrapper<NxSparePartEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(NxSparePartEntity.class);
        lambdaQueryWrapper.in(NxSparePartEntity::getPartSbid,deviceCodes)
                .eq(NxSparePartEntity::getPartYjzt, "0000")
                .and(wq -> {
                    wq.ne(NxSparePartEntity::getPartSfjj, "1")
                            .or().isNull(NxSparePartEntity::getPartSfjj);
                })
                //.ne(NxSparePartEntity::getPartSfjj,"1")
                .between(NxSparePartEntity::getPartYjsj,DateUtil.format(DateUtil.rollDay(new Date(),-3)),DateUtil.format(new Date()))
                .inSql(NxSparePartEntity::getPartSbbm,"select sblb_sbbm from nx_jcxx_sblb where nx_jcxx_sblb.nx_jcxx_sblb_id = nx_spare_part.part_sbid");
        List<NxSparePartEntity> sparePartEntityList = this.list(lambdaQueryWrapper);*/
        if (CollectionUtil.isEmpty(sparePartEntityList)) {
            return result;
        }
        sparePartEntityList.stream().forEach(temp -> {
            String content = contentData;
            MyAdviceEntity myAdviceEntity = new MyAdviceEntity();
            myAdviceEntity.setServicePlanId(temp.getNxSparePartId());//主键id
            myAdviceEntity.setAdviceClass(ResourceUtil.getI18nMessage("60005")); //备件寿命预警提醒
            myAdviceEntity.setAdviceTime(temp.getPartYjsj());
            myAdviceEntity.setAdviceName(temp.getPartBjmc() + ResourceUtil.getI18nMessage("60006")); //喷头/QSWJM025630-BGS--0140F寿命提醒
            myAdviceEntity.setAdvicObjectCode(temp.getPartSbbm());
            //寿命预警计算四舍五入保留两位小数   20241012whl
            //BigDecimal[] days = new BigDecimal(temp.getPartSm()).subtract(new BigDecimal(temp.getPartGzsc())).setScale(2, RoundingMode.HALF_UP).divideAndRemainder(new BigDecimal(24));
            BigDecimal[] days = new BigDecimal(temp.getPartSm()).subtract(new BigDecimal(temp.getPartGzsc())).setScale(1, RoundingMode.HALF_UP).divideAndRemainder(new BigDecimal(1));
            if (StringUtils.isNotBlank(content) && StringUtils.isNotEmpty(temp.getPartSbbm())) {
                content = content.replace("【" + ResourceUtil.getI18nMessage("60007") + "】", temp.getPartSbbm());
                myAdviceEntity.setAdviceContent(content);
            } else {
                //1800-7 寿命剩余时长 8.00天18.00小时 ->1800-7 寿命剩余时长 8.0天
                //myAdviceEntity.setAdviceContent(temp.getPartSbbm() + ResourceUtil.getI18nMessage("60008") + days[0] + ResourceUtil.getI18nMessage("60009") + days[1] + ResourceUtil.getI18nMessage("60010"));
                myAdviceEntity.setAdviceContent(temp.getPartSbbm() + ResourceUtil.getI18nMessage("60008") + days[0] + ResourceUtil.getI18nMessage("60009"));
                String adviceContent = myAdviceEntity.getAdviceContent().replace("【" + ResourceUtil.getI18nMessage("60007") + "】", temp.getPartSbbm());
                myAdviceEntity.setAdviceContent(adviceContent);

                if (StringUtils.isNotEmpty(temp.getPartSbbm())) {
                    //1800-7寿命剩余时长8.00天18.00小时安装位号23SN09723J05080->1800-7 寿命剩余时长 8.0天安装位号23,SN:09723J05080
                    //myAdviceEntity.setAdviceContent(temp.getPartSbbm() + ResourceUtil.getI18nMessage("60016") + days[0] + ResourceUtil.getI18nMessage("60009") + days[1] + ResourceUtil.getI18nMessage("60010") + ResourceUtil.getI18nMessage("60017") + temp.getPartAzwh() + "SN" + temp.getPartSn());
                    myAdviceEntity.setAdviceContent(temp.getPartSbbm() + ResourceUtil.getI18nMessage("60016") + days[0] + ResourceUtil.getI18nMessage("60009")  + ","+ResourceUtil.getI18nMessage("60017") + temp.getPartAzwh() + ",SN:" + temp.getPartSn());
                    String adviceContent1 = myAdviceEntity.getAdviceContent().replace("【" + ResourceUtil.getI18nMessage("60007") + "】", temp.getPartSbbm());
                    myAdviceEntity.setAdviceContent(adviceContent1);
                } else {
                    //1800-7寿命剩余时长8.00天18.00小时安装位号23SN09723J05080->1800-7 寿命剩余时长 8.0天安装位号23,SN:09723J05080
                    //myAdviceEntity.setAdviceContent(ResourceUtil.getI18nMessage("60016") + days[0] + ResourceUtil.getI18nMessage("60009") + days[1] + ResourceUtil.getI18nMessage("60010") + ResourceUtil.getI18nMessage("60017") + temp.getPartAzwh() + "SN" + temp.getPartSn());
                    myAdviceEntity.setAdviceContent(ResourceUtil.getI18nMessage("60016") + days[0] + ResourceUtil.getI18nMessage("60009") + ","+ResourceUtil.getI18nMessage("60017") + temp.getPartAzwh() + ",SN:" + temp.getPartSn());
                }
            }
            if (StringUtils.isNotEmpty(temp.getPartSbbm())) {
                myAdviceEntity.setAdviceObjectName(getDeviceName(temp, devices));
            }
            myAdviceEntity.setTypeCode(ServicePlanEnums.MessageType.SPARE_PARTS_LIFE_WARNING_REMINDER.getCode());
            myAdviceEntity.setAdviceCode(ServicePlanEnums.MessageCodeType.SPARE_PARTS_LIFE.getCode());
            myAdviceEntity.setSignData(temp.getNxSparePartId());
            myAdviceEntity.setSn(temp.getPartSn());
            myAdviceEntity.setPartAzwh(temp.getPartAzwh());
            myAdviceEntity.setPartGzsc(temp.getPartGzsc().toString());//工作时长
            myAdviceEntity.setPartXtmk(temp.getPartXtmk());//系统模块
            myAdviceEntity.setPartBjmc(temp.getPartBjmc());//备件名称
            List<NxTodoLibrary> nxTodoLibraries = nxTodoLibraryService.selectBySign(myAdviceEntity.getAdviceCode(), myAdviceEntity.getSignData(), myAdviceEntity.getAdviceTime());
            if (nxTodoLibraries.isEmpty()) {
                result.add(myAdviceEntity);
            }
        });
        return result;
    }

    @Override
    public void eqMesSfjj(String id) {
        nxOrderMapper.eqMesSfjj(id);
    }

    /**
     * 自动填充租户ID
     */
    @Override
    public void paddingZhid() {
        sparePartMapper.updateZhid();
    }

    @Override
    public void syncWorkingTime() {
        List<NxSparePartVo> list = sparePartMapper.selectNoSyncPart();
        //1.先刷更换日期
        if(CollectionUtil.isNotEmpty(list)){
            list.stream().parallel().forEach(e->{
                sparePartMapper.updateWorkTime(e.getNxSparePartId());
                //sparePartMapper.updateWorkTime(e.getNxSparePartId(),new BigDecimal(StrUtil.isNotEmpty(workingHours)?workingHours:"0"));
            });
        }
        //2.再刷运行时长
        List<NxSparePartVo> list2 = sparePartMapper.selectNoSyncPart();
        if(CollectionUtil.isNotEmpty(list2)){
            list2.stream().parallel().forEach(e->{
                //String workingHours = spareFilerMapper.getWorkingHours(e.getPartSbid(), e.getPartAzrq(), e.getPartGhrq());
                //String days = workingHours==""?"0":String.format("%.1f",Double.valueOf(workingHours)/24); //20241012
                LocalDate date1 = LocalDate.parse(e.getPartGhrq());
                LocalDate date2 = LocalDate.parse(e.getPartAzrq());
                long daysDiff = ChronoUnit.DAYS.between(date2, date1);
                String days = String.valueOf(Math.abs(daysDiff));//自然天数 20241012
                sparePartMapper.updateWorkTime2(e.getNxSparePartId(),new BigDecimal(StrUtil.isNotEmpty(days)?days:"0"));
                //sparePartMapper.updateWorkTime(e.getNxSparePartId(),new BigDecimal(StrUtil.isNotEmpty(workingHours)?workingHours:"0"));
            });
        }
    }


    /**
     * 根据设备编码获取设备名字
     */
    public String getDeviceName(NxSparePartEntity entity, List<DeviceListEntity> deviceListEntities) {
        String result = "";
        for (DeviceListEntity deviceListEntity : deviceListEntities) {
            if (entity.getPartSbbm().equals(deviceListEntity.getSblbSbbm())) {
                result = deviceListEntity.getSblbSbmc();
                break;
            }
        }
        return result;
    }
}
