package com.rootcloud.domain.service.gbsj.impl;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.JsonPageResult;
import com.rootcloud.domain.dao.ModelPropertyDao;
import com.rootcloud.domain.entity.base.ModelPropertyEntity;
import com.rootcloud.domain.mapper.ModelPropertyMapper;
import com.rootcloud.domain.service.gbsj.ModelPropertyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ruixiang.feng
 * @ClassName
 * @Description
 * @date 2022/4/15 10:46
 */
@Service("propertyService")
@Slf4j
public class ModelPropertyServiceImpl extends ServiceImpl<ModelPropertyDao, ModelPropertyEntity> implements ModelPropertyService {
    @Autowired
    private ModelPropertyMapper modelPropertyMapper;

    protected static final String CODE_SUCCESS = "200";
    private static final String CODE_CLIENT_ERROR = "500";
    @Override
    public Map<String, List<ModelPropertyEntity>> getWorkingCondition(String deviceSblx, String deviceSbxh, String workingConditionType,String dataSource) {
        HashMap<String, String> req = new HashMap<>();
        req.put("deviceSblx",deviceSblx);
        req.put("deviceSbxh",deviceSbxh);
        req.put("workingConditionType",workingConditionType);
        req.put("dataSource",dataSource);
        List<ModelPropertyEntity> workingCondition = modelPropertyMapper.getWorkingCondition(req);
        Map<String, List<ModelPropertyEntity>> collect = workingCondition.stream().collect(Collectors.groupingBy(ModelPropertyEntity::getPropertyTag));
        List<String> titles = modelPropertyMapper.getWorkingTitle(req);

        LinkedHashMap<String, List<ModelPropertyEntity>> linkedHashMap = new LinkedHashMap<>();
        titles.forEach(item->{
            List<ModelPropertyEntity> list = collect.get(item);
            if(list!=null){
                linkedHashMap.put(item,collect.get(item));
            }
        });
        return linkedHashMap;
    }

    @Override
    @Transactional
    public JsonPageResult updateWorkingCondition(List<ModelPropertyEntity> modelPropertyEntityList) {
        JsonPageResult jsonPageResult = new JsonPageResult(CODE_SUCCESS);
        if(modelPropertyEntityList.size() == 0){
            jsonPageResult.setCode(CODE_CLIENT_ERROR);
        }

        Map<String, List<ModelPropertyEntity>> collect = modelPropertyEntityList.stream().collect(Collectors.groupingBy(ModelPropertyEntity::getPropertyTag));
        for (String key:collect.keySet()) {
            List<ModelPropertyEntity> modelPropertyEntitieList = collect.get(key);
            List<ModelPropertyEntity> collect1 = modelPropertyEntitieList
                    .stream().filter(e -> StringUtils.isNotBlank(e.getPropertyTagCode())).collect(Collectors.toList());
            ModelPropertyEntity modelPropertyEntity = modelPropertyEntitieList.get(0);
            if(collect1.size() == 0){ //全部都是新增
                String uuid = UUID.randomUUID().toString().trim().replaceAll("-", "");
                modelPropertyEntitieList.forEach(e->{
                    e.setPropertyTagCode(uuid);
                });
                ModelPropertyEntity entity = new ModelPropertyEntity();
                entity.setDataSource(modelPropertyEntity.getDataSource());
                entity.setDeviceSblx(modelPropertyEntity.getDeviceSblx());
                entity.setDeviceSbxh(modelPropertyEntity.getDeviceSbxh());
                entity.setPropertyTag(modelPropertyEntity.getPropertyTag());
                entity.setPropertyTagCode(modelPropertyEntity.getPropertyTagCode());
                entity.setWorkingConditionType(modelPropertyEntity.getWorkingConditionType());
                entity.setDataSjy(modelPropertyEntity.getDataSjy());
                entity.setCreateTime(DateUtil.now());
                entity.setUpdateTime(DateUtil.now());
                modelPropertyEntitieList.add(entity);
                modelPropertyMapper.batchinsert(modelPropertyEntitieList);
            }else{//部分新增 部分修改
                /*List<ModelPropertyEntity> modelPropertyEntities = queryWorkingCondition(collect1.get(0).getPropertyTag(), collect1.get(0).getWorkingConditionType());
                if(modelPropertyEntities.size() > 0){
                    List<String> idList = modelPropertyEntities.stream().map(e -> e.getId()).collect(Collectors.toList());
                    modelPropertyMapper.deleteByIds(idList);
                    //delWorkingCondition(collect1.get(0).getPropertyTagCode(),collect1.get(0).getWorkingConditionType());
                }*/
                //通过以后的数据标题code删除之前的数据
                ModelPropertyEntity modelPropertyEntity1 = collect1.get(0);
                modelPropertyMapper.deleteByCode(modelPropertyEntity1.getPropertyTagCode(),"1");
                //modelPropertyMapper.del(modelPropertyEntity1);
                //List<String> idList = modelPropertyEntitieList.stream().filter(e -> StringUtils.isNotEmpty(e.getId())).map(e -> e.getId()).collect(Collectors.toList());
                //modelPropertyMapper.deleteByIds(idList);
//                String uuid = UUID.randomUUID().toString().trim().replaceAll("-", "");
                modelPropertyEntitieList.forEach(e->{
                    e.setPropertyTagCode(modelPropertyEntity1.getPropertyTagCode());
                });
                modelPropertyMapper.batchinsert(modelPropertyEntitieList);
            }
        }

        return jsonPageResult;
    }

    @Override
    public JsonPageResult delWorkingCondition(String propertyTagCode,String workingConditionType) {
        HashMap<String, String> req = new HashMap<>();
        req.put("propertyTagCode",propertyTagCode);
        req.put("workingConditionType",workingConditionType);
        //modelPropertyMapper.delWorkingCondition(req);
        modelPropertyMapper.deleteByCode(propertyTagCode,null);
        return new JsonPageResult(CODE_SUCCESS);
    }

    public List<ModelPropertyEntity> queryWorkingCondition(String propertyTag,String workingConditionType) {
        HashMap<String, String> req = new HashMap<>();
        req.put("propertyTag",propertyTag);
        req.put("workingConditionType",workingConditionType);
        return modelPropertyMapper.queryWorkingCondition(req);
    }

    @Override
    public List<ModelPropertyEntity> getGroupAtt(String workingConditionType, String propertyTagCode,String deviceSblx, String deviceSbxh) {
        HashMap<String, String> req = new HashMap<>();
        req.put("workingConditionType",workingConditionType);
        req.put("propertyTagCode",propertyTagCode);
        req.put("deviceSblx",deviceSblx);
        req.put("deviceSbxh",deviceSbxh);
        if(StringUtils.isNotEmpty(propertyTagCode)){
            return modelPropertyMapper.getGroupAttData(req);
        }
        return modelPropertyMapper.getGroupAtt(req);
    }

    @Override
    public List<ModelPropertyEntity> getmodelPropertyBySbxh(List<String> sbxh) {
        return modelPropertyMapper.getmodelPropertyBySbxh(sbxh);
    }
}
