package com.rootcloud.domain.service.gkjk;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.rootcloud.common.JsonResult;
import com.rootcloud.domain.ao.ExportHistorySituationAO;
import com.rootcloud.domain.ao.HistorySituationAO;
import com.rootcloud.domain.dto.DeviceVO;
import com.rootcloud.domain.entity.NxJeDeviceEntity;
import com.rootcloud.domain.model.WorkStatusMonitorModel;
import com.rootcloud.vo.device.DeviceHistorySituationVO;
import com.rootcloud.vo.device.DeviceStatusVO;
import com.rootcloud.vo.device.DeviceTypeVO;
import com.rootcloud.vo.resp.OtherDeviceResp;
import com.rootcloud.vo.rest.DevicePropertyValueParam;
import com.rootcloud.vo.rest.DeviceTrendParam;

import java.util.List;
import java.util.Map;

/**
 * 宁夏_设备台账
 *
 * @author lipingan
 * @email lpa2273597772@163.com
 * @date 2022-04-18 15:19:58
 */
public interface NxJeDeviceService extends IService<NxJeDeviceEntity> {

    Map<String, List<WorkStatusMonitorModel>> queryWorkStatus(String deviceId);

    Map<String, List<WorkStatusMonitorModel>> queryWorkStatus(String deviceId,String dataSjy);

    Map<String, List<WorkStatusMonitorModel>> queryMonitoringData(String deviceId);
    /**
     * @Author w-liuyi01
     * @Description //获取历史工况列表数据
     * @Date 2022/4/20 15:42
     **/
    JsonResult<DeviceHistorySituationVO> queryHistorySituation(HistorySituationAO historySituationAO);


    JsonResult<DeviceHistorySituationVO> exportQueryHistorySituation(ExportHistorySituationAO historySituationAO);

    List<OtherDeviceResp> queryOtherDevices(String deviceId);

    JSONObject queryWorkTrendData(DeviceTrendParam deviceTrendParam);

    /**
     * 查询所有的设备Id
     *
     * @return 设备Id列表
     */
    List<NxJeDeviceEntity> queryAllJeDevice();


    /**
     * 通过设别id 和工况码list返回对应的map
     * @param deviceId
     * @param paramsList
     * @return
     */
    Map<String,Object> getWorkingMap(String deviceId,List<String> paramsList);


    Map<String, Object> getRootcloudWorkingMap(String deviceId, List<String> paramsList);


    /**
     * 通过模型查询数据
     * @param model
     * @param deviceIds
     * @param paramsList
     * @return
     */
    Map<String,Map<String,Object>> getWorkingMapByModelGroup(String model,List<String> deviceIds, List<String> paramsList);


    /**
     * 通过设备Id 获取设备首页详情
     * @param deviceId
     * @return
     */
    DeviceTypeVO  getByDeviceId(String deviceId);


    /**
     * get
     * @param deviceId
     * @return
     */
    DeviceStatusVO getStatusByDeviceId(String deviceId);


    /**
     * 根据前台指定的工况和设备查询数据
     */
    Map<String,Object> queryPropertyValueByParam(DevicePropertyValueParam devicePropertyValueParam);

    /**
     * 查询设备状态
     *
     * @param queryData
     *            查询条件(支持地点、状态、型号、名称)
     * @param isCare
     *            是否重要
     * @param page
     *            当前页
     * @param size
     *            每页显示数
     * @param userId
     * @return 分页数据
     */
    IPage<DeviceVO> devicePage(String queryData, String isCare, int page, int size, String userId,String syZhid);

    /**
     * 查询设备是否在线
     * @param deviceNo 设备编码
     * @return true是
     */
    Boolean queryOnlineStatus(String deviceNo);


    /**
     * 通过设备编码查询设备信息
     * @param sbbm
     * @return
     */
    NxJeDeviceEntity selectBySbbm(String sbbm);


    /**
     * 获取实时工况信息
     * @param jeDeviceEntity
     * @return
     */
    Map<String, List<WorkStatusMonitorModel>> getrealTimeWork(NxJeDeviceEntity jeDeviceEntity);


    /**
     * 修改入云时间
     */
    void updateOnlineTime(String deviceIds);

}

