package com.rootcloud.domain.service.gkjk.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpStatus;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageInfo;
import com.rootcloud.common.*;
import com.rootcloud.common.enums.DeviceOnlineStatus;
import com.rootcloud.common.enums.HttpRequestMethedEnum;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.common.util.GaoDeAPIUtils;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.ao.ExportHistorySituationAO;
import com.rootcloud.domain.ao.HistorySituationAO;
import com.rootcloud.domain.dao.gkjk.NxJeDeviceDao;
import com.rootcloud.domain.dto.DeviceVO;
import com.rootcloud.domain.entity.*;
import com.rootcloud.domain.entity.base.ModelPropertyEntity;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.entity.device.NxDeviceLocationEntity;
import com.rootcloud.domain.mapper.ModelPropertyMapper;
import com.rootcloud.domain.mapper.NxDeviceLocationMapper;
import com.rootcloud.domain.mapper.NxDeviceMapper;
import com.rootcloud.domain.mapper.NxOrderMapper;
import com.rootcloud.domain.model.WorkStatusMonitorModel;
import com.rootcloud.domain.service.device.DeviceAlarmService;
import com.rootcloud.domain.service.ds.ModelPropertySyncService;
import com.rootcloud.domain.service.gkjk.NxJeDeviceService;
import com.rootcloud.vo.device.*;
import com.rootcloud.vo.resp.OtherDeviceResp;
import com.rootcloud.vo.rest.DevicePropertyValueParam;
import com.rootcloud.vo.rest.DeviceTrendParam;
import groovy.util.logging.Slf4j;
import jline.internal.Log;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


@Service("nxJeDeviceServiceImpl")
@Slf4j
public class NxJeDeviceServiceImpl extends ServiceImpl<NxJeDeviceDao, NxJeDeviceEntity> implements NxJeDeviceService {


    @Autowired
    private GaoDeAPIUtils gaoDeAPIUtils;

    @Autowired
    private NxDeviceLocationMapper deviceLocationMapper;
    @Autowired
    private TokenUtil tokenUtil;
    @Resource(name = "modelPropertySyncService2")
    private ModelPropertySyncService modelPropertySyncService;

    @Value("${os.iam.superToken}")
    private String superToken;

    @Value("${os.iam.domain}")
    private String reqUrl;

    @Value(value = "${cms.thingmodel.uri}")
    private String thingModelUri;

    @Value("${cms.propertystatus.uri}")
    private String propertyStatusUrl;

    @Value(value = "${os.iam.domain}")
    private String domain;

    @Value(value = "${cms.thing.uri}")
    private String thingUri;

    @Autowired
    private ModelPropertyMapper modelPropertyMapper;

    @Autowired
    private NxJeDeviceDao nxJeDeviceDao;

    @Resource(name = "deviceAlarmServiceImpl")
    private DeviceAlarmService deviceAlarmService;

    private final NxDeviceMapper nxDeviceMapper;
    private final NxOrderMapper nxOrderMapper;

    public NxJeDeviceServiceImpl(NxDeviceMapper nxDeviceMapper, NxOrderMapper nxOrderMapper) {
        this.nxDeviceMapper = nxDeviceMapper;
        this.nxOrderMapper = nxOrderMapper;
    }

    @Override
    public Map<String, List<WorkStatusMonitorModel>> queryWorkStatus(String deviceId) {
        return queryModelProperty(deviceId, false, null);
    }

    @Override
    public Map<String, List<WorkStatusMonitorModel>> queryWorkStatus(String deviceId, String dataSjy) {
        return queryModelProperty(deviceId, false, dataSjy);
    }

    @Override
    public Map<String, List<WorkStatusMonitorModel>> queryMonitoringData(String deviceId) {
        return queryModelProperty(deviceId, true, null);
    }

    @Override
    public JsonResult queryHistorySituation(HistorySituationAO historySituationAO) {
        //页数处理
        Integer skip = historySituationAO.getPageSize() * (historySituationAO.getCurrentPage() - 1);
        Integer limit = historySituationAO.getPageSize();

//        Integer skip = historySituationAO.getCurrentPage();
//        Integer limit = historySituationAO.getPageSize();
        //根据设备id查询设备
        /*NxJeDeviceEntity jeDeviceEntity = this.getById(historySituationAO.getDeviceId());
        if (ObjectUtils.isEmpty(jeDeviceEntity)) {
            //设备不存在
            throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(),StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
        }*/
        String stTime = DateUtil.handlerTime(historySituationAO.getStartTime());
        String enTime = DateUtil.handlerTime(historySituationAO.getEndTime());
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("startTime", stTime);
        paramMap.put("endTime", enTime);
        paramMap.put("skip",skip);
        paramMap.put("limit",limit);
        paramMap.put("sort","desc");
        //调用第三方接口获取历史工况数据信息列表
        List<DeviceHistorySituationVO> deviceHistorySituationVO = callHistoryInterface1(historySituationAO.getDeviceId(), null, paramMap);
        Log.info("历史工况数据查询数据:"+deviceHistorySituationVO.size());
        Map<String, Object> countMap = new HashMap<>(16);
        countMap.put("startTime", stTime);
        countMap.put("endTime", enTime);
//        String[] properties = {"__cloud_time__"};
        countMap.put("properties", "[\"__cloud_time__\"]");
        //查询总条数
        Integer count = queryHistoryCount(historySituationAO.getDeviceId(), null, countMap);
        NxJeDeviceEntity jeDeviceEntity = this.getById(historySituationAO.getDeviceId());
//        PageInfo<DeviceHistorySituationVO> deviceHistorySituationVOPageInfo = this.queryPageInfo(skip, limit, deviceHistorySituationVO);
        Page<DeviceHistorySituationVO> page = new Page<DeviceHistorySituationVO>(skip, limit);
        page.addAll(deviceHistorySituationVO);
        page.setPages((count + limit - 1) / limit);
        page.setTotal(count);
        PageInfo<DeviceHistorySituationVO> deviceHistorySituationVOPageInfo = new PageInfo<DeviceHistorySituationVO>(page);
        deviceHistorySituationVOPageInfo.getList().stream().forEach(item -> {
            Map<String, List<WorkStatusMonitorModel>> work = new HashMap<>();
            Map<String, List<WorkStatusMonitorModel>> noWork = new HashMap<>();
            Map<String, List<WorkStatusMonitorModel>> groupMap = item.getGroupMap();
            Map<String, List<WorkStatusMonitorModel>> stringListMap = getrealTimeWork(jeDeviceEntity);
            for (String key : groupMap.keySet()) {
                if (stringListMap.containsKey(key)) {
                    work.put(key, groupMap.get(key));
                } else {
                    noWork.put(key, groupMap.get(key));
                }
            }
            Map<String, Map<String, List<WorkStatusMonitorModel>>> groupByTypeMap = new HashMap<>();
            for (String key : work.keySet()) {
                //筛选实时工况中是否存在boolean类型数据
                List<WorkStatusMonitorModel> workStatusMonitorModels = work.get(key).stream().filter(e ->
                        Objects.equals("boolean", e.getPropertyDataType()) || Objects.equals("Boolean", e.getPropertyDataType()))
                        .collect(Collectors.toList());
                if (null != workStatusMonitorModels && workStatusMonitorModels.size() > 0) {
                    //存在boolean类型，将其放到设备检测中
                    noWork.put(key, workStatusMonitorModels);
                    //过滤掉布尔类型，重新赋值实时工况
                    List<WorkStatusMonitorModel> workStatusMonitorModels1 = work.get(key).stream().filter(e ->
                            !Objects.equals("boolean", e.getPropertyDataType()) && !Objects.equals("Boolean", e.getPropertyDataType()))
                            .collect(Collectors.toList());
                    if (null != workStatusMonitorModels1 && workStatusMonitorModels1.size() > 0) {
                        work.put(key, workStatusMonitorModels1);
                    }
                }
            }
         /*   for (String key:noWork.keySet()) {
            /*for (String key:noWork.keySet()) {
                //筛选设备检测中是否存在非boolean类型
                List<WorkStatusMonitorModel> workStatusMonitorModels = noWork.get(key).stream().filter(e->
                        !Objects.equals("boolean",e.getPropertyDataType()) && !Objects.equals("Boolean",e.getPropertyDataType()))
                        .collect(Collectors.toList());
                if(null != workStatusMonitorModels || workStatusMonitorModels.size() > 0){
                    //存在非boolean类型，将其放到实时工况中
                    work.put(key,workStatusMonitorModels);
                    //重新赋值设备检测(去掉非布尔类型数值)
                    List<WorkStatusMonitorModel> workStatusMonitorModels1 = noWork.get(key).stream().filter(e->
                            Objects.equals("boolean",e.getPropertyDataType()) || Objects.equals("Boolean",e.getPropertyDataType()))
                            .collect(Collectors.toList());
                    noWork.put(key,workStatusMonitorModels1);
                }
            }*/
            groupByTypeMap.put(ResourceUtil.getI18nMessage("60011"), work);
            groupByTypeMap.put(ResourceUtil.getI18nMessage("60012"), noWork);
            item.setGroupByTypeMap(groupByTypeMap);
        });
        Log.info("历史工况数据最终返回数据:"+deviceHistorySituationVOPageInfo.getList().size());
        return JsonResult.success(deviceHistorySituationVOPageInfo);
    }

    @Override
    public JsonResult exportQueryHistorySituation(ExportHistorySituationAO historySituationAO) {
        //根据设备id查询设备
        NxJeDeviceEntity jeDeviceEntity = this.getById(historySituationAO.getDeviceId());
        if (ObjectUtils.isEmpty(jeDeviceEntity)) {
            //设备不存在
            //throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13506"));
        }


        String stTime = DateUtil.handlerTime(historySituationAO.getStartTime());
        String enTime = DateUtil.handlerTime(historySituationAO.getEndTime());
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("startTime", stTime);
        paramMap.put("endTime", enTime);
        paramMap.put("limit",6000);
        paramMap.put("sort","desc");
        paramMap.put("timeZone",historySituationAO.getTimeZone());
        //调用第三方接口获取历史工况数据信息列表
        List<DeviceHistorySituationVO> deviceHistorySituationVO = callHistoryInterface1(historySituationAO.getDeviceId(), jeDeviceEntity.getModelId(), paramMap);
        Log.info("导出查询数据:"+deviceHistorySituationVO.size());

        return JsonResult.success(deviceHistorySituationVO);
    }


    public static <T> PageInfo<T> queryPageInfo(int currentPage, int pageSize, List<T> list) {
        int total = list.size();
        if (total > pageSize) {
            int toIndex = pageSize * currentPage;
            if (toIndex > total) {
                toIndex = total;
            }
            list = list.subList(pageSize * (currentPage - 1), toIndex);
        }
        Page<T> page = new Page<>(currentPage, pageSize);
        page.addAll(list);
        page.setPages((total + pageSize - 1) / pageSize);
        page.setTotal(total);

        PageInfo<T> pageInfo = new PageInfo<>(page);
        return pageInfo;
    }


    @Override
    public List<OtherDeviceResp> queryOtherDevices(String deviceId) {
        LambdaQueryWrapper<NxJeDeviceEntity> lambdaQueryWrapper = Wrappers.lambdaQuery(NxJeDeviceEntity.class);
        lambdaQueryWrapper.ne(NxJeDeviceEntity::getNxJeDeviceId, deviceId);
        List<NxJeDeviceEntity> list = this.list();
        List<OtherDeviceResp> resultList = new ArrayList<>(16);
        list.stream().forEach(temp -> {
            OtherDeviceResp otherDeviceResp = new OtherDeviceResp();
            if (!deviceId.equals(temp.getNxJeDeviceId())) {
                otherDeviceResp.setDeviceId(temp.getNxJeDeviceId());
                otherDeviceResp.setDeviceSbbm(temp.getDeviceSbbm());
                resultList.add(otherDeviceResp);
            }
        });
        return resultList;
    }

    @Override
    public JSONObject queryWorkTrendData(DeviceTrendParam deviceTrendParam) {
        JSONObject resultJson = new JSONObject();
        //stp1：先查询设备1的Y1和Y2数据
        List<Map<String, Object>> device1 = queryDevicesData(deviceTrendParam.getDeviceId1(), deviceTrendParam);
        resultJson.put("device1", device1);
        //stp2：如果设备2不为空 查数据库modelProperty判断设备2是否有入参中传的工况属性
        List<Map<String, Object>> device2 = new ArrayList<>();
        if (StringUtils.isNotBlank(deviceTrendParam.getDeviceId2())) {
            //判断设备2是否具有 Y1和Y2对应的工况属性
            boolean result = hasSameProperty(deviceTrendParam);
            if (result) {
                device2 = queryDevicesData(deviceTrendParam.getDeviceId2(), deviceTrendParam);
            }
        }
        //如果stp2有数据，做合并处理
        if (device2 != null && device2.size() > 0) {
            resultJson.put("device2", device2);
        }

        return resultJson;
    }

    @Override
    public List<NxJeDeviceEntity> queryAllJeDevice() {
        List<NxJeDeviceEntity> list = this.list(Wrappers.lambdaQuery(NxJeDeviceEntity.class).eq(NxJeDeviceEntity::getDeviceSource, 1));
        return list;
    }


    @Override
    public Map<String, Object> getWorkingMap(String deviceId, List<String> paramsList) {
        Map<String, Object> resultMap = new HashMap<>();
        NxJeDeviceEntity jeDeviceEntity = this.getById(deviceId);
        if (jeDeviceEntity == null) {
            //throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13506"));
        }
        Map<String, Object> map = new HashMap<>();
        JSONArray properties = JSONArray.parseArray(JSON.toJSONString(paramsList));
        map.put("properties", properties.toString());
        try {
            HttpRequest request = HttpRequest.get(reqUrl + "/realtime-manage/v1/realtime/models/" + jeDeviceEntity.getModelId() + "/things/" + deviceId);
            request.form(map);
            request.header("Authorization", "Bearer " + tokenUtil.getToken());
            request.header("accept", "application/json");
            request.header("Content-Type", "application/json");
            HttpResponse response = request.execute();
            String result = response.body();
//            Log.info("平台实时工况接口返回数据：{}", result);
            JSONObject jsonObject = JSONObject.parseObject(result);
            JSONArray array = jsonObject.getJSONArray("payload");

            if (array == null || array.size() == 0) {
                //throw new BusinessException(StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getMessage());
                throw new BusinessException(StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13508"));
            }
            JSONObject data = array.getJSONObject(0).getJSONObject("data");
            Map<String, Object> params = JSONObject.parseObject(data.toJSONString(), new TypeReference<Map<String, Object>>() {
            });
            for (Map.Entry<String, Object> entry : params.entrySet()) {
                Map<String, Object> value = (Map<String, Object>) entry.getValue();
                for (Map.Entry<String, Object> mapResult : value.entrySet()) {
                    if ("value".equals(mapResult.getKey())) {
                        resultMap.put(entry.getKey(), mapResult.getValue());
                    }
                }
            }
        } catch (Exception e) {
            log.error("调用平台查询实时工况信息error", e);
        }
        return resultMap;
    }

    @Override
    public Map<String, Object> getRootcloudWorkingMap(String deviceId, List<String> paramsList) {
        NxJeDeviceEntity jeDeviceEntity = this.getById(deviceId);
        if (jeDeviceEntity == null) {
            //throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13506"));
        }
        Map<String, Object> map = new HashMap<>();
        JSONArray properties = JSONArray.parseArray(JSON.toJSONString(paramsList));
        map.put("properties", properties.toString());
        try {
            HttpRequest request = HttpRequest.get(reqUrl + "/realtime-manage/v1/realtime/models/" + jeDeviceEntity.getModelId() + "/things/" + deviceId);
            request.form(map);
            request.header("Authorization", "Bearer " + tokenUtil.getToken());
            request.header("accept", "application/json");
            request.header("Content-Type", "application/json");
            HttpResponse response = request.execute();
            String result = response.body();
//            Log.info("平台实时工况接口返回数据：{}", result);
            JSONObject jsonObject = JSONObject.parseObject(result);
            JSONArray array = jsonObject.getJSONArray("payload");

            if (array == null || array.size() == 0) {
                return new HashMap<>();
            }
            JSONObject data = array.getJSONObject(0).getJSONObject("data");
            Map<String, Object> params = JSONObject.parseObject(data.toJSONString(), new TypeReference<Map<String, Object>>() {
            });
            return params;
        } catch (Exception e) {
            log.error("调用平台查询实时工况信息error", e);
        }
        return null;
    }

    @Override
    public Map<String, Map<String, Object>> getWorkingMapByModelGroup(String model, List<String> deviceIds, List<String> paramsList) {
        Map<String, Map<String, Object>> resultMap = new HashedMap();
        Map<String, Object> map = new HashMap<>();
        JSONArray properties = JSONArray.parseArray(JSON.toJSONString(paramsList));
        JSONArray deviceIdList = JSONArray.parseArray(JSON.toJSONString(deviceIds));
        map.put("modelId", model);
        map.put("properties", properties.toString());
        map.put("thingIds", deviceIdList.toString());
        try {
            HttpRequest request = HttpRequest.get(reqUrl + "/realtime-manage/v1/realtime/models/" + model + "/things");
            request.form(map);
            //Log.info("请求4.0接口AccessToken:"+tokenUtil.getToken());
            request.header("Authorization", "Bearer " + tokenUtil.getToken());
            request.header("accept", "application/json");
            request.header("Content-Type", "application/json");
            HttpResponse response = request.execute();
            String result = response.body();
//            Log.info("平台实时工况接口返回数据：{}", result);
            JSONObject jsonObject = JSONObject.parseObject(result);
            JSONArray array = jsonObject.getJSONArray("payload");
            if (array == null || array.size() == 0) {
                //throw new BusinessException(StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getMessage());
                throw new BusinessException(StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13508"));
            }
            for (int i = 0; i < array.size(); i++) {
                JSONObject data = array.getJSONObject(i).getJSONObject("data");
                Object thingId = array.getJSONObject(i).get("thingId");
                Map<String, Object> params = JSONObject.parseObject(data.toJSONString(), new TypeReference<Map<String, Object>>() {
                });
                /*Map<String,Object> childerMap = new HashMap<>();
                for (Map.Entry<String, Object> entry : params.entrySet()) {
                    Map<String, Object> value = (Map<String, Object>) entry.getValue();
                    for (Map.Entry<String, Object> mapResult : value.entrySet()) {
                        if ("value".equals(mapResult.getKey())) {
                            childerMap.put(entry.getKey(), mapResult.getValue());
                        }
                    }
                }*/
                resultMap.put(String.valueOf(thingId), params);
            }
        } catch (Exception e) {
            log.error("调用平台查询实时工况信息error", e);
        }
        return resultMap;
    }


    @Override
    public DeviceTypeVO getByDeviceId(String deviceId) {
        DeviceTypeVO deviceTypeVO = nxJeDeviceDao.getByDeviceId(deviceId);
        if (ObjectUtils.isEmpty(deviceTypeVO)) {
            //throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13506"));
        }
        NxJeDeviceEntity jeDeviceEntity = this.getById(deviceId);
        if (StringUtils.isNotBlank(deviceTypeVO.getOnlineTime())) {
            String unit = "";
            if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                unit = modelPropertyMapper.selectUnitByModelId("HMI_TotalTimeCost", jeDeviceEntity.getModelId());
            }
            deviceTypeVO.setOnlineTime(deviceTypeVO.getOnlineTime() + " " + unit);
        }
        List<String> paramsList = new ArrayList<>();
        //打印编号(任务名称)
        paramsList.add("HMI_JobID");
        //开始打印时间
        paramsList.add("HMI_BeginTime");
        //预计打印结束时间
        paramsList.add("HMI_EndTime");
        //打印耗时
        paramsList.add("HMI_TotalTimeCost");
        //当前打印层数
        paramsList.add("HMI_CurPrintLayer");
        //打印体积
        paramsList.add("HMI_Volume");
        //打印层厚
        paramsList.add("HMI_Thickness");
        //打印层耗时
        paramsList.add("HMI_LayerCostTime");
        //打印总层数
        paramsList.add("HMI_TotalLayer");
        //打印中
        paramsList.add("SB_ZT_Running");
        //待机
        paramsList.add("SB_ZT_Ready");
        //报警
        paramsList.add("SB_ZT_alarm");
        Map<String, Object> workingMap = getWorkingMap(deviceId, paramsList);
        if (!workingMap.isEmpty()) {
            for (Map.Entry<String, Object> entry : workingMap.entrySet()) {
                if ("HMI_JobID".equals(entry.getKey())) {
                    deviceTypeVO.setTaskName(String.valueOf(entry.getValue()));
                }
                if ("HMI_BeginTime".equals(entry.getKey())) {
                    deviceTypeVO.setStartTime(String.valueOf(entry.getValue()));
                }
                if ("HMI_EndTime".equals(entry.getKey())) {
                    deviceTypeVO.setEndTime(String.valueOf(entry.getValue()));
                }
                if ("HMI_TotalTimeCost".equals(entry.getKey())) {
                    String value = String.valueOf(entry.getValue());
                    DecimalFormat decimalFormat = new DecimalFormat("0.00");
                    String strVal = decimalFormat.format(new BigDecimal(value));

                    String unit = "";
                    if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                        unit = modelPropertyMapper.selectUnitByModelId("HMI_TotalTimeCost", jeDeviceEntity.getModelId());
                    }
                    deviceTypeVO.setPrintTime(strVal + " " + unit);
                }
                if ("HMI_CurPrintLayer".equals(entry.getKey())) {
                    String value = String.valueOf(entry.getValue());
                    DecimalFormat decimalFormat = new DecimalFormat("0.00");
                    String strVal = decimalFormat.format(new BigDecimal(value));
                    deviceTypeVO.setNowPrintLayer(strVal);
                }
                if ("HMI_Volume".equals(entry.getKey())) {
                    String value = String.valueOf(entry.getValue());
                    DecimalFormat decimalFormat = new DecimalFormat("0.00");
                    String strVal = decimalFormat.format(new BigDecimal(value));

                    String unit = "";
                    if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                        unit = modelPropertyMapper.selectUnitByModelId("HMI_Volume", jeDeviceEntity.getModelId());
                    }
                    deviceTypeVO.setPrintVolume(strVal + " " + unit);
                }
                if ("HMI_Thickness".equals(entry.getKey())) {
                    String value = String.valueOf(entry.getValue());
                    DecimalFormat decimalFormat = new DecimalFormat("0.00");
                    String strVal = decimalFormat.format(new BigDecimal(value));

                    String unit = "";
                    if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                        unit = modelPropertyMapper.selectUnitByModelId("HMI_Thickness", jeDeviceEntity.getModelId());
                    }
                    deviceTypeVO.setPrintLayer(strVal + " " + unit);
                }
                if ("HMI_LayerCostTime".equals(entry.getKey())) {
                    deviceTypeVO.setPrintLayerTime(String.valueOf(entry.getValue()));
                }
                if ("HMI_TotalLayer".equals(entry.getKey())) {
                    String value = String.valueOf(entry.getValue());
                    DecimalFormat decimalFormat = new DecimalFormat("0.00");
                    String strVal = decimalFormat.format(new BigDecimal(value));
                    deviceTypeVO.setPrintAllLayer(strVal);
                }
            }
            deviceTypeVO.setDeviceStatus(ServicePlanEnums.PrintStatus.OFFLINE.getCode());
            if (StringUtils.isNotBlank(deviceTypeVO.getDeviceSource()) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(deviceTypeVO.getDeviceSource())) {
                if (workingMap.containsKey("SB_ZT_alarm")) {
                    deviceTypeVO.setAlarm((Boolean) workingMap.get("SB_ZT_alarm"));
                }
                if (workingMap.containsKey("SB_ZT_Ready")) {
                    if ((Boolean) workingMap.get("SB_ZT_Ready")) {
                        deviceTypeVO.setDeviceStatus(ServicePlanEnums.PrintStatus.STANDBY.getCode());
                    }
                }
                if (workingMap.containsKey("SB_ZT_Running")) {
                    if ((Boolean) workingMap.get("SB_ZT_Running")) {
                        deviceTypeVO.setDeviceStatus(ServicePlanEnums.PrintStatus.PRINTING.getCode());
                    }
                }
            }
        }
        return deviceTypeVO;
    }

    @Override
    public DeviceStatusVO getStatusByDeviceId(String deviceId) {
        DeviceStatusVO deviceStatusVO = new DeviceStatusVO();
        NxJeDeviceEntity jeDeviceEntity = this.getById(deviceId);
        if (jeDeviceEntity == null) {
            //throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13506"));
        }
        deviceStatusVO.setDeviceStatus(ServicePlanEnums.PrintStatus.OFFLINE.getCode());
        if (StringUtils.isNotBlank(jeDeviceEntity.getDeviceSource()) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(jeDeviceEntity.getDeviceSource())) {
            //在线
            List<String> paramsList = new ArrayList<>();
            //打印中
            paramsList.add("SB_ZT_Running");
            //待机
            paramsList.add("SB_ZT_Ready");
            //报警
            paramsList.add("SB_ZT_alarm");
            Map<String, Object> workingMap = getWorkingMap(deviceId, paramsList);
            if (!workingMap.isEmpty()) {
                if (workingMap.containsKey("SB_ZT_alarm")) {
                    deviceStatusVO.setAlarm((Boolean) workingMap.get("SB_ZT_alarm"));
                }
                if (workingMap.containsKey("SB_ZT_Ready")) {
                    if ((Boolean) workingMap.get("SB_ZT_Ready")) {
                        deviceStatusVO.setDeviceStatus(ServicePlanEnums.PrintStatus.STANDBY.getCode());
                    }
                }
                if (workingMap.containsKey("SB_ZT_Running")) {
                    if ((Boolean) workingMap.get("SB_ZT_Running")) {
                        deviceStatusVO.setDeviceStatus(ServicePlanEnums.PrintStatus.PRINTING.getCode());
                    }
                }
            }
        }
        return deviceStatusVO;
    }

    @Override
    public IPage<DeviceVO> devicePage(String queryData, String isCare, int page, int size, String userId,String syZhid) {

        //查询设备位置
        List<NxDeviceLocationEntity> allDeviceLocation = deviceLocationMapper.findAllDeviceLocation();
        Map<String, NxDeviceLocationEntity> deviceLocationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(allDeviceLocation)) {
            deviceLocationMap = allDeviceLocation.stream().collect(Collectors.toMap(NxDeviceLocationEntity::getDeviceId, Function.identity()));
        }
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<DeviceVO> pageParam =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(page, size);
        IPage<DeviceVO> result = pageParam;
        // 在人员表中查询该员工的角色信息
        JeCoreEnduserEntity user = nxOrderMapper.getCoreEnduserByUsrId(userId);
        user.getRoleids();
        if (user.getRolenames().contains("系统管理员") || user.getRolenames().contains("管理员") || user.getRolenames().contains("售后领导")) {
            // 所有的设备列表信息
            result = nxJeDeviceDao.devicePage(pageParam, queryData, isCare, null, null, UserContext.getTenantId());
        } else if (user.getRolenames().contains("服务经理") || user.getRolenames().contains("工程师")) {
            // 一个人只能绑定一个服务网点，所以list里只有一个或者没有
            List<FwryDTO> fwryList = nxOrderMapper.getNxgxJcdaFwryByUserId(userId);
            if (CollectionUtils.isEmpty(fwryList)) {
                throw new BusinessException("500", "未在网点下查到该员工信息!");
            }
            FwryDTO fwryDTO = fwryList.get(0);
            // 通过网点id获取服务网点信息
            NetworkEntity networkEntity = nxOrderMapper.getfwwd(fwryDTO);
            // 获取网点下的服务区域
            String[] split = networkEntity.getFwwdSsqy().split(",");
            // 通过网点的服务区域，去查询客户
            List<KhglDTO> khglDTOList = nxOrderMapper.getKhBykhdzs(split);
            // 通过客户名称查询设备列表信息
            // 所有的设备列表信息
            if (CollectionUtils.isEmpty(khglDTOList)) {
                return pageParam;
            }
            StringBuilder khmc = new StringBuilder();
            for (KhglDTO dto : khglDTOList) {
                khmc.append("'").append(dto.getKhglKhmc()).append("',");
            }
            khmc = new StringBuilder(khmc.substring(0, khmc.length() - 1));
            result = nxJeDeviceDao.devicePage(pageParam, queryData, isCare, khmc.toString(), null,UserContext.getTenantId());
        } else if (user.getRolenames().contains("客户") || user.getRolenames().contains("客户领导")) {
            HashMap<String, String> req = new HashMap<>();
            req.put("userId",user.getUserid());
            String join = "";
            List<String> kmcList = nxOrderMapper.getKmcList(req);
            if(kmcList.size()>0){
                for (String khmc:kmcList) {
                    join+="'"+khmc+"'"+",";
                }
                join =  join.substring(0,join.length()-1);
            }
            result = nxJeDeviceDao.devicePage(pageParam, queryData, isCare, join, null,UserContext.getTenantId());
        }
        if (CollectionUtils.isNotEmpty(result.getRecords())) {
            Map<String, NxDeviceLocationEntity> finalDeviceLocationMap = deviceLocationMap;
            result.getRecords().forEach(v -> {
                DeviceOnlineStatus status = DeviceOnlineStatus.getByCode(v.getStatusCode());
                if (status != null) {
                    v.setStatusDesc(status.getDesc());
                }
                NxDeviceLocationEntity nxDeviceLocationEntity = finalDeviceLocationMap.get(v.getDeviceId());
                if (nxDeviceLocationEntity != null) {
                    String gps = nxDeviceLocationEntity.getLongitude() + "," + nxDeviceLocationEntity.getLatitude();
                    //获取经纬度返回地址
                    if(org.apache.commons.lang3.StringUtils.isNotBlank(gps)){
                        v.setDeviceAddress(gaoDeAPIUtils.getAddressByJwd(gps));
                    }
                }

            });
        }
        return result;
    }


    @Override
    public Boolean queryOnlineStatus(String deviceNo) {
        NxJeDeviceEntity entity = getOne(Wrappers.lambdaQuery(NxJeDeviceEntity.class).eq(NxJeDeviceEntity::getDeviceSbbm, deviceNo));
        if (entity != null) {
            List<String> list = new ArrayList<>();
            list.add(DeviceOnlineStatus.ONLINE.getPropertyId());
            Map<String, Object> workingMap = getWorkingMap(entity.getNxJeDeviceId(), list);
            if (workingMap.containsKey(DeviceOnlineStatus.ONLINE.getPropertyId())) {
                return (Boolean) workingMap.get(DeviceOnlineStatus.ONLINE.getPropertyId());
            }
        }
        return null;
    }

    @Override
    public NxJeDeviceEntity selectBySbbm(String sbbm) {
        QueryWrapper<NxJeDeviceEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("DEVICE_SBBM", sbbm);
        queryWrapper.last("limit 1");
        NxJeDeviceEntity nxJeDeviceEntity = nxJeDeviceDao.selectOne(queryWrapper);
        return nxJeDeviceEntity;
    }

    @Override
    public Map<String, List<WorkStatusMonitorModel>> getrealTimeWork(NxJeDeviceEntity jeDeviceEntity) {
        LambdaQueryWrapper<ModelPropertyEntity> mpeWrapper = new LambdaQueryWrapper<>();
        mpeWrapper.eq(ModelPropertyEntity::getDeviceSblx, jeDeviceEntity.getDeviceSblx());
        mpeWrapper.ne(ModelPropertyEntity::getPropertyId, "");
        mpeWrapper.isNotNull(ModelPropertyEntity::getPropertyId);
        mpeWrapper.and(wq -> {
            wq.eq(ModelPropertyEntity::getDeviceSbxh, jeDeviceEntity.getDeviceSbxh())
                    .or().eq(ModelPropertyEntity::getDeviceSbxh, "-1");
        });
        mpeWrapper.ne(ModelPropertyEntity::getPropertyDataType, "boolean").ne(ModelPropertyEntity::getPropertyDataType, "Boolean");
        List<ModelPropertyEntity> entities = modelPropertyMapper.selectList(mpeWrapper);
        List<WorkStatusMonitorModel> models = entities.stream().map(v -> {
            WorkStatusMonitorModel model = new WorkStatusMonitorModel();
            BeanUtils.copyProperties(v, model);
            return model;
        }).collect(Collectors.toList());
        return models.stream().collect(Collectors.groupingBy(WorkStatusMonitorModel::getPropertyTag));
    }


    @Override
    public void updateOnlineTime(String deviceIds) {
        //查询没有onlineTime的数据
        QueryWrapper<NxJeDeviceEntity> queryWrapper = new QueryWrapper();
        queryWrapper.eq("DEVICE_SOURCE", "1");
        if (StringUtils.isNotBlank(deviceIds)) {
            List<String> stList = Arrays.asList(deviceIds.split(","));
            queryWrapper.in("NX_JE_DEVICE_ID", stList);
        }
        List<NxJeDeviceEntity> nxJeDeviceEntities = nxJeDeviceDao.selectList(queryWrapper);
        nxJeDeviceEntities.stream().forEach(item -> {
            String thingUrl = domain + thingUri;
            HttpRequest thingRequest = HttpRequest.get(thingUrl);
            //处理入参
            Map<String, Object> map = new HashMap<>();
            map.put("thingId", item.getNxJeDeviceId());
            thingRequest.form(map);
            //Log.info("请求4.0接口AccessToken:"+tokenUtil.getToken());
            thingRequest.header("Authorization", "Bearer " + tokenUtil.getToken());
//            thingRequest.header("AuthHeaderAlias", tokenUtil.superToken());
//            thingRequest.header("X-RCD-Tenant-Id",UserContext.getTenantId());
            HttpResponse thingResponse = thingRequest.execute();
            if (thingResponse.getStatus() != HttpStatus.HTTP_OK) {
                log.error("调用平台接口" + thingUrl + "返回" + thingResponse);
            }
            cn.hutool.json.JSONObject thingJsonObj = new cn.hutool.json.JSONObject(thingResponse.body());
            cn.hutool.json.JSONArray thingArray = thingJsonObj.getJSONArray("payload");
            if (!thingArray.isEmpty()) {
                Map<String, Object> obj = (Map<String, Object>) thingArray.get(0);
                if (!ObjectUtils.isEmpty(obj)) {
                    if (obj.get("created") != null) {
                        String created = DateUtil.specialDate(String.valueOf(obj.get("created")));
                        if (StringUtils.isNotBlank(created)) {
                            if (StringUtils.isBlank(item.getCreateTime()) || !created.equals(item.getCreateTime())) {
                                item.setCreateTime(created);
                                NxJeDeviceEntity deviceEntity = new NxJeDeviceEntity();
                                deviceEntity.setNxJeDeviceId(item.getNxJeDeviceId());
                                deviceEntity.setCreateTime(created);
                                nxJeDeviceDao.updateById(deviceEntity);
                            }
                        }
                    }
                }
            }
        });
    }


    /**
     * 根据前端指定的设备和工况查询对应数据
     *
     * @return
     */
    @Override
    public Map<String, Object> queryPropertyValueByParam(DevicePropertyValueParam devicePropertyValueParam) {
        String deviceId = devicePropertyValueParam.getDeviceId();
        List<String> propertyIds = devicePropertyValueParam.getPropertyIds();
        NxJeDeviceEntity jeDeviceEntity = this.getById(deviceId);
        if (jeDeviceEntity == null) {
            //throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13506"));
        }
        Map<String, Object> result = new HashMap<>();
        result = getWorkingMap(deviceId, propertyIds);
        //计算进度
        if (result.containsKey("HMI_CurPrintLayer") && result.containsKey("HMI_TotalLayer")) {
            BigDecimal curPrintLayer = new BigDecimal(result.get("HMI_CurPrintLayer").toString());
            BigDecimal totalLayer = new BigDecimal(result.get("HMI_TotalLayer").toString());
            if (totalLayer.compareTo(BigDecimal.ZERO) == 0) {
                result.put("HMI_progress", "0%");
            } else {
                BigDecimal divide = curPrintLayer.divide(totalLayer, 4, RoundingMode.HALF_UP);
                result.put("HMI_progress", divide.multiply(new BigDecimal(100)).setScale(2, BigDecimal.ROUND_HALF_UP) + "%");
            }

        }
        //用砂量 A砂累计用量+B砂累计用量
        if (result.containsKey("SB_MX_SandTotal_A") && result.containsKey("SB_MX_SandTotal_B")) {
            Float sandTotalA = Float.valueOf(result.get("SB_MX_SandTotal_A").toString());
            Float sandTotalB = Float.valueOf(result.get("SB_MX_SandTotal_B").toString());
            result.put("SB_MX_SandTotal", sandTotalA + sandTotalB);
        }


        //增加单位
        if (result.containsKey("HMI_TotalTimeCost")) {
            String value = String.valueOf(result.get("HMI_TotalTimeCost"));
            DecimalFormat decimalFormat = new DecimalFormat("0.00");
            String strVal = decimalFormat.format(new BigDecimal(value));

            String unit = "";
            if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                unit = modelPropertyMapper.selectUnitByModelId("HMI_TotalTimeCost", jeDeviceEntity.getModelId());
            }
            String hmi_totalTimeCost = strVal + " " + (StringUtils.isBlank(unit) ? "" : unit);
            result.put("HMI_TotalTimeCost", hmi_totalTimeCost);
        }
        if (result.containsKey("HMI_Thickness")) {
            String value = String.valueOf(result.get("HMI_Thickness"));
            DecimalFormat decimalFormat = new DecimalFormat("0.00");
            String strVal = decimalFormat.format(new BigDecimal(value));

            String unit = "";
            if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                unit = modelPropertyMapper.selectUnitByModelId("HMI_Thickness", jeDeviceEntity.getModelId());
            }
            String HMI_Thickness = strVal + " " + (StringUtils.isBlank(unit) ? "" : unit);
            result.put("HMI_Thickness", HMI_Thickness);
        }
        if (result.containsKey("HMI_Volume")) {
            String value = String.valueOf(result.get("HMI_Volume"));
            DecimalFormat decimalFormat = new DecimalFormat("0.00");
            String strVal = decimalFormat.format(new BigDecimal(value));

            String unit = "";
            if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                unit = modelPropertyMapper.selectUnitByModelId("HMI_Volume", jeDeviceEntity.getModelId());
            }
            String HMI_Volume = strVal + " " + (StringUtils.isBlank(unit) ? "" : unit);
            result.put("HMI_Volume", HMI_Volume);
        }
        if (result.containsKey("SB_MX_SandTotal")) {
            String value = String.valueOf(result.get("SB_MX_SandTotal"));
            DecimalFormat decimalFormat = new DecimalFormat("0.00");
            String strVal = decimalFormat.format(new BigDecimal(value));

            String unit = "";
            if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                unit = modelPropertyMapper.selectUnitByModelId("SB_MX_SandTotal_A", jeDeviceEntity.getModelId());
            }
            String SB_MX_SandTotal_A = strVal + " " + (StringUtils.isBlank(unit) ? "" : unit);
            result.put("SB_MX_SandTotal", SB_MX_SandTotal_A);
        }
        if (result.containsKey("SB_BinderCost")) {
            String value = String.valueOf(result.get("SB_BinderCost"));
            DecimalFormat decimalFormat = new DecimalFormat("0.00");
            String strVal = decimalFormat.format(new BigDecimal(value));

            String unit = "";
            if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                unit = modelPropertyMapper.selectUnitByModelId("SB_BinderCost", jeDeviceEntity.getModelId());
            }
            String SB_BinderCost = strVal + " " + (StringUtils.isBlank(unit) ? "" : unit);
            result.put("SB_BinderCost", SB_BinderCost);
        }
        if (result.containsKey("SB_ActivatorCost")) {
            String value = String.valueOf(result.get("SB_ActivatorCost"));
            DecimalFormat decimalFormat = new DecimalFormat("0.00");
            String strVal = decimalFormat.format(new BigDecimal(value));

            String unit = "";
            if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                unit = modelPropertyMapper.selectUnitByModelId("SB_ActivatorCost", jeDeviceEntity.getModelId());
            }
            String SB_ActivatorCost = strVal + " " + (StringUtils.isBlank(unit) ? "" : unit);
            result.put("SB_ActivatorCost", SB_ActivatorCost);
        }

        String startTime = null;
        String endTime = null;
        if (result.containsKey("HMI_BeginTime") && result.containsKey("HMI_EndTime")) {
            startTime = String.valueOf(result.get("HMI_BeginTime"));
            endTime = String.valueOf(result.get("HMI_EndTime"));
        }


        //封装报警条数
        Integer count = deviceAlarmService.getDeviceAlarmCount(deviceId, startTime, endTime);
        result.put("alarmCount", count);
        return result;
    }

    /**
     * 查询工况属性
     *
     * @param deviceId 设备id
     * @param monitor  是否设备监测数据，设备监测数据只查询数据类型为boolean类型的数据
     * @return 分组好的工况数据
     */
    Map<String, List<WorkStatusMonitorModel>> queryModelProperty(String deviceId, boolean monitor, String dataSjy) {
        //根据设备id查询设备
        NxJeDeviceEntity jeDeviceEntity = this.getById(deviceId);
        if (jeDeviceEntity == null) {
            //throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.DEVICE_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13506"));
        }
        LambdaQueryWrapper<ModelPropertyEntity> mpeWrapper = new LambdaQueryWrapper<ModelPropertyEntity>();
        if (StringUtils.isNotBlank(dataSjy) && "0".equals(dataSjy)) {
            mpeWrapper.eq(ModelPropertyEntity::getDataSjy, dataSjy);
        }
        mpeWrapper.eq(ModelPropertyEntity::getDeviceSblx, jeDeviceEntity.getDeviceSblx());
        mpeWrapper.ne(ModelPropertyEntity::getPropertyId, "");
        mpeWrapper.isNotNull(ModelPropertyEntity::getPropertyId);
        mpeWrapper.and(wq -> {
            wq.eq(ModelPropertyEntity::getDeviceSbxh, jeDeviceEntity.getDeviceSbxh())
                    .or().eq(ModelPropertyEntity::getDeviceSbxh, "-1");
        });
        //构建查询排序title条件
        HashMap<String, String> req = new HashMap<>();
        req.put("deviceSblx",jeDeviceEntity.getDeviceSblx());
        req.put("deviceSbxh",jeDeviceEntity.getDeviceSbxh());
        if (monitor) {
            mpeWrapper.and(wq -> {
                wq.eq(ModelPropertyEntity::getPropertyDataType, "boolean")
                        .or().eq(ModelPropertyEntity::getPropertyDataType, "Boolean");
            });
            req.put("workingConditionType","2");
        } else {
            mpeWrapper.ne(ModelPropertyEntity::getPropertyDataType, "boolean").ne(ModelPropertyEntity::getPropertyDataType, "Boolean");
            req.put("workingConditionType","1");
        }
        List<ModelPropertyEntity> entities = modelPropertyMapper.selectList(mpeWrapper);
        this.callInterface(jeDeviceEntity.getNxJeDeviceId(), jeDeviceEntity.getModelId(), entities, monitor);
        List<WorkStatusMonitorModel> models = entities.stream().map(v -> {
            WorkStatusMonitorModel model = new WorkStatusMonitorModel();
            BeanUtils.copyProperties(v, model);
            return model;
        }).collect(Collectors.toList());
        Map<String, List<WorkStatusMonitorModel>> collect = models.stream().collect(Collectors.groupingBy(WorkStatusMonitorModel::getPropertyTag));

        List<String> titles = modelPropertyMapper.getWorkingTitle(req);
        LinkedHashMap<String, List<WorkStatusMonitorModel>> linkedHashMap = new LinkedHashMap<>();
        titles.forEach(item->{
            List<WorkStatusMonitorModel> list = collect.get(item);
            if(list!=null){
                linkedHashMap.put(item,collect.get(item));
            }
        });
        return linkedHashMap;
    }

    /**
     * 调用平台接口赋值实时工况数据
     *
     * @param deviceId 设备id
     * @param modelId  模型id
     * @param entities jp内部工况表数据
     */
    public void callInterface(String deviceId, String modelId, List<ModelPropertyEntity> entities, boolean monitor) {
        Map<String, String> headers = new HashMap<>();
        //Log.info("请求4.0接口AccessToken:"+tokenUtil.getToken());
        headers.put("Authorization", "Bearer " + tokenUtil.getToken());
        try {
            String resp = HttpClientUtil.sendHttp(HttpRequestMethedEnum.HttpGet, reqUrl + "/realtime-manage/v1/realtime/models/" + modelId + "/things/" + deviceId, null, headers);
            //Log.info("平台实时工况接口返回数据：{}", resp);
            JSONObject jsonObject = JSONObject.parseObject(resp);
            JSONArray array = jsonObject.getJSONArray("payload");

            if (array == null || array.size() == 0) {
                //throw new BusinessException(StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getRet(), StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getMessage());
                throw new BusinessException(StatusCodeEnum.DEVICE_WORK_STATUS_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("13508"));
            }
            JSONObject data = array.getJSONObject(0).getJSONObject("data");

            //接口数据赋值
            entities.stream().peek(o -> {
                if (!data.containsKey(o.getPropertyId())) {
                    return;
                }
                if (("boolean".equals(o.getPropertyDataType()) || "Boolean".equals(o.getPropertyDataType())) && monitor) {
                    if (data.getJSONObject(o.getPropertyId()).getBoolean("value")) {
                        o.setPropertyData(o.getPropertyColorTrue());
                    } else {
                        o.setPropertyData(o.getPropertyColorFalse());
                    }
                } else {
                    o.setPropertyData(data.getJSONObject(o.getPropertyId()).getString("value"));
                }
            }).collect(Collectors.toList());
        } catch (Exception e) {
            log.error("调用平台查询实时工况信息error", e);
        }
    }


    /**
     * 调用平台接口赋值历史工况数据
     *
     * @param deviceId 设备id
     * @param modelId  模型id
     * @param paramMap
     */
    public List<Map<String, Object>> callHistoryInterface(String deviceId, String modelId, Map<String, Object> paramMap) {

        List<Map<String, Object>> resultList = new ArrayList<>();
        try {
            HttpRequest request = HttpRequest.get(reqUrl + "historian-manage/v1/historian/models/" + modelId + "/things/" + deviceId);
            request.form(paramMap);
            //Log.info("请求4.0接口AccessToken:"+tokenUtil.getToken());
            request.header("Authorization", "Bearer " + tokenUtil.getToken());
//            request.header("AuthHeaderAlias", tokenUtil.superToken());
//            request.header("X-RCD-Tenant-Id",UserContext.getTenantId());

            request.header("accept", "application/json");
            request.header("Content-Type", "application/json");
            HttpResponse response = request.execute();
            if (response.getStatus() != HttpStatus.HTTP_OK) {
                log.error("调用平台接口出错" + response);
                //throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
                throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(), ResourceUtil.getI18nMessage("40000"));
            }
            String result = response.body();
            JSONObject jsonObject = JSONObject.parseObject(result);
            JSONArray array = jsonObject.getJSONArray("payload");
            array.stream().forEach(temp -> {
                JSONObject obj = (JSONObject) temp;
                JSONArray array2 = obj.getJSONArray("rows");
                array2.stream().forEach(temp1 -> {
                    JSONArray array1 = obj.getJSONArray("columns");
                    JSONArray array3 = (JSONArray) temp1;
                    Map<String, Object> map = new HashMap<>();
                    for (int i = 0; i < array1.size(); i++) {
                        if ("time".equals(array1.get(i).toString())) {
                            String date = array3.get(i).toString();
                            map.put(array1.get(i).toString(), DateUtil.getDateByUct(date));
                        } else {
                            map.put(array1.get(i).toString(), array3.get(i));
                        }
                    }
                    resultList.add(map);
                });
            });

        } catch (Exception e) {
            System.out.println("调用平台历史工况信息error");
            log.error("查询历史工况数据报错" + e);
        }
        return resultList;

    }

    public Integer queryHistoryCount(String deviceId, String modelId, Map<String, Object> paramMap){
        NxJeDeviceEntity jeDeviceEntity = this.getById(deviceId);
        if (ObjectUtils.isEmpty(jeDeviceEntity)) {
            return 0;
        }
        if (StringUtils.isBlank(modelId)) {
            modelId = jeDeviceEntity.getModelId();
        }
        HttpRequest request = HttpRequest.get(reqUrl + "historian-manage/v1/historian/models/" + modelId + "/things/" + deviceId+"/count");
        request.form(paramMap);
        //Log.info("请求4.0接口AccessToken:"+ tokenUtil.getToken());
        request.header("Authorization", "Bearer " + tokenUtil.getToken());
        request.header("accept", "application/json");
        request.header("Content-Type", "application/json");
        HttpResponse response = request.execute();
        String result = response.body();
        Log.info("查询条数",result);
        JSONObject jsonObject = JSONObject.parseObject(result);
        return jsonObject.getJSONObject("payload").getInteger("count");
    }


    public List<DeviceHistorySituationVO> callHistoryInterface1(String deviceId, String modelId, Map<String, Object> paramMap) {
        List<DeviceHistorySituationVO> situationVOS = new ArrayList<>();
        NxJeDeviceEntity jeDeviceEntity = this.getById(deviceId);
        if (ObjectUtils.isEmpty(jeDeviceEntity)) {
            return situationVOS;
        }
        if (StringUtils.isBlank(modelId)) {
            modelId = jeDeviceEntity.getModelId();
        }
        LambdaQueryWrapper<ModelPropertyEntity> mpeWrapper = new LambdaQueryWrapper<ModelPropertyEntity>();
        mpeWrapper.eq(ModelPropertyEntity::getDeviceSblx, jeDeviceEntity.getDeviceSblx());
        mpeWrapper.ne(ModelPropertyEntity::getPropertyId, "");
        mpeWrapper.isNotNull(ModelPropertyEntity::getPropertyId);
        mpeWrapper.and(wq -> {
            wq.eq(ModelPropertyEntity::getDeviceSbxh, jeDeviceEntity.getDeviceSbxh())
                    .or().eq(ModelPropertyEntity::getDeviceSbxh, "-1");
        });
        List<ModelPropertyEntity> entities = modelPropertyMapper.selectList(mpeWrapper);

        try {
            HttpRequest request = HttpRequest.get(reqUrl + "historian-manage/v1/historian/models/" + modelId + "/things/" + deviceId);
            request.form(paramMap);
            //Log.info("请求4.0接口AccessToken:" + tokenUtil.getToken());
            request.header("Authorization", "Bearer " + tokenUtil.getToken());
            request.header("accept", "application/json");
            request.header("Content-Type", "application/json");
            HttpResponse response = request.execute();
            String result = response.body();
            JSONObject jsonObject = JSONObject.parseObject(result);
            JSONArray array = jsonObject.getJSONArray("payload");
            array.stream().forEach(temp -> {
                JSONObject obj = (JSONObject) temp;
                List<Map<String, Object>> list = new ArrayList<>();
//                List<BaseDeviceVO> baseDeviceVOS = new ArrayList<>();
                JSONArray array2 = obj.getJSONArray("rows");
                array2.stream().forEach(temp1 -> {
                    JSONArray array1 = obj.getJSONArray("columns");
                    JSONArray array3 = (JSONArray) temp1;
                    Map<String, Object> map = new HashMap<>();
                    for (int i = 0; i < array1.size(); i++) {
                        map.put(array1.get(i).toString(), array3.get(i));
                    }
                    list.add(map);
                });
                for (int i = 0; i < list.size(); i++) {
                    DeviceHistorySituationVO situationVO = new DeviceHistorySituationVO();
                    situationVO.setDeviceId(obj.get("thingId").toString());
                    situationVO.setModelId(obj.get("modelId").toString());
                    if (!ObjectUtils.isEmpty(jeDeviceEntity)) {
                        situationVO.setDeviceSbbm(jeDeviceEntity.getDeviceSbbm());
                        situationVO.setDeviceSblxName(jeDeviceEntity.getDeviceSblxName() == null ? "" : jeDeviceEntity.getDeviceSblxName());
                        situationVO.setDeviceSbmc(jeDeviceEntity.getDeviceSbmc());
                        situationVO.setDeviceSbxhName(jeDeviceEntity.getDeviceSbxhName() == null ? "" : jeDeviceEntity.getDeviceSbxhName());
                    }
                    //时间格式化处理
                   // situationVO.setUpdateTime(DateUtil.reverseHandlerTime(list.get(i).get("time").toString()));
                    int timeZone = 0;
                    if( paramMap.get("timeZone")!=null){
                        timeZone = Integer.valueOf( paramMap.get("timeZone").toString());
                    }
                    situationVO.setUpdateTime(DateUtil.handlerTimeNoUTC(list.get(i).get("time").toString(),timeZone));
                    situationVO.setMap(list.get(i));
                    Map<String, Object> stringObjectMap = list.get(i);

                    entities.stream().peek(o -> {
                        for (int a = 0; a < stringObjectMap.size(); a++) {
                            if (!(stringObjectMap).containsKey(o.getPropertyId())) {
                                return;
                            }

                            if (!stringObjectMap.isEmpty() && stringObjectMap.get(o.getPropertyId()) != null) {
                                o.setPropertyData((stringObjectMap).get(o.getPropertyId()).toString());
                            }
                        }
                    }).collect(Collectors.toList());

                    List<WorkStatusMonitorModel> models = entities.stream().map(v -> {
                        WorkStatusMonitorModel model = new WorkStatusMonitorModel();
                        BeanUtils.copyProperties(v, model);
                        return model;
                    }).collect(Collectors.toList());
                    Map<String, List<WorkStatusMonitorModel>> collect = models.stream().collect(Collectors.groupingBy(WorkStatusMonitorModel::getPropertyTag));
                    situationVO.setGroupMap(collect);
                    situationVOS.add(situationVO);
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("调用平台历史工况信息error");
        }
        return situationVOS;

    }

    /**
     * 查询设备1的工况数据
     */
    public List<Map<String, Object>> queryDevicesData(String deviceId, DeviceTrendParam deviceTrendParam) {
        SimpleDateFormat stf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //拿到设备1的物模型
        String modelId = queryModelIdByDeviceId(deviceId);
        // String modelId = "197ZUeGcXkY";
        //处理时间
        String time = deviceTrendParam.getTime();
        Map<String, Object> paramMap = new HashMap<>(16);
        String[] times = time.split("~");
        Long longTime = 0L;
        //计算开始时间和结束时间之间
        float v = 0;
        try {
            String date1 = times[0].replace("T"," ").replace("Z","");
            String date2 = times[1].replace("T"," ").replace("Z","");
            v = DateUtil.dayDiff(date1, date2);
        } catch (ParseException e) {
            e.printStackTrace();
        }
//        String startTime = DateUtil.handlerTime(times[0]);
//        String endTime = DateUtil.handlerTime(times[1]);
        paramMap.put("startTime", times[0]);
        paramMap.put("endTime", times[1]);

        JSONArray properties = new JSONArray();
        if (StringUtils.isNotBlank(deviceTrendParam.getProperty1())) {
            properties.add(deviceTrendParam.getProperty1());
        }
        if (StringUtils.isNotBlank(deviceTrendParam.getProperty2())) {
            properties.add(deviceTrendParam.getProperty2());
        }
        if (StringUtils.isNotBlank(deviceTrendParam.getProperty3())) {
            properties.add(deviceTrendParam.getProperty3());
        }
        if (StringUtils.isNotBlank(deviceTrendParam.getProperty4())) {
            properties.add(deviceTrendParam.getProperty4());
        }
        paramMap.put("properties", properties.toString());
        //limit默认一千，数据量太大时查不全，现按最大值处理
        paramMap.put("limit", 100000);
        //调接口查询工况数据
        List<Map<String, Object>> listMap = callHistoryInterface(deviceId, modelId, paramMap);
        List<Map<String, Object>> resultListMap = new ArrayList<>();
        if (v > 1) {
            if (!listMap.isEmpty() && listMap.size() > 0) {
                Log.info("需要处理的times[0]:{}", times[0]);
                SimpleDateFormat localDateTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
                SimpleDateFormat dateTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                try {
                    Date date = localDateTime.parse(times[0]);
                    String strDate = dateTime.format(date);
                    longTime = DateUtil.getTimesByString(strDate);
                } catch (ParseException e) {
                    Log.info("时间转换异常,times[0]:{},longTime:{}", times[0], longTime);
                    e.printStackTrace();
                }
                Long finalLongTime = longTime;
                float finalV = v;
                listMap.stream().forEach(item -> {
                    String resultTime = String.valueOf(item.get("time"));
                    String flagTimeString = "0";
                    if (resultListMap.size() > 1) {
                        flagTimeString = String.valueOf(resultListMap.get(resultListMap.size() - 1).get("time"));
                    } else {
                        SimpleDateFormat originalFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
                        SimpleDateFormat targetFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        try {
                            Date date = originalFormat.parse(times[0]);
                            flagTimeString = targetFormat.format(date);
                        } catch (ParseException e) {
                            Log.info("时间转换异常,resultTime:{},flagTimeString:{}", resultTime, flagTimeString);
                            e.printStackTrace();
                        }
                    }
                    Long itemTime = 0L;
                    Long flagTime = 0L;
                    try {
                        Log.info("queryDevicesData resultTime:{}, flagTimeString:{}", resultTime, flagTimeString);
                        itemTime = stf.parse(resultTime).getTime();
                        flagTime = DateUtil.getTimesByString(flagTimeString);
                    } catch (ParseException e) {
                        Log.info("时间转换异常,resultTime:{},flagTimeString:{},itemTime:{},flagTime:{}", resultTime, flagTimeString, itemTime, flagTime);
                        //e.printStackTrace();
                    }
                    //当前元素大于已存最大时间差值
                    Long chaTime = itemTime - flagTime;
                    long l = itemTime - finalLongTime;
                    if (finalV > 1 && finalV <= 7) {
                        if (l % (2 * 60 * 1000) == 0 || l == 0 || chaTime > (2 * 60 * 1000)) {
                            if (item.get("time") != null) {
                                resultListMap.add(item);
                            }
                        }
                    } else if (finalV > 7 && finalV <= 14) {
                        if (l % (5 * 60 * 1000) == 0 || l == 0 || chaTime > (5 * 60 * 1000)) {
                            if (item.get("time") != null) {
                                resultListMap.add(item);
                            }
                        }
                    } else if (finalV > 14 && finalV <= 21) {
                        if (l % (15 * 60 * 1000) == 0 || l == 0 || chaTime > (15 * 60 * 1000)) {
                            if (item.get("time") != null) {
                                resultListMap.add(item);
                            }
                        }
                    } else if (finalV > 21) {
                        if (l % (20 * 60 * 1000) == 0 || l == 0 || chaTime > (20 * 60 * 1000)) {
                            if (item.get("time")!=null) {
                                resultListMap.add(item);
                            }
                        }
                    }
                });
                resultListMap.remove(0);
            }
        } else {
            return listMap;
        }
        return resultListMap;
    }

    /**
     * 根据设备ID查询物模型ID
     */
    public String queryModelIdByDeviceId(String deviceId) {
        String modelId = null;
        NxJeDeviceEntity deviceEntity = this.getById(deviceId);

        if (deviceEntity != null && StringUtils.isNotBlank(deviceEntity.getModelId())) {
            modelId = deviceEntity.getModelId();
        }
        return modelId;
    }

    /**
     * 判断设备2是否具有相同的工况数据
     *
     * @param deviceTrendParam
     * @return
     */
    public boolean hasSameProperty(DeviceTrendParam deviceTrendParam) {
        boolean result = false;
        //根据设备查到物模型
        NxJeDeviceEntity deviceEntity = this.getById(deviceTrendParam.getDeviceId2());
        List<String> propertys = new ArrayList<>();
        propertys.add(deviceTrendParam.getProperty1());
        propertys.add(deviceTrendParam.getProperty2());
        List<String> list = new ArrayList<>();
        if (StringUtils.isNotBlank(deviceEntity.getModelId())) {
            list = modelPropertySyncService.queryPropertysByModelId(deviceEntity.getModelId());
        }
        if (list.containsAll(propertys)) {
            result = true;
        }
        return result;
    }
}