package com.rootcloud.domain.service.network;

import cn.hutool.core.lang.hash.Hash;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.PageRequest;
import com.rootcloud.common.constant.BusinessConstant;
import com.rootcloud.common.constant.RoleIdConstant;
import com.rootcloud.common.util.GaoDeAPIUtils;
import com.rootcloud.config.auth.LoginUserInfo;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.auth.UserInfo;
import com.rootcloud.domain.dao.JeCoreEnduserDao;
import com.rootcloud.domain.dao.JeCoreRoleUserDao;
import com.rootcloud.domain.dto.PendingOrderModel;
import com.rootcloud.domain.dto.ServiceViewResultModel;
import com.rootcloud.domain.entity.*;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.dto.EngineerModel;
import com.rootcloud.domain.entity.JeCoreEnduserEntity;
import com.rootcloud.domain.entity.JeCoreRoleUserEntity;
import com.rootcloud.domain.entity.NetworkUserEntity;
import com.rootcloud.domain.entity.NxOrderEntity;
import com.rootcloud.domain.entity.device.NxDeviceLocationEntity;
import com.rootcloud.domain.entity.ds.NxDsLogEntity;
import com.rootcloud.domain.mapper.NetworkUserMapper;
import com.rootcloud.domain.mapper.NxDeviceLocationMapper;
import com.rootcloud.domain.mapper.NxOrderMapper;
import com.rootcloud.domain.mapper.NxgxJcdaFwryMapper;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import com.rootcloud.vo.network.NetWorkUserWeekVO;
import jline.internal.Log;
import org.apache.commons.collections.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * TODO
 *
 * @author wei.liao
 * @version 1.0
 * @date 2022/4/27 10:30
 */
@Service
@Slf4j
public class NetworkUserService {

    @Resource
    private NxgxJcdaFwryMapper nxgxJcdaFwryMapper;

    @Autowired
    private NetworkUserMapper networkUserMapper;

    @Autowired
    private JeCoreRoleUserDao jeCoreRoleUserDao;

    @Autowired
    private JeCoreEnduserDao jeCoreEnduserDao;

    @Autowired
    private NxOrderMapper nxOrderMapper;

    @Autowired
    JeCoreEnduserDao enduserDao;

    @Autowired
    NxDeviceLocationMapper nxDeviceLocationMapper;

    @Autowired
    private GaoDeAPIUtils gaoDeAPIUtils;

    //图片预览请求地址
    @Value("${image.preview_url:http://je-paas-frontend.uwb-dev.rootcloudapp.com/je/document/preview?fileKey=}")
    private String previewUrl;

    /**
     * @param roleId 网点服务人员角色id
     * @return
     */
    public List<JeCoreEnduserEntity> getList(String roleId) {
        //所有用户
        List<Object> userIds = jeCoreRoleUserDao.selectObjs(new QueryWrapper<JeCoreRoleUserEntity>().select("USERID").lambda().eq(JeCoreRoleUserEntity::getRoleid, roleId));
        //已绑定的用户
        List<NetworkUserEntity> networkUserList = networkUserMapper.selectList(new QueryWrapper<NetworkUserEntity>().lambda().in(NetworkUserEntity::getUserid, userIds));
        //排除已绑定的用户
        Set<String> exUsers = new HashSet<>();
        if (CollectionUtils.isNotEmpty(networkUserList)) {
            exUsers = networkUserList.stream().collect(Collectors.groupingBy(NetworkUserEntity::getUserid)).keySet();
        }
        //剩余用户id
        List<Object> rsUserIds = new ArrayList<>();
        for (Object userId : userIds) {
            if (!exUsers.contains(userId)) {
                rsUserIds.add(userId);
            }
        }
        if (CollectionUtils.isNotEmpty(rsUserIds)) {
            return jeCoreEnduserDao.selectList(
                    new QueryWrapper<JeCoreEnduserEntity>().lambda()
                            .in(JeCoreEnduserEntity::getUserid, rsUserIds)
                            .eq(JeCoreEnduserEntity::getZhid, UserContext.getUserTenantId()));
        }
        return null;
    }


    /**
     * 查询所有工程师前7天的订单状态
     *
     * @param roleId 网点服务人员角色id
     * @return
     */
    public Page<NetWorkUserWeekVO> getWeekList(PageRequest pageRequest, String roleId) {
        List<Object> userIds = new ArrayList<>();
        LoginUserInfo loginUserInfo = UserContext.current();
        //判断是管理角色---现有逻辑查询所有无需改动  还是网点角色 ---根据登录人所属网点查看对应工程师角色的数据
        String checkUserIsRole = checkUserIsRole(loginUserInfo.getUserId());
        log.info("getWeekList loginUserInfo--->[{}]  checkUserIsRole---->[{}]", JSONObject.toJSONString(loginUserInfo), checkUserIsRole);
        if (checkUserIsRole.equals(BusinessConstant.CUSTOMER_ROLE_MANGE)) {
            userIds = jeCoreRoleUserDao.selectObjs(new QueryWrapper<JeCoreRoleUserEntity>().select("USERID").lambda().eq(JeCoreRoleUserEntity::getRoleid, roleId));
        }
        if (checkUserIsRole.equals(BusinessConstant.CUSTOMER_ROLE_OUTLETS)) {
            SiteDTO siteByUserId = jeCoreEnduserDao.getSiteByUserId(loginUserInfo.getUserId());
            if (siteByUserId != null) {
                userIds = nxgxJcdaFwryMapper.selectObjs(new QueryWrapper<NetworkUserEntity>().select("USERID").lambda().eq(NetworkUserEntity::getFwryWdid, siteByUserId.getSiteId()).like(NetworkUserEntity::getFwryJscode, roleId));
            }
        }
        Log.info("getWeekList userIds--->[{}]" + JSONObject.toJSONString(userIds));

        //所有用户
        if (CollectionUtils.isNotEmpty(userIds)) {
            List<NetWorkUserWeekVO> userList = new ArrayList<>();
            Page<NetWorkUserWeekVO> rsPage = new Page<>();
            Page<JeCoreEnduserEntity> page = new Page<>(pageRequest.getPage() == null ? 1 : pageRequest.getPage(), pageRequest.getSize() == null ? 5 : pageRequest.getSize());
            Page<JeCoreEnduserEntity> userListPage = jeCoreEnduserDao.selectPage(page, new QueryWrapper<JeCoreEnduserEntity>().lambda().in(JeCoreEnduserEntity::getUserid, userIds));
            List<JeCoreEnduserEntity> records = userListPage.getRecords();
            for (JeCoreEnduserEntity record : records) {
                NetWorkUserWeekVO netWorkUserWeekVO = new NetWorkUserWeekVO();
                netWorkUserWeekVO.setName(record.getUsername());
                String photo = record.getPhoto();
                if (StringUtils.isNotBlank(photo)) {
                    String fileKey = photo.split("\\*")[1];
                    netWorkUserWeekVO.setImageUrl(previewUrl + fileKey);
                }
                netWorkUserWeekVO.setRemark("工程师");
                List<String> weekStatus = returnWeekListNew(record.getUserid());
                netWorkUserWeekVO.setNetworkUserStatusWeek(weekStatus);
                userList.add(netWorkUserWeekVO);
            }
            BeanUtils.copyProperties(page, rsPage);
            Collections.sort(userList, new Comparator<NetWorkUserWeekVO>() {
                @Override
                public int compare(NetWorkUserWeekVO p1, NetWorkUserWeekVO p2) {
                    int p1_sum = p1.getNetworkUserStatusWeek().stream().mapToInt(Integer::parseInt).sum();
                    int p2_sum = p2.getNetworkUserStatusWeek().stream().mapToInt(Integer::parseInt).sum();
                    if (p1_sum >= p2_sum) {
                        return -1;
                    }
                    return 1;
                }
            });
            rsPage.setRecords(userList);
            return rsPage;
        }
        return new Page<>();
    }

    public String checkUserIsRole(String userId) {
        String roles = jeCoreEnduserDao.getRolesByUserId(userId);
        if (StringUtils.isBlank(roles)) {
            return BusinessConstant.CUSTOMER_ROLE_CUSTOMER;
        }
        if (roles.contains(RoleIdConstant.GLY) || roles.contains(RoleIdConstant.SHLD)) {
            return BusinessConstant.CUSTOMER_ROLE_MANGE;
        }
        if (roles.contains(RoleIdConstant.FWJL) || roles.contains(RoleIdConstant.GCS)) {
            return BusinessConstant.CUSTOMER_ROLE_OUTLETS;
        }
        return BusinessConstant.CUSTOMER_ROLE_CUSTOMER;
    }


    /**
     * 获取每个工程师空闲/繁忙
     *
     * @param engineerId
     * @return
     */
    public List<String> returnWeekListNew(String engineerId) {
        int count = nxOrderMapper.selectByEngineerBusyness(engineerId);
        //7天内的空闲/繁忙  0 空闲  1繁忙
        LinkedHashMap<String, String> dateMap = new LinkedHashMap();
        for (int i = 0; i < 7; i++) {
            String afterDate = DateUtil.getDateStringAfter(new Date(), i);
            if (count > 0) {
                dateMap.put(afterDate, "1");
            } else {
                dateMap.put(afterDate, "0");
            }
        }
        List<String> weekStatus = new ArrayList<>();
        for (String key : dateMap.keySet()) {
            weekStatus.add(dateMap.get(key));
        }
        return weekStatus;
    }

    public List<String> returnWeekList(String engineerId) {

        String curDate = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
        List<NxOrderEntity> orderList = nxOrderMapper.selectByEngineer(engineerId, curDate);

        //初始化日期,默认都是空闲  0 空闲  1繁忙
        LinkedHashMap<String, String> dateMap = new LinkedHashMap();
        for (int i = 0; i < 7; i++) {
            String afterDate = DateUtil.getDateStringAfter(new Date(), i);
            dateMap.put(afterDate, "0");
        }

        for (NxOrderEntity item : orderList) {
            //只要当天订单状态是存在1，3，4就是繁忙，之后一周都是繁忙
            String statusCode = item.getOrderOrderstatusCode();
            if ("1".equals(statusCode) || "3".equals(statusCode) || "4".equals(statusCode)) {
                dateMap.clear();
                for (int i = 0; i < 7; i++) {
                    String afterDate = DateUtil.getDateStringAfter(new Date(), i);
                    dateMap.put(afterDate, "1");
                }
                break;
            }

//            String orderApplydate = item.getOrderApplydate();   //订单开始时间
//            String orderShtgsj = item.getOrderShtgsj();    //审核通过时间
//            String orderPlanendtime = item.getOrderPlanendtime();  // 订单结束时间
//            //如果存在订单，结束时间小于=当天，还没有审核通过，之后7天都变成繁忙，退出标志。
//            for (String key : dateMap.keySet()) {
//                if(dateMap.get(key).equals("1")){
//                    continue;
//                }
//                //区间的第一天，开始时间小于第一天，并且审核时间为空，当天变成繁忙
//                if(orderPlanendtime.compareTo(curDate) <= 0 && StringUtils.isBlank(orderShtgsj)){
//                    dateMap.put(curDate,"1");
//                    continue;
//                }
//                //判断订单开始小于这个区间任意一天，订单结束时间大于这个区间任意一天
//                if (orderApplydate.compareTo(key) <= 0 && orderPlanendtime.compareTo(key) >= 0) {
//                    if(StringUtils.isNotBlank(orderShtgsj)){
//                        if(orderShtgsj.compareTo(key)==0 ){
//                            dateMap.put(key,"1");
//                            break;
//                        }
//                    }else{
//                        dateMap.put(key,"1");   //繁忙
//                    }
//
//                }
//            }
        }

        List<String> weekStatus = new ArrayList<>();
        for (String key : dateMap.keySet()) {
            weekStatus.add(dateMap.get(key));
        }
        return weekStatus;
    }


    public List<JeCoreEnduserEntity> findBindUser() {
        List<NetworkUserEntity> networkUserList = networkUserMapper.selectList(new QueryWrapper<NetworkUserEntity>().lambda().select(NetworkUserEntity::getUserid));
        if (CollectionUtils.isNotEmpty(networkUserList)) {
            Set<String> userIds = networkUserList.stream().collect(Collectors.groupingBy(NetworkUserEntity::getUserid)).keySet();
            return jeCoreEnduserDao.selectList(new QueryWrapper<JeCoreEnduserEntity>().lambda().in(JeCoreEnduserEntity::getUserid, userIds));
        }
        return new ArrayList<>();
    }

    /**
     * 查询忙碌工程师和空闲工程师
     *
     * @return
     */
    public List<ServiceViewResultModel> engineerList() {
        List<ServiceViewResultModel> resultList = new ArrayList<>();
        //所有网点下的工程师
        List<EngineerModel> networkUserList = nxOrderMapper.getEngineer();
        //通过工程师查询名下的
        networkUserList.forEach(e -> {
            ServiceViewResultModel model = new ServiceViewResultModel();
            model.setFwryYhxm(e.getFwryYhxm());//姓名
            model.setFwrySjh(e.getFwrySjh());//手机号
            model.setUserId(e.getUserId());//工程师id
            // model.setPhoto(e.getPhoto());//头像
            String orderAssignengineer = e.getFwryYhxm();
            List<NxOrderEntity> orderList = nxOrderMapper.getOrderByEngineer(orderAssignengineer);
            if (CollectionUtils.isNotEmpty(orderList)) {
                NxOrderEntity nxOrderEntity = orderList.get(0);
                //取订单经纬度，倒序排第一条就是最近的
                model.setOrderNum(orderList.size());//订单数量
                model.setOrderSignaddress(nxOrderEntity.getOrderSignaddress());//地址
                model.setOrderSignaddressLongitude(nxOrderEntity.getOrderSignaddressLongitude());//签到经度
                model.setOrderSignaddressDimension(nxOrderEntity.getOrderSignaddressDimension());//签到纬度
                model.setOrderServiceaddressLongitude(nxOrderEntity.getOrderServiceaddressLongitude());//服务经度
                model.setOrderServiceaddressDimension(nxOrderEntity.getOrderServiceaddressDimension());//服务纬度
                model.setOrderWdjd(nxOrderEntity.getOrderWdjd());//网点经度
                model.setOrderWdwd(nxOrderEntity.getOrderWdwd());//网点维度

                //筛选    订单处于待报工、待审核，他就是忙碌的 其他状态的单，就是空闲
                List<NxOrderEntity> collect = orderList.stream().filter(NxOrderEntity -> "4".equals(NxOrderEntity.getOrderOrderstatusCode())
                        || "5".equals(NxOrderEntity.getOrderOrderstatusCode())).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(collect)) {
                    model.setStatus(1);
                } else {
                    model.setStatus(0);
                }
            } else {//工程师无订单
                model.setOrderNum(0);
                model.setStatus(0);
                if (null != e.getFwwdWdjwd()) {
                    String[] split = e.getFwwdWdjwd().split(",");
                    model.setOrderSignaddressLongitude(split[0]);//经度
                    model.setOrderSignaddressDimension(split[1]);//纬度
                }
            }
            resultList.add(model);
        });
        for (ServiceViewResultModel model : resultList) {
            ServiceViewResultModel dto = nxOrderMapper.getPhotoByUserId(model.getUserId());
            if (null != dto) {
                model.setPhoto(dto.getPhoto());
            }
        }

        return resultList;
    }

    //待处理订单
    public List<PendingOrderModel> pendingOrder() {
        return nxOrderMapper.pendingOrder();
    }

    public int equipmentOrderNum(String orderDeviceCode) {
        return nxOrderMapper.equipmentOrderNum(orderDeviceCode);
    }

    public List<FwryDTO> getNxgxJcdaFwryList(FwryDTO reqDto) {
        List<FwryDTO> resultList = nxOrderMapper.getNxgxJcdaFwryList(reqDto);
        if (null == reqDto.getFwryJscode() || reqDto.getFwryJscode().size() == 0) {
            return resultList;
        }
        List<FwryDTO> fwryDTOS = new ArrayList<>();
        for (FwryDTO fwryDTO : resultList) {
            for (String jsCode : reqDto.getFwryJscode()) {
                if (fwryDTO.getFwryJs().contains(jsCode)) {
                    fwryDTOS.add(fwryDTO);
                    break;
                }
            }
        }
        return fwryDTOS;
    }

    public Integer delFwry(String nxgxJcdaFwryId) {
        return nxOrderMapper.delFwry(nxgxJcdaFwryId);
    }

    public FwryDTO updateNxgxJcdaFwry(FwryDTO reqDto) {
        //获取网点下当前工程师角色是否有服务经理 如果是服务经理，则不修改状态
        List<FwryDTO> fwryDtoList = nxOrderMapper.getNxgxJcdaFwryByUserId(reqDto.getUserId());

        for (FwryDTO dto : fwryDtoList) {
            if (!"服务经理".equals(dto.getFwryJs())) {
                List<NxOrderEntity> collect = nxOrderMapper.getOrderByEngineerId(dto.getUserId()).stream().filter(NxOrderEntity -> "4".equals(NxOrderEntity.getOrderOrderstatusCode())
                        || "5".equals(NxOrderEntity.getOrderOrderstatusCode())).collect(Collectors.toList());
                HashMap<String, String> req = new HashMap<>();
                req.put("userId", reqDto.getUserId());
                if (CollectionUtils.isNotEmpty(collect)) {
                    req.put("fwryZt", "忙碌");
                    req.put("fwryZtCode", BusinessConstant.NETWORK_USER_STATUS_BUSY);
                } else {
                    req.put("fwryZt", "空闲");
                    req.put("fwryZtCode", BusinessConstant.NETWORK_USER_STATUS_FREE);
                }
                nxOrderMapper.updateNxgxJcdaFwryByUserId(req);
            }
        }
        return new FwryDTO();
    }

    public Integer getOrderNum(String startDate, String endDate) {
        HashMap<String, String> req = new HashMap<>();
        req.put("startDate", startDate);
        req.put("endDate", endDate);
        return nxOrderMapper.getOrderNum(req);
    }

    /**
     * 用户查询自己的设备
     *
     * @param userId
     * @param sblbSbbm
     * @param nxJcxxSblbId
     * @param pageSize
     * @param pageNum
     * @return
     */
    public JsonResult<Object> getSblbByUserId(String userId, String sblbSbbm, String nxJcxxSblbId, Integer pageSize, Integer pageNum) {
        Log.info("userId：" + userId + "sblbSbbm：" + sblbSbbm + "nxJcxxSblbId：" + nxJcxxSblbId);
        //返回map
        Map<String, Object> result = new HashMap<>();
        //在人员表中查询该员工的角色信息
        JeCoreEnduserEntity user = nxOrderMapper.getCoreEnduserByUsrId(userId);
        String roleIds = user.getRoleids();
        if (StringUtils.isNotBlank(roleIds)) {
            //根据id查询，单独写出来
            if (StringUtils.isNotBlank(nxJcxxSblbId)) {
                Map<String, Object> req = new HashMap<>();
                req.put("id", nxJcxxSblbId);
                req.put("size", 1);
                req.put("page", 0);
                List<NxDeviceEntity> sblbList = nxOrderMapper.getNxDeviceEntity(req);
                result.put("total", sblbList.size());
                result.put("list", sblbList);
                return JsonResult.success(result);
            }
            //系统管理员，管理员，售后领导
            if (roleIds.contains(RoleIdConstant.XTGLY) || roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
                //所有的设备列表信息
                Map<String, Object> req = new HashMap<>();
                req.put("sblbSbbm", sblbSbbm);
                req.put("size", pageSize);
                req.put("page", (pageNum - 1) * pageSize);
                List<NxDeviceEntity> sblbList = nxOrderMapper.getNxDeviceEntity(req);
                int count = nxOrderMapper.getNxDeviceEntityCount(req);
                result.put("total", count);
                result.put("list", sblbList);
                return JsonResult.success(result);
            }
            //服务经理，工程师
            if (roleIds.contains(RoleIdConstant.FWJL) || roleIds.contains(RoleIdConstant.GCS)) {
                //一个人只能绑定一个服务网点，所以list里只有一个或者没有
                List<FwryDTO> fwryList = nxOrderMapper.getNxgxJcdaFwryByUserId(userId);
                if (CollectionUtils.isEmpty(fwryList)) {
                    return JsonResult.failed("500", "未在网点下查到该员工信息！");
                }
                FwryDTO fwryDTO = fwryList.get(0);
                //通过网点id获取服务网点信息
                NetworkEntity networkEntity = nxOrderMapper.getfwwd(fwryDTO);
                //获取网点下的服务区域
                String[] split = networkEntity.getFwwdSsqy().split(",");
                //通过网点的服务区域，去查询客户
                List<KhglDTO> khglDTOList = nxOrderMapper.getKhBykhdzs(split);
                //通过客户名称查询设备列表信息
                //所有的设备列表信息
                if (CollectionUtils.isEmpty(khglDTOList)) {
                    return JsonResult.success(null);
                }
                HashMap<String, Object> req = new HashMap<>();
                String khmc = "";
                for (KhglDTO dto : khglDTOList) {
                    khmc += "'" + dto.getKhglKhmc() + "',";
                }
                khmc = khmc.substring(0, khmc.length() - 1);
                req.put("khmc", khmc);
                req.put("sblbSbbm", sblbSbbm);
                Log.info("工程师或服务经理查询客户名称:{},设备编码:{}", khmc + "===============》" + sblbSbbm);
                //设置分页
                req.put("size", pageSize);
                req.put("page", (pageNum - 1) * pageSize);
                List<NxDeviceEntity> sblbList = nxOrderMapper.getNxDeviceEntity(req);
                int count = nxOrderMapper.getNxDeviceEntityCount(req);
                result.put("total", count);
                result.put("list", sblbList);
                return JsonResult.success(result);
            }
            //客户，客户领导
            if (roleIds.contains(RoleIdConstant.KH) || roleIds.contains(RoleIdConstant.KHLD)) {
                Map<String, Object> req = new HashMap<>();
                req.put("userId", user.getUserid());
                req.put("sblbSbbm", sblbSbbm);
                req.put("size", pageSize);
                req.put("page", (pageNum - 1) * pageSize);
                List<NxDeviceEntity> sblbList = nxOrderMapper.getNxDeviceEntityList(req);
                int count = nxOrderMapper.getNxDeviceEntityListCount(req);
                result.put("total", count);
                result.put("list", sblbList);
                return JsonResult.success(result);
            }
        }
        return JsonResult.success(null);
    }

    public JsonResult<List<PendingOrderModel>> getServiceView(String isService) {
        if (StringUtils.isEmpty(isService)) {
            return JsonResult.failed("500", "未传必要参数！");
        }
        HashMap<String, Object> req = new HashMap<>();
        req.put("isService", isService);
        LoginUserInfo current = UserContext.current();
        if (null != current && StringUtils.isEmpty(current.getUserId())) {
            return JsonResult.failed("500", "token异常！");
        }
        String userId = current.getUserId();
        String userName = current.getUserName();
        //Log.info("current.getUserName()查询到的userName：",userName);
        if (StringUtils.isEmpty(userName)) {
            //LoginUserInfo userInfo = enduserDao.getUserInfo(userId);
            UserInfo user = enduserDao.getUser(userId);

            userName = user.getUserName();
        }


        String roleids = nxOrderMapper.getUserInfo(userId);
        if (roleids.contains(RoleIdConstant.GLY) || roleids.contains(RoleIdConstant.SHLD)) {
            List<PendingOrderModel> resultList = nxOrderMapper.getServiceView(req);
            return JsonResult.success(resultList);
        }
        if (roleids.contains(RoleIdConstant.KH) ||
                roleids.contains(RoleIdConstant.KHLD)) {//客户或者客户领导
            String khid = enduserDao.getCustomByUserId(userId);
            List<String> khids = new ArrayList<String>();
            khids.add(khid);
            req.put("khids", khids);
            List<PendingOrderModel> resultList = nxOrderMapper.getServiceView(req);
            return JsonResult.success(resultList);
        }
        if (roleids.contains(RoleIdConstant.FWJL)) {//服务经理
            String[] split = enduserDao.getSiteByUserId(userId).getServiceArea().split(",");
            //通过网点的服务区域，去查询客户id
            List<String> collect = nxOrderMapper.getKhBykhdzs(split).stream()
                    .map(item -> {
                        return item.getNxKhglId();
                    })
                    .collect(Collectors.toList());
            req.put("khids", collect);
            List<PendingOrderModel> resultList = nxOrderMapper.getServiceView(req);
            return JsonResult.success(resultList);
        }
        if (roleids.contains(RoleIdConstant.GCS)) {//工程师
            String[] split = enduserDao.getSiteByUserId(userId).getServiceArea().split(",");
            //通过网点的服务区域，去查询客户id
            List<String> collect = nxOrderMapper.getKhBykhdzs(split).stream()
                    .map(item -> {
                        return item.getNxKhglId();
                    })
                    .collect(Collectors.toList());
            req.put("khids", collect);
            //req.put("userId",userId);
            req.put("userId", userName);
            List<PendingOrderModel> resultList = nxOrderMapper.getServiceView(req);
            return JsonResult.success(resultList);
        }
        return JsonResult.success(null);
    }

    public JsonResult<List<NxOrderEntity>> getServiceViewBySbbm(String sbbm, String isService) {
        /*if(StringUtils.isEmpty(sbbm)){
            return JsonResult.failed("500","未传必要参数设备编码！");
        }*/
        HashMap<String, Object> req = new HashMap<>();
        req.put("sbbm", sbbm);
        req.put("isService", isService);
        LoginUserInfo current = UserContext.current();
        if (null != current && StringUtils.isEmpty(current.getUserId())) {
            return JsonResult.failed("500", "token异常！");
        }
        String userId = current.getUserId();
        String userName = current.getUserName();
        if (StringUtils.isEmpty(userName)) {
            //LoginUserInfo userInfo = enduserDao.getUserInfo(userId);
            UserInfo user = enduserDao.getUser(userId);
            userName = user.getUserName();
        }
        String roleids = nxOrderMapper.getUserInfo(userId);
        if (roleids.contains(RoleIdConstant.GLY) || roleids.contains(RoleIdConstant.SHLD)) {
            List<NxOrderEntity> resultList = nxOrderMapper.getServiceViewBySbbm(req);
            return JsonResult.success(resultList);
        }
        if (roleids.contains(RoleIdConstant.KH) ||
                roleids.contains(RoleIdConstant.KHLD)) {//客户或者客户领导
            String khid = enduserDao.getCustomByUserId(userId);
            List<String> khids = new ArrayList<String>();
            khids.add(khid);
            req.put("khids", khids);
            List<NxOrderEntity> resultList = nxOrderMapper.getServiceViewBySbbm(req);
            return JsonResult.success(resultList);
        }
        if (roleids.contains(RoleIdConstant.FWJL)) {//服务经理
            String[] split = enduserDao.getSiteByUserId(userId).getServiceArea().split(",");
            //通过网点的服务区域，去查询客户id
            List<String> collect = nxOrderMapper.getKhBykhdzs(split).stream()
                    .map(item -> {
                        return item.getNxKhglId();
                    })
                    .collect(Collectors.toList());
            req.put("khids", collect);
            List<NxOrderEntity> resultList = nxOrderMapper.getServiceViewBySbbm(req);

            return JsonResult.success(resultList);
        }
        if (roleids.contains(RoleIdConstant.GCS)) {//工程师
            String[] split = enduserDao.getSiteByUserId(userId).getServiceArea().split(",");
            //通过网点的服务区域，去查询客户id
            List<String> collect = nxOrderMapper.getKhBykhdzs(split).stream()
                    .map(item -> {
                        return item.getNxKhglId();
                    })
                    .collect(Collectors.toList());
            req.put("khids", collect);
            //req.put("userId",userId);
            req.put("userId", userName);
            List<NxOrderEntity> resultList = nxOrderMapper.getServiceViewBySbbm(req);
            return JsonResult.success(resultList);
        }
        return JsonResult.success(null);
    }

    public JsonResult<List<NxDeviceLocationEntity>> getSbjwd(String deviceId) {
        if (StringUtils.isNotEmpty(deviceId)) {
            List<NxDeviceLocationEntity> collect = nxDeviceLocationMapper.findAllDeviceLocation()
                    .stream().filter(e -> Objects.equals(deviceId, e.getDeviceId())).collect(Collectors.toList());
            for (NxDeviceLocationEntity entity : collect) {
                String gps = entity.getLongitude() + "," + entity.getLatitude();
                String address = null;
                address = gaoDeAPIUtils.getAddressByJwd(gps);
                entity.setAddress(address);
            }

            return JsonResult.success(collect);
        }
        List<NxDeviceLocationEntity> allDeviceLocation = nxDeviceLocationMapper.findAllDeviceLocation();
        for (NxDeviceLocationEntity entity : allDeviceLocation) {
            String gps = entity.getLongitude() + "," + entity.getLatitude();
            String address = null;
            address = gaoDeAPIUtils.getAddressByJwd(gps);
            entity.setAddress(address);
        }
        return JsonResult.success(allDeviceLocation);
    }

    public JsonResult<HashMap<String, Integer>> getServiceViewCount() {
        HashMap<String, Integer> res = new HashMap<>();
        res.put("fwz", 0);//服务中
        res.put("dfw", 0);//待服务
        res.put("dcl", 0);//待处理
        List<NxOrderEntity> data = getServiceViewBySbbm(null, "2").getData();
        if (CollectionUtils.isNotEmpty(data)) {
            List<NxOrderEntity> dfw = data.stream().filter(e -> Objects.equals("1", e.getOrderOrderstatusCode()) || Objects.equals("3", e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> fwz = data.stream().filter(e -> Objects.equals("4", e.getOrderOrderstatusCode()) || Objects.equals("5", e.getOrderOrderstatusCode())).collect(Collectors.toList());
            res.put("fwz", fwz.size());//服务中
            res.put("dfw", dfw.size());//待服务
            res.put("dcl", data.size());//待处理
        }
        return JsonResult.success(res);
    }
}
