package com.rootcloud.domain.service.order;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.rootcloud.common.JsonResult;
import com.rootcloud.domain.ao.OrderDispatchForm;
import com.rootcloud.domain.ao.ReportStatisticAO;
import com.rootcloud.domain.dto.OrderVO;
import com.rootcloud.domain.entity.MyMessageEntity;
import com.rootcloud.domain.entity.NxOrderEntity;
import com.rootcloud.domain.vo.*;
import com.rootcloud.vo.order.OrderParam;
import com.rootcloud.vo.order.OrderSignInParam;

import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 订单service
 * </p>
 * 
 * @author HeJia
 * @since 2022/5/17
 */
public interface OrderService extends IService<NxOrderEntity> {

    /**
     * 订单分页
     *
     * @param page
     *            当前页
     * @param size
     *            每页显示数
     * @param serviceType
     *            服务类型
     * @param orderStatus
     *            订单状态
     * @return 分页的数据
     */
    IPage<OrderVO> orderPage(int page, int size, String serviceType, String orderStatus);

    /**
     * 订单详情
     *
     * @param orderId
     *            订单主键
     * @return 详情数据
     */
    OrderVO orderInfo(String orderId);


    /**
     * 订单签到功能
     * @param orderSignInParam
     */
    void orderSignIn(OrderSignInParam orderSignInParam);

    /**
     * 派工
     * 
     * @param form
     *            请求参数
     * @return true 成功
     */
    Boolean dispatch(OrderDispatchForm form);

    /**
     * 作废
     * 
     * @param orderId
     *            订单主键
     * @return true 成功
     */
    Boolean invalid(String orderId);

    List orderProcess(String orderId);

    /**
     * 根据设备查询
     */
    //List<MyMessageEntity> queryOrderInfo(List<DeviceListEntity> deviceListEntities);


    /**
     * 根据参数查询订单汇总
     * @param reportStatisticAO
     * @return
     */
    OrderStatusStatistic orderStatusStatistic(ReportStatisticAO reportStatisticAO);


    /**
     * 查询设备状态汇总
     * @return
     */
    DeviceStatusStatistic  deviceStatusStatistic();


    /**
     * 订单和设备状态查询
     * @param reportStatisticAO
     * @return
     */
    StatusVO  orderAndDeviceStatus(ReportStatisticAO reportStatisticAO);


    /**
     * 网点统计
     * @param reportStatisticAO
     * @return
     */
    List<Outlets>  regionStatistic(ReportStatisticAO reportStatisticAO);


    /**
     * 按照日期订单统计
     * @param reportStatisticAO
     * @return
     */
    List<OrderStatistic> orderStatisticByDay(ReportStatisticAO reportStatisticAO);


    /**
     * 人员统计所有查询
     * @param reportStatisticAO
     * @return
     */
    PersonAllStatistic checkPersonAllStatistic(ReportStatisticAO reportStatisticAO);


    /**
     * 查询人员统计数据
     * @param reportStatisticAO
     * @param sortType
     * @return
     */
    List<PersonnelStatistic> checkPersonStatistic(ReportStatisticAO reportStatisticAO,String sortType);


    /**
     * 查询设备故障前十统计
     * @param reportStatisticAO
     * @return
     */
    List<FaultNumber> faultStatistic(ReportStatisticAO reportStatisticAO);


    /**
     * 配件故障统计
     * @param reportStatisticAO
     * @return
     */
    List<PartsFaultNumber> partsFaultStatistic(ReportStatisticAO reportStatisticAO);


    /**
     * 配件销售额统计
     * @param reportStatisticAO
     * @return
     */
    List<FittingsSales> saleFittingsStatistic(ReportStatisticAO reportStatisticAO);


    /**
     * 人员销售
     * @param reportStatisticAO
     * @return
     */
    List<CustomerSales> saleCustomerStatistic(ReportStatisticAO reportStatisticAO);


    /**
     * 通过用户id查询订单
     * @param userId
     * @param type
     * @param startNo
     * @param endNo
     * @return
     */
    List<MyMessageEntity>  selectByUserId(String zhId,String userId,String type,Integer startNo,Integer endNo);


    /**
     *  结算合同到期查询
     * @param userId
     * @return
     */
    List<MyMessageEntity> jsGetByUserId(String userId);


    /**
     * 服务合同到期查询
     * @param userId
     * @return
     */
    List<MyMessageEntity>  fwGetByUserId(String userId);


    /**
     * 订单移入代办库
     * @param orderId 订单id
     */
    void orderInsertLibrary(String orderId);
    /**
     * 订单消息已解决
     * @param orderId
     */
    void mesSfjj(String orderId);

    void htSerMesSfjj(String id);

    void jsSerMesSfjj(String id);

    HashMap getOrderTypeNum();

    HashMap getOrderTypeNum2();

    /**
     * 修改订单审核时间，完工时间，指派工程师时间
     */
    void updateDate();

    /**
     * 创建订单
     * @param param
     * @return
     */
    JsonResult<Object> createOrder(OrderParam param)  throws Exception;
    JsonResult<Object> createOrder2(OrderParam param)  throws Exception;
    JsonResult<Object> createOrder3(OrderParam param)  throws Exception;
    JsonResult<Object> createOrder4(OrderParam param)  throws Exception;
}
