package com.rootcloud.domain.service.order;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.*;
import com.rootcloud.common.constant.Properties;
import com.rootcloud.common.constant.RoleIdConstant;
import com.rootcloud.common.constant.TemplateConstant;
import com.rootcloud.common.enums.OrderEnums;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.common.util.DateUtils;
import com.rootcloud.common.util.OrderNumberUtils;
import com.rootcloud.config.auth.LoginUserInfo;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.auth.UserInfo;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.ao.OrderDispatchForm;
import com.rootcloud.domain.ao.ReportStatisticAO;
import com.rootcloud.domain.dao.KhDao;
import com.rootcloud.domain.dto.DeviceVO;
import com.rootcloud.domain.dto.OrderVO;
import com.rootcloud.domain.dto.UserRoleDeptDTO;
import com.rootcloud.domain.entity.*;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.entity.device.NxDeviceLocationEntity;
import com.rootcloud.domain.mapper.*;
import com.rootcloud.domain.service.auth.RootCloudAuthService;
import com.rootcloud.domain.service.auth.RootCloudAuthServiceImpl;
import com.rootcloud.domain.service.device.NxDeviceService;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import com.rootcloud.domain.service.gkjk.NxJeDeviceService;
import com.rootcloud.domain.service.network.NetworkUserService;
import com.rootcloud.domain.service.todo.NxTodoLibraryService;
import com.rootcloud.domain.vo.*;
import com.rootcloud.vo.order.DeviceDataVo;
import com.rootcloud.vo.order.OrderLcinfoParam;
import com.rootcloud.vo.order.OrderParam;
import com.rootcloud.vo.order.OrderSignInParam;
import com.rootcloud.vo.service.OutletsVO;
import jline.internal.Log;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单service实现类
 * </p>
 *
 * @author HeJia
 * @since 2022/5/17
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class OrderServiceImpl extends ServiceImpl<NxOrderMapper, NxOrderEntity> implements OrderService {

    private final Logger logger = LoggerFactory.getLogger(OrderServiceImpl.class);

    @Resource(name = "nxJeDeviceServiceImpl")
    private NxJeDeviceService nxJeDeviceService;

    @Resource(name = "nxDeviceServiceImpl")
    private NxDeviceService nxDeviceService;

    @Autowired
    private NxDeviceLocationMapper deviceLocationMapper;

    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;

    @Resource(name = "nxTodoLibraryServiceImpl")
    private NxTodoLibraryService nxTodoLibraryService;

    @Resource(name = "cloudAuthService")
    private RootCloudAuthService rootCloudAuthService;

    private final NxOrderMapper nxOrderMapper;

    private final NxPlpzMapper nxPlpzMapper;

    @Autowired
    private final KhDao kuMapper;
    @Autowired
    Properties wfProperties;
    @Autowired
    private TechnicalTaskDAO technicalTaskDAO;
    @Autowired
    ServicePlanMapper servicePlanMapper;
    private final NxDeviceMapper nxDeviceMapper;
    private static double EARTH_RADIUS = 6378137;
    private static double RAD = Math.PI / 180.0;
    @Autowired
    NetworkUserService networkUserService;

    @Override
    public IPage<OrderVO> orderPage(int page, int size, String serviceType, String orderStatus) {

        LambdaQueryWrapper<NxOrderEntity> queryWrapper = Wrappers.lambdaQuery(NxOrderEntity.class)
                .eq(StrUtil.isNotBlank(serviceType), NxOrderEntity::getOrderServicetypeCode, serviceType)
                .eq(StrUtil.isNotBlank(orderStatus), NxOrderEntity::getOrderOrderstatusCode, orderStatus)
                .orderByDesc(NxOrderEntity::getSyCreatetime);

        Page<NxOrderEntity> result = this.page(new Page<>(page, size), queryWrapper);
        Page<OrderVO> voPage = new Page<>();
        BeanUtil.copyProperties(result, voPage);
        return voPage;
    }

    @Override
    public OrderVO orderInfo(String orderId) {
        NxOrderEntity entity = Optional.ofNullable(getById(orderId)).orElse(new NxOrderEntity());
        OrderVO vo = BeanUtil.copyProperties(entity, OrderVO.class);
        if (StrUtil.isNotBlank(vo.getOrderDevicecode())) {
            vo.setOnline(nxJeDeviceService.queryOnlineStatus(vo.getOrderDevicecode()));
        }
        return vo;
    }

    @Override
    public Boolean dispatch(OrderDispatchForm form) {
        NxOrderEntity entity = BeanUtil.copyProperties(form, NxOrderEntity.class);
        entity.setOrderDispatchtime(DateUtil.now());
        entity.setOrderOrderstatusCode(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode());
        entity.setOrderOrderstatusName(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getDesc());
        return this.updateById(entity);
    }

    @Override
    public Boolean invalid(String orderId) {
        NxOrderEntity entity = new NxOrderEntity();
        entity.setNxOrderId(orderId);
        entity.setOrderOrderstatusCode(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode());
        entity.setOrderOrderstatusName(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getDesc());
        return this.updateById(entity);
    }

    @Override
    public List orderProcess(String orderId) {
        NxOrderEntity entity = Optional.ofNullable(getById(orderId)).orElse(new NxOrderEntity());
        List<HashMap> result = new ArrayList<HashMap>();

        HashMap<String, Object> map1 = new HashMap<String, Object>();
        map1.put("title", "派工信息");
        map1.put("time", entity.getOrderDispatchtime());//派单时间
        HashMap<String, String> map1a = new HashMap<String, String>();
        map1a.put("orderAssignengineer", entity.getOrderAssignengineer());//指派工程师
        map1a.put("orderAssistengineer", entity.getOrderAssistengineer());//协助工程师
        map1a.put("orderPlanstarttime", entity.getOrderPlanstarttime());//计划开始时间
        map1a.put("orderPlanendtime", entity.getOrderPlanendtime());//计划完成时间
        map1.put("desc", map1a);
        result.add(map1);

        HashMap<String, Object> map2 = new HashMap<String, Object>();
        map2.put("title", "已签到");
        map2.put("time", entity.getOrderSigntime());//签到时间
        HashMap<String, String> map2a = new HashMap<String, String>();
        map2a.put("orderSignaddress", entity.getOrderSignaddress());//签到地址
        map1.put("desc", map2a);
        result.add(map2);

        HashMap<String, Object> map3 = new HashMap<String, Object>();
        map3.put("title", "报工信息");
        HashMap<String, String> map3a = new HashMap<String, String>();
        map3a.put("orderFaulttyprName", entity.getOrderFaulttyprName());//故障类型
        map3a.put("orderCauseofissue", entity.getOrderCauseofissue());//故障原因
        map3a.put("orderCauseofissuefile", entity.getOrderCauseofissuefile());//故障原因图片
        map3a.put("orderSolution", entity.getOrderSolution());//解决方案
        map3a.put("orderSolutionfile", entity.getOrderSolutionfile());//解决方案图片
        map3a.put("orderSolutionfile", entity.getOrderSolutionfile());//解决方案图片
        map3a.put("orderRemark", entity.getOrderRemark());//备注
        map3a.put("orderRemarkfile", entity.getOrderRemarkfile());//备注图片
        map3.put("desc", map3a);
        result.add(map3);

        HashMap<String, Object> map4 = new HashMap<String, Object>();
        map4.put("title", "审核信息");
        HashMap<String, String> map4a = new HashMap<String, String>();
        if (entity.getOrderOrderstatusCode().equals("6") ||
                entity.getOrderOrderstatusCode().equals("7")) {
            map4a.put("syAudflag", "通过");
        }
        if (entity.getOrderOrderstatusCode().equals("8")) {
            map4a.put("syAudflag", "拒绝");
        }
        map4.put("desc", map4a);
        result.add(map4);
        return result;
    }

    @Override
    public OrderStatusStatistic orderStatusStatistic(ReportStatisticAO reportStatisticAO) {
        LoginUserInfo current = UserContext.current();
        SimpleDateFormat stf = new SimpleDateFormat("yyyy-MM-dd");
        OrderStatusStatistic orderStatusStatistic = new OrderStatusStatistic();
        String wdUserId = null;
        String userIds = null;
        List<String> list = new ArrayList<>();
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            SiteDTO siteByUserId = enduserService.getSiteByUserId(current.getUserId());
            if (!ObjectUtils.isEmpty(siteByUserId)) {
                String[] strings = siteByUserId.getServiceArea().split(",");
                for (int i = 0; i < strings.length; i++) {
                    String areaName = strings[i];
                    String customs = enduserService.getCustomsBySiteAreaName(areaName);
                    if (StringUtils.isNotBlank(customs)) {
                        list.add(customs);
                    }
                }
            }
            if (list.isEmpty()) {
                return orderStatusStatistic;
            }
        } else {
            return orderStatusStatistic;
        }
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
//        String orderStatus = reportStatisticAO.getType();
        if (list.size() > 0) {
            userIds = String.join(",", list);
        }

        orderStatusStatistic = nxOrderMapper.orderCount(startTime, endTime, userIds, wdUserId);


        //今日新增
        startTime = stf.format(new Date());
        endTime = startTime;
        List<NxOrderEntity> orderAdd = nxOrderMapper.orderStatusStatistic(startTime, endTime, null, userIds, wdUserId);

        //今日完成
//        orderStatus = ;
        List<NxOrderEntity> oderFInish = nxOrderMapper.orderStatusStatistic(startTime, endTime, OrderEnums.OrderStatus.DONE.getCode(), userIds, wdUserId);

        orderStatusStatistic.setTodayAdd(String.valueOf(orderAdd.size()));
        orderStatusStatistic.setTodayFinish(String.valueOf(oderFInish.size()));

        return orderStatusStatistic;
    }

    @Override
    public DeviceStatusStatistic deviceStatusStatistic() {
        LoginUserInfo current = UserContext.current();
        DeviceStatusStatistic deviceStatusStatistic = new DeviceStatusStatistic();
        String userIds = null;
        List<String> list = new ArrayList<>();
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            SiteDTO siteByUserId = enduserService.getSiteByUserId(current.getUserId());
            if (!ObjectUtils.isEmpty(siteByUserId)) {
                String[] strings = siteByUserId.getServiceArea().split(",");
                for (int i = 0; i < strings.length; i++) {
                    String areaName = strings[i];
                    String customs = enduserService.getCustomsBySiteAreaName(areaName);
                    if (StringUtils.isNotBlank(customs)) {
                        list.add(customs);
                    }
                }
            }
            if (list.isEmpty()) {
                return deviceStatusStatistic;
            }
        } else {
            return deviceStatusStatistic;
        }
        if (list.size() > 0) {
            userIds = String.join(",", list);
        }
        deviceStatusStatistic = nxOrderMapper.deviceStatusStatistic(userIds);
        return deviceStatusStatistic;

    }

    @Override
    public StatusVO orderAndDeviceStatus(ReportStatisticAO reportStatisticAO) {
        OrderStatusStatistic orderStatusStatistic = orderStatusStatistic(reportStatisticAO);
        DeviceStatusStatistic deviceStatusStatistic = deviceStatusStatistic();
        StatusVO statusVO = new StatusVO();
        statusVO.setOrderStatusStatistic(orderStatusStatistic);
        statusVO.setDeviceStatusStatistic(deviceStatusStatistic);
        return statusVO;
    }

    @Override
    public List<Outlets> regionStatistic(ReportStatisticAO reportStatisticAO) {
        LoginUserInfo current = UserContext.current();
        String wdUserId = null;
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            //返回网点人员
            wdUserId = current.getUserId();
        } else {
            return new ArrayList<Outlets>();
        }
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
        List<Outlets> outlets = nxOrderMapper.outletsStatistic(startTime, endTime, wdUserId);
        return outlets;
    }

    @Override
    public List<OrderStatistic> orderStatisticByDay(ReportStatisticAO reportStatisticAO) {
        LoginUserInfo current = UserContext.current();
        String userIds = null;
        List<String> list = new ArrayList<>();
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            SiteDTO siteByUserId = enduserService.getSiteByUserId(current.getUserId());
            if (!ObjectUtils.isEmpty(siteByUserId)) {
                String[] strings = siteByUserId.getServiceArea().split(",");
                for (int i = 0; i < strings.length; i++) {
                    String areaName = strings[i];
                    String customs = enduserService.getCustomsBySiteAreaName(areaName);
                    if (StringUtils.isNotBlank(customs)) {
                        list.add(customs);
                    }
                }
            }
            if (list.isEmpty()) {
                return new ArrayList<OrderStatistic>();
            }
        } else {
            return new ArrayList<OrderStatistic>();
        }
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
        if (list.size() > 0) {
            userIds = String.join(",", list);
        }
        List<OrderStatistic> orderStatistics = new ArrayList<>();
        if ("4".equals(reportStatisticAO.getType())) {
            orderStatistics = nxOrderMapper.orderStatisticByDayMonth(UserContext.getTenantId(), startTime, endTime, userIds, null);
        } else {
            orderStatistics = nxOrderMapper.orderStatisticByDay(UserContext.getTenantId(), startTime, endTime, userIds, null);
        }

        return orderStatistics;
    }


    @Override
    public PersonAllStatistic checkPersonAllStatistic(ReportStatisticAO reportStatisticAO) {
        PersonAllStatistic personAllStatistic = new PersonAllStatistic();
        if (reportStatisticAO.getSelectType().contains("orders")) {
            List<PersonnelStatistic> orderNum = checkPersonStatistic(reportStatisticAO, "orderNum");
            personAllStatistic.setOrders(orderNum);
        }
        if (reportStatisticAO.getSelectType().contains("appraises")) {
            List<PersonnelStatistic> serviceNum = checkPersonStatistic(reportStatisticAO, "serviceNum");
            personAllStatistic.setAppraises(serviceNum);
        }
        if (reportStatisticAO.getSelectType().contains("works")) {
            /*List<PersonnelStatistic> workTimeNum = checkPersonStatistic(reportStatisticAO, "workTimeNum");
            personAllStatistic.setWorks(workTimeNum);*/
            List<PersonnelStatistic> workTimeNum = this.getGzsc(reportStatisticAO);
            personAllStatistic.setWorks(workTimeNum);
        }
        /*if (reportStatisticAO.getSelectType().contains("test")) {
            List<PersonnelStatistic> workTimeNum = this.getGzsc(reportStatisticAO);
            personAllStatistic.setWorks(workTimeNum);
        }*/
        if (reportStatisticAO.getSelectType().contains("faultNumbers")) {
            List<FaultNumber> faultNumbers = faultStatistic(reportStatisticAO);
            personAllStatistic.setFaultNumbers(faultNumbers);
        }
        if (reportStatisticAO.getSelectType().contains("partsFaultNumbers")) {
            List<PartsFaultNumber> partsFaultNumbers = partsFaultStatistic(reportStatisticAO);
            personAllStatistic.setPartsFaultNumbers(partsFaultNumbers);
        }
        if (reportStatisticAO.getSelectType().contains("fittingsSales")) {
            List<FittingsSales> fittingsSales = saleFittingsStatistic(reportStatisticAO);
            personAllStatistic.setFittingsSales(fittingsSales);
        }
        if (reportStatisticAO.getSelectType().contains("customerSales")) {
            List<CustomerSales> customerSales = saleCustomerStatistic(reportStatisticAO);
            personAllStatistic.setCustomerSales(customerSales);
        }
        if (reportStatisticAO.getSelectType().contains("serviceItem")) {
            //目前修改成10分制了
            //需要查询出评价项个数了
            int serviceItem = nxPlpzMapper.getCountServiceItem(null);
            personAllStatistic.setServiceItem(serviceItem);
        }
        return personAllStatistic;
    }


    @Override
    public List<PersonnelStatistic> checkPersonStatistic(ReportStatisticAO reportStatisticAO, String sortType) {
        LoginUserInfo current = UserContext.current();
        String wdUserId = null;
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            //返回网点人员
            wdUserId = current.getUserId();

        } else {
            return new ArrayList<>();
        }
        String serveType = reportStatisticAO.getServeType();
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
        String orderStatus = reportStatisticAO.getOrderStatus();
        String sortReverseOrder = reportStatisticAO.getSortReverseOrder();
        String syZhid = UserContext.getUserTenantId();
        List<PersonnelStatistic> personnelStatistics = nxOrderMapper.checkPersonStatistic(startTime, endTime, wdUserId, sortType, serveType, sortReverseOrder, orderStatus, syZhid);
        return personnelStatistics;
    }

    /**
     * 获取工作时长重写
     *
     * @param reportStatisticAO
     * @return
     */
    private List<PersonnelStatistic> getGzsc(ReportStatisticAO reportStatisticAO) {
        LoginUserInfo current = UserContext.current();
        String wdUserId = null;
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            //返回网点人员
            wdUserId = current.getUserId();

        } else {
            return new ArrayList<>();
        }
        String serveType = reportStatisticAO.getServeType();
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
        String orderStatus = reportStatisticAO.getOrderStatus();
        String sortReverseOrder = reportStatisticAO.getSortReverseOrder();
        String syZhid = UserContext.getUserTenantId();
        //获取工程师和协助工程师
        Set<String> gcsIdList = new HashSet<>();
        List<PersonnelStatistic> gcsIds = nxOrderMapper.getGcsIds(startTime, endTime, wdUserId, serveType, syZhid);
        if (gcsIds.size() > 0) {
            gcsIds.forEach(o -> {
                if (StringUtils.isNotBlank(o.getOrderAssistengineerid())) {
                    if (o.getOrderAssistengineerid().contains(",")) {
                        gcsIdList.addAll(Arrays.asList(o.getOrderAssistengineerid().split(",")));
                    } else {
                        gcsIdList.add(o.getOrderAssistengineerid());
                    }
                }
                if (StringUtils.isNotBlank(o.getOrderAssignengineerid())) {
                    if (o.getOrderAssignengineerid().contains(",")) {
                        gcsIdList.addAll(Arrays.asList(o.getOrderAssignengineerid().split(",")));
                    } else {
                        gcsIdList.add(o.getOrderAssignengineerid());
                    }
                }
            });
        }
        //循环查询工程师时长  enduserService
        List<PersonnelStatistic> personnelStatistics = new ArrayList<>();
        gcsIdList.forEach(o -> {
            PersonnelStatistic personnelStatistic = new PersonnelStatistic();
            personnelStatistic.setGcsId(o);
            UserInfo u = enduserService.getUser(o);
            if (u != null) {
                personnelStatistic.setPersonName(u.getUserName());
                //查询时长
                BigDecimal b = nxOrderMapper.getGcsGzsc(startTime, endTime, serveType, syZhid, o);
                personnelStatistic.setWorkTimeNum(b == null ? "0" : b.toString());
                personnelStatistic.setSc(b == null ? new BigDecimal(0) : b); //2024-03-22 whl修复
                personnelStatistics.add(personnelStatistic);
            }

        });
        //排序
        List<PersonnelStatistic> listSort = personnelStatistics.stream().sorted(Comparator.comparing(PersonnelStatistic::getSc).reversed().thenComparing(PersonnelStatistic::getSc)).collect(Collectors.toList());
        return listSort;
    }

    @Override
    public List<FaultNumber> faultStatistic(ReportStatisticAO reportStatisticAO) {
        LoginUserInfo current = UserContext.current();
        String wdUserId = null;
        List<String> list = new ArrayList<>();
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            //返回网点人员
            wdUserId = current.getUserId();
        } else {
            return new ArrayList<FaultNumber>();
        }
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
        List<FaultNumber> faultNumbers = nxOrderMapper.faultStatistic(startTime, endTime, null, wdUserId);
        return faultNumbers;
    }

    @Override
    public List<PartsFaultNumber> partsFaultStatistic(ReportStatisticAO reportStatisticAO) {
        LoginUserInfo current = UserContext.current();
        String wdUserId = null;
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            //返回网点人员
            wdUserId = current.getUserId();
        } else {
            return new ArrayList<PartsFaultNumber>();
        }
        String serveType = reportStatisticAO.getServeType();
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
        List<PartsFaultNumber> partsFaultNumbers = nxOrderMapper.partsFaultStatistic(startTime, endTime, wdUserId, serveType);
        return partsFaultNumbers;
    }

    @Override
    public List<FittingsSales> saleFittingsStatistic(ReportStatisticAO reportStatisticAO) {
        LoginUserInfo current = UserContext.current();
        String wdUserId = null;
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            //返回网点人员
            wdUserId = current.getUserId();
        } else {
            return new ArrayList<>();
        }
        String businessType = reportStatisticAO.getBusinessType();
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
        List<FittingsSales> fittingsSales = nxOrderMapper.saleFittingsStatistic(startTime, endTime, wdUserId, businessType);
        return fittingsSales;
    }

    @Override
    public List<CustomerSales> saleCustomerStatistic(ReportStatisticAO reportStatisticAO) {
        LoginUserInfo current = UserContext.current();
        String userIds = null;
        List<String> list = new ArrayList<>();
        UserInfo userInfo = enduserService.getUser(current.getUserId());
        String roleIds = userInfo.getRoleIds();
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD)) {
            //管理员
        } else if (roleIds.contains(RoleIdConstant.FWJL)) {
            SiteDTO siteByUserId = enduserService.getSiteByUserId(current.getUserId());
            if (!ObjectUtils.isEmpty(siteByUserId)) {
                String[] strings = siteByUserId.getServiceArea().split(",");
                for (int i = 0; i < strings.length; i++) {
                    String areaName = strings[i];
                    String customs = enduserService.getCustomsBySiteAreaName(areaName);
                    if (StringUtils.isNotBlank(customs)) {
                        list.add(customs);
                    }
                }
            }
            if (list.isEmpty()) {
                return new ArrayList<>();
            }
        }
        String businessType = reportStatisticAO.getBusinessType();
        String startTime = reportStatisticAO.getStartTime();
        String endTime = reportStatisticAO.getEndTime();
        if (list.size() > 0) {
            userIds = String.join(",", list);
        }
        List<CustomerSales> customerSales = nxOrderMapper.saleCustomerStatistic(startTime, endTime, userIds, businessType);
        return customerSales;
    }


    @Override
    public List<MyMessageEntity> selectByUserId(String zhId, String userId, String type, Integer startNo, Integer endNo) {
        //根据userId获取角色通过不同的角色来获取模板
        UserInfo userInfo = enduserService.getUser(userId);
        String roleIds = userInfo.getRoleIds();
        String contentData = nxOrderMapper.getContentByMbId(TemplateConstant.TEMPLATE_ORDRE);
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD) || roleIds.contains(RoleIdConstant.FWJL) || roleIds.contains(RoleIdConstant.GCS)) {
            contentData = nxOrderMapper.getContentByMbId(TemplateConstant.TEMPLATE_AFTER_ORDRE);
        }
        //获取模板
        List<MyMessageEntity> list = new ArrayList<>();
        List<NxOrderEntity> nxOrderEntities = nxOrderMapper.selectByUserId(zhId, userId, type, startNo, endNo);
        if (!nxOrderEntities.isEmpty()) {
            String finalContentData = contentData;
            nxOrderEntities.stream().forEach(item -> {
                String content = finalContentData;
                if (StringUtils.isNotBlank(content)) {
                    content = content.replace("【订单状态】", item.getOrderOrderstatusName());
                    content = content.replace("【用户名】", item.getOrderCustomerName());
                    content = content.replace("【订单编号】", item.getOrderOrdernumber());
                    content = content.replace("【状态】", item.getOrderOrderstatusName());
                }

                MyMessageEntity myMessageEntity = new MyMessageEntity();
                myMessageEntity.setMessageClass("订单消息");
                myMessageEntity.setMessageObjectCode(item.getOrderOrdernumber());
                myMessageEntity.setMessageObjectName(item.getOrderOrderstatusName());
                myMessageEntity.setMessageContent(content);
                myMessageEntity.setMessageTime(item.getSyCreatetime());
                myMessageEntity.setTypeCode(item.getOrderServicetypeCode());
                myMessageEntity.setAdviceCode(ServicePlanEnums.MessageCodeType.ORDER_MESSAGE.getCode());
                myMessageEntity.setSignData(item.getOrderOrdernumber());
                myMessageEntity.setOrderAssignengineer(item.getOrderAssignengineer());//工程师
                myMessageEntity.setOrderOrdernumber(item.getOrderOrdernumber());//订单编号
                myMessageEntity.setOrderDemanddescription(item.getOrderDemanddescription());//需求描述
                myMessageEntity.setOrderDevicename(item.getOrderDevicename());//设备名称
                myMessageEntity.setOrderDevicecode(item.getOrderDevicecode());//设备编码
                myMessageEntity.setNxOrderId(item.getNxOrderId());//订单id
                //不排除待办库中存在订单
                //List<NxTodoLibrary> nxTodoLibraries = nxTodoLibraryService.selectBySign(myMessageEntity.getAdviceCode(), myMessageEntity.getSignData(), myMessageEntity.getMessageTime());
                //if(nxTodoLibraries.isEmpty()){
                list.add(myMessageEntity);
                //}
            });
        }
        return list;
    }

    @Override
    public List<MyMessageEntity> jsGetByUserId(String userId) {
        List<MyMessageEntity> messageEntityList = new ArrayList<>();
        //获取模板通知
        String contentData = nxOrderMapper.getContentByMbId(TemplateConstant.TEMPLATE_JSHT);
        List<ContractEntity> contractEntities = nxOrderMapper.jsGetByUserId(userId);
        if (!contractEntities.isEmpty()) {
            Log.info("contractEntities ==>{}", JSON.toJSONString(contractEntities));
            String finalContentData = contentData;
            contractEntities.stream().forEach(item -> {
                String content = finalContentData;
                if (StringUtils.isNotBlank(content)) {
                    content = item.getXsjl() == null ? content : content.replace("【销售经理姓名】", item.getXsjl());
                    content = item.getNumberCode() == null ? content : content.replace("【合同编码】", item.getNumberCode());
                    content = item.getFwqkzje() == null ? content : content.replace("【服务欠缴总金额（元）】", item.getFwqkzje());
                    // content = content.replace("【销售经理姓名】",item.getXsjl());
                    //content = content.replace("【合同编码】",item.getNumberCode());
                    //content = content.replace("【服务欠缴总金额（元）】",item.getFwqkzje());
                }
                MyMessageEntity myMessageEntity = new MyMessageEntity();
                myMessageEntity.setId(item.getNxJsxxId());//id
                myMessageEntity.setMesType("1");
                myMessageEntity.setMessageClass("结算消息通知");
                myMessageEntity.setMessageObjectCode(item.getNumberCode());
                myMessageEntity.setMessageObjectName(item.getNumberName());
                myMessageEntity.setMessageContent(content);
                myMessageEntity.setMessageTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
                myMessageEntity.setTypeCode(ServicePlanEnums.MessageType.CONTRACT_SETTLEMENT.getCode());
                myMessageEntity.setAdviceCode(ServicePlanEnums.MessageCodeType.CONTRACTS_SETTLEMENTS.getCode());
                myMessageEntity.setSignData(item.getNumberCode());
                List<NxTodoLibrary> nxTodoLibraries = nxTodoLibraryService.selectBySign(myMessageEntity.getAdviceCode(), myMessageEntity.getSignData(), myMessageEntity.getMessageTime());
                if (nxTodoLibraries.isEmpty()) {
                    messageEntityList.add(myMessageEntity);
                }
            });
        }
        return messageEntityList;
    }

    @Override
    public List<MyMessageEntity> fwGetByUserId(String userId) {
        List<MyMessageEntity> messageEntityList = new ArrayList<>();
        String contentData = nxOrderMapper.getContentByMbId(TemplateConstant.TEMPLATE_FWHT);
        List<ContractEntity> contractEntities = nxOrderMapper.fwGetByUserId(userId);
        if (!contractEntities.isEmpty()) {
            String finalContentData = contentData;
            Log.info("contractEntities ==>{}", JSON.toJSONString(contractEntities));
            contractEntities.stream().forEach(item -> {
                String content = finalContentData;
                if (StringUtils.isNotBlank(content)) {
                    content = item.getXsjl() == null ? content : content.replace("【销售经理姓名】", item.getXsjl());
                    content = item.getNumberCode() == null ? content : content.replace("【合同编码】", item.getNumberCode());
                    //content = content.replace("【销售经理姓名】",item.getXsjl());
                    //content = content.replace("【合同编码】",item.getNumberCode());
                }
                MyMessageEntity myMessageEntity = new MyMessageEntity();
                myMessageEntity.setMesType("0");
                myMessageEntity.setId(item.getNxJsxxId());//id
                myMessageEntity.setMessageClass("服务合同通知");
                myMessageEntity.setMessageObjectCode(item.getNumberCode());
                myMessageEntity.setMessageObjectName(item.getNumberName());
                myMessageEntity.setMessageContent(content);
                myMessageEntity.setMessageTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
                myMessageEntity.setTypeCode(ServicePlanEnums.MessageType.CONTRACT_SETTLEMENT.getCode());
                myMessageEntity.setAdviceCode(ServicePlanEnums.MessageCodeType.CONTRACTS_SETTLEMENTS.getCode());
                myMessageEntity.setSignData(item.getNumberCode());
                List<NxTodoLibrary> nxTodoLibraries = nxTodoLibraryService.selectBySign(myMessageEntity.getAdviceCode(), myMessageEntity.getSignData(), myMessageEntity.getMessageTime());
                if (nxTodoLibraries.isEmpty()) {
                    messageEntityList.add(myMessageEntity);
                }
            });
        }
        return messageEntityList;
    }

    @Override
    public void orderInsertLibrary(String orderId) {
        //根据userId获取角色通过不同的角色来获取模板
        UserInfo userInfo = enduserService.getUser(UserContext.getUserId());
        String roleIds = userInfo.getRoleIds();
        String contentData = nxOrderMapper.getContentByMbId(TemplateConstant.TEMPLATE_ORDRE);
        if (roleIds.contains(RoleIdConstant.GLY) || roleIds.contains(RoleIdConstant.SHLD) || roleIds.contains(RoleIdConstant.FWJL) || roleIds.contains(RoleIdConstant.GCS)) {
            contentData = nxOrderMapper.getContentByMbId(TemplateConstant.TEMPLATE_AFTER_ORDRE);
        }
        NxOrderEntity orderEntity = nxOrderMapper.selectById(orderId);
        String newContent = "";
        if (StringUtils.isNotBlank(contentData)) {
//            contentData = contentData.replace("【订单状态】",orderEntity.getOrderOrderstatusName());
//            contentData = contentData.replace("【用户名】",orderEntity.getOrderCustomerName());
//            contentData = contentData.replace("【订单编码】",orderEntity.getOrderOrdernumber());
//            contentData = contentData.replace("【状态】",orderEntity.getOrderOrderstatusName());
            newContent = contentData.replace("【订单状态】", orderEntity.getOrderOrderstatusName());
        }
        NxTodoLibrary library = new NxTodoLibrary();
        library.setTitle("维修单消息");
        library.setContent(newContent);
        library.setDeviceName(orderEntity.getOrderDevicename());
        library.setDeviceCode(orderEntity.getOrderDevicecode());
        library.setNxOrderId(orderEntity.getNxOrderId());
        //此处原逻辑未存在相关数据
//        library.setOperation(myMessageEntity.getMessageName());
        library.setTime(orderEntity.getSyCreatetime());
        library.setSyCreatetime(com.rootcloud.common.DateUtil.format(new Date()));
        library.setNxTodoLibraryId(UUID.randomUUID().toString().substring(0, 10));
        library.setAdviceCode(ServicePlanEnums.MessageCodeType.REPAIR_ORDER_MESSAGE.getCode());
        library.setTypeCode(orderEntity.getOrderServicetypeCode());
        library.setSignData(orderEntity.getOrderOrdernumber());
        library.setStatus("0");
        library.setUserid(UserContext.getUserId());
        library.setXqms(orderEntity.getOrderDemanddescription());//需求描述
        nxTodoLibraryService.save(library);
    }

    @Override
    public void mesSfjj(String orderId) {
        nxOrderMapper.mesSfjj(orderId);
    }

    @Override
    public void htSerMesSfjj(String id) {
        nxOrderMapper.htSerMesSfjj(id);
    }

    @Override
    public void jsSerMesSfjj(String id) {
        nxOrderMapper.jsSerMesSfjj(id);
    }

    @Autowired
    private UserRoleDao userRoleDao;

    @Override
    public HashMap getOrderTypeNum() {

        String userId = UserContext.getUserId();
        /*
        String userTenantId = UserContext.getUserTenantId();
//        String userId = "8gjvg0ngCHRLNlAE9xs"; //mJZM140P7O36xoVgdCm
//        String userTenantId ="624e7ce1549c110050c5da47";
        UserRoleVO v1 = new UserRoleVO();
        v1.setUserId(userId);
        v1.setTenantId(userTenantId);
        List<UserRoleVO> rolesList = userRoleDao.getUseListByRoleInfo(v1); //查找客户领导、客户角色
        String userName ="";
        List<String> userNameList = new ArrayList<String>();
        List<String> deviceCodeList = new ArrayList<>();
        for (UserRoleVO roleinfo : rolesList)
        {
            userName= roleinfo.getUserName();
        }
        if(!userName.equals("")){
            Map<String,Object> arm = new HashMap<>();
            arm.put("userId", userId);
            List<KhVO> kuList = kuMapper.getKuList(userId);
            for (KhVO info : kuList){
                userNameList.add(info.getUserName());
            }
            List<DeviceDataVo> deviceItemList = nxDeviceService.getKuDeviceList(userId);
            for (DeviceDataVo info : deviceItemList){
                deviceCodeList.add(info.getSblbSbbm());
            }
            Log.error("客户领导、客户角色,APP首页查询当前用户姓名:" +userName+"|userId:"+userId+"|userTenantId:"+userTenantId+"|informantList:"+userNameList.toString()
            +"|deviceCodeList:"+deviceCodeList.toString());
            Map<String,Object> query = new HashMap<>();
            query.put("userId", userId);
            query.put("informantList", userNameList);
            query.put("zhid", userTenantId);
            query.put("deviceCodeList",deviceCodeList);
            List<NxOrderEntity> orderList = nxOrderMapper.getOrderTypeNumByCustomer(query);
            List<NxOrderEntity> dpgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dqdOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_CHECK_IN.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dbgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REPORTING.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dshOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REVIEW.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dpjOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.TO_BE_RETURNED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> ywcOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.DONE.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            HashMap<String, Integer> res = new HashMap<>();
            res.put("dpg", dpgOrderList.size());
            res.put("dqd", dqdOrderList.size());
            res.put("dbg", dbgOrderList.size());
            res.put("dsh", dshOrderList.size());
            res.put("dpj", dpjOrderList.size());
            res.put("ywc", ywcOrderList.size());
            return res;
        }
        else
        {
            Log.error("非客户角色,APP首页查询当前用户userId:"+userId);
            List<NxOrderEntity> orderList = nxOrderMapper.getOrderTypeNum(userId);
            List<NxOrderEntity> dpgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dqdOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_CHECK_IN.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dbgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REPORTING.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dshOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REVIEW.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dpjOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.TO_BE_RETURNED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> ywcOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.DONE.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            HashMap<String, Integer> res = new HashMap<>();
            res.put("dpg", dpgOrderList.size());
            res.put("dqd", dqdOrderList.size());
            res.put("dbg", dbgOrderList.size());
            res.put("dsh", dshOrderList.size());
            res.put("dpj", dpjOrderList.size());
            res.put("ywc", ywcOrderList.size());
            return res;
        }
        */

        List<NxOrderEntity> orderList = nxOrderMapper.getOrderTypeNum(userId);
        List<NxOrderEntity> dpgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> dqdOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_CHECK_IN.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> dbgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REPORTING.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> dshOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REVIEW.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> dpjOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.TO_BE_RETURNED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> ywcOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.DONE.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        HashMap<String, Integer> res = new HashMap<>();
        res.put("dpg", dpgOrderList.size());
        res.put("dqd", dqdOrderList.size());
        res.put("dbg", dbgOrderList.size());
        res.put("dsh", dshOrderList.size());
        res.put("dpj", dpjOrderList.size());
        res.put("ywc", ywcOrderList.size());
        return res;
    }

    @Override
    public HashMap getOrderTypeNum2() {

        String userId = UserContext.getUserId();
        /*
        String userTenantId = UserContext.getUserTenantId();
//        String userId = "8gjvg0ngCHRLNlAE9xs"; //mJZM140P7O36xoVgdCm
//        String userTenantId ="624e7ce1549c110050c5da47";
        UserRoleVO v1 = new UserRoleVO();
        v1.setUserId(userId);
        v1.setTenantId(userTenantId);
        List<UserRoleVO> rolesList = userRoleDao.getUseListByRoleInfo(v1); //查找客户领导、客户角色
        String userName ="";
        List<String> userNameList = new ArrayList<String>();
        List<String> deviceCodeList = new ArrayList<>();
        for (UserRoleVO roleinfo : rolesList)
        {
            userName= roleinfo.getUserName();
        }
        if(!userName.equals("")){
            Map<String,Object> arm = new HashMap<>();
            arm.put("userId", userId);
            List<KhVO> kuList = kuMapper.getKuList(userId);
            for (KhVO info : kuList){
                userNameList.add(info.getUserName());
            }
            List<DeviceDataVo> deviceItemList = nxDeviceService.getKuDeviceList(userId);
            for (DeviceDataVo info : deviceItemList){
                deviceCodeList.add(info.getSblbSbbm());
            }
            Log.error("客户领导、客户角色,APP首页查询当前用户姓名:" +userName+"|userId:"+userId+"|userTenantId:"+userTenantId+"|informantList:"+userNameList.toString()
            +"|deviceCodeList:"+deviceCodeList.toString());
            Map<String,Object> query = new HashMap<>();
            query.put("userId", userId);
            query.put("informantList", userNameList);
            query.put("zhid", userTenantId);
            query.put("deviceCodeList",deviceCodeList);
            List<NxOrderEntity> orderList = nxOrderMapper.getOrderTypeNumByCustomer(query);
            List<NxOrderEntity> dpgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dqdOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_CHECK_IN.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dbgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REPORTING.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dshOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REVIEW.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dpjOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.TO_BE_RETURNED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> ywcOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.DONE.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            HashMap<String, Integer> res = new HashMap<>();
            res.put("dpg", dpgOrderList.size());
            res.put("dqd", dqdOrderList.size());
            res.put("dbg", dbgOrderList.size());
            res.put("dsh", dshOrderList.size());
            res.put("dpj", dpjOrderList.size());
            res.put("ywc", ywcOrderList.size());
            return res;
        }
        else
        {
            Log.error("非客户角色,APP首页查询当前用户userId:"+userId);
            List<NxOrderEntity> orderList = nxOrderMapper.getOrderTypeNum(userId);
            List<NxOrderEntity> dpgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dqdOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_CHECK_IN.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dbgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REPORTING.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dshOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REVIEW.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> dpjOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.TO_BE_RETURNED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            List<NxOrderEntity> ywcOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.DONE.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
            HashMap<String, Integer> res = new HashMap<>();
            res.put("dpg", dpgOrderList.size());
            res.put("dqd", dqdOrderList.size());
            res.put("dbg", dbgOrderList.size());
            res.put("dsh", dshOrderList.size());
            res.put("dpj", dpjOrderList.size());
            res.put("ywc", ywcOrderList.size());
            return res;
        }
        */

        List<NxOrderEntity> orderList = nxOrderMapper.getOrderTypeNum2(userId);
        List<NxOrderEntity> dpgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> dqdOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_CHECK_IN.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> dbgOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REPORTING.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> dshOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.PENDING_REVIEW.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> dpjOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.TO_BE_RETURNED.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        List<NxOrderEntity> ywcOrderList = orderList.stream().filter(e -> Objects.equals(OrderEnums.OrderStatus.DONE.getCode(), e.getOrderOrderstatusCode())).collect(Collectors.toList());
        HashMap<String, Integer> res = new HashMap<>();
        res.put("dpg", dpgOrderList.size());
        res.put("dqd", dqdOrderList.size());
        res.put("dbg", dbgOrderList.size());
        res.put("dsh", dshOrderList.size());
        res.put("dpj", dpjOrderList.size());
        res.put("ywc", ywcOrderList.size());
        return res;
    }

    /**
     * 修改订单审核时间，完工时间，指派工程师时间
     */
    @Override
    public void updateDate() {
        nxOrderMapper.updateOrderShsj();
        nxOrderMapper.updateOrderWgsj();
        nxOrderMapper.updateOrderZpgcssj();
    }

    /**
     * 创建订单
     *
     * @param param
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult<Object> createOrder(OrderParam param) throws Exception {
        String deviceIds = param.getDeviceIds();
        if (StringUtils.isBlank(deviceIds)) {
            return JsonResult.failed("450", "获取设备ID失败");
        }
        //查询设备信息，多个设备拼接
        DeviceDataVo deviceDataVo = nxDeviceService.getJcxxByIds(deviceIds);
        if (deviceDataVo == null) {
            return JsonResult.failed("450", "获取设备信息失败");
        }
        //获取服务网点
        //判断设备是否存在
        NxDeviceEntity nxDeviceEntity = nxDeviceService.getById(Arrays.asList(deviceIds.split(",")).get(0));
        if (nxDeviceEntity == null) {
            return JsonResult.failed("450", "获取设备信息失败");
        }
        //获取服务网点id
        String fwwdId = null;
        String khid = nxDeviceEntity.getSblbKhid();
        if (StringUtils.isNotBlank(khid)) {
            fwwdId = nxDeviceService.selectFwwdId(khid, nxDeviceEntity.getSyZhid());
        }
        if (StringUtils.isBlank(fwwdId)) {
            return JsonResult.failed("450", "获取服务网点失败");
        }
        //校验账号密码
        OAuthResult login = rootCloudAuthService.login(param.getLoginName(), param.getLoginPassword(), null);
        if (login == null) {
            return JsonResult.failed("450", "获取登录信息失败");
        }
        //获取用户信息
        LambdaQueryWrapper<JeCoreEnduserEntity> mpeWrapper = new LambdaQueryWrapper<JeCoreEnduserEntity>();
        mpeWrapper.eq(JeCoreEnduserEntity::getBackusercode, param.getLoginName());
        mpeWrapper.last("limit 1");
        JeCoreEnduserEntity info = enduserService.getOne(mpeWrapper);
        //创建订单实体类
        NxOrderEntity order = new NxOrderEntity();
        String orderId = JEUuidUtil.uuid();
        order.setNxOrderId(orderId);
        StringBuffer orderNum = new StringBuffer();
        String currentDate = DateUtils.format(new Date(), DateUtils.C_DATA_PATTON_YYYYMMDD);
        //根据服务类型创建订单编号
        if ("001".equals(param.getOrderServicetypeCode())) {
            orderNum.append("R");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.REPAIR.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.REPAIR.getDesc());
        } else if ("002".equals(param.getOrderServicetypeCode())) {
            orderNum.append("B");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.MAINTENANCE.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.MAINTENANCE.getDesc());
        } else if ("003".equals(param.getOrderServicetypeCode())) {
            orderNum.append("H");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.HANDOVER.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.HANDOVER.getDesc());
        } else if ("004".equals(param.getOrderServicetypeCode())) {
            orderNum.append("T");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.TRANSFORMATION.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.TRANSFORMATION.getDesc());
        }
        orderNum.append(orderNumberUtils.generateOrderNumber());
        order.setOrderOrdernumber(orderNum.toString());
        order.setOrderSource("app");
        order.setOrderDevicecode(deviceDataVo.getSblbSbbm());
        order.setOrderDevicename(deviceDataVo.getSblbSbmc());
        order.setOrderDevicetype(deviceDataVo.getSblbSblxName());
        order.setOrderDevicemodel(deviceDataVo.getSblbSbxhName());
        order.setOrderServiceaddress(deviceDataVo.getSblbSbxxdz());
        order.setOrderServiceaddressLongitude(deviceDataVo.getSblbJd());
        order.setOrderServiceaddressDimension(deviceDataVo.getSblbWd());
        order.setOrderCustomerName(deviceDataVo.getSblbKhmc());
        order.setOrderKhlxr(deviceDataVo.getSblbKhlxr());
        order.setOrderContactway(deviceDataVo.getSblbLxfs());
        order.setOrderCustomerId(deviceDataVo.getSblbKhid());
        order.setOrderOrderstatusCode(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode());
        order.setOrderOrderstatusName(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getDesc());
        order.setSyCreatetime(DateUtils.format(new Date(), DateUtils.C_TIME_PATTON_DEFAULT));
        order.setSyStatus("1");
        order.setOrderApplydate(currentDate);
        List<OrderLcinfoParam> list = new ArrayList<>();
        OrderLcinfoParam orderLcinfoParam = new OrderLcinfoParam();
        orderLcinfoParam.setId(1);
        orderLcinfoParam.setName("已受理");
        orderLcinfoParam.setUserName(info.getUsername());
        orderLcinfoParam.setUserId(info.getUserid());
        orderLcinfoParam.setIcon("sl_icon");
        orderLcinfoParam.setEndDate(String.valueOf(com.rootcloud.common.DateUtil.getTimesByString(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))));
        list.add(orderLcinfoParam);
        order.setOrderLcinfo(JSON.toJSONString(list).toString());
        order.setSyZhid(info.getZhid());//设置租户ID
        /*
        order.setOrderWbjsrq(nxDeviceEntity.getSblbWbjsrq());
        if (StringUtils.isNotBlank(nxDeviceEntity.getSblbWbjsrq())) {
            if (new SimpleDateFormat("yyyy-MM-dd").format(new Date()).compareTo(nxDeviceEntity.getSblbWbjsrq()) <= 0) {
                order.setOrderWarrantiesCode(ServicePlanEnums.CheckType.CHECK_TURE.getCode());
                order.setOrderWarrantiesName(ServicePlanEnums.CheckType.CHECK_TURE.getDesc());
            } else {
                order.setOrderWarrantiesCode(ServicePlanEnums.CheckType.CHECK_FALSE.getCode());
                order.setOrderWarrantiesName(ServicePlanEnums.CheckType.CHECK_FALSE.getDesc());
            }
            Log.info("Wbjsrq:"+nxDeviceEntity.getSblbWbjsrq()+">>"+"WarrantiesCode:"+ServicePlanEnums.CheckType.CHECK_TURE.getCode()+">>"+
                    "WarrantiesName:"+ServicePlanEnums.CheckType.CHECK_TURE.getDesc());
        }
        */
        nxOrderMapper.insert(order);
        //流程引擎
        if (StringUtils.isNotBlank(order.getOrderFwwdid())) {
            //调用登录接口
            /*String loginName = servicePlanMapper.queryStringByCode("loginName") == null ? "app_master_admin" : servicePlanMapper.queryStringByCode("loginName");
            String loginPassword = servicePlanMapper.queryStringByCode("loginPassword") == null ? "Liu@123456" : servicePlanMapper.queryStringByCode("loginPassword");
            OAuthResult login = rootCloudAuthService.login(loginName, loginPassword, null);*/
            Log.info("OAuthResult:" + login + "loginName:" + param.getLoginName() + " loginPassword:" + param.getLoginPassword());
            OutletsVO outletsVO = servicePlanMapper.returnUserNames(order.getOrderFwwdid());
            String sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/sponsorProcess";
            Map<String, String> map = new HashMap();
            map.put("pdid", "frx-ningxia-dingdan_22051015-7");
            map.put("beanId", orderId);
            map.put("currentTaskName", "");
            map.put("submitComments", "订单待派工，请尽快处理");
            map.put("targerTaskName", "是否派工");
            map.put("targerTransition", "");
            map.put("assigeeName", outletsVO.getUserNames());
            map.put("assigeeId", outletsVO.getUserIds());
            map.put("submitType", "to_submit");
            Map<String, String> headers = new HashMap<>();
             headers.put("authorization", login.getAccess_token());
//             Log.info("authorization"+login.getAccess_token());
//            headers.put("authorization", "Hpq9v3R1hoCTVTaVD0p");
            headers.put("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
            Log.info(">>>>到这里了1:");
            try {
                Map<String, String> querys = new HashMap<>();
                org.apache.http.HttpResponse post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                int statusCode = post.getStatusLine().getStatusCode();
                String responseContent = EntityUtils.toString(post.getEntity(), "UTF-8");
                Log.info("服务计划派工订单待派工状态码:" + statusCode + " 返回值：" + responseContent);
                sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/completeTask";
                map.clear();
                Log.info(">>>>到这里了2:");
                JSONObject res = JSONObject.parseObject(responseContent);
                Object obj = res.get("obj");
                if (obj != null) {
                    Log.info(">>>>到这里了3:");
                    String taskId = ((JSONObject) obj).get("taskId").toString();
                    map.put("taskId", taskId);
                    map.put("pdid", "frx-ningxia-dingdan_22051015-7");
                    map.put("beanId", orderId);
                    map.put("submitComments", "同意");
                    map.put("currentTaskName", "是否派工");
                    map.put("targerTaskName", "待派工");
                    map.put("targerTransition", "to_decision2");
                    map.put("assigeeName", outletsVO.getUserNames());
                    map.put("assigeeId", outletsVO.getUserIds());
                    map.put("submitType", "to_submit");
                    querys.clear();
                    Log.info(">>>>到这里了4:");
                    post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                    statusCode = post.getStatusLine().getStatusCode();
                    Log.info(">>>>到这里了5:"+statusCode);
                    Log.info("服务计划派同意派工状态码:" + statusCode);
                } else {
                    Log.error("服务计划派同意派工taskId获取不到");
                    throw new Exception();
                }
            } catch (Exception e) {
                Log.error("流程引擎错误" + e.getMessage());
                e.printStackTrace();
                throw new Exception();    //事务回滚
            }
        }
        return JsonResult.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult<Object> createOrder2(OrderParam param) throws Exception {
        String deviceIds = param.getDeviceIds();
        logger.info("deviceIds:"+deviceIds+"-----------------");
        Log.info("deviceIds:"+deviceIds+"-----------------");
        if (StringUtils.isBlank(deviceIds)) {
            return JsonResult.failed("450", "获取设备ID失败");
        }
        //查询设备信息，多个设备拼接
        DeviceDataVo deviceDataVo = nxDeviceService.getJcxxByIds(deviceIds);
        logger.info("deviceDataVo:"+deviceDataVo+"-----------------");
        Log.info("deviceDataVo:"+deviceDataVo+"-----------------");
        if (deviceDataVo == null) {
            return JsonResult.failed("450", "获取设备信息失败");
        }
        //获取服务网点
        //判断设备是否存在
        NxDeviceEntity nxDeviceEntity = nxDeviceService.getById(Arrays.asList(deviceIds.split(",")).get(0));
        logger.info("nxDeviceEntity:"+nxDeviceEntity+"-----------------");
        logger.info("SblbKhid:"+nxDeviceEntity.getSblbKhid()+"-----------------");
        logger.info("SblbWbjsrq:"+nxDeviceEntity.getSblbWbjsrq()+"-----------------");
        if (nxDeviceEntity == null) {
            return JsonResult.failed("450", "获取设备信息失败");
        }
        //获取服务网点id
        String fwwdId = null;
        String khid = nxDeviceEntity.getSblbKhid();
        if (StringUtils.isNotBlank(khid)) {
            fwwdId = nxDeviceService.selectFwwdId(khid, nxDeviceEntity.getSyZhid());
        }
        logger.info("服务网点:"+fwwdId+"-----------------");
        Log.info("服务网点:"+fwwdId+"-----------------");
        if (StringUtils.isBlank(fwwdId)) {
            return JsonResult.failed("450", "获取服务网点失败");
        }
        //校验账号密码
        OAuthResult login = rootCloudAuthService.login(param.getLoginName(), param.getLoginPassword(), null);
        if (login == null) {
            return JsonResult.failed("450", "获取登录信息失败");
        }
        //获取用户信息
        LambdaQueryWrapper<JeCoreEnduserEntity> mpeWrapper = new LambdaQueryWrapper<JeCoreEnduserEntity>();
        mpeWrapper.eq(JeCoreEnduserEntity::getBackusercode, param.getLoginName());
        mpeWrapper.last("limit 1");
        JeCoreEnduserEntity info = enduserService.getOne(mpeWrapper);
        //创建订单实体类
        NxOrderEntity order = new NxOrderEntity();
        String orderId = JEUuidUtil.uuid();
        order.setNxOrderId(orderId);
        StringBuffer orderNum = new StringBuffer();
        String currentDate = DateUtils.format(new Date(), DateUtils.C_DATA_PATTON_YYYYMMDD);
        //根据服务类型创建订单编号
        if ("001".equals(param.getOrderServicetypeCode())) {
            orderNum.append("R");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.REPAIR.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.REPAIR.getDesc());
        } else if ("002".equals(param.getOrderServicetypeCode())) {
            orderNum.append("B");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.MAINTENANCE.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.MAINTENANCE.getDesc());
        } else if ("003".equals(param.getOrderServicetypeCode())) {
            orderNum.append("H");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.HANDOVER.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.HANDOVER.getDesc());
        } else if ("004".equals(param.getOrderServicetypeCode())) {
            orderNum.append("T");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.TRANSFORMATION.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.TRANSFORMATION.getDesc());
        }else if ("005".equals(param.getOrderServicetypeCode())) {
            orderNum.append("I");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.INSPECTION.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.INSPECTION.getDesc());
        }
        orderNum.append(orderNumberUtils.generateOrderNumber());
        order.setOrderOrdernumber(orderNum.toString());
        order.setOrderSource("app");
        order.setOrderDevicecode(deviceDataVo.getSblbSbbm());
        order.setOrderDevicename(deviceDataVo.getSblbSbmc());
        order.setOrderDevicetype(deviceDataVo.getSblbSblxName());
        order.setOrderDevicemodel(deviceDataVo.getSblbSbxhName());
        order.setOrderServiceaddress(deviceDataVo.getSblbSbxxdz());
        order.setOrderServiceaddressLongitude(deviceDataVo.getSblbJd());
        order.setOrderServiceaddressDimension(deviceDataVo.getSblbWd());
        order.setOrderCustomerName(deviceDataVo.getSblbKhmc());
        order.setOrderKhlxr(deviceDataVo.getSblbKhlxr());
        order.setOrderContactway(deviceDataVo.getSblbLxfs());
        order.setOrderCustomerId(deviceDataVo.getSblbKhid());
        order.setOrderOrderstatusCode(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode());
        order.setOrderOrderstatusName(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getDesc());
        order.setSyCreatetime(DateUtils.format(new Date(), DateUtils.C_TIME_PATTON_DEFAULT));
        order.setSyStatus("1");
        order.setOrderApplydate(currentDate);
        List<OrderLcinfoParam> list = new ArrayList<>();
        OrderLcinfoParam orderLcinfoParam = new OrderLcinfoParam();
        orderLcinfoParam.setId(1);
        orderLcinfoParam.setName("已受理");
        orderLcinfoParam.setUserName(info.getUsername());
        orderLcinfoParam.setUserId(info.getUserid());
        orderLcinfoParam.setIcon("sl_icon");
        orderLcinfoParam.setEndDate(String.valueOf(com.rootcloud.common.DateUtil.getTimesByString(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))));
        list.add(orderLcinfoParam);
        order.setOrderLcinfo(JSON.toJSONString(list).toString());
        order.setSyZhid(info.getZhid());//设置租户ID

        logger.info("服务需求:"+param.getOrderDemanddescription()+"-----------------");
        Log.info("服务需求:"+param.getOrderDemanddescription()+"-----------------");
        order.setOrderDemanddescription(param.getOrderDemanddescription()); //需求描述
        order.setOrderFwwdid(fwwdId); //服务网点ID
        //order.setSyCurrenttask("待派工");
        logger.info("fwwdId:"+fwwdId+">>"+"Demanddescription:"+param.getOrderDemanddescription()+">>");
        Log.info("fwwdId:"+fwwdId+">>"+"Demanddescription:"+param.getOrderDemanddescription()+">>");
        if (nxDeviceEntity != null)
        {
            logger.info("Wbjsrq:"+nxDeviceEntity.getSblbWbjsrq()+">>"+"WarrantiesCode:"+ServicePlanEnums.CheckType.CHECK_TURE.getCode()+">>"+
                    "WarrantiesName:"+ServicePlanEnums.CheckType.CHECK_TURE.getDesc());
            order.setOrderWbjsrq(nxDeviceEntity.getSblbWbjsrq());
            if (StringUtils.isNotBlank(nxDeviceEntity.getSblbWbjsrq())) {
                if (new SimpleDateFormat("yyyy-MM-dd").format(new Date()).compareTo(nxDeviceEntity.getSblbWbjsrq()) <= 0) {
                    order.setOrderWarrantiesCode(ServicePlanEnums.CheckType.CHECK_TURE.getCode());
                    order.setOrderWarrantiesName(ServicePlanEnums.CheckType.CHECK_TURE.getDesc());
                } else {
                    order.setOrderWarrantiesCode(ServicePlanEnums.CheckType.CHECK_FALSE.getCode());
                    order.setOrderWarrantiesName(ServicePlanEnums.CheckType.CHECK_FALSE.getDesc());
                }
            }
        }
        nxOrderMapper.insert(order);
        logger.info("order:"+order);
        logger.info("nxOrderMapper.insert 执行完成-----------------");
        //流程引擎
        if (StringUtils.isNotBlank(order.getOrderFwwdid())) {
            //调用登录接口
            /*String loginName = servicePlanMapper.queryStringByCode("loginName") == null ? "app_master_admin" : servicePlanMapper.queryStringByCode("loginName");
            String loginPassword = servicePlanMapper.queryStringByCode("loginPassword") == null ? "Liu@123456" : servicePlanMapper.queryStringByCode("loginPassword");
            OAuthResult login = rootCloudAuthService.login(loginName, loginPassword, null);*/
            logger.info("OAuthResult:" + login + "loginName:" + param.getLoginName() + " loginPassword:" + param.getLoginPassword());
            OutletsVO outletsVO = servicePlanMapper.returnUserNames(order.getOrderFwwdid());
            String sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/sponsorProcess";
            Map<String, String> map = new HashMap();
            map.put("pdid", "frx-ningxia-dingdan_22051015-7");
            map.put("beanId", orderId);
            map.put("currentTaskName", "");
            map.put("submitComments", "订单待派工，请尽快处理");
            map.put("targerTaskName", "是否派工");
            map.put("targerTransition", "");
            map.put("assigeeName", outletsVO.getUserNames());
            map.put("assigeeId", outletsVO.getUserIds());
            map.put("submitType", "to_submit");
            Map<String, String> headers = new HashMap<>();
             headers.put("authorization", login.getAccess_token());
            // Log.info("authorization"+login.getAccess_token());
//            headers.put("authorization", "Hpq9v3R1hoCTVTaVD0p");
            headers.put("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
            logger.info(">>>>到这里了1:");
            try {
                Map<String, String> querys = new HashMap<>();
                org.apache.http.HttpResponse post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                int statusCode = post.getStatusLine().getStatusCode();
                String responseContent = EntityUtils.toString(post.getEntity(), "UTF-8");
                logger.info("服务计划派工订单待派工状态码:" + statusCode + " 返回值：" + responseContent);
                sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/completeTask";
                map.clear();
                logger.info(">>>>到这里了2:");
                JSONObject res = JSONObject.parseObject(responseContent);
                Object obj = res.get("obj");
                if (obj != null) {
                    logger.info(">>>>到这里了3:");
                    String taskId = ((JSONObject) obj).get("taskId").toString();
                    map.put("taskId", taskId);
                    map.put("pdid", "frx-ningxia-dingdan_22051015-7");
                    map.put("beanId", orderId);
                    map.put("submitComments", "同意");
                    map.put("currentTaskName", "是否派工");
                    map.put("targerTaskName", "待派工");
                    map.put("targerTransition", "to_decision2");
                    map.put("assigeeName", outletsVO.getUserNames());
                    map.put("assigeeId", outletsVO.getUserIds());
                    map.put("submitType", "to_submit");
                    querys.clear();
                    logger.info(">>>>到这里了4:");
                    post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                    statusCode = post.getStatusLine().getStatusCode();
                    logger.info(">>>>到这里了5:"+statusCode);
                    logger.info("服务计划派同意派工状态码:" + statusCode);
                } else {
                    logger.error("服务计划派同意派工taskId获取不到");
                    throw new Exception();
                }
            } catch (Exception e) {
                logger.error("流程引擎错误" + e.getMessage());
                e.printStackTrace();
                throw new Exception();    //事务回滚
            }
        }
        return JsonResult.success();
    }

    @Resource
    OrderNumberUtils orderNumberUtils;
    @Override
    public JsonResult<Object> createOrder3(OrderParam param) throws Exception {
        String deviceIds = param.getDeviceIds();
        logger.info("deviceIds:"+deviceIds+"-----------------");
        Log.info("deviceIds:"+deviceIds+"-----------------");
        if (StringUtils.isBlank(deviceIds)) {
            return JsonResult.failed("450", "获取设备ID失败");
        }
        //查询设备信息，多个设备拼接
        DeviceDataVo deviceDataVo = nxDeviceService.getJcxxByIds(deviceIds);
        logger.info("deviceDataVo:"+deviceDataVo+"-----------------");
        Log.info("deviceDataVo:"+deviceDataVo+"-----------------");
        if (deviceDataVo == null) {
            return JsonResult.failed("450", "获取设备信息失败");
        }
        //获取服务网点
        //判断设备是否存在
        NxDeviceEntity nxDeviceEntity = nxDeviceService.getById(Arrays.asList(deviceIds.split(",")).get(0));
        logger.info("nxDeviceEntity:"+nxDeviceEntity+"-----------------");
        logger.info("SblbKhid:"+nxDeviceEntity.getSblbKhid()+"-----------------");
        logger.info("SblbWbjsrq:"+nxDeviceEntity.getSblbWbjsrq()+"-----------------");
        if (nxDeviceEntity == null) {
            return JsonResult.failed("450", "获取设备信息失败");
        }
        //获取服务网点id
        String fwwdId = null;
        String khid = nxDeviceEntity.getSblbKhid();
        if (StringUtils.isNotBlank(khid)) {
            fwwdId = nxDeviceService.selectFwwdId(khid, nxDeviceEntity.getSyZhid());
        }
        logger.info("服务网点:"+fwwdId+"-----------------");
        Log.info("服务网点:"+fwwdId+"-----------------");
        if (StringUtils.isBlank(fwwdId)) {
            return JsonResult.failed("450", "获取服务网点失败");
        }
        //校验账号密码
        OAuthResult login = rootCloudAuthService.login(param.getLoginName(), param.getLoginPassword(), null);
        if (login == null) {
            return JsonResult.failed("450", "获取登录信息失败");
        }
        //获取用户信息
        LambdaQueryWrapper<JeCoreEnduserEntity> mpeWrapper = new LambdaQueryWrapper<JeCoreEnduserEntity>();
        mpeWrapper.eq(JeCoreEnduserEntity::getBackusercode, param.getLoginName());
        mpeWrapper.last("limit 1");
        JeCoreEnduserEntity info = enduserService.getOne(mpeWrapper);
        //创建订单实体类
        NxOrderEntity order = new NxOrderEntity();
        String orderId = JEUuidUtil.uuid();
        order.setNxOrderId(orderId);
        StringBuffer orderNum = new StringBuffer();
        String currentDate = DateUtils.format(new Date(), DateUtils.C_DATA_PATTON_YYYYMMDD);
        //根据服务类型创建订单编号
        if ("001".equals(param.getOrderServicetypeCode())) {
            orderNum.append("R");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.REPAIR.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.REPAIR.getDesc());
        } else if ("002".equals(param.getOrderServicetypeCode())) {
            orderNum.append("B");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.MAINTENANCE.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.MAINTENANCE.getDesc());
        } else if ("003".equals(param.getOrderServicetypeCode())) {
            orderNum.append("H");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.HANDOVER.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.HANDOVER.getDesc());
        } else if ("004".equals(param.getOrderServicetypeCode())) {
            orderNum.append("T");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.TRANSFORMATION.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.TRANSFORMATION.getDesc());
        }else if ("005".equals(param.getOrderServicetypeCode())) {
            orderNum.append("I");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.INSPECTION.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.INSPECTION.getDesc());
        }
        orderNum.append(orderNumberUtils.generateOrderNumber());
        order.setOrderOrdernumber(orderNum.toString());
        order.setOrderSource("app");
        order.setOrderDevicecode(deviceDataVo.getSblbSbbm());
        order.setOrderDevicename(deviceDataVo.getSblbSbmc());
        order.setOrderDevicetype(deviceDataVo.getSblbSblxName());
        order.setOrderDevicemodel(deviceDataVo.getSblbSbxhName());
        order.setOrderServiceaddress(deviceDataVo.getSblbSbxxdz());
        order.setOrderServiceaddressLongitude(deviceDataVo.getSblbJd());
        order.setOrderServiceaddressDimension(deviceDataVo.getSblbWd());
        order.setOrderCustomerName(deviceDataVo.getSblbKhmc());
        order.setOrderKhlxr(deviceDataVo.getSblbKhlxr());
        order.setOrderContactway(deviceDataVo.getSblbLxfs());
        order.setOrderCustomerId(deviceDataVo.getSblbKhid());
        order.setOrderOrderstatusCode(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode());
        order.setOrderOrderstatusName(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getDesc());
        order.setSyCreatetime(DateUtils.format(new Date(), DateUtils.C_TIME_PATTON_DEFAULT));
        order.setSyStatus("1");
        order.setOrderApplydate(currentDate);
        List<OrderLcinfoParam> list = new ArrayList<>();
        OrderLcinfoParam orderLcinfoParam = new OrderLcinfoParam();
        orderLcinfoParam.setId(1);
        orderLcinfoParam.setName("已受理");
        orderLcinfoParam.setUserName(info.getUsername());
        orderLcinfoParam.setUserId(info.getUserid());
        orderLcinfoParam.setIcon("sl_icon");
        orderLcinfoParam.setEndDate(String.valueOf(com.rootcloud.common.DateUtil.getTimesByString(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))));
        list.add(orderLcinfoParam);
        order.setOrderLcinfo(JSON.toJSONString(list).toString());
        order.setSyZhid(info.getZhid());//设置租户ID

        logger.info("服务需求:"+param.getOrderDemanddescription()+"-----------------");
        Log.info("服务需求:"+param.getOrderDemanddescription()+"-----------------");
        order.setOrderDemanddescription(param.getOrderDemanddescription()); //需求描述
        order.setOrderFwwdid(fwwdId); //服务网点ID
        //order.setSyCurrenttask("待派工");
        logger.info("fwwdId:"+fwwdId+">>"+"Demanddescription:"+param.getOrderDemanddescription()+">>");
        Log.info("fwwdId:"+fwwdId+">>"+"Demanddescription:"+param.getOrderDemanddescription()+">>");
        /*
        if (nxDeviceEntity != null)
        {
            logger.info("Wbjsrq:"+nxDeviceEntity.getSblbWbjsrq()+">>"+"WarrantiesCode:"+ServicePlanEnums.CheckType.CHECK_TURE.getCode()+">>"+
                    "WarrantiesName:"+ServicePlanEnums.CheckType.CHECK_TURE.getDesc());
            order.setOrderWbjsrq(nxDeviceEntity.getSblbWbjsrq());
            if (StringUtils.isNotBlank(nxDeviceEntity.getSblbWbjsrq())) {
                if (new SimpleDateFormat("yyyy-MM-dd").format(new Date()).compareTo(nxDeviceEntity.getSblbWbjsrq()) <= 0) {
                    order.setOrderWarrantiesCode(ServicePlanEnums.CheckType.CHECK_TURE.getCode());
                    order.setOrderWarrantiesName(ServicePlanEnums.CheckType.CHECK_TURE.getDesc());
                } else {
                    order.setOrderWarrantiesCode(ServicePlanEnums.CheckType.CHECK_FALSE.getCode());
                    order.setOrderWarrantiesName(ServicePlanEnums.CheckType.CHECK_FALSE.getDesc());
                }
            }
        }
        */
        order.setOrderWbjsrq("2023-10-14");
        order.setOrderWarrantiesCode("1");
        order.setOrderWarrantiesName("是");
        order.setOrderDdms("1");//自动触发
        nxOrderMapper.insert(order);
        logger.info("order:"+order);
        logger.info("nxOrderMapper.insert 执行完成-----------------");
        //流程引擎
        if (StringUtils.isNotBlank(order.getOrderFwwdid())) {
            //调用登录接口
            /*String loginName = servicePlanMapper.queryStringByCode("loginName") == null ? "app_master_admin" : servicePlanMapper.queryStringByCode("loginName");
            String loginPassword = servicePlanMapper.queryStringByCode("loginPassword") == null ? "Liu@123456" : servicePlanMapper.queryStringByCode("loginPassword");
            OAuthResult login = rootCloudAuthService.login(loginName, loginPassword, null);*/
            logger.info("OAuthResult:" + login + "loginName:" + param.getLoginName() + " loginPassword:" + param.getLoginPassword());
            OutletsVO outletsVO = servicePlanMapper.returnUserNames(order.getOrderFwwdid());
            String sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/sponsorProcess";
            Map<String, String> map = new HashMap();
            map.put("pdid", "frx-ningxia-dingdan_22051015-7");
            map.put("beanId", orderId);
            map.put("currentTaskName", "");
            map.put("submitComments", "订单待派工，请尽快处理");
            map.put("targerTaskName", "是否派工");
            map.put("targerTransition", "");
            map.put("assigeeName", outletsVO.getUserNames());
            map.put("assigeeId", outletsVO.getUserIds());
            map.put("submitType", "to_submit");
            Map<String, String> headers = new HashMap<>();
             headers.put("authorization", login.getAccess_token());
            // Log.info("authorization"+login.getAccess_token());
//            headers.put("authorization", "Hpq9v3R1hoCTVTaVD0p");
            headers.put("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
            logger.info(">>>>到这里了1:");
            try {
                Map<String, String> querys = new HashMap<>();
                org.apache.http.HttpResponse post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                int statusCode = post.getStatusLine().getStatusCode();
                String responseContent = EntityUtils.toString(post.getEntity(), "UTF-8");
                logger.info("服务计划派工订单待派工状态码:" + statusCode + " 返回值：" + responseContent);
                sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/completeTask";
                map.clear();
                logger.info(">>>>到这里了2:");
                JSONObject res = JSONObject.parseObject(responseContent);
                Object obj = res.get("obj");
                if (obj != null) {
                    logger.info(">>>>到这里了3:");
                    String taskId = ((JSONObject) obj).get("taskId").toString();
                    map.put("taskId", taskId);
                    map.put("pdid", "frx-ningxia-dingdan_22051015-7");
                    map.put("beanId", orderId);
                    map.put("submitComments", "同意");
                    map.put("currentTaskName", "是否派工");
                    map.put("targerTaskName", "待派工");
                    map.put("targerTransition", "to_decision2");
                    map.put("assigeeName", outletsVO.getUserNames());
                    map.put("assigeeId", outletsVO.getUserIds());
                    map.put("submitType", "to_submit");
                    querys.clear();
                    logger.info(">>>>到这里了4:");
                    post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                    statusCode = post.getStatusLine().getStatusCode();
                    logger.info(">>>>到这里了5:"+statusCode);
                    logger.info("服务计划派同意派工状态码:" + statusCode);
                } else {
                    logger.error("服务计划派同意派工taskId获取不到");
                    throw new Exception();
                }
            } catch (Exception e) {
                logger.error("流程引擎错误" + e.getMessage());
                e.printStackTrace();
                throw new Exception();    //事务回滚
            }
        }
        return JsonResult.success();
    }

    @Override
    public JsonResult<Object> createOrder4(OrderParam param) throws Exception {
        String deviceIds = param.getDeviceIds();
        //logger.info("deviceIds:"+deviceIds+"-----------------");
        Log.info("deviceIds:"+deviceIds+"-----------------");
        if (StringUtils.isBlank(deviceIds)) {
            return JsonResult.failed("450", "获取设备ID失败");
        }
        //查询设备信息，多个设备拼接
        DeviceDataVo deviceDataVo = nxDeviceService.getJcxxByIds(deviceIds);
        //logger.info("deviceDataVo:"+deviceDataVo+"-----------------");
        Log.info("deviceDataVo:"+deviceDataVo+"-----------------");
        if (deviceDataVo == null) {
            return JsonResult.failed("450", "获取设备信息失败");
        }
        //获取服务网点
        //判断设备是否存在
        NxDeviceEntity nxDeviceEntity = nxDeviceService.getById(Arrays.asList(deviceIds.split(",")).get(0));
        //logger.info("nxDeviceEntity:"+nxDeviceEntity+"-----------------");
        //logger.info("SblbKhid:"+nxDeviceEntity.getSblbKhid()+"-----------------");
        //logger.info("SblbWbjsrq:"+nxDeviceEntity.getSblbWbjsrq()+"-----------------");
        if (nxDeviceEntity == null) {
            return JsonResult.failed("450", "获取设备信息失败");
        }
        //获取服务网点id
        String fwwdId = null;
        String khid = nxDeviceEntity.getSblbKhid();
        if (StringUtils.isNotBlank(khid)) {
            fwwdId = nxDeviceService.selectFwwdId(khid, nxDeviceEntity.getSyZhid());
        }
        //logger.info("服务网点:"+fwwdId+"-----------------");
        Log.info("服务网点:"+fwwdId+"-----------------");
        if (StringUtils.isBlank(fwwdId)) {
            return JsonResult.failed("450", "获取服务网点失败");
        }
        //校验账号密码
        OAuthResult login = rootCloudAuthService.login(param.getLoginName(), param.getLoginPassword(), null);
        if (login == null) {
            return JsonResult.failed("450", "获取登录信息失败");
        }
        //获取用户信息
        LambdaQueryWrapper<JeCoreEnduserEntity> mpeWrapper = new LambdaQueryWrapper<JeCoreEnduserEntity>();
        mpeWrapper.eq(JeCoreEnduserEntity::getBackusercode, param.getLoginName());
        mpeWrapper.last("limit 1");
        JeCoreEnduserEntity info = enduserService.getOne(mpeWrapper);
        //创建订单实体类
        NxOrderEntity order = new NxOrderEntity();
        String orderId = JEUuidUtil.uuid();
        order.setNxOrderId(orderId);
        StringBuffer orderNum = new StringBuffer();
        String currentDate = DateUtils.format(new Date(), DateUtils.C_DATA_PATTON_YYYYMMDD);
        //根据服务类型创建订单编号
        if ("001".equals(param.getOrderServicetypeCode())) {
            orderNum.append("R");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.REPAIR.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.REPAIR.getDesc());
        } else if ("002".equals(param.getOrderServicetypeCode())) {
            orderNum.append("B");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.MAINTENANCE.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.MAINTENANCE.getDesc());
        } else if ("003".equals(param.getOrderServicetypeCode())) {
            orderNum.append("H");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.HANDOVER.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.HANDOVER.getDesc());
        } else if ("004".equals(param.getOrderServicetypeCode())) {
            orderNum.append("T");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.TRANSFORMATION.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.TRANSFORMATION.getDesc());
        }else if ("005".equals(param.getOrderServicetypeCode())) {
            orderNum.append("I");
            order.setOrderServicetypeCode(ServicePlanEnums.MessageType.INSPECTION.getCode());
            order.setOrderServicetypeName(ServicePlanEnums.MessageType.INSPECTION.getDesc());
        }

        //订单号重复问题处理 2024-11-05 commit
        SimpleDateFormat sdff = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdff2 = new SimpleDateFormat("yyyyMMdd");
        Date date = new Date();
        Timestamp timestamp = new Timestamp(date.getTime());
        sdff.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        sdff2.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        String formattedt1 = sdff.format(timestamp);
        String formattedt2 = sdff2.format(timestamp);
        String startDate = formattedt1 + " "+"00:00:00";
        String endDate = formattedt1 + " "+"23:59:59";
        HashMap<String, String> req = new HashMap<>();
        req.put("startDate", startDate);
        req.put("endDate", endDate);
        int num = nxOrderMapper.getOrderNum(req);
        Integer nums = num*100+ RandomUtil.randomInt(1,99);
        orderNum.append(formattedt2);
        orderNum.append(String.valueOf(nums));
        //orderNum.append(OrderNumberUtils.generateOrderNumber());
        order.setOrderOrdernumber(orderNum.toString());
        order.setOrderSource("app");
        order.setOrderDevicecode(deviceDataVo.getSblbSbbm());
        order.setOrderDevicename(deviceDataVo.getSblbSbmc());
        order.setOrderDevicetype(deviceDataVo.getSblbSblxName());
        order.setOrderDevicemodel(deviceDataVo.getSblbSbxhName());
        order.setOrderServiceaddress(deviceDataVo.getSblbSbxxdz());
        order.setOrderServiceaddressLongitude(deviceDataVo.getSblbJd());
        order.setOrderServiceaddressDimension(deviceDataVo.getSblbWd());
        order.setOrderCustomerName(deviceDataVo.getSblbKhmc());
        order.setOrderKhlxr(deviceDataVo.getSblbKhlxr());
        order.setOrderContactway(deviceDataVo.getSblbLxfs());
        order.setOrderCustomerId(deviceDataVo.getSblbKhid());
        order.setOrderOrderstatusCode(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode());
        order.setOrderOrderstatusName(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getDesc());

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        sdf.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        String formattedDate = sdf.format(new Date());
        order.setSyCreatetime(formattedDate);//创建时间 new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date())

        order.setSyStatus("1");
        order.setOrderApplydate(currentDate);
        List<OrderLcinfoParam> list = new ArrayList<>();
        OrderLcinfoParam orderLcinfoParam = new OrderLcinfoParam();
        orderLcinfoParam.setId(1);
        orderLcinfoParam.setName("已受理");
        orderLcinfoParam.setUserName(info.getUsername());
        orderLcinfoParam.setUserId(info.getUserid());
        orderLcinfoParam.setIcon("sl_icon");
        orderLcinfoParam.setEndDate(String.valueOf(com.rootcloud.common.DateUtil.getTimesByString(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))));
        list.add(orderLcinfoParam);
        order.setOrderLcinfo(JSON.toJSONString(list).toString());
        order.setSyZhid(info.getZhid());//设置租户ID

        //logger.info("服务需求:"+param.getOrderDemanddescription()+"--------logger---------");
        Log.info("服务需求:"+param.getOrderDemanddescription()+"-------Log----------");
        order.setOrderDemanddescription(param.getOrderDemanddescription()); //需求描述

        QueryWrapper<NxDeviceEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("NX_JCXX_SBLB_ID", Arrays.asList(deviceIds.split(",")).get(0));
        NxDeviceEntity nxDeviceEntity2 = nxDeviceMapper.selectOne(queryWrapper);
        //根据customerId 查询  NX_KHGL
        String wdId = servicePlanMapper.selectOutletsIdByCustomerId(nxDeviceEntity2.getSblbKhid(), nxDeviceEntity2.getSyZhid());
        Log.info("自动触发派单获取服务网点ID-->" + wdId+"-----Log----");

        if (StringUtils.isNotBlank(wdId)) {
            order.setOrderFwwdid(wdId);
        }
        //order.setOrderFwwdid(fwwdId); //服务网点ID
        //order.setSyCurrenttask("待派工");
        //logger.info("服务网点fwwdId:"+fwwdId+">>"+"服务需求Demanddescription:"+param.getOrderDemanddescription()+">>----logger----");
        Log.info("服务网点fwwdId:"+fwwdId+">>"+"服务需求Demanddescription:"+param.getOrderDemanddescription()+">>-------Log--------");
        Log.info("维保结束日期Wbjsrq:"+nxDeviceEntity2.getSblbWbjsrq()+">>-------Log--------");
        if (nxDeviceEntity2 != null)
        {
            order.setOrderWbjsrq(nxDeviceEntity2.getSblbWbjsrq());
            if (StringUtils.isNotBlank(nxDeviceEntity2.getSblbWbjsrq())) {
                if (new SimpleDateFormat("yyyy-MM-dd").format(new Date()).compareTo(nxDeviceEntity2.getSblbWbjsrq()) <= 0) {
                    order.setOrderWarrantiesCode("1");
                    order.setOrderWarrantiesName("是");
                } else {
                    order.setOrderWarrantiesCode("0");
                    order.setOrderWarrantiesName("否");
                }
            }
        }
        order.setOrderDdms("1");//自动触发
        nxOrderMapper.insert(order);
        Log.info("order:"+order);
        Log.info("nxOrderMapper.insert 执行完成-----------------");
        //流程引擎
        if (StringUtils.isNotBlank(order.getOrderFwwdid())) {
            //调用登录接口
            /*String loginName = servicePlanMapper.queryStringByCode("loginName") == null ? "app_master_admin" : servicePlanMapper.queryStringByCode("loginName");
            String loginPassword = servicePlanMapper.queryStringByCode("loginPassword") == null ? "Liu@123456" : servicePlanMapper.queryStringByCode("loginPassword");
            OAuthResult login = rootCloudAuthService.login(loginName, loginPassword, null);*/
            Log.info("OAuthResult:" + login + "loginName:" + param.getLoginName() + " loginPassword:" + param.getLoginPassword());
            OutletsVO outletsVO = servicePlanMapper.returnUserNames(order.getOrderFwwdid());
            String sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/sponsorProcess";
            Map<String, String> map = new HashMap();
            map.put("pdid", "frx-ningxia-dingdan_22051015-7");
            map.put("beanId", orderId);
            map.put("currentTaskName", "");
            map.put("submitComments", "订单待派工，请尽快处理");
            map.put("targerTaskName", "是否派工");
            map.put("targerTransition", "");
            map.put("assigeeName", outletsVO.getUserNames());
            map.put("assigeeId", outletsVO.getUserIds());
            map.put("submitType", "to_submit");
            Map<String, String> headers = new HashMap<>();
             headers.put("authorization", login.getAccess_token());
            // Log.info("authorization"+login.getAccess_token());
//            headers.put("authorization", "Hpq9v3R1hoCTVTaVD0p");
            headers.put("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
            Log.info(">>>>到这里了1:");
            try {
                Map<String, String> querys = new HashMap<>();
                org.apache.http.HttpResponse post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                int statusCode = post.getStatusLine().getStatusCode();
                String responseContent = EntityUtils.toString(post.getEntity(), "UTF-8");
                Log.info("服务计划派工订单待派工状态码:" + statusCode + " 返回值：" + responseContent);
                sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/completeTask";
                map.clear();
                Log.info(">>>>到这里了2:");
                JSONObject res = JSONObject.parseObject(responseContent);
                Object obj = res.get("obj");
                if (obj != null) {
                    Log.info(">>>>到这里了3:");
                    String taskId = ((JSONObject) obj).get("taskId").toString();
                    map.put("taskId", taskId);
                    map.put("pdid", "frx-ningxia-dingdan_22051015-7");
                    map.put("beanId", orderId);
                    map.put("submitComments", "同意");
                    map.put("currentTaskName", "是否派工");
                    map.put("targerTaskName", "待派工");
                    map.put("targerTransition", "to_decision2");
                    map.put("assigeeName", outletsVO.getUserNames());
                    map.put("assigeeId", outletsVO.getUserIds());
                    map.put("submitType", "to_submit");
                    querys.clear();
                    Log.info(">>>>到这里了4:");
                    post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                    statusCode = post.getStatusLine().getStatusCode();
                    Log.info(">>>>到这里了5:"+statusCode);
                    Log.info("服务计划派同意派工状态码:" + statusCode);

                    //技改工单创建成aa功后，将orderId、orderNum.toString() 写入mes技改任务表，为报工节点的信息填报做数据填充用
                    if ("004".equals(param.getOrderServicetypeCode())){
                        TechnicalTaskVO v1 = new TechnicalTaskVO();
                        v1.setId(param.getUid());
                        v1.setOrderId(orderId);
                        v1.setOrderNum(orderNum.toString());
                        technicalTaskDAO.updateTechnicalTask(v1);
                        Log.info("技改订单id、订单编码写入成功...");
                    }
                } else {
                    Log.error("服务计划派同意派工taskId获取不到");
                    throw new Exception();
                }
            } catch (Exception e) {
                Log.error("流程引擎错误" + e.getMessage());
                e.printStackTrace();
                throw new Exception();    //事务回滚
            }
        }
        return JsonResult.success();
    }

    @Override
    public void orderSignIn(OrderSignInParam orderSignInParam) {
        NxOrderEntity order = this.getOne((Wrappers.lambdaQuery(NxOrderEntity.class)
                .eq(NxOrderEntity::getNxOrderId, orderSignInParam.getNxOrderId())));
        if (ObjectUtils.isEmpty(order)) {
            //throw new BusinessException(StatusCodeEnum.ORDER_NOT_EXIST.getRet(),StatusCodeEnum.ORDER_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.ORDER_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("15001"));
        }
        if (!OrderEnums.OrderStatus.PENDING_CHECK_IN.getCode().equals(order.getOrderOrderstatusCode())) {
            //throw new BusinessException(StatusCodeEnum.SING_IN_ONLY_STATUS.getRet(),StatusCodeEnum.SING_IN_ONLY_STATUS.getMessage());
            throw new BusinessException(StatusCodeEnum.SING_IN_ONLY_STATUS.getRet(), ResourceUtil.getI18nMessage("15002"));
        }
        String lng1 = order.getOrderServiceaddressLongitude();
        String lat1 = order.getOrderServiceaddressDimension();
        Log.info("经纬度1：" + lng1 + "," + lat1);
        //查询设备位置
        List<NxDeviceLocationEntity> allDeviceLocation = deviceLocationMapper.findAllDeviceLocation();
        Map<String, NxDeviceLocationEntity> deviceLocationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(allDeviceLocation)) {
            deviceLocationMap = allDeviceLocation.stream().collect(Collectors.toMap(NxDeviceLocationEntity::getDeviceId, Function.identity()));
        }
        //查询设备 如果来自于4.0 取设备地址
        NxJeDeviceEntity nxJeDeviceEntity = nxJeDeviceService.selectBySbbm(order.getOrderDevicecode());
        if (!ObjectUtils.isEmpty(nxJeDeviceEntity)) {
            if (StringUtils.isNotBlank(nxJeDeviceEntity.getDeviceSource()) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(nxJeDeviceEntity.getDeviceSource())) {
                NxDeviceLocationEntity nxDeviceLocationEntity = deviceLocationMap.get(nxJeDeviceEntity.getNxJeDeviceId());
                if (nxDeviceLocationEntity != null && nxDeviceLocationEntity.getLongitude() != null && nxDeviceLocationEntity.getLatitude() != null) {
                    lng1 = String.valueOf(nxDeviceLocationEntity.getLongitude());
                    lat1 = String.valueOf(nxDeviceLocationEntity.getLatitude());
                    Log.info("经纬度2：" + lng1 + "," + lat1);
                }
            }
        }
//        Log.info("该设备经纬度==>{}",lng1+","+lat1);
        String lng2 = orderSignInParam.getOrderSignaddressLongitude();
        String lat2 = orderSignInParam.getOrderSignaddressDimension();
//        Log.info("签到经纬度==>{}",lng2+","+lat2);
        if (StringUtils.isNotBlank(lng1) && StringUtils.isNotBlank(lat1)
                && StringUtils.isNotBlank(lng2) && StringUtils.isNotBlank(lat2)) {
            double v = GetDistance(Double.parseDouble(lng1), Double.parseDouble(lat1), Double.parseDouble(lng2), Double.parseDouble(lat2));
            Log.info("经纬度3：" + lng2 + "," + lat2);
            if (v > 2000) {
                //throw new BusinessException(StatusCodeEnum.SING_IN_NOT_PASS.getRet(),StatusCodeEnum.SING_IN_NOT_PASS.getMessage());
                throw new BusinessException(StatusCodeEnum.SING_IN_NOT_PASS.getRet(), ResourceUtil.getI18nMessage("17001"));
            }
        } else {
            if (StringUtils.isBlank(lng1) || StringUtils.isBlank(lat1)) {
                throw new BusinessException(StatusCodeEnum.ABNORMAL_CHECK_IN.getRet(), ResourceUtil.getI18nMessage("60021"));
            }
            if (StringUtils.isBlank(lng2) || StringUtils.isBlank(lat2)) {
                throw new BusinessException(StatusCodeEnum.GET_CURRENT_LOCATION_EXCEPTION.getRet(), ResourceUtil.getI18nMessage("60022"));
            }
        }
        if (StringUtils.isBlank(orderSignInParam.getOrderSigntime())) {
            orderSignInParam.setOrderSigntime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
        }

        String orderLcinfo = order.getOrderLcinfo();
        List<OrderLcinfoParam> list = new ArrayList<>();
        if (StringUtils.isNotBlank(orderLcinfo)) {
            list = JSON.parseArray(orderLcinfo, OrderLcinfoParam.class);
            if (!ObjectUtils.isEmpty(orderSignInParam.getOrderLcinfo())) {
                list.add(orderSignInParam.getOrderLcinfo());
            }
        }

        NxOrderEntity updateOrder = new NxOrderEntity();
        BeanUtils.copyProperties(orderSignInParam, updateOrder);
        updateOrder.setOrderLcinfo(JSON.toJSONString(list).toString());
        updateOrder.setOrderOrderstatusCode(OrderEnums.OrderStatus.PENDING_REPORTING.getCode());
        updateOrder.setOrderOrderstatusName(OrderEnums.OrderStatus.PENDING_REPORTING.getDesc());
        nxOrderMapper.updateById(updateOrder);
    }


    /**
     * 根据提供的两个经纬度计算距离(米)
     *
     * @param lng1 经度1
     * @param lat1 纬度1
     * @param lng2 经度2
     * @param lat2 纬度2
     * @return 距离（米）
     */
    public static double GetDistance(double lng1, double lat1, double lng2, double lat2) {
        double radLat1 = lat1 * RAD;
        double radLat2 = lat2 * RAD;
        double a = radLat1 - radLat2;
        double b = (lng1 - lng2) * RAD;
        double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2) +
                Math.cos(radLat1) * Math.cos(radLat2) * Math.pow(Math.sin(b / 2), 2)));
        s = s * EARTH_RADIUS;
        s = Math.round(s * 10000) / 10000;
        return s;
    }

    /* @Override
   public List<MyMessageEntity> queryOrderInfo(List<DeviceListEntity> deviceListEntities) {
        List<MyMessageEntity> result = new ArrayList<>();
        List<String> deviceCodes = deviceListEntities.stream().map(DeviceListEntity::getSblbSbbm).collect(Collectors.toList());
        LambdaQueryWrapper<NxOrderEntity> orderEntityLambdaQueryWrapper =  Wrappers.lambdaQuery(NxOrderEntity.class);
        List<String> fwjgs = new ArrayList<>();
        fwjgs.add("2");
        fwjgs.add("3");
        //todo 目前根据产品之前的描述只查询维修单，且维修状态时未解决和部分解决的订单（订单状态为待审核）
        orderEntityLambdaQueryWrapper.in(NxOrderEntity::getOrderDevicecode,deviceCodes)
                .eq(NxOrderEntity::getOrderServicetypeCode,"1")
                .eq(NxOrderEntity::getOrderOrderstatusCode,"5")
                .in(NxOrderEntity::getOrderFwjg,fwjgs)
                .between(NxOrderEntity::getOrderOrdertime, com.rootcloud.common.DateUtil.format(new Date()), com.rootcloud.common.DateUtil.format(com.rootcloud.common.DateUtil.rollDay(new Date(),-3)));
        List<NxOrderEntity> orderEntities = this.list(orderEntityLambdaQueryWrapper);
        if(CollectionUtil.isEmpty(orderEntities)){
            return result;
        }
        orderEntities.stream().forEach(temp->{
            MyMessageEntity myMessageEntity = new MyMessageEntity();
            myMessageEntity.setMessageClass("服务结果提醒");
            myMessageEntity.setMessageObjectCode(temp.getOrderDevicecode());
            myMessageEntity.setMessageObjectName(temp.getOrderDevicename());
            myMessageEntity.setMessageName("工单验收");
            //产品经理确认让查服务计划中发送给用户的站内信消息
            myMessageEntity.setMessageContent("您的工单"+temp.getNxOrderId()+"已完成请验收！");
            //产品经理让取服务计划触发时间
            myMessageEntity.setMessageTime(temp.getOrderSigntime());
            result.add(myMessageEntity);
        });
        return result;
    }*/
}
