package com.rootcloud.domain.service.part.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.JsonResult;
import com.rootcloud.config.auth.LoginUserInfo;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.domain.dao.part.NxPartFileConfigDao;
import com.rootcloud.domain.dto.EnterpriseInfoDTO;
import com.rootcloud.domain.entity.NxPartFileConfigEntity;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import com.rootcloud.domain.service.part.NxPartFileConfigService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.UUID;

/**
 * @author lujiumin
 * @date 2022/10/13
 * 描述：
 */
@Slf4j
@Service("nxpartFileConfigService")
public class NxPartFileConfigServiceImpl extends ServiceImpl<NxPartFileConfigDao, NxPartFileConfigEntity> implements NxPartFileConfigService {

    @Autowired
    private NxPartFileConfigDao nxPartFileConfigDao;

    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;


    /**
     * 新增
     *
     * @param entity
     */
    @Override
    public JsonResult insertOrUpdate(NxPartFileConfigEntity entity) {
        try {
            LoginUserInfo current = UserContext.current();
            String zhid = current.getTenantId();  //需要进行租户ID过滤
            if(StringUtils.isNotBlank(zhid)){
                entity.setSyZhid(zhid);
            }else{
                EnterpriseInfoDTO dto = enduserService.getTenantInfo(current.getUserId());
                entity.setSyZhid(dto.getTenantId());
            }

            //登记人
            entity.setSyCreateuserid(current.getUserId());
            entity.setSyCreatetime(DateUtil.formatDateTime(new Date()));
            entity.setSyCreateusername(current.getUserName());
            if (StringUtils.isBlank(entity.getNxPartFileConfigId())) {
                //新增
                entity.setNxPartFileConfigId(UUID.randomUUID().toString().replaceAll("-", ""));
                nxPartFileConfigDao.insert(entity);
            } else {
                //修改
                nxPartFileConfigDao.updateById(entity);
            }
            return JsonResult.success();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 根据租户id的配置
     *
     * @return
     */
    @Override
    public JsonResult<NxPartFileConfigEntity> selectData() {
        LoginUserInfo current = UserContext.current();
        String zhid = current.getTenantId();  //需要进行租户ID过滤
        if (StringUtils.isBlank(zhid)) {
            EnterpriseInfoDTO dto = enduserService.getTenantInfo(current.getUserId());
            zhid = dto.getTenantId();
        }
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("SY_ZHID", zhid);
        NxPartFileConfigEntity nxPartFileConfigEntity = nxPartFileConfigDao.selectOne(queryWrapper);
        return JsonResult.success(nxPartFileConfigEntity);
    }


}
