package com.rootcloud.domain.service.part.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.util.StringUtil;
import com.rootcloud.common.*;
import com.rootcloud.config.auth.LoginUserInfo;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.domain.dao.part.NxPartFileDao;
import com.rootcloud.domain.dto.EnterpriseInfoDTO;
import com.rootcloud.domain.entity.JeCoreEnduserEntity;
import com.rootcloud.domain.entity.NxPartFileConfigEntity;
import com.rootcloud.domain.entity.NxPartFileEntity;
import com.rootcloud.domain.entity.NxPartRespEntity;
import com.rootcloud.domain.service.auth.UserService;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import com.rootcloud.domain.service.part.NxPartFileConfigService;
import com.rootcloud.domain.service.part.NxPartFileService;
import com.rootcloud.domain.service.part.NxPartRespService;
import jline.internal.Log;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Service("nxpartFileService")
public class NxPartFileServiceImpl extends ServiceImpl<NxPartFileDao, NxPartFileEntity> implements NxPartFileService {

    private static final Logger logger = LoggerFactory.getLogger(NxPartFileServiceImpl.class);

    // 命名空间
    public static final String ACTION = "http://sap.com/xi/WebService/soap1.1";
    // 调用方法
    public static final String METHODNAME = "ZDM_INVENTORYINF";
    //webservice地址
    public static final String WSDLLOCATION = "http://61.133.215.5:50000/XISOAPAdapter/MessageServlet?senderParty=&senderService=BS_LIMS_PRD&receiverParty=&receiverService=&interface=SI_ZDM_INVENTORY_Sender&interfaceNamespace=http://www.kocel.com/lims/zdm_inventoryinf";

    //    private static final String DEFAULTSWERKS = "1100,11X0,1200,12X0,1300,13X0,1430,1500,15X0,1600,1610,1620,1630,1640,1650,1670,1680,1690,16A0,16B0,16C0,16K0,16L0,16M0,16N0,16P0,1700,1800,1900,19X0,2000,2100,2200,2300,3000,3100,3200,3300,3400";
    private static final String DEFAULTSWERKS = "1430";

    private static final String DEFAULTLGORT = "1004";

    //配件预警通知角色
    private static final String ROLE = "ROLE_2022070039";

    @Autowired
    NxPartFileDao nxPartFileDao;

    @Autowired
    UserService userService;

    @Resource(name = "partRespService")
    NxPartRespService partRespService;

    @Autowired
    SendMessageUtils sendMessageUtils;

    @Resource(name = "nxpartFileConfigService")
    private NxPartFileConfigService nxPartFileConfigService;

    @Resource(name = "endUserService")
    private JeCoreEnduserService enduserService;

    @Override
    public NxPartFileEntity getByPartCode(String filePjbm, String fileCkbm, String fileKwbm, String syZhid) {
        return nxPartFileDao.getByPartCode(filePjbm, fileCkbm, fileKwbm, syZhid);
    }

    @Override
    public int updateByPartCode(NxPartFileEntity record) {
        return nxPartFileDao.updateByPartCode(record);
    }

    /**
     * 更新配件仓库名称
     *
     * @param record
     * @return
     */
    @Override
    public JsonResult updateCkmc(NxPartFileEntity record, LoginUserInfo current) {
        EnterpriseInfoDTO dto = enduserService.getTenantInfo(current.getUserId());
        String zhid = dto.getTenantId();
        record.setSyZhid(zhid);
        Log.info("传参:" + record);
        //获取该租户下的配置
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("sy_zhid", record.getSyZhid());
        List<NxPartFileConfigEntity> list = nxPartFileConfigService.list(queryWrapper);
        if (list.size() > 0) {
            Log.info("同步配件信息开始...");
            dataSync(list);
            Log.info("同步配件信息结束...");
        }
        //修改仓库名称
        Log.info("修改配件信息仓库名称开始...");
        nxPartFileDao.updateCkmc(record);
        Log.info("修改配件信息仓库名称结束...");
        return JsonResult.success();
    }


    @Override
    public boolean syncPartData() {
        Log.info("同步配件信息开始...");
        List<NxPartFileConfigEntity> list = nxPartFileConfigService.list();
        if (list.size() > 0) {
            dataSync(list);
        }
        Log.info("同步配件信息结束...");
        return true;

    }

    /**
     * 同步数据公共方法
     *
     * @param list
     */
    private void dataSync(List<NxPartFileConfigEntity> list) {
        //拿到配件仓库对应的仓库名称
        Map<String, String> preMap = queryPre();
        Log.info("获取到租户配置数据：" + list.size());
        for (int i = 0; i < list.size(); i++) {
            NxPartFileConfigEntity o = list.get(i);
            try {
                //运维中心、泉州中心租户 采用新版ERP系统
                if(o.getSyZhid().equals("624e7ce1549c110050c5da47")||o.getSyZhid().equals("647e8e8a40a0e4003fa75429"))
                    continue;
                Log.info("获取到数据：" + o.getSyZhid() + "，" + o.getConfigQqrc());
                String configQqlj = StringUtils.isNotBlank(o.getConfigQqlj()) ? o.getConfigQqlj() : WSDLLOCATION;
                String configQqrc = o.getConfigQqrc();
                String configDyff = StringUtils.isNotBlank(o.getConfigDyff()) ? o.getConfigDyff() : METHODNAME;
                String configMmkj = StringUtils.isNotBlank(o.getConfigMmkj()) ? o.getConfigMmkj() : ACTION;
                if (StringUtils.isNotBlank(configQqrc)) {
                    Map<String, String> params = null;
                    try {
                        params = JSON.parseObject(configQqrc, Map.class);
                    } catch (Exception e) {
                        logger.error("入参数据格式错误：{}", configQqrc);
                        throw new RuntimeException(e);
                    }
                    //params.put("Z_WERKS", split[0]);
                    //params.put("Z_LGORT", split[1]);
                    if (params == null) {
                        continue;
                    }
                    Log.info("正在启动SAP库同步，路径：" + configQqlj + "，命名空间：" + configMmkj + "，调用方法：" + configDyff + "，入参：" + params);
                    WebServiceClient client = new WebServiceClient(configQqlj, configMmkj, configDyff, params);
                    List<NxPartFileEntity> partList = client.getPartData();
                    Log.info("接口返回数据条数：", partList.size());
                    if (partList.size() > 0) {
                        //重复的配件 把数量相加
                        List<NxPartFileEntity> collect = partList.stream().collect(Collectors.toMap(NxPartFileEntity::getFilePjbm, a -> a, (o1, o2) -> {
                            o1.setFileKcsl(o1.getFileKcsl().add(o2.getFileKcsl()));
                            return o1;
                        })).values().stream().collect(Collectors.toList());
                        Log.info("合并后数据条数：", collect.size());
                        for (NxPartFileEntity part : collect) {
                            part.setSyCreatetime(DateUtil.getCurrentTime());
                            NxPartFileEntity byPartCode = this.getByPartCode(part.getFilePjbm(), params.get("Z_WERKS"), params.get("Z_LGORT"), o.getSyZhid());
                            //通过租户id实时查询仓库名称
                            try {
                                String fileCkbm = part.getFileCkbm();
                                String fileKwbm = part.getFileKwbm();
                                String syZhid = o.getSyZhid();
                                LambdaQueryWrapper<NxPartRespEntity> npre = new LambdaQueryWrapper<>();
                                npre.eq(NxPartRespEntity::getRespCkbm, fileCkbm);
                                npre.eq(NxPartRespEntity::getRespKwbm, fileKwbm);
                                npre.eq(NxPartRespEntity::getSyZhid, syZhid);
                                NxPartRespEntity one = partRespService.getOne(npre);
                                part.setFileCkmc(one.getRespCkmc());
                            } catch (Exception e) {
                                Log.info("查询仓库名称异常:" + e.getMessage());
                                e.printStackTrace();
                            }
                            //                            if (StringUtil.isNotEmpty(preMap.get(part.getFileCkbm()))) {
//                                part.setFileCkmc(preMap.get(part.getFileCkbm()));
//                            }
                            if (byPartCode != null) {
                                part.setNxPartFileId(byPartCode.getNxPartFileId());
                                part.setSyZhid(o.getSyZhid());
                                this.updateByPartCode(part);
                            } else {
                                part.setSyZhid(o.getSyZhid());
                                this.save(part);
                            }
                            nxPartFileDao.insertPartFile2(part);
                            //判断是否是关键备件
                            if (byPartCode != null && "1".equals(byPartCode.getFileSfsgjpj())) {
                                if ((byPartCode.getFileKcsx() != 0 && byPartCode.getFileKcsl().intValue() > byPartCode.getFileKcsx()) || (byPartCode.getFileKcxx() != 0 && byPartCode.getFileKcsl().intValue() < byPartCode.getFileKcxx())) {//大于上限或小于下限
                                    List<String> phones = userService.getUserListByRole(ROLE, o.getSyZhid()).stream().filter(e -> e.getBackusercode() != null)
                                            .map(JeCoreEnduserEntity::getBackusercode).collect(Collectors.toList());
                                    if (!CollectionUtils.isEmpty(phones)) {
                                        Log.info("给" + phones + "发送短信!");
                                        List<String> param = new ArrayList<>();
                                        param.add(byPartCode.getFilePjbm());//配件编码
                                        param.add(byPartCode.getFilePjmc());//配件名称
                                        param.add(byPartCode.getFileKcsx().toString());//库存上限
                                        param.add(byPartCode.getFileKcxx().toString());//库存下限
                                        param.add(byPartCode.getFileKcsl().toString());//库存数量
                                        //发送短信通知
                                        sendMessageUtils.send(phones, param);
                                    }
                                }
                            }
                        }
                        Log.info("租户ID：" + o.getSyZhid() + "，同步数据完成，同步数据：" + collect.size());
                    }
                }
            } catch (Exception e) {
                Log.info("租户ID：" + o.getSyZhid() + "，同步数据失败");
                e.printStackTrace();
            }

        }
    }

    @Override
    public List<String> queryLgortCode() {
        return nxPartFileDao.queryLgortCode();
    }

    @Override
    public boolean syncPartWareData() {
        //获取的地方做了调整
        List<NxPartFileConfigEntity> list = nxPartFileConfigService.list();
        if (list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                NxPartFileConfigEntity o = list.get(i);
                String configQqrc = o.getConfigQqrc();
                if (StringUtils.isNotBlank(configQqrc)) {
                    Map<String, String> params = null;
                    try {
                        params = JSON.parseObject(configQqrc, Map.class);
                    } catch (Exception e) {
                        logger.error("入参数据格式错误：{}", configQqrc);
                        throw new RuntimeException(e);
                    }
                    if (params == null) {
                        continue;
                    }
                    String Z_WERKS = params.get("Z_WERKS");
                    String Z_LGORT = params.get("Z_LGORT");
                    String syZhid = o.getSyZhid();
                    LambdaQueryWrapper<NxPartRespEntity> npre = new LambdaQueryWrapper<>();
                    npre.eq(NxPartRespEntity::getSyZhid, syZhid);
                    NxPartRespEntity one = partRespService.getOne(npre);
                    if (one != null) {
                        one.setRespCkbm(Z_WERKS);
                        one.setRespKwbm(Z_LGORT);
                        partRespService.updateById(one);
                    } else {
                        NxPartRespEntity entity = new NxPartRespEntity();
                        entity.setNxPartRespId(JEUuidUtil.uuid());
                        entity.setRespCkbm(Z_WERKS);
                        entity.setSyZhid(syZhid);
                        //  entity.setRespCkmc(s);
                        entity.setRespKwbm(Z_LGORT);
                        entity.setSyCreatetime(DateUtil.getCurrentTime());
                        entity.setSyCreateuser("system");
                        partRespService.save(entity);
                    }
                }
            }
        }
        return true;
    }

    public Map<String, String> queryPre() {
        Map<String, String> maps = new HashMap<>();
        List<NxPartRespEntity> list = partRespService.list();
        if (list.size() > 0) {
            list.forEach(o -> {
                maps.put(o.getRespCkbm(), o.getRespCkmc());
            });
        }
        return maps;
    }


}