package com.rootcloud.domain.service.sp;

import com.baomidou.mybatisplus.extension.service.IService;
import com.rootcloud.common.JsonResult;
import com.rootcloud.domain.ao.ServicePlanAO;
import com.rootcloud.domain.ao.ServicePlanCalendarAO;
import com.rootcloud.domain.ao.ServicePlanCalendarNewAO;
import com.rootcloud.domain.entity.MyAdviceEntity;
import com.rootcloud.domain.entity.ds.ModelPropertySyncEntity;
import com.rootcloud.domain.entity.sp.ServicePlanEntity;
import com.rootcloud.vo.device.ServicePlanCalendarList;
import com.rootcloud.vo.device.ServicePlanCalendarVO;
import com.rootcloud.vo.device.ServicePlanDetailVO;

import java.util.List;

public interface ServicePlanService extends IService<ServicePlanEntity> {
    // 新增服务计划
    JsonResult addPlan(ServicePlanAO planAO);

    public JsonResult startServicePlan(String servicePlanId);

    /**
     * 根据设备编号查询工况条件
     *
     * @param deviceNo 设备编号
     * @return 工况属性列表
     */
    List<ModelPropertySyncEntity> getWorkCondition(String deviceNo);

    /**
     * 根据设备编号查询ModelId
     *
     * @param deviceNo 设备编号
     * @return modelId
     */
    String getModelIdByDeviceNo(String deviceNo);


    /**
     * 定时处理  设置了触发条件为系统时间的数据
     */
    void scheduledProcessing();

    void scheduledProcessingNew(String thatDay, String planId);


    /**
     * 定时修改服务状态的任务
     */
    void updateStatusScheduled();


    /**
     * 更新服务计划状态
     *
     * @param servicePlanId     服务主键
     * @param servicePlanStatus 状态
     * @return
     */
    boolean updateServicePlanStatus(String servicePlanId, int triggerType, Integer servicePlanStatus, boolean tempTime, String thatDay);


    /**
     * 每天十点的定时任务
     */
    void execute();


    /**
     * 定时查询报警
     */
    void queryAlarmQz();


    /**
     * 查询详情
     *
     * @param servicePlanId
     * @return
     */
    ServicePlanDetailVO selectDetailById(String servicePlanId);


    /**
     * 根据服务计划id删除服务计划
     *
     * @param servicePlanId
     */
    int deleteServicePlan(String servicePlanId);


    /**
     * 查询任务日历
     *
     * @param planCalendarAO
     * @return
     */
    ServicePlanCalendarVO checkServicePlanCalendar(ServicePlanCalendarAO planCalendarAO);


    /**
     * 计算完成率
     *
     * @param list
     * @return
     */
    String finishRate(List<ServicePlanCalendarList> list);

    /**
     * 查询已触发的保养单
     */
    List<MyAdviceEntity> getmMaintainPlanService(String userId, Integer startNo, Integer endNo);


    void eqMesSfjj(String id);


    /**
     * 日历总查询
     *
     * @param servicePlanCalendarNewAO
     * @return
     */
    ServicePlanCalendarVO checkDataByParams(ServicePlanCalendarNewAO servicePlanCalendarNewAO);


    /**
     * 一键生成订单
     *
     * @param servicePlan
     */
    void generateOrder(ServicePlanEntity servicePlan, int triggerType, String thatDay,boolean temp) throws Exception;
    //修改服务计划临时时间触发的订单生成逻辑，增加状态字段用以识别  20241231

    /**
     * 手动触发服务计划订单
     *
     * @param planId
     */
    JsonResult<Object> manualTrigger(String planId);

    /**
     * 更新服务计划下一周期日期
     */
    void updateDate();

    /**
     * 暂停服务计划
     *
     * @param planId
     * @return
     */
    JsonResult<Object> pause(String planId,String type);
    /**
     * 提前触发服务计划
     *
     * @param planId
     * @return
     */
    JsonResult<Object> pretrigger(String planId);
}
