package com.rootcloud.domain.service.sp.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.rootcloud.common.ResourceUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.JEUuidUtil;
import com.rootcloud.common.TokenUtil;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.ao.TriggerConditionAO;
import com.rootcloud.domain.entity.ds.ModelPropertySyncEntity;
import com.rootcloud.domain.entity.sp.NxTriggerConditionEntity;
import com.rootcloud.domain.mapper.NxTriggerConditionMapper;
import com.rootcloud.domain.service.ds.ModelPropertySyncService;
import com.rootcloud.domain.service.sp.NxTriggerConditionService;
import com.rootcloud.domain.service.sp.ServicePlanService;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

/**
 * <p>
 * 触发条件service实现
 * </p>
 *
 * @author hejia
 */
@Service
@Slf4j
public class NxTriggerConditionServiceImpl extends ServiceImpl<NxTriggerConditionMapper, NxTriggerConditionEntity>
    implements NxTriggerConditionService {

    @Value("${os.iam.domain}")
    private String domain;

    @Value("${cms.addrule.uri}")
    private String addRuleUrl;

    @Value("${cms.thingdraft.uri}")
    private String thingDraftUrl;

    @Value("${cms.delrule.uri}")
    private String delRuleUrl;

    private final TokenUtil tokenUtil;
    private final ServicePlanService servicePlanService;
    private final ModelPropertySyncService modelPropertySyncService;

    public NxTriggerConditionServiceImpl(TokenUtil tokenUtil,
        @Lazy @Qualifier("servicePlanServiceImpl") ServicePlanService servicePlanService,
        @Qualifier("modelPropertySyncService2") ModelPropertySyncService modelPropertySyncService) {
        this.tokenUtil = tokenUtil;
        this.servicePlanService = servicePlanService;
        this.modelPropertySyncService = modelPropertySyncService;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void add(TriggerConditionAO conditionAo, String planId, String deviceNo) {
        if (conditionAo == null || ObjectUtils.isEmpty(conditionAo)) {
            return;
        }
        // 这里需要先设置id，下面调用平台接口的时候用到
        conditionAo.setNxTriggerConditionId(JEUuidUtil.uuid());

        String alarmTypeId = null;
        // 调用平台接口进行添加操作
        if (ServicePlanEnums.TriggerType.WORK_CONDITION.getCode().equals(conditionAo.getTriggerCondition())) {
            alarmTypeId = this.addRule(conditionAo, deviceNo);
        }

        NxTriggerConditionEntity conditionEntity = new NxTriggerConditionEntity();
        conditionEntity.setServicePlanId(planId);
        conditionEntity.setAlarmTypeId(alarmTypeId);
        conditionEntity.setCreateTime(DateUtil.getSysdateStr());
        BeanUtils.copyProperties(conditionAo, conditionEntity);
        this.save(conditionEntity);
    }

    @Override
    public void update(TriggerConditionAO conditionAo, String planId, String deviceNo) {

        String triggerCode = ServicePlanEnums.TriggerType.WORK_CONDITION.getCode();
        if ("del".equals(conditionAo.getOperationType())) {
            this.removeByPlanId(planId, deviceNo);
        } else {
            NxTriggerConditionEntity entity = this.getById(conditionAo.getNxTriggerConditionId());
            if (ObjectUtil.isNull(entity)) {
                return;
            }

            String alarmTypeId = null;
            // 之前是工况类型，现在改为不是工况类型，要删除掉平台的数据
            if (!triggerCode.equals(conditionAo.getTriggerCondition())
                && triggerCode.equals(entity.getTriggerCondition())) {
                removeRule(deviceNo, entity.getAlarmTypeId());
            }
            // 现在是工况，之前不是工况，在平台那里进行新增
            if (triggerCode.equals(conditionAo.getTriggerCondition())
                && !triggerCode.equals(entity.getTriggerCondition())) {
                alarmTypeId = addRule(conditionAo, deviceNo);
            }
            // 现在是工况，之前也是工况，在平台就行修改
            if (triggerCode.equals(conditionAo.getTriggerCondition())
                && triggerCode.equals(entity.getTriggerCondition())) {
                alarmTypeId = updateRule(conditionAo, deviceNo, entity.getAlarmTypeId());
            }
            // 注：这里【alarmTypeId】字段在实体里面修改了更新策略，null值允许修改
            NxTriggerConditionEntity conditionEntity = new NxTriggerConditionEntity();
            BeanUtils.copyProperties(conditionAo, conditionEntity);
            conditionEntity.setAlarmTypeId(alarmTypeId);
            this.updateById(conditionEntity);
        }
    }

    @Override
    public void removeByPlanId(String servicePlanId, String deviceNo) {
        NxTriggerConditionEntity condition =
                this.getOne(Wrappers.lambdaQuery(NxTriggerConditionEntity.class)
                        .eq(NxTriggerConditionEntity::getServicePlanId, servicePlanId)
                        .last("limit 1"));
        if (ObjectUtil.isNull(condition)) {
            return;
        }
        if (StrUtil.isNotBlank(condition.getAlarmTypeId())) {
            this.removeRule(deviceNo, condition.getAlarmTypeId());
        }
        this.remove(Wrappers.lambdaQuery(NxTriggerConditionEntity.class).eq(NxTriggerConditionEntity::getServicePlanId,
                servicePlanId));
    }

    private String updateRule(TriggerConditionAO conditionAo, String deviceNo, String alarmTypeId) {
        String modelId = servicePlanService.getModelIdByDeviceNo(deviceNo);
        // 构建请求参数
        Map<String, Object> param = buildParam(conditionAo, modelId);
        // 发送数据
        String url = delRuleUrl.replace("{alarmTypeId}", alarmTypeId);
        HttpRequest request = HttpRequest.put(domain + url);
        setHeader(request);
        log.info("【修改报警规则】请求参数==>{}", JSON.toJSONString(param));
        request.body(JSON.toJSONString(param));
        HttpResponse response = request.execute();
        // 判断状态码
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("调用平台【修改报警规则】接口出错:{}", response);
            //throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
             //   StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                    ResourceUtil.getI18nMessage("40000"));
        }

        log.info("【修改报警规则】返回参数==>{}", response.body());
        JSONObject json = JSON.parseObject(response.body());
        JSONObject payload = json.getJSONObject("payload");

        // 请求发布的接口
        this.releaseRule(modelId);

        return payload.getString("alarmTypeId");
    }

    private void removeRule(String deviceNo, String alarmTypeId) {
        String url = delRuleUrl.replace("{alarmTypeId}", alarmTypeId);
        HttpRequest request = HttpRequest.delete(domain + url);
        setHeader(request);
        HttpResponse response = request.execute();
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("调用平台【删除报警类型】接口出错:{}", response);
           // throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
            //        StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                    ResourceUtil.getI18nMessage("40000"));
        }

        String modelId = servicePlanService.getModelIdByDeviceNo(deviceNo);
        this.releaseRule(modelId);
    }

    private void releaseRule(String modelId) {
        String url = thingDraftUrl.replace("{modelId}", modelId);
        HttpRequest request = HttpRequest.patch(domain + url);
        setHeader(request);
        HttpResponse response = request.execute();
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("调用平台【发布物模型】接口出错:{}", response);
            /*throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());*/
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                    ResourceUtil.getI18nMessage("40000"));
        }
    }

    private String addRule(TriggerConditionAO conditionAo, String deviceNo) {
        String modelId = servicePlanService.getModelIdByDeviceNo(deviceNo);
        // 构建请求参数
        Map<String, Object> param = buildParam(conditionAo, modelId);
        // 发送数据
        HttpRequest request = HttpRequest.post(domain + addRuleUrl);
        setHeader(request);
        log.info("【创建报警类型草稿版】请求参数==>{}", JSON.toJSONString(param));
        request.body(JSON.toJSONString(param));
        HttpResponse response = request.execute();
        if (response.getStatus() != HttpStatus.HTTP_OK) {
            log.error("平台【创建报警类型草稿版】接口出错:{}", response);
            /*throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());*/
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                    ResourceUtil.getI18nMessage("40000"));
        }
        log.info("【创建报警类型草稿版】返回参数==>{}", response.body());
        JSONObject json = JSON.parseObject(response.body());
        JSONObject payload = json.getJSONObject("payload");

        // 请求发布的接口
        this.releaseRule(modelId);

        return payload.getString("alarmTypeId");
    }

    private void setHeader(HttpRequest request) {
        request.header("Authorization", "Bearer " + tokenUtil.getToken());
        request.header("accept", "application/json");
        request.header("Content-Type", "application/json");
    }

    private Map<String, Object> buildParam(TriggerConditionAO conditionAo, String modelId) {

        ModelPropertySyncEntity property =
            modelPropertySyncService.queryByModelIdAndPropertyId(modelId, conditionAo.getConditionTj());
        Optional.ofNullable(property)
            .orElseThrow(() -> new BusinessException(StatusCodeEnum.EXCEPTION.getRet(), ResourceUtil.getI18nMessage("50001")));

        Map<String, Object> param = new HashMap<>(6);
        StringBuilder subject = new StringBuilder(property.getPropertyName());
        if (ServicePlanEnums.Condition.GT.getCode().equals(conditionAo.getConditionValue())) {
            subject.append("过高");
        } else if (ServicePlanEnums.Condition.LT.getCode().equals(conditionAo.getConditionValue())) {
            subject.append("过低");
        }
        param.put("subject", subject.toString());
        param.put("severityLevel", 1);
        param.put("when", "becomesTrue");
        param.put("alarmTypeName", conditionAo.getNxTriggerConditionId());
        param.put("modelId", modelId);
        param.put("requiredAck", Boolean.TRUE);
        param.put("condition",
            conditionAo.getConditionTj() + " " + conditionAo.getConditionValue() + " " + conditionAo.getSetValue());
        return param;
    }
}
