package com.rootcloud.domain.service.sp.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.HttpUtils;
import com.rootcloud.common.JEUuidUtil;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.TokenUtil;
import com.rootcloud.common.constant.Properties;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.config.auth.LoginUserInfo;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.auth.UserInfo;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.dao.GbsjbDao;
import com.rootcloud.domain.dao.JeCoreEnduserDao;
import com.rootcloud.domain.dao.JeSysUsermsgDao;
import com.rootcloud.domain.dao.sp.NxUserInfoDao;
import com.rootcloud.domain.entity.JeSysUsermsgEntity;
import com.rootcloud.domain.entity.NxJeDeviceEntity;
import com.rootcloud.domain.entity.sp.NxTriggerConditionEntity;
import com.rootcloud.domain.entity.sp.NxUserInfoEntity;
import com.rootcloud.domain.entity.sp.ServicePlanEntity;
import com.rootcloud.domain.mapper.NxTriggerConditionMapper;
import com.rootcloud.domain.mapper.NxUserInfoMapper;
import com.rootcloud.domain.mapper.ServicePlanMapper;
import com.rootcloud.domain.service.gkjk.NxJeDeviceService;
import com.rootcloud.domain.service.sp.NxUserInfoService;
import com.rootcloud.vo.service.NxUserInfoVo;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName NxUserInfoServiceImpl
 * @Description 提醒用户
 * @Author w-liuyi01
 * @Date 2022/4/28 20:18
 * @Version 1.0
 */

@Service("NxUserInfoServiceImpl")
@Slf4j
public class NxUserInfoServiceImpl extends ServiceImpl<NxUserInfoDao, NxUserInfoEntity> implements NxUserInfoService {


    @Value("${os.iam.domain}")
    private String domain;

    @Value("${cms.email.uri}")
    private String emailUrl;

    @Autowired
    JeCoreEnduserDao jeCoreEnduserDao;

    @Autowired
    JeSysUsermsgDao jeSysUsermsgDao;

    private final TokenUtil tokenUtil;
    private final NxUserInfoMapper nxUserInfoMapper;
    private final Properties wfProperties;
    private final GbsjbDao gbsjbDao;
    private final ServicePlanMapper servicePlanMapper;
    private final NxJeDeviceService nxJeDeviceService;
    private final NxTriggerConditionMapper nxTriggerConditionMapper;

    public NxUserInfoServiceImpl(TokenUtil tokenUtil, NxUserInfoMapper nxUserInfoMapper, Properties wfProperties,
                                 GbsjbDao gbsjbDao, ServicePlanMapper servicePlanMapper, @Qualifier("nxJeDeviceServiceImpl") NxJeDeviceService nxJeDeviceService,
                                 NxTriggerConditionMapper nxTriggerConditionMapper) {
        this.tokenUtil = tokenUtil;
        this.nxUserInfoMapper = nxUserInfoMapper;
        this.wfProperties = wfProperties;
        this.gbsjbDao = gbsjbDao;
        this.servicePlanMapper = servicePlanMapper;
        this.nxJeDeviceService = nxJeDeviceService;
        this.nxTriggerConditionMapper = nxTriggerConditionMapper;
    }


    @Override
    public void setReminders(String servicePlanId) {
        String templateContent = getTemplateContent(servicePlanId);
        QueryWrapper<NxUserInfoEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("SERVICE_PLAN_ID", servicePlanId);
        NxUserInfoEntity nxUserInfoEntity = nxUserInfoMapper.selectOne(queryWrapper);
        if (ObjectUtil.isEmpty(nxUserInfoEntity)) {
            Log.info("服务计划{}nxUserInfoEntity is  Empty", servicePlanId);
            return;
        }

        QueryWrapper<ServicePlanEntity> entityQueryWrapper = new QueryWrapper<>();
        entityQueryWrapper.eq("SERVICE_PLAN_ID", servicePlanId);
        ServicePlanEntity servicePlan = servicePlanMapper.selectOne(entityQueryWrapper);
        if (ObjectUtil.isEmpty(servicePlan)) {
            Log.info("服务计划{}servicePlan is  Empty", servicePlanId);
            return;
        }
        List<String> customerAndRoles = getCustomerAndRoles(servicePlan.getDeviceCode());
        if (customerAndRoles.isEmpty()) {
            Log.info("服务计划{}customerAndRoles is  Empty", servicePlanId);
            return;
        }
        customerAndRoles = customerAndRoles.stream().distinct().collect(Collectors.toList());
        String userIds = customerAndRoles.stream()
                .collect(Collectors.joining(","));

        //如果提醒方式存在站内信 发送站内信
        if (StringUtils.isNotBlank(nxUserInfoEntity.getSmsReminders()) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(nxUserInfoEntity.getSmsReminders())) {
            //发送短信
        }
        if (StringUtils.isNotBlank(nxUserInfoEntity.getEmailReminders()) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(nxUserInfoEntity.getEmailReminders())) {
            //发送邮件
            List<Map> receiverList = returnByUserList(customerAndRoles);
            if (!receiverList.isEmpty()) {
                Map<String, Object> param = buildEmailParam(null, null, null, templateContent, new ArrayList<>(), returnByUserList(customerAndRoles));
                if (!param.isEmpty()) {
                    try {
                        sendEmail(param);
                    } catch (Exception e) {
                        log.info("服务计划{}错误信息==>{}", servicePlanId, e.getMessage());
                    }
                }
            }
        }
        if (StringUtils.isNotBlank(nxUserInfoEntity.getStationReminders()) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(nxUserInfoEntity.getStationReminders())) {
            //发送站内信
            sendStationReminders(userIds, ResourceUtil.getI18nMessage("60013"), templateContent);
            Log.info("服务计划{}发送站内信成功", servicePlanId);
        }

        if (StringUtils.isNotBlank(nxUserInfoEntity.getExternalReminders()) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(nxUserInfoEntity.getExternalReminders())
                && StringUtils.isNotBlank(nxUserInfoEntity.getSmsReminders()) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(nxUserInfoEntity.getSmsReminders())) {
            //todo 存在外部提醒,并且勾选了短信   根据设备编号查询相应的客户，发送短信，（暂时预留）

        }
    }


    public List<Map> returnByUserList(List<String> userList) {
        List<Map> listMap = new ArrayList<>();
        for (String userId : userList) {
            Map<String, Object> map = new HashMap<>();
            Map<String, String> stringStringMap = gbsjbDao.selectByUserId(userId);
            if (stringStringMap != null && !stringStringMap.isEmpty()) {
                map.put("language", "zh");
                //企业标识
                map.put("organizationId", "NX");
                //接收人名称
                map.put("receiveUser", stringStringMap.get("userName"));
                //接收人用户ID
                map.put("receiveUserId", stringStringMap.get("userId"));
                //接收类型的值
                map.put("receiveValue", stringStringMap.get("email"));
                listMap.add(map);
            }
        }
        return listMap;
    }


    /**
     * @param businessId   业务方ID
     * @param messageTitle 消息标题
     * @param messageType  消息类型
     * @param messageList  邮件内容
     * @param receiverList 接收人信息
     * @param content      内容
     * @return
     */
    public Map<String, Object> buildEmailParam(String businessId, String messageTitle, String messageType, String content, List<Map> messageList, List<Map> receiverList) {
        Map<String, Object> param = new HashMap<>();
        //业务方ID
        param.put("businessId", StringUtils.isNotBlank(businessId) ? businessId : "11");
        //消息标题
        param.put("messageTitle", StringUtils.isNotBlank(messageTitle) ? messageTitle : ResourceUtil.getI18nMessage("60013"));
        //消息类型
        param.put("messageType", StringUtils.isNotBlank(messageType) ? messageType : "ANNOUNCEMENT");
        List<Map> messagelist = new ArrayList<>();
        Map<String, Object> message = new HashMap<>();
        //邮件内容
        message.put("content", StringUtils.isNotBlank(content) ? content : "[iFSM]尊敬的【客户名称】您好，您编号为【设备编号】的设备工况【属性】已经【条件】【设置值】请及时跟进处理。");
        message.put("language", "zh");
        //邮件标题
        message.put("title", ResourceUtil.getI18nMessage("60013"));
        messagelist.add(message);
        if (messageList.isEmpty()) {
            param.put("messageContent", messagelist);
        } else {
            param.put("messageContent", messageList);
        }
        List<Map> list = new ArrayList<>();
        param.put("receiver", receiverList);
        //消息发送时间 0为即时发送
        param.put("sendTime", "0");
        return param;
    }


    @Override
    public boolean sendStationReminders(String userIds, String title, String context) {
        boolean flag = false;
        //JP推送消息地址与接口路径
        String host = wfProperties.getWfHost();
        String url = Properties.dwr_sender_uri;
        String[] userid = userIds.split(",");
        for (String id : userid) {
            UserInfo user = jeCoreEnduserDao.getUser(id);
            if (user == null) {
                continue;
            }
            //组装请求调用JP的发起站内信接口
            JSONObject params = new JSONObject();
            params.put("userId", id);
            params.put("title", title);
            params.put("context", user.getUserName() + "," + context);
            Map<String, String> headers = new HashMap<>();
            try {
                Map<String, String> querys = new HashMap<>();
                log.info("站内信，发送内容为==>{}", params.toJSONString());
                HttpResponse response = HttpUtils.doPost(host + url, "", "post", headers, querys, params.toJSONString());
                if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                    log.info("发送站内信成功，发送userid为==>{}", userIds);
                    log.info("发送站内信成功，请求地址为==>{}", host + url);
                    flag = true;
                }
                //站内信日志记录
                String[] split = userIds.split(",");
                //for (String userId:split) {
                JeSysUsermsgEntity usermsgEntity = new JeSysUsermsgEntity();
                usermsgEntity.setUsermsgJsrId(id);//接收人id
                usermsgEntity.setUsermsgYd("0");//默认已读
                usermsgEntity.setUsermsgNr(user.getUserName() + "," + context);//内容
                usermsgEntity.setUsermsgBt(ResourceUtil.getI18nMessage("60013"));//标题
                usermsgEntity.setUsermsgJssj(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()).toString());//接受时间
                usermsgEntity.setUsermsgJsrName(user.getUserName());//接收人姓名
                usermsgEntity.setUsermsgMsgtypeName(ResourceUtil.getI18nMessage("60013"));//用户消息类型_NAME
                usermsgEntity.setUsermsgMsgtypeCode("NX");//用户消息类型
                usermsgEntity.setJeSysUsermsgId(JEUuidUtil.uuid());//主键
                usermsgEntity.setSyCreatetime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()).toString());//创建时间
                LoginUserInfo current = UserContext.current();
                usermsgEntity.setSyCreateuser(current.getUserName());//当前登陆人姓名
                jeSysUsermsgDao.insert(usermsgEntity);
                // }
            } catch (Exception e) {
                log.info("发送站内信错误==>{}", e.getMessage());
                log.error("error", e);
            }
        }
        return flag;
    }

    @Override
    public boolean sendEmail(Map<String, Object> param) {
        boolean flag = false;
        HttpRequest request = HttpRequest.post(domain + emailUrl);
        request.header("Authorization", "Bearer " + tokenUtil.getToken());
        request.header("accept", "application/json");
        request.header("Content-Type", "application/json");
        log.info("发送邮件请求参数==>{}", JSON.toJSONString(param));
        request.body(JSON.toJSONString(param));
        cn.hutool.http.HttpResponse response = request.execute();
        if (response.getStatus() != cn.hutool.http.HttpStatus.HTTP_OK) {
            log.error("平台【发送邮件】接口出错:{}", response);
            /*throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                    StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());*/
            throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                    ResourceUtil.getI18nMessage("40000"));
        } else {
            flag = true;
        }
        log.info("发送邮件返回参数==>{}", response.body());
        return flag;
    }


    @Override
    public NxUserInfoVo selectByServicePlanId(String servicePlanId, String serviceCode, String syZhid) {
        NxUserInfoVo vo = new NxUserInfoVo();
        QueryWrapper<NxUserInfoEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("SERVICE_PLAN_ID", servicePlanId);
        NxUserInfoEntity nxUserInfoEntity = nxUserInfoMapper.selectOne(queryWrapper);
        if (!ObjectUtil.isEmpty(nxUserInfoEntity)) {
            BeanUtils.copyProperties(nxUserInfoEntity, vo);
            //把对应部门名称，用户名称，角色名称传进去
            try {
                if (StringUtils.isNotBlank(nxUserInfoEntity.getDepartmentId())) {
                    String deptName = nxUserInfoMapper.getDeptName(Arrays.asList(nxUserInfoEntity.getDepartmentId().split(",")));
                    vo.setDepartmentName(deptName);
                }
                if (StringUtils.isNotBlank(nxUserInfoEntity.getUserId())) {
                    String userName = nxUserInfoMapper.getUserName(Arrays.asList(nxUserInfoEntity.getUserId().split(",")));
                    vo.setUserName(userName);
                }
                if (StringUtils.isNotBlank(nxUserInfoEntity.getRoleId())) {
                    String roleName = nxUserInfoMapper.getRoleName(Arrays.asList(nxUserInfoEntity.getRoleId().split(",")));
                    vo.setRoleName(roleName);
                }
            } catch (Exception e) {
                log.info("查询部门角色错误信息：{}", e);
            }
            //预留外部提醒
            String externalReminders = nxUserInfoEntity.getExternalReminders();
            if (StringUtils.isNotBlank(externalReminders) && ServicePlanEnums.CheckType.CHECK_TURE.getCode().equals(externalReminders)) {
                //查询外部提醒
                NxUserInfoVo nxUserInfoVo = nxUserInfoMapper.selectBySeviceCode(serviceCode, syZhid);
                if (!ObjectUtil.isEmpty(nxUserInfoVo)) {
                    vo.setMainContact(nxUserInfoVo.getMainContact());
                    vo.setMainPhone(nxUserInfoVo.getMainPhone());
                }
            }
        }

        return vo;
    }


    @Override
    public int deleteByServicePlanId(String servicePlanId) {
        QueryWrapper<NxUserInfoEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("SERVICE_PLAN_ID", servicePlanId);
        return nxUserInfoMapper.delete(queryWrapper);
    }


    @Override
    public int saveNxUserInfo(NxUserInfoEntity nxUserInfoEntity) {
        return nxUserInfoMapper.insert(nxUserInfoEntity);
    }

    @Override
    public int updateNxUserInfo(NxUserInfoEntity nxUserInfoEntity) {
        QueryWrapper<NxUserInfoEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("SERVICE_PLAN_ID", nxUserInfoEntity.getServicePlanId());
        return nxUserInfoMapper.update(nxUserInfoEntity, queryWrapper);
    }


    @Override
    public List<NxUserInfoEntity> queryUserInfoByPlanIds(List<String> servicePlanIds) {
        QueryWrapper<NxUserInfoEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("SERVICE_PLAN_ID", servicePlanIds);
        return nxUserInfoMapper.selectList(queryWrapper);
    }

    @Override
    public String getTemplateContent(String servicePlanId) {
        String templateContent = null;
        ServicePlanEntity servicePlan = servicePlanMapper.selectById(servicePlanId);

        QueryWrapper<NxUserInfoEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("SERVICE_PLAN_ID", servicePlanId);
        queryWrapper.last("limit 1");
        NxUserInfoEntity nxUserInfoEntity = nxUserInfoMapper.selectOne(queryWrapper);

        QueryWrapper<NxTriggerConditionEntity> entityQueryWrapper = new QueryWrapper<>();
        entityQueryWrapper.eq("SERVICE_PLAN_ID", servicePlanId);
        entityQueryWrapper.last("limit 1");
        NxTriggerConditionEntity triggerConditionEntity = nxTriggerConditionMapper.selectOne(entityQueryWrapper);

        if (!ObjectUtil.isEmpty(servicePlan)) {
            if (!ObjectUtil.isEmpty(nxUserInfoEntity)) {
                String infoContent = nxUserInfoEntity.getInfoContent();
                if (StringUtils.isNotBlank(infoContent)) {
                    //【设备编码】【计划名称】【计划类型】【计划周期】【计划间隔】
                    // 【计划时间段】【内部提醒】【触发类型】【选择条件】【提醒方式】
                    // 【外部提醒】【条件值】【设置值】【完成的时间节点】【选择保养内容】
                    infoContent = infoContent.replace("【设备编码】", servicePlan.getDeviceCode());
                    infoContent = infoContent.replace("【计划名称】", servicePlan.getPlanName());
                    infoContent = infoContent.replace("【计划类型】", servicePlan.getPlanTypeName());
                    infoContent = infoContent.replace("【计划周期】", servicePlan.getPlanCycleName());
                    infoContent = infoContent.replace("【计划间隔】", servicePlan.getPlanInterval() + "");
                    infoContent = infoContent.replace("【计划时间段】", StringUtils.isNotBlank(servicePlan.getPlanStartTime()) ? servicePlan.getPlanStartTime() : "" + "~" + (StringUtils.isNotBlank(servicePlan.getPlanEndTime()) ? servicePlan.getPlanEndTime() : ""));
                    /*infoContent = infoContent.replace("【内部提醒】", StringUtils.isNotBlank(nxUserInfoEntity.getDepartmentIdName())?nxUserInfoEntity.getDepartmentIdName():""
                            +(StringUtils.isNotBlank(nxUserInfoEntity.getUserIdName())?nxUserInfoEntity.getUserIdName():"")
                            +(StringUtils.isNotBlank(nxUserInfoEntity.getRoleIdName())?nxUserInfoEntity.getRoleIdName():""));*/
                    infoContent = infoContent.replace("【内部提醒】", "");
                    infoContent = infoContent.replace("【提醒方式】", servicePlan.getPlanRemindWayName());

                    //查询外部提醒
                    NxUserInfoVo nxUserInfoVo = nxUserInfoMapper.selectBySeviceCode(servicePlan.getDeviceCode(), servicePlan.getSyZhid());
                    if (!ObjectUtil.isEmpty(nxUserInfoVo)) {
                        infoContent = infoContent.replace("【外部提醒】", nxUserInfoVo.getMainContact());
                    } else {
                        infoContent = infoContent.replace("【外部提醒】", "");
                    }

                    infoContent = infoContent.replace("【完成的时间节点】", servicePlan.getFinishTimeNode() + "");
                    infoContent = infoContent.replace("【选择保养内容】", servicePlan.getMaintainContentName());
                }
                if (!ObjectUtil.isEmpty(triggerConditionEntity)) {
                    infoContent = infoContent.replace("【触发类型】", triggerConditionEntity.getTriggerConditionName());
                    infoContent = infoContent.replace("【选择条件】", triggerConditionEntity.getConditionTjName());
                    infoContent = infoContent.replace("【条件值】", triggerConditionEntity.getConditionValueName());
                    infoContent = infoContent.replace("【设置值】", triggerConditionEntity.getSetValueName());
                } else {
                    infoContent = infoContent.replace("【触发类型】", "");
                    infoContent = infoContent.replace("【选择条件】", "");
                    infoContent = infoContent.replace("【条件值】", "");
                    infoContent = infoContent.replace("【设置值】", "");
                }
                templateContent = infoContent;
            }
        }
        return templateContent;
    }


    @Override
    public List<String> getCustomerAndRoles(String deviceCode) {
        NxJeDeviceEntity nxJeDeviceEntity = nxJeDeviceService.selectBySbbm(deviceCode);
        if (!ObjectUtils.isEmpty(nxJeDeviceEntity) && StringUtils.isNotBlank(nxJeDeviceEntity.getDeviceSskh())) {
            List<String> customerAndRoles = servicePlanMapper.getCustomerAndRoles(nxJeDeviceEntity.getDeviceSskh(), UserContext.getUserTenantId());
            return customerAndRoles;
        }
        return new ArrayList<>();
    }
}
