package com.rootcloud.domain.service.sp.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpStatus;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.*;
import com.rootcloud.common.constant.Properties;
import com.rootcloud.common.enums.OrderEnums;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.common.util.CommonUtils;
import com.rootcloud.common.util.DateUtils;
import com.rootcloud.common.util.OrderNumberUtils;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.ao.ServicePlanAO;
import com.rootcloud.domain.ao.ServicePlanCalendarAO;
import com.rootcloud.domain.ao.ServicePlanCalendarNewAO;
import com.rootcloud.domain.ao.ServicePlanParamsAO;
import com.rootcloud.domain.dao.sp.ServicePlanDao;
import com.rootcloud.domain.dto.ConditionModel;
import com.rootcloud.domain.entity.*;
import com.rootcloud.domain.entity.device.NxDeviceEntity;
import com.rootcloud.domain.entity.ds.ModelPropertySyncEntity;
import com.rootcloud.domain.entity.sp.NxTriggerConditionEntity;
import com.rootcloud.domain.entity.sp.NxUserInfoEntity;
import com.rootcloud.domain.entity.sp.ServicePlanEntity;
import com.rootcloud.domain.entity.sp.ServicePlanHistoryEntity;
import com.rootcloud.domain.mapper.NxDeviceMapper;
import com.rootcloud.domain.mapper.NxOrderMapper;
import com.rootcloud.domain.mapper.ServicePlanMapper;
import com.rootcloud.domain.service.auth.RootCloudAuthService;
import com.rootcloud.domain.service.ds.ModelPropertySyncService;
import com.rootcloud.domain.service.gkjk.NxJeDeviceService;
import com.rootcloud.domain.service.network.NetworkUserService;
import com.rootcloud.domain.service.sp.NxTriggerConditionService;
import com.rootcloud.domain.service.sp.NxUserInfoService;
import com.rootcloud.domain.service.sp.ServicePlanHistoryService;
import com.rootcloud.domain.service.sp.ServicePlanService;
import com.rootcloud.domain.service.todo.NxTodoLibraryService;
import com.rootcloud.vo.device.ServicePlanCalendarDetailsVO;
import com.rootcloud.vo.device.ServicePlanCalendarList;
import com.rootcloud.vo.device.ServicePlanCalendarVO;
import com.rootcloud.vo.device.ServicePlanDetailVO;
import com.rootcloud.vo.order.OrderLcinfoParam;
import com.rootcloud.vo.service.*;
import jline.internal.Log;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.util.EntityUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.StandardCharsets;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName ServicePlanServiceImpl
 * @Description 服务计划实现类
 * @Author w-liuyi01
 * @Date 2022/4/28 20:02
 * @Version 1.0
 */
@SuppressWarnings("LanguageDetectionInspection")
@Slf4j
@Service("servicePlanServiceImpl")
public class ServicePlanServiceImpl extends ServiceImpl<ServicePlanDao, ServicePlanEntity> implements ServicePlanService {

    private final NxTriggerConditionService nxTriggerConditionService;
    private final NxJeDeviceService nxJeDeviceService;
    private final ModelPropertySyncService modelPropertySyncService;
    private final TokenUtil tokenUtil;
    private final NxUserInfoService nxUserInfoService;
    private final NxOrderMapper nxOrderMapper;
    private final NxTodoLibraryService nxTodoLibraryService;
    private final NxDeviceMapper nxDeviceMapper;
    private final RootCloudAuthService rootCloudAuthService;
    @Autowired
    ServicePlanMapper servicePlanMapper;
    @Autowired
    NetworkUserService networkUserService;
    @Autowired
    Properties wfProperties;
    @Autowired
    Request request;
    @Value("${os.iam.domain}")
    private String domain;
    @Value("${cms.codealarm.uri}")
    private String codeAlarmUri;
    @Value("${cms.codealarmbatch.uri}")
    private String codeAlarmBatchUrl;
    @Resource(name = "servicePlanHistoryServiceImpl")
    private ServicePlanHistoryService servicePlanHistoryService;

    public ServicePlanServiceImpl(@Qualifier("nxTriggerConditionServiceImpl") NxTriggerConditionService nxTriggerConditionService,
                                  @Qualifier("nxJeDeviceServiceImpl") NxJeDeviceService nxJeDeviceService,
                                  @Qualifier("modelPropertySyncService2") ModelPropertySyncService modelPropertySyncService,
                                  TokenUtil tokenUtil, NxOrderMapper nxOrderMapper,
                                  @Qualifier("NxUserInfoServiceImpl") NxUserInfoService nxUserInfoService,
                                  @Qualifier("nxTodoLibraryServiceImpl") NxTodoLibraryService nxTodoLibraryService,
                                  @Qualifier("cloudAuthService") RootCloudAuthService rootCloudAuthService,
                                  NxDeviceMapper nxDeviceMapper) {
        this.nxTriggerConditionService = nxTriggerConditionService;
        this.nxJeDeviceService = nxJeDeviceService;
        this.modelPropertySyncService = modelPropertySyncService;
        this.tokenUtil = tokenUtil;
        this.nxOrderMapper = nxOrderMapper;
        this.nxUserInfoService = nxUserInfoService;
        this.nxTodoLibraryService = nxTodoLibraryService;
        this.rootCloudAuthService = rootCloudAuthService;
        this.nxDeviceMapper = nxDeviceMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult addPlan(ServicePlanAO planAO) {
        //新增基础服务计划
        String planId = buildServicePlan(planAO);

        if (StrUtil.isNotBlank(planAO.getServicePlanId()) && planAO.getConditionAO() != null
                && StrUtil.isNotBlank(planAO.getConditionAO().getNxTriggerConditionId())) {
            nxTriggerConditionService.update(planAO.getConditionAO(), planId, planAO.getDeviceCode());
        } else {
            // 添加触发条件
            nxTriggerConditionService.add(planAO.getConditionAO(), planId, planAO.getDeviceCode());
        }

        //如果选择系统时间则需要将基础服务中的基础时间同步为触发条件中所选时间
//        if (ServicePlanEnums.TriggerType.YSY_TIME.getCode().equals(conditionEntity.getTriggerCondition())){
//            LambdaUpdateWrapper<ServicePlanEntity> lambda = new UpdateWrapper<ServicePlanEntity>().lambda();
//            lambda.set(ServicePlanEntity::getPlanStartTime, conditionEntity.getSetValue())
//                    .eq(ServicePlanEntity::getServicePlanId, planId);
//            this.update(lambda);
//        }
        //提醒用户信息添加

        NxUserInfoEntity userInfo = new NxUserInfoEntity();
        BeanUtils.copyProperties(planAO.getUserInfoAO(), userInfo);
        //判断是新增还修改
        if (StringUtils.isNotBlank(planAO.getUserInfoAO().getNxUserInfoId())) {
            nxUserInfoService.updateNxUserInfo(userInfo);
        } else {
            userInfo.setServicePlanId(planId);
            userInfo.setNxUserInfoId(JEUuidUtil.uuid());
            userInfo.setCreateTime(DateUtil.getSysdateStr());
            nxUserInfoService.saveNxUserInfo(userInfo);
        }
        return JsonResult.success();
    }

    /**
     * @return java.lang.String
     * @Author w-liuyi01
     * @Description //基础信息插入
     * @Date 2022/4/29 16:29
     * @Param [planAO]
     **/
    private String buildServicePlan(ServicePlanAO planAO) {
        ServicePlanEntity planEntity = new ServicePlanEntity();
        BeanUtils.copyProperties(planAO, planEntity);
        if (StringUtils.isNotBlank(planAO.getServicePlanId())) {
            //获取旧数据保存到历史表中
            ServicePlanEntity historyInfo = servicePlanMapper.selectById(planAO.getServicePlanId());
            ServicePlanHistoryEntity servicePlanHistoryEntity = new ServicePlanHistoryEntity();
            servicePlanHistoryEntity.setId(CommonUtils.uuid());
            servicePlanHistoryEntity.setContent(JSON.toJSONString(historyInfo));
            servicePlanHistoryEntity.setServicePlanId(planAO.getServicePlanId());
            servicePlanHistoryEntity.setOperator(UserContext.getUserId());
            servicePlanHistoryEntity.setOperatorTime(DateUtils.format(new Date(), DateUtils.C_TIME_PATTON_DEFAULT));
            servicePlanHistoryService.save(servicePlanHistoryEntity);
            //修改下一个周期时间
            try {
                String nextTime = getNextDateByStartDateNew(planEntity);
                planEntity.setNextCycleTime(nextTime);
            } catch (Exception e) {
                Log.error("设置下一个计划周期失败，失败原因：" + e.getMessage());
                e.printStackTrace();
            }
            //修改服务计划数据
            servicePlanMapper.updateById(planEntity);
        } else {
            planEntity.setDetailStatus(ServicePlanEnums.DetailStatus.DEFAULT.getCode());
            planEntity.setCreateTime(DateUtil.getSysdateStr());
            planEntity.setNextCycleTime(planEntity.getPlanStartTime());
            planEntity.setServicePlanId(JEUuidUtil.uuid());
            planEntity.setSyZhid(UserContext.getUserTenantId());
            servicePlanMapper.insert(planEntity);

        }
        return planEntity.getServicePlanId();
    }

    @Override
    public JsonResult startServicePlan(String servicePlanId) {
        ServicePlanEntity servicePlanEntity = servicePlanMapper.selectById(servicePlanId);
        return JsonResult.success();
    }

    @Override
    public List<ModelPropertySyncEntity> getWorkCondition(String deviceNo) {
        String modelId = this.getModelIdByDeviceNo(deviceNo);
        return modelPropertySyncService.queryListByModelId(modelId);
    }

    @Override
    public String getModelIdByDeviceNo(String deviceNo) {
        String syZhid = UserContext.getTenantId();
        NxJeDeviceEntity device = nxJeDeviceService
                .getOne(Wrappers.lambdaQuery(NxJeDeviceEntity.class).eq(NxJeDeviceEntity::getDeviceSbbm, deviceNo).eq(NxJeDeviceEntity::getSyZhid, syZhid));
        Optional.ofNullable(device)
                .orElseThrow(() -> new BusinessException(StatusCodeEnum.EXCEPTION.getRet(), ResourceUtil.getI18nMessage("13500")));
        return device.getModelId();
    }


    @RabbitListener(queues = "base.queue.nxgx.alarm")
    public void getMessage(Message message) {
        String body = new String(message.getBody(), StandardCharsets.UTF_8);
        Log.info("工况消费者监听消息：{}", body);
        JSONObject result = JSONObject.parseObject(body);
        JSONObject payload = JSONObject.parseObject(result.getString("payload"));
        if (payload == null) {
            return;
        }
        String typeId = payload.getString("alarmTopicId");
        if (StrUtil.isBlank(typeId)) {
            return;
        }

        String now = DateUtil.format(new Date());
        // 查询出所有在时间范围内，然后状态在进行中，并且类型是工况的数据
        List<ConditionModel> conditionList = servicePlanMapper.findConditionList(now,
                ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode(), ServicePlanEnums.TriggerType.WORK_CONDITION.getCode(),
                ServicePlanEnums.DetailStatus.DEFAULT.getCode(), typeId);
        if (CollectionUtil.isEmpty(conditionList)) {
            //Log.info("工况触发条件list为空");
            return;
        }
        List<String> planIdList =
                conditionList.stream().map(ConditionModel::getServicePlanId).collect(Collectors.toList());
        // 修改状态为待发送
        servicePlanMapper.updateDetailStatusByPlanIds(planIdList, ServicePlanEnums.DetailStatus.DETAIL.getCode());
    }

//    @Scheduled(cron = "${nx.plan.query:0 0/15 * * * ? }")

    public void queryAlarmQz() {
        // 查询到所有的台账
        List<NxJeDeviceEntity> devices = nxJeDeviceService.queryAllJeDevice();
        List<List<NxJeDeviceEntity>> lists = splitListByCount(devices, 50);
        for (List<NxJeDeviceEntity> list : lists) {
            queryAlarm(list);
        }

    }

    public List<List<NxJeDeviceEntity>> splitListByCount(List<NxJeDeviceEntity> list, int count) {
        List<List<NxJeDeviceEntity>> listAll = new ArrayList<>();
        int size = list.size();
        if (size > count) {
            int absInt = Math.abs(size / count);
            if (size - absInt * count > 0) {
                listAll.add(list.subList(absInt * count, size));
            }
            for (int i = 1; i < absInt + 1; i++) {
                listAll.add(list.subList((i - 1) * count, i * count));
            }
        } else {
            listAll.add(list);
        }
        return listAll;
    }

    public void queryAlarm(List<NxJeDeviceEntity> devices) {


        Map<Object, List<NxJeDeviceEntity>> map = devices.stream().filter(x -> StringUtils.isNotBlank(x.getSyZhid())).collect(Collectors.toList()).stream().collect(Collectors.groupingBy(b -> b.getSyZhid()));
        for (Object zhid : map.keySet()) {

            List<String> thingIds = map.get(zhid.toString()).stream().map(NxJeDeviceEntity::getNxJeDeviceId).collect(Collectors.toList());

            HttpRequest request = HttpRequest.post(domain + codeAlarmBatchUrl);
            request.header("Authorization", "Bearer " + tokenUtil.getToken(zhid.toString()));
            request.header("accept", "application/json");
            request.header("Content-Type", "application/json");
            Map<String, Object> param = new HashMap<>(1);
            param.put("thingIds", thingIds);
            String jsonString = JSON.toJSONString(param);
            //Log.info("【查询报警信息】请求参数 ==> {}", jsonString);
            request.body(jsonString);
            HttpResponse response = request.execute();


            if (response.getStatus() != HttpStatus.HTTP_OK) {
                Log.error("调用平台【查询报警信息】接口出错：{}", response);
                // throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                //   StatusCodeEnum.FAIL_CALL_INTERFACE.getMessage());
                throw new BusinessException(StatusCodeEnum.FAIL_CALL_INTERFACE.getRet(),
                        ResourceUtil.getI18nMessage("40000"));
            }

            JSONObject resultJson = JSONObject.parseObject(response.body());
            // Log.info("调用平台【查询报警信息】返回结果 ==> {}", resultJson);
            JSONArray payloadArray = resultJson.getJSONArray("payload");
            if (payloadArray.isEmpty()) {
                Log.info("查询报警信息【payload】节点为空，==> {}", resultJson);
                return;
            }

            // 获取到所有的故障码
            List<String> codeList = payloadArray.stream().map(item -> {
                JSONObject payloadJson = (JSONObject) item;
                return payloadJson.getString("code");
            }).distinct().collect(Collectors.toList());

            // 查询出所有的报警等级
            List<NxZskFaultCodeEntity> alarmLevel = servicePlanMapper.selectAlarmLevel(codeList);
            if (CollectionUtil.isEmpty(alarmLevel)) {
                return;
            }

            String now = DateUtil.format(new Date());
            // 查询出所有时间范围内，并且是进行中，类型为报警的触发条件
            List<ConditionModel> conditionList =
                    servicePlanMapper.findConditionList(now, ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode(),
                            ServicePlanEnums.TriggerType.ALARM.getCode(), ServicePlanEnums.DetailStatus.DEFAULT.getCode(), null);
            if (CollectionUtil.isEmpty(conditionList)) {
                Log.info("报警触发条件list为空");
                return;
            }

            List<String> planIdList = conditionList.stream().filter(v -> {

                // 等级越小，值越大
                // 任意一个等级值大于所设置的值，条件设置为【<】都满足条件
                // 任意一个等级值小于所设置的值，条件设置为【>】都满足条件
                if (ServicePlanEnums.Condition.GT.getCode().equals(v.getConditionValue())) {
                    int value = Integer.parseInt(v.getSetValue());
                    for (NxZskFaultCodeEntity fault : alarmLevel) {
                        try {
                            int level = Integer.parseInt(fault.getFaultcodeGzdjcode());
                            // 平台发送过来的等级值只要有一个值小于设置的值，说明就有等级大于设置的等级，就进行发短信等操作；
                            if (level < value && v.getDeviceTypeCode().equals(fault.getFaultcodeSblxcode())) {
                                return true;
                            }
                        } catch (Exception e) {
                            continue;
                        }
                    }

                }

                if (ServicePlanEnums.Condition.LT.getCode().equals(v.getConditionValue())) {
                    int value = Integer.parseInt(v.getSetValue());
                    for (NxZskFaultCodeEntity fault : alarmLevel) {
                        try {
                            int level = Integer.parseInt(fault.getFaultcodeGzdjcode());
                            // 平台发送过来的等级值只要有一个值大于设置的值，说明就有等级小于设置的等级，就进行发短信等操作；
                            if (level > value && v.getDeviceTypeCode().equals(fault.getFaultcodeSblxcode())) {
                                return true;
                            }
                        } catch (Exception e) {
                            continue;
                        }
                    }
                }

                if (ServicePlanEnums.Condition.EQ.getCode().equals(v.getConditionValue())) {
                    int value = Integer.parseInt(v.getSetValue());
                    for (NxZskFaultCodeEntity fault : alarmLevel) {
                        try {
                            int level = Integer.parseInt(fault.getFaultcodeGzdjcode());
                            if (level == value && v.getDeviceTypeCode().equals(fault.getFaultcodeSblxcode())) {
                                return true;
                            }
                        } catch (Exception e) {
                            continue;
                        }
                    }
                }
                // 都不满足就去掉这条数据
                return false;
            }).map(ConditionModel::getServicePlanId).collect(Collectors.toList());
            if (CollectionUtil.isEmpty(planIdList)) {
                return;
            }
            // 修改状态为待发送
            servicePlanMapper.updateDetailStatusByPlanIds(planIdList, ServicePlanEnums.DetailStatus.DETAIL.getCode());
        }
    }

    //@Scheduled(cron = "${nx.plan.execute:0 0 10 * * ? }")
    @Override
    public void execute() {
        List<ServicePlanEntity> servicePlanEntities = servicePlanMapper.selectScheduledTasksList();
        List<String> ids = new ArrayList<>();
        if (!servicePlanEntities.isEmpty()) {
            for (ServicePlanEntity servicePlan : servicePlanEntities) {
                ids.add(servicePlan.getServicePlanId());
                nxUserInfoService.setReminders(servicePlan.getServicePlanId());
                //如果发送成功修改状态
                updateServicePlanStatus(servicePlan.getServicePlanId(), 0, ServicePlanEnums.PlanStatus.FINISHED.getCode(), false, null);
            }
            servicePlanMapper.updateStatus(ids, ServicePlanEnums.PlanStatus.FINISHED,
                    ServicePlanEnums.DetailStatus.DEFAULT.getCode());
        }
    }

    @Override
    public ServicePlanDetailVO selectDetailById(String servicePlanId) {
        ServicePlanDetailVO servicePlanVo = new ServicePlanDetailVO();
        ServicePlanEntity servicePlanEntity = servicePlanMapper.selectById(servicePlanId);
        if (ObjectUtil.isEmpty(servicePlanEntity)) {
            // throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getRet(),StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("16001"));
        }

        BeanUtils.copyProperties(servicePlanEntity, servicePlanVo);
        //获取设备id
        LambdaQueryWrapper<NxDeviceEntity> nxDeviceEntityLambdaQueryWrapper = new LambdaQueryWrapper<>();
        nxDeviceEntityLambdaQueryWrapper.eq(NxDeviceEntity::getSblbSbbm, servicePlanEntity.getDeviceCode());
        nxDeviceEntityLambdaQueryWrapper.eq(NxDeviceEntity::getSyZhid, servicePlanEntity.getSyZhid());
        NxDeviceEntity nxDeviceEntity = nxDeviceMapper.selectOne(nxDeviceEntityLambdaQueryWrapper);
        if (nxDeviceEntity != null) {
            servicePlanVo.setDeviceId(nxDeviceEntity.getNxJcxxSblbId());
        }
        if (StringUtils.isNotBlank(servicePlanVo.getMaintainContentName())) {
            //List<MaintenanceListVo> maintenanceListVos = servicePlanMapper.selectListByName(servicePlanVo.getMaintainContentName());
            List<MaintenanceListVo> maintenanceListVos = servicePlanMapper.selectListByName2(servicePlanVo.getMaintainContentName(), servicePlanVo.getMaintainContent());
            servicePlanVo.setMaintenanceListVo(maintenanceListVos);
        }

        NxTriggerConditionEntity condition =
                nxTriggerConditionService.getOne(Wrappers.lambdaQuery(NxTriggerConditionEntity.class)
                        .eq(NxTriggerConditionEntity::getServicePlanId, servicePlanId).last("limit 1"));
        servicePlanVo.setCondition(condition);
        NxUserInfoVo nxUserInfoVo = nxUserInfoService.selectByServicePlanId(servicePlanId, servicePlanEntity.getDeviceCode(), servicePlanEntity.getSyZhid());
        servicePlanVo.setNxUserInfoVo(nxUserInfoVo);
        return servicePlanVo;
    }

    @Override
    public int deleteServicePlan(String servicePlanId) {
        QueryWrapper<ServicePlanEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("SERVICE_PLAN_ID", servicePlanId);
        ServicePlanEntity servicePlanEntity = servicePlanMapper.selectOne(queryWrapper);
        if (ObjectUtil.isEmpty(servicePlanEntity)) {
            //throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getRet(),StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("16001"));
        }
        if (ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode().equals(servicePlanEntity.getPlanStatus())) {
            //throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_DELETE.getRet(),StatusCodeEnum.SERVICE_PLAN_NOT_DELETE.getMessage());
            throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_DELETE.getRet(), ResourceUtil.getI18nMessage("16002"));
        }
        //删除触发条件
        nxTriggerConditionService.removeByPlanId(servicePlanId, servicePlanEntity.getDeviceCode());

        nxUserInfoService.deleteByServicePlanId(servicePlanId);

        servicePlanEntity.setSyStatus("0");
        int i = servicePlanMapper.updateById(servicePlanEntity);
        //int i = servicePlanMapper.deleteById(servicePlanId);
        return i;
    }


    @Override
    public void scheduledProcessing() {
        //获取当天日期
        String thatDay = DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE);
        List<ServicePlanVo> servicePlanVos = servicePlanMapper.selectExecutedServicePlan(thatDay);
        log.info("服务计划执行日期：{},总执行条数：{}" , thatDay,servicePlanVos.size());
        if (servicePlanVos.size() > 0) {
            for (ServicePlanVo item : servicePlanVos) {
                try {
                    log.info("服务计划ID:{}已进入执行", item.getServicePlanId());
                    //判断是否临时触发时间的服务计划
                    boolean tempTime = false;
                    if (StringUtils.isNotBlank(item.getPlanLscfsj())) {
                        tempTime = true;
                        log.info("服务计划:{}为临时触发计划",item.getServicePlanId());
                    }
                    QueryWrapper<ServicePlanEntity> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("SERVICE_PLAN_ID", item.getServicePlanId());
                    ServicePlanEntity servicePlanEntity = servicePlanMapper.selectOne(queryWrapper);
                    //如果是自动触发则需要判定当前日期所在的周期内是否有存在服务计划订单
                    //todo 2024-12-27 新增逻辑：由于原有逻辑在查单时无法准确判断已生成的工单是当次延迟工单还是下次提前工单 导致此段逻辑异常
                    //异常表现为：如果查到的是当月延迟的工单会导致下次工单不生成
                    //新逻辑为：订单表增加一个字段service_plan_order_status，这个字段在临时触发时间生效，生成工单时，进行赋值，赋值为after
                    //当下次定时任务生成开始生成工单时，查询到有工单后原逻辑会不生成，现逻辑改为查询时筛选掉是after的工单后，如果还存在工单，那么工单就是手动提前触发了下次的工单，此时不生成，否则生成
                    String startDate = getCurrentDateStartDate(servicePlanEntity, thatDay);
                    String endDate = thatDay;
                    if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
                        Integer count = servicePlanMapper.checkOrderCount(servicePlanEntity.getServicePlanId(), startDate, endDate);
                        if (count > 0) {
                            log.info("该服务计划{}当前周期：{}>>>{}内，已生成订单...", item.getServicePlanId(), startDate, endDate);
                            //本次周期提前触发过 调整下次触发时间
                            String nextTime = getNextDateByStartDateNew(servicePlanEntity);
                            servicePlanEntity.setNextCycleTime(nextTime);
                            servicePlanMapper.update(servicePlanEntity, queryWrapper);
                            continue;
                        }
                    }
                    //根据服务计划和开始日期查询代办库，看服务计划是否执行生成了订单
                    Integer count = servicePlanMapper.checkOrder(item.getServicePlanId(), thatDay);
                    if (count > 0) {
                        log.info("该服务计划{}当天已生成订单...", item.getServicePlanId());
                    } else {
                        //发送
                        nxUserInfoService.setReminders(item.getServicePlanId());
                        //如果发送成功修改状态
                        updateServicePlanStatus(item.getServicePlanId(), 0, ServicePlanEnums.PlanStatus.FINISHED.getCode(), tempTime, thatDay);
                    }
                } catch (Exception e) {
                    log.info("发送以及修改状态错误信息:{}", e);
                }
            }
        }
    }

    @Override
    public void scheduledProcessingNew(String thatDay, String planId) {
        if (StringUtils.isNotBlank(thatDay)) {
            //获取当天日期
            Log.info("服务计划执行日期：" + thatDay);
            List<ServicePlanVo> servicePlanVos = servicePlanMapper.selectExecutedServicePlan(thatDay);
            if (servicePlanVos.size() > 0) {
                for (ServicePlanVo item : servicePlanVos) {
                    try {
                        Log.info("服务计划：" + item);
                        //判断是否临时触发时间的服务计划
                        boolean tempTime = false;
                        if (StringUtils.isNotBlank(item.getPlanLscfsj())) {
                            tempTime = true;
                        }
                        //根据服务计划和开始日期查询代办库，看服务计划是否执行生成了订单
                        Integer count = servicePlanMapper.checkOrder(item.getServicePlanId(), thatDay);
                        if (count > 0) {
                            Log.info("该服务计划当天已生成订单...");
                            continue;
                        } else {
                            //发送
                            nxUserInfoService.setReminders(item.getServicePlanId());
                            //如果发送成功修改状态
                            updateServicePlanStatus(item.getServicePlanId(), 0, ServicePlanEnums.PlanStatus.FINISHED.getCode(), tempTime, thatDay);
                        }
                    } catch (Exception e) {
                        Log.info("发送以及修改状态错误信息:{}", e);
                    }
                }
            }
        }
        //单个执行
        if (StringUtils.isNotBlank(planId) && StringUtils.isNotBlank(thatDay)) {
            List<ServicePlanVo> servicePlanVos = servicePlanMapper.selectExecutedServicePlanByPlanId(planId);
            if (servicePlanVos.size() > 0) {
                for (ServicePlanVo item : servicePlanVos) {
                    try {
                        Log.info("服务计划：" + item);
                        //判断是否临时触发时间的服务计划
                        boolean tempTime = false;
                        if (StringUtils.isNotBlank(item.getPlanLscfsj())) {
                            tempTime = true;
                        }
                        //根据服务计划和开始日期查询代办库，看服务计划是否执行生成了订单
                        Integer count = servicePlanMapper.checkOrder(item.getServicePlanId(), thatDay);
                        if (count > 0) {
                            Log.info("该服务计划当天已生成订单...");
                            continue;
                        } else {
                            //发送
                            nxUserInfoService.setReminders(item.getServicePlanId());
                            //如果发送成功修改状态
                            updateServicePlanStatus(item.getServicePlanId(), 0, ServicePlanEnums.PlanStatus.FINISHED.getCode(), tempTime, thatDay);
                        }
                    } catch (Exception e) {
                        Log.info("发送以及修改状态错误信息:{}", e);
                    }
                }
            }
        }
    }

    @SneakyThrows
    @Override
    public void updateStatusScheduled() {
        //所有进行中和 已经完成状态的， 进行中的  看看是否在时间周期之内  如果不在 则修改成已完成状态
        ServicePlanParamsAO ao = new ServicePlanParamsAO();
        ao.setPlanStatus(ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode());
        List<ServicePlanEntity> servicePlanEntities = servicePlanMapper.selectListByParams(ao);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        if (!servicePlanEntities.isEmpty()) {
            servicePlanEntities.stream().forEach(item -> {
                try {
                    if (StringUtils.isNotBlank(item.getPlanEndTime())
                            && LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")).compareTo(sdf.format(sdf.parse(item.getPlanEndTime()))) >= 0) {
                        updateServicePlanStatus(item.getServicePlanId(), 0, ServicePlanEnums.PlanStatus.FINISHED.getCode(), false, null);
                    }
                } catch (ParseException e) {
                    Log.info("错误信息={}", e);
                }
            });
        }

        //完成状态  看看是否在时间段之内
        ao.setPlanStatus(ServicePlanEnums.PlanStatus.FINISHED.getCode());
        ao.setSysTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        ao.setPlanCycle("planCycle");
        List<ServicePlanEntity> finishEntities = servicePlanMapper.selectListByParams(ao);
        if (!finishEntities.isEmpty()) {
            finishEntities.stream().forEach(item -> {
                if (item.getPlanCycle() == null) {
                    //无周期 直接改
                    updateServicePlanStatus(item.getServicePlanId(), 0, ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode(), false, null);
                } else {
                    //有周期 特殊处理周期
                    //查询有没有下一周期值
                    //如果没有下一周期值   直接修改为进行中  并且计算下一周期值修改进去
                    //如果有下一周期值     判断是不是当前时间是不是大于等于这个时间  如果是的进行修改为进行中  并且计算下一周期的值修改进去，因为要提前一天触发，所护
                    if (StringUtils.isNotBlank(item.getNextCycleTime())) {
                        try {
                            if (LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")).compareTo(sdf.format(DateUtils.addDays(sdf.parse(item.getNextCycleTime()), -1))) < 0) {
                                return;
                            }
                        } catch (ParseException e) {
                            Log.info("错误信息={}", e);
                            return;
                        }
                    }
                    //修改状态以及下一个周期时间
                    updateServicePlanStatus(item.getServicePlanId(), 0, ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode(), false, null);
                }
            });
        }
    }

    @Override
    @SneakyThrows
    public boolean updateServicePlanStatus(String servicePlanId, int triggerType, Integer servicePlanStatus, boolean tempTime, String thatDay) {
        log.info("服务计划{}自动触发修改计划状态", servicePlanId);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        QueryWrapper<ServicePlanEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("SERVICE_PLAN_ID", servicePlanId);
        ServicePlanEntity servicePlanEntity = servicePlanMapper.selectOne(queryWrapper);
        if (ObjectUtil.isEmpty(servicePlanEntity)) {
            log.info("{}16001异常", servicePlanId);
            //throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getRet(),StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("16001"));
        }
        if (ServicePlanEnums.PlanStatus.PAUSED.getCode().equals(servicePlanStatus)) {
            //停止
            if (ServicePlanEnums.PlanStatus.PAUSED.getCode().equals(servicePlanEntity.getPlanStatus())) {
                log.info("{}16003异常", servicePlanId);
                //throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_STOP.getRet(),StatusCodeEnum.SERVICE_PLAN_NOT_STOP.getMessage());
                throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_STOP.getRet(), ResourceUtil.getI18nMessage("16003"));
            }
            servicePlanEntity.setPlanStatusName("已暂停");
        }
        if (ServicePlanEnums.PlanStatus.FINISHED.getCode().equals(servicePlanStatus)) {
            //已完成的状态如果订单为空的可以手动触发
            if (!ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode().equals(servicePlanEntity.getPlanStatus()) && !ServicePlanEnums.PlanStatus.FINISHED.getCode().equals(servicePlanEntity.getPlanStatus())) {
                log.info("{}16004异常", servicePlanId);
                throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_FINISH.getRet(), ResourceUtil.getI18nMessage("16004"));
            }
            servicePlanEntity.setPlanStatusName("已完成");
            //已经完成 时间没有超过时间范围  就更新最新触发时间;
            if (StringUtils.isBlank(servicePlanEntity.getPlanEndTime())) {
                servicePlanEntity.setLatestTriggerTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            } else {
                try {
                    if (LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")).compareTo(sdf.format(sdf.parse(servicePlanEntity.getPlanEndTime()))) <= 0) {
                        servicePlanEntity.setLatestTriggerTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
                    }
                } catch (ParseException e) {
                    log.info("{}错误信息={}", servicePlanId, e.getMessage());
                }
            }
            try {
                //一键生成订单  只有由状态变成已完成生成
                generateOrder(servicePlanEntity, triggerType, thatDay, tempTime);  //20241231
            } catch (Exception e) {
                log.info("生成订单异常:" + e);
                e.printStackTrace();
            }
            //修改下一周期时间
            String nextTime = getNextDateByStartDateNew(servicePlanEntity);
            servicePlanEntity.setNextCycleTime(nextTime);

            //存在临时时间，清空
            if (tempTime) {
                servicePlanEntity.setPlanLscfsj("");
            }
        }
        if (ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode().equals(servicePlanStatus)) {
            //进行中
            if (ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode().equals(servicePlanEntity.getPlanStatus())) {
                //throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_STAR.getRet(),StatusCodeEnum.SERVICE_PLAN_NOT_STAR.getMessage());
                throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_STAR.getRet(), ResourceUtil.getI18nMessage("16005"));
            }
            servicePlanEntity.setPlanStatusName("进行中");
        }
        servicePlanEntity.setPlanStatus(servicePlanStatus);
        int update = servicePlanMapper.update(servicePlanEntity, queryWrapper);

        if (update == 1) {
            return true;
        }
        return false;
    }

    /*public String getNextDateByStartDate(Date startDate, ServicePlanEntity servicePlan) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Integer planCycle = servicePlan.getPlanCycle();
        Integer planInterval = servicePlan.getPlanInterval();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        if (ServicePlanEnums.PlanCycle.DAY.getCode().equals(planCycle)) {
            //日
            calendar.add(Calendar.DATE, planInterval);
        }
        if (ServicePlanEnums.PlanCycle.WEEK.getCode().equals(planCycle)) {
            //周
            calendar.add(Calendar.DATE, 7 * planInterval);
        }
        if (ServicePlanEnums.PlanCycle.MONTH.getCode().equals(planCycle)) {
            //月
            calendar.add(Calendar.MONTH, planInterval);
        }
        if (ServicePlanEnums.PlanCycle.YEAR.getCode().equals(planCycle)) {
            //年
            calendar.add(Calendar.YEAR, planInterval);
        }
        String endDate = sdf.format(calendar.getTime());
        return endDate;
    }*/

    /**
     * 获取本次周期的开始时间
     *
     * @param servicePlan
     * @param currentDay
     * @return
     */
    public String getCurrentDateStartDate(ServicePlanEntity servicePlan, String currentDay) {
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
            //获取计划开始日期
            Date endDate = df.parse(currentDay);
            Integer planCycle = servicePlan.getPlanCycle();
            //转为负数
            Integer planInterval = -(servicePlan.getPlanInterval());
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(endDate);
            //日
            if (ServicePlanEnums.PlanCycle.DAY.getCode().equals(planCycle)) {
                calendar.add(Calendar.DATE, planInterval);
            }
            //周
            if (ServicePlanEnums.PlanCycle.WEEK.getCode().equals(planCycle)) {
                calendar.add(Calendar.DATE, 7 * planInterval);
            }
            //月
            if (ServicePlanEnums.PlanCycle.MONTH.getCode().equals(planCycle)) {
                calendar.add(Calendar.MONTH, planInterval);
            }
            //年
            if (ServicePlanEnums.PlanCycle.YEAR.getCode().equals(planCycle)) {
                calendar.add(Calendar.YEAR, planInterval);
            }
            //必须加1 因为周期开始时间必须加1，例如 6.21日--9.20日算一个周期的结束
            calendar.add(Calendar.DATE, 1);
            Date startDate = calendar.getTime();
            return df.format(startDate);
        } catch (Exception e) {
            Log.error("获取数据失败，失败原因：" + e.getMessage());
            return null;
        }
    }

    /**
     * 获取下一个周期时间
     *
     * @param servicePlan
     * @return
     */
    public String getNextDateByStartDateNew(ServicePlanEntity servicePlan) {
        Date endDate = null;
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
            //获取当前日期
            String current = df.format(new Date());
            Date currentDate = df.parse(current);
            //获取计划开始日期
            endDate = df.parse(servicePlan.getPlanStartTime());
            Integer planCycle = servicePlan.getPlanCycle();
            Integer planInterval = servicePlan.getPlanInterval();
            do {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(endDate);
                //日
                if (ServicePlanEnums.PlanCycle.DAY.getCode().equals(planCycle)) {
                    calendar.add(Calendar.DATE, planInterval);
                }
                //周
                if (ServicePlanEnums.PlanCycle.WEEK.getCode().equals(planCycle)) {
                    calendar.add(Calendar.DATE, 7 * planInterval);
                }
                //月
                if (ServicePlanEnums.PlanCycle.MONTH.getCode().equals(planCycle)) {
                    calendar.add(Calendar.MONTH, planInterval);
                }
                //年
                if (ServicePlanEnums.PlanCycle.YEAR.getCode().equals(planCycle)) {
                    calendar.add(Calendar.YEAR, planInterval);
                }
                endDate = calendar.getTime();
            } while (endDate.getTime() <= currentDate.getTime());  //whl.2024241122
            Log.info("下一周期日期：" + df.format(endDate));
            //针对有结束时间的服务计划做判断
            if (StringUtils.isNotBlank(servicePlan.getPlanEndTime())) {
                Date planEndTime = df.parse(servicePlan.getPlanEndTime());
                if (endDate.getTime() > planEndTime.getTime()) {
                    return null;
                }
            }
            return df.format(endDate);
        } catch (Exception e) {
            Log.error("获取数据失败，失败原因：" + e.getMessage());
            return null;
        }

    }

    public String getPreviousDateByStartDate(Date startDate, ServicePlanEntity servicePlan) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Integer planCycle = servicePlan.getPlanCycle();
        Integer planInterval = servicePlan.getPlanInterval();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        if (ServicePlanEnums.PlanCycle.DAY.getCode().equals(planCycle)) {
            //日
            calendar.add(Calendar.DATE, -planInterval);
        }
        if (ServicePlanEnums.PlanCycle.WEEK.getCode().equals(planCycle)) {
            //周
            calendar.add(Calendar.DATE, -7 * planInterval);
        }
        if (ServicePlanEnums.PlanCycle.MONTH.getCode().equals(planCycle)) {
            //月
            calendar.add(Calendar.MONTH, -planInterval);
        }
        if (ServicePlanEnums.PlanCycle.YEAR.getCode().equals(planCycle)) {
            //年
            calendar.add(Calendar.YEAR, -planInterval);
        }
        String endDate = sdf.format(calendar.getTime());
        return endDate;
    }


    @Override
    public ServicePlanCalendarVO checkServicePlanCalendar(ServicePlanCalendarAO planCalendarAO) {
        ServicePlanCalendarVO vo = new ServicePlanCalendarVO();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        ServicePlanEntity servicePlanEntity = servicePlanMapper.selectById(planCalendarAO.getServicePlanId());
        if (ObjectUtil.isEmpty(servicePlanEntity)) {
            // throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getRet(),StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("16001"));
        }
        if (!ServicePlanEnums.PlanType.MAINTAIN.getCode().toString().equals(servicePlanEntity.getPlanType())) {
            //throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_CALENDAR.getRet(),StatusCodeEnum.SERVICE_PLAN_NOT_CALENDAR.getMessage());
            throw new BusinessException(StatusCodeEnum.SERVICE_PLAN_NOT_CALENDAR.getRet(), ResourceUtil.getI18nMessage("16006"));
        }
        NxJeDeviceEntity deviceEntity = nxJeDeviceService.selectBySbbm(servicePlanEntity.getDeviceCode());

        List<ServicePlanCalendarList> list = new ArrayList<ServicePlanCalendarList>();
        //根据查询获取月份的开始和结束日期
        String queryTime = planCalendarAO.getQueryTime();
        Date parse = null;
        try {
            parse = format.parse(queryTime);
        } catch (ParseException e) {
            Log.info("获取月份错误信息：{}", e.getMessage());
        }
        Calendar cld = Calendar.getInstance();
        cld.setTime(parse);
        cld.add(Calendar.MONTH, 1);
        cld.set(Calendar.DAY_OF_MONTH, 0);
        String lastDay = sdf.format(cld.getTime());
        Log.info("当月最后一天={}", lastDay);
        cld.set(Calendar.DAY_OF_MONTH, 1);
        String firstDay = sdf.format(cld.getTime());
        Log.info("当月第一天={}", firstDay);

        //查询待办库里面的是否满足条件数据,历史数据
        Map param = new HashMap();
        param.put("planId", servicePlanEntity.getServicePlanId());
        param.put("firstDay", firstDay);
        param.put("lastDay", lastDay);
        List<Map<String, String>> todoLibrary = servicePlanMapper.getTodoLibrary(param);
        List<String> dates = new ArrayList<>();
        Map<String, String> orderMap = new HashMap<>();

        if (todoLibrary.size() > 0) {
            for (Map<String, String> item :
                    todoLibrary) {
                dates.add(item.get("TIME"));
                orderMap.put(item.get("TIME"), item.get("ORDER_STATUS"));
            }
        }
        //存在临时时间  如果临时时间不在当前月， 不参与此次展示
        if (StringUtils.isNotBlank(servicePlanEntity.getPlanLscfsj())) {
            if (servicePlanEntity.getPlanLscfsj().contains(queryTime)) {
                dates.add(servicePlanEntity.getPlanLscfsj());
            }
        }
        String planStartTime = servicePlanEntity.getPlanStartTime();
        String planEndTime = servicePlanEntity.getPlanEndTime();
        List<String> days = com.rootcloud.common.live.DateUtil.findDates(planStartTime, planEndTime,
                String.valueOf(servicePlanEntity.getPlanCycle()), servicePlanEntity.getPlanInterval(), firstDay, lastDay);
        Log.info("获取到日期1：" + days);
        //取出最后一个历史数据时间
        /*if (dates.size() > 0) {
            String lastTime = dates.get(dates.size() - 1).substring(0, 10);
            days.stream().forEach(item -> {
                //保留大于最后一个时间的数据
                if (item.length() >= 10) {
                    if (item.substring(0, 10).compareTo(lastTime) > 0) {
                        dates.add(item);
                    }
                }
            });
        } else {
            dates.addAll(days);
        }*/
        if (dates.size() > 0) {
            if (dates.size() < days.size()) {
                for (int i = dates.size(); i < days.size(); i++) {
                    dates.add(days.get(i));
                }
            }
        } else {
            dates.addAll(days);
        }
        //清理掉提前触发日期
        List<String> newDates = new ArrayList<>();
        String planTqcfrq = servicePlanEntity.getPlanTqcfrq();
        if (StringUtils.isNotBlank(planTqcfrq)) {
            List<String> tqfcrqList = Arrays.asList(servicePlanEntity.getPlanTqcfrq().split(","));
            if (tqfcrqList.size() > 0) {
                dates.forEach(o -> {
                    if (!tqfcrqList.contains(o)) {
                        newDates.add(o);
                    }
                });
            } else {
                newDates.addAll(dates);
            }
        } else {
            newDates.addAll(dates);
        }
        //数据状态为暂停 则取最后一条数据比对，大于这个时间的数据全部隐藏
        List<String> liss = new ArrayList<>();
        String planStatus = servicePlanEntity.getPlanStatus().toString();
        if (ServicePlanEnums.PlanStatus.PAUSED.getCode().toString().equals(planStatus)) {
            String ztrq = servicePlanEntity.getPlanZtrq();
            List<String> ztrqList = new ArrayList<>();
            if (StringUtils.isNotBlank(ztrq)) {
                ztrqList = new ArrayList<>(Arrays.asList(ztrq.split(",")));
            }
            //获取暂停开始时间
            String start = null;
            if (ztrqList.size() > 0) {
                start = ztrqList.get(ztrqList.size() - 1);
            }
            if (StringUtils.isNotBlank(start)) {
                for (int i = 0; i < newDates.size(); i++) {
                    if (newDates.get(i).compareTo(start) < 0) {
                        liss.add(newDates.get(i));
                    }
                }
            } else {
                liss.addAll(newDates);
            }
        } else {
            liss.addAll(newDates);
        }
        //清理掉暂停日期的
        List<String> screenDates = new ArrayList<>();
        String ztrq = servicePlanEntity.getPlanZtrq();
        if (StringUtils.isNotBlank(ztrq)) {
            List<String> ztrqList = Arrays.asList(ztrq.split(","));
            if (ztrqList.size() > 0) {
                liss.forEach(o -> {
                    if (!ztrqList.contains(o)) {
                        screenDates.add(o);
                    }
                });
            } else {
                screenDates.addAll(liss);
            }
        } else {
            screenDates.addAll(liss);
        }
        Log.info("服务计划：" + servicePlanEntity.getPlanName() + " 获取到日期：" + screenDates);
        screenDates.stream().forEach(item -> {
            ServicePlanCalendarList servicePlanCalendarList = new ServicePlanCalendarList();
            servicePlanCalendarList.setCalendarDate(item);
            servicePlanCalendarList.setOrderStatus(orderMap.get(item));
            ServicePlanCalendarDetailsVO servicePlanCalendarDetailsVO = new ServicePlanCalendarDetailsVO();
            servicePlanCalendarDetailsVO.setDeviceCode(deviceEntity.getDeviceSbbm());
            servicePlanCalendarDetailsVO.setDviceName(deviceEntity.getDeviceSbmc());
            servicePlanCalendarDetailsVO.setCustomerName(deviceEntity.getDeviceSskhmc());
            servicePlanCalendarDetailsVO.setPlanStartTime(servicePlanEntity.getPlanStartTime());
            servicePlanCalendarDetailsVO.setServiceName(servicePlanEntity.getPlanName());
            servicePlanCalendarDetailsVO.setPlanEndTime(servicePlanEntity.getPlanEndTime());
            servicePlanCalendarDetailsVO.setFinishTimeNode(servicePlanEntity.getFinishTimeNode());
            String s = returnDateStatusNew(servicePlanEntity, item);
            servicePlanCalendarDetailsVO.setDateStatus(s);
            servicePlanCalendarList.setDateStatus(s);
            servicePlanCalendarList.setServicePlanCalendarDetailsVO(servicePlanCalendarDetailsVO);
            list.add(servicePlanCalendarList);
        });
        vo.setList(list);
        vo.setFinishRate(finishRateNew(list));
        return vo;
    }


    /**
     * 计算日期
     *
     * @param startTime
     * @param servicePlanEntity
     * @return
     */
    public String calcDate(String startTime, ServicePlanEntity servicePlanEntity) throws ParseException {
        Integer finishTimeNode = servicePlanEntity.getFinishTimeNode() == null ? 0 : servicePlanEntity.getFinishTimeNode();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        if (finishTimeNode != 0) {
            Date parse = sdf.parse(startTime);
            Calendar cld = Calendar.getInstance();
            cld.setTime(parse);
            cld.add(Calendar.DATE, finishTimeNode);
            String format = sdf.format(cld.getTime());
            return format;
        } else {
            return sdf.format(sdf.parse(startTime));
        }
    }

    /**
     * 当前任务日历状态
     *
     * @param servicePlanEntity
     * @return
     */
    public String returnDateStatus(ServicePlanEntity servicePlanEntity, String time) {
        //1进行中，2已经完成，0暂停
        String servicePlanId = servicePlanEntity.getServicePlanId();
        NxOrderEntity order = nxOrderMapper.selectByServicePlanId(servicePlanId, time);
        Log.info("任务日历查询订单", JSON.toJSON(order));
        if (ObjectUtils.isNotEmpty(order)) {
            if (OrderEnums.OrderStatus.DONE.getCode().equals(order.getOrderOrderstatusCode())) {
                return "2";
            }
            if (OrderEnums.OrderStatus.DEPRECATED.getCode().equals(order.getOrderOrderstatusCode())
                    || OrderEnums.OrderStatus.DELETED.getCode().equals(order.getOrderOrderstatusCode())) {
                return "0";
            }
        }
        return "1";
    }


    public Date returnEndDate(Date startData, ServicePlanEntity servicePlanEntity) {
        Date endData = null;
        Calendar cld = Calendar.getInstance();
        cld.setTime(startData);
        if (ServicePlanEnums.PlanCycle.DAY.getCode().equals(servicePlanEntity.getPlanCycle())) {
            Integer day = servicePlanMapper.queryLgortsByCode("DAY");
            if (day == null) {
                day = 0;
            }
            cld.add(Calendar.DATE, day);
        } else if (ServicePlanEnums.PlanCycle.WEEK.getCode().equals(servicePlanEntity.getPlanCycle())) {
            Integer week = servicePlanMapper.queryLgortsByCode("WEEK");
            if (week == null) {
                week = 3;
            }
            cld.add(Calendar.DATE, week);
        } else if (ServicePlanEnums.PlanCycle.MONTH.getCode().equals(servicePlanEntity.getPlanCycle())) {
            Integer month = servicePlanMapper.queryLgortsByCode("MONTH");
            if (month == null) {
                month = 7;
            }
            cld.add(Calendar.DATE, month);
        } else if (ServicePlanEnums.PlanCycle.YEAR.getCode().equals(servicePlanEntity.getPlanCycle())) {
            Integer year = servicePlanMapper.queryLgortsByCode("YEAR");
            if (year == null) {
                year = 180;
            }
            cld.add(Calendar.DATE, year);
        }
        endData = cld.getTime();
        return endData;
    }


    /**
     * 当前任务日历状态
     *
     * @param servicePlanEntity
     * @param time
     * @return
     */
    @SneakyThrows
    public String returnDateStatusNew(ServicePlanEntity servicePlanEntity, String time) {
        //设计要求：任务完成状态划分为： 未发起状态灰色、已发起状态蓝色、正常完成绿色（审核完成）、延期未完成-浅绿色、 拖期完成橙色、未完成红色
        //具体要求：拖期完成 - （天保养往后默认延期0天、 周是往后默认延期3天、月是往后默认延期7天、年是往后默认延期180天)，新增计划页面上添加配置项
        String servicePlanId = servicePlanEntity.getServicePlanId();
        SimpleDateFormat sdf = null;
        if (time.length() > 10) {
            sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        } else {
            sdf = new SimpleDateFormat("yyyy-MM-dd");
        }
        //获取到当前日期
        String currentDate = sdf.format(new Date());
        //Log.info("获取到服务计划ID:" + servicePlanId + "，时间：" + time);
        NxOrderEntity order = nxOrderMapper.selectByServicePlanId(servicePlanId, time);
        /*if (order == null) {
            Date beginDate = sdf.parse(time);
            Date endDate = sdf.parse(currentDate);
            if (beginDate.before(endDate)) {
                return ServicePlanEnums.MaintenanceType.STATUS_FINISH.getCode();
            }
        }*/
        //Log.info("获取到服务计划ID对应订单：" + order);
        String dataStatus = null;
        //获取正常完成时间
        Calendar cld = Calendar.getInstance();
        cld.setTime(sdf.parse(time));
        if (servicePlanEntity.getFinishTimeNode() != null) {
            cld.add(Calendar.DATE, (servicePlanEntity.getFinishTimeNode()));
        }
        Date startData = cld.getTime();
        //计算范围
        Date endData = returnEndDate(startData, servicePlanEntity);
        if (ObjectUtils.isNotEmpty(order) && "1".equals(order.getSyStatus())) {
            //已发起状态蓝色
            dataStatus = ServicePlanEnums.MaintenanceType.STATUS_LAUNCH.getCode();
            //先看状态
            if ((OrderEnums.OrderStatus.TO_BE_RETURNED.getCode().equals(order.getOrderOrderstatusCode()) ||
                    OrderEnums.OrderStatus.DONE.getCode().equals(order.getOrderOrderstatusCode()))
                    && StringUtils.isNotBlank(order.getOrderShtgsj())) {
                //完成：正常完成绿色（审核完成）、拖期完成橙色、
                dataStatus = ServicePlanEnums.MaintenanceType.STATUS_FINISH.getCode();
                //拖期完成橙色 审核时间在拖期范围
                Date date = DateUtil.timestampToDate(Long.valueOf(order.getOrderShtgsj()));
                Date shsj = sdf.parse(sdf.format(date));
                //完成时间可以加完成时间节点
                /*Calendar cldar = Calendar.getInstance();
                cldar.setTime(endData);
                if(servicePlanEntity.getFinishTimeNode()!=null){
                    cldar.add(Calendar.DATE, servicePlanEntity.getFinishTimeNode());
                }
                endData = cldar.getTime();*/
                if (shsj.getTime() <= endData.getTime() && shsj.getTime() > startData.getTime()) {
                    dataStatus = ServicePlanEnums.MaintenanceType.STATUS_EXTENDED_FINISH.getCode();
                }
                //未完成
                //审核时间大于拖期范围
                if (shsj.getTime() > endData.getTime()) {
                    dataStatus = ServicePlanEnums.MaintenanceType.STATUS_NO_FINISH.getCode();
                }
            } else {
                //未完成：延期未完成-浅绿色、未完成红色
                //dataStatus = ServicePlanEnums.MaintenanceType.STATUS_NO_FINISH.getCode();
                //延期未完成-浅绿色 日历日期在延期时间范围
                //现在日期 比较日历日期
                Date nowDate = sdf.parse(sdf.format(new Date()));
                if (nowDate.getTime() <= endData.getTime() && nowDate.getTime() > startData.getTime()) {
                    dataStatus = ServicePlanEnums.MaintenanceType.STATUS_EXTENDED_NO_FINISH.getCode();
                }
                if (nowDate.getTime() > endData.getTime()) {
                    dataStatus = ServicePlanEnums.MaintenanceType.STATUS_NO_FINISH.getCode();
                }
            }
        }
        if (StringUtils.isNotBlank(dataStatus)) {
            return dataStatus;
        }
        //未发起状态灰色
        return ServicePlanEnums.MaintenanceType.STATUS_NO_LAUNCH.getCode();
    }


    /**
     * 当前计划完成率
     *
     * @param list
     * @return
     */
    @Override
    public String finishRate(List<ServicePlanCalendarList> list) {
        //已经完成成率 = （已经完成的保养计划数量/(当月所有保养计划数量-已经暂停计划数量)）*100%
        if (!list.isEmpty()) {
            //已经完成
            int finshSize = list.stream().filter(temp -> "2".equals(temp.getDateStatus())).collect(Collectors.toList()).size();
            //当月所有保养计划数量-已经暂停计划数量
            int size = list.stream().filter(temp -> !"0".equals(temp.getDateStatus())).collect(Collectors.toList()).size();
            if (size == 0) {
                return "0%";
            } else {
                BigDecimal multiply = (new BigDecimal(finshSize).multiply(new BigDecimal(100)).divide(new BigDecimal(size), 1, RoundingMode.HALF_UP));
                return multiply.setScale(1, BigDecimal.ROUND_HALF_UP) + "%";
            }
        }
        return "0%";
    }


    /**
     * 当前计划完成率新的
     *
     * @param list
     * @return
     */
    public String finishRateNew(List<ServicePlanCalendarList> list) {

        //除去删除的订单
        List<ServicePlanCalendarList> newlist = list.stream().filter(temp -> (!"0".equals(temp.getOrderStatus()))).collect(Collectors.toList());
        //已经完成成率 = （已经完成的保养计划数量/(当月所有保养计划数量）*100%
        if (!newlist.isEmpty()) {
            //已经完成
            int finshSize = newlist.stream().filter(temp -> ((ServicePlanEnums.MaintenanceType.STATUS_EXTENDED_FINISH.getCode().equals(temp.getDateStatus()) || ServicePlanEnums.MaintenanceType.STATUS_FINISH.getCode().equals(temp.getDateStatus())))).collect(Collectors.toList()).size();
            //当月所有保养计划数量
            int size = newlist.size();
            if (size == 0) {
                return "0%";
            } else {
                BigDecimal multiply = (new BigDecimal(finshSize).multiply(new BigDecimal(100)).divide(new BigDecimal(size), 1, RoundingMode.HALF_UP));
                return multiply.setScale(1, BigDecimal.ROUND_HALF_UP) + "%";
            }
        }
        return "0%";
    }

    /**
     * 当前计划完成率新的
     *
     * @param list
     * @return
     */
    public String finishRateNewTotal(List<ServicePlanCalendarList> list) {

        List<ServicePlanCalendarDetailsVO> newlist = new ArrayList<>();
        for (ServicePlanCalendarList temp : list) {
            if (temp.getLists() != null && temp.getLists().size() > 0) {
                for (ServicePlanCalendarDetailsVO vo :
                        temp.getLists()) {
                    if (!"0".equals(vo.getOrderStatus())) {
                        newlist.add(vo);
                    }
                }
            }
        }
        Log.info("未删除的订单个数:" + newlist.size());
        //除去删除的订单
        //已经完成成率 = （已经完成的保养计划数量/(当月所有保养计划数量）*100%
        if (!newlist.isEmpty()) {
            //已经完成
            int finshSize = newlist.stream().filter(temp -> ((ServicePlanEnums.MaintenanceType.STATUS_EXTENDED_FINISH.getCode().equals(temp.getDateStatus()) || ServicePlanEnums.MaintenanceType.STATUS_FINISH.getCode().equals(temp.getDateStatus())))).collect(Collectors.toList()).size();
            //当月所有保养计划数量
            int size = newlist.size();
            if (size == 0) {
                return "0%";
            } else {
                BigDecimal multiply = (new BigDecimal(finshSize).multiply(new BigDecimal(100)).divide(new BigDecimal(size), 1, RoundingMode.HALF_UP));
                return multiply.setScale(1, BigDecimal.ROUND_HALF_UP) + "%";
            }
        }
        return "0%";
    }


    /**
     * 保养通知信息查询 根据设备编码查询设备保养信息
     *
     * @param userId
     * @param startNo
     * @param endNo
     * @return
     */
    @Override
    public List<MyAdviceEntity> getmMaintainPlanService(String userId, Integer startNo, Integer endNo) {
        List<MyAdviceEntity> resultList = new ArrayList<>();
        List<ServicePlanEntity> planEntityList = servicePlanMapper.selectByUserId(userId, startNo, endNo);
        if (CollectionUtil.isEmpty(planEntityList)) {
            return resultList;
        }
        List<String> servicePlanIds = planEntityList.stream().map(ServicePlanEntity::getServicePlanId).collect(Collectors.toList());

        //根据已出发的保养任务计划id查询推送的消息内容
        List<NxUserInfoEntity> userInfoEntities = nxUserInfoService.queryUserInfoByPlanIds(servicePlanIds);
        for (ServicePlanEntity temp : planEntityList) {
            NxJeDeviceEntity nxJeDeviceEntity = nxJeDeviceService.selectBySbbm(temp.getDeviceCode());
            MyAdviceEntity myAdviceEntity = new MyAdviceEntity();
            myAdviceEntity.setAdviceClass(ResourceUtil.getI18nMessage("60014"));
            myAdviceEntity.setAdvicObjectCode(temp.getDeviceCode());
            myAdviceEntity.setAdviceName(temp.getMaintainContent());
            myAdviceEntity.setAdviceTime(temp.getLatestTriggerTime());
            myAdviceEntity.setAdviceContent(getMessageContent(temp, userInfoEntities));
            myAdviceEntity.setTypeCode(ServicePlanEnums.MessageType.MAINTENANCE_REMINDERS.getCode());
            myAdviceEntity.setAdviceCode(ServicePlanEnums.MessageCodeType.MAINTENANCE_MESSAGE.getCode());
            myAdviceEntity.setSignData(temp.getServicePlanId());
            myAdviceEntity.setServicePlanId(temp.getServicePlanId());
            myAdviceEntity.setPlanName(temp.getPlanName());//计划名称
            myAdviceEntity.setPlanEndTime(temp.getPlanEndTime());//计划完成时间
            if (!ObjectUtil.isEmpty(nxJeDeviceEntity)) {
                myAdviceEntity.setAdviceObjectName(nxJeDeviceEntity.getDeviceSbmc());
            }
            List<NxTodoLibrary> nxTodoLibraries = nxTodoLibraryService.selectBySign(myAdviceEntity.getAdviceCode(), myAdviceEntity.getSignData(), myAdviceEntity.getAdviceTime());
            if (nxTodoLibraries.isEmpty()) {
                resultList.add(myAdviceEntity);
            }


        }
        return resultList;
    }


    @Resource
    private OrderNumberUtils orderNumberUtils;
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void generateOrder(ServicePlanEntity servicePlan, int triggerType, String thatDay, boolean tempTime) throws Exception {
        log.info("{}订单创建日志-服务计划{}触发订单创建方法，是否临时触发订单{}，触发类型:{}", new Date(), servicePlan.getServicePlanId(),tempTime,triggerType);
        //判断是否存在派工， 只有存在派工才走订单申请
        Map<String, String> islcpzb = servicePlanMapper.getIslcpzb(servicePlan.getSyZhid());
        if (islcpzb != null) {
            String LCPZB_SFXYPG = islcpzb.get("LCPZB_SFXYPG");
            if (LCPZB_SFXYPG.equals("0")) {
                //没有派工
                Log.info(new Date() + " " + "订单创建日志-没有派工:" + servicePlan.getSyZhid() + " " + servicePlan.getServicePlanId());
                return;
            }
        }
        List servicePlanIds = new ArrayList();
        servicePlanIds.add(servicePlan.getServicePlanId());
        //默认通知是提前一天发起的，订单创建时间晚一天
        //String tom_day = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(DateUtils.addDays(, 1));  //获取明天的时间
        String tom_day = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());  //获取当天的时间
        String order_time = tom_day;  //默认订单时间是明天时间
        String start_time = null;
        if (StringUtils.isNotBlank(thatDay)) {
            start_time = thatDay;
        } else {
            start_time = tom_day.substring(0, 10);//默认开工时候是当天时间
        }
        //根据服务计划和开始日期查询代办库，看服务计划是否执行生成了订单
        Integer count = servicePlanMapper.checkOrder(servicePlan.getServicePlanId(), start_time);
        if (count > 0) {
            log.info("订单创建日志-查询待办库{}，有值，未生成订单,服务计划ID：{}", new Date(), servicePlan.getServicePlanId());
            return;
        }
        /*String curTime = DateUtil.dateToString(new Date(), DateUtil.PATTERN_CLASSICAL_SIMPLE);  //获取当前年月日时间
        String servicePlanTime = DateUtil.dateToString(DateUtil.parse(servicePlan.getPlanStartTime(), DateUtil.PATTERN_CLASSICAL_SIMPLE), DateUtil.PATTERN_CLASSICAL_SIMPLE);
        if (servicePlanTime.equals(curTime) || servicePlanTime.equals(servicePlan.getPlanLscfsj())) {
            // 遇到计划开始时间/临时时间是当天情况：订单创建时间不需要晚一天,设置当天时间, 开工时间
            order_time = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
            start_time = curTime;
        } else if (servicePlanTime.compareTo(curTime) >= 0) {
            // 在计划开始时间之前触发的订单，开工时间都是计划时间
            start_time = DateUtil.dateToString(DateUtil.parse(servicePlanTime, DateUtil.PATTERN_CLASSICAL_SIMPLE), DateUtil.PATTERN_CLASSICAL_SIMPLE);
        }*/
        //根据已出发的保养任务计划id查询推送的消息内容
        List<NxUserInfoEntity> userInfoEntities = nxUserInfoService.queryUserInfoByPlanIds(servicePlanIds);
        QueryWrapper<NxDeviceEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("NX_JCXX_SBLB_ID", servicePlan.getDeviceId());
        NxDeviceEntity nxDeviceEntity = nxDeviceMapper.selectOne(queryWrapper);
        log.info("{}订单创建日志-服务计划{}触发派单获取设备信息-->{}", new Date(), servicePlan.getServicePlanId(), nxDeviceEntity);
        if (nxDeviceEntity == null) {
            log.info("{}订单创建日志-{}触发1404行return", new Date(), servicePlan.getServicePlanId());
            return;
        }
        //新增订单
        log.info("{}订单创建日志-{}新增订单开始---->", new Date(), servicePlan.getServicePlanId());
        NxOrderEntity order = new NxOrderEntity();
        String orderId = JEUuidUtil.uuid();
        order.setNxOrderId(orderId);
        String orderNumber = "B" + orderNumberUtils.generateOrderNumber();
        order.setOrderOrdernumber(orderNumber);
        order.setOrderSource("app");
        order.setOrderDevicecode(nxDeviceEntity.getSblbSbbm());
        order.setOrderDevicename(nxDeviceEntity.getSblbSbmc());
        order.setOrderDevicetype(nxDeviceEntity.getSblbSblxName());
        order.setOrderDevicemodel(nxDeviceEntity.getSblbSbxhName());
        //判断是否为临时触发时间生成的订单
        order.setServicePlanOrderStatus(tempTime ? "after" : "");
        order.setTriggerType(triggerType);
        if (!ObjectUtil.isEmpty(nxDeviceEntity)) {
            order.setOrderWbjsrq(nxDeviceEntity.getSblbWbjsrq());
            if (StringUtils.isNotBlank(nxDeviceEntity.getSblbWbjsrq())) {
                if (new SimpleDateFormat("yyyy-MM-dd").format(new Date()).compareTo(nxDeviceEntity.getSblbWbjsrq()) <= 0) {
                    order.setOrderWarrantiesCode(ServicePlanEnums.CheckType.CHECK_TURE.getCode());
                    order.setOrderWarrantiesName(ServicePlanEnums.CheckType.CHECK_TURE.getDesc());
                } else {
                    order.setOrderWarrantiesCode(ServicePlanEnums.CheckType.CHECK_FALSE.getCode());
                    order.setOrderWarrantiesName(ServicePlanEnums.CheckType.CHECK_FALSE.getDesc());
                }
            }
            order.setOrderServiceaddress(nxDeviceEntity.getSblbSbxxdz());
            order.setOrderServiceaddressLongitude(nxDeviceEntity.getSblbJd());
            order.setOrderServiceaddressDimension(nxDeviceEntity.getSblbWd());
            order.setOrderCustomerName(nxDeviceEntity.getSblbKhmc());
            order.setOrderKhlxr(nxDeviceEntity.getSblbKhlxr());
            order.setOrderContactway(nxDeviceEntity.getSblbLxfs());
            order.setOrderCustomerId(nxDeviceEntity.getSblbKhid());
            //根据customerId 查询  NX_KHGL
            String wdId = servicePlanMapper.selectOutletsIdByCustomerId(nxDeviceEntity.getSblbKhid(), servicePlan.getSyZhid());
            log.info("订单创建日志-服务计划{}触发派单获取服务网点ID-->{}", servicePlan.getServicePlanId(), wdId);
            if (StringUtils.isNotBlank(wdId)) {
                order.setOrderFwwdid(wdId);
            }
        }
        order.setOrderServicetypeCode(ServicePlanEnums.MessageType.MAINTENANCE.getCode());
        order.setOrderServicetypeName(ServicePlanEnums.MessageType.MAINTENANCE.getDesc());
        order.setOrderOrderstatusCode(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getCode());
        order.setOrderOrderstatusName(OrderEnums.OrderStatus.WORKERS_TO_BE_DISPATCHED.getDesc());
        order.setSyCreatetime(order_time);
        order.setSyStatus("1");
        order.setOrderBynr(servicePlan.getMaintainContent());
        //2023.3.22日加上保养级别和保养内容名称
        order.setOrderByjb(servicePlan.getMaintenanceType());
        order.setOrderBynrname(servicePlan.getMaintainContentName());
        order.setOrderDemanddescription(servicePlan.getMaintainContentName());
        String latestTriggerTime = servicePlan.getLatestTriggerTime();
        Calendar cld = Calendar.getInstance();
        try {
            cld.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(latestTriggerTime));
        } catch (Exception e) {
            e.printStackTrace();
            log.info("订单创建日志{}-1458日期转换错误", servicePlan.getServicePlanId());
        }
        cld.add(Calendar.DAY_OF_MONTH, 1);
        String orderApplydate = new SimpleDateFormat("yyyy-MM-dd").format(cld.getTime());
        order.setOrderApplydate(orderApplydate);
        List<OrderLcinfoParam> list = new ArrayList<>();
        OrderLcinfoParam orderLcinfoParam = new OrderLcinfoParam();
        orderLcinfoParam.setId(1);
        orderLcinfoParam.setName("已受理");
        orderLcinfoParam.setUserName("管理员");
        orderLcinfoParam.setIcon("sl_icon");
        orderLcinfoParam.setEndDate(String.valueOf(DateUtil.getTimesByString(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))));
        list.add(orderLcinfoParam);
        order.setOrderLcinfo(JSON.toJSONString(list).toString());
        order.setSyZhid(servicePlan.getSyZhid());//设置租户ID 20221020号
        log.info("订单创建日志-order：{}", order);
        nxOrderMapper.insert(order);
        log.info("订单创建日志-服务计划{}生成订单成功", servicePlan.getServicePlanId());
        //生成待办库
        NxTodoLibrary library = new NxTodoLibrary();
        library.setTitle(ResourceUtil.getI18nMessage("60014"));
        library.setContent(getMessageContent(servicePlan, userInfoEntities));
        library.setDeviceName(nxDeviceEntity.getSblbSbmc());
        library.setDeviceCode(servicePlan.getDeviceCode());
        library.setOperation(servicePlan.getMaintainContent());
        library.setTime(start_time);   //实工开始时间
        library.setSyCreatetime(DateUtil.format(new Date()));
        library.setAdviceCode(ServicePlanEnums.MessageCodeType.MAINTENANCE_MESSAGE.getCode());
        library.setTypeCode(ServicePlanEnums.MessageType.MAINTENANCE_REMINDERS.getCode());
        library.setSignData(servicePlan.getServicePlanId());
        library.setStatus("0");
        library.setPlanName(servicePlan.getPlanName());//计划名称
        library.setPlanEndTime(servicePlan.getPlanEndTime());//计划完成时间
        library.setServicePlanId(servicePlan.getServicePlanId());
        library.setSyCreatetime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
        String libraryId = JEUuidUtil.uuid();
        library.setNxTodoLibraryId(libraryId);
        library.setNxOrderId(order.getNxOrderId());
        nxTodoLibraryService.save(library);
        log.info("订单创建日志-服务计划{}生成代办库成功-->{}", servicePlan.getServicePlanId(), library);
        //保养内容需要增加
        List<Map> selectListXMBY = servicePlanMapper.selectListXMBY(servicePlan.getMaintainContent(), servicePlan.getSyZhid());
        if (selectListXMBY.size() > 0) {
            for (Map map : selectListXMBY
            ) {
                map.put("NX_BYNR_ID", JEUuidUtil.uuid());
            }
            servicePlanMapper.insertXmbynr(selectListXMBY, order.getNxOrderId());
        }
        //流程引擎
        if (StringUtils.isNotBlank(order.getOrderFwwdid())) {
            log.info("订单创建日志-{}进入派工流程引擎", servicePlan.getServicePlanId());
            //调用登录接口
            String loginName = servicePlanMapper.queryStringByCode("loginName") == null ? "app_master_admin" : servicePlanMapper.queryStringByCode("loginName");
            String loginPassword = servicePlanMapper.queryStringByCode("loginPassword") == null ? "Liu@123456" : servicePlanMapper.queryStringByCode("loginPassword");
            OAuthResult login = rootCloudAuthService.login(loginName, loginPassword, null);
            Log.info(servicePlan.getServicePlanId() + "订单创建日志-登录接口:" + login + "loginName:" + loginName + " loginPassword:" + loginPassword);
            OutletsVO outletsVO = servicePlanMapper.returnUserNames(order.getOrderFwwdid());
            String sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/sponsorProcess";
            Map<String, String> map = new HashMap();
            map.put("pdid", "frx-ningxia-dingdan_22051015-7");
            map.put("beanId", orderId);
            map.put("currentTaskName", "");
            map.put("submitComments", "订单待派工，请尽快处理");
            map.put("targerTaskName", "是否派工");
            map.put("targerTransition", "");
            map.put("assigeeName", outletsVO.getUserNames());
            map.put("assigeeId", outletsVO.getUserIds());
            map.put("submitType", "to_submit");
            Map<String, String> headers = new HashMap<>();
            headers.put("authorization", login.getAccess_token());
            log.info("订单创建日志-authorization{}",login.getAccess_token());
//            headers.put("authorization", "Hpq9v3R1hoCTVTaVD0p");
            headers.put("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
            try {
                Map<String, String> querys = new HashMap<>();
                org.apache.http.HttpResponse post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                int statusCode = post.getStatusLine().getStatusCode();
                String responseContent = EntityUtils.toString(post.getEntity(), "UTF-8");
                log.info("订单创建日志-服务计划{},派工订单待派工状态码:{},返回值:{}", servicePlan.getServicePlanId(), statusCode, responseContent);
                sponsorProcess = wfProperties.getWfHost() + "/je/jbpm/taskInfo/completeTask";
                map.clear();
                JSONObject res = JSONObject.parseObject(responseContent);
                Object obj = res.get("obj");
                if (obj != null) {
                    String taskId = ((JSONObject) obj).get("taskId").toString();
                    map.put("taskId", taskId);
                    map.put("pdid", "frx-ningxia-dingdan_22051015-7");
                    map.put("beanId", orderId);
                    map.put("submitComments", "同意");
                    map.put("currentTaskName", "是否派工");
                    map.put("targerTaskName", "待派工");
                    map.put("targerTransition", "to_decision2");
                    map.put("assigeeName", outletsVO.getUserNames());
                    map.put("assigeeId", outletsVO.getUserIds());
                    map.put("submitType", "to_submit");
                    querys.clear();
                    post = HttpUtils.doPost(sponsorProcess, "", "post", headers, querys, map);
                    statusCode = post.getStatusLine().getStatusCode();
                    log.info("订单创建日志-服务计划派同意派工状态码:{}", statusCode);
                } else {
                    log.error("订单创建日志-服务计划{}派同意派工taskId获取不到", servicePlan.getServicePlanId());
                    throw new Exception();
                }
            } catch (Exception e) {
                log.error("订单创建日志-{}流程引擎错误触发回滚:{}", servicePlan.getServicePlanId(), e.getMessage());
                e.printStackTrace();
                throw new Exception();    //事务回滚
            }
        }

    }


    @Override
    public void eqMesSfjj(String id) {
        servicePlanMapper.eqMesSfjj(id);
    }

    @Override
    public ServicePlanCalendarVO checkDataByParams(ServicePlanCalendarNewAO servicePlanCalendarNewAO) {
        ServicePlanCalendarVO servicePlanCalendarVO = new ServicePlanCalendarVO();
        servicePlanCalendarNewAO.setSyZhid(UserContext.getUserTenantId());
        List<ServicePlanInfoVO> servicePlanInfoVOS = servicePlanMapper.selectByParams(servicePlanCalendarNewAO);
        List<ServicePlanCalendarList> list = new ArrayList<>();
        servicePlanInfoVOS.stream().forEach(item -> {
            //List<ServicePlanCalendarList> servicePlanCalendarLists = byServicePlanCalendarNewAO(servicePlanCalendarNewAO, item);
            //Log.info("计划名称：{}", item.getPlanName());
            List<ServicePlanCalendarList> servicePlanCalendarLists = checkServicePlanCalendar(servicePlanCalendarNewAO, item);
            list.addAll(servicePlanCalendarLists);
        });
        List<ServicePlanCalendarList> servicePlanCalendarLists = new ArrayList<>();
        Map<String, List<ServicePlanCalendarList>> collect = list.stream().collect(Collectors.groupingBy(ServicePlanCalendarList::getCalendarDate));
        for (String key : collect.keySet()) {
            ServicePlanCalendarList servicePlanCalendarList = new ServicePlanCalendarList();
            servicePlanCalendarList.setCalendarDate(key);
            List<ServicePlanCalendarDetailsVO> lists = collect.get(key).stream().map(ServicePlanCalendarList::getServicePlanCalendarDetailsVO).collect(Collectors.toList());
            servicePlanCalendarList.setLists(lists);
            servicePlanCalendarLists.add(servicePlanCalendarList);
        }
        servicePlanCalendarLists.sort((ServicePlanCalendarList cale1, ServicePlanCalendarList cale2) -> cale1.getCalendarDate().compareTo(cale2.getCalendarDate()));
        servicePlanCalendarVO.setList(servicePlanCalendarLists);
        servicePlanCalendarVO.setFinishRate(finishRateNewTotal(servicePlanCalendarLists));
        return servicePlanCalendarVO;
    }

    /**
     * 获取每个服务计划对应周期日期
     *
     * @param servicePlanCalendarNewAO
     * @param servicePlanInfoVO
     * @return
     */
    public List<ServicePlanCalendarList> checkServicePlanCalendar(ServicePlanCalendarNewAO servicePlanCalendarNewAO, ServicePlanInfoVO servicePlanInfoVO) {
        ServicePlanCalendarVO vo = new ServicePlanCalendarVO();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        ServicePlanEntity servicePlanEntity = servicePlanMapper.selectById(servicePlanInfoVO.getServicePlanId());
        List<ServicePlanCalendarList> list = new ArrayList<>();
        //根据查询获取月份的开始和结束日期
        String queryTime = servicePlanCalendarNewAO.getQueryTime();
        Date parse = null;
        try {
            parse = format.parse(queryTime);
        } catch (ParseException e) {
            Log.info("获取月份错误信息：{}", e.getMessage());
        }
        Calendar cld = Calendar.getInstance();
        cld.setTime(parse);
        cld.add(Calendar.MONTH, 1);
        cld.set(Calendar.DAY_OF_MONTH, 0);
        String lastDay = sdf.format(cld.getTime());
        //Log.info("当月最后一天={}",lastDay);
        cld.set(Calendar.DAY_OF_MONTH, 1);
        String firstDay = sdf.format(cld.getTime());
        //Log.info("当月第一天={}",firstDay);

        //查询待办库里面的是否满足条件数据,历史数据
        Map param = new HashMap();
        param.put("planId", servicePlanEntity.getServicePlanId());
        param.put("firstDay", firstDay);
        param.put("lastDay", lastDay);
        List<Map<String, String>> todoLibrary = servicePlanMapper.getTodoLibrary(param);
        List<String> dates = new ArrayList<>();
        Map<String, String> orderMap = new HashMap<>();

        if (todoLibrary.size() > 0) {
            for (Map<String, String> item :
                    todoLibrary) {
                dates.add(item.get("TIME"));
                orderMap.put(item.get("TIME"), item.get("ORDER_STATUS"));
            }
        }
        //存在临时时间
        if (StringUtils.isNotBlank(servicePlanEntity.getPlanLscfsj())) {
            if (servicePlanEntity.getPlanLscfsj().contains(queryTime)) {
                dates.add(servicePlanEntity.getPlanLscfsj());
            }
        }
        String planStartTime = servicePlanEntity.getPlanStartTime();
        String planEndTime = servicePlanEntity.getPlanEndTime();
        List<String> days = com.rootcloud.common.live.DateUtil.findDates(planStartTime, planEndTime,
                String.valueOf(servicePlanEntity.getPlanCycle()), servicePlanEntity.getPlanInterval(), firstDay, lastDay);
        //Log.info("获取到日期1：" + days);
        //取出最后一个历史数据时间
        /*if (dates.size() > 0) {
            String lastTime = dates.get(dates.size() - 1).substring(0, 10);
            days.stream().forEach(item -> {
                //保留大于最后一个时间的数据
                if (item.length() >= 10) {
                    if (item.substring(0, 10).compareTo(lastTime) > 0) {
                        dates.add(item);
                    }
                }
            });
        } else {
            dates.addAll(days);
        }*/
        if (dates.size() > 0) {
            if (dates.size() < days.size()) {
                for (int i = dates.size(); i < days.size(); i++) {
                    dates.add(days.get(i));
                }
            }
        } else {
            dates.addAll(days);
        }
        //清理掉提前触发日期
        List<String> newDates = new ArrayList<>();
        String planTqcfrq = servicePlanEntity.getPlanTqcfrq();
        if (StringUtils.isNotBlank(planTqcfrq)) {
            List<String> tqfcrqList = Arrays.asList(servicePlanEntity.getPlanTqcfrq().split(","));
            if (tqfcrqList.size() > 0) {
                dates.forEach(o -> {
                    if (!tqfcrqList.contains(o)) {
                        newDates.add(o);
                    }
                });
            } else {
                newDates.addAll(dates);
            }
        } else {
            newDates.addAll(dates);
        }
        //数据状态为暂停 则取最后一条数据比对，大于这个时间的数据全部隐藏
        List<String> liss = new ArrayList<>();
        String planStatus = servicePlanEntity.getPlanStatus().toString();
        Log.info("服务计划状态：" + planStatus);
        if (ServicePlanEnums.PlanStatus.PAUSED.getCode().toString().equals(planStatus)) {
            String ztrq = servicePlanEntity.getPlanZtrq();
            Log.info("服务计划暂停日期：" + ztrq);
            List<String> ztrqList = new ArrayList<>();
            if (StringUtils.isNotBlank(ztrq)) {
                ztrqList = new ArrayList<>(Arrays.asList(ztrq.split(",")));
            }
            //获取暂停开始时间
            String start = null;
            if (ztrqList.size() > 0) {
                start = ztrqList.get(ztrqList.size() - 1);
            }
            if (StringUtils.isNotBlank(start)) {
                for (int i = 0; i < newDates.size(); i++) {
                    if (newDates.get(i).compareTo(start) < 0) {
                        liss.add(newDates.get(i));
                    }
                }
            }
        } else {
            liss.addAll(newDates);
        }
        //清理掉暂停日期的
        List<String> screenDates = new ArrayList<>();
        String ztrq = servicePlanEntity.getPlanZtrq();
        if (StringUtils.isNotBlank(ztrq)) {
            List<String> ztrqList = Arrays.asList(ztrq.split(","));
            if (ztrqList.size() > 0) {
                liss.forEach(o -> {
                    if (!ztrqList.contains(o)) {
                        screenDates.add(o);
                    }
                });
            } else {
                screenDates.addAll(liss);
            }
        } else {
            screenDates.addAll(liss);
        }
        Log.info("服务计划：" + servicePlanEntity.getPlanName() + " 获取到日期：" + screenDates);
        screenDates.stream().forEach(item -> {
            ServicePlanCalendarList servicePlanCalendarList = new ServicePlanCalendarList();
            servicePlanCalendarList.setCalendarDate(item);
            servicePlanCalendarList.setOrderStatus(orderMap.get(item));
            ServicePlanCalendarDetailsVO servicePlanCalendarDetailsVO = new ServicePlanCalendarDetailsVO();
            servicePlanCalendarDetailsVO.setDeviceCode(servicePlanInfoVO.getDeviceCode());
            servicePlanCalendarDetailsVO.setDviceName(servicePlanInfoVO.getDeviceTypeName());
            servicePlanCalendarDetailsVO.setCustomerName(servicePlanInfoVO.getCustomerName());
            servicePlanCalendarDetailsVO.setPlanStartTime(servicePlanEntity.getPlanStartTime());
            servicePlanCalendarDetailsVO.setServiceName(servicePlanEntity.getPlanName());
            servicePlanCalendarDetailsVO.setPlanEndTime(servicePlanEntity.getPlanEndTime());
            servicePlanCalendarDetailsVO.setFinishTimeNode(servicePlanEntity.getFinishTimeNode());
            String s = returnDateStatusNew(servicePlanEntity, item);
            servicePlanCalendarDetailsVO.setDateStatus(s);
            servicePlanCalendarList.setDateStatus(s);
            servicePlanCalendarList.setServicePlanCalendarDetailsVO(servicePlanCalendarDetailsVO);
            list.add(servicePlanCalendarList);
        });
        return list;
    }


    /**
     * 就算任务在某个月份所有执行的日子
     *
     * @param servicePlanCalendarNewAO
     * @param servicePlanInfoVO
     * @return
     */
    public List<ServicePlanCalendarList> calcMonthDay(ServicePlanCalendarNewAO servicePlanCalendarNewAO, ServicePlanInfoVO servicePlanInfoVO) {
        List<ServicePlanCalendarList> list = new ArrayList<>();
        ServicePlanEntity servicePlanEntity = new ServicePlanEntity();
        BeanUtils.copyProperties(servicePlanInfoVO, servicePlanEntity);
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //根据查询获取月份的开始和结束日期
        String queryTime = servicePlanCalendarNewAO.getQueryTime();
        Date parse = null;
        try {
            parse = format.parse(queryTime);
        } catch (ParseException e) {
            Log.info("获取月份错误信息：{}", e.getMessage());
        }
        Calendar cld = Calendar.getInstance();
        cld.setTime(parse);
        cld.add(Calendar.MONTH, 1);
        cld.set(Calendar.DAY_OF_MONTH, 0);
        String lastDay = sdf.format(cld.getTime());
        cld.set(Calendar.DAY_OF_MONTH, 1);
        String firstDay = sdf.format(cld.getTime());

        //查询待办库里面的是否满足条件数据,历史数据
        Map param = new HashMap();
        param.put("planId", servicePlanEntity.getServicePlanId());
        param.put("firstDay", firstDay);
        param.put("lastDay", lastDay);
        List<Map<String, String>> todoLibrary = servicePlanMapper.getTodoLibrary(param);
        List<String> dates = new ArrayList<>();
        Map<String, String> orderMap = new HashMap<>();
        if (todoLibrary.size() > 0) {
            for (Map<String, String> item :
                    todoLibrary) {
                dates.add(item.get("TIME"));
                orderMap.put(item.get("TIME"), item.get("ORDER_STATUS"));
            }
        }
        //存在临时时间
        if (StringUtils.isNotBlank(servicePlanEntity.getPlanLscfsj())) {
            dates.add(servicePlanEntity.getPlanLscfsj());
        }
        String planStartTime = servicePlanEntity.getPlanStartTime();
        String planEndTime = servicePlanInfoVO.getPlanEndTime();
        List<String> days = com.rootcloud.common.live.DateUtil.findDates(planStartTime, planEndTime,
                String.valueOf(servicePlanInfoVO.getPlanCycle()), servicePlanInfoVO.getPlanInterval(), firstDay, lastDay);
        //取出最后一个历史数据时间
        if (dates.size() > 0) {
            String lastTime = dates.get(dates.size() - 1).substring(0, 10);
            days.stream().forEach(item -> {
                //保留大于最后一个时间的数据
                if (item.length() >= 10) {
                    if (item.substring(0, 10).compareTo(lastTime) > 0) {
                        dates.add(item);
                    }
                }
            });
        } else {
            dates.addAll(days);
        }
        //Log.info("获取到日期：" + dates);
        dates.stream().forEach(item -> {
            ServicePlanCalendarList servicePlanCalendarList = new ServicePlanCalendarList();
            servicePlanCalendarList.setCalendarDate(item);
            ServicePlanCalendarDetailsVO servicePlanCalendarDetailsVO = new ServicePlanCalendarDetailsVO();
            servicePlanCalendarDetailsVO.setDeviceCode(servicePlanInfoVO.getDeviceCode());
            servicePlanCalendarDetailsVO.setDviceName(servicePlanInfoVO.getDeviceName());
            servicePlanCalendarDetailsVO.setCustomerName(servicePlanInfoVO.getCustomerName());
            servicePlanCalendarDetailsVO.setPlanStartTime(servicePlanInfoVO.getPlanStartTime());
            servicePlanCalendarDetailsVO.setServiceName(servicePlanInfoVO.getPlanName());
            servicePlanCalendarDetailsVO.setPlanEndTime(servicePlanInfoVO.getPlanEndTime());
            servicePlanCalendarDetailsVO.setFinishTimeNode(servicePlanInfoVO.getFinishTimeNode());
            servicePlanCalendarDetailsVO.setOrderStatus(orderMap.get(item));
            String s = returnDateStatusNew(servicePlanEntity, item);
            servicePlanCalendarDetailsVO.setDateStatus(s);
            servicePlanCalendarList.setDateStatus(s);
            servicePlanCalendarList.setServicePlanCalendarDetailsVO(servicePlanCalendarDetailsVO);
            list.add(servicePlanCalendarList);
        });
        return list;
    }


    /**
     * 根据计划的名字获取发给用户的短息内容
     *
     * @param entity
     * @param userInfoEntities
     * @return
     */
    public String getMessageContent(ServicePlanEntity entity, List<NxUserInfoEntity> userInfoEntities) {
        //todo 模板内容未定，消息内容具体怎么拼接还需要产品确认
        String result = "";
        for (NxUserInfoEntity userInfoEntity : userInfoEntities) {
            if (userInfoEntity.getServicePlanId().equals(entity.getServicePlanId())) {
                result = userInfoEntity.getInfoContent();
                break;
            }
        }
        if (StringUtils.isNotBlank(result)) {
            result = nxUserInfoService.getTemplateContent(entity.getServicePlanId());
        }
        return result;
    }


    /**
     * 手动触发服务计划订单
     *
     * @param planId
     */
    @Override
    public JsonResult<Object> manualTrigger(String planId) {
        Map param = new HashMap();
        //查询服务计划
        ServicePlanEntity servicePlanEntity = servicePlanMapper.selectById(planId);
        //获取下一次周期时间
        String nextCycleTime = servicePlanEntity.getNextCycleTime();
        if (StringUtils.isNotBlank(nextCycleTime)) {
            param.put("nextCycleTime", nextCycleTime);
            String previousTime = getPreviousDateByStartDate(DateUtil.parse(nextCycleTime, DateUtil.PATTERN_CLASSICAL_SIMPLE), servicePlanEntity);
            param.put("previousTime", previousTime);
        }
        param.put("planId", planId);

        List<Map<String, String>> exsitOrder = servicePlanMapper.getPlanExsitOrder(param);
        if (exsitOrder.size() > 0) {
            //存在订单号
            return JsonResult.failed("500", "当前期计划已存在订单");
        } else {
            try {
                //发送
                nxUserInfoService.setReminders(planId);
                //如果发送成功修改状态
                updateServicePlanStatus(planId, 1, ServicePlanEnums.PlanStatus.FINISHED.getCode(), true, null);
                return JsonResult.success("操作成功");
            } catch (Exception e) {
                Log.info("手动创建发送以及修改状态错误信息:{}", e);
                return JsonResult.failed("500", "手动创建订单出现异常");
            }
        }
    }

    /**
     * 更新服务计划下一周期日期
     */
    @Override
    public void updateDate() {
        List<ServicePlanEntity> list = servicePlanMapper.selectData();
        if (list.size() > 0) {
            list.forEach(o -> {
                String date = getNextDateByStartDate(o);
                if (!date.equals(o.getNextCycleTime())) {
                    o.setNextCycleTime(date);
                    Log.info("服务计划id：" + o.getServicePlanId() + "，计划周期：" + o.getPlanCycle() + "，计划间隔：" + o.getPlanInterval() + "，开始日期：" + o.getPlanStartTime() + "，原下一个周期日期：" + o.getNextCycleTime() + "，需修改成的日期：" + date);
                    servicePlanMapper.updateData(o.getServicePlanId(), date);
                }
            });
        }
    }

    /**
     * 暂停服务计划
     *
     * @param planId
     * @param type
     * @return
     */
    @Override
    public JsonResult<Object> pause(String planId, String type) {
        if (StringUtils.isNotBlank(planId)) {
            ServicePlanEntity info = servicePlanMapper.selectById(planId);
            if (info == null) {
                return JsonResult.failed("450", "获取数据失败");
            }
            Log.info("获取到服务计划：" + info);
            //判断是否删除
            if ("0".equals(info.getSyStatus())) {
                return JsonResult.failed("450", "服务计划已被删除");
            }
            String date = null;
            //暂停日期
            List<String> ztrqList = new ArrayList<>();
            String planZtrq = info.getPlanZtrq();
            //暂停
            if (ServicePlanEnums.PlanStatus.PAUSED.getCode().toString().equals(type)) {
                if (StringUtils.isNotBlank(planZtrq)) {
                    ztrqList = new ArrayList<>(Arrays.asList(planZtrq.split(",")));
                }
                ztrqList.add(DateUtil.format(DateUtil.getNextWorkday(), DateUtil.PATTERN_CLASSICAL_SIMPLE));
            }
            //启动服务计划
            if (ServicePlanEnums.PlanStatus.IN_PROGRESS.getCode().toString().equals(type)) {
                //修改下一周期时间
                date = getNextDateByStartDateNew(info);
                //启动服务计划计算出暂停日期
                if (StringUtils.isNotBlank(planZtrq)) {
                    ztrqList = new ArrayList<>(Arrays.asList(planZtrq.split(",")));
                    if (ztrqList.size() > 0) {
                        String startDate = ztrqList.get(ztrqList.size() - 1);
                        ztrqList.remove(startDate);
                        String endDate = DateUtil.format(new Date());
                        List<String> rqList = DateUtil.getDays(startDate, endDate);
                        if (rqList.size() > 0) {
                            ztrqList.addAll(rqList);
                        }
                    }
                }
            }
            planZtrq = ztrqList.stream().collect(Collectors.joining(","));
            Log.info("修改服务计划：planId" + planId + ",NEXT_CYCLE_TIME：" + date + ",PLAN_STATUS:" + type + ",PLAN_ZTRQ：" + planZtrq);
            servicePlanMapper.updateServicePlanStatus(planId, date, type, planZtrq);
            return JsonResult.success();
        }
        return JsonResult.failed("450", "获取数据失败");
    }

    /**
     * 提前触发服务计划
     *
     * @param planId
     * @return
     */
    @Override
    public JsonResult<Object> pretrigger(String planId) {
        if (StringUtils.isNotBlank(planId)) {
            ServicePlanEntity info = servicePlanMapper.selectById(planId);
            if (info == null) {
                return JsonResult.failed("450", "获取数据失败");
            }
            //获取提前触发日期字段
            List<String> tqcfrqList = new ArrayList<>();
            String tqcfrq = info.getPlanTqcfrq();
            if (StringUtils.isNotBlank(tqcfrq)) {
                tqcfrqList = Arrays.asList(info.getPlanTqcfrq().split(","));
            }
            String nextCycleTime = info.getNextCycleTime();

            //判断是否提前触发过
            Map param = new HashMap();
            if (StringUtils.isNotBlank(nextCycleTime)) {
                param.put("nextCycleTime", nextCycleTime);
                String previousTime = getPreviousDateByStartDate(DateUtil.parse(nextCycleTime, DateUtil.PATTERN_CLASSICAL_SIMPLE), info);
                param.put("previousTime", previousTime);
            }
            Log.info("获取到数据：" + param);
            param.put("planId", planId);
            List<Map<String, String>> exsitOrder = servicePlanMapper.getPlanExsitOrder(param);
            if (exsitOrder.size() > 0) {
                return JsonResult.failed("450", "该服务计划已提前触发过！");
            }
            //判断上一个服务计划是否完成
            String orderStatus = servicePlanMapper.lastServicePlanOrderStatus(planId);
            if (StringUtils.isNotBlank(orderStatus)) {
                //已完成订单才支持提前触发
                if (OrderEnums.OrderStatus.DONE.getCode().equals(orderStatus)) {
                    List<ServicePlanVo> servicePlanVos = servicePlanMapper.selectExecutedServicePlanByPlanId(planId);
                    if (servicePlanVos.size() > 0) {
                        for (ServicePlanVo item : servicePlanVos) {
                            try {
                                Log.info("服务计划：" + item);
                                //判断是否临时触发时间的服务计划
                                boolean tempTime = false;
                                if (StringUtils.isNotBlank(item.getPlanLscfsj())) {
                                    tempTime = true;
                                }
                                //根据服务计划和开始日期查询代办库，看服务计划是否执行生成了订单
                                Integer count = servicePlanMapper.checkOrder(item.getServicePlanId(), info.getNextCycleTime());
                                if (count > 0) {
                                    return JsonResult.failed("450", "该服务计划已提前触发过！");
                                } else {
                                    //发送
                                    nxUserInfoService.setReminders(item.getServicePlanId());
                                    //如果发送成功修改状态
                                    updateServicePlanStatus(item.getServicePlanId(), 1, ServicePlanEnums.PlanStatus.FINISHED.getCode(), true, null);
                                    //记录提前触发日期
                                    //格式化日期 只保留年月日
                                    tqcfrqList.add(nextCycleTime);
                                    servicePlanMapper.updateTqcfrq(planId, tqcfrqList.stream().collect(Collectors.joining(",")));
                                    return JsonResult.success("操作成功");
                                }
                            } catch (Exception e) {
                                Log.info("发送以及修改状态错误信息:{}", e);
                            }
                        }
                    }
                } else {
                    return JsonResult.failed("450", "上一次服务计划未完成，请完成后在操作！");
                }
            } else {
                try {
                    //发送
                    nxUserInfoService.setReminders(planId);
                    //如果发送成功修改状态
                    updateServicePlanStatus(planId, 1, ServicePlanEnums.PlanStatus.FINISHED.getCode(), true, null);
                    //记录提前触发日期
                    //格式化日期 只保留年月日
                    tqcfrqList.add(nextCycleTime);
                    servicePlanMapper.updateTqcfrq(planId, tqcfrqList.stream().collect(Collectors.joining(",")));
                    return JsonResult.success("操作成功");
                } catch (Exception e) {
                    Log.info("手动创建发送以及修改状态错误信息:{}", e);
                    return JsonResult.failed("500", "手动创建订单出现异常");
                }
            }
        }
        return JsonResult.failed("450", "提前触发失败");
    }

    /**
     * 获取下一个周期时间
     *
     * @param servicePlan
     * @return
     */
    public String getNextDateByStartDate(ServicePlanEntity servicePlan) {
        Date endDate = null;
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
            //获取当前日期
            String current = df.format(new Date());
            Date currentDate = df.parse(current);
            //获取计划开始日期
            endDate = df.parse(servicePlan.getPlanStartTime());
            Integer planCycle = servicePlan.getPlanCycle();
            Integer planInterval = servicePlan.getPlanInterval();
            do {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(endDate);
                //日
                if (ServicePlanEnums.PlanCycle.DAY.getCode().equals(planCycle)) {
                    calendar.add(Calendar.DATE, planInterval);
                }
                //周
                if (ServicePlanEnums.PlanCycle.WEEK.getCode().equals(planCycle)) {
                    calendar.add(Calendar.DATE, 7 * planInterval);
                }
                //月
                if (ServicePlanEnums.PlanCycle.MONTH.getCode().equals(planCycle)) {
                    calendar.add(Calendar.MONTH, planInterval);
                }
                //年
                if (ServicePlanEnums.PlanCycle.YEAR.getCode().equals(planCycle)) {
                    calendar.add(Calendar.YEAR, planInterval);
                }
                endDate = calendar.getTime();
                if (endDate.getTime() == currentDate.getTime()) {
                    Log.info("今天需要执行的服务计划：" + servicePlan);
                }
            } while (endDate.getTime() <= currentDate.getTime());
            return df.format(endDate);
        } catch (Exception e) {
            Log.error("获取数据失败，失败原因：" + e.getMessage());
            return null;
        }

    }


}
