package com.rootcloud.domain.service.todo.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.ResourceUtil;
import com.rootcloud.common.enums.OrderEnums;
import com.rootcloud.common.enums.ServicePlanEnums;
import com.rootcloud.common.enums.StatusCodeEnum;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.config.exception.BusinessException;
import com.rootcloud.domain.entity.MyAdviceEntity;
import com.rootcloud.domain.entity.MyMessageEntity;
import com.rootcloud.domain.entity.NxJeDeviceEntity;
import com.rootcloud.domain.entity.NxTodoLibrary;
import com.rootcloud.domain.entity.ds.DeviceListEntity;
import com.rootcloud.domain.mapper.NxTodoLibraryMapper;
import com.rootcloud.domain.service.gkjk.NxJeDeviceService;
import com.rootcloud.domain.service.todo.NxTodoLibraryService;
import com.rootcloud.vo.todo.TodoVo;
import jline.internal.Log;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * <p>
 * 宁夏_待办库 服务实现类
 * </p>
 *
 * @author yangyijun
 * @since 2022-05-27
 */
@Service("nxTodoLibraryServiceImpl")
public class NxTodoLibraryServiceImpl extends ServiceImpl<NxTodoLibraryMapper, NxTodoLibrary> implements NxTodoLibraryService {


    private final NxTodoLibraryMapper nxTodoLibraryMapper;
    @Resource(name = "nxJeDeviceServiceImpl")
    private final NxJeDeviceService nxJeDeviceService;

    public NxTodoLibraryServiceImpl(NxTodoLibraryMapper nxTodoLibraryMapper, NxJeDeviceService nxJeDeviceService) {
        this.nxTodoLibraryMapper = nxTodoLibraryMapper;
        this.nxJeDeviceService = nxJeDeviceService;
    }


    @Override
    public IPage<TodoVo> getPage(int page, int size, String queryData, String userId) {
        IPage<TodoVo> iPage = new Page(page, size);
        IPage<TodoVo> listPage = nxTodoLibraryMapper.getPageNew(iPage, queryData, userId, UserContext.getUserTenantId());
        Log.info("获取到用户id：" + userId + "，租户ID：" + UserContext.getUserTenantId());
        List<TodoVo> records = listPage.getRecords();
        records.stream().forEach(item -> {
            if (ServicePlanEnums.MessageCodeType.MAINTENANCE_MESSAGE.getCode().equals(item.getAdviceCode())) {
                String deviceCode = item.getDeviceCode();
                NxJeDeviceEntity nxJeDeviceEntity = nxJeDeviceService.selectBySbbm(deviceCode);
                if (!ObjectUtils.isEmpty(nxJeDeviceEntity)) {
                    item.setDeviceId(nxJeDeviceEntity.getNxJeDeviceId());
                }
            }
        });
        return listPage;
    }


    @Override
    public void updateStatus(String nxTodoLibraryId) {
        NxTodoLibrary nxTodoLibrary = getById(nxTodoLibraryId);
        if (ObjectUtils.isEmpty(nxTodoLibrary)) {
            //throw new BusinessException(StatusCodeEnum.TODO_NOT_EXIST.getRet(), StatusCodeEnum.TODO_NOT_EXIST.getMessage());
            throw new BusinessException(StatusCodeEnum.TODO_NOT_EXIST.getRet(), ResourceUtil.getI18nMessage("18001"));
        }
        if (OrderEnums.TodoStatus.RESOLVED.getCode().equals(nxTodoLibrary.getStatus())) {
            //throw new BusinessException(StatusCodeEnum.TODO_NOT_STATUS.getRet(), StatusCodeEnum.TODO_NOT_STATUS.getMessage());
            throw new BusinessException(StatusCodeEnum.TODO_NOT_STATUS.getRet(), ResourceUtil.getI18nMessage("18002"));
        }
        nxTodoLibrary.setStatus(OrderEnums.TodoStatus.RESOLVED.getCode());
        nxTodoLibraryMapper.updateById(nxTodoLibrary);
    }


    @Override
    public void updateLibrary(String nxTodoLibraryId, String nxOrderId) {
        NxTodoLibrary nxTodoLibrary = getById(nxTodoLibraryId);
        if (ObjectUtils.isEmpty(nxTodoLibrary)) {
            throw new BusinessException(StatusCodeEnum.TODO_NOT_EXIST.getRet(), StatusCodeEnum.TODO_NOT_EXIST.getMessage());
        }
        nxTodoLibrary.setNxOrderId(nxOrderId);
        nxTodoLibraryMapper.updateById(nxTodoLibrary);
    }

    @Override
    public void insertMessageLibarary(MyMessageEntity myMessageEntity) {
        NxTodoLibrary library = new NxTodoLibrary();
        library.setTitle(myMessageEntity.getMessageClass());
        library.setContent(myMessageEntity.getMessageContent());
        library.setDeviceName(myMessageEntity.getMessageObjectName());
        library.setDeviceCode(myMessageEntity.getMessageObjectCode());
        library.setOperation(myMessageEntity.getMessageName());
        library.setTime(myMessageEntity.getMessageTime());
        library.setSyCreatetime(DateUtil.format(new Date()));
        library.setNxTodoLibraryId(UUID.randomUUID().toString().substring(0, 10));
        library.setAdviceCode(myMessageEntity.getAdviceCode());
        library.setTypeCode(myMessageEntity.getTypeCode());
        library.setSignData(myMessageEntity.getSignData());
        library.setStatus("0");
        library.setUserid(myMessageEntity.getUserId());
        library.setGzdj(myMessageEntity.getGzdj());//故障等级
        library.setBjbm(myMessageEntity.getBjbm());//报警编码
        library.setBjsj(myMessageEntity.getBjsj());//报警时间
        library.setGcs(myMessageEntity.getOrderAssignengineer());//工程师
        library.setDdbh(myMessageEntity.getOrderOrdernumber());//订单编号
        this.save(library);
    }

    @Override
    public void insertAdviceToLibarary(MyAdviceEntity myAdviceEntity) {
        NxTodoLibrary library = new NxTodoLibrary();
        library.setTitle(myAdviceEntity.getAdviceClass());
        if (ServicePlanEnums.MessageCodeType.SPARE_PARTS_LIFE.getCode().equals(myAdviceEntity.getAdviceCode())) {
            library.setTitle(myAdviceEntity.getAdviceName());
        }
        library.setContent(myAdviceEntity.getAdviceContent());
        library.setDeviceName(myAdviceEntity.getAdviceObjectName());
        library.setDeviceCode(myAdviceEntity.getAdvicObjectCode());
        library.setOperation(myAdviceEntity.getAdviceName());
        library.setTime(myAdviceEntity.getAdviceTime());
        library.setSyCreatetime(DateUtil.format(new Date()));
        library.setAdviceCode(myAdviceEntity.getAdviceCode());
        library.setTypeCode(myAdviceEntity.getTypeCode());
        library.setSignData(myAdviceEntity.getSignData());
        library.setStatus("0");
        library.setUserid(myAdviceEntity.getUserId());
        library.setSn(myAdviceEntity.getSn());//SN
        library.setPartAzwh(myAdviceEntity.getPartAzwh());//安装位号
        library.setPartGzsc(myAdviceEntity.getPartGzsc());//工作时长
        library.setPlanName(myAdviceEntity.getPlanName());//计划名称
        library.setPlanEndTime(myAdviceEntity.getPlanEndTime());//计划完成时间
        library.setPartXtmk(myAdviceEntity.getPartXtmk());//系统模块
        library.setPartBjmc(myAdviceEntity.getPartBjmc());//备件名称
        if (ServicePlanEnums.MessageCodeType.MAINTENANCE_MESSAGE.getCode().equals(myAdviceEntity.getAdviceCode())) {
            library.setServicePlanId(myAdviceEntity.getSignData());
        }
        library.setNxTodoLibraryId(UUID.randomUUID().toString().substring(0, 10));
        this.save(library);
    }

    @Override
    public List<NxTodoLibrary> selectBySign(String adviceCode, String signData, String time) {
        QueryWrapper<NxTodoLibrary> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ADVICE_CODE", adviceCode);
        queryWrapper.eq("SIGN_DATA", signData);
        queryWrapper.eq("TIME", time);
        return nxTodoLibraryMapper.selectList(queryWrapper);
    }

    @Override
    public void delTodoLibrary(String nxTodoLibraryId) {
        nxTodoLibraryMapper.deleteById(nxTodoLibraryId);
    }

    @Override
    public List<String> selectByData(String adviceCode, List<String> list) {
        //Log.info("【代办表】查询开始时间 ==> {}", DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL));
        QueryWrapper<NxTodoLibrary> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ADVICE_CODE", adviceCode);
        queryWrapper.in("SIGN_DATA", list);
        List<NxTodoLibrary> ls = nxTodoLibraryMapper.selectList(queryWrapper);
        //Log.info("【代办表】查询结束时间 ==> {}", DateUtil.format(new Date(), DateUtil.PATTERN_CLASSICAL));
        return ls.stream().map(NxTodoLibrary::getSignData).collect(Collectors.toList());
    }


    public NxTodoLibrary getById(String nxTodoLibraryId) {
        QueryWrapper<NxTodoLibrary> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("NX_TODO_LIBRARY_ID", nxTodoLibraryId);
        NxTodoLibrary nxTodoLibrary = nxTodoLibraryMapper.selectOne(queryWrapper);
        return nxTodoLibrary;
    }
}
