package com.rootcloud.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.rootcloud.config.auth.UserContext;
import com.rootcloud.domain.dto.CarouselDTO;
import com.rootcloud.domain.dto.ImageUrlsDTO;
import com.rootcloud.domain.entity.CarouselEntity;
import com.rootcloud.domain.mapper.CarouselMapper;
import com.rootcloud.domain.vo.CarouselVO;
import com.rootcloud.service.CarouselService;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.Asserts;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author w-yanxiang
 * @Date 2022/6/16 10:19
 * @Version 1.0
 * @Description
 */
@Service
@Slf4j
public class CarouselServiceImpl implements CarouselService {

    @Resource
    private CarouselMapper carouselMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveOrsubmitCarousel(CarouselDTO carouselDTO) {
            if(carouselDTO.getOperationType().equals("2")){
                Asserts.notBlank(carouselDTO.getTimeGap(),"timeGap");
            }
            String now = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            String syZhid = UserContext.getTenantId();
            CarouselEntity carouselEntity =new CarouselEntity();
            carouselEntity.setSyZhid(syZhid);
            carouselMapper.delete(Wrappers.lambdaQuery(carouselEntity));
            carouselEntity.setLbxxCslx("timeGap");
            carouselEntity.setLbxxCsz(carouselDTO.getTimeGap());
            carouselEntity.setLbxxSfsx(carouselDTO.getOperationType());
            carouselEntity.setSyCreatetime(now);
            carouselEntity.setSyCreateuserid(UserContext.getUserId());
            carouselEntity.setSyZhid(syZhid);
            carouselMapper.insert(carouselEntity);
            if(CollectionUtils.isNotEmpty(carouselDTO.getImageUrls())){
                if(carouselDTO.getImageUrls().size()<=20){
                    carouselDTO.getImageUrls().forEach(v->{
                        CarouselEntity carouselEntitys =new CarouselEntity();
                        carouselEntitys.setLbxxCslx("imageUrl");
                        carouselEntitys.setLbxxCsz(JSONObject.toJSONString(v));
                        carouselEntitys.setLbxxSfsx(carouselDTO.getOperationType());
                        carouselEntitys.setSyCreatetime(now);
                        carouselEntitys.setSyCreateuserid(UserContext.getUserId());
                        carouselEntitys.setSyZhid(syZhid);
                        carouselMapper.insert(carouselEntitys);
                    });
                }else {
                    throw new RuntimeException("轮播图片数据过多");
                }
            }
    }

    @Override
    public CarouselVO findCarouselInfoData(String source) {
        LambdaQueryWrapper<CarouselEntity> queryWrapper = Wrappers.lambdaQuery(CarouselEntity.class);
        if("APP".equalsIgnoreCase(source)){
           // queryWrapper.eq(CarouselEntity::getLbxxSfsx,"2");   //没有入口配置生效
        }
        List<CarouselEntity> entities = carouselMapper.selectList(queryWrapper);
        CarouselVO carouselVO = new CarouselVO();
        if(CollectionUtils.isNotEmpty(entities)){
            carouselVO.setTimeGap(entities.stream().filter(v->"timeGap".equals(v.getLbxxCslx())).findFirst().orElse(new CarouselEntity()).getLbxxCsz());
            List<CarouselEntity> imageUrls = entities.stream().filter(v -> "imageUrl".equals(v.getLbxxCslx())).collect(Collectors.toList());
            List<JSONObject> collect = imageUrls.stream().map(v -> JSONObject.parseObject(v.getLbxxCsz())).collect(Collectors.toList());
            carouselVO.setImageUrls(JSONObject.parseArray(JSONObject.toJSONString(collect),ImageUrlsDTO.class));
        }
        return carouselVO;
    }

    @Override
    public void saveOrUpdateOneCarousel(String id, String imageUrl) {
        if(StringUtils.isNotBlank(id)){
            if(carouselMapper.selectCount(Wrappers.lambdaQuery(CarouselEntity.class).eq(CarouselEntity::getLbxxCslx,"timeGap"))>19){
                throw new RuntimeException("轮播图片数据过多");
            }
            CarouselEntity carouselEntity = new CarouselEntity();
            carouselEntity.setNxgxLbxxId(id);
            carouselEntity.setLbxxCsz(imageUrl);
            carouselMapper.updateById(carouselEntity);
        }else {
            CarouselEntity carouselEntity = new CarouselEntity();
            carouselEntity.setSyCreateuserid(UserContext.getUserId());
            carouselEntity.setSyCreatetime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
            carouselEntity.setLbxxCslx("imageUrl");
            carouselEntity.setLbxxCsz(imageUrl);
            carouselEntity.setLbxxSfsx("1");
            carouselMapper.insert(carouselEntity);
        }
    }

    @Override
    public void deleteCarousel(List<String> ids) {
        carouselMapper.deleteBatchIds(ids);
    }
}
