package com.rootcloud.service.impl;

import com.alibaba.fastjson.JSON;
import com.rootcloud.common.util.EncryptUtil;
import com.rootcloud.common.util.MapBuilder;
import com.rootcloud.domain.vo.RepPrinterReportVO;
import com.rootcloud.service.EmailService;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.WebResource;
import io.undertow.client.ClientResponse;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Map;

@Service("emailServices")
public class EmailServiceImpl implements EmailService {

    private static final Logger logger = LoggerFactory.getLogger(EmailServiceImpl.class);

    /*
    @Value("${mail.server.path}")
    private String mailServerPath;

    @Value("${mail.link.path}")
    private String mailLinkPath;

    @Value("${mail.link.path2}")
    private String mailLinkPath2;

    @Value("${mail.link.path3}")
    private String mailLinkPath3;

    @Value("${mail.link.path4}")
    private String mailLinkPath4;
    */

    @Value("${mail.iam.server_path}")
    private String mailServerPath;

    @Value("${mail.iam.link_path}")
    private String mailLinkPath;

    /**
     * 给指定邮箱发送用户运维报告
     */
    public String mailUserReport(RepPrinterReportVO vo) throws Exception {

        if (StringUtils.isBlank(vo.getEmails())) {
            throw new Exception("发送邮件地址为空");
        }

        EncryptUtil encryptUtil = EncryptUtil.getInstance() ;
        String[] emailArray = vo.getEmails().split(",");
        String id = encryptUtil.DESencode(vo.getId()+"", "paras");
        String tenantId = encryptUtil.DESencode(vo.getTenantId()+"", "paras");
        String link = "";
        for(String to : emailArray) {
            //标题
            String title = vo.getTitle().replace("打印机运维报告", "");
            // 内容
            String text = vo.getTitle();
            // 内容的链接
            link = mailLinkPath.replace("{0}", id).replace("{1}", tenantId);

            logger.info("mailUserReport start getMailMap");
            Map<String, Object> params = getMailMap(to, title, text, link);
            logger.info("mailUserReport end getMailMap ="+params.toString());


            logger.info("mailUserReport create Client");
            Client client = Client.create();
            client.setConnectTimeout(60 * 1000);

            logger.info("mailUserReport client get resource");
            WebResource webResource = client.resource(mailServerPath);
            logger.info("mailUserReport client get resource mailServerPath="+mailServerPath);
            String paramsString = JSON.toJSONString(params);
            webResource.type("application/json").post(ClientResponse.class, paramsString);
            logger.info("mailUserReport post mailServerPath="+mailServerPath+" params="+paramsString);
            logger.info("当前产线运维发送成功!");

        }
        return link;
    }


    private static Map<String, Object> getMailMap(String to, String title, String text, String link) {
        return MapBuilder.of(5).put("templateName", "nxgxPrinterReport").put("to", to)
                .put("subjectKey", "email.printer.report.title.zh_CN").put("subjectKeyValue", Arrays.asList(title))
                .put("data", MapBuilder.of(2).put("printerReportUrl", link).put("printerReportText", text).build())
                .build();
    }

    /*
    public static void main(String[] args) {
        // url配置，以下测试地址，生产地址为https://openapi.irootech.com/base-notification/api/send-mail
        String url = "http://base-notification.bdn-base-alauda-qa.irootechapp.com/api/send-mail";
        // 邮箱为单个
        String to = "8274862@qq.com";
        // 标题
        String title = "2020年3月装备成形工厂2";
        // 内容
        String text = "2020年3月装备成形工厂2打印机运维报告";
        // 内容的链接
        String link = "http://iof.kocelcloud.cn/index-3.html?id={0}&tenantId={1}";

        Map<String, Object> params = getMailMap(to, title, text, link);

        try {
            Client client = Client.create();
            client.setConnectTimeout(60 * 1000);
            WebResource webResource = client.resource(url);
            webResource.type("application/json").post(ClientResponse.class, JSON.toJSONString(params));
            System.out.println("发送成功!");
        } catch (Exception e) {

        }
    }
    */


}
