package com.rootcloud.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.rootcloud.common.Constants;
import com.rootcloud.common.DateUtil;
import com.rootcloud.common.HttpClientUtil;
import com.rootcloud.common.TokenUtil;
import com.rootcloud.common.enums.HttpRequestMethedEnum;
import com.rootcloud.common.util.DateUtils;
import com.rootcloud.common.util.PlatformDeviceTransUtil;
import com.rootcloud.common.vo.PrinterWorkBoxSize;
import com.rootcloud.domain.dao.NxDeviceDao;
import com.rootcloud.domain.dao.PrinterEfficiencyWorkDao;
import com.rootcloud.domain.dao.RepPrinterStarEvaluateDAO;
import com.rootcloud.domain.mapper.DeviceMapper;
import com.rootcloud.domain.vo.*;
import com.rootcloud.service.ConfigService;
import com.rootcloud.service.PrinterEfficiencyService;
import com.rootcloud.web.dto.PrinterReportDTO;
import jline.internal.Log;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service("efficiency")
public class PrinterEfficiencyServiceImpl implements PrinterEfficiencyService {

    private static final Logger logger = LoggerFactory.getLogger(PrinterEfficiencyServiceImpl.class);

    @Autowired
    private PrinterEfficiencyWorkDao printerEfficiencyWorkDao;

    @Autowired
    private NxDeviceDao nxDeviceDao;

    @Autowired
    private RepPrinterStarEvaluateDAO starEvaluateDAO;
    @Resource(name = "configServices")
    private ConfigService configService;

    @Autowired
    private TokenUtil tokenUtil;

    @Autowired
    private DeviceMapper deviceMapper;

    @Value("${os.iam.domain}")
    private String reqUrl;
    private static final String PRINTER_DEVICE_TYPE = "printer_device_type";

    public static final String DATE_FORMAT_YYYY_MM_DD = "yyyy-MM-dd HH:mm:ss";

    public List<Map<String, Object>> getPrinterEfficiencyData() {
        LocalDate localDate = LocalDate.now().minusDays(1);
        return printerEfficiencyWorkDao.getPrinterEfficiencyData(localDate.getYear(), localDate.getMonthValue(), localDate.getDayOfMonth());
    }

    private boolean isDaylight(TimeZone zone,Date date) {
        return zone.useDaylightTime()&&zone.inDaylightTime(date);
    }


    /** 单台机用，实际从0-24点内的有效打印时间、打印效率*/
//    public void run0(Calendar calendar) throws Exception {
//        int year = calendar.get(Calendar.YEAR);
//        int month = calendar.get(Calendar.MONTH) + 1;
//        int day = calendar.get(Calendar.DAY_OF_MONTH);
//        // 需要从ES中查询前一天00-24内数据
//        LocalDateTime startDatetime = LocalDateTime.of(year, month, day, 0, 0, 0);
//        LocalDateTime endDatetime = startDatetime.plusHours(24);
//        NxDeviceVO v = new NxDeviceVO();
//        v.setModeID("197ZUeGcXkY");
//        v.setDeviceTypeName("3DP打印机");
//        //1.取出打印机所有的设备编号
//        List<NxDeviceVO> list = nxDeviceDao.getDeviceList(v);
//        Log.info("取出打印机所有的设备编号返回数据:" + list.size());
//        List<PrinterEfficiencyWorkVO> resultList = new ArrayList<>(16);
//        for (int i = 0; i < list.size(); i++) {
//            String deviceCode = list.get(i).getDeviceCode();
//            String modelId = list.get(i).getModeID();
//            String thingId = list.get(i).getThingID();
//            String deviceTypeId = list.get(i).getDeviceTypeId();
//            try {
//                resultList.addAll(processDeviceHistoryGK0(LocalDate.of(year, month, day), startDatetime, endDatetime,
//                        deviceCode, deviceTypeId, modelId, thingId));
//            } catch (Exception ex) {
//                Log.error("本次错误,设备:" + deviceCode + "|error:" + ex.toString());
//            }
//        }
//        if (CollectionUtils.isEmpty(resultList)) {
//            Log.info("本次resultList获取0条");
//            return;
//        }
//
//        Log.info("本次resultList获取" + resultList.size(), resultList);
//        //保存到临时表
//        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
//        String time = dateFormat.format(calendar.getTime());
//        Map<String, Object> param = new HashMap<>();
//        param.put("time", time + " 00:00:00");
//        param.put("resultList", resultList);
//        param.put("syCreatetime", new Date());
//        deviceMapper.clearDeviceTimesDay(param);
//        deviceMapper.batchAddDeviceTimesDay(param);
//    }

    /**
     * 需要统计一下月时长 */
    public void run0_statistics_month(Calendar calendar)throws Exception {
        //跑完后，需要统计一下月时长
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM");
        String month = dateFormat.format(calendar.getTime());
        Map<String, Object> param = new HashMap<>();
        param.put("month", month);
        List<Map<String, String>> statisticsDeviceTimesMonth = deviceMapper.statisticsDeviceTimesMonth(param);
        if (statisticsDeviceTimesMonth.size() > 0) {
            for (Map<String, String> map : statisticsDeviceTimesMonth) {
                String deviceId = map.get("deviceId");
                param.put("deviceId", deviceId);
                param.put("onlinedTimes", map.get("onlinedTimes"));
                param.put("time", month + "-01 00:00:00");
                List<Map<String, String>> getDeviceTimesMonth = deviceMapper.getDeviceTimesMonth(param);
                if (getDeviceTimesMonth != null && getDeviceTimesMonth.size() > 0) {
                    deviceMapper.updateDeviceTimesMonth(param);
                } else {
                    deviceMapper.insertDeviceTimesMonth(param);
                }
            }
        }
    }

//    public void runEurop0(Calendar calendar) throws Exception{
//        Integer year = calendar.get(Calendar.YEAR);
//        Integer month = calendar.get(Calendar.MONTH) + 1;
//        Integer day = calendar.get(Calendar.DAY_OF_MONTH);
//
//
//        String mon ="";
//        String days ="";
//        mon = month<10?"0"+month.toString():month.toString();
//        days = day<10?"0"+day.toString():day.toString();
//        String startTime = year.toString() +"-" + mon +"-" + days.toString() + " " + "00:00:00"; //2021-01-12 00:00:00
//        String endTime = year.toString() +"-" + mon + "-" + days.toString() + " " + "23:59:59"; //2021-01-12 23:59:59
//        String startDate = year.toString() +"-" + mon +"-" + days;
//
//        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//        //sf.setTimeZone(TimeZone.getTimeZone("GMT+0"));
////  	Date dateTime=sf.parse("2016-11-6 5:59:59");
////        Date d1=sf.parse("2016-03-13 6:59:59");//false
////        Date d1 = new Date();
//        LocalDateTime currentDatetime = LocalDateTime.now(ZoneId.of("Europe/Berlin"));  //2022-09-12 06:00:00
//        Date d1 = Date.from(currentDatetime.atZone(ZoneId.systemDefault()).toInstant()); //欧洲时间
//        //LocalDateTime endDatetime = currentDatetime.plusHours(24);  //2022-09-13 06:00:00
//        TimeZone york = TimeZone.getTimeZone("Europe/Berlin"); //GMT-1
//        //System.out.println("目标时区是否使用了夏令时:"+isDaylight(york, d1));
//
//        LocalDateTime startDatetime;  //2022-09-12 06:00:00
//        LocalDateTime endDatetime;  //2022-09-13 06:00:00
//        if(isDaylight(york, d1))
//        {
//            //夏令时
//            startDatetime = LocalDateTime.of(year,month,day,6,0,0);  //2022-09-12 06:00:00  -->2022-09-12 00:00:00
//            endDatetime = startDatetime.plusHours(24);  //2022-09-13 06:00:00  -->2022-09-13 00:00:00
//        }
//        else
//        {
//            //夏令时
//            startDatetime = LocalDateTime.of(year,month,day,7,0,0);  //2022-11-12 07:00:00  -->2022-09-12 00:00:00
//            endDatetime = startDatetime.plusHours(24);  //2022-11-13 07:00:00  -->2022-09-13 00:00:00
//        }
//        // 夏季东八区时间  2022-09-12 06:00:00~2022-09-13 06:00:00 对应奥钢联时间的 2022-09-12 00:00:00~2022-09-13 00:00:00
//        // 冬季东八区时间  2022-09-12 07:00:00~2022-09-13 07:00:00 对应奥钢联时间的 2022-09-12 00:00:00~2022-09-13 00:00:00
////        LocalDateTime startDatetime = LocalDateTime.of(year,month,day,6,0,0);  //2022-09-12 06:00:00
////        LocalDateTime endDatetime = startDatetime.plusHours(24);  //2022-09-13 06:00:00
//        NxDeviceVO v = new NxDeviceVO();
////        v.setModeID("197ZUeGcXkY");
////        v.setDeviceTypeName("3DP打印机");
//        //1.取出打印机所有的设备编号
//        List<NxDeviceVO> list = nxDeviceDao.getDeviceListEurope(v);
//        List<PrinterEfficiencyWorkVO> resultList = new ArrayList<>(16);
//        for(int i=0;i<list.size();i++)
//        {
//            String deviceCode=list.get(i).getDeviceCode();
//            String modelId=list.get(i).getModeID();
//            String thingId=list.get(i).getThingID();
//            String deviceTypeId=list.get(i).getDeviceTypeId();
//            try
//            {
//                //处理海外东1区奥钢联设备，相比东八区，夏季偏差6(夏令时)、冬季偏差7h
//                resultList.addAll(processDeviceHistoryGK0(LocalDate.of(year,month,day), startDatetime, endDatetime,
//                        deviceCode,deviceTypeId,modelId,thingId));
//            }
//            catch (Exception ex){
//                logger.error("本次错误,设备:"+deviceCode +"|error:"+ex.toString());
//            }
//        }
//
//        if (CollectionUtils.isEmpty(resultList)) {
//            return;
//        }
//
//        Log.info("本次resultList获取" + resultList.size(), resultList);
//        //保存到临时表
//        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
//        String time = dateFormat.format(calendar.getTime());
//        Map<String, Object> param = new HashMap<>();
//        param.put("time", time + " 00:00:00");
//        param.put("resultList", resultList);
//        param.put("syCreatetime", new Date());
//        param.put("clearAll", false);
//        deviceMapper.clearDeviceTimesDay(param);
//        deviceMapper.batchAddDeviceTimesDay(param);
//
//    }



//    private Collection<PrinterEfficiencyWorkVO> processDeviceHistoryGK0(
//            LocalDate localDate,
//            LocalDateTime startDatetime,
//            LocalDateTime endDatetime,
//            String deviceId,String deviceTypeId,String modelId,String thingId) {
//
//        List<PrinterEfficiencyWorkVO> workVOS = new ArrayList<>();
//        String DATE_FORMAT_DAY = "yyyy-MM-dd";
//        DateTimeFormatter dtfDay = DateTimeFormatter.ofPattern(DATE_FORMAT_DAY);
//        String today = localDate.format(dtfDay);
//        String yestoday = localDate.minusDays(1).format(dtfDay);
//        String tomorrow = localDate.plusDays(1).format(dtfDay);
//        Map<String, Map<String, Object>> resultMap = new HashMap<>(16);
//        boolean isRepeat = false;
//        boolean yestdayisFinieshed=false;
//        boolean todayisFinieshed=false;
//        Map<String, String> headers = new HashMap<>();
//        headers.put("Authorization", "Bearer " + tokenUtil.getToken());
//        try
//        {
//            DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
//            String startTime =startDatetime.plusHours(-8).format(dtf);
//            String endTime=   endDatetime.plusHours(-8).format(dtf);
//            startTime = DateUtil.handlerTime(startTime);
//            endTime = DateUtil.handlerTime(endTime);
//            //http://openapi.zengcaicloud.com/historian-manage/v1/historian/models/197ZUeGcXkY/things/199DK04jaWA?startTime=2022-05-01T02:30:00.000Z&endTime=2022-05-06T03:20:30.000Z
////            String resp = HttpClientUtil.sendHttp(HttpRequestMethedEnum.HttpGet, reqUrl + "/historian-manage/v1/historian/models/"
////                    + modelId + "/things/" + thingId+""+"?startTime="+startTime+"&endTime="+endTime+"&skip=0&limit=10000", null, headers);
//            String resp = HttpClientUtil.sendHttp(HttpRequestMethedEnum.HttpGet,  reqUrl + "/historian-manage/v1/historian/models/"
//                    + modelId + "/things/" + thingId+"?startTime="+startTime+"&endTime="+endTime+"&skip=0&limit=10000" +
//                    "&properties="+ URLEncoder.encode("[\"HMI_JobID\",\"time\",\"HMI_TotalLayer\",\"HMI_CurPrintLayer\",\"HMI_Volume\",\"HMI_Thickness\",\"HMI_TotalTimeCost\",\"HMI_BeginTime\",\"HMI_EndTime\",\"SB_ZT_Running\"]", "UTF-8"), null, headers);
//            Thread.sleep(1000);
////            logger.info("平台历史工况接口返回数据：{}", resp);
//            JSONObject jsonObject = JSONObject.parseObject(resp);
//            JSONArray array = jsonObject.getJSONArray("payload");
//            JSONArray colums = array.getJSONObject(0).getJSONArray("columns");
//            JSONArray rows = array.getJSONObject(0).getJSONArray("rows");
//
//            //取出需要分析的属性字段的下标
//            int jobid_serque = getSequeByKey(colums, "HMI_JobID");
//            int time_serque = getSequeByKey(colums, "time");
//            int HMI_TotalLayer_seq = getSequeByKey(colums, "HMI_TotalLayer");
//            int HMI_CurPrintLayer_seq = getSequeByKey(colums, "HMI_CurPrintLayer");
//            int HMI_Volume_seq = getSequeByKey(colums, "HMI_Volume");
//            int HMI_Thicknessr_seq = getSequeByKey(colums, "HMI_Thickness");
//            int HMI_TotalTimeCost_seq = getSequeByKey(colums, "HMI_TotalTimeCost");
//            int HMI_BeginTime_seq = getSequeByKey(colums, "HMI_BeginTime");
//            int HMI_EndTime_seq = getSequeByKey(colums, "HMI_EndTime");
//            int SB_ZT_Running_seq = getSequeByKey(colums, "SB_ZT_Running");
//            for (int idx=0;idx< rows.size();idx++)
//            {
//                String cur_HMI_JobID =getString(rows.getJSONArray(idx).get(jobid_serque));//HMI_JobID	打印编号
//                String Timestamp_Local ="";  //当前工况时间
//                String UTCtimeStr = getString(rows.getJSONArray(idx).get(time_serque)); //UTC 格式的时间
//                Timestamp_Local =  DateUtil.UTCtoBeijing_1(UTCtimeStr);//UTC转换为北京时间
////                if (!Timestamp_Local.startsWith(today) && !Timestamp_Local.startsWith(yestoday) && !Timestamp_Local.startsWith(tomorrow)) {
////                    continue;
////                }
//                if (!Timestamp_Local.substring(0,10).startsWith(today))
//                {
//                    continue;
//                }
//                if( StringUtils.isBlank(cur_HMI_JobID)){
//                    continue;
//                }
//                /*
//                if(cur_HMI_JobID.contains("试块") || cur_HMI_JobID.contains("调试")||
//                        cur_HMI_JobID.contains("测试")|| cur_HMI_JobID.contains("实验")
//                        || cur_HMI_JobID.contains("调整"))
//                {
//                    continue;
//                    //试块、调试的打印任务自动过滤掉
//                }
//                */
//                float HMI_Volume = getFloat(rows.getJSONArray(idx).get(HMI_Volume_seq)); //打印体积(HMI_Volume)
//                //HMI_CurPrintLayer 当前打印层数
//                float HMI_CurPrintLayer = getFloat(rows.getJSONArray(idx).get(HMI_CurPrintLayer_seq));
//                float HMI_TotalLayer = getFloat(rows.getJSONArray(idx).get(HMI_TotalLayer_seq));
//                String HMI_BeginTime = getString(rows.getJSONArray(idx).get(HMI_BeginTime_seq)); //MapUtils.getString(data, "HMI_BeginTime");
//                String HMI_EndTime = getString(rows.getJSONArray(idx).get(HMI_EndTime_seq));//MapUtils.getString(data, "HMI_EndTime");
//                String HMI_RUNNING = getString(rows.getJSONArray(idx).get(SB_ZT_Running_seq));//MapUtils.getString(data, "SB_ZT_Running");
////                if (HMI_CurPrintLayer < 200) {
////                    continue;
////                }
//                //***判断开头已经打印结束的
//                if(idx == 0)
//                {
//                    //判断前一天已经打印完成的
//                    if((HMI_CurPrintLayer == HMI_TotalLayer) &&HMI_CurPrintLayer > 200)
//                    {
//                        yestdayisFinieshed =true;
//                        continue;  //过滤已经打印完的
//                    }
//                }
//                if(yestdayisFinieshed && HMI_TotalLayer == 0.0 && HMI_CurPrintLayer == 0.0)
//                {
//                    yestdayisFinieshed = false;//清除开头已经打印结束的任务,继续打印
//                }
//                if(yestdayisFinieshed && HMI_RUNNING.equals("true"))//开头已经打印结束后面重新开始打印的情况，则继续扫描
//                {
//                    yestdayisFinieshed = false;
//                }
//                if(yestdayisFinieshed)
//                {
//                    continue;  //
//                }
//                //重新清零参数进行新的打印
//                if(HMI_TotalLayer == 0.0 && HMI_CurPrintLayer == 0.0)
//                {
//                    todayisFinieshed = false;
//                }
//                if(HMI_RUNNING.equals("true")) //当日继续打印，继续
//                {
//                    todayisFinieshed = false;
//                }
//                //判断当前正常打印结束之后，再次打印
//                //*** 判断当前批次任务已经打印结束,记录
//                if(todayisFinieshed)
//                {
//                    continue;  //当前批次任务已经打印结束后面数据不再处理
//                }
//                //200层以下不计入，针对一些前后任务切换的特殊情况，数据会超过24h,需要过滤掉
//                if (HMI_CurPrintLayer < 200 && HMI_TotalLayer < 200) {
//                    continue;
//                }
//                if (idx > 1) {
//                    String last_HMI_JobID = getString(rows.getJSONArray(idx-1).get(jobid_serque)); // HMI_JobID
//                    float last_HMI_Volume = getFloat(rows.getJSONArray(idx-1).get(HMI_Volume_seq));//HMI_Volume
//                    if ( !cur_HMI_JobID.equals(last_HMI_JobID) && HMI_Volume ==  last_HMI_Volume) {
//                        isRepeat = true;
//                        continue;
//                    }
//                    if (isRepeat && HMI_Volume ==  last_HMI_Volume) {
//                        continue;
//                    } else if (isRepeat && HMI_Volume !=  last_HMI_Volume) {
//                        isRepeat = false;
//                    }
//                }
////              String cloudtime = rows.getJSONArray(idx).get(time_serque).toString().substring(0,10);
//                String cloudtime = Timestamp_Local.substring(0,10);
//                String cloudtime_1 = HMI_BeginTime;
//
//                Map<String, Object> hisData = resultMap.computeIfAbsent( cloudtime_1 + cur_HMI_JobID, m -> new HashMap<>(16)); //cloudtime
//                hisData.put("cloudtime", cloudtime);
//                hisData.put("deviceId", deviceId);
//                hisData.put("HMI_JobID", cur_HMI_JobID);
//                float HMI_Thickness = getFloat(rows.getJSONArray(idx).get(HMI_Thicknessr_seq));//HMI_Thickness
//                float HMI_TotalTimeCost = getFloat(rows.getJSONArray(idx).get(HMI_TotalTimeCost_seq));// HMI_TotalTimeCost
//
//                float his_HMI_Volume = getFloat(hisData,"HMI_Volume");// "HMI_Volume"
//                float his_HMI_TotalTimeCost = getFloat(hisData,"HMI_TotalTimeCost");
//                float his_HMI_CurPrintLayer = getFloat(hisData,"HMI_CurPrintLayer");
//                float his_HMI_Thickness = getFloat(hisData,"HMI_Thickness");
//                float his_HMI_TotalLayers = getFloat(hisData,"HMI_TotalLayer");
//                String his_HMI_RUNNING =  getStringS(hisData,"HMI_RUNNING");
//
//                //正常情况，满足下面的条件；特殊的如2515切换批次不清空上一批次打印参数的情况，完全不适用
//                if ( HMI_TotalTimeCost > his_HMI_TotalTimeCost) {
//                    hisData.put("HMI_TotalTimeCost", HMI_TotalTimeCost);
//                }
//                if ( HMI_Volume > his_HMI_Volume) {
//                    hisData.put("HMI_Volume", HMI_Volume);
//                }
//                if ( HMI_CurPrintLayer > his_HMI_CurPrintLayer) {
//                    hisData.put("HMI_CurPrintLayer", HMI_CurPrintLayer);
//                }
//                if ( HMI_Thickness > his_HMI_Thickness) {
//                    hisData.put("HMI_Thickness", HMI_Thickness);
//                }
//                //正常打印结束的任务  正常情况 当前打印层数 = 总层数
//                if(HMI_TotalLayer > 200 && HMI_TotalLayer == HMI_CurPrintLayer)
//                {
//                    hisData.put("Timestamp_Local", Timestamp_Local);
//                    todayisFinieshed = true;
//                }
//                //正常打印结束的任务   特殊情况 2515 当前打印层数 = 总层数+1
//                if(HMI_TotalLayer > 200 && HMI_TotalLayer + 1 == HMI_CurPrintLayer)
//                {
//                    hisData.put("Timestamp_Local", Timestamp_Local);
//                    todayisFinieshed = true;
//                }
//                //正常打印中中止的情况？
//                if(HMI_TotalLayer > 200 && HMI_CurPrintLayer < HMI_TotalLayer && HMI_RUNNING.equals("false"))
//                {
//                    hisData.put("Timestamp_Local", Timestamp_Local);
//                    todayisFinieshed = true;
//                }
//
////                if(HMI_CurPrintLayer >= his_HMI_CurPrintLayer && HMI_CurPrintLayer > 0 && his_HMI_CurPrintLayer >0){
//                if(HMI_TotalLayer >= his_HMI_TotalLayers) //> 1800-1数据异常特殊情况,当前层数N,总层0，存在下面参数进不去
//                {
//                    //记录每一个批次内的遍历参数,为后面根据批次标识(jobid+startTime)进行归类分析
//                    hisData.put("HMI_TotalTimeCost", HMI_TotalTimeCost);
//                    //hisData.put("HMI_CurPrintLayer", HMI_CurPrintLayer);
//                    if ( HMI_CurPrintLayer > his_HMI_CurPrintLayer) {
//                        hisData.put("HMI_CurPrintLayer", HMI_CurPrintLayer);
//                    }
//                    hisData.put("HMI_TotalLayer", HMI_TotalLayer);
//                    hisData.put("HMI_BeginTime", HMI_BeginTime);
//                    hisData.put("HMI_EndTime", HMI_EndTime);
//                    hisData.put("Timestamp_Local", Timestamp_Local);
//                    hisData.put("HMI_Volume", HMI_Volume);
//                    hisData.put("HMI_RUNNING", HMI_RUNNING);
//                }
//                //logger.info("设备编号:"+deviceId+ " |JobID:"+cur_HMI_JobID +" |总层数："+HMI_TotalLayer+" |当前打印层数:"+HMI_CurPrintLayer+" |Beijing-time:"+Timestamp_Local+" |UTC-time:"+time);
//            }
//        }catch (Exception ex)
//        {
//            Log.error("报错位置：deviceId:"+deviceId);
//            Log.error("调用平台查询实时工况信息error",ex);
//        }
//
//        List<Map<String,Object>> todayList1 = resultMap.values().stream()
//                .filter(x->today.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
//
//        Map<String, Map<String, Object>> resultMap2 = new HashMap<>(16);
//        List<Map<String,Object>> todayList = new ArrayList<Map<String,Object>>();
//        //MMP
//        for(int i=0;i<todayList1.size();i++)
//        {
//            Map<String,Object> newMap = todayList1.get(i);
//            String HMI_JobID1 = newMap.getOrDefault("HMI_JobID","").toString();
//            String HMI_Volume1 = newMap.getOrDefault("HMI_Volume","").toString();
//            String HMI_CurPrintLayer1 = newMap.getOrDefault("HMI_CurPrintLayer","").toString();
//            if(HMI_CurPrintLayer1.equals("")) //处理特殊情况不存在层数的情况()
//                HMI_CurPrintLayer1 = "0.0";
//            //判断当前批次的最大打印层数是否200以内,剔除200层以内的试块（特殊情况下试块打印总层数>200,最大打印层数超过200层）
////            if(Double.parseDouble(HMI_CurPrintLayer1) < 200)  //2022-09-06 暂时200层以下计入
////            {
////                continue;
////            }
//            String HMI_TotalLayer1 = newMap.getOrDefault("HMI_TotalLayer","").toString();
//            String HMI_BeginTime1 = newMap.getOrDefault("HMI_BeginTime","").toString();
//            String HMI_EndTime1 = newMap.getOrDefault("HMI_EndTime","").toString();
//            String HMI_Thickness1 = newMap.getOrDefault("HMI_Thickness","").toString();
//            String Timestamp_Local1 = newMap.getOrDefault("Timestamp_Local","").toString();
//            String cloudtime1 = newMap.getOrDefault("cloudtime","").toString();
//            String deviceId1 = newMap.getOrDefault("deviceId","").toString();
//            String HMI_TotalTimeCost1 = newMap.getOrDefault("HMI_TotalTimeCost","").toString();
//            String HMI_RUNNING =  newMap.getOrDefault("HMI_RUNNING","").toString() == "false" ? "0":"1";  //runnning 状态值
//
//            if(HMI_TotalTimeCost1.equals(""))
//                HMI_TotalTimeCost1 ="0.0";
//            LocalDateTime t1= LocalDateTime.parse(HMI_BeginTime1, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
//            LocalDateTime t2 = LocalDateTime.parse(today +" "+"00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
//
//            float totalTimeCost =0;
//            if(t1.isBefore(t2)) //隔天情况(添加判断不在同一天),前一天没有打印完当天接着打印的，打印耗时按从当天0点到打印结束时间这段时间为有效时间
//            {
//                //这里有坑，结束后会有一段时间的静置期，Timestamp_Local1会持续到下一批任务，所以应该取打印结束时间比较准确
//                LocalDateTime temp= LocalDateTime.parse(HMI_EndTime1.substring(0,19) , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
//                LocalDateTime current_RecordTime = LocalDateTime.parse(Timestamp_Local1, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
//                //1800-3 09-09 数据异常，前一批故障未结束且打印参数未清零，下一批任务jobid切换进来后，打印开始、结束，层数还是上一批的参数。几分钟后才开始清空
//                if(temp.isBefore(current_RecordTime) && HMI_CurPrintLayer1.equals("0.0") && HMI_Volume1.equals("0.0") && HMI_TotalTimeCost1.equals("0.0"))
//                {
//                    totalTimeCost = Float.valueOf(0); //默认0
//                }
//                else
//                {
//                    String spanT = String.valueOf(Duration.between(t2,temp).toMinutes()); //t1  t2之间相差多少h
//                    if(Float.parseFloat(spanT) < 0)
//                    {
//                        totalTimeCost = 0; //异常情况，前一天打印未结束直接异常数据没清空
//                    }
//                    else
//                    {
//                        //temp -- 预计结束时间(动态可变)
//                        LocalDateTime current_task_endTime = temp;
//                        //Timestamp_Local1  --任务当前时间
//                        LocalDateTime current_taskTime = LocalDateTime.parse(Timestamp_Local1, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
//                        //特殊情况，跨天到当天的打印任务，前一天打印故障数据未清零持续到当天的情况 Max-A  2022-09-11
//                        float rate = Float.parseFloat(HMI_CurPrintLayer1) * 100 /Float.parseFloat(HMI_TotalLayer1);
//                        //若跨天已打印总任务的95%以下，且任务停止时间(故障停机水平停止线)< 任务预计结束时间，则大概率是失败的情况，否则不计算；大于>95%以上，则成功(当前层>=总层数 或 当前层<=总层数)
//                        if( rate < 95 && Float.parseFloat(HMI_CurPrintLayer1) < Float.parseFloat(HMI_TotalLayer1) && current_taskTime.isBefore(current_task_endTime))
//                        {
//                            totalTimeCost = Float.valueOf(0); //默认0
//                        }
//                        else
//                        {
//                            Float tt = Math.abs(Float.parseFloat(spanT)) /60;
//                            //totalTimeCost = Float.valueOf(spanT);
//                            totalTimeCost = Math.abs(Float.parseFloat(spanT)) /60;
//                            DecimalFormat decl = new DecimalFormat("0.0");
//                            String p = decl.format(totalTimeCost);
//                            totalTimeCost = Float.valueOf(p);
//                            HMI_BeginTime1 =  today +" "+"00:00:00";
//                        }
//
//                    }
//                }
//
//
//            }
//            else  //当日内打完的情况；当日没打结束跨到下一天，则打印耗时直接是批次中最后一条记录(若数据22点掉线了，可能会统计数据偏小？)
//            {
//                totalTimeCost = Math.abs(Float.valueOf(HMI_TotalTimeCost1));
//            }
//            Map<String,Object> map1 = new HashMap<String,Object>();
//            map1.put("HMI_JobID", HMI_JobID1);
//            map1.put("HMI_Volume", HMI_Volume1);
//            map1.put("HMI_CurPrintLayer", HMI_CurPrintLayer1);
//            map1.put("HMI_TotalLayer", HMI_TotalLayer1);
//            map1.put("HMI_BeginTime", HMI_BeginTime1);
//            map1.put("HMI_EndTime", HMI_EndTime1);
//            map1.put("HMI_Thickness", HMI_Thickness1);
//            map1.put("Timestamp_Local", Timestamp_Local1);
//            map1.put("cloudtime", cloudtime1);
//            map1.put("deviceId", deviceId1);
//            map1.put("HMI_TotalTimeCost", totalTimeCost);
//            todayList.add(map1);
//
//            //重构resultMap2
//            String cloudtime = cloudtime1;
//            String cloudtime_1 = HMI_BeginTime1;
//            Map<String, Object> hisData = resultMap2.computeIfAbsent( cloudtime_1 + HMI_JobID1, m -> new HashMap<>(16));
//
//            hisData.put("HMI_JobID", HMI_JobID1);
//            hisData.put("HMI_Volume", HMI_Volume1);
//            hisData.put("HMI_CurPrintLayer", HMI_CurPrintLayer1);
//            hisData.put("HMI_TotalLayer", HMI_TotalLayer1);
//            hisData.put("HMI_BeginTime", HMI_BeginTime1);
//            hisData.put("HMI_EndTime", HMI_EndTime1);
//            hisData.put("HMI_Thickness", HMI_Thickness1);
//            hisData.put("Timestamp_Local", Timestamp_Local1);
//            hisData.put("cloudtime", cloudtime1);
//            hisData.put("deviceId", deviceId1);
//            hisData.put("HMI_TotalTimeCost", totalTimeCost);
//        }
//
//        List<Map<String,Object>> yestodayList = resultMap.values().stream()
//                .filter(x->yestoday.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
//        List<Map<String,Object>> tomorrowList = resultMap.values().stream()
//                .filter(x->tomorrow.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
//        Set<String> excludeJobIds = new HashSet<>();
//        for (Map<String,Object> todayData: todayList) {
//            String jobId = MapUtils.getString(todayData, "HMI_JobID");
//            float HMI_Volume = getFloat(todayData, "HMI_Volume");
//            if (yestodayList.stream().anyMatch(x->{
//                if(jobId.equals(MapUtils.getString(x, "HMI_JobID"))
//                        && HMI_Volume <= getFloat(x,"HMI_Volume")){
//                    return true;}else{return false;}})) {
//                excludeJobIds.add(jobId);
//            }else if (tomorrowList.stream().anyMatch(x->{
//                if(jobId.equals(MapUtils.getString(x, "HMI_JobID"))
//                        && HMI_Volume < getFloat(x,"HMI_Volume")){
//                    return true;}else{return false;}})) {
//                excludeJobIds.add(jobId);
//            }
//        }
//        Double dyhs = 0d;
//        Double dyTime =0d;
//        Double ghjhl =0d;
//        List<Map<String,Object>> resultList = resultMap2.values().stream()
//                .filter(x-> today.equals(MapUtils.getString(x, "cloudtime"))
//                        &&!excludeJobIds.contains(MapUtils.getString(x, "HMI_JobID"))).collect(Collectors.toList());
//        if (!CollectionUtils.isEmpty(resultList)) {
//            dyhs = resultList.stream().mapToDouble(x -> getFloat(x, "HMI_TotalTimeCost")).sum();
//            dyhs = dyhs >= 24 ?  24: dyhs;  //异常超过24H情况
//            dyTime = 0d;//resultList.stream().mapToDouble(x -> getDyxl(x)).average().orElse(0d);
//            ghjhl = 0d;//resultList.stream().mapToDouble(x -> getBtl(x)).average().orElse(0d);
//        }
//        Log.info("计算过程：resultMap2：{},excludeJobIds:{}", resultMap2,excludeJobIds);
//        PrinterEfficiencyWorkVO newWorkVO = new PrinterEfficiencyWorkVO(thingId, deviceTypeId)
//                .dyhs(dyhs.floatValue())
//                .dyTime(dyTime.floatValue())
//                .ghjhl(ghjhl.floatValue());
//        workVOS.add(newWorkVO);
//        return workVOS;
//    }

    private String getStringS(
            Map<String, Object> map,
            String key) {
        Object str = map.getOrDefault(key, "0");
        String value = "";
        if (null != str && !"null".equals(str)) {
            value = str.toString();
        }
        return value;
    }


    /** 有效打印时间、打印效率*/
    public void run1(Calendar calendar) throws Exception{

        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1;
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        // 需要从ES中查询前一天及当天的数据，因为前一天开始的有可能当天才结束 前一天23:00开始到后一天01:00
        LocalDateTime startDatetime = LocalDateTime.of(year,month,day,0,0,0).minusHours(1);
        LocalDateTime endDatetime = startDatetime.plusHours(26);

        //DEVICE_SBLX-设备类型CODE --5719    DEVICE_SBLX_NAME-设备类型名称  --3DP打印机
        NxDeviceVO v = new NxDeviceVO();
//        v.setDEVICE_SBLX("5719");
//        v.setDEVICE_SBLX_NAME("3DP打印机");
        v.setDeviceTypeId("5719");
        v.setDeviceTypeName("3DP打印机");


        //1.取出打印机所有的设备编号
        List<NxDeviceVO> list = nxDeviceDao.getDeviceList(v);
//        List<NxDeviceVO> list2 = nxDeviceDao.getDeviceList2();
        List<PrinterEfficiencyWorkVO> resultList = new ArrayList<>(16);

        for(int i=0;i<list.size();i++)
        {
//            String deviceCode=list.get(i).getDEVICE_SBBM();
//            String modelId=list.get(i).getMODEL_ID();
//            String thingId=list.get(i).getNX_JE_DEVICE_ID();
            String deviceCode=list.get(i).getDeviceCode();
            String modelId=list.get(i).getModeID();
            String thingId=list.get(i).getThingID();
            String deviceTypeId=list.get(i).getDeviceTypeId();

            resultList.addAll(processDeviceHistoryGK(LocalDate.of(year,month,day), startDatetime, endDatetime,
                    deviceCode,deviceTypeId,modelId,thingId));
        }
        if (CollectionUtils.isEmpty(resultList)) {
            return;
        }
        // 将计算的结果存储到结果表中去，需要将之前的数据清空
        printerEfficiencyWorkDao.clear(year, month, day);
        printerEfficiencyWorkDao.batchAdd(resultList, year, month, day);



    }
    private Collection<PrinterEfficiencyWorkVO> processDeviceHistoryGK(
            LocalDate localDate,
            LocalDateTime startDatetime,
            LocalDateTime endDatetime,
            String deviceId,String deviceTypeId,String modelId,String thingId) {

        List<PrinterEfficiencyWorkVO> workVOS = new ArrayList<>();

        String DATE_FORMAT_DAY = "yyyy-MM-dd";
        DateTimeFormatter dtfDay = DateTimeFormatter.ofPattern(DATE_FORMAT_DAY);
        String today = localDate.format(dtfDay);
        String yestoday = localDate.minusDays(1).format(dtfDay);
        String tomorrow = localDate.plusDays(1).format(dtfDay);
        Map<String, Map<String, Object>> resultMap = new HashMap<>(16);
        boolean isRepeat = false;


        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer " + tokenUtil.getToken());
        try
        {
            DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            String startTime =startDatetime.plusHours(-8).format(dtf);
            String endTime=   endDatetime.plusHours(-8).format(dtf);
            startTime = DateUtil.handlerTime(startTime);
            endTime = DateUtil.handlerTime(endTime);

            //http://openapi.zengcaicloud.com/historian-manage/v1/historian/models/197ZUeGcXkY/things/199DK04jaWA?startTime=2022-05-01T02:30:00.000Z&endTime=2022-05-06T03:20:30.000Z
            String resp = HttpClientUtil.sendHttp(HttpRequestMethedEnum.HttpGet, reqUrl + "/historian-manage/v1/historian/models/"
                    + modelId + "/things/" + thingId+""+"?startTime="+startTime+"&endTime="+endTime+"&skip=0&limit=10000", null, headers);
//            logger.info("平台历史工况接口返回数据：{}", resp);
            JSONObject jsonObject = JSONObject.parseObject(resp);
            JSONArray array = jsonObject.getJSONArray("payload");
            JSONArray colums = array.getJSONObject(0).getJSONArray("columns");
            JSONArray rows = array.getJSONObject(0).getJSONArray("rows");

            //取出需要分析的属性字段的下标
            int jobid_serque = getSequeByKey(colums, "HMI_JobID");
            int time_serque = getSequeByKey(colums, "time");
            int HMI_TotalLayer_seq = getSequeByKey(colums, "HMI_TotalLayer");
            int HMI_CurPrintLayer_seq = getSequeByKey(colums, "HMI_CurPrintLayer");
            int HMI_Volume_seq = getSequeByKey(colums, "HMI_Volume");
            int HMI_Thicknessr_seq = getSequeByKey(colums, "HMI_Thickness");
            int HMI_TotalTimeCost_seq = getSequeByKey(colums, "HMI_TotalTimeCost");


            for (int idx=0;idx< rows.size();idx++) {

//                String cur_HMI_JobID ="";
//                String HMI_TotalLayer ="";
//                String HMI_CurPrintLayer = "";
//                String HMI_Volume="";
//                String HMI_Thicknessr ="";
//                String HMI_TotalTimeCost ="";
//                if(rows.getJSONArray(idx).get(jobid_serque)!=null)
//                {
//                    cur_HMI_JobID = rows.getJSONArray(idx).get(jobid_serque).toString();
//                }
//                HMI_TotalLayer = rows.getJSONArray(id).get(HMI_TotalLayer_seq).toString();
//                HMI_CurPrintLayer = rows.getJSONArray(id).get(HMI_CurPrintLayer_seq).toString();

                String cur_HMI_JobID =getString(rows.getJSONArray(idx).get(jobid_serque));//HMI_JobID	打印编号
                String Timestamp_Local ="";  //当前工况时间
                String UTCtimeStr = getString(rows.getJSONArray(idx).get(time_serque)); //UTC 格式的时间
                Timestamp_Local =  DateUtil.UTCtoBeijing_1(UTCtimeStr);//UTC转换为北京时间
                if (!Timestamp_Local.startsWith(today) && !Timestamp_Local.startsWith(yestoday) && !Timestamp_Local.startsWith(tomorrow)) {
                    continue;
                }
                if( StringUtils.isBlank(cur_HMI_JobID)){
                    continue;
                }
//            if(cur_HMI_JobID.contains("试块") || cur_HMI_JobID.contains("调试")||
//                    cur_HMI_JobID.contains("测试")|| cur_HMI_JobID.contains("实验")
//                    || cur_HMI_JobID.contains("调整"))
//            {
//                continue;
//                //试块、调试的打印任务自动过滤掉
//            }
                float HMI_Volume = getFloat(rows.getJSONArray(idx).get(HMI_Volume_seq)); //打印体积(HMI_Volume)
                //HMI_CurPrintLayer 当前打印层数
                float HMI_CurPrintLayer = getFloat(rows.getJSONArray(idx).get(HMI_CurPrintLayer_seq));
                if (HMI_CurPrintLayer < 200) {
                    continue;
                }
                if (idx > 1) {
                    String last_HMI_JobID = getString(rows.getJSONArray(idx-1).get(jobid_serque)); // HMI_JobID
                    float last_HMI_Volume = getFloat(rows.getJSONArray(idx-1).get(HMI_Volume_seq));//HMI_Volume
                    if ( !cur_HMI_JobID.equals(last_HMI_JobID) && HMI_Volume ==  last_HMI_Volume) {
                        isRepeat = true;
                        continue;
                    }
                    if (isRepeat && HMI_Volume ==  last_HMI_Volume) {
                        continue;
                    } else if (isRepeat && HMI_Volume !=  last_HMI_Volume) {
                        isRepeat = false;
                    }
                }

//              String cloudtime = rows.getJSONArray(idx).get(time_serque).toString().substring(0,10);
                String cloudtime = Timestamp_Local.substring(0,10);
                Map<String, Object> hisData = resultMap.computeIfAbsent( cloudtime + cur_HMI_JobID, m -> new HashMap<>(16));
                hisData.put("cloudtime", cloudtime);
                hisData.put("deviceId", deviceId);
                hisData.put("HMI_JobID", cur_HMI_JobID);
                float HMI_Thickness = getFloat(rows.getJSONArray(idx).get(HMI_Thicknessr_seq));//HMI_Thickness
                float HMI_TotalTimeCost = getFloat(rows.getJSONArray(idx).get(HMI_TotalTimeCost_seq));// HMI_TotalTimeCost

                float his_HMI_Volume = getFloat(hisData,"HMI_Volume");// "HMI_Volume"
                float his_HMI_TotalTimeCost = getFloat(hisData,"HMI_TotalTimeCost");
                float his_HMI_CurPrintLayer = getFloat(hisData,"HMI_CurPrintLayer");
                float his_HMI_Thickness = getFloat(hisData,"HMI_Thickness");

                if ( HMI_TotalTimeCost > his_HMI_TotalTimeCost) {
                    hisData.put("HMI_TotalTimeCost", HMI_TotalTimeCost);
                }
                if ( HMI_Volume > his_HMI_Volume) {
                    hisData.put("HMI_Volume", HMI_Volume);
                }
                if ( HMI_CurPrintLayer > his_HMI_CurPrintLayer) {
                    hisData.put("HMI_CurPrintLayer", HMI_CurPrintLayer);
                }
                if ( HMI_Thickness > his_HMI_Thickness) {
                    hisData.put("HMI_Thickness", HMI_Thickness);
                }
                //logger.info("设备编号:"+deviceId+ " |JobID:"+cur_HMI_JobID +" |总层数："+HMI_TotalLayer+" |当前打印层数:"+HMI_CurPrintLayer+" |Beijing-time:"+Timestamp_Local+" |UTC-time:"+time);
            }
            //JSONObject rows2 = array.getJSONObject(0).getJSONObject("rows");
//            List<DeviceHistorySituationVO> deviceHistorySituationVO = callHistoryInterface1(historySituationAO.getDeviceId(), jeDeviceEntity.getModelId(), paramMap);
//            PageInfo<DeviceHistorySituationVO> deviceHistorySituationVOPageInfo = this.queryPageInfo(skip, limit, deviceHistorySituationVO);

        }catch (Exception ex)
        {
            logger.error("调用平台查询实时工况信息error", ex);
        }
//        JSONObject metric = get4StaueCountInstance("","",null,new Date(),new Date());
//        String key = "temperature";
//        Object[] objects = getMetricByKey(metric, key);


        List<Map<String,Object>> todayList = resultMap.values().stream()
                .filter(x->today.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
        List<Map<String,Object>> yestodayList = resultMap.values().stream()
                .filter(x->yestoday.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
        List<Map<String,Object>> tomorrowList = resultMap.values().stream()
                .filter(x->tomorrow.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
        Set<String> excludeJobIds = new HashSet<>();
        for (Map<String,Object> todayData: todayList) {
            String jobId = MapUtils.getString(todayData, "HMI_JobID");
            float HMI_Volume = getFloat(todayData, "HMI_Volume");
            if (yestodayList.stream().anyMatch(x->{
                if(jobId.equals(MapUtils.getString(x, "HMI_JobID"))
                        && HMI_Volume <= getFloat(x,"HMI_Volume")){
                    return true;}else{return false;}})) {
                excludeJobIds.add(jobId);
            }else if (tomorrowList.stream().anyMatch(x->{
                if(jobId.equals(MapUtils.getString(x, "HMI_JobID"))
                        && HMI_Volume < getFloat(x,"HMI_Volume")){
                    return true;}else{return false;}})) {
                excludeJobIds.add(jobId);
            }
        }

        Double dyhs = 0d;
        Double dyTime =0d;
        Double ghjhl =0d;
        List<Map<String,Object>> resultList = resultMap.values().stream()
                .filter(x-> today.equals(MapUtils.getString(x, "cloudtime"))
                        &&!excludeJobIds.contains(MapUtils.getString(x, "HMI_JobID"))).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(resultList)) {
            dyhs = resultList.stream().mapToDouble(x -> getFloat(x, "HMI_TotalTimeCost")).sum();
            dyTime = resultList.stream().mapToDouble(x -> getDyxl(x)).average().orElse(0d);
            ghjhl = resultList.stream().mapToDouble(x -> getBtl(x)).average().orElse(0d);
        }
        logger.error("计算过程：resultMap：{},excludeJobIds:{}", resultMap,excludeJobIds);
        PrinterEfficiencyWorkVO newWorkVO = new PrinterEfficiencyWorkVO(deviceId, deviceTypeId)
                .dyhs(dyhs.floatValue())
                .dyTime(dyTime.floatValue())
                .ghjhl(ghjhl.floatValue());
        workVOS.add(newWorkVO);

        return workVOS;
    }


    /** 打印体积、产量数据*/
    public void run2(LocalDate calendar) throws Exception{

        int year = calendar.getYear();
        int month = calendar.getMonthValue();
        int day = calendar.getDayOfMonth();

        // 需要从ES中查询前一天及当天的数据，因为前一天开始的有可能当天才结束
        LocalDateTime startDatetime = LocalDateTime.of(year,month,day,0,0,0).minusHours(1);
        LocalDateTime endDatetime = startDatetime.plusHours(26);

    }



    /**
     * 布图率 = 一个打印任务最大打印体积 / （打印层厚*最大打印层数*工作箱长*工作箱宽*2）（2215、2515、2518、2218、SDream机型）
     * 布图率 = 一个打印任务最大打印体积 / （打印层厚*最大打印层数*工作箱长*工作箱宽）（Max、1800、800、1000、2600机型）
     * @param data
     * @return
     */
    private double getBtl(Map<String, Object> data) {
        String deviceId = MapUtils.getString(data, "deviceId");
        return getWorkBoxSize(deviceId).map(workBoxSize -> {
            float dycs = getFloat(data, "HMI_CurPrintLayer");
            float dyhd = getFloat(data, "HMI_Thickness");
            float HMI_Volume = getFloat(data, "HMI_Volume");
            double length = workBoxSize.getLength();
            double width = workBoxSize.getWidth();
            logger.error(deviceId+"当前打印层数:{},层厚:{},打印体积:{},工作箱长:{},工作箱宽:{}", dycs,dyhd,HMI_Volume,length,width);
            if (dycs * dyhd < 0.01) {
                return 0f;
            }
            Double result = 0d;
            if (!deviceId.startsWith("1800") && !deviceId.toLowerCase().startsWith("max")
                    && !deviceId.startsWith("800")&& !deviceId.startsWith("1000")&& !deviceId.startsWith("2600")
                    && !deviceId.startsWith("2600B")&& !deviceId.startsWith("2500")&& !deviceId.startsWith("300")&& !deviceId.startsWith("1200")) {
                result = HMI_Volume / (dycs * dyhd * length * width * 2 * 0.01);
                return result.floatValue();
            } else {
                result = HMI_Volume / (dycs * dyhd * length * width * 0.01);
                logger.error(deviceId + "布图率:{}", result);
                return result.floatValue();
            }
        }).orElse(0F);
    }


    private int getSequeByKey(JSONArray metric, String key) {
        JSONArray columns = metric;
        int keyIndex = -1;
        for (int i=0;i<columns.size();i++) {
            String columnName = columns.getString(i);
            if (key.equals(columnName)) {
                keyIndex=i;
                break;
            }
        }
        return keyIndex;
    }

    private static Object[] getMetricByKey(JSONObject metric, String key) {
        JSONArray columns = metric.getJSONArray("columns");
        int keyIndex = -1;
        for (int i=0;i<columns.size();i++) {
            String columnName = columns.getString(i);
            if (key.equals(columnName)) {
                keyIndex=i;
                break;
            }
        }
        JSONArray rows = metric.getJSONArray("rows");
        Object[] objects = new Object[rows.size()];
        for (int row=0;row<rows.size();row++) {
            objects[row] = rows.getJSONArray(row).get(keyIndex);
        }
        return objects;
    }



    /**
     *
     * 800,1000,2600 打印效率 = 打印层数 * 打印层厚 * 工作箱长 * 工作箱宽
     * @param data
     * @return
     */
    private double getDyxl(Map<String, Object> data) {
        String deviceId = MapUtils.getString(data, "deviceId");
        return getWorkBoxSize(deviceId).map(workBoxSize -> {
            float dycs = getFloat(data, "HMI_CurPrintLayer");
            float dyhd = getFloat(data, "HMI_Thickness");
            float dyhs = getFloat(data, "HMI_TotalTimeCost");
            double length = workBoxSize.getLength();
            double width = workBoxSize.getWidth();
            logger.error(deviceId+"当前打印层数:{},层厚:{},耗时:{},工作箱长:{},工作箱宽:{}", dycs,dyhd,dyhs,length,width);
            if (dyhs < 0.01) {
                return 0f;
            }
            Double result = 0d;
            if(deviceId.startsWith("800") || deviceId.startsWith("1000") || deviceId.startsWith("2600")
                    || deviceId.startsWith("2600B") || deviceId.startsWith("2500") || deviceId.startsWith("300")|| deviceId.startsWith("1200")){
                result =(dycs * dyhd * length * width * 0.01) / dyhs;
                logger.error(deviceId + "打印效率:{}", result);
                return result.floatValue();
            }else if (deviceId.startsWith("1800") || deviceId.toLowerCase().startsWith("max")) {
                result = (dycs * dyhd * length * width * 0.01)  / dyhs;
                return result.floatValue();
            } else {
                result = (dycs * dyhd * length * width * 2 *0.01) /dyhs;
                return result.floatValue();
            }
        }).orElse(0F);
    }


    /**
     * 根据设备编号查询工作箱尺寸
     *
     * @param deviceId 设备编号
     * @return 工作箱尺寸
     */
    private Optional<PrinterWorkBoxSize> getWorkBoxSize(String deviceId) {
        for (String key : Constants.PRINTER_WORK_BOX_SIZE_MAP.keySet()) {
            if (deviceId.startsWith(key)) {
                return Optional.ofNullable(Constants.PRINTER_WORK_BOX_SIZE_MAP.get(key));
            }
        }

        return Optional.empty();
    }

    /**
     * 从Map中获取浮点数
     */
    private float getFloat(
            Map<String, Object> map,
            String key) {
        Object str = map.getOrDefault(key, "0");
        float value = 0f;
        if (null != str && !"null".equals(str)) {
            value = Float.parseFloat(str.toString());
        }

        return value;
    }

    public String getString(Object value){
        if (value != null){
            return  value.toString();
        }
        else
            return "";
    }
    public float getFloat(Object str){
        float value = 0f;
        if (str != null) {
            value = Float.parseFloat(str.toString());
        }
        return value;
    }



    //工厂级运维报告-开动率、效率、补图率逻辑入口函数
    public Map<String, Object> getReportBarDataByDevice(PrinterReportDTO printerReportDTO, String tenantId, Map<String,String> printerDeviceRelationData, Map<String, Object> targetmap){
        Map<String, Object> data = new HashMap<String, Object>();
        if( printerReportDTO.getStartDate()==null && printerReportDTO.getEndDate()==null ){
            return data;
        }
        Map<String, Object>  dyhsResult = getReportDYHSByDevice(printerReportDTO, tenantId, "",printerDeviceRelationData);
        Map<String, Object>  dyxlResult = getReportDYXLByDevice(printerReportDTO, tenantId, "",printerDeviceRelationData,targetmap);
        Map<String, Object>  btlResult = getReportBTLByDevice(printerReportDTO, tenantId, "",printerDeviceRelationData);
        //星级评价
        Map<String, Object>  starResult = new HashMap<>(20);
//        Map<String, Object>  starResult = getReportStarEvaluation(printerReportDTO, tenantId, "",printerDeviceRelationData);
        if( dyhsResult.isEmpty() ){
            return data ;
        }
        //打印耗时
        List<Map<String, Object>> dataListDYHS = (List)dyhsResult.get("list");
        int length = dataListDYHS==null?0:dataListDYHS.size();
        String[] xListDYHS = new String[length];//开动率设备结果集(x轴)
        List<String> dyhsList = new ArrayList<String>();//开动率
        for( int i=0; i<length;i++ ){
            Map<String, Object> map = (Map)dataListDYHS.get(i);

            xListDYHS[i]= map.get("deviceId").toString();
            dyhsList.add(map.get("dyhs").toString());
        }
        data.put("xListDYHS", xListDYHS);
        data.put("dyhsList", dyhsList);
        data.put("avgKdlValue", dyhsResult.get("avgKdlValue"));//平均开动率
        data.put("minKdlValue", dyhsResult.get("minKdlValue"));//设备最小开动率
        data.put("minKdlDevice", dyhsResult.get("minKdlDevice"));//最小开动率设备


        //打印效率
        data.put("xListDYXL", dyxlResult.get("xList"));
        data.put("dyxlList", dyxlResult.get("dyxlList"));
        data.put("dyxltargetValue", dyxlResult.get("targetValue"));
        data.put("ratelist", dyxlResult.get("ratelist"));//平均打印效率

        //布图率
        List<Map<String, Object>> dataListBTL = (List)btlResult.get("list");
        String[] xListBTL = new String[dataListBTL==null?0:dataListBTL.size()];//布图率设备结果集(x轴)
        List<String> btlList = new ArrayList<String>();//布图率
        for( int i=0; dataListBTL!=null && i<dataListBTL.size();i++ ){
            Map<String, Object> map = (Map)dataListBTL.get(i);

            xListBTL[i]= map.get("deviceId").toString();
            btlList.add(map.get("btl").toString());
        }
        data.put("xListBTL", xListBTL);
        data.put("btlList", btlList);
        data.put("avgBtlValue", btlResult.get("avgBtlValue"));//平均空间利用率
        data.put("minBtlValue", btlResult.get("minBtlValue"));//设备最小空间利用率
        data.put("minBtlDevice", btlResult.get("minBtlDevice"));//最小空间利用率设备

        //星级评价
        List<Map<String, Object>> dataListStar = (List)starResult.get("list");
        int length2 = dataListStar==null?0:dataListStar.size();
        String[] xListStar = new String[length2];//结果集(x轴)
        List<String> staryList = new ArrayList<String>();//
        for( int i=0; i<length2;i++ ){
            Map<String, Object> map = (Map)dataListStar.get(i);
            xListStar[i]= map.get("deviceid").toString();
            staryList.add(map.get("score").toString());
        }
        data.put("xListStar", xListStar);
        data.put("ystarList", staryList);
        data.put("avgstarValue", starResult.get("avgstarValue"));
        data.put("top3DeviceStar", starResult.get("top3Device"));
        data.put("end3DeviceStar", starResult.get("end3Device"));

        return data;
    }

    //产线-打印耗时
    public Map<String, Object> getReportDYHSByDevice(PrinterReportDTO printerReportDTO, String tenantId, String pUid, Map<String,String> printerDeviceRelationData){
        Map<String, Object> result = new HashMap<String, Object>();
        if( printerReportDTO.getStartDate()==null && printerReportDTO.getEndDate()==null ){
            return result;
        }

        List<Map<String, Object>> list = printerEfficiencyWorkDao.getReportDYHSByDevice(printerReportDTO);
        if( list==null || list.size()==0 ){
            return result;
        }
        list = PlatformDeviceTransUtil.repalceDevice(printerDeviceRelationData, list);

        // 数据结果转换成前台需要的格式[#打印机开动率 dyhs  布图率 btl	打印效率 dyxl		开动率=打印时间/(21*28)	]
        List<Map<String, Object>> dataList = list.stream().map(item -> {
            Map<String, Object> map = new HashMap<>(3);

            int jsTime = 21*28*12 ;//默认按年查询
            if( printerReportDTO.getQueryedType()==1 ){//按周查询
                jsTime = 21*7 ;
            }else if(printerReportDTO.getQueryedType()==3){//按季度查询
                jsTime = 21*28*3 ;
            }else if(printerReportDTO.getQueryedType()==2){//按月查询
                jsTime = 21*28 ;
            }else if(printerReportDTO.getQueryedType()==5){//按时间段
                int day = 1;
                try {
                    day = DateUtils.getDayDifference(printerReportDTO.getStartDate(), printerReportDTO.getEndDate());
                } catch (Exception e) {
                    e.printStackTrace();
                }
                jsTime = 21*day ;
            }

            float dyhs = Float.parseFloat(item.getOrDefault("dyhs", "0").toString());
            map.put("dyhs", String.format("%.4f", dyhs / jsTime));

            map.put("deviceId", item.get("device_id"));
            return map;
        }).collect(Collectors.toList());

        //计算平均值
        int jsTime = 21*28*12 ;//默认按年查询
        if( printerReportDTO.getWeek()!=null && printerReportDTO.getWeek()>0 ){//按周查询
            jsTime = 21*7 ;
        }else if(printerReportDTO.getQuarter()!=null && printerReportDTO.getQuarter()>0){//按季度查询
            jsTime = 21*28*3 ;
        }else if(printerReportDTO.getMonth()!=null && printerReportDTO.getMonth()>0){//按月查询
            jsTime = 21*28 ;
        }

        float total_dyhs=0;
        String ave_dyhs="0.00";
        Float minKdlValue = null; //设备最小开动率
        String minKdlDevice = ""; //最小开动率设备
        Map<String, Map<String, Object>> deviceMap = new HashMap<String, Map<String, Object>>();
        Map<String, Map<String, Object>> deviceTypeMap = new HashMap<String, Map<String, Object>>();
        logger.info("开动率结果：{}", JSON.toJSONString(dataList == null?"":dataList));
        for( Map<String, Object> item : dataList ){

            float dyhs = Float.parseFloat(item.getOrDefault("dyhs", "0").toString());

            String device = item.get("deviceId").toString();

            if(minKdlValue==null || minKdlValue > dyhs) {
                minKdlValue = dyhs;
                minKdlDevice = device;
            }

            total_dyhs += dyhs;
        }
        if( dataList!=null && dataList.size()>0 ){
            ave_dyhs =  String.format("%.4f", total_dyhs/dataList.size());
        }
        Map<String, Object> ave_map = new HashMap<String, Object>();
        ave_map.put("dyhs", ave_dyhs);
        ave_map.put("deviceId", "平均值");
        dataList.add(ave_map);

        result.put("list", dataList);
        result.put("avgKdlValue", ave_dyhs);//平均开动率
        result.put("minKdlValue", minKdlValue==null?0F:minKdlValue);
        result.put("minKdlDevice", minKdlDevice);

        return result;
    }
    //产线-打印效率
    public Map<String, Object> getReportDYXLByDevice(PrinterReportDTO printerReportDTO, String tenantId, String pUid, Map<String,String> printerDeviceRelationData, Map<String, Object> targetmap){
        Map<String, Object> result = new HashMap<String, Object>();
        if( printerReportDTO.getStartDate()==null && printerReportDTO.getEndDate()==null ){
            return result;
        }

        List<Map<String, Object>> list = printerEfficiencyWorkDao.getReportDYXLByDevice(printerReportDTO);
        if( list==null || list.size()==0 ){
            return result;
        }
        list = PlatformDeviceTransUtil.repalceDevice(printerDeviceRelationData, list);

        // 数据结果转换成前台需要的格式[#打印效率 dyxl]
        List<Map<String, Object>> dataList = list.stream().map(item -> {
            Map<String, Object> map = new HashMap<>(3);

            float dyxl = Float.parseFloat(item.getOrDefault("dyxl", "0").toString());
            map.put("dyxl", String.format("%.2f", dyxl));

            map.put("deviceId", item.get("device_id"));

            return map;
        }).collect(Collectors.toList());

        //从配置表中获取
        Optional<ConfigVO> configVOOptional = configService.findOne(PRINTER_DEVICE_TYPE);
        if (!configVOOptional.isPresent()) {
            logger.error("{}配置项不存在", PRINTER_DEVICE_TYPE);
        }
        String deviceTypes = configVOOptional.get().getValue();

        return  PlatformDeviceTransUtil.groupByDeviceType(deviceTypes.split(","), dataList, targetmap);
    }
    //产线-补图率
    public Map<String, Object> getReportBTLByDevice(PrinterReportDTO printerReportDTO, String tenantId, String pUid, Map<String,String> printerDeviceRelationData){
        Map<String, Object> result = new HashMap<String, Object>();
        if( printerReportDTO.getStartDate()==null && printerReportDTO.getEndDate()==null ){
            return result;
        }

        List<Map<String, Object>> list = printerEfficiencyWorkDao.getReportBTLByDevice(printerReportDTO);
        if( list==null || list.size()==0 ){
            return result;
        }
        list = PlatformDeviceTransUtil.repalceDevice(printerDeviceRelationData, list);

        // 数据结果转换成前台需要的格式[#打印机布图率 btl]
        List<Map<String, Object>> dataList = list.stream().map(item -> {
            Map<String, Object> map = new HashMap<>(3);

            float btl = Float.parseFloat(item.getOrDefault("btl", "0").toString());
            map.put("btl", String.format("%.4f", btl));
            map.put("deviceId", item.get("device_id"));

            return map;
        }).collect(Collectors.toList());

        float total_btl=0;
        String ave_btl="0.00";
        Float minBtlValue = null; //设备最小率
        String minBtlDevice = ""; //最小设备
        logger.info("空间利用率结果：{}", JSON.toJSONString(dataList == null?"":dataList));
        for( Map<String, Object> item : list ){
            String device = item.get("device_id").toString();


            float btl = Float.parseFloat(item.getOrDefault("btl", "0").toString());

            if(minBtlValue==null || minBtlValue > btl) {
                minBtlValue = btl;
                minBtlDevice = device;
            }
            total_btl += btl ;
        }
        if( list!=null && list.size()>0 ){
            ave_btl =  String.format("%.4f", total_btl/list.size());
        }
        Map<String, Object> ave_map = new HashMap<String, Object>();
        ave_map.put("btl", ave_btl);
        ave_map.put("deviceId", "平均值");
        dataList.add(ave_map);

        result.put("list", dataList);
        result.put("avgBtlValue", ave_btl);//平均
        result.put("minBtlValue", minBtlValue==null?0f:minBtlValue);
        result.put("minBtlDevice", minBtlDevice);
        return result;
    }
    //产线-星级评价
    public Map<String, Object> getReportStarEvaluation(PrinterReportDTO printerReportDTO, String tenantId, String pUid, Map<String,String> printerDeviceRelationData){
        Map<String, Object> result = new HashMap<String, Object>();
        if( printerReportDTO.getStartDate()==null && printerReportDTO.getEndDate()==null ){
            return result;
        }
        //取当前设备的数据
        double d1 = 0;
        double d1_weight = 0;
        double d2 =0;
        double d2_weight = 0;
        double d3 =0;
        double d3_weight = 0;
        double d4 =0;
        double d4_weight = 0;
        double d5 =0;
        double d5_weight = 0;
        double d6 =0;
        double d6_weight = 0;
        double d7 =0;
        double d7_weight = 0;
        double d8 =0;
        double d8_weight = 0;
        double endD =0;

        Map<String, Object> query = new HashMap<String, Object>();
        query.put("officeid", printerReportDTO.getOfficeId());
        query.put("data_year", printerReportDTO.getYear());
        query.put("data_month", printerReportDTO.getMonth());
        String officeid =printerReportDTO.getOfficeId();
        String data_year =printerReportDTO.getYear().toString();
        String data_month = printerReportDTO.getMonth().toString();

        List<RepMttDataVO> listAlls = new ArrayList<>();
        List<Map<String, Object>> dataList = new ArrayList<Map<String, Object>>();

        //可以是指定机构或者全部机构
        List<Map<String, String>> deviceList = starEvaluateDAO.selectDistinctDeviceList(query);
        for(int i = 0; i< deviceList.size(); i++)
        {
            Map<String, String> map = deviceList.get(i);
            String cur_device = map.get("deviceid");
            //取当前设备的数据
            d1 = 0;
            d1_weight = 0;
            d2 =0;
            d2_weight = 0;
            d3 =0;
            d3_weight = 0;
            d4 =0;
            d4_weight = 0;
            d5 =0;
            d5_weight = 0;
            d6 =0;
            d6_weight = 0;
            d7 =0;
            d7_weight = 0;
            d8 =0;
            d8_weight = 0;
            endD =0;
            //1. 不指定机构所有设备 或者  指定机构下的设备
            List<RepStarEvaluateVO> ListData = starEvaluateDAO.QueryDeviceInfo(officeid,"","5719",data_year,data_month);
            for(int k = 0; k< ListData.size(); k++)
            {

                String curdevice  =ListData.get(k).getDeviceid();
                if(cur_device.equals(curdevice))
                {
                    String statistics_type = ListData.get(k).getStatistics_type();
                    if(statistics_type.equals("GZTJL")) //d1
                    {
                        d1 = ListData.get(k).getResult_score().doubleValue();
                        d1_weight = ListData.get(k).getTarget_weight().doubleValue();
                    }
                    if(statistics_type.equals("DYXLLYL")) //d2
                    {
                        d2 = ListData.get(k).getResult_score().doubleValue();
                        d2_weight = ListData.get(k).getTarget_weight().doubleValue();
                    }
                    if(statistics_type.equals("FPL")) //d3
                    {
                        d3 = ListData.get(k).getResult_score().doubleValue();
                        d3_weight = ListData.get(k).getTarget_weight().doubleValue();
                    }
                    if(statistics_type.equals("CNLYV")) //d4
                    {
                        d4 = ListData.get(k).getResult_score().doubleValue();
                        d4_weight = ListData.get(k).getTarget_weight().doubleValue();
                    }
                    if(statistics_type.equals("QDHGL")) //d5
                    {
                        d5 = ListData.get(k).getResult_score().doubleValue();
                        d5_weight = ListData.get(k).getTarget_weight().doubleValue();
                    }
                    if(statistics_type.equals("SIZEHGL")) //d6
                    {
                        d6 = ListData.get(k).getResult_score().doubleValue();
                        d6_weight = ListData.get(k).getTarget_weight().doubleValue();
                    }
                    if(statistics_type.equals("OUTWARDHGL")) //d7
                    {
                        d7 = ListData.get(k).getResult_score().doubleValue();
                        d7_weight = ListData.get(k).getTarget_weight().doubleValue();
                    }
                    if(statistics_type.equals("PRECISIONHGL")) //d8
                    {
                        d8 = ListData.get(k).getResult_score().doubleValue();
                        d8_weight = ListData.get(k).getTarget_weight().doubleValue();
                    }
                }
            }
            //计算当前设备的最终的评分
//				endD = d1 * d1_weight + d2 * d2_weight + d3 * d3_weight;
//            endD = d1  + d2  + d3 ;
            endD = d1  + d2  + d3 + d4  + d5 + d6 + d7 + d8 ;
            //以上计算该机构下所有的设备的评分
            RepMttDataVO vo = new RepMttDataVO();
            vo.setName(cur_device);
            vo.setText(cur_device);
            vo.setStrValue1("评分");
            String strScore = String.format("%.2f", endD);
            vo.setValue11(Double.parseDouble(strScore));
            listAlls.add(vo);

//            Map<String, Object> data = new HashMap<String, Object>();
//            data.put("deviceid", cur_device);
//            data.put("score", Double.parseDouble(strScore));
//            dataList.add((Map<String, Object>) data);
        }
        //前三、后三
        String top3Device ="";
        String end3Device ="";
        listAlls = listAlls.stream().sorted((Comparator.comparingDouble(RepMttDataVO::getValue11).reversed())).collect(Collectors.toList());
        if(listAlls.size() >0)
        {

            //从大到小排序
            for(int i=0;i<listAlls.size();i++)
            {
                Map<String, Object> data = new HashMap<String, Object>();
                data.put("deviceid", listAlls.get(i).getName());
                data.put("score", listAlls.get(i).getValue11());
                dataList.add((Map<String, Object>) data);
                if(i<2)
                {
                    top3Device += listAlls.get(i).getName()+"、";
                }
                if(i==2)
                    top3Device += listAlls.get(i).getName();

                if(i>=listAlls.size()-3 && i<listAlls.size()-1)
                    end3Device += listAlls.get(i).getName()+"、";
                if(i==listAlls.size()-1)
                    end3Device += listAlls.get(i).getName();
            }
        }

        float total_score=0;
        int hasscore=0;
        for( Map<String, Object> item : dataList ){

            float score = Float.parseFloat(item.getOrDefault("score", "0").toString());
            String device = item.get("deviceid").toString();
            total_score += score;

            if(score !=0.0)
                hasscore++;
        }
        String ave_score="0.00";
        if( dataList!=null && dataList.size()>0 ){
            ave_score =  String.format("%.2f", total_score / hasscore);  //平均评分
        }
        Map<String, Object> ave_map = new HashMap<String, Object>();
        ave_map.put("score", ave_score);
        ave_map.put("deviceid", "平均值");
        dataList.add(ave_map);

        result.put("list", dataList);
        result.put("avgstarValue", ave_score);//平均得分
        result.put("top3Device", top3Device);
        result.put("end3Device", end3Device);
        return result;
    }


}
