package com.rootcloud.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.github.pagehelper.PageInfo;
import com.rootcloud.common.*;
import com.rootcloud.common.enums.HttpRequestMethedEnum;
import com.rootcloud.common.util.DateUtils;
import com.rootcloud.common.vo.PrinterWorkBoxSize;
import com.rootcloud.domain.dao.*;
import com.rootcloud.domain.vo.*;
import com.rootcloud.service.*;
import com.rootcloud.service.condition.QueryUserReportCondition;
import com.rootcloud.web.dto.ChanNengLiYongLvDto;
import com.rootcloud.web.dto.GuHuaJiDanHaoDto;
import com.rootcloud.web.dto.PinJunDaYinGaoDuDto;
import com.rootcloud.web.dto.PrinterReportDTO;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

//import org.apache.http.client.utils.DateUtils;

@Service("reportService")
public class RepPrinterReportServiceImpl implements RepPrinterReportService {

    private static final Logger logger = LoggerFactory.getLogger(RepPrinterReportServiceImpl.class);
    @Autowired
    private NxDeviceDao nxDeviceDao;
    @Autowired
    private CustomerDao nxCustomerDao;
    @Autowired
    private RepPrinterSandPercentDAO2 sandDAO;
    @Autowired
    private RepPrinterSandPercentDAO repPrinterSandPercentDAO; //打印过程表DAO  产创的设备
    @Autowired
    private RepPrinterSandPercentDAO2 repPrinterSandPercentDAO2; //打印过程表DAO 全部设备跑批
    @Autowired
    private PrinterEfficiencyWorkDao printerEfficiencyWorkDao;

    @Autowired
    private RepPrinterReportDAO nxReportDao;
    @Autowired
    private RepPrinterReportItemDao repPrinterReportItemDao;

    @Resource(name = "configServices")
    private ConfigService configService;
    @Resource(name = "targetService")
    private RepPrinterTargetDicService repPrinterTargetDicService;
    @Resource(name = "efficiency")
    private PrinterEfficiencyService printerEfficiencyService;
    @Resource(name = "repPrinterServices")
    private RepPrinterService repPrinterService;
    @Resource(name = "sandPercentServices")
    private RepPrinterSandPercentService repPrinterSandPercentService ;
    @Resource(name = "emailServices")
    private EmailService emailService;
    
    
    @Autowired
    private TokenUtil tokenUtil;
    @Value("${os.iam.domain}")
    private String reqUrl;
    public static final String DATE_FORMAT_YYYY_MM_DD = "yyyy-MM-dd HH:mm:ss";

    //用于过滤产创砂密度设置
    private List<String> CYCXZX_DEVICES = Arrays.asList("2215-18","Max-A","Max-B","Max-C","2215-12","2215-27","2215-7","2215-17","2215-10", "2600A-1",
            "1800-4","1800-7","1800-8","1800-10","1800-11","1800-12","1800-14","1800-15","1800-16","2500-2","300-2","2215-24",
            "2215-13","2215-16","2215-19","2500-11");


    //跑批产线级过程统计表
    public void run(Calendar calendar) throws Exception {
        Integer year = calendar.get(Calendar.YEAR);
        Integer month = calendar.get(Calendar.MONTH) + 1;
        Integer day = calendar.get(Calendar.DAY_OF_MONTH);

        LocalDateTime startDatetime = LocalDateTime.of(year,month,day,0,0,0).minusDays(1);
        LocalDateTime endDatetime = startDatetime.plusDays(2).plusHours(1);

        Optional<ConfigVO> printerDevices = configService.findOne(PRINTER_DEVICE_RELATION);
        if (!printerDevices.isPresent()) {
            logger.error("{}配置项不存在", PRINTER_DEVICE_RELATION);
            throw new Exception(PRINTER_DEVICE_RELATION + "配置项不存在");
        }
        //产创的设备
        List<Map> deviceList = JSONObject.parseArray(printerDevices.get().getValue(),Map.class);
        /*
        //DEVICE_SBLX-设备类型CODE --5719    DEVICE_SBLX_NAME-设备类型名称  --3DP打印机
        NxDeviceVO v = new NxDeviceVO();
//        v.setDEVICE_SBLX("5719");
//        v.setDEVICE_SBLX_NAME("3DP打印机");
        v.setDeviceTypeId("5719");
        v.setDeviceTypeName("3DP打印机");
        //1.取出打印机所有的设备编号
        List<NxDeviceVO> list = nxDeviceDao.getDeviceList(v);
        */
        // 对每一个设备型号进行处理
        List<RepPrinterSandPercentVO> resultList = new ArrayList<>(16);

        for (Map device: deviceList) {
            Object deviceId = device.get("device");
            if (deviceId != null) {
                NxDeviceVO v = new NxDeviceVO();
                v.setDeviceTypeId("5719");
                v.setDeviceTypeName("3DP打印机");
                v.setDeviceCode(deviceId.toString());
                List<NxDeviceVO> list1 = nxDeviceDao.getDeviceById(v); //返回只有一条记录

                if(list1.size() >0){
                    String deviceTypeId = list1.get(0).getDeviceTypeId();
                    deviceId =  list1.get(0).getDeviceCode();
                    String modelId = list1.get(0).getModeID();
                    String thingId = list1.get(0).getThingID();
                    resultList.addAll(processResultByGK(LocalDate.of(year,month,day), startDatetime, endDatetime,
                            deviceTypeId,deviceId.toString().trim(),modelId,thingId));
                }
            }
        }
/*
        for(int i=0;i<list.size();i++)
        {
//            String deviceCode=list.get(i).getDEVICE_SBBM();
//            String modelId=list.get(i).getMODEL_ID();
//            String thingId=list.get(i).getNX_JE_DEVICE_ID();
            String deviceId=list.get(i).getDeviceCode();
            String modelId=list.get(i).getModeID();
            String thingId=list.get(i).getThingID();
            String deviceTypeId=list.get(i).getDeviceTypeId();

            resultList.addAll(processResultByGK(LocalDate.of(year,month,day), startDatetime, endDatetime,
                    deviceTypeId,deviceId.toString().trim(),modelId,thingId));
        }
        */
        if (CollectionUtils.isEmpty(resultList)) {
            return;
        }

        logger.info("统计打印机工况数据结果 {}-{}-{} size：{}", year,month,day,resultList==null?"0":resultList.size());
        List<RepPrinterSandPercentVO> saveResultList = resultList.stream()
                .filter(x->year.equals(x.getDataYear()) && month.equals(x.getDataMonth()) && day.equals(x.getDataDay()))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(saveResultList)) {
            logger.info("统计打印机工况数据结果爲空");
            return;
        }
        logger.info("统计打印机工况数据结果待保存的 {}-{}-{} size：{}", year,month,day,saveResultList==null?"0":saveResultList.size());
        //将计算的结果存储到结果表中去，需要将之前的数据清空
        repPrinterSandPercentDAO.clear(year, month, day);
        repPrinterSandPercentDAO.batchAdd(saveResultList, year, month, day);
    }

    //跑批所有单台机过程统计表
    public void run1(Calendar calendar) throws Exception {
        Integer year = calendar.get(Calendar.YEAR);
        Integer month = calendar.get(Calendar.MONTH) + 1;
        Integer day = calendar.get(Calendar.DAY_OF_MONTH);

        LocalDateTime startDatetime = LocalDateTime.of(year,month,day,0,0,0).minusDays(1);
        LocalDateTime endDatetime = startDatetime.plusDays(2).plusHours(1);

        //DEVICE_SBLX-设备类型CODE --5719    DEVICE_SBLX_NAME-设备类型名称  --3DP打印机
        NxDeviceVO v = new NxDeviceVO();
//        v.setDEVICE_SBLX("5719");
//        v.setDEVICE_SBLX_NAME("3DP打印机");
        v.setDeviceTypeId("5719");
        v.setDeviceTypeName("3DP打印机");
        //1.取出打印机所有的设备编号
        List<NxDeviceVO> list = nxDeviceDao.getDeviceList(v);
        // 对每一个设备型号进行处理
        List<RepPrinterSandPercentVO> resultList = new ArrayList<>(16);
        for(int i=0;i<list.size();i++)
        {
//            String deviceCode=list.get(i).getDEVICE_SBBM();
//            String modelId=list.get(i).getMODEL_ID();
//            String thingId=list.get(i).getNX_JE_DEVICE_ID();
            String deviceId=list.get(i).getDeviceCode();
            String modelId=list.get(i).getModeID();
            String thingId=list.get(i).getThingID();
            String deviceTypeId=list.get(i).getDeviceTypeId();
            resultList.addAll(processResultByGK(LocalDate.of(year,month,day), startDatetime, endDatetime,
                    deviceTypeId,deviceId.toString().trim(),modelId,thingId));
        }
        if (CollectionUtils.isEmpty(resultList)) {
            return;
        }

        logger.info("统计打印机工况数据结果 {}-{}-{} size：{}", year,month,day,resultList==null?"0":resultList.size());
        List<RepPrinterSandPercentVO> saveResultList = resultList.stream()
                .filter(x->year.equals(x.getDataYear()) && month.equals(x.getDataMonth()) && day.equals(x.getDataDay()))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(saveResultList)) {
            logger.info("统计打印机工况数据结果爲空");
            return;
        }
        logger.info("统计打印机工况数据结果待保存的 {}-{}-{} size：{}", year,month,day,saveResultList==null?"0":saveResultList.size());
        //将计算的结果存储到结果表中去，需要将之前的数据清空
        repPrinterSandPercentDAO2.clear(year, month, day);
        repPrinterSandPercentDAO2.batchAdd(saveResultList, year, month, day);
    }


    /**
     * 处理每一个设备型号
     *  Timestamp_Local	设备本地数据采集时间
     *  HMI_JobID	打印编号
     *	HMI_BeginTime	开始打印时间
     *	HMI_EndTime	打印结束时间
     *  HMI_TotalTimeCost 打印耗时
     *  HMI_CurPrintLayer 当前打印层数
     *  树脂耗量(SB_BinderCost)
     *  打印体积(HMI_Volume)
     *  A砂用量 (SB_MX_ASandLv)
     *  B砂用量 (SB_MX_BSandLv)
     *  混砂计数(SB_MX_MixingTimes)
     *  计算公式：
     *	树脂加入比 = 树脂用量/（打印体积*1.5 ）
     *	新旧砂加入占比 = A砂用量 / B砂用量  数据显示用3/7、4/6、5/5这种类型展示数据
     *	漏砂量=混砂计数*（A砂用量 + B砂用量） - 打印体积*1.5）
     *
     *  同一设备 同一打印编码 打印体积最大
     *
     */
    private Collection<RepPrinterSandPercentVO> processResultByGK(
            LocalDate localDate,
            LocalDateTime startDatetime,
            LocalDateTime endDatetime,
            String deviceTypeId,
            String deviceId,String modelId,String thingId){

        int from = 0, total = 0;
        String DATE_FORMAT_YYYY_MM_DD = "yyyy-MM-dd HH:mm:ss";
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern(DATE_FORMAT_YYYY_MM_DD);
        List<RepPrinterSandPercentVO> resultList = new ArrayList<>(16);
        Map<String, Map<String, Object>> resultMap = new HashMap<>(16);
        Map<String, Map<String, Object>> resultMap1 = new HashMap<>(16);
        List<Map<String, Object>> sortedList = new ArrayList<>();

        Set<String> jobIds = new HashSet<>();
        float lastVolume = 0f;
        float lastTotalTimeCost = 0f;
        String DATE_FORMAT_DAY = "yyyy-MM-dd";
        DateTimeFormatter dtfDay = DateTimeFormatter.ofPattern(DATE_FORMAT_DAY);
        String currDate = startDatetime.toLocalDate().format(dtfDay);
        String today = localDate.format(dtfDay);
        String yestoday = localDate.minusDays(1).format(dtfDay);
        String tomorrow = localDate.plusDays(1).format(dtfDay);

        //调用API
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer " + tokenUtil.getToken());
        String errorPos ="";
        int sumCount =0;
        try
        {
            DateTimeFormatter dfs = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            String startTime =startDatetime.plusHours(-8).format(dfs);
            String endTime=   endDatetime.plusHours(-8).format(dfs);
            startTime = DateUtil.handlerTime(startTime);
            endTime = DateUtil.handlerTime(endTime);

            //http://openapi.zengcaicloud.com/historian-manage/v1/historian/models/197ZUeGcXkY/things/199DK04jaWA?startTime=2022-05-01T02:30:00.000Z&endTime=2022-05-06T03:20:30.000Z
            String resp = HttpClientUtil.sendHttp(HttpRequestMethedEnum.HttpGet, reqUrl + "/historian-manage/v1/historian/models/"
                    + modelId + "/things/" + thingId+""+"?startTime="+startTime+"&endTime="+endTime+"&skip=0&limit=20000", null, headers);
//            logger.info("平台历史工况接口返回数据：{}", resp);
            JSONObject jsonObject = JSONObject.parseObject(resp);
            JSONArray array = jsonObject.getJSONArray("payload");
            JSONArray colums = array.getJSONObject(0).getJSONArray("columns");
            JSONArray rows = array.getJSONObject(0).getJSONArray("rows");

            //取出需要分析的属性字段的下标
            int jobid_serque = getSequeByKey(colums, "HMI_JobID");
            int time_serque = getSequeByKey(colums, "time");
            int HMI_TotalLayer_seq = getSequeByKey(colums, "HMI_TotalLayer");
            int HMI_CurPrintLayer_seq = getSequeByKey(colums, "HMI_CurPrintLayer");
            int HMI_Volume_seq = getSequeByKey(colums, "HMI_Volume");
            int HMI_Thicknessr_seq = getSequeByKey(colums, "HMI_Thickness");
            int HMI_TotalTimeCost_seq = getSequeByKey(colums, "HMI_TotalTimeCost");
            int HMI_BeginTime_req = getSequeByKey(colums, "HMI_BeginTime");
            int HMI_EndTime_req = getSequeByKey(colums, "HMI_EndTime");
//            int GK_SC_DY_BeginTime_req= getSequeByKey(colums, "GK_SC_DY_BeginTime");
//            int GK_SC_DY_EndTime_req = getSequeByKey(colums, "GK_SC_DY_EndTime");
            int SB_BinderCost_req= getSequeByKey(colums, "SB_BinderCost"); //树脂耗量
            int SB_ActivatorCost_req =getSequeByKey(colums, "SB_ActivatorCost"); //固化剂耗量
            int SB_CleanerCost_req= getSequeByKey(colums, "SB_CleanerCost"); //清洗剂耗量
            int SB_MX_SandTotal_A_req = getSequeByKey(colums, "SB_MX_SandTotal_A"); //B56 A砂累计用量
            int SB_MX_SandTotal_B_req = getSequeByKey(colums, "SB_MX_SandTotal_B");  //B57 B砂累计用量
            int SB_MX_MixingTimes_req =getSequeByKey(colums, "SB_MX_MixingTimes");  //混砂计数
            int HMI_Binder_add_req = getSequeByKey(colums, "HMI_Binder_add");  //树脂加入量
            int HMI_LayerCostTime_req = getSequeByKey(colums, "HMI_LayerCostTime");  //打印层耗时  B18


//            for (int dx=1;dx< rows.size();dx++)
//            {
//                if(dx%20 == 0)
//                {
//                    String Timestamp_Local ="";  //当前工况时间
//                    String UTCtimeStr = getString(rows.getJSONArray(dx).get(time_serque)); //UTC 格式的时间
//                    Timestamp_Local =  DateUtil.UTCtoBeijing(UTCtimeStr);//UTC转换为北京时间   Timestamp_Local
//                    logger.info(Timestamp_Local);
//                }
//            }
            //由于4.0工况中某个批次打印结束的阶段，可能会出现打印编号、A砂累计用量 值为空值 出现跳空甚至值为null的情况，为了延续前后两条记录之间是同一批次，
            // 则要记录上一次有效的数据序号last_idx，以便于前后两条记录之间的比较
            int last_idx =0;
            for (int idx=1;idx< rows.size();idx++ ) {
//                if(idx%10 != 0)  //每间隔10条记录输出一条
//                {
//                    continue;
//                }
                sumCount++;
                errorPos = String.valueOf(idx);
                String cur_HMI_JobID = getString(rows.getJSONArray(idx).get(jobid_serque));//HMI_JobID	打印编号
                if( StringUtils.isBlank(cur_HMI_JobID))
                {
                    continue;
                }
                if(idx == 1)
                    last_idx = 0; //记录上一条有效的记录

    //			if(cur_HMI_JobID.contains("试块") || cur_HMI_JobID.contains("调试")||
    //					cur_HMI_JobID.contains("测试")|| cur_HMI_JobID.contains("实验")
    //					|| cur_HMI_JobID.contains("调整"))
    //			{
    //				continue;
    //				//试块、调试的打印任务自动过滤掉
    //			}
                //打印体积(HMI_Volume)
                float HMI_Volume = getFloat(rows.getJSONArray(idx).get(HMI_Volume_seq)); //HMI_Volume
                //HMI_CurPrintLayer 当前打印层数
                float HMI_CurPrintLayer = getFloat(rows.getJSONArray(idx).get(HMI_CurPrintLayer_seq)); //HMI_CurPrintLayer

                float HMI_TotalTimeCost = getFloat(rows.getJSONArray(idx).get(HMI_TotalTimeCost_seq));// HMI_TotalTimeCost
                String last_HMI_JobID = getString(rows.getJSONArray(last_idx).get(jobid_serque)); //idx-1
                //MapUtils.getString(lastData, "HMI_JobID", MapUtils.getString(data, "GK_SC_DY_JobID"));
                if( !cur_HMI_JobID.equals(last_HMI_JobID))
                {
                    float last_HMI_Volume = getFloat(rows.getJSONArray(last_idx).get(HMI_Volume_seq)); //"HMI_Volume"  前一条  idx-1
                    float last_HMI_CurPrintLayer = getFloat(rows.getJSONArray(last_idx).get(HMI_CurPrintLayer_seq));// "HMI_CurPrintLayer" 前一条 idx-1
                    float last_HMI_TotalTimeCost = getFloat(rows.getJSONArray(last_idx).get(HMI_TotalTimeCost_seq)); //"HMI_TotalTimeCost" 前一条 idx-1
                    if ( HMI_Volume == last_HMI_Volume || HMI_CurPrintLayer ==  last_HMI_CurPrintLayer
                            ||  HMI_TotalTimeCost == last_HMI_TotalTimeCost)
                    {
                        lastVolume = last_HMI_Volume;
                        lastTotalTimeCost = last_HMI_TotalTimeCost;
                        last_idx =idx; //记录上一条有效的记录
                        continue;
                    }else{
                        lastVolume = 0;
                        lastTotalTimeCost = 0;
                    }
                }
                else if( lastVolume == HMI_Volume || lastTotalTimeCost == HMI_TotalTimeCost)
                {
                    last_idx =idx; //记录上一条有效的记录
                    continue;
                }

                jobIds.add(cur_HMI_JobID);
                String Timestamp_Local ="";  //当前工况时间
                String UTCtimeStr = getString(rows.getJSONArray(idx).get(time_serque)); //UTC 格式的时间
                Timestamp_Local =  DateUtil.UTCtoBeijing(UTCtimeStr);//UTC转换为北京时间   Timestamp_Local
                String cloudtime = Timestamp_Local;//  MapUtils.getString(data, "cloudtime");
                String key = Timestamp_Local.substring(0,10) + "-" + cur_HMI_JobID ;//设备编码和打印编码 表示一条打印记录
                Map<String, Object> hisData = resultMap.computeIfAbsent(cur_HMI_JobID, m -> new HashMap<>(16));
                Map<String, Object> hisData1 = resultMap1.computeIfAbsent(key, m -> new HashMap<>(16));
                hisData.put("cloudtime", Timestamp_Local.substring(0,10));
                hisData1.put("cloudtime", Timestamp_Local.substring(0,10));
                hisData.put("deviceId", deviceId);
                hisData.put("HMI_JobID", cur_HMI_JobID);
                hisData1.put("HMI_JobID", cur_HMI_JobID);
                hisData.put("GK_SC_DY_JobID", "");
                hisData.put("Timestamp_Local",Timestamp_Local);
                hisData.put("HMI_BeginTime", getString(rows.getJSONArray(idx).get(HMI_BeginTime_req)));
                hisData.put("GK_SC_DY_BeginTime", "");
                hisData.put("HMI_EndTime", getString(rows.getJSONArray(idx).get(HMI_EndTime_req)));
                hisData.put("GK_SC_DY_EndTime", "");

                if (HMI_Volume > getFloat(hisData,"HMI_Volume")) {
                    hisData.put("HMI_Volume",HMI_Volume);
                }
                if (HMI_Volume > getFloat(hisData1,"HMI_Volume")) {
                    hisData1.put("HMI_Volume", HMI_Volume);
                }
                if (HMI_TotalTimeCost > getFloat(hisData,"HMI_TotalTimeCost")) {
                    hisData.put("HMI_TotalTimeCost", HMI_TotalTimeCost);
                }
                if (HMI_CurPrintLayer > getFloat(hisData,"HMI_CurPrintLayer")) {
                    hisData.put("HMI_CurPrintLayer", HMI_CurPrintLayer);
                }
                float SB_BinderCost = getFloat(rows.getJSONArray(idx).get(SB_BinderCost_req));// SB_BinderCost
                float SB_ActivatorCost = getFloat(rows.getJSONArray(idx).get(SB_ActivatorCost_req));// SB_ActivatorCost
                float SB_CleanerCost = getFloat(rows.getJSONArray(idx).get(SB_CleanerCost_req));// SB_CleanerCost

                if (SB_BinderCost > getFloat(hisData,"SB_BinderCost")) {  //树脂耗量
                    hisData.put("SB_BinderCost", SB_BinderCost);
                }
                if (SB_ActivatorCost > getFloat(hisData,"SB_ActivatorCost")) { //固化剂耗量
                    hisData.put("SB_ActivatorCost", SB_ActivatorCost);
                }
                if (SB_CleanerCost > getFloat(hisData,"SB_CleanerCost")) {  //清洗剂耗量
                    hisData.put("SB_CleanerCost", SB_CleanerCost);
                }
                float SB_MX_SandTotal_A = getFloat(rows.getJSONArray(idx).get(SB_MX_SandTotal_A_req));// B56  A砂累积用量
                float SB_MX_SandTotal_A_last = getFloat(rows.getJSONArray(last_idx).get(SB_MX_SandTotal_A_req));//  idx-1
                float SB_MX_SandTotal_B = getFloat(rows.getJSONArray(idx).get(SB_MX_SandTotal_B_req));// B57  B砂累积用量
                float SB_MX_SandTotal_B_last = getFloat(rows.getJSONArray(last_idx).get(SB_MX_SandTotal_B_req));// idx-1
                float HMI_CurPrintLayer_last = getFloat(rows.getJSONArray(last_idx).get(HMI_CurPrintLayer_seq)); //HMI_CurPrintLayer  idx-1
                /*
                if (last_HMI_JobID.equals(cur_HMI_JobID))
                {
                    if ( SB_MX_SandTotal_A >= SB_MX_SandTotal_A_last && HMI_CurPrintLayer >= HMI_CurPrintLayer_last) {
                        float B56_ALL = SB_MX_SandTotal_A + getFloat(hisData,"B56_SUM_");
                        hisData.put("B56", B56_ALL);
                        hisData.put("SB_MX_ASandLv", B56_ALL);//A砂累积用量
                    }
                    else if (HMI_CurPrintLayer >= HMI_CurPrintLayer_last  && SB_MX_SandTotal_A <= SB_MX_SandTotal_A_last ) {  // /2
                        float  B56_SUM_ = getFloat(hisData,"B56_SUM_") + SB_MX_SandTotal_A_last;
                        hisData.put("B56_SUM_", B56_SUM_);
                    }
                    if (SB_MX_SandTotal_B >= SB_MX_SandTotal_B_last && HMI_CurPrintLayer >= HMI_CurPrintLayer_last) {
                        float B57_ALL = SB_MX_SandTotal_B + getFloat(hisData,"B57_SUM_");
                        hisData.put("B57", B57_ALL);
                        hisData.put("SB_MX_BSandLv", B57_ALL);//B砂累积用量
                    }else if (HMI_CurPrintLayer > HMI_CurPrintLayer_last && SB_MX_SandTotal_B <= SB_MX_SandTotal_B_last ) {   // /2
                        float  B57_SUM_ = getFloat(hisData,"B57_SUM_") + SB_MX_SandTotal_B_last;
                        hisData.put("B57_SUM_", B57_SUM_);
                    }
                }
                */

//                hisData.put("B56", SB_MX_SandTotal_A);
//                hisData.put("SB_MX_ASandLv", SB_MX_SandTotal_A);//A砂累积用量
//
//                hisData.put("B57", SB_MX_SandTotal_B);
//                hisData.put("SB_MX_BSandLv", SB_MX_SandTotal_B);//A砂累积用量
//                if(cur_HMI_JobID.equals("")&& last_HMI_JobID.equals("X25D21.cli") && SB_MX_SandTotal_A >= 3000 && HMI_CurPrintLayer >=1355){
//                    String str = "";
//                }
                if (last_HMI_JobID.equals(cur_HMI_JobID))
                {
//                    if(last_HMI_JobID.equals("X25L07.cli") && HMI_CurPrintLayer == 1559){
//                        String id ="X25L07.cli";
//                    }
//                    if(last_HMI_JobID.equals("X25D21.cli")){
//                        String id ="X25D21.cli";
//                    }
//                    if(last_HMI_JobID.equals("X25D21.cli") && SB_MX_SandTotal_A >= 3000 && HMI_CurPrintLayer >=1355){
//                        String id ="X25D21.cli";
//                    }
                    //if ...比较当前记录和上一条有效记录之间的大小关系，正常的上升情况则记录当前的值为该批次打印任务的最新值
                    //else...SB_MX_SandTotal_A=0,SB_MX_SandTotal_A_last =3412,则当前记录跳空或者异常，则要把上一条有效值作为该批次打印任务当前的最新值
                    float sum56 = getFloat(hisData,"B56_SUM_"); //A砂总累计数据
                    float sum57 = getFloat(hisData,"B57_SUM_"); //B砂总累计数据
                    if ( SB_MX_SandTotal_A >= SB_MX_SandTotal_A_last && HMI_CurPrintLayer >= HMI_CurPrintLayer_last && SB_MX_SandTotal_A >= sum56) {
                        float B56_ALL = SB_MX_SandTotal_A + getFloat(hisData,"B56_SUM_");
                        hisData.put("B56", B56_ALL);
                        hisData.put("SB_MX_ASandLv", B56_ALL);//A砂累积用量
                    }else if (HMI_CurPrintLayer >= HMI_CurPrintLayer_last && SB_MX_SandTotal_A < SB_MX_SandTotal_A_last ) {  // /2
                        float  B56_SUM_ = getFloat(hisData,"B56_SUM_") + SB_MX_SandTotal_A_last;
                        hisData.put("B56_SUM_", B56_SUM_);
                    }
                    if (SB_MX_SandTotal_B >= SB_MX_SandTotal_B_last && HMI_CurPrintLayer >= HMI_CurPrintLayer_last && SB_MX_SandTotal_B >= sum57) {
                        float B57_ALL = SB_MX_SandTotal_B + getFloat(hisData,"B57_SUM_");
                        hisData.put("B57", B57_ALL);
                        hisData.put("SB_MX_BSandLv", B57_ALL);//B砂累积用量
                    }else if (HMI_CurPrintLayer >= HMI_CurPrintLayer_last && SB_MX_SandTotal_B < SB_MX_SandTotal_B_last ) {   // /2
                        float  B57_SUM_ = getFloat(hisData,"B57_SUM_") + SB_MX_SandTotal_B_last;
                        hisData.put("B57_SUM_", B57_SUM_);
                    }
                }
                float SB_MX_MixingTimes = getFloat(rows.getJSONArray(idx).get(SB_MX_MixingTimes_req));//混砂计数
                float HMI_Binder_add = getFloat(rows.getJSONArray(idx).get(HMI_Binder_add_req));//B58 树脂加入量
                float HMI_LayerCostTime = getFloat(rows.getJSONArray(idx).get(HMI_LayerCostTime_req));//B18 打印层耗时
                float HMI_Thickness = getFloat(rows.getJSONArray(idx).get(HMI_Thicknessr_seq));//HMI_Thickness

                if (SB_MX_MixingTimes > 0) {//混砂计数
                    hisData.put("SB_MX_MixingTimes",SB_MX_MixingTimes);
                }
                if (HMI_Binder_add > 0) {  //B58 树脂加入量 --HMI_Binder_add
                    hisData.put("B58",HMI_Binder_add);
                }
                if (HMI_LayerCostTime > 0) {   //B18 层打印耗时 HMI_LayerCostTime
                    hisData.put("B18",HMI_LayerCostTime);
                }
                if (HMI_Thickness > getFloat(hisData,"HMI_Thickness")){  //打印层厚
                    hisData.put("HMI_Thickness", HMI_Thickness);
                }
                hisData.put("cloudtime", cloudtime);
                last_idx =idx; //记录上一条有效的记录
                //logger.info("设备编号:"+deviceId+ " |JobID:"+cur_HMI_JobID +" |总层数："+HMI_TotalLayer+" |当前打印层数:"+HMI_CurPrintLayer+" |Beijing-time:"+Timestamp_Local+" |UTC-time:"+time);
            }
            //JSONObject rows2 = array.getJSONObject(0).getJSONObject("rows");
//            List<DeviceHistorySituationVO> deviceHistorySituationVO = callHistoryInterface1(historySituationAO.getDeviceId(), jeDeviceEntity.getModelId(), paramMap);
//            PageInfo<DeviceHistorySituationVO> deviceHistorySituationVOPageInfo = this.queryPageInfo(skip, limit, deviceHistorySituationVO);
        }catch (Exception ex)
        {
            logger.info(errorPos);
            logger.error("调用平台查询实时工况信息error", ex);
        }
        logger.info("有效取数次数:"+sumCount);
        List<Map<String,Object>> todayList = resultMap1.values().stream()
                .filter(x->today.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
        List<Map<String,Object>> yestodayList = resultMap1.values().stream()
                .filter(x->yestoday.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
        List<Map<String,Object>> tomorrowList = resultMap1.values().stream()
                .filter(x->tomorrow.equals(MapUtils.getString(x, "cloudtime"))).collect(Collectors.toList());
        Set<String> includeJobIds = new HashSet<>();
        for (Map<String,Object> todayData: todayList)
        {
            String jobId = MapUtils.getString(todayData, "HMI_JobID");
            boolean include = true;
            float HMI_Volume = getFloat(todayData, "HMI_Volume");
            if (yestodayList.stream().anyMatch(x->{
                if(jobId.equals(MapUtils.getString(x, "HMI_JobID"))&& HMI_Volume <= getFloat(x,"HMI_Volume"))
                {
                    return true;
                }
                else {
                    return false;
                }}))
            {
                include=false;
            }
            else if (tomorrowList.stream().anyMatch(x->{
                if(jobId.equals(MapUtils.getString(x, "HMI_JobID"))&& HMI_Volume < getFloat(x,"HMI_Volume"))
                {
                    return true;
                }
                else
                {
                    return false;
                }}))
            {
                include=false;
            }
            if (include)
            {
                includeJobIds.add(jobId);
            }
        }

        if( CollectionUtils.isEmpty(includeJobIds) ) {
            return resultList;
        }
        //将结果排序
        Collection<Map<String, Object>> collection = resultMap.values().stream()
                .filter(x->includeJobIds.contains(MapUtils.getString(x, "HMI_JobID"))).collect(Collectors.toList());
        if(logger.isDebugEnabled()){
            logger.debug("--->collection:{}", JSON.toJSONString(collection));
        }
        if( CollectionUtils.isEmpty(collection) ) {
            return resultList;
        }
        int year = localDate.getYear();
        int month = localDate.getMonthValue();
        int day = localDate.getDayOfMonth();

        for(Map<String, Object> data: collection)
        {
            //打印体积(HMI_Volume)
            float HMI_Volume = getFloat(data, "HMI_Volume");
            if (HMI_Volume < 5) {
                continue;
            }
            //HMI_CurPrintLayer 当前打印层数
            int HMI_CurPrintLayer = 0;
            try{
                HMI_CurPrintLayer = new Float(getFloat(data,"HMI_CurPrintLayer")).intValue();
            }catch(Exception e){}
            //低于200层的是垃圾数据
            if( HMI_CurPrintLayer < 200 ){
                continue;
            }
            //A砂累积用量 (SB_MX_ASandLv)
            float SB_MX_ASandLv = getFloat(data, "SB_MX_ASandLv");
            //HMI_JobID	打印编号
            String HMI_JobID = MapUtils.getString(data, "HMI_JobID");

            //HMI_BeginTime	开始打印时间
            String HMI_BeginTime = MapUtils.getString(data, "HMI_BeginTime", MapUtils.getString(data, "GK_SC_DY_BeginTime"));
            //HMI_EndTime	打印结束时间
            String HMI_EndTime = MapUtils.getString(data, "HMI_EndTime", MapUtils.getString(data, "GK_SC_DY_EndTime"));
            //HMI_TotalTimeCost 打印耗时
            float HMI_TotalTimeCost = getFloat(data, "HMI_TotalTimeCost");


            //树脂耗量(SB_BinderCost)
            float sB_BinderCost = getFloat(data, "SB_BinderCost");

            //B砂累积用量 (SB_MX_BSandLv)
            float SB_MX_BSandLv = getFloat(data, "SB_MX_BSandLv");
            //混砂计数(SB_MX_MixingTimes)
            float SB_MX_MixingTimes = getFloat(data, "SB_MX_MixingTimes");
            //树脂加入量
            float b58 = getFloat(data, "B58");
            float HMI_Thickness = getFloat(data, "HMI_Thickness");
            float SB_ActivatorCost = getFloat(data, "SB_ActivatorCost"); //固化剂耗量
            float SB_CleanerCost = getFloat(data, "SB_CleanerCost"); //清洗剂耗量
            float b18 = getFloat(data, "B18"); //层打印耗时

            /*
             *  计算公式：
             *	树脂加入比 = 树脂用量/（打印体积*1.5 ）
             *	新砂占比计算公式=A砂累计用量/（A砂累计用量+B砂累计用量）
             *	漏砂量=混砂计数*（A砂用量 + B砂用量） - 打印体积*1.5）
             */
            //gkScDyLsl漏砂量
            double gkScWsysl =  getWsySL(data);
            double gkScDyLsl =  getLSL(data,gkScWsysl);

            //gkScDyXjszb新旧砂加入占比
            String gkScDyXjszb = "0";
            if (SB_MX_ASandLv+SB_MX_BSandLv > 0.01) {
                gkScDyXjszb = new DecimalFormat("0.000").format(SB_MX_ASandLv/(SB_MX_ASandLv+SB_MX_BSandLv));
            }

            RepPrinterSandPercentVO vo = new RepPrinterSandPercentVO(deviceId,deviceTypeId)
                    .officeId("")
                    .dataYear(year)
                    .dataMonth(month)
                    .dataDay(day)
                    .gkScDyLsl(BigDecimal.valueOf(gkScDyLsl < 0?0:gkScDyLsl))
                    .gkScDyWsysl(BigDecimal.valueOf(gkScWsysl < 0?0:gkScWsysl))
                    .gkScDySxmd(BigDecimal.valueOf(b58)) //树脂加入量
                    .gkScDyXjszb(gkScDyXjszb)
                    .sbBindercost(BigDecimal.valueOf(sB_BinderCost))
                    .sbMxAsandlv(BigDecimal.valueOf(SB_MX_ASandLv))
                    .sbMxBsandlv(BigDecimal.valueOf(SB_MX_BSandLv))
                    .sbMxMixingtimes(BigDecimal.valueOf(SB_MX_MixingTimes))
                    .hmiVolume(BigDecimal.valueOf(HMI_Volume))
                    .hmiJobid(HMI_JobID)
                    .hmiBegintime(HMI_BeginTime)
                    .hmiEndtime(HMI_EndTime)
                    .hmiTotaltimecost(BigDecimal.valueOf(HMI_TotalTimeCost))
                    .hmiLayercosttime(BigDecimal.valueOf(b18)) //层打印耗时
                    .hmiCurprintlayer(HMI_CurPrintLayer)
                    .hmiThickness(BigDecimal.valueOf(HMI_Thickness))
                    .sbActivatorcost(BigDecimal.valueOf(SB_ActivatorCost))
                    .hmiCleanerCost(BigDecimal.valueOf(SB_CleanerCost));

            resultList.add(vo);
        }
        return  resultList;
    }

    public String getString(Object value){
        if (value != null){
            return  value.toString();
        }
        else
            return "";
    }
    public float getFloat(Object str){
        float value = 0f;
        if (str != null) {
            value = Float.parseFloat(str.toString());
        }
        return value;
    }



    /**
     * 未使用砂量=（A砂累计用量+B砂累计用量）-2*打印机工作箱长*工作箱宽*打印层厚*(打印层数+12)*1.5
     * 以上计算公式为2215、2515、2518、SDream、2218机型的计算公式，Max、1800机型的计算公式中无*2
     * @param data 工况数据
     * @return 打印效率
     */
    private float getWsySL(Map<String, Object> data) {
        // 新模型时进行计算，计算逻辑见doc/打印机模型切换需求整理.md
        String deviceId = MapUtils.getString(data, "deviceId");
        String jobID = MapUtils.getString(data, "HMI_JobID");
        final float ratio2 = CYCXZX_DEVICES.contains(deviceId)?1.4f:1.5f;
        float rat = 0f;
        //创业中心设备 && 陶粒砂1.5
        if(jobID.indexOf("陶粒砂") > 0 && CYCXZX_DEVICES.contains(deviceId))
        {
            rat = 1.5f;
        }
        else
        {
            rat = 1.4f;
        }
        final float ratio = rat;
        return getWorkBoxSize(deviceId).map(workBoxSize -> {
            float dycs = getFloat(data, "HMI_CurPrintLayer");
            float dyhd = getFloat(data, "HMI_Thickness");
            float aSandLv = getFloat(data, "SB_MX_ASandLv"); //A砂用量
            float bSandLv = getFloat(data, "SB_MX_BSandLv"); //B砂用量
            double length = workBoxSize.getLength();
            double width = workBoxSize.getWidth();
            if (aSandLv+bSandLv < 0.01) {
                return 0f;
            }
            Double result = 0d;

            // 2215 2515 2518 2218 SDream 共5类双箱机型
            if (deviceId.startsWith("2215") || deviceId.startsWith("2515") || deviceId.startsWith("2518")
                    ||deviceId.startsWith("2218")||deviceId.startsWith("SDream")) {
                result = (aSandLv + bSandLv ) - ((dycs+12) * dyhd * length * width * 2 * ratio)/100F;
                return result.floatValue();
            } else {
                result = (aSandLv + bSandLv ) - ((dycs+12) * dyhd * length * width * ratio)/100F;
                return result.floatValue();
            }

//            if (!deviceId.startsWith("1800") && !deviceId.startsWith("Max")
//                    && !deviceId.startsWith("800")&& !deviceId.startsWith("1000")&& !deviceId.startsWith("2600")) {
//                result = (aSandLv + bSandLv ) - ((dycs+12) * dyhd * length * width * 2 * ratio)/100F;
//                return result.floatValue();
//            } else {
//                result = (aSandLv + bSandLv ) - ((dycs+12) * dyhd * length * width * ratio)/100F;
//                return result.floatValue();
//            }
        }).orElse(0F);
    }


    /**
     * 砂利用率=（混砂用量 - 未使用砂量）/ 混砂用量
     * @param data 工况数据
     * @return 打印效率
     */
    private float getLSL(Map<String, Object> data,double gkScWsysl) {
        // 新模型时进行计算，计算逻辑见doc/打印机模型切换需求整理.md
        Double result = 0d;
        float aSandLv = getFloat(data, "SB_MX_ASandLv");
        float bSandLv = getFloat(data, "SB_MX_BSandLv");
        if (aSandLv + bSandLv < 0.1) {
            return 0;
        }
        result = (aSandLv + bSandLv - gkScWsysl)/(aSandLv + bSandLv);
        return result.floatValue();
    }


    /**
     * 根据设备编号查询工作箱尺寸
     *
     * @param deviceId 设备编号
     * @return 工作箱尺寸
     */
    private Optional<PrinterWorkBoxSize> getWorkBoxSize(String deviceId) {
        for (String key : Constants.PRINTER_WORK_BOX_SIZE_MAP_slyl.keySet()) {
            if (deviceId.startsWith(key)) {
                return Optional.ofNullable(Constants.PRINTER_WORK_BOX_SIZE_MAP_slyl.get(key));
            }
        }

        return Optional.empty();
    }

    private int getSequeByKey(JSONArray metric, String key) {
        JSONArray columns = metric;
        int keyIndex = -1;
        for (int i=0;i<columns.size();i++) {
            String columnName = columns.getString(i);
            if (key.equals(columnName)) {
                keyIndex=i;
                break;
            }
        }
        return keyIndex;
    }


    /**
     * 从Map中获取浮点数
     */
    private float getFloat(
            Map<String, Object> map,
            String key) {
        Object str = map.getOrDefault(key, "0");
        float value = 0f;
        if (null != str && !"null".equals(str)) {
            value = Float.parseFloat(str.toString());
        }
        return value;
    }


    public void createUserReportV2(QueryUserReportCondition c) throws Exception{

        String param = JSON.toJSONString(c);
        logger.info("method : createUserReport,  param : "+ param);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        //获得客户信息
        List<CustomerVO> customers = nxCustomerDao.getAllCustomer(c);
        if(customers==null || customers.size()==0) {
            logger.info("未查询到客户信息，生成打印机用户报告失败，请联系管理员！！！");
            return;
        }
        logger.info("method : createUserReport,  param : "+ param+" customers.size="+customers.size());
        //查询报表各个指标目标值
        Map<String, Object> targetmap = new HashMap<String, Object>();
        List<RepPrinterTargetDicVO> targets = repPrinterTargetDicService.groupByPrinterTargetDic();

        //从配置表中获取平台设备编码及用户设备编码关系，加载 配置表中的关系
        Map<String,String>	printerDeviceRelationData =	getPrinterDeviceRelation();
        logger.info("method : createUserReport,  param : "+ param+" printerDeviceRelationData="+printerDeviceRelationData);

        Date now = new Date();
        String md = DateUtil.format(now, "MM-dd");
        String d = DateUtil.format(now, "dd");
        int day = now.getDay();//周几

        //系统当前时间
        Calendar calendar = Calendar.getInstance();
        calendar.setFirstDayOfWeek(Calendar.MONDAY);//设置星期一为一周开始的第一天
        calendar.setTimeInMillis(System.currentTimeMillis());//获得当前的时间戳
        int year = calendar.get(Calendar.YEAR);//获得当前的年
        int month = calendar.get(Calendar.MONTH)+1;//获得当前的月
        int weekOfYear = calendar.get(Calendar.WEEK_OF_YEAR);//获得当前日期属于今年的第几周

        String startDate = c.getStartDt();
        String endDate = c.getEndDt() ;
        int createType = c.getCreateType()==null?5:c.getCreateType() ;
        int dataYear = c.getDataYear()==null?0:c.getDataYear() ;
        int dataQuarter = c.getDataQuarter()==null?0:c.getDataQuarter();
        int dataMonth = c.getDataMonth()==null?0:c.getDataMonth();
        int dataWeek = c.getDataWeek()==null?0:c.getDataWeek() ;
        if( StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate) ){
            logger.info("method : createUserReport, 指定开始日期结束日期，生成该时间段数据. startDate:"+startDate+"  endDate:"+endDate);
            c.setCreateType(5);
        }else{
            logger.info("method : createUserReport,根据系统时间生成报告");
        }

        List<RepPrinterReportVO> reports = new ArrayList<RepPrinterReportVO>();
        for(CustomerVO customer : customers) {
            if(c.getCreateCustomerCode()!=null && !c.getCreateCustomerCode().equals(customer.getCustomerCode())) {
                logger.info("method : createUserReport,  param : create customer code="+ c.getCreateCustomerCode() +" customer code ="+customer.getCustomerCode());
                continue;
            }
//            if(StringUtils.isBlank(customer.getOfficeId())) {
//                logger.info("method : createUserReport,  param : customer code ="+customer.getCustomerCode()+" officeId is null");
//                continue;
//            }
            for(RepPrinterTargetDicVO v : targets.stream().filter(
                    x->customer.getCustomerCode().equals(x.getCustomerId())).collect(Collectors.toList())) {
                String key = v.getStatisticsType();
                if(!targetmap.containsKey(key)) {
                    targetmap.put(key, v);
                }
            }
            logger.info("method : createUserReport,  param : "+ param+" targetmap="+targetmap);
            //CreateType 1为周，2为月，3为季，4为年 5指定时间段 (生成报告)
            if(c.getCreateType()==5) {
                Map<String, Object> query = new HashMap<String, Object>();
                query.put("customerId", customer.getCustomerCode());
                query.put("startDate", c.getStartDt());
                query.put("endDate", c.getEndDt());
                logger.info("getPrinterUserReports select date query="+query.toString());
                RepPrinterReportVO v = createPrinterUserReport( targetmap, customer,  5,  year,  null,  month,
                        weekOfYear-1, c.getStartDt(), c.getEndDt(), printerDeviceRelationData);
                reports.add(v);
                continue;
            }

        }
    }

    private RepPrinterReportVO createPrinterUserReport(Map<String, Object> targetmap, CustomerVO c, Integer type, Integer year, Integer quarter, Integer month, Integer week, String startDate, String endDate,Map<String, String> printerDeviceRelationData) throws Exception {
        RepPrinterReportVO vo = new RepPrinterReportVO();
        DecimalFormat df =new DecimalFormat("0.0");
        vo.setEmails(null);
        String reportType = year+"年"+week+"周";
        if(type == 1) {
            //默认周
        } else if(type == 2) {
            reportType = year+"年"+month+"月";
        }else if(type == 3) {
            reportType = year+"年"+quarter+"季";
        }else if(type==4) {
            reportType = year+"年";
        }  else if(type == 5) {
            reportType = startDate+"至"+endDate;
        }
        String title = c.getCustomerName()+reportType+"打印机运维报告";
        vo.setTitle(title);
        vo.setCreateType(type);
        vo.setTenantId(c.getTenantId());
        vo.setCustomerId(c.getCustomerCode());
        vo.setCustomerName(c.getCustomerName());
//        vo.setOfficeId(c.getOfficeId());
//        vo.setOfficeName(c.getOfficeName());
        vo.setOfficeId(c.getCustomerCode());
        vo.setOfficeName(c.getCustomerName());
        vo.setDataYear(year);
        vo.setDataQuarter(quarter);
        vo.setDataMonth(month);
        vo.setDataWeek(week);
        vo.setStartDate(startDate);
        vo.setEndDate(endDate);

        JSONArray dicArray = new JSONArray();
        for (Map.Entry<String, Object> entry : targetmap.entrySet()) {
            JSONObject json = new JSONObject();
            String key = entry.getKey();
            RepPrinterTargetDicVO dic = (RepPrinterTargetDicVO) entry.getValue();
            json.put("statistics_type", key);
            json.put("target_value", dic.getTargetValue());
            json.put("target_show", dic.getTargetShow());
            json.put("start_date", DateUtils.formatDate(dic.getStartDate()));
            json.put("end_date", DateUtils.formatDate(dic.getEndDate()));
            dicArray.add(json);
        }
        vo.setTargetValues(dicArray.toJSONString());

        PrinterReportDTO printerReportDTO = setPrinterReportDTO(c, vo);
        if (CollectionUtils.isEmpty(printerReportDTO.getDeviceIdList())) {
            logger.info("未找到此客户的设备列表,结束报告创建。");
            return null;
        }
        String reportParam = JSON.toJSONString(printerReportDTO);
        logger.info("method : createPrinterUserReport-printerEfficiencyService.getReportBarDataByDevice ,  param : "+reportParam);
        //打印机效率（开动率/布图率打印效率）
        Map<String, Object> efficiencyData = printerEfficiencyService.getReportBarDataByDevice(printerReportDTO, c.getTenantId(),printerDeviceRelationData,targetmap);
        logger.info("method : createPrinterUserReport-printerEfficiencyService.getReportBarDataByDevice ,  param : "+reportParam+" res="+efficiencyData);
        String avgKdlValue = "0", minKdlDevice="", ratelist = "";
        String avgBtlValue = "0", minBtlDevice="";
        Float minKdlValue = 0F;
        Float minBtlValue = 0F;
        Double dyhsTargetValue = null;
        Double btlTargetValue = null;
        String avgStarValue = ""; //星级评价均分
        String top3DeviceStar = ""; //星级评价最高分3台设备
        String end3DeviceStar = ""; //星级评价最低分3台设备
        if( !efficiencyData.isEmpty() ){
            avgStarValue =  efficiencyData.get("avgstarValue")==null?"":efficiencyData.get("avgstarValue").toString();//星级评价平均分
            top3DeviceStar =  efficiencyData.get("top3DeviceStar")==null?"":efficiencyData.get("top3DeviceStar").toString();
            end3DeviceStar =  efficiencyData.get("end3DeviceStar")==null?"":efficiencyData.get("end3DeviceStar").toString();
            avgKdlValue = efficiencyData.get("avgKdlValue")==null?"":efficiencyData.get("avgKdlValue").toString();//平均开动率
            minKdlValue = efficiencyData.get("minKdlValue")==null?0F:Float.parseFloat(efficiencyData.get("minKdlValue").toString());//设备最小开动率
            minKdlDevice = (String) efficiencyData.get("minKdlDevice");//最小开动率设备
            avgBtlValue = efficiencyData.get("avgBtlValue")==null?"0":efficiencyData.get("avgBtlValue").toString();//平均空间利用率
            minBtlValue = efficiencyData.get("minBtlValue")==null?0F:Float.parseFloat(efficiencyData.get("minBtlValue").toString());//设备最小空间利用率
            minBtlDevice = (String) efficiencyData.get("minBtlDevice");//最小空间利用率设备
            ratelist = (String) efficiencyData.get("ratelist");//平均打印效率

            RepPrinterTargetDicVO SBKDL = (RepPrinterTargetDicVO) targetmap.get("SBKDL");
            RepPrinterTargetDicVO BTL = (RepPrinterTargetDicVO) targetmap.get("BTL");

            try{
                dyhsTargetValue = Double.parseDouble( df.format(SBKDL.getTargetValue()) );
            }catch(Exception e){}
            try{
                btlTargetValue = Double.parseDouble( df.format(BTL.getTargetValue()) );
            }catch(Exception e){}

            String[] xListDYHS = (String[])efficiencyData.get("xListDYHS");
            if( xListDYHS!=null && xListDYHS.length>0 ){
                List<String> dyhsList = (List<String>)efficiencyData.get("dyhsList");//开动率

                //设备开动率统计
                Map<String, Object> dyhsData = new HashMap<String, Object>();
                dyhsData.put("xList", xListDYHS);
                dyhsData.put("dyhsList", dyhsList);
                dyhsData.put("targetValue", dyhsTargetValue);
                vo.setData1( JSONObject.toJSONString(dyhsData) );
            }

            List<String> xListDYXL = (List<String>)efficiencyData.get("xListDYXL");
            if( xListDYXL!=null && xListDYXL.size()>0 ){
                List<String> dyxlList = (List<String>)efficiencyData.get("dyxlList");//打印效率
                //打印效率
                Map<String, Object> dyxlData = new HashMap<String, Object>();
                dyxlData.put("xList", xListDYXL);
                dyxlData.put("dyxlList", dyxlList);
                dyxlData.put("targetValue", efficiencyData.get("dyxltargetValue"));
                vo.setData3( JSONObject.toJSONString(dyxlData) );
            }


            String[] xListBTL = (String[])efficiencyData.get("xListBTL");
            if( xListBTL!=null && xListBTL.length>0 ){
                List<String> btlList = (List<String>)efficiencyData.get("btlList");//布图率
                //布图率
                Map<String, Object> btlData = new HashMap<String, Object>();
                btlData.put("xList", xListBTL);
                btlData.put("btlList", btlList);
                btlData.put("targetValue", btlTargetValue);
                vo.setData2( JSONObject.toJSONString(btlData) );
            }

        }

        logger.info("method : createPrinterUserReport-repPrinterService.sumDeviceProductWasteData ,  param : "+reportParam);
        Map<String, Object> deviceProductWasteData = repPrinterService.sumDeviceProductWasteData(printerReportDTO, c.getTenantId(),printerDeviceRelationData);
        logger.info("method : createPrinterUserReport-repPrinterService.sumDeviceProductWasteData ,  param : "+reportParam+" res="+deviceProductWasteData);
        String avgFpl = "0", avgDevFpl = "0", top3DevfplDevices = "";
        Double valueMaker = null;
        if( !deviceProductWasteData.isEmpty() ){
            String[] categories4 = (String[]) deviceProductWasteData.get("deviceArray");
            Object[] clArray  = (Object[]) deviceProductWasteData.get("clArray");
            Object[] hglArray  = (Object[]) deviceProductWasteData.get("hglArray");
            Object[] fplArray  = (Object[]) deviceProductWasteData.get("fplArray");
            Object[] devfplArray  = (Object[]) deviceProductWasteData.get("devfplArray");

            avgFpl = (String) deviceProductWasteData.get("avgFpl");
            avgDevFpl = (String) deviceProductWasteData.get("avgDevFpl");
            top3DevfplDevices = (String) deviceProductWasteData.get("top3DevfplDevices");

            RepPrinterTargetDicVO FPL = (RepPrinterTargetDicVO) targetmap.get("FPL");
            try{
                //valueMaker = Double.parseDouble( df.format(FPL.getTargetValue()) );
                valueMaker = FPL.getTargetValue().doubleValue();
            }catch(Exception e){}

            //合格率
            Map<String, Object> jsonData = new HashMap<String, Object>();
            jsonData.put("xArray", categories4);
            jsonData.put("clArray", clArray);
            jsonData.put("hglArray", hglArray);
            jsonData.put("fplArray", fplArray);
            jsonData.put("devfplArray", devfplArray);
            jsonData.put("targetValue", valueMaker);
            vo.setData4( JSONObject.toJSONString(jsonData) );
        }

        logger.info("method : createPrinterUserReport-repPrinterService.sumWastereasonData ,  param : "+reportParam);
        Map<String, Object> wastereasonData =repPrinterService.sumWastereasonData(printerReportDTO, c.getTenantId(),printerDeviceRelationData);
        logger.info("method : createPrinterUserReport-repPrinterService.sumWastereasonData ,  param : "+reportParam+" res="+wastereasonData);
        String top3WastereasonTypes = "";
        if( !wastereasonData.isEmpty() ){
            String[] categories5 = (String[]) wastereasonData.get("types");
            Object[] datas5 = (String[]) wastereasonData.get("proportions");
            top3WastereasonTypes = (String) wastereasonData.get("top3WastereasonTypes");
            //废品原因统计
            Map<String, Object> jsonData = new HashMap<String, Object>();
            jsonData.put("xArray", categories5);
            jsonData.put("array", datas5);
            vo.setData5( JSONObject.toJSONString(jsonData) );
        }


        logger.info("method : createPrinterUserReport-repPrinterService.sumPrinterSandPassrateData ,  param : "+reportParam);
        RepPrinterTargetDicVO SXMDHGL = (RepPrinterTargetDicVO) targetmap.get("SXMDHGL");
        Double sandPassrateTargetValue = null;
        try{
            sandPassrateTargetValue = SXMDHGL.getTargetValue().doubleValue();
        }catch(Exception e){}
        printerReportDTO.setSandPassrateTargetValue(sandPassrateTargetValue);
        Map<String, Object> sxmdData =repPrinterService.sumPrinterSandPassrateData(printerReportDTO, c.getTenantId(),printerDeviceRelationData);
        logger.info("method : createPrinterUserReport-repPrinterService.sumPrinterSandPassrateData ,  param : "+reportParam+" res="+sxmdData);
        String avgSxmd = "0", top3MdbhgDevices = "";
        if( !sxmdData.isEmpty() ){
            String[] categories6 = (String[]) sxmdData.get("deviceArray");
            Object[] datas6 = (Object[]) sxmdData.get("rateArray");
            Object[] aveArray = (Object[]) sxmdData.get("aveArray");
            avgSxmd = (String) sxmdData.get("avgSxmd");
            top3MdbhgDevices = (String) sxmdData.get("top3MdbhgDevices");
            //砂型密度
            Map<String, Object> jsonData = new HashMap<String, Object>();
            jsonData.put("xArray", categories6);
            jsonData.put("rateArray", datas6);
            jsonData.put("aveArray", aveArray);
            jsonData.put("targetValue", sandPassrateTargetValue);
            vo.setData6( JSONObject.toJSONString(jsonData));
        }

        //打印机树脂加入比/新旧砂加入占比/漏砂量统计
        logger.info("method : createPrinterUserReport-repPrinterSandPercentService.getRepPrinterSandPercentBarData ,  param : "+reportParam);
        Map<String, Object> sandPercentData = repPrinterSandPercentService.getRepPrinterSandPercentBarData(printerReportDTO, c.getTenantId(),printerDeviceRelationData);
        logger.info("method : createPrinterUserReport-repPrinterSandPercentService.getRepPrinterSandPercentBarData ,  param : "+reportParam+" res="+sandPercentData);

        Double xjszbTargetValue = null;
        Double lslTargetValue = null;
        if( !sandPercentData.isEmpty() ){
            RepPrinterTargetDicVO DYJSZJRB = (RepPrinterTargetDicVO) targetmap.get("DYJSZJRB");
            RepPrinterTargetDicVO XJSJRZB = (RepPrinterTargetDicVO) targetmap.get("XJSJRZB");
            RepPrinterTargetDicVO LSL = (RepPrinterTargetDicVO) targetmap.get("LSL");

            Double dyjszjrbTargetValue = null;
            try{
                dyjszjrbTargetValue = DYJSZJRB.getTargetValue().doubleValue();
            }catch(Exception e){}
            try{
                xjszbTargetValue = XJSJRZB.getTargetValue().doubleValue();
            }catch(Exception e){}
            try{
                lslTargetValue = LSL.getTargetValue().doubleValue();
            }catch(Exception e){}

            String[] sandPercentCategories = (String[])sandPercentData.get("xListSXMD");
            if( sandPercentCategories!=null && sandPercentCategories.length>0 ){
                List<String> sxmdList = (List<String>)sandPercentData.get("sxmdList");//打印机树脂加入比
                List<String> xjszbList = (List<String>)sandPercentData.get("xjszbList");//新旧砂加入占比

                //打印机树脂加入比
                Map<String, Object> jsonData7 = new HashMap<String, Object>();
                jsonData7.put("xArray", sandPercentData.get("xListSXMD"));
                jsonData7.put("sxmdArray", sxmdList);
                jsonData7.put("targetValue", dyjszjrbTargetValue * 100);
                vo.setData7( JSONObject.toJSONString(jsonData7) );

                //新旧砂加入占比
                Map<String, Object> jsonData8 = new HashMap<String, Object>();
                jsonData8.put("xArray", sandPercentData.get("xListXJSZB"));
                jsonData8.put("xjszbArray", xjszbList);
                jsonData8.put("targetValue", xjszbTargetValue);
                vo.setData8( JSONObject.toJSONString(jsonData8) );

                //漏砂量
                Map<String, Object> jsonData9 = new HashMap<String, Object>();
                jsonData9.put("xArray", sandPercentData.get("xListLSL"));
                jsonData9.put("slylArray", sandPercentData.get("slylArray"));
                jsonData9.put("hszlArray", sandPercentData.get("hszlArray"));
                jsonData9.put("wsyslArray", sandPercentData.get("wsyslArray"));
                jsonData9.put("avgArray", sandPercentData.get("avgArray"));
                jsonData9.put("targetValue", lslTargetValue);
                vo.setData9( JSONObject.toJSONString(jsonData9) );
            }
        }

        //喷头寿命
        logger.info("method : createPrinterUserReport-getRepPrinterSprinklerBarData ,  param : "+reportParam);
        //暂时预留
        vo.setData10("");
        vo.setValue8("");

        //产能利用率 = 产量 / 产能，其中，产能=产能目标值 * 运维报告所取天数；产量为导入废品量时导入的总打印量
        List<ChanNengLiYongLvDto> chanNengLiYongLvData =repPrinterService.chanNengLiYongLv(printerReportDTO,
                c.getTenantId(),printerDeviceRelationData,targetmap);
        if (chanNengLiYongLvData != null &&  !chanNengLiYongLvData.isEmpty()) {
            JSONObject data11 = new JSONObject();
            data11.put("xArray",chanNengLiYongLvData.stream().map(ChanNengLiYongLvDto::getDeviceId).collect(Collectors.toList()));
            data11.put("chanLiang",chanNengLiYongLvData.stream().map(x->df.format(x.getChanLiang())).collect(Collectors.toList()));
            data11.put("chanNeng",chanNengLiYongLvData.stream().map(x->df.format(x.getChanNeng())).collect(Collectors.toList()));
            data11.put("liYongLv",chanNengLiYongLvData.stream().map(x->df.format(x.getLiYongLv()*100)).collect(Collectors.toList()));
            data11.put("avg",chanNengLiYongLvData.stream().map(x->df.format(x.getAvg()*100)).collect(Collectors.toList()));
            vo.setData11(data11.toJSONString());
            Double avg = chanNengLiYongLvData.stream().mapToDouble(ChanNengLiYongLvDto::getLiYongLv).average().orElse(0d);
            vo.setRate12(avg.floatValue());
            int size = chanNengLiYongLvData.size();
            String top3Device = chanNengLiYongLvData.stream().sorted(Comparator.comparing(ChanNengLiYongLvDto::getLiYongLv))
                    .map(ChanNengLiYongLvDto::getDeviceId).limit(size >= 3?3:size).collect(Collectors.joining("、"));
            String top3LiYongLv = chanNengLiYongLvData.stream().sorted(Comparator.comparing(ChanNengLiYongLvDto::getLiYongLv)).map(x->{
                Double d = x.getLiYongLv() * 100;
                return df.format(d)+"%";
            }).limit(size >= 3?3:size).collect(Collectors.joining("、"));
            vo.setValue12("其中产能利用率最低的三台设备为<span class='xh'> "+top3Device+
                    " </span>；产能利用率分别为<span class='xh'> "+top3LiYongLv+" </span>；");
        }else{
            vo.setValue12("其中产能利用率最低的三台设备为____、____、____；产能利用率分别为____、____、____；");
        }


        //平均打印高度 = = 打印层厚 * 最大打印层数. 按每日打印的批次计算当日平均值，再计算运维报告所取天数计算平均值
        List<PinJunDaYinGaoDuDto> pinJunDaYinGaoDu = repPrinterSandPercentService.pinJunDaYinGaoDu(printerReportDTO, c.getTenantId(),printerDeviceRelationData);
        if (pinJunDaYinGaoDu != null &&  !pinJunDaYinGaoDu.isEmpty()) {
            PinJunDaYinGaoDuDto last = pinJunDaYinGaoDu.get(pinJunDaYinGaoDu.size() - 1);
            RepPrinterTargetDicVO avghight = (RepPrinterTargetDicVO) targetmap.get("AVGHIGHT");
            String[] target = new String[pinJunDaYinGaoDu.size()+1];
            Arrays.fill(target,df.format(avghight.getTargetValue()));
            Double avg = pinJunDaYinGaoDu.stream().mapToDouble(PinJunDaYinGaoDuDto::getHeight).average().orElse(0d);
            PinJunDaYinGaoDuDto avgObj = new PinJunDaYinGaoDuDto("平均值",avg,0d,0d);
            pinJunDaYinGaoDu.add(avgObj);
            JSONObject data12 = new JSONObject();
            data12.put("xArray",pinJunDaYinGaoDu.stream().map(PinJunDaYinGaoDuDto::getDeviceId).collect(Collectors.toList()));
            data12.put("height",pinJunDaYinGaoDu.stream().map(x->df.format(x.getHeight())).collect(Collectors.toList()));
            data12.put("target", target);
            vo.setData12(data12.toJSONString());
            vo.setRate13(avg.floatValue());
            vo.setValue13("<span class='xh'> " +last.getDeviceId()+" </span>的平均打印高度最低，为<span class='xh'>  "
                    +last.getHeight()+" </span> cm；");
        }else{
            vo.setValue13("________的平均打印高度最低，为______cm；");
        }

        //砂芯强度合格率统计报表,与砂芯密度合格率报表导入模板一致（换一下字段名称即可），该报表为手动导入数据报表
        logger.info("method : createPrinterUserReport-repPrinterService.sumPrinterSandPassrateQiangDuData ,  param : "+reportParam);
        RepPrinterTargetDicVO SXQDHGL = (RepPrinterTargetDicVO) targetmap.get("SXQDHGL");
        Double sandPassrateQiangDuTargetValue = null;
        try{
            sandPassrateQiangDuTargetValue = SXQDHGL.getTargetValue().doubleValue();
        }catch(Exception e){}
        Map<String, Object> sxmdData1 =repPrinterService.sumPrinterSandPassrateData1(printerReportDTO, c.getTenantId(),printerDeviceRelationData);
        logger.info("method : createPrinterUserReport-repPrinterService.sumPrinterSandPassrateData ,  param : "+reportParam+" res="+sxmdData1);
        String avgSxmd1 = "0", top3MdbhgDevices1 = "";
        if( !sxmdData1.isEmpty() ){
            String[] categories6 = (String[]) sxmdData1.get("deviceArray");
            Object[] datas6 = (Object[]) sxmdData1.get("rateArray");
            Object[] aveArray = (Object[]) sxmdData1.get("aveArray");
            avgSxmd1 = (String) sxmdData1.get("avgSxmd");
            top3MdbhgDevices1 = (String) sxmdData1.get("top3MdbhgDevices");
            //砂型强度
            Map<String, Object> jsonData = new HashMap<String, Object>();
            jsonData.put("xArray", categories6);
            jsonData.put("rateArray", datas6);
            jsonData.put("aveArray", aveArray);
            jsonData.put("targetValue", sandPassrateQiangDuTargetValue);
            vo.setData13( JSONObject.toJSONString(jsonData) );
        }

        //维修费用统计,统计运维报告所选时间区间内，每台设备维修单中维修换件的总价
//        vo.setData14("");
//        vo.setRate14(0d);
        vo.setValue14("");

        //固化剂单耗 = 打印一批次最大固化剂耗量（取工况字段）/ （最大打印体积*密度1.5）,单位：kg / T
        List<GuHuaJiDanHaoDto> guHuaJiDanHao =repPrinterSandPercentService.guHuaJiDanHao(printerReportDTO, c.getTenantId(),printerDeviceRelationData);
        if (guHuaJiDanHao != null &&  !guHuaJiDanHao.isEmpty()) {
            GuHuaJiDanHaoDto first = guHuaJiDanHao.get(0);
            RepPrinterTargetDicVO ghjdh = (RepPrinterTargetDicVO) targetmap.get("GHJDH");
            String[] target = new String[guHuaJiDanHao.size()+1];
            Arrays.fill(target, df.format(ghjdh.getTargetValue()));
            Double avg = guHuaJiDanHao.stream().mapToDouble(GuHuaJiDanHaoDto::getActivatorCost).average().orElse(0d);
            GuHuaJiDanHaoDto avgObj = new GuHuaJiDanHaoDto("平均值",avg,0d,0d);
            guHuaJiDanHao.add(avgObj);
            JSONObject data15 = new JSONObject();
            data15.put("xArray",guHuaJiDanHao.stream().map(GuHuaJiDanHaoDto::getDeviceId).collect(Collectors.toList()));
            data15.put("activatorCost",guHuaJiDanHao.stream().map(x->df.format(x.getActivatorCost())).collect(Collectors.toList()));
            data15.put("target", target);
            vo.setData15(data15.toJSONString());
            vo.setRate15(avg.floatValue());
            vo.setValue15("固化剂单耗最大为<span class='xh'>" +first.getActivatorCost()+
                    "</span>kg/T砂芯，固化剂单耗最大的设备是<span class='xh'>" +first.getDeviceId()+"</span>；");
        }else{
            vo.setValue15("固化剂单耗最大为_______kg/T砂芯，固化剂单耗最大的设备是_________；");
        }

        //换线时间
        String[] XTimeArray = null;
        String[] YTimeArray = null;
        ArrayList devList = new ArrayList();
        List<String> strCurLists = printerReportDTO.getDeviceIdList();

        int valid_dayNum =0; //有效的设备个数
        for(int i=0;i<strCurLists.size();i++)
        {
            String curdevid = strCurLists.get(i).toString();
            List<RepPrinterSandPercentVO> PrintListObj = sandDAO.getPrintJobTaskinfo(curdevid,startDate+" "+"00:00:00",endDate+" "+"23:59:59");
            if(PrintListObj.size()> 0)
            {
                valid_dayNum++;
                devList.add(curdevid);
            }
        }
        XTimeArray = new String[valid_dayNum+1];
        YTimeArray= new String[valid_dayNum+1];
        List<RepMttDataVO> changetimelistAlls = new ArrayList<>();
        Double sumchangeTime =0.0;
        for(int i=0;i< devList.size();i++)
        {
            String cur_devid = devList.get(i).toString();
//			if(cur_devid.equals("2500-2"))
//			{
//				cur_devid =cur_devid;
//			}
            List<RepPrinterSandPercentVO> PrintJobListDataObj = sandDAO.getPrintJobTaskinfo(cur_devid,startDate+" "+"00:00:00",endDate+" "+"23:59:59");
            List datelist = new ArrayList();
            for(int j= PrintJobListDataObj.size()-1; j >=0 ;j--)
            {
                String sy = PrintJobListDataObj.get(j).getDataYear().toString();
                String sm = PrintJobListDataObj.get(j).getDataMonth().toString();
                if(Integer.parseInt(sm)<10)
                    sm ="0"+sm;
                String sd = PrintJobListDataObj.get(j).getDataDay().toString();
                if(Integer.parseInt(sd)<10)
                    sd ="0"+sd;
                String cur_day = sy+"-"+ sm+"-"+ sd;

                if (!datelist.contains(cur_day))
                {
                    datelist.add(cur_day);
                }
            }
            if(PrintJobListDataObj.size()> 0)
            {
                int valid_days =0;
                Double totalchangeTime =0.0;
//				for(int j= PrintJobListDataObj.size()-1; j >=0 ;j--)
                for(int j= datelist.size()-1; j >=0 ;j--)
                {
//					String sy = PrintJobListDataObj.get(j).getDataYear().toString();
//					String sm = PrintJobListDataObj.get(j).getDataMonth().toString();
//					if(Integer.parseInt(sm)<10)
//						sm ="0"+sm;
//					String sd = PrintJobListDataObj.get(j).getDataDay().toString();
//					if(Integer.parseInt(sm)<10)
//						sd ="0"+sd;
//					String cur_day = sy+"-"+ sm+"-"+ sd;
                    String cur_day = datelist.get(j).toString();  //取出所有不重复的日期
                    String cur_day_changeTime="";
                    double cur_changeTime=0d;
                    boolean curday_isnotExist = false;
                    int curday_hasCountNum =0; //记录当日的记录数目
                    LocalDateTime ksT1 =  LocalDateTime.now(); //记录当日的开始时间（任务结束时间）
                    int cur_index=0; //记录当前索引

                    for(int k= PrintJobListDataObj.size()-1; k >=0 ;k--)
                    {
                        //任务结束时间
                        String printEndTime = PrintJobListDataObj.get(k).getHmiEndtime();
                        LocalDateTime t2 = LocalDateTime.parse(printEndTime , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                        if(!printEndTime.substring(0,10).equals(cur_day))
                        {
                            continue;  //过滤不是同一天的数据
                        }
                        if(!printEndTime.substring(0,10).equals(cur_day)) //判断当日是否存在打印记录
                        {
                            //当日不存在记录，则当日换线时间为0
                            curday_isnotExist = true;
                        }
                        else
                        {
                            if(printEndTime.substring(0,10).equals(cur_day))
                            {
                                curday_hasCountNum++;
                                ksT1 = t2;  //任务结束时间 -即 计算的起始时间
                                cur_index = k;
                            }
                        }
                    }
                    if(!curday_isnotExist)
                    {
                        cur_changeTime = 0;
                    }
                    if(curday_hasCountNum == 1)//当日记录1条记录,则当日换线时间后推一天计算（最多24H）
                    {
                        LocalDateTime t1 = ksT1; //开始时间
                        if(cur_index == 0) //当月数据不够用，则往后推一个月取该月的最后1条记录
                        {
                            LocalDateTime Starttime1 = LocalDateTime.parse(startDate+" "+"00:00:00"  , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusMonths(1);
                            LocalDateTime Endtime1 = LocalDateTime.parse(endDate +" "+"23:59:59" , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusMonths(1);
                            String startDate1 =  DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(Starttime1);
                            String endDate1 =  DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(Endtime1);
                            List<RepPrinterSandPercentVO> PrintJobListDataObj1 = sandDAO.getPrintJobTaskinfo(cur_devid,startDate1,endDate1);
                            if(PrintJobListDataObj1.size() >0)
                            {
                                int k1 = PrintJobListDataObj1.size() - 1;//当天第一条（唯一）/时间最小的，索引最大
                                String str1 = PrintJobListDataObj1.get(k1).getHmiBegintime();
                                LocalDateTime printStartTime1 = LocalDateTime.parse(str1 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                                DateTimeFormatter dfs = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                                String cur_time_1 = dateDiff(dfs.format(t1),dfs.format(printStartTime1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                                if(Double.valueOf(cur_time_1) > 24 || Double.valueOf(cur_time_1) < 0)
                                    cur_changeTime += 0.0;
                                else
                                    cur_changeTime += Double.valueOf(cur_time_1);
                            }

                        }
                        else
                        {
                            //后1条 开始时间
                            String str1 = PrintJobListDataObj.get(cur_index-1).getHmiBegintime();
                            LocalDateTime t1_1 = LocalDateTime.parse(str1 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                            DateTimeFormatter dfs = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                            String cur_time1 = dateDiff(dfs.format(t1),dfs.format(t1_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                            if(Double.valueOf(cur_time1) > 24 || Double.valueOf(cur_time1) < 0)
                                cur_changeTime += 0.0;
                            else
                                cur_changeTime += Double.valueOf(cur_time1);
                        }
                    }
                    if(curday_hasCountNum ==2)
                    {
                        double changeTime_temp =0.0;
                        //后1条 开始时间

                        String str1 = PrintJobListDataObj.get(cur_index+1).getHmiEndtime();
                        LocalDateTime t1 = LocalDateTime.parse(str1 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//第2条的结束时间  -起算时间点

                        String str2 = PrintJobListDataObj.get(cur_index).getHmiBegintime();
                        LocalDateTime t1_1 = LocalDateTime.parse(str2 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                        DateTimeFormatter dfs = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                        String cur_time1 = dateDiff(dfs.format(t1),dfs.format(t1_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                        if(Double.valueOf(cur_time1) > 24 || Double.valueOf(cur_time1) < 0)
                            changeTime_temp += 0.0;
                        else
                            changeTime_temp += Double.valueOf(cur_time1);

                        String str3 = PrintJobListDataObj.get(cur_index).getHmiEndtime();
                        LocalDateTime t2 = LocalDateTime.parse(str3 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//换算成第2条的开始时间 -起算时间点

                        if(cur_index == 0) //当月数据不够用，则往后推一个月取该月的最后1条记录
                        {
                            LocalDateTime Starttime1 = LocalDateTime.parse(startDate+" "+"00:00:00"  , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusMonths(1);
                            LocalDateTime Endtime1 = LocalDateTime.parse(endDate +" "+"23:59:59" , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusMonths(1);
                            String startDate1 =  DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(Starttime1);
                            String endDate1 =  DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(Endtime1);

                            List<RepPrinterSandPercentVO> PrintJobListDataObj1 = sandDAO.getPrintJobTaskinfo(cur_devid,startDate1,endDate1);
                            if(PrintJobListDataObj1.size() >0)
                            {
                                int k1 =PrintJobListDataObj1.size()-1;//当天第一条（唯一）/时间最小的，索引最大
                                String stT1 = PrintJobListDataObj1.get(k1).getHmiBegintime();
                                LocalDateTime printStartTime1 = LocalDateTime.parse(stT1 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                                String cur_time_1 = dateDiff(dfs.format(t2),dfs.format(printStartTime1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                                if(Double.valueOf(cur_time_1) > 24 || Double.valueOf(cur_time_1) < 0)
                                    changeTime_temp += 0.0;
                                else
                                    changeTime_temp += Double.valueOf(cur_time_1);
                            }

                        }
                        else
                        {
                            //LocalDateTime t2_1 = pageInfo.getList().get(cur_index-1).getPrintStartTime();//往后推1条的开始时间
                            String stT1 = PrintJobListDataObj.get(cur_index-1).getHmiBegintime();
                            LocalDateTime t2_1 =LocalDateTime.parse(stT1 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                            String cur_time2 = dateDiff(dfs.format(t2),dfs.format(t2_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                            if(Double.valueOf(cur_time2) > 24 || Double.valueOf(cur_time2) < 0 )
                                changeTime_temp += 0.0;
                            else
                                changeTime_temp += Double.valueOf(cur_time2);
                        }
                        changeTime_temp = changeTime_temp / 2;
                        cur_changeTime += changeTime_temp;
                    }
                    if(curday_hasCountNum==3)
                    {
                        double changeTime_temp =0.0;
                        //1  后1条 开始时间

                        String stT1 = PrintJobListDataObj.get(cur_index+2).getHmiEndtime(); //第3条的结束时间 -起算时间点
                        LocalDateTime t1 = LocalDateTime.parse(stT1 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//第3条的结束时间 -起算时间点

                        String strT2 =PrintJobListDataObj.get(cur_index+1).getHmiBegintime();
                        LocalDateTime t1_1 = LocalDateTime.parse(strT2 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));;
                        DateTimeFormatter dfs = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

                        String cur_time1 = dateDiff(dfs.format(t1),dfs.format(t1_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h

                        if(Double.valueOf(cur_time1) > 24 || Double.valueOf(cur_time1) < 0)
                            changeTime_temp += 0.0;
                        else
                            changeTime_temp += Double.valueOf(cur_time1);


                        String stT11 = PrintJobListDataObj.get(cur_index+1).getHmiEndtime(); //第2条的开始时间
                        LocalDateTime t2 = LocalDateTime.parse(stT11 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//第2条的开始时间

                        String strT22 = PrintJobListDataObj.get(cur_index).getHmiBegintime();
                        LocalDateTime t2_1 = LocalDateTime.parse(strT22 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                        String cur_time2 = dateDiff(dfs.format(t2),dfs.format(t2_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                        if(Double.valueOf(cur_time2) > 24 || Double.valueOf(cur_time2) < 0)
                            changeTime_temp += 0.0;
                        else
                            changeTime_temp += Double.valueOf(cur_time2);

                        String stT3 = PrintJobListDataObj.get(cur_index).getHmiEndtime();
                        LocalDateTime t3 = LocalDateTime.parse(stT3 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//第2条的开始时间
                        if(cur_index == 0) //当月数据不够用，则往后推一个月取该月的最后1条记录
                        {
                            LocalDateTime Starttime1 = LocalDateTime.parse(startDate+" "+"00:00:00"  , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusMonths(1);
                            LocalDateTime Endtime1 = LocalDateTime.parse(endDate +" "+"23:59:59" , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusMonths(1);
                            String startDate1 =  DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(Starttime1);
                            String endDate1 =  DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(Endtime1);

                            List<RepPrinterSandPercentVO> PrintJobListDataObj1 = sandDAO.getPrintJobTaskinfo(cur_devid,startDate1,endDate1);


                            if(PrintJobListDataObj1.size() >0)
                            {
                                int k1 =PrintJobListDataObj1.size()-1;//当天第一条（唯一）/时间最小的，索引最大
                                //LocalDateTime printStartTime1 = pageInfo1.getList().get(k1).getPrintStartTime();
                                String stTemp = PrintJobListDataObj1.get(k1).getHmiBegintime(); //取下一月的第一条记录 修改 PrintJobListDataObj
                                LocalDateTime printStartTime1 = LocalDateTime.parse(stTemp , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                                String cur_time_1 = dateDiff(dfs.format(t3),dfs.format(printStartTime1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                                if(Double.valueOf(cur_time_1) > 24 || Double.valueOf(cur_time_1) < 0)
                                    changeTime_temp += 0.0;
                                else
                                    changeTime_temp += Double.valueOf(cur_time_1);
                            }

                        }
                        else
                        {
                            String stTemp = PrintJobListDataObj.get(cur_index-1).getHmiBegintime();
                            LocalDateTime t3_1 = LocalDateTime.parse(stTemp , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                            String cur_time3 = dateDiff(dfs.format(t3),dfs.format(t3_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                            if(Double.valueOf(cur_time3) > 24 || Double.valueOf(cur_time3) < 0)
                                changeTime_temp += 0.0;
                            else
                                changeTime_temp += Double.valueOf(cur_time3);
                        }

                        changeTime_temp = changeTime_temp /3;
                        cur_changeTime +=changeTime_temp;
                    }

                    if(curday_hasCountNum==4)
                    {
                        double changeTime_temp =0.0;
                        //1  后1条 开始时间
                        String stTemp1 = PrintJobListDataObj.get(cur_index+3).getHmiEndtime();
                        LocalDateTime t1 = LocalDateTime.parse(stTemp1 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//第4条的结束时间 -起算时间点

                        String stTemp_1 = PrintJobListDataObj.get(cur_index+2).getHmiEndtime();
                        LocalDateTime t1_1 = LocalDateTime.parse(stTemp_1 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//第4条的结束时间 -起算时间点

                        DateTimeFormatter dfs = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                        String cur_time1 = dateDiff(dfs.format(t1),dfs.format(t1_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h

                        if(Double.valueOf(cur_time1) > 24 || Double.valueOf(cur_time1) < 0)
                            changeTime_temp += 0.0;
                        else
                            changeTime_temp += Double.valueOf(cur_time1);

                        String stTemp2 = PrintJobListDataObj.get(cur_index+2).getHmiEndtime();
                        LocalDateTime t2 = LocalDateTime.parse(stTemp2 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//第4条的结束时间 -起算时间点

                        String stTemp_2 = PrintJobListDataObj.get(cur_index+1).getHmiBegintime();
                        LocalDateTime t2_1 = LocalDateTime.parse(stTemp_2 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));


                        String cur_time2 = dateDiff(dfs.format(t2),dfs.format(t2_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                        if(Double.valueOf(cur_time2) > 24 || Double.valueOf(cur_time2) < 0)
                            changeTime_temp += 0.0;
                        else
                            changeTime_temp += Double.valueOf(cur_time2);


                        String stTemp3 = PrintJobListDataObj.get(cur_index+1).getHmiEndtime();
                        LocalDateTime t3 = LocalDateTime.parse(stTemp3 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));//第4条的结束时间 -起算时间点

                        String stTemp_3 = PrintJobListDataObj.get(cur_index).getHmiBegintime();
                        LocalDateTime t3_1 = LocalDateTime.parse(stTemp_3 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                        String cur_time3 = dateDiff(dfs.format(t3),dfs.format(t3_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                        if(Double.valueOf(cur_time3) > 24 || Double.valueOf(cur_time3) < 0)
                            changeTime_temp += 0.0;
                        else
                            changeTime_temp += Double.valueOf(cur_time3);

                        String stTemp4 = PrintJobListDataObj.get(cur_index).getHmiEndtime();
                        LocalDateTime t4 = LocalDateTime.parse(stTemp4 , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                        if(cur_index == 0) //当月数据不够用，则往后推一个月取该月的最后1条记录
                        {
                            LocalDateTime Starttime1 = LocalDateTime.parse(startDate+" "+"00:00:00"  , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusMonths(1);
                            LocalDateTime Endtime1 = LocalDateTime.parse(endDate +" "+"23:59:59" , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).plusMonths(1);
                            String startDate1 =  DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(Starttime1);
                            String endDate1 =  DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(Endtime1);
                            List<RepPrinterSandPercentVO> PrintJobListDataObj1 = sandDAO.getPrintJobTaskinfo(cur_devid,startDate1,endDate1);

                            if(PrintJobListDataObj1.size() >0)
                            {
                                int k1 =PrintJobListDataObj1.size()-1;//当天第一条（唯一）/时间最小的，索引最大
                                //LocalDateTime printStartTime1 = pageInfo1.getList().get(k1).getPrintStartTime();

                                String stTemp = PrintJobListDataObj1.get(k1).getHmiBegintime(); //修改 PrintJobListDataObj
                                LocalDateTime printStartTime1 = LocalDateTime.parse(stTemp , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                                String cur_time_1 = dateDiff(dfs.format(t4),dfs.format(printStartTime1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                                if(Double.valueOf(cur_time_1) > 24 || Double.valueOf(cur_time_1) < 0)
                                    changeTime_temp += 0.0;
                                else
                                    changeTime_temp += Double.valueOf(cur_time_1);
                            }

                        }
                        else
                        {
                            String stTemp = PrintJobListDataObj.get(cur_index-1).getHmiBegintime();
                            LocalDateTime t4_1 = LocalDateTime.parse(stTemp , DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                            String cur_time4 = dateDiff(dfs.format(t4),dfs.format(t4_1),"yyyy-MM-dd HH:mm:ss"); //换线时间h
                            if(Double.valueOf(cur_time4) > 24 || Double.valueOf(cur_time4) < 0)
                                changeTime_temp += 0.0;
                            else
                                changeTime_temp += Double.valueOf(cur_time4);
                        }
                        changeTime_temp = changeTime_temp /4;
                        cur_changeTime += changeTime_temp;
                    }

                    //当日的换线时间
                    cur_day_changeTime = String.valueOf(cur_changeTime);
                    String strYArry = String.valueOf(String.format("%.2f",Double.valueOf(cur_day_changeTime)));
                    if(Double.valueOf(cur_day_changeTime) > 0)
                    {
                        totalchangeTime += Double.valueOf(cur_day_changeTime);
                        valid_days++;
                    }

                }

                //对当前设备进行求均值
                if(valid_days != 0)
                    totalchangeTime = totalchangeTime / valid_days;
                else
                    totalchangeTime = 0.0;

                //获取用户设备编码[含设备位置]
                String userDeviceId = printerDeviceRelationData.get(cur_devid)==null?cur_devid:printerDeviceRelationData.get(cur_devid);
//				XTimeArray[i] =userDeviceId;//cur_devid;
//				YTimeArray[i] = String.format("%.2f",totalchangeTime);
                sumchangeTime += totalchangeTime;

                RepMttDataVO mtt = new RepMttDataVO();
                mtt.setName(userDeviceId);
                mtt.setValue11(Double.valueOf(String.format("%.2f",totalchangeTime)));
                changetimelistAlls.add(mtt);
            }
        }
        if(changetimelistAlls.size() > 0)
        {
            //从大到小排序
            changetimelistAlls = changetimelistAlls.stream().sorted((Comparator.comparingDouble(RepMttDataVO::getValue11).reversed())).collect(Collectors.toList());
            String maxtime_device = changetimelistAlls.get(0).getName();
            String maxtime_device_time = String.valueOf(changetimelistAlls.get(0).getValue11());
            for(int i=0;i<changetimelistAlls.size();i++)
            {
                XTimeArray[i] = changetimelistAlls.get(i).getName();
                YTimeArray[i] = String.valueOf(changetimelistAlls.get(i).getValue11());
            }
            sumchangeTime = sumchangeTime /devList.size();
            XTimeArray[devList.size()] = "平均值"; //添加列
            YTimeArray[devList.size()] = String.valueOf(String.format("%.2f",sumchangeTime));  //添加列

            Double ChangeJobTimeTarget = null;
            RepPrinterTargetDicVO ChangeTime = (RepPrinterTargetDicVO) targetmap.get("ChangeJobTimeTarget");
            try{
                ChangeJobTimeTarget = Double.parseDouble( df.format(ChangeTime.getTargetValue()) );
            }catch(Exception e){}

            Map<String, Object> changtimeData = new HashMap<String, Object>();
            changtimeData.put("xList", XTimeArray);
            changtimeData.put("yList", YTimeArray);
//		if(ChangeJobTimeTarget == 4)
//		{
//			changtimeData.put("targetValue", ChangeJobTimeTarget);//ChangeJobTimeTarget
//		}
//		else
//			changtimeData.put("targetValue", "4");//ChangeJobTimeTarget
            changtimeData.put("targetValue", "4");//ChangeJobTimeTarget
            vo.setData16(JSONObject.toJSONString(changtimeData));
            vo.setValue16(String.valueOf(String.format("%.2f",sumchangeTime)));//平均换线时长
            vo.setValue17(maxtime_device);//换线时长最高的设备
            vo.setValue18(maxtime_device_time);//换线时长最高的设备的时长
        }


        //---------MTBF MTTR MTTF指标------------------
//        vo.setValue24("所有设备平均星级评价得分为____分,其中得分最高的三台设备是____、____、____,得分最低的三台设备是____、____、____。");

        vo.setRate1(Float.parseFloat(avgKdlValue));
        vo.setRate10(Float.parseFloat(avgBtlValue));
        vo.setRate2(minKdlValue);
        vo.setRate3(Float.parseFloat(avgFpl));
        vo.setRate4(Float.parseFloat(avgDevFpl));
        vo.setRate5(Float.parseFloat(avgSxmd));
        vo.setRate11(Float.parseFloat(avgSxmd1));
        vo.setRate6(Float.parseFloat(sandPercentData.get("rate6")==null?"0":sandPercentData.get("rate6").toString()));
        vo.setRate7(Float.parseFloat(sandPercentData.get("rate7")==null?"0":sandPercentData.get("rate7").toString()));
        vo.setRate8(Float.parseFloat(sandPercentData.get("rate8")==null?"0":sandPercentData.get("rate8").toString()));
        vo.setRate9(Float.parseFloat(sandPercentData.get("rate9")==null?"0":sandPercentData.get("rate9").toString()));

        String value0 = "<span class='xh'>"+(minKdlDevice==null?"":minKdlDevice)+"</span>的开动率最低，为"+"<span class='xh'>"+(minKdlValue==null?"":String.format("%.1f%%", minKdlValue*100))+"</span>；";
        String value9 = "<span class='xh'>"+(minBtlDevice==null?"":minBtlDevice)+"</span>的布图率最低，为"+"<span class='xh'>"+(minBtlValue==null?"":String.format("%.1f%%", minBtlValue*100))+"</span>；";
        vo.setValue0(value0);
        vo.setValue9(value9);
        vo.setValue1(top3WastereasonTypes==null?"":top3WastereasonTypes);
        vo.setValue2(top3DevfplDevices==null?"":top3DevfplDevices);
        vo.setValue3(top3MdbhgDevices==null?"":top3MdbhgDevices);
        vo.setValue11(top3MdbhgDevices1==null?"":top3MdbhgDevices1);
        vo.setValue4(sandPercentData.get("value4")==null?"":sandPercentData.get("value4").toString());
        vo.setValue5(sandPercentData.get("value5")==null?"":sandPercentData.get("value5").toString());
        vo.setValue5_1(sandPercentData.get("value5_1")==null?"":sandPercentData.get("value5_1").toString());
        vo.setValue6(sandPercentData.get("value6")==null?"":sandPercentData.get("value6").toString());
        vo.setValue6_1(sandPercentData.get("value6_1")==null?"":sandPercentData.get("value6_1").toString());
//        vo.setValue7(workTimeData.get("aveWorkTime")==null?"":workTimeData.get("aveWorkTime").toString());  //喷头
        vo.setRatelist(ratelist==null?"":ratelist);

        String targetValue1 = dyhsTargetValue==null			?"":String.format("%.1f", dyhsTargetValue*100);
        String targetValue2 = btlTargetValue==null			?"":String.format("%.1f", btlTargetValue*100);
        String targetValue3 = sandPassrateTargetValue==null	?"":String.format("%.1f", sandPassrateTargetValue*1.0);
        String targetValue4 = valueMaker==null				?"":String.format("%.1f", valueMaker*100);
        String targetValue8 = xjszbTargetValue==null		?"":String.format("%.1f", xjszbTargetValue*100);

        vo.setRemark1("周开动率=本周打印时间之和/(21*7)。月开动率=本周打印时间之和/(21*28)，目标值"+targetValue1+"%");
        vo.setRemark2("布图率=打印体积/（打印高度*工作箱长*宽*2）；Max、800、1800机型布图率=打印体积/（打印层厚*打印层数*工作箱长*宽）,"+targetValue2+"%目标");
        vo.setRemark3("打印效率=（打印层厚*打印层数*工作箱长*宽*2）/打印耗时；Max、800、1800机型打印效率=（打印层厚*打印层数*工作箱长*宽）/打印耗时；（单位：升/小时）");
        vo.setRemark4("产量=打印体积*1.5(T)；每周理论值2.5*7(T),每月理论值2.5*28(T)；设备废品率指标"+targetValue4+"%");
        vo.setRemark5("");
        vo.setRemark6("密度合格率=每台设备密度合格试块数量/送检试块总数*100%，密度标准值："+targetValue3+"g/cm^3");
        vo.setRemark7("树脂加入比=树脂用量/打印量*100%");
        vo.setRemark8("新砂占比=新砂占总用砂的比例，目标值"+targetValue8+"%");
        vo.setRemark9("未使用砂量=（新砂累计用量+旧砂累计用量）-2*打印机工作箱长*工作箱宽*打印层厚*打印层数*1.5 ；数据为每个打印批次的平均值");
        vo.setRemark10("喷头使用时长=喷头更换时间-喷头安装时间（喷头的一次使用时长）；工作时长为打印喷墨工作时长");
        vo.setRemark11("");//砂型强度合格率最低的三台的设备
        vo.setRemark12("");//产能利用率最低的三台设备
        vo.setRemark13("密度合格率=每台设备强度合格试块数量/送检试块总数*100%，强度标准值：6mPa");
        vo.setRemark14("");//维修费用最高三台的设备
        vo.setRemark15("固化剂单耗=固化剂耗量之和 / 砂芯重量之和");
        vo.setRemark16("废品率、打印效率利用率、故障率分别以40%、30%、30%权重计算星级评价得分");
        nxReportDao.insert(vo);
        batchAddItemVOs(vo);
        return vo;
    }

    private void batchAddItemVOs(RepPrinterReportVO vo) {
        List<RepPrinterReportItemVO> vos = new ArrayList<RepPrinterReportItemVO>();
        RepPrinterReportItemVO item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data1");
        item.setReportData(vo.getData1());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data2");
        item.setReportData(vo.getData2());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data3");
        item.setReportData(vo.getData3());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data4");
        item.setReportData(vo.getData4());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data5");
        item.setReportData(vo.getData5());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data6");
        item.setReportData(vo.getData6());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data7");
        item.setReportData(vo.getData7());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data8");
        item.setReportData(vo.getData8());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data9");
        item.setReportData(vo.getData9());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data10");
        item.setReportData(vo.getData10());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data11");
        item.setReportData(vo.getData11());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data12");
        item.setReportData(vo.getData12());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data13");
        item.setReportData(vo.getData13());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data14");
        item.setReportData(vo.getData14());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data15");
        item.setReportData(vo.getData15());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data16");
        item.setReportData(vo.getData16());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data17");
        item.setReportData(vo.getData17());
        vos.add(item);

        item = new RepPrinterReportItemVO();
        item.setReportId(vo.getId());
        item.setReportType("data18");
        item.setReportData(vo.getData18());
        vos.add(item);

        repPrinterReportItemDao.add(vos);
    }





    /**
     * 分页查找用户报告列表
     */
    public JsonResult2 reportPageList(String tenantId, String user_id, QueryUserReportCondition c) throws Exception {
        Integer pageNum = StringUtils.isNotBlank(c.getPageNo())?Integer.parseInt(c.getPageNo()):1;
        Integer pageSize = StringUtils.isNotBlank(c.getPageSize())?Integer.parseInt(c.getPageSize()):10;

        JsonResult2 jsonResult=new JsonResult2();
        Map<String,Object> query = new HashMap<String,Object>();
        query.put("tenantId", ""); //""
        query.put("emails", c.getEmails());
        query.put("officeId", c.getOfficeId());

        QueryUserReportCondition vo =new QueryUserReportCondition();
        vo.setCustomerId(c.getCustomerId());
        List<CustomerVO> customers = nxCustomerDao.getAllCustomer(vo);
        List<String> officeIds = new ArrayList<>();
        for(int i=0;i<customers.size();i++)
        {
            officeIds.add(customers.get(i).getCustomerCode());
            query.put("officeIds", officeIds);
        }
//        if(StringUtils.isBlank(c.getOfficeId())) {
//            JsonResult jr = baseClient.getOfficesByUser(tenantId, user_id);
//            List<Map<String, Object>> orgList = FeignClientUtil.getResult(jr);
//            List<String> officeIds = orgList.stream().map(item -> item.get("id").toString()).collect(Collectors.toList());
//            query.put("officeIds", officeIds);
//        }
        query.put("officeName", c.getOfficeName());
        query.put("customerId", c.getCustomerId());
        query.put("customerName", c.getCustomerName());
        query.put("dataYear", c.getDataYear());
        query.put("dataQuarter", c.getDataQuarter());
        query.put("dataMonth", c.getDataMonth());
        query.put("dataWeek", c.getDataWeek());


        int startIndex  = (pageNum -1 ) * pageSize;
        query.put("startNo", startIndex);
        query.put("endNo", pageNum * pageSize);
        List<RepPrinterReportVO> list = nxReportDao.getPrinterUserReportsList(query);
        Integer count = nxReportDao.getPrinterUserReportsCount(query).size();
        PageInfo<RepPrinterReportVO> pageInfo = new PageInfo<>(list);
        pageInfo.setPageNum(pageNum);
        pageInfo.setPageSize(pageSize);
        pageInfo.setTotal(count);
        return new JsonResult2(pageInfo);
    }

    /**
     * 	根据租户id与用户报告ID 得到用户报告
     */
    public RepPrinterReportVO getUserReport(Map<String, Object> params) throws Exception {
        if(params.get("reportId")==null || "".equals(params.get("reportId").toString().trim())) {
            throw new Exception("用户报告id为空");
        }
//        if(params.get("tenantId")==null || "".equals(params.get("tenantId").toString().trim())) {
//            throw new Exception("租户id为空");
//        }
        List<RepPrinterReportVO> reports = nxReportDao.getPrinterUserReports(params);
        RepPrinterReportVO v = null;

        if(reports!=null&&reports.size()>0) {
            DecimalFormat df =new DecimalFormat("0.0");
            v = reports.get(0);
            v.setRate1Str( 	df.format(v.getRate1()*100)	+"%");
            v.setRate3Str( 	df.format(v.getRate3()*100)	+"%");
            v.setRate4Str( 	df.format(v.getRate4()*100)	+"%" );
            v.setRate5Str( 	df.format(v.getRate5()*100)	+"%" );
            v.setRate11Str( 	df.format(v.getRate11()*100)	+"%" );
            v.setRate12Str( 	df.format(v.getRate12()*100)	+"%" );
            v.setRate6Str( 	df.format(v.getRate6())	+"%");
            v.setRate7Str(  df.format(v.getRate7())		+"%");
            v.setRate8Str(  df.format(v.getRate8()*100)	+"%" );
            v.setRate10Str( df.format(v.getRate10()*100)	+"%");
            v.setRate13(Math.round(v.getRate13()*10)/10.0f);
            v.setRate14(Math.round(v.getRate14()*10)/10.0f);
            v.setRate15(Math.round(v.getRate15()*10)/10.0f);
            v = setPrinterReportItem(v);
        }
        return v;
    }


    private RepPrinterReportVO setPrinterReportItem(RepPrinterReportVO v) throws Exception {
        RepPrinterReportItemVO vo = new RepPrinterReportItemVO();
        vo.setReportId(v.getId());
        List<RepPrinterReportItemVO> list = repPrinterReportItemDao.list(vo);
        if(list!=null && list.size()>0) {
            for(RepPrinterReportItemVO bean : list) {
                if("data1".equals(bean.getReportType())) {
                    v.setData1(bean.getReportData());
                }else if("data2".equals(bean.getReportType())) {
                    v.setData2(bean.getReportData());
                }else if("data3".equals(bean.getReportType())) {
                    v.setData3(bean.getReportData());
                }else if("data4".equals(bean.getReportType())) {
                    v.setData4(bean.getReportData());
                }else if("data5".equals(bean.getReportType())) {
                    v.setData5(bean.getReportData());
                }else if("data6".equals(bean.getReportType())) {
                    v.setData6(bean.getReportData());
                }else if("data7".equals(bean.getReportType())) {
                    v.setData7(bean.getReportData());
                }else if("data8".equals(bean.getReportType())) {
                    v.setData8(bean.getReportData());
                }else if("data9".equals(bean.getReportType())) {
                    v.setData9(bean.getReportData());
                }else if("data10".equals(bean.getReportType())) {
                    v.setData10(bean.getReportData());
                }else if("data11".equals(bean.getReportType())) {
                    v.setData11(bean.getReportData());
                }else if("data12".equals(bean.getReportType())) {
                    v.setData12(bean.getReportData());
                }else if("data13".equals(bean.getReportType())) {
                    v.setData13(bean.getReportData());
                }else if("data14".equals(bean.getReportType())) {
                    v.setData14(bean.getReportData());
                }else if("data15".equals(bean.getReportType())) {
                    v.setData15(bean.getReportData());
                }else if("data16".equals(bean.getReportType())) {
                    v.setData16(bean.getReportData());
                }else if("data17".equals(bean.getReportType())) {
                    v.setData17(bean.getReportData());
                }else if("data18".equals(bean.getReportType())) {
                    v.setData18(bean.getReportData());
                }
            }
        }
        return v;
    }


    public List<RepPrinterReportItemVO> getReportItem(Long reportId) {
        RepPrinterReportItemVO vo = new RepPrinterReportItemVO();
        vo.setReportId(reportId);
        return repPrinterReportItemDao.list(vo);
    }

    public void updateReportItem(List<RepPrinterReportItemVO> items) {
        if (!CollectionUtils.isEmpty(items)) {
            repPrinterReportItemDao.update(items);
        }
    }

    public void updateUserReportValue(RepPrinterReportVO vo) {
        nxReportDao.updateUserReportValue(vo);
    }


    /**
     * 产线级-指定邮箱发送打印机用户运维报告
     * @throws Exception
     */
    public String sendUserReportByUser(Map<String, Object> params) throws Exception {
        if(params.get("reportId")==null || "".equals(params.get("reportId").toString().trim())) {
            throw new Exception("用户报告id为空");
        }
        if(params.get("emails")==null || "".equals(params.get("emails").toString().trim())) {
            throw new Exception("发送邮箱为空");
        }
        if(params.get("tenantId")==null || "".equals(params.get("tenantId").toString().trim())) {
            throw new Exception("租户id为空");
        }
        Long reportId = (Long) params.get("reportId");
        Map<String, Object> query = new HashMap<String, Object>();
        query.put("reportId", reportId);
        query.put("tenantId", params.get("tenantId"));
        List<RepPrinterReportVO> reports = nxReportDao.getPrinterUserReports(query);
        String link = "";
        if(reports!=null && reports.size()>0) {
            RepPrinterReportVO v = reports.get(0);
            String emails = params.get("emails").toString();
            String vEmails = v.getEmails();
            String[] emailArray = emails.split(",");
            for(String e : emailArray) {
                if(v.getEmails()==null) {
                    vEmails = e + ",";
                }else if(vEmails.indexOf(e)==-1) {
                    vEmails = vEmails + e + ",";
                }
            }
            params.put("emails", vEmails);

            v.setEmails(emails);
            link = emailService.mailUserReport(v);
            //发送成功后将成功的邮箱记录下来
            params.put("id", reportId);
            nxReportDao.updateEmailsById(params);
        }
        return link;
    }


    public List<JSONObject> queryUserReport(String tenantId, QueryUserReportCondition c) throws Exception {
        List<CustomerVO> customers = nxCustomerDao.getAllCustomer(c);
        if (CollectionUtils.isEmpty(customers)) {
            return new ArrayList<>();
        }
        CustomerVO customer = customers.get(0);
        //从配置表中获取平台设备编码及用户设备编码关系
        Map<String,String>	printerDeviceRelationData =	getPrinterDeviceRelation();

        List<String> deviceIdList = nxCustomerDao.getRegisterDeviceByUser(Arrays.asList(customer.getOfficeId()), tenantId);

        if (CollectionUtils.isEmpty(deviceIdList)) {
            return new ArrayList<>();
        }

        PrinterReportDTO printerReportDTO = new PrinterReportDTO();
        printerReportDTO.setDeviceIdList(deviceIdList);
        printerReportDTO.setStartDate(c.getStartDt());
        printerReportDTO.setEndDate(c.getEndDt());

        int day = DateUtils.getDayDifference(printerReportDTO.getStartDate(), printerReportDTO.getEndDate());

        List<Map<String, Object>> dyhsList = printerEfficiencyWorkDao.getReportDYHSByDevice(printerReportDTO);
        Map<String, Object> kdlMap = new HashMap<>();
        Map<String, Object> dyxlMap = new HashMap<>();
        Map<String, Object> sxmdMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(dyhsList)) {
            dyhsList.forEach(item -> {
                Map<String, Object> map = new HashMap<>(3);
                float dyhs = Float.parseFloat(item.getOrDefault("dyhs", "0").toString());
                kdlMap.put(item.get("device_id").toString(),String.format("%.2f", dyhs*100 / (21*day)));
            });
        }
        List<Map<String, Object>> dyxlList = printerEfficiencyWorkDao.getReportDYXLByDevice(printerReportDTO);
        if (!CollectionUtils.isEmpty(dyxlList)) {
            dyxlList.forEach(
                    x->{
                        dyxlMap.put(x.get("device_id").toString(), String.format("%.2f",x.get("dyxl")));
                    }
            );
        }
        List<Map<String, Object>> sxmdList = repPrinterSandPercentDAO.getRepPrinterSandSXMDList(printerReportDTO);
        if (!CollectionUtils.isEmpty(sxmdList)) {
            sxmdList.forEach(
                    x->{
                        sxmdMap.put(x.get("device_id").toString(), String.format("%.2f",x.get("sxmd")));
                    }
            );
        }
        List<JSONObject> result = new ArrayList<>();
        deviceIdList.forEach(
                x->{
                    JSONObject json = new JSONObject();
                    json.put("deviceId", printerDeviceRelationData.getOrDefault(x,x));
                    json.put("kdl", kdlMap.getOrDefault(x, null));
                    json.put("dyxl", dyxlMap.getOrDefault(x, null));
                    json.put("szjrb", sxmdMap.getOrDefault(x, null));
                    result.add(json);
                }
        );
        logger.debug("result:{}",JSON.toJSONString(result));
        return result;
    }


    /**
     * 	根据id删除用户报告
     */
    public JsonResult2 deleteReportById(Long reportId){
        JsonResult2 jsonResult=new JsonResult2();
        try {
            Map<String,Object> params = new HashMap<String,Object>();
            params.put("reportId", reportId);
            nxReportDao.deleteReportById(params);
            repPrinterReportItemDao.deleteByReportId(params);
        }catch(Exception e) {
            return new JsonResult2(JsonResultStatus.DEVICE_FAIL.getRet(), "产线运维报告删除失败！");
        }
        return jsonResult;
    }


    private static final String PRINTER_DEVICE_RELATION = "printer_device_relation";
    //从配置表中获取平台设备编码及用户设备编码关系
    public Map<String,String> getPrinterDeviceRelation() throws Exception{

        Map<String, String> data= new HashMap();
        Optional<ConfigVO> configVOOptional = configService.findOne(PRINTER_DEVICE_RELATION);
        if (!configVOOptional.isPresent()) {
            //logger.error("{}配置项不存在", PRINTER_TENANT_ID);
            throw new Exception(PRINTER_DEVICE_RELATION + "配置项不存在");
        }

        String printerDeviceRelation = configVOOptional.get().getValue();
        if(StringUtils.isNotBlank(printerDeviceRelation)){
            JSONArray array =  JSONObject.parseArray(printerDeviceRelation);
            for( int i=0; array!=null&&i<array.size(); i++ ){
                JSONObject jobj = array.getJSONObject(i);
                String  device= jobj.get("device")==null?"":jobj.get("device").toString();
                String  code= jobj.get("code")==null?"":jobj.get("code").toString();
                String  loc= jobj.get("loc")==null?"":jobj.get("loc").toString();
                if( StringUtils.isNotBlank(loc) ){
                    code = code+"("+loc+")";
                }
                data.put(device, code);
            }
        }
        return data;
    }

    /**
     * 组装查询条件的数据
     */
    private PrinterReportDTO setPrinterReportDTO(CustomerVO c, RepPrinterReportVO vo) {
        PrinterReportDTO printerReportDTO = new PrinterReportDTO();
        printerReportDTO.setOfficeId(c.getOfficeId());
        printerReportDTO.setYear(vo.getDataYear());
        printerReportDTO.setQuarter(vo.getDataQuarter());
        printerReportDTO.setMonth(vo.getDataMonth());
        printerReportDTO.setWeek(vo.getDataWeek());
        printerReportDTO.setStartDate( vo.getStartDate() );
        printerReportDTO.setEndDate( vo.getEndDate() );
        printerReportDTO.setQueryedType( vo.getCreateType() );

        // 如果需要根据机构查询，则需要调用query服务查询该机构下的所有设备；
//        String officeId = c.getOfficeId();
        String officeId = c.getCustomerCode();
        if (StringUtils.isNotBlank(officeId)) {
            List<String> officeIds = new ArrayList<String>();
            officeIds.add(officeId);
            List<String> deviceIdList = nxCustomerDao.getRegisterDeviceByUser(officeIds, c.getTenantId()); //编码转换可能会有问题
            if( deviceIdList!=null && deviceIdList.size()>0 ) {
                printerReportDTO.setDeviceIdList(deviceIdList);
            } else {
                printerReportDTO.setDeviceIdList(null);
            }
            printerReportDTO.setOfficeId(c.getCustomerCode());
            printerReportDTO.setQueryedDeviceIdList(1);//每个统计查询里是否需要再设置设备id串的标识
        }
        return printerReportDTO;
    }

    private String  dateDiff(String startTime, String endTime, String format) throws Exception {
        //按照传入的格式生成一个simpledateformate对象
        SimpleDateFormat sd = new SimpleDateFormat(format);
        long nd = 1000*24*60*60;//一天的毫秒数
        long nh = 1000*60*60;//一小时的毫秒数
        long nm = 1000*60;//一分钟的毫秒数
        long ns = 1000;//一秒钟的毫秒数
        long diff;
        //获得两个时间的毫秒时间差异
        diff = sd.parse(endTime).getTime() - sd.parse(startTime).getTime();
//		long day = diff/nd;//计算差多少天
        double diff2 = (double) diff;
        double hour =diff2 /(1000 * 60 * 60);//计算差多少小时
        String result = String.format("%.2f",hour);

//		long hour = diff%nd/nh;//计算差多少小时
//		long min = diff%nd%nh/nm;//计算差多少分钟
//		long sec = diff%nd%nh%nm/ns;//计算差多少秒//输出结果
//		System.out.println("时间相差："+day+"天"+hour+"小时"+min+"分钟"+sec+"秒。");
        return result ;
    }


}
