package com.rootcloud.web;


import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.JsonResult2;
import com.rootcloud.common.util.DateUtils;
import com.rootcloud.domain.service.fittings.SparePartService;
import com.rootcloud.service.PrinterEfficiencyService;
import groovy.util.logging.Slf4j;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jline.internal.Log;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.Date;


@Api(tags = "历史工况跑批控制器")
@RestController
@Slf4j
@RequestMapping("/runHistortGK")
public class PrinterEfficiencyController {

    private static final Logger logger = LoggerFactory.getLogger(PrinterEfficiencyController.class);


    @Resource(name = "efficiency")
    private PrinterEfficiencyService efficiencyService;
    @Resource(name = "sparePartServiceImpl")
    private SparePartService sparePartService;

    @ApiOperation("同步备件工作时长")
    @PostMapping("/runWorkingTime")
    public void syncWorkingTime(){
        Log.info("同步备件工作时长开始");
        sparePartService.syncWorkingTime();
        Log.info("同步备件工作时长结束");
    }

    @ApiOperation("开动率效率补图率指定日期跑批")
    @PostMapping("/getHisdataByDate")
    public JsonResult2 runHistoryGKData(@RequestParam("startDate") String startDate)
    {
        Calendar startCalendar = Calendar.getInstance();
        Date date = DateUtils.parseDate("yyyy-MM-dd", startDate);
        if (null == date) {
            logger.error("开始日期不能为空");
            return new JsonResult2("500", "开始日期不能为空");
        }
        startCalendar.setTime(date);
        try {
            Calendar nowCalendar = Calendar.getInstance();
            nowCalendar.add(Calendar.DAY_OF_MONTH, -1);
            efficiencyService.run1((Calendar) startCalendar.clone());
            return new JsonResult2();
        }
        catch (Exception ex){
            return new JsonResult2("500", "跑批失败...");
        }
    }

    @ApiOperation("开动率效率补图率指定时间段跑批")
    @PostMapping("/getHisdataBySpanTime")
    public JsonResult2 getHisdataBySpanTime(@RequestParam("startDate") String startDate,@RequestParam("endDate") String endDateStr)
    {

        Calendar startCalendar = Calendar.getInstance();
        Date date = DateUtils.parseDate("yyyy-MM-dd", startDate);
        if (null == date) {
            logger.error("开始日期不能为空");
            return new JsonResult2("500", "开始日期不能为空");
        }
        startCalendar.setTime(date);
        Calendar nowCalendar = Calendar.getInstance();
        Date endDate = DateUtils.parseDate("yyyy-MM-dd", endDateStr);
        if (null == endDate) {
            logger.error("结束日期不能为空");
            return new JsonResult2("500", "结束日期不能为空");
        }
        nowCalendar.setTime(endDate);
        // 从开始日期开始跑批直到结束日期
        do {
            try {
                efficiencyService.run1((Calendar) startCalendar.clone());
            } catch (Exception e) {
                logger.error("跑批失败", e);
                return new JsonResult2("500", e.getMessage());
            }
            logger.info("---> 日期{}，打印机效率跑批结束",startCalendar.toInstant());
            startCalendar.add(Calendar.DAY_OF_MONTH, 1);
        } while (startCalendar.before(nowCalendar));
        return new JsonResult2();

    }

//2022-09-16  由温海龙那边跑p统一处理
//    @GetMapping("/runDayEfficiencyBatch")
//    @ApiOperation("按打印批次24H内，指定日期 开动率(打印耗时)跑批(NX_DEVICE_WORKING_TIMES_DAY)")
//    public JsonResult2 runBatch2(@RequestParam("startDate") String startDate) {
//        Calendar startCalendar = Calendar.getInstance();
//        Date date = DateUtils.parseDate("yyyy-MM-dd", startDate);
//        if (null == date) {
//            Log.error("开始日期不能为空");
//            return new JsonResult2("500", "开始日期不能为空");
//        }
//        startCalendar.setTime(date);
//        try {
//            Calendar nowCalendar = Calendar.getInstance();
//            nowCalendar.add(Calendar.DAY_OF_MONTH, -1);
//            efficiencyService.run0((Calendar) startCalendar.clone());
//            //统计月
//            try {
//                efficiencyService.run0_statistics_month((Calendar) startCalendar.clone());
//            } catch (Exception ee) {
//                Log.error("统计月时长报错,"+ee.getMessage());
//            }
//            return new JsonResult2();
//        } catch (Exception ex) {
//            return new JsonResult2("500", "跑批失败...");
//        }
//    }

//2022-09-16  由温海龙那边跑p统一处理
//    @GetMapping("/runEuropeEfficiency24Batch")
//    @ApiOperation("海外 按打印批次24H内，指定日期 开动率(打印耗时)跑批(NX_DEVICE_WORKING_TIMES_DAY)")
//    public JsonResult2 runEuropeEfficiency24Batch(@RequestParam("startDate") String startDate) {
//        Calendar startCalendar = Calendar.getInstance();
//        Date date = DateUtils.parseDate("yyyy-MM-dd", startDate);
//        if (null == date) {
//            logger.error("开始日期不能为空");
//            return new JsonResult2("500", "开始日期不能为空");
//        }
//        startCalendar.setTime(date);
//        try {
//            efficiencyService.runEurop0((Calendar) startCalendar.clone());
//            //统计月
//            try {
//                efficiencyService.run0_statistics_month((Calendar) startCalendar.clone());
//            } catch (Exception ee) {
//                Log.error("统计月时长报错,"+ee.getMessage());
//            }
//            return new JsonResult2();
//        }
//        catch (Exception ex){
//            return new JsonResult2("500", "跑批失败...");
//        }
//    }


    @GetMapping("/runStatisticsMonth")
    @ApiOperation("统计月的时长")
    public JsonResult2 runBatch2() {
        Calendar startCalendar = Calendar.getInstance();
        //统计月
        try {
            efficiencyService.run0_statistics_month((Calendar) startCalendar.clone());
        } catch (Exception ee) {
            Log.error("统计月时长报错,"+ee.getMessage());
            new JsonResult2("500", "统计月时长报错...");
        }
        return new JsonResult2();
    }

}
