package com.rootcloud.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.rootcloud.common.JsonResult;
import com.rootcloud.common.JsonResult2;
import com.rootcloud.common.JsonResultStatus;
import com.rootcloud.common.util.DateUtils;
import com.rootcloud.domain.dao.RepPrinterSandPercentDAO2;
import com.rootcloud.domain.vo.RepPrinterReportItemVO;
import com.rootcloud.domain.vo.RepPrinterReportVO;
import com.rootcloud.domain.vo.RepPrinterTargetDicVO;
import com.rootcloud.service.RepPrinterReportService;
import com.rootcloud.service.RepPrinterTargetDicService;
import com.rootcloud.service.condition.*;
import com.rootcloud.service.constant.ExceptionStatus;
import groovy.util.logging.Slf4j;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;


@Api(tags = "运维报告报表、打印过程分析统计服务控制器")
@RestController
@Slf4j
@RequestMapping("/reportService")
public class PrinterReportController {

    private static final Logger logger = LoggerFactory.getLogger(PrinterReportController.class);

    @Resource(name = "reportService")
    private RepPrinterReportService service;
    @Resource(name = "targetService")
    private RepPrinterTargetDicService repPrinterTargetDicService;

    @Resource
    private RepPrinterSandPercentDAO2 repPrinterSandPercentDAO2;


    @GetMapping("/runPrintingProcessBatch")
    @ApiOperation("指定日期 打印过程统计(打印机树脂加入比、新旧砂加入占比、漏砂量等）跑批")
    public JsonResult2 runPrintingProcessBatch(@RequestParam("startDate") String startDate) {
        Calendar startCalendar = Calendar.getInstance();
        Date date = DateUtils.parseDate("yyyy-MM-dd", startDate);
        if (null == date) {
            logger.error("开始日期不能为空");
            return new JsonResult2("500", "开始日期不能为空");
        }
        startCalendar.setTime(date);
        Calendar nowCalendar = Calendar.getInstance();
        nowCalendar.add(Calendar.DAY_OF_MONTH, -1);
        // 从开始日期开始跑批直到当前日期的的前一天
        try {
            service.run((Calendar) startCalendar.clone());
        } catch (Exception ex) {
            logger.error("打印过程统计跑批失败...", ex.toString());
            return new JsonResult2("500", ex.getMessage());
        }
        return new JsonResult2();
    }

    @GetMapping("/runPrintingProcessSpanTimeBatch")
    @ApiOperation("时间段 打印过程统计(打印机树脂加入比、新旧砂加入占比、漏砂量等）跑批")
    public JsonResult2 runPrintingProcessSpanTimeBatch(@RequestParam("startDate") String startDate,@RequestParam("endDate") String endDateStr) {

        Calendar startCalendar = Calendar.getInstance();
        Date date = DateUtils.parseDate("yyyy-MM-dd", startDate);
        if (null == date) {
            logger.error("开始日期不能为空");
            return new JsonResult2("500", "开始日期不能为空");
        }
        startCalendar.setTime(date);
        Calendar nowCalendar = Calendar.getInstance();
        Date endDate = DateUtils.parseDate("yyyy-MM-dd", endDateStr);
        if (null == endDate) {
            logger.error("结束日期不能为空");
            return new JsonResult2("500", "结束日期不能为空");
        }
        nowCalendar.setTime(endDate);
        // 从开始日期开始跑批直到结束日期
        do {
            try {
                service.run((Calendar) startCalendar.clone());
            } catch (Exception e) {
                logger.error("跑批失败", e);
                return new JsonResult2("500", e.getMessage());
            }
            logger.info("---> 日期{}，打印机效率跑批结束",startCalendar.toInstant());
            startCalendar.add(Calendar.DAY_OF_MONTH, 1);
        } while (startCalendar.before(nowCalendar));
        return new JsonResult2();
    }


//    @GetMapping("/runPrintingProcessBatch2")
//    @ApiOperation("当天打印过程统计(打印机树脂加入比、新旧砂加入占比、漏砂量等）跑批")
//    public JsonResult2 runPrintingProcessBatch2() {
//        Calendar c = Calendar.getInstance();
//        c.add(Calendar.DAY_OF_MONTH, -1);
//        try {
//            logger.info("定时生成按批次跑批打印耗时、树脂加入、漏砂、打印体积、AB砂、打印耗时、打印层数、固化剂等(rep_printer_sand_percent)等报表数据...");
//            service.run((Calendar) c.clone());
//        } catch (Exception ex) {
//            logger.error("定时生成按批次跑批打印耗时、树脂加入、漏砂、打印体积、AB砂、打印耗时、打印层数、固化剂等(rep_printer_sand_percent)等报表数据失败...", ex);
//        }
//        return new JsonResult2();
//    }

    /**
     * 手动触发生成产线运维报告
     */
    @ApiOperation(value="手动触发生成产线运维报告", notes="手动触发生成产线运维报告")
    @PostMapping("/createUserReport")
    public JsonResult2 createUserReport(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("user_id") String user_id, @RequestBody QueryUserReportCondition condition) {
        String param = JSON.toJSONString(condition);
        logger.info("start method : createUserReport, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param);
        try {
            JsonResult2 jsonResult = new JsonResult2();
            service.createUserReportV2(condition);

            jsonResult.setData(true);
            logger.info("end method : createUserReport,  tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param+" result: "+JSON.toJSONString(jsonResult));
            return jsonResult;
        } catch (Exception e) {
            logger.error("method : createUserReport, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param+ " exception", e);
            String msg = e.getMessage()!=null?e.getMessage(): JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }

    /**
     * 分页查找用户运维报告列表
     * @param tenant_id
     * @return
     */
    @ApiOperation(value="分页查找用户运维报告列表", notes="分页查找用户运维报告列表")
    @PostMapping("/reportPageList")
    public JsonResult2 reportPageList(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("user_id") String user_id, @RequestBody QueryUserReportCondition c) {
        String param = JSON.toJSONString(c);
        logger.info("start method : reportPageList, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param);
        try {
            JsonResult2 res = service.reportPageList(tenant_id, user_id, c);
            logger.info("end method : reportPageList,  tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param+" result: "+JSON.toJSONString(res));
            return res;
        } catch (Exception e) {
            logger.error("method : reportPageList, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param + " exception:", e);
            String msg = e.getMessage()!=null?e.getMessage():JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }
    /**
     * 	根据租户id与用户报告ID 得到用户报告
     */
    @ApiOperation(value="根据租户id与用户报告ID 得到用户报告", notes="根据租户id与用户报告ID 得到用户报告")
    @PostMapping("/getUserReportById")
    public JsonResult2 getUserReportById(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("user_id") String user_id, @RequestBody QueryUserReportCondition c) {
        String param = JSON.toJSONString(c);
        logger.info("start method : getUserReportById, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param);
        JsonResult2 jsonResult = new JsonResult2();
        try {
            Map<String,Object> query = new HashMap<String,Object>();
            query.put("reportId", c.getId());
            query.put("tenantId", "");
            RepPrinterReportVO data = service.getUserReport(query);

            jsonResult.setData(data);
            logger.info("end method : getUserReportById, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param +" res: "+JSON.toJSONString(jsonResult));
            return jsonResult;
        } catch (Exception e) {
            logger.error("method : getUserReportById, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param+" exception:", e);
            String msg = e.getMessage()!=null?e.getMessage():JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }

    /**
     * 根据id删除报告
     */
    @ApiOperation(value="根据id删除报告-3DP产线", notes="根据id删除报告-3DP产线")
    @PostMapping("/deleteReportById")
    public JsonResult2 deleteReportById(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("user_id") String user_id, @RequestParam Long reportId) {
        logger.info("start method : deleteReportById, tenant_id ="+tenant_id+" user_id = "+user_id+" reportId : "+ reportId);
        try {
            JsonResult2 res = service.deleteReportById(reportId);
            logger.info("end method : deleteReportById,  tenant_id ="+tenant_id+" user_id = "+user_id+" reportId : "+ reportId+" result: "+JSON.toJSONString(res));
            return res;
        } catch (Exception e) {
            logger.error("method : deleteReportById, tenant_id ="+tenant_id+" user_id = "+user_id+" reportId : "+ reportId + " exception:", e);
            String msg = e.getMessage()!=null?e.getMessage():JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }

    /**
     * 	查询产线报告详情
     */
    @ApiOperation(value="查询报告详情", notes="查询报告详情")
    @PostMapping("/getReportItem")
    public List<RepPrinterReportItemVO> getReportItem(Long reportId){
        return service.getReportItem(reportId);
    }
    /**
     * 	更新产线报告详情
     */
    @ApiOperation(value="更新报告详情", notes="更新报告详情")
    @PostMapping("/updateReportItem")
    public JsonResult2 updateReportItem(@RequestBody List<RepPrinterReportItemVO> items){
        service.updateReportItem(items);
        return new JsonResult2();
    }

    /**
     * 修改用户报告备注
     */
    @ApiOperation(value="修改用户报告备注", notes="修改用户报告备注")
    @PostMapping("/updateUserReportValue")
    public JsonResult2 updateUserReportValue( @RequestBody RepPrinterReportVO vo) {
        try {
            JsonResult2 jsonResult = new JsonResult2();
            service.updateUserReportValue(vo);
            jsonResult.setData(true);
            return jsonResult;
        } catch (Exception e) {
            String msg = e.getMessage()!=null?e.getMessage():JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }

    /**
     * 新增打印机目标值
     * @param request
     * @param condition
     * @return
     */
    @ApiOperation(value = "新增目标值", notes = "新增目标值",httpMethod="POST",
            response=JsonResult2.class,produces="application/json",consumes="application/json",protocols="HTTP/1.1")
    @RequestMapping(value="/addResPrinterTargetDic",method=RequestMethod.POST)
    public JsonResult2 addResPrinterTargetDic(HttpServletRequest request,
                                             @RequestBody AddRepPrinterTargetDicCondition condition) {

        RepPrinterTargetDicVO bean = new RepPrinterTargetDicVO();
        BeanUtils.copyProperties(condition, bean);
        if(StringUtils.isNotBlank(condition.getStartDate())) {
            Date startDate = DateUtils.parseDate("yyyy-MM-dd", condition.getStartDate());
            bean.setStartDate(startDate);
        }
        if(StringUtils.isNotBlank(condition.getEndDate())) {
            Date endDate = DateUtils.parseDate("yyyy-MM-dd", condition.getEndDate());
            bean.setEndDate(endDate);
        }
        return repPrinterTargetDicService.insert(bean);
    }
    /**
     * 根据ID删除目标值
     * @param condition
     * @return
     */
    @ApiOperation(value = "根据ID删除目标值", notes = "根据ID删除目标值",httpMethod="POST",response= JsonResult.class,produces="application/json",consumes="application/json",protocols="HTTP/1.1")
    @RequestMapping(value="/deleteResPrinterTargetDic",method=RequestMethod.POST)
    public JsonResult2 deleteResPrinterTargetDic(HttpServletRequest request,
                                                @RequestBody DelRepPrinterTargetDicCondition condition) {

        return repPrinterTargetDicService.deleteById(condition.getId());
    }

    /**
     * 更新目标值
     * @return
     */
    @ApiOperation(value = "更新目标值", notes = "更新目标值",httpMethod="POST",response= JsonResult.class,produces="application/json",consumes="application/json",protocols="HTTP/1.1")
    @RequestMapping(value="/updateResPrinterTargetDic",method=RequestMethod.POST)
    public JsonResult2 updateResPrinterTargetDic(HttpServletRequest request,@RequestBody UpdateRepPrinterTargetDicCondition condition){

        RepPrinterTargetDicVO bean = new RepPrinterTargetDicVO();
        BeanUtils.copyProperties(condition, bean);
        if(StringUtils.isNotBlank(condition.getStartDate())) {
            Date startDate = DateUtils.parseDate("yyyy-MM-dd", condition.getStartDate());
            bean.setStartDate(startDate);
        }
        if(StringUtils.isNotBlank(condition.getEndDate())) {
            Date endDate = DateUtils.parseDate("yyyy-MM-dd", condition.getEndDate());
            bean.setEndDate(endDate);
        }

        return repPrinterTargetDicService.update(bean);
    }

    /**
     * 通过id获取目标值
     *
     * @param request
     * @param condition
     * @return
     */
    @ApiOperation(value = "通过id获取目标值", notes = "通过id获取目标值", httpMethod = "POST", response = JsonResult.class, produces = "application/json", consumes = "application/json", protocols = "HTTP/1.1")
    @RequestMapping(value = "/getResPrinterTargetDicById", method = RequestMethod.POST)
    public JsonResult2 getResPrinterTargetDicById(HttpServletRequest request,
                                                 @RequestBody DelRepPrinterTargetDicCondition condition) {
        logger.debug("通过id获取目标值");
        try {
            RepPrinterTargetDicVO vo = repPrinterTargetDicService.getRepPrinterTargetDicById(condition.getId());
            return new JsonResult2(vo);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("通过id获取目标值异常:" + e);
            return new JsonResult2(ExceptionStatus.RET_EXCEPTION, ExceptionStatus.RET_EXCEPTION_CN);
        }
    }

    /**
     * 目标值列表
     * @param request
     * @return
     */
    @ApiOperation(value = "目标值列表", notes = "目标值列表",httpMethod="POST",response= JsonResult.class,produces="application/json",consumes="application/json",protocols="HTTP/1.1")
    @RequestMapping(value="/getResPrinterTargetDicList",method=RequestMethod.POST)
    public JsonResult2 getResPrinterTargetDicList(HttpServletRequest request, @RequestHeader("tenant_id") String tenant_id, @RequestBody QueryRepPrinterTargetDicCondition query ){
        return repPrinterTargetDicService.getRepPrinterTargetDicList(tenant_id, query);
    }

    /**
     * 	产线运维报告-指定邮箱发送用户报告
     */
    @ApiOperation(value="指定邮箱发送用户报告", notes="指定邮箱发送用户报告")
    @PostMapping("/mailUserReport")
    public JsonResult2 mailUserReport(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("user_id") String user_id, @RequestBody QueryUserReportCondition c) {
        String param = JSON.toJSONString(c);
        logger.info("start method : mailUserReport, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param);
        JsonResult2 jsonResult = new JsonResult2();
        try {
            Map<String,Object> params = new HashMap<String,Object>();
            params.put("emails", c.getEmails());
            params.put("reportId", c.getId());
            params.put("tenantId", tenant_id);
            String link = service.sendUserReportByUser(params);

            jsonResult.setData(link);
            jsonResult.setMsg("链接地址是:"+link);
            logger.info("end method : mailUserReport,  param : "+ param+" result: "+JSON.toJSONString(jsonResult));
            return jsonResult;
        } catch (Exception e) {
            logger.error("method : mailUserReport, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param +" exception:", e);
            String msg = e.getMessage()!=null?e.getMessage():JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }



    /**
     * 查询用户报告数据
     */
    @ApiOperation(value="查询用户报告数据", notes="查询用户报告数据")
    @PostMapping("/queryUserReport")
    public JsonResult2 queryUserReport(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("user_id") String user_id, @RequestBody QueryUserReportCondition condition) {
        String param = JSON.toJSONString(condition);
        try {
            JsonResult2 jsonResult = new JsonResult2();
            List<JSONObject> datas = service.queryUserReport(tenant_id,condition);
            jsonResult.setData(datas);
            return jsonResult;
        } catch (Exception e) {
            logger.error("method : createUserReport, tenant_id ="+tenant_id+" user_id = "+user_id+" param : "+ param+ " exception", e);
            String msg = e.getMessage()!=null?e.getMessage():JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }


    @ApiOperation(value = "打印过程数据-大屏用", notes = "打印过程数据-大屏用打印过程数据-大屏用")
    @RequestMapping(method = RequestMethod.POST, value = "/getJobProcessingData")
    public JsonResult2 getJobProcessingData(@RequestHeader("tenant_id") String tenant_id,@RequestHeader("user_id") String user_id,
                                           @RequestHeader("device_id") String device_id,@RequestParam("startDate") String startDate,@RequestParam("endDate") String endDate) {

//		String userId = userSessionUtil.getUserId(request);
//		String tenantId = userSessionUtil.getTenantId(request);
//		String accessToken = request.getHeader("access_token");
//		UserInfo userInfo = userSessionUtil.getUserInfo(request);
        Map<String, Object> params =new HashMap<>(20);
        params.put("tenantId", tenant_id);
        params.put("user_id", user_id);
        params.put("startDate", startDate);
        params.put("endDate", endDate);
        params.put("device_id", device_id);

        List<Map<String, Object>> list1 = repPrinterSandPercentDAO2.getRepPrinterProcessData(params);
        return new JsonResult2(list1);

    }



}
