package com.rootcloud.web;


import com.rootcloud.common.JsonResult2;
import com.rootcloud.common.JsonResultStatus;
import com.rootcloud.domain.dao.CustomerDao;
import com.rootcloud.service.RepCustomerService;
import groovy.util.logging.Slf4j;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 4.0系统中只有客户概念，没有机构，客户和机构相当于同一类概念
 * */

@Api(tags = "客户关系数据控制器")
@RestController
@Slf4j
@RequestMapping("/customer")
public class RepCustomerController {

    private static final Logger logger = LoggerFactory.getLogger(RepPrinterController.class);
    @Autowired
    private CustomerDao customerDao;


    @Resource(name = "customerServices")
    private RepCustomerService service;

    @ApiOperation(value="获取所有的客户列表", notes="获取所有的客户列表")
    @PostMapping("/getAllCustomerList")
    public JsonResult2 getAllCustomerList(@RequestHeader("tenant_id") String tenant_id) {
        try {
            JsonResult2 res = service.getAllCustomerList(tenant_id);
            return res;
        } catch (Exception e) {
            logger.error("客户信息列表信息获取失败", e);
            String msg = e.getMessage()!=null?e.getMessage(): JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }


    @ApiOperation(value="获取指定用户所属的客户列表", notes="客户信息列表")
    @PostMapping("/getCustomerListByUserId")
    public JsonResult2 getCustomerListByUserId(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("user_id") String user_id) {
        try {
            JsonResult2 res = service.getCustomerListByUserId(tenant_id, user_id);
            return res;
        } catch (Exception e) {
            logger.error("客户信息列表信息获取失败", e);
            String msg = e.getMessage()!=null?e.getMessage(): JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }
    @ApiOperation(value="查询指定客户下所有的设备编号列表", notes="查询指定客户下所有的设备编号列表")
    @PostMapping("/getDeviceCodeListByCustomerCode")
    public JsonResult2 getDeviceCodeListByCustomerCode(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("customerCode") String customerCode) {
        try {
            //只返回设备编号
            JsonResult2 res = service.getDeviceCodeListByCustomerCode(tenant_id, customerCode);
            return res;
        } catch (Exception e) {
            logger.error("客户信息列表信息获取失败", e);
            String msg = e.getMessage()!=null?e.getMessage(): JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }
    @ApiOperation(value="查询指定客户下所有的设备列表信息", notes="查询指定客户下所有的设备列表信息")
    @PostMapping("/getDeviceListByCustomerCode")
    public JsonResult2 getDeviceListByCustomerCode(@RequestHeader("tenant_id") String tenant_id, @RequestHeader("customerCode") String customerCode) {
        try {
            JsonResult2 res = service.getDeviceListByCustomerCode(tenant_id, customerCode);
            return res;
        } catch (Exception e) {
            logger.error("客户信息列表信息获取失败", e);
            String msg = e.getMessage()!=null?e.getMessage(): JsonResultStatus.SYS_EXCEPTION.getMsg();
            return new JsonResult2(JsonResultStatus.SYS_EXCEPTION.getRet(), msg);
        }
    }



}
